@@ -66,13 +66,9 @@ calBackendLoader.prototype = {
 
 function lazyFactoryFor(backendScope, classID) {
   return {
-    createInstance(aOuter, aIID) {
+    createInstance(aIID) {
       let realFactory = backendScope.NSGetFactory(classID);
-      return realFactory.createInstance(aOuter, aIID);
-    },
-    lockFactory(lock) {
-      let realFactory = backendScope.NSGetFactory(classID);
-      return realFactory.lockFactory(lock);
+      return realFactory.createInstance(aIID);
     },
   };
 }
