@@ -81,6 +81,11 @@ calIcalProperty::GetValue(nsACString& str) {
       return NS_OK;
     }
 
+    if (icalerrno == ICAL_NO_ERROR) {
+      str.Truncate();
+      return NS_OK;
+    }
+
 #ifdef DEBUG
     fprintf(stderr, "Error getting string value: %d (%s)\n", icalerrno,
             icalerror_strerror(icalerrno));
@@ -432,12 +437,12 @@ calIcalComponent::~calIcalComponent() {
   }
 }
 NS_IMETHODIMP
-calIcalComponent::GetIcalComponent(JS::MutableHandleValue) {
+calIcalComponent::GetIcalComponent(JS::MutableHandle<JS::Value>) {
   return NS_ERROR_NOT_IMPLEMENTED;
 }
 
 NS_IMETHODIMP
-calIcalComponent::SetIcalComponent(JS::HandleValue) {
+calIcalComponent::SetIcalComponent(JS::Handle<JS::Value>) {
   return NS_ERROR_NOT_IMPLEMENTED;
 }
 
@@ -449,12 +454,12 @@ calIcalComponent::GetParent(calIIcalComponent** parent) {
 }
 
 NS_IMETHODIMP
-calIcalComponent::GetIcalTimezone(JS::MutableHandleValue) {
+calIcalComponent::GetIcalTimezone(JS::MutableHandle<JS::Value>) {
   return NS_ERROR_NOT_IMPLEMENTED;
 }
 
 NS_IMETHODIMP
-calIcalComponent::SetIcalTimezone(JS::HandleValue) {
+calIcalComponent::SetIcalTimezone(JS::Handle<JS::Value>) {
   return NS_ERROR_NOT_IMPLEMENTED;
 }
 
@@ -464,7 +469,7 @@ calIcalComponent::AddTimezoneReference(calITimezone* aTimezone) {
   nsAutoCString tzid;
   nsresult rv = aTimezone->GetTzid(tzid);
   NS_ENSURE_SUCCESS(rv, rv);
-  mReferencedTimezones.Put(tzid, aTimezone);
+  mReferencedTimezones.InsertOrUpdate(tzid, aTimezone);
 
   return NS_OK;
 }
@@ -670,12 +675,12 @@ calIcalProperty::GetParent(calIIcalComponent** parent) {
 }
 
 NS_IMETHODIMP
-calIcalProperty::GetIcalProperty(JS::MutableHandleValue) {
+calIcalProperty::GetIcalProperty(JS::MutableHandle<JS::Value>) {
   return NS_ERROR_NOT_IMPLEMENTED;
 }
 
 NS_IMETHODIMP
-calIcalProperty::SetIcalProperty(JS::HandleValue) {
+calIcalProperty::SetIcalProperty(JS::Handle<JS::Value>) {
   return NS_ERROR_NOT_IMPLEMENTED;
 }
 
