@@ -2,10 +2,9 @@
  * License, v. 2.0. If a copy of the MPL was not distributed with this
  * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
-/* import-globals-from item-editing/calendar-item-editing.js */
-/* import-globals-from calendar-command-controller.js */
-/* import-globals-from calendar-management.js */
-/* import-globals-from calendar-views-utils.js */
+/* globals getSelectedCalendar, getSelectedItems, promptOccurrenceModification,
+   calendarViewController, currentView, startBatchTransaction, doTransaction,
+   endBatchTransaction */
 
 var { Services } = ChromeUtils.import("resource://gre/modules/Services.jsm");
 var { cal } = ChromeUtils.import("resource:///modules/calendar/calUtils.jsm");
@@ -30,8 +29,7 @@ function canPaste() {
       return false;
     }
   } else {
-    let calendars = cal
-      .getCalendarManager()
+    let calendars = cal.manager
       .getCalendars()
       .filter(cal.acl.isCalendarWritable)
       .filter(cal.acl.userCanAddItemsToCalendar);
@@ -232,8 +230,7 @@ function pasteFromClipboard() {
         let validPasteText = pasteText != "paste" && !pasteText.endsWith("Item");
         pasteText += items.length == withAttendees.length ? "Only" : "Also";
 
-        let calendars = cal
-          .getCalendarManager()
+        let calendars = cal.manager
           .getCalendars()
           .filter(cal.acl.isCalendarWritable)
           .filter(cal.acl.userCanAddItemsToCalendar)
