@@ -2,30 +2,20 @@
  * License, v. 2.0. If a copy of the MPL was not distributed with this
  * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
-/* import-globals-from ../../../../toolkit/content/globalOverlay.js */
-/* import-globals-from ../../../mail/base/content/utilityOverlay.js */
-/* import-globals-from ../../../mail/base/content/mailWindow.js */
-/* import-globals-from item-editing/calendar-item-editing.js */
-/* import-globals-from agenda-listbox-utils.js */
-/* import-globals-from calendar-clipboard.js */
-/* import-globals-from calendar-management.js */
-/* import-globals-from calendar-modes.js */
-/* import-globals-from calendar-tabs.js */
-/* import-globals-from calendar-task-tree-utils.js */
-/* import-globals-from calendar-ui-utils.js */
-/* import-globals-from calendar-unifinder.js */
-/* import-globals-from calendar-views-utils.js */
-/* import-globals-from import-export.js */
-/* import-globals-from publish.js */
+/* globals goUpdateCommand, currentView, TodayPane, createEventWithDialog,
+   getSelectedCalendar, editSelectedEvents, viewSelectedEvents,
+   modifyTaskFromContext, deleteSelectedEvents, deleteToDoCommand,
+   createTodoWithDialog, deleteToDoCommand, promptDeleteCalendar,
+   toImport, loadEventsFromFile, exportEntireCalendar, saveEventsToFile,
+   publishEntireCalendar, publishCalendarData, toggleUnifinder, toggleOrientation
+   toggleWorkdaysOnly, switchCalendarView, getTaskTree, selectAllEvents,
+   gCurrentMode, getSelectedTasks, canPaste, goSetMenuValue, canUndo, canRedo,
+   cutToClipboard, copyToClipboard, pasteFromClipboard, undo, redo,
+   setupAttendanceMenu, goUpdateCommand, PrintUtils */
 
 var { Services } = ChromeUtils.import("resource://gre/modules/Services.jsm");
 var { cal } = ChromeUtils.import("resource:///modules/calendar/calUtils.jsm");
 
-/* exported injectCalendarCommandController, removeCalendarCommandController,
- *          setupContextItemType, getSelectedItems,
- *          deleteSelectedItems, calendarUpdateNewItemsCommand
- */
-
 var CalendarDeleteCommandEnabled = false;
 var CalendarNewEventsCommandEnabled = false;
 var CalendarNewTasksCommandEnabled = false;
@@ -41,6 +31,7 @@ var calendarController = {
     // Common commands
     "calendar_new_event_command",
     "calendar_new_event_context_command",
+    "calendar_new_event_todaypane_command",
     "calendar_modify_event_command",
     "calendar_view_event_command",
     "calendar_delete_event_command",
@@ -136,7 +127,7 @@ var calendarController = {
       case "calendar_new_event_todaypane_command":
         return CalendarNewEventsCommandEnabled;
       case "calendar_modify_focused_item_command":
-        return this.item_selected;
+        return this.item_selected && canEditSelectedItems();
       case "calendar_modify_event_command":
         return this.item_selected && canEditSelectedItems();
       case "calendar_view_event_command":
@@ -266,8 +257,8 @@ var calendarController = {
           if (aCommand == "cmd_delete" || aCommand == "button_delete") {
             let focusedElement = document.commandDispatcher.focusedElement;
             if (focusedElement) {
-              if (focusedElement.getAttribute("id") == "agenda-listbox") {
-                return agendaListbox.isEventSelected();
+              if (focusedElement == TodayPane.agenda) {
+                return TodayPane.agenda.selectedIndex > -1;
               } else if (focusedElement.className == "calendar-task-tree") {
                 return this.writable && this.todo_items_selected && this.todo_items_writable;
               }
@@ -304,6 +295,9 @@ var calendarController = {
         createEventWithDialog(getSelectedCalendar(), newStart, null, null, null, newStart.isDate);
         break;
       }
+      case "calendar_new_event_todaypane_command":
+        createEventWithDialog(getSelectedCalendar(), cal.dtz.getDefaultStartDate(TodayPane.start));
+        break;
       case "calendar_modify_event_command":
         editSelectedEvents();
         break;
@@ -314,15 +308,12 @@ var calendarController = {
         let focusedElement = document.commandDispatcher.focusedElement;
         if (!focusedElement && this.defaultController && !this.isCalendarInForeground()) {
           this.defaultController.doCommand(aCommand);
-        } else {
-          let focusedRichListbox = cal.view.getParentNodeOrThis(focusedElement, "richlistbox");
-          if (focusedRichListbox && focusedRichListbox.id == "agenda-listbox") {
-            agendaListbox.editSelectedItem();
-          } else if (focusedElement && focusedElement.className == "calendar-task-tree") {
-            modifyTaskFromContext();
-          } else if (this.isInMode("calendar")) {
-            editSelectedEvents();
-          }
+        } else if (focusedElement == TodayPane.agenda) {
+          TodayPane.agenda.editSelectedItem();
+        } else if (focusedElement && focusedElement.className == "calendar-task-tree") {
+          modifyTaskFromContext();
+        } else if (this.isInMode("calendar")) {
+          editSelectedEvents();
         }
         break;
       }
@@ -333,15 +324,12 @@ var calendarController = {
         let focusedElement = document.commandDispatcher.focusedElement;
         if (!focusedElement && this.defaultController && !this.isCalendarInForeground()) {
           this.defaultController.doCommand(aCommand);
-        } else {
-          let focusedRichListbox = cal.view.getParentNodeOrThis(focusedElement, "richlistbox");
-          if (focusedRichListbox && focusedRichListbox.id == "agenda-listbox") {
-            agendaListbox.deleteSelectedItem(false);
-          } else if (focusedElement && focusedElement.className == "calendar-task-tree") {
-            deleteToDoCommand(null, false);
-          } else if (this.isInMode("calendar")) {
-            deleteSelectedEvents();
-          }
+        } else if (focusedElement == TodayPane.agenda) {
+          TodayPane.agenda.deleteSelectedItem(false);
+        } else if (focusedElement && focusedElement.className == "calendar-task-tree") {
+          deleteToDoCommand(false);
+        } else if (this.isInMode("calendar")) {
+          deleteSelectedEvents();
         }
         break;
       }
@@ -368,17 +356,17 @@ var calendarController = {
           null,
           null,
           null,
-          cal.dtz.getDefaultStartDate(agendaListbox.today.start)
+          cal.dtz.getDefaultStartDate(TodayPane.start)
         );
         break;
       case "calendar_delete_todo_command":
         deleteToDoCommand();
         break;
       case "calendar_modify_todo_command":
-        modifyTaskFromContext(null, cal.dtz.getDefaultStartDate(currentView().selectedDay));
+        modifyTaskFromContext(cal.dtz.getDefaultStartDate(currentView().selectedDay));
         break;
       case "calendar_modify_todo_todaypane_command":
-        modifyTaskFromContext(null, cal.dtz.getDefaultStartDate(agendaListbox.today.start));
+        modifyTaskFromContext(cal.dtz.getDefaultStartDate(TodayPane.start));
         break;
 
       case "calendar_new_calendar_command":
@@ -392,7 +380,11 @@ var calendarController = {
         break;
 
       case "calendar_import_command":
-        loadEventsFromFile();
+        if (Services.prefs.getBoolPref("mail.import.in_new_tab")) {
+          toImport("calendar");
+        } else {
+          loadEventsFromFile();
+        }
         break;
       case "calendar_export_command":
         exportEntireCalendar();
@@ -552,10 +544,7 @@ var calendarController = {
    * calendar.
    */
   get writable() {
-    return cal
-      .getCalendarManager()
-      .getCalendars()
-      .some(cal.acl.isCalendarWritable);
+    return cal.manager.getCalendars().some(cal.acl.isCalendarWritable);
   },
 
   /**
@@ -570,8 +559,7 @@ var calendarController = {
    * Returns a boolean indicating if all calendars are readonly.
    */
   get all_readonly() {
-    let calMgr = cal.getCalendarManager();
-    return calMgr.readOnlyCalendarCount == calMgr.calendarCount;
+    return cal.manager.readOnlyCalendarCount == cal.manager.calendarCount;
   },
 
   /**
@@ -580,8 +568,7 @@ var calendarController = {
    * or a file URL, but both are reloadable.
    */
   get has_enabled_reloadable_calendars() {
-    return cal
-      .getCalendarManager()
+    return cal.manager
       .getCalendars()
       .some(
         calendar =>
@@ -595,8 +582,7 @@ var calendarController = {
    * network access.
    */
   get has_local_calendars() {
-    let calMgr = cal.getCalendarManager();
-    return calMgr.networkCalendarCount < calMgr.calendarCount;
+    return cal.manager.networkCalendarCount < cal.manager.calendarCount;
   },
 
   /**
@@ -604,8 +590,7 @@ var calendarController = {
    * network access.
    */
   get has_cached_calendars() {
-    let calMgr = cal.getCalendarManager();
-    let calendars = calMgr.getCalendars();
+    let calendars = cal.manager.getCalendars();
     for (let calendar of calendars) {
       if (calendar.getProperty("cache.enabled") || calendar.getProperty("cache.always")) {
         return true;
@@ -618,7 +603,7 @@ var calendarController = {
    * Returns a boolean indicating that there is only one calendar left.
    */
   get last_calendar() {
-    return cal.getCalendarManager().calendarCount < 2;
+    return cal.manager.calendarCount < 2;
   },
 
   /**
@@ -627,7 +612,7 @@ var calendarController = {
   get all_local_calendars_readonly() {
     // We might want to speed this part up by keeping track of this in the
     // calendar manager.
-    let calendars = cal.getCalendarManager().getCalendars();
+    let calendars = cal.manager.getCalendars();
     let count = calendars.length;
     for (let calendar of calendars) {
       if (!cal.acl.isCalendarWritable(calendar)) {
@@ -904,8 +889,7 @@ function deleteSelectedItems() {
 
 function calendarUpdateNewItemsCommand() {
   // Re-calculate command status.
-  let calendars = cal
-    .getCalendarManager()
+  let calendars = cal.manager
     .getCalendars()
     .filter(cal.acl.isCalendarWritable)
     .filter(cal.acl.userCanAddItemsToCalendar);
@@ -916,6 +900,7 @@ function calendarUpdateNewItemsCommand() {
   [
     "calendar_new_event_command",
     "calendar_new_event_context_command",
+    "calendar_new_event_todaypane_command",
     "calendar_new_todo_command",
     "calendar_new_todo_context_command",
     "calendar_new_todo_todaypane_command",
