@@ -2,8 +2,7 @@
  * License, v. 2.0. If a copy of the MPL was not distributed with this
  * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
-/* global MozElements, MozXULElement, setBooleanAttribute, onMouseOverItem
-   invokeEventDragSession, setElementValue */
+/* global MozElements, MozXULElement, onMouseOverItem, invokeEventDragSession */
 
 "use strict";
 
@@ -22,11 +21,7 @@
   class MozCalendarEditableItem extends MozXULElement {
     static get inheritedAttributes() {
       return {
-        ".calendar-event-box-container":
-          "readonly,flashing,alarm,allday,priority,progress,status,calendar,categories",
-        ".calendar-category-box": "categories",
         ".alarm-icons-box": "flashing",
-        ".calendar-event-details > vbox": "context",
       };
     }
     constructor() {
@@ -62,7 +57,8 @@
         if (
           this.selected &&
           !(event.ctrlKey || event.metaKey) &&
-          cal.acl.isCalendarWritable(this.mOccurrence.calendar)
+          cal.acl.isCalendarWritable(this.mOccurrence.calendar) &&
+          !cal.itip.isInvitation(this.mOccurrence)
         ) {
           if (this.editingTimer) {
             clearTimeout(this.editingTimer);
@@ -91,7 +87,11 @@
 
         if (this.calendarView && this.calendarView.controller) {
           let item = event.ctrlKey ? this.mOccurrence.parentItem : this.mOccurrence;
-          this.calendarView.controller.modifyOccurrence(item);
+          if (Services.prefs.getBoolPref("calendar.events.defaultActionEdit", true)) {
+            this.calendarView.controller.modifyOccurrence(item);
+            return;
+          }
+          this.calendarView.controller.viewOccurrence(item);
         }
       });
 
@@ -102,9 +102,9 @@
         }
       });
 
-      // We have two event listeners for dragstart. This event listener is for the capturing phase.
+      // We have two event listeners for dragstart. This event listener is for the bubbling phase.
       this.addEventListener("dragstart", event => {
-        if (document.monthDragEvent.localName == "calendar-event-box") {
+        if (document.monthDragEvent?.localName == "calendar-event-box") {
           return;
         }
         let item = this.occurrence;
@@ -130,59 +130,26 @@
       }
       this.appendChild(
         MozXULElement.parseXULToFragment(`
-          <vbox flex="1">
-            <hbox>
-              <box class="calendar-color-box"
-                   flex="1">
-                <box class="calendar-event-selection"
-                     orient="horizontal"
-                     flex="1">
-                  <stack class="calendar-event-box-container"
-                         flex="1">
-                    <hbox class="calendar-event-details">
-                      <vbox align="start"
-                            flex="1">
-                        <label class="event-name-label"
-                               crop="end"
-                               style="margin: 0;">
-                        </label>
-                        <html:input class="calendar-event-details-core title-desc"
-                                    hidden="hidden"
-                                    placeholder='${cal.l10n.getCalString("newEvent")}'
-                                    style="background: transparent !important;"/>
-                        <label crop="end"
-                               class="calendar-event-details-core location-desc">
-                        </label>
-                        <spacer flex="1">
-                        </spacer>
-                      </vbox>
-                      <hbox>
-                        <hbox align="center">
-                          <hbox class="alarm-icons-box"
-                                align="center">
-                          </hbox>
-                          <image class="item-classification-box"
-                                 pack="end">
-                          </image>
-                        </hbox>
-                        <hbox class="calendar-category-box category-color-box calendar-event-selection"
-                              flex="1" pack="end">
-                          <image class="calendar-category-box-gradient">
-                          </image>
-                        </hbox>
-                      </hbox>
-                    </hbox>
-                  </stack>
-                </box>
-              </box>
-            </hbox>
-          </vbox>
+          <html:div class="calendar-item-flex">
+            <html:img class="item-type-icon" alt="" />
+            <html:div class="event-name-label"></html:div>
+            <html:input class="plain event-name-input"
+                        hidden="hidden"
+                        placeholder='${cal.l10n.getCalString("newEvent")}'/>
+            <html:div class="alarm-icons-box"></html:div>
+            <html:img class="item-classification-icon" />
+            <html:img class="item-recurrence-icon" />
+          </html:div>
+          <html:div class="location-desc"></html:div>
+          <html:div class="calendar-category-box"></html:div>
         `)
       );
 
-      // We have two event listeners for dragstart. This event listener is for the bubbling phase
+      this.classList.add("calendar-color-box", "calendar-item-container");
+
+      // We have two event listeners for dragstart. This event listener is for the capturing phase
       // where we are setting up the document.monthDragEvent which will be used in the event listener
-      // in the capturing phase.
+      // in the bubbling phase.
       this.addEventListener(
         "dragstart",
         event => {
@@ -210,11 +177,12 @@
       if (val && !this.mSelected) {
         this.mSelected = true;
         this.setAttribute("selected", "true");
+        this.focus();
       } else if (!val && this.mSelected) {
         this.mSelected = false;
         this.removeAttribute("selected");
+        this.blur();
       }
-      return val;
     }
 
     get selected() {
@@ -223,7 +191,6 @@
 
     set calendarView(val) {
       this.mCalendarView = val;
-      return val;
     }
 
     get calendarView() {
@@ -235,7 +202,6 @@
       this.setEditableLabel();
       this.setLocationLabel();
       this.setCSSClasses();
-      return val;
     }
 
     get occurrence() {
@@ -247,7 +213,7 @@
     }
 
     get eventNameTextbox() {
-      return this.querySelector(".title-desc");
+      return this.querySelector(".event-name-input");
     }
 
     addEventNameTextboxListener() {
@@ -280,7 +246,7 @@
     setEditableLabel() {
       let label = this.eventNameLabel;
       let item = this.mOccurrence;
-      label.value = item.title
+      label.textContent = item.title
         ? item.title.replace(/\n/g, " ")
         : cal.l10n.getCalString("eventUntitled");
     }
@@ -290,8 +256,8 @@
       let location = this.mOccurrence.getProperty("LOCATION");
       let showLocation = Services.prefs.getBoolPref("calendar.view.showLocation", false);
 
-      locationLabel.value = showLocation && location ? location : "";
-      setBooleanAttribute(locationLabel, "hidden", !showLocation || !location);
+      locationLabel.textContent = showLocation && location ? location : "";
+      locationLabel.hidden = !showLocation || !location;
     }
 
     setCSSClasses() {
@@ -299,53 +265,99 @@
       let cssSafeId = cal.view.formatStringForCSSRule(item.calendar.id);
       this.style.setProperty("--item-backcolor", `var(--calendar-${cssSafeId}-backcolor)`);
       this.style.setProperty("--item-forecolor", `var(--calendar-${cssSafeId}-forecolor)`);
-      let categoriesArray = item.getCategories();
-      if (categoriesArray.length > 0) {
-        let cssClassesArray = categoriesArray.map(cal.view.formatStringForCSSRule);
-        this.setAttribute("categories", cssClassesArray.join(" "));
-        let categoriesBox = this.querySelector(".calendar-category-box");
-        categoriesBox.style.backgroundColor = `var(--category-${cssClassesArray[0]}-color)`;
+      let categoriesBox = this.querySelector(".calendar-category-box");
+
+      let categoriesArray = item.getCategories().map(cal.view.formatStringForCSSRule);
+      // Find the first category with a colour.
+      let firstCategory = categoriesArray.find(
+        category => Services.prefs.getStringPref("calendar.category.color." + category, "") != ""
+      );
+      if (firstCategory) {
+        categoriesBox.hidden = false;
+        categoriesBox.style.backgroundColor = `var(--category-${firstCategory}-color)`;
+      } else {
+        categoriesBox.hidden = true;
       }
 
       // Add alarm icons as needed.
       let alarms = item.getAlarms();
       if (alarms.length && Services.prefs.getBoolPref("calendar.alarms.indicator.show", true)) {
         let iconsBox = this.querySelector(".alarm-icons-box");
-        cal.alarms.addReminderImages(iconsBox, alarms);
-
         // Set suppressed status on the icons box.
-        setElementValue(
-          iconsBox,
-          item.calendar.getProperty("suppressAlarms") || false,
-          "suppressed"
-        );
+        iconsBox.toggleAttribute("suppressed", item.calendar.getProperty("suppressAlarms"));
+
+        cal.alarms.addReminderImages(iconsBox, alarms);
       }
 
       // Item classification / privacy.
-      let classificationBox = this.querySelector(".item-classification-box");
-      if (classificationBox) {
-        classificationBox.setAttribute("classification", item.privacy || "PUBLIC");
+      let classificationIcon = this.querySelector(".item-classification-icon");
+      if (classificationIcon) {
+        switch (item.privacy) {
+          case "PRIVATE":
+            classificationIcon.setAttribute(
+              "src",
+              "chrome://calendar/skin/shared/icons/private.svg"
+            );
+            // Set the alt attribute.
+            document.l10n.setAttributes(
+              classificationIcon,
+              "calendar-editable-item-privacy-icon-private"
+            );
+            break;
+          case "CONFIDENTIAL":
+            classificationIcon.setAttribute(
+              "src",
+              "chrome://calendar/skin/shared/icons/confidential.svg"
+            );
+            // Set the alt attribute.
+            document.l10n.setAttributes(
+              classificationIcon,
+              "calendar-editable-item-privacy-icon-confidential"
+            );
+            break;
+          default:
+            classificationIcon.removeAttribute("src");
+            classificationIcon.removeAttribute("data-l10n-id");
+            classificationIcon.setAttribute("alt", "");
+            break;
+        }
       }
 
-      // Set up event box attributes for use in css selectors. Note if
-      // something is added here, it should also be inherited correctly
-      // in the <content> section of this custom element, and all that inherit it.
+      let recurrenceIcon = this.querySelector(".item-recurrence-icon");
+      if (item.parentItem != item && item.parentItem.recurrenceInfo) {
+        if (item.parentItem.recurrenceInfo.getExceptionFor(item.recurrenceId)) {
+          recurrenceIcon.setAttribute(
+            "src",
+            "chrome://messenger/skin/icons/new/recurrence-exception.svg"
+          );
+          document.l10n.setAttributes(
+            recurrenceIcon,
+            "calendar-editable-item-recurrence-exception"
+          );
+        } else {
+          recurrenceIcon.setAttribute("src", "chrome://messenger/skin/icons/new/recurrence.svg");
+          document.l10n.setAttributes(recurrenceIcon, "calendar-editable-item-recurrence");
+        }
+        recurrenceIcon.hidden = false;
+      } else {
+        recurrenceIcon.removeAttribute("src");
+        recurrenceIcon.removeAttribute("data-l10n-id");
+        recurrenceIcon.setAttribute("alt", "");
+        recurrenceIcon.hidden = true;
+      }
 
       // Event type specific properties.
-      if (cal.item.isEvent(item)) {
-        if (item.startDate.isDate) {
-          this.setAttribute("allday", "true");
-        }
-        this.setAttribute("itemType", "event");
-      } else if (cal.item.isToDo(item)) {
-        // Progress attribute.
-        this.setAttribute("progress", cal.item.getProgressAtom(item));
-        // Attribute for tasks and tasks image.
-        this.setAttribute("itemType", "todo");
-        if (item.entryDate && !item.dueDate) {
-          this.setAttribute("todoType", "start");
-        } else if (!item.entryDate && item.dueDate) {
-          this.setAttribute("todoType", "end");
+      if (item.isEvent() && item.startDate.isDate) {
+        this.setAttribute("allday", "true");
+      }
+      if (item.isTodo()) {
+        let icon = this.querySelector(".item-type-icon");
+        if (cal.item.getProgressAtom(item) === "completed") {
+          icon.setAttribute("src", "chrome://calendar/skin/shared/todo-complete.svg");
+          document.l10n.setAttributes(icon, "calendar-editable-item-todo-icon-completed-task");
+        } else {
+          icon.setAttribute("src", "chrome://calendar/skin/shared/todo.svg");
+          document.l10n.setAttributes(icon, "calendar-editable-item-todo-icon-task");
         }
       }
 
@@ -383,9 +395,6 @@
           "invitation-status",
           cal.itip.getInvitedAttendee(item).participationStatus
         );
-        this.setAttribute("readonly", "true");
-      } else if (!cal.acl.isCalendarWritable(item.calendar)) {
-        this.setAttribute("readonly", "true");
       }
     }
 
@@ -393,15 +402,19 @@
       this.editingTimer = null;
       this.mOriginalTextLabel = this.mOccurrence.title;
 
-      this.eventNameLabel.setAttribute("hidden", "true");
+      this.eventNameLabel.hidden = true;
 
       this.mEditing = true;
 
       this.eventNameTextbox.value = this.mOriginalTextLabel;
-      this.eventNameTextbox.removeAttribute("hidden");
+      this.eventNameTextbox.hidden = false;
       this.eventNameTextbox.focus();
     }
 
+    get isEditing() {
+      return this.mEditing || false;
+    }
+
     select(event) {
       if (!this.calendarView) {
         return;
@@ -441,8 +454,8 @@
         return;
       }
 
-      this.eventNameTextbox.setAttribute("hidden", "hidden");
-      this.eventNameLabel.removeAttribute("hidden");
+      this.eventNameTextbox.hidden = true;
+      this.eventNameLabel.hidden = false;
     }
   }
 
