@@ -2,16 +2,13 @@
  * License, v. 2.0. If a copy of the MPL was not distributed with this
  * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
-/* exported promptDeleteCalendar, loadCalendarManager, unloadCalendarManager,
- *         calendarListTooltipShowing, calendarListSetupContextMenu,
- *         ensureCalendarVisible, toggleCalendarVisible, showAllCalendars,
- *         showOnlyCalendar, openCalendarSubscriptionsDialog,
- *         calendarOfflineManager
- */
+/* globals sortCalendarArray, gDataMigrator, calendarUpdateNewItemsCommand, currentView */
 
-/* import-globals-from calendar-migration.js */
-/* import-globals-from calendar-command-controller.js */
-/* import-globals-from calendar-ui-utils.js */
+/* exported promptDeleteCalendar, loadCalendarManager, unloadCalendarManager,
+ *   calendarListTooltipShowing, calendarListSetupContextMenu,
+ *   ensureCalendarVisible, toggleCalendarVisible, showAllCalendars,
+ *   showOnlyCalendar, calendarOfflineManager, openLocalCalendar,
+ */
 
 var { cal } = ChromeUtils.import("resource:///modules/calendar/calUtils.jsm");
 var { Services } = ChromeUtils.import("resource://gre/modules/Services.jsm");
@@ -36,11 +33,7 @@ function getSelectedCalendar() {
  * @param aCalendar     The calendar to delete.
  */
 function promptDeleteCalendar(aCalendar) {
-  const nIPS = Ci.nsIPromptService;
-  const cICM = Ci.calICalendarManager;
-
-  let calMgr = cal.getCalendarManager();
-  let calendars = calMgr.getCalendars();
+  let calendars = cal.manager.getCalendars();
   if (calendars.length <= 1) {
     // If this is the last calendar, don't delete it.
     return;
@@ -52,20 +45,21 @@ function promptDeleteCalendar(aCalendar) {
   let textKey, b0text, b2text;
   let removeFlags = 0;
   let promptFlags =
-    nIPS.BUTTON_POS_0 * nIPS.BUTTON_TITLE_IS_STRING + nIPS.BUTTON_POS_1 * nIPS.BUTTON_TITLE_CANCEL;
+    Ci.nsIPromptService.BUTTON_POS_0 * Ci.nsIPromptService.BUTTON_TITLE_IS_STRING +
+    Ci.nsIPromptService.BUTTON_POS_1 * Ci.nsIPromptService.BUTTON_TITLE_CANCEL;
 
   if (modes.has("delete") && !modes.has("unsubscribe")) {
     textKey = "removeCalendarMessageDelete";
-    promptFlags += nIPS.BUTTON_DELAY_ENABLE;
+    promptFlags += Ci.nsIPromptService.BUTTON_DELAY_ENABLE;
     b0text = cal.l10n.getCalString("removeCalendarButtonDelete");
   } else if (modes.has("delete")) {
     textKey = "removeCalendarMessageDeleteOrUnsubscribe";
-    promptFlags += nIPS.BUTTON_POS_2 * nIPS.BUTTON_TITLE_IS_STRING;
+    promptFlags += Ci.nsIPromptService.BUTTON_POS_2 * Ci.nsIPromptService.BUTTON_TITLE_IS_STRING;
     b0text = cal.l10n.getCalString("removeCalendarButtonUnsubscribe");
     b2text = cal.l10n.getCalString("removeCalendarButtonDelete");
   } else if (modes.has("unsubscribe")) {
     textKey = "removeCalendarMessageUnsubscribe";
-    removeFlags |= cICM.REMOVE_NO_DELETE;
+    removeFlags |= Ci.calICalendarManager.REMOVE_NO_DELETE;
     b0text = cal.l10n.getCalString("removeCalendarButtonUnsubscribe");
   } else {
     return;
@@ -89,17 +83,40 @@ function promptDeleteCalendar(aCalendar) {
     if (textKey == "removeCalendarMessageDeleteOrUnsubscribe" && res == 0) {
       // Both unsubscribing and deleting is possible, but unsubscribing was
       // requested. Make sure no delete is executed.
-      removeFlags |= cICM.REMOVE_NO_DELETE;
+      removeFlags |= Ci.calICalendarManager.REMOVE_NO_DELETE;
     }
 
-    calMgr.removeCalendar(aCalendar, removeFlags);
+    cal.manager.removeCalendar(aCalendar, removeFlags);
+  }
+}
+
+/**
+ * Call to refresh the status image of a calendar item when the
+ * calendar-readfailed or calendar-readonly attributes are added or removed.
+ *
+ * @param {MozRichlistitem} item - The calendar item to update.
+ */
+function updatedCalendarReadStatus(item) {
+  let calendarName = item.querySelector(".calendar-name").textContent;
+  let image = item.querySelector("img.calendar-readstatus");
+  if (item.hasAttribute("calendar-readfailed")) {
+    image.setAttribute("src", "chrome://messenger/skin/icons/new/compact/warning.svg");
+    let tooltip = cal.l10n.getCalString("tooltipCalendarDisabled", [calendarName]);
+    image.setAttribute("title", tooltip);
+  } else if (item.hasAttribute("calendar-readonly")) {
+    image.setAttribute("src", "chrome://messenger/skin/icons/new/compact/lock.svg");
+    let tooltip = cal.l10n.getCalString("tooltipCalendarReadOnly", [calendarName]);
+    image.setAttribute("title", tooltip);
+  } else {
+    image.removeAttribute("src");
+    image.removeAttribute("title");
   }
 }
 
 /**
  * Called to initialize the calendar manager for a window.
  */
-function loadCalendarManager() {
+async function loadCalendarManager() {
   let calendarList = document.getElementById("calendar-list");
 
   // Set up the composite calendar in the calendar list widget.
@@ -109,7 +126,7 @@ function loadCalendarManager() {
   compositeCalendar.addObserver(compositeObserver);
 
   // Create the home calendar if no calendar exists.
-  let calendars = cal.getCalendarManager().getCalendars();
+  let calendars = cal.manager.getCalendars();
   if (calendars.length) {
     // migration code to make sure calendars, which do not support caching have cache enabled
     // required to further clean up on top of bug 1182264
@@ -125,63 +142,49 @@ function loadCalendarManager() {
     initHomeCalendar();
   }
 
-  let calendarManager = cal.getCalendarManager();
-
-  for (let calendar of sortCalendarArray(calendarManager.getCalendars())) {
+  for (let calendar of sortCalendarArray(cal.manager.getCalendars())) {
     addCalendarItem(calendar);
   }
 
-  reportCalendars();
-
   function addCalendarItem(calendar) {
-    let item = document.createXULElement("richlistitem");
+    let item = document
+      .getElementById("calendar-list-item")
+      .content.firstElementChild.cloneNode(true);
+    item.id = `calendar-listitem-${calendar.id}`;
     item.searchLabel = calendar.name;
+    item.setAttribute("aria-label", calendar.name);
     item.setAttribute("calendar-id", calendar.id);
-    if (calendar.getProperty("disabled")) {
-      item.setAttribute("calendar-disabled", "true");
-    }
-    if (!Components.isSuccessCode(calendar.getProperty("currentStatus"))) {
-      item.setAttribute("calendar-readfailed", "true");
-    }
-    if (calendar.readOnly) {
-      item.setAttribute("calendar-readonly", "true");
-    }
+    item.toggleAttribute("calendar-disabled", calendar.getProperty("disabled"));
+    item.toggleAttribute(
+      "calendar-readfailed",
+      !Components.isSuccessCode(calendar.getProperty("currentStatus"))
+    );
+    item.toggleAttribute("calendar-readonly", calendar.readOnly);
 
     let cssSafeId = cal.view.formatStringForCSSRule(calendar.id);
-    let image = document.createXULElement("image");
-    image.classList.add("calendar-color");
-    item.appendChild(image);
+    let colorMarker = item.querySelector(".calendar-color");
     if (calendar.getProperty("disabled")) {
-      image.style.backgroundColor = "transparent";
-      image.style.border = `2px solid var(--calendar-${cssSafeId}-backcolor)`;
+      colorMarker.style.backgroundColor = "transparent";
+      colorMarker.style.border = `2px solid var(--calendar-${cssSafeId}-backcolor)`;
     } else {
-      image.style.backgroundColor = `var(--calendar-${cssSafeId}-backcolor)`;
+      colorMarker.style.backgroundColor = `var(--calendar-${cssSafeId}-backcolor)`;
     }
 
-    let label = document.createXULElement("label");
-    label.setAttribute("crop", "end");
-    label.classList.add("calendar-name");
-    label.value = calendar.name;
-    item.appendChild(label);
+    let label = item.querySelector(".calendar-name");
+    label.textContent = calendar.name;
 
-    image = document.createXULElement("image");
-    image.classList.add("calendar-readstatus");
-    image.setAttribute("tooltip", "calendar-list-tooltip");
-    item.appendChild(image);
+    updatedCalendarReadStatus(item);
 
-    let enable = document.createXULElement("button");
+    let enable = item.querySelector(".calendar-enable-button");
     if (calendar.getProperty("disabled")) {
       if (!bundle) {
         bundle = Services.strings.createBundle("chrome://messenger/locale/addons.properties");
       }
-      enable.label = bundle.GetStringFromName("webextPerms.sideloadEnable.label");
+      enable.textContent = bundle.GetStringFromName("webextPerms.sideloadEnable.label");
     }
-    enable.classList.add("calendar-enable-button");
     enable.hidden = !calendar.getProperty("disabled");
-    item.appendChild(enable);
 
-    let displayedCheckbox = document.createXULElement("checkbox");
-    displayedCheckbox.classList.add("calendar-displayed");
+    let displayedCheckbox = item.querySelector(".calendar-displayed");
     displayedCheckbox.checked = calendar.getProperty("calendar-main-in-composite");
     displayedCheckbox.hidden = calendar.getProperty("disabled");
     let stringName = cal.view.getCompositeCalendar(window).getCalendarById(calendar.id)
@@ -191,11 +194,13 @@ function loadCalendarManager() {
       "tooltiptext",
       cal.l10n.getCalString(stringName, [calendar.name])
     );
-    item.appendChild(displayedCheckbox);
 
     calendarList.appendChild(item);
     if (calendar.getProperty("calendar-main-default")) {
-      calendarList.selectedItem = item;
+      // The list needs to handle the addition of the row before we can select it.
+      setTimeout(() => {
+        calendarList.selectedIndex = calendarList.rows.indexOf(item);
+      });
     }
   }
 
@@ -211,21 +216,21 @@ function loadCalendarManager() {
 
   calendarList.addEventListener("click", event => {
     if (event.target.matches(".calendar-enable-button")) {
-      let calendar = calendarManager.getCalendarById(
-        event.target.closest("richlistitem").getAttribute("calendar-id")
+      let calendar = cal.manager.getCalendarById(
+        event.target.closest("li").getAttribute("calendar-id")
       );
       calendar.setProperty("disabled", false);
       calendarList.focus();
       return;
     }
 
-    if (!event.target.matches("checkbox.calendar-displayed")) {
+    if (!event.target.matches(".calendar-displayed")) {
       return;
     }
 
-    let item = event.target.closest("richlistitem");
+    let item = event.target.closest("li");
     let calendarId = item.getAttribute("calendar-id");
-    let calendar = calendarManager.getCalendarById(calendarId);
+    let calendar = cal.manager.getCalendarById(calendarId);
 
     if (event.target.checked) {
       compositeCalendar.addCalendar(calendar);
@@ -240,13 +245,13 @@ function loadCalendarManager() {
   });
   calendarList.addEventListener("dblclick", event => {
     if (
-      event.target.matches("checkbox.calendar-displayed") ||
+      event.target.matches(".calendar-displayed") ||
       event.target.matches(".calendar-enable-button")
     ) {
       return;
     }
 
-    let item = event.target.closest("richlistitem");
+    let item = event.target.closest("li");
     if (!item) {
       // Click on an empty part of the richlistbox.
       cal.window.openCalendarWizard(window);
@@ -254,97 +259,17 @@ function loadCalendarManager() {
     }
 
     let calendarId = item.getAttribute("calendar-id");
-    let calendar = calendarManager.getCalendarById(calendarId);
+    let calendar = cal.manager.getCalendarById(calendarId);
     cal.window.openCalendarProperties(window, calendar);
   });
-  calendarList.addEventListener("dragstart", event => {
-    let item = event.target.closest("richlistitem");
-    if (!item) {
-      return;
-    }
-
-    let calendarId = item.getAttribute("calendar-id");
-    event.dataTransfer.setData("application/x-moz-calendarid", calendarId);
-    event.dataTransfer.effectAllowed = "move";
-  });
-  calendarList.addEventListener("dragenter", event => {
-    if (
-      event.target == calendarList &&
-      event.dataTransfer.types.includes("application/x-moz-calendarid")
-    ) {
-      event.dataTransfer.dropEffect = "move";
-      event.preventDefault();
-    }
-  });
-  calendarList.addEventListener("dragover", event => {
-    event.preventDefault();
-
-    let existing = calendarList.querySelector("[drop-on]");
-    if (existing) {
-      existing.removeAttribute("drop-on");
-    }
-
-    if (event.target == calendarList) {
-      calendarList.lastElementChild.setAttribute("drop-on", "bottom");
-      return;
-    }
-
-    let item = event.target.closest("richlistitem");
-    if (item) {
-      // If we're dragging/dropping in bottom half of attachmentitem,
-      // adjust target to target.nextElementSibling (to show dropmarker above that).
-      if ((event.screenY - item.screenY) / item.getBoundingClientRect().height >= 0.5) {
-        item = item.nextElementSibling;
-      }
-      if (item) {
-        item.setAttribute("drop-on", "top");
-      } else {
-        calendarList.lastElementChild.setAttribute("drop-on", "bottom");
-      }
-    }
-  });
-  calendarList.addEventListener("dragexit", event => {
-    if (event.target != calendarList) {
-      return;
-    }
-
-    let existing = calendarList.querySelector("[drop-on]");
-    if (existing) {
-      existing.removeAttribute("drop-on");
-    }
-  });
-  calendarList.addEventListener("dragend", event => {
-    let existing = calendarList.querySelector("[drop-on]");
-    if (existing) {
-      existing.removeAttribute("drop-on");
-    }
-  });
-  calendarList.addEventListener("drop", event => {
-    let existing = calendarList.querySelector("[drop-on]");
-    let position = existing.getAttribute("drop-on");
-    if (!existing) {
-      return;
-    }
-
-    existing.removeAttribute("drop-on");
-
-    let calendarId = event.dataTransfer.getData("application/x-moz-calendarid");
-    if (calendarId == existing.getAttribute("calendar-id")) {
-      return;
-    }
-
-    let item = calendarList.getElementsByAttribute("calendar-id", calendarId)[0];
-    if (position == "bottom") {
-      existing = null;
-    }
-    calendarList.insertBefore(item, existing);
-
+  calendarList.addEventListener("ordered", event => {
     saveSortOrder();
+    calendarList.selectedIndex = calendarList.rows.indexOf(event.detail);
   });
   calendarList.addEventListener("keypress", event => {
-    let item = calendarList.selectedItem;
+    let item = calendarList.rows[calendarList.selectedIndex];
     let calendarId = item.getAttribute("calendar-id");
-    let calendar = calendarManager.getCalendarById(calendarId);
+    let calendar = cal.manager.getCalendarById(calendarId);
 
     switch (event.key) {
       case "Delete":
@@ -367,15 +292,15 @@ function loadCalendarManager() {
     }
   });
   calendarList.addEventListener("select", event => {
-    let item = calendarList.selectedItem;
+    let item = calendarList.rows[calendarList.selectedIndex];
     let calendarId = item.getAttribute("calendar-id");
-    let calendar = calendarManager.getCalendarById(calendarId);
+    let calendar = cal.manager.getCalendarById(calendarId);
 
     compositeCalendar.defaultCalendar = calendar;
   });
 
   calendarList._calendarObserver = {
-    QueryInterface: ChromeUtils.generateQI([Ci.calIObserver]),
+    QueryInterface: ChromeUtils.generateQI(["calIObserver"]),
 
     onStartBatch() {},
     onEndBatch() {},
@@ -393,43 +318,47 @@ function loadCalendarManager() {
 
       switch (name) {
         case "disabled":
-          setBooleanAttribute(item, "calendar-disabled", value);
-          setBooleanAttribute(item.querySelector(".calendar-displayed"), "hidden", value);
+          item.toggleAttribute("calendar-disabled", value);
+          item.querySelector(".calendar-displayed").hidden = value;
           // Update the "ENABLE" button.
           let enableButton = item.querySelector(".calendar-enable-button");
-          setBooleanAttribute(enableButton, "hidden", !value);
+          enableButton.hidden = !value;
           // We need to set the string if the button was hidden on creation.
-          if (value && enableButton.label == "") {
+          if (value && enableButton.textContent == "") {
             if (!bundle) {
               bundle = Services.strings.createBundle("chrome://messenger/locale/addons.properties");
             }
-            enableButton.label = bundle.GetStringFromName("webextPerms.sideloadEnable.label");
+            enableButton.textContent = bundle.GetStringFromName("webextPerms.sideloadEnable.label");
           }
           // Update the color preview.
           let cssSafeId = cal.view.formatStringForCSSRule(calendar.id);
-          let image = item.querySelector(".calendar-color");
-          image.style.backgroundColor = value
+          let colorMarker = item.querySelector(".calendar-color");
+          colorMarker.style.backgroundColor = value
             ? "transparent"
             : `var(--calendar-${cssSafeId}-backcolor)`;
-          image.style.border = value ? `2px solid var(--calendar-${cssSafeId}-backcolor)` : "none";
+          colorMarker.style.border = value
+            ? `2px solid var(--calendar-${cssSafeId}-backcolor)`
+            : "none";
           break;
         case "calendar-main-default":
           if (value) {
-            calendarList.selectedItem = item;
+            calendarList.selectedIndex = calendarList.rows.indexOf(item);
           }
           break;
         case "calendar-main-in-composite":
-          setBooleanAttribute(item.querySelector(".calendar-displayed"), "checked", value);
+          item.querySelector(".calendar-displayed").checked = value;
           break;
         case "name":
           item.searchLabel = calendar.name;
-          item.querySelector(".calendar-name").value = value;
+          item.querySelector(".calendar-name").textContent = value;
           break;
         case "currentStatus":
-          setBooleanAttribute(item, "calendar-readfailed", !Components.isSuccessCode(value));
+          item.toggleAttribute("calendar-readfailed", !Components.isSuccessCode(value));
+          updatedCalendarReadStatus(item);
           break;
         case "readOnly":
-          setBooleanAttribute(item, "calendar-readonly", value);
+          item.toggleAttribute("calendar-readonly", value);
+          updatedCalendarReadStatus(item);
           break;
       }
     },
@@ -440,10 +369,10 @@ function loadCalendarManager() {
       this.onPropertyChanged(calendar, name, null, null);
     },
   };
-  calendarManager.addCalendarObserver(calendarList._calendarObserver);
+  cal.manager.addCalendarObserver(calendarList._calendarObserver);
 
   calendarList._calendarManagerObserver = {
-    QueryInterface: ChromeUtils.generateQI([Ci.calICalendarManagerObserver]),
+    QueryInterface: ChromeUtils.generateQI(["calICalendarManagerObserver"]),
 
     onCalendarRegistered(calendar) {
       addCalendarItem(calendar);
@@ -456,16 +385,37 @@ function loadCalendarManager() {
     },
     onCalendarDeleting(calendar) {},
   };
-  calendarManager.addObserver(calendarList._calendarManagerObserver);
+  cal.manager.addObserver(calendarList._calendarManagerObserver);
+
+  // Called here to avoid an issue in tests where onPropertyChanged() is called
+  // before a default calendar is set.
+  await reportCalendars();
 }
 
 /**
  * A telemetry probe to report calendar count and read only calendar count.
  */
-function reportCalendars() {
+async function reportCalendars() {
   let telemetryReport = {};
+  let home = cal.l10n.getCalString("homeCalendarName");
 
-  for (let calendar of cal.getCalendarManager().getCalendars()) {
+  for (let calendar of cal.manager.getCalendars()) {
+    if (calendar.name == home && calendar.type == "storage") {
+      // Ignore the "Home" calendar if it is disabled or unused as it's
+      // automatically added.
+      if (calendar.getProperty("disabled")) {
+        continue;
+      }
+      let items = await calendar.getItemsAsArray(
+        Ci.calICalendar.ITEM_FILTER_ALL_ITEMS,
+        1,
+        null,
+        null
+      );
+      if (!items.length) {
+        continue;
+      }
+    }
     if (!telemetryReport[calendar.type]) {
       telemetryReport[calendar.type] = { count: 0, readOnlyCount: 0 };
     }
@@ -489,14 +439,13 @@ function reportCalendars() {
  * Creates the initial "Home" calendar if no calendar exists.
  */
 function initHomeCalendar() {
-  let calMgr = cal.getCalendarManager();
   let composite = cal.view.getCompositeCalendar(window);
   let url = Services.io.newURI("moz-storage-calendar://");
-  let homeCalendar = calMgr.createCalendar("storage", url);
+  let homeCalendar = cal.manager.createCalendar("storage", url);
   homeCalendar.name = cal.l10n.getCalString("homeCalendarName");
   homeCalendar.setProperty("disabled", true);
 
-  calMgr.registerCalendar(homeCalendar);
+  cal.manager.registerCalendar(homeCalendar);
   Services.prefs.setStringPref("calendar.list.sortOrder", homeCalendar.id);
   composite.addCalendar(homeCalendar);
 
@@ -522,33 +471,8 @@ function unloadCalendarManager() {
   compositeCalendar.removeObserver(compositeObserver);
 
   let calendarList = document.getElementById("calendar-list");
-  let calendarManager = cal.getCalendarManager();
-  calendarManager.removeCalendarObserver(calendarList._calendarObserver);
-  calendarManager.removeObserver(calendarList._calendarManagerObserver);
-}
-
-/**
- * Handler function to call when the tooltip is showing on the calendar list.
- *
- * @param event     The DOM event provoked by the tooltip showing.
- */
-function calendarListTooltipShowing(event) {
-  let realTarget = document.elementFromPoint(event.clientX, event.clientY);
-  let item = realTarget.closest("richlistitem");
-  if (!item) {
-    return false;
-  }
-
-  let calendarName = item.querySelector(".calendar-name").value;
-  let tooltipText;
-  if (item.hasAttribute("calendar-readfailed")) {
-    tooltipText = cal.l10n.getCalString("tooltipCalendarDisabled", [calendarName]);
-  } else if (item.hasAttribute("calendar-readonly")) {
-    tooltipText = cal.l10n.getCalString("tooltipCalendarReadOnly", [calendarName]);
-  }
-
-  setElementValue("calendar-list-tooltip", tooltipText, "label");
-  return !!tooltipText;
+  cal.manager.removeCalendarObserver(calendarList._calendarObserver);
+  cal.manager.removeObserver(calendarList._calendarManagerObserver);
 }
 
 /**
@@ -561,34 +485,32 @@ function calendarListSetupContextMenu(event) {
   let calendar;
   let composite = cal.view.getCompositeCalendar(window);
 
-  if (document.popupNode.matches("checkbox.calendar-displayed")) {
+  if (event.target.triggerNode.matches(".calendar-displayed")) {
     return;
   }
 
-  let item = document.popupNode.closest("richlistitem");
+  let item = event.target.triggerNode.closest("li");
   if (item) {
+    let calendarList = document.getElementById("calendar-list");
+    calendarList.selectedIndex = calendarList.rows.indexOf(item);
     let calendarId = item.getAttribute("calendar-id");
-    calendar = cal.getCalendarManager().getCalendarById(calendarId);
+    calendar = cal.manager.getCalendarById(calendarId);
   }
 
   document.getElementById("list-calendars-context-menu").contextCalendar = calendar;
 
   if (calendar) {
     let stringName = composite.getCalendarById(calendar.id) ? "hideCalendar" : "showCalendar";
-    setElementValue(
-      "list-calendars-context-togglevisible",
-      cal.l10n.getCalString(stringName, [calendar.name]),
-      "label"
-    );
+    document.getElementById(
+      "list-calendars-context-togglevisible"
+    ).label = cal.l10n.getCalString(stringName, [calendar.name]);
     let accessKey = document
       .getElementById("list-calendars-context-togglevisible")
       .getAttribute(composite.getCalendarById(calendar.id) ? "accesskeyhide" : "accesskeyshow");
-    setElementValue("list-calendars-context-togglevisible", accessKey, "accesskey");
-    setElementValue(
-      "list-calendars-context-showonly",
-      cal.l10n.getCalString("showOnlyCalendar", [calendar.name]),
-      "label"
-    );
+    document.getElementById("list-calendars-context-togglevisible").accessKey = accessKey;
+    document.getElementById(
+      "list-calendars-context-showonly"
+    ).label = cal.l10n.getCalString("showOnlyCalendar", [calendar.name]);
     setupDeleteMenuitem("list-calendars-context-delete", calendar);
     for (let elem of event.target.querySelectorAll(".needs-calendar")) {
       elem.removeAttribute("collapsed");
@@ -598,10 +520,6 @@ function calendarListSetupContextMenu(event) {
       elem.setAttribute("collapsed", "true");
     }
   }
-
-  // Only enable calendar search if there's actually the chance of finding something:
-  let hasProviders = cal.getCalendarSearchService().getProviders().length < 1 && "true";
-  setElementValue("list-calendars-context-find", hasProviders, "collapsed");
 }
 
 /**
@@ -625,8 +543,10 @@ function setupDeleteMenuitem(aDeleteId, aCalendar) {
   }
 
   let deleteItem = document.getElementById(aDeleteId);
-  setElementValue(deleteItem, deleteItem.getAttribute("label" + type), "label");
-  setElementValue(deleteItem, deleteItem.getAttribute("accesskey" + type), "accesskey");
+  // Dynamically set labelremove, labeldelete, labelunsubscribe
+  deleteItem.label = deleteItem.getAttribute("label" + type);
+  // Dynamically set accesskeyremove, accesskeydelete, accesskeyunsubscribe
+  deleteItem.accessKey = deleteItem.getAttribute("accesskey" + type);
 }
 
 /**
@@ -665,7 +585,7 @@ function toggleCalendarVisible(aCalendar) {
  */
 function showAllCalendars() {
   let composite = cal.view.getCompositeCalendar(window);
-  let cals = cal.getCalendarManager().getCalendars();
+  let cals = cal.manager.getCalendars();
 
   composite.startBatch();
   for (let calendar of cals) {
@@ -696,7 +616,7 @@ function showOnlyCalendar(aCalendar) {
 }
 
 var compositeObserver = {
-  QueryInterface: cal.generateQI([Ci.calIObserver, Ci.calICompositeObserver]),
+  QueryInterface: ChromeUtils.generateQI(["calIObserver", "calICompositeObserver"]),
 
   onStartBatch() {},
   onEndBatch() {},
@@ -742,25 +662,53 @@ var compositeObserver = {
 };
 
 /**
- * Opens the subscriptions dialog modally.
+ * Shows the filepicker and creates a new calendar with a local file using the ICS
+ * provider.
  */
-function openCalendarSubscriptionsDialog() {
-  // the dialog will reset this to auto when it is done loading
-  window.setCursor("wait");
+function openLocalCalendar() {
+  let picker = Cc["@mozilla.org/filepicker;1"].createInstance(Ci.nsIFilePicker);
+  picker.init(window, cal.l10n.getCalString("Open"), Ci.nsIFilePicker.modeOpen);
+  let wildmat = "*.ics";
+  let description = cal.l10n.getCalString("filterIcs", [wildmat]);
+  picker.appendFilter(description, wildmat);
+  picker.appendFilters(Ci.nsIFilePicker.filterAll);
 
-  // open the dialog modally
-  window.openDialog(
-    "chrome://calendar/content/calendar-subscriptions-dialog.xhtml",
-    "_blank",
-    "chrome,titlebar,modal,resizable"
-  );
+  picker.open(rv => {
+    if (rv != Ci.nsIFilePicker.returnOK || !picker.file) {
+      return;
+    }
+
+    let calendars = cal.manager.getCalendars();
+    let calendar = calendars.find(x => x.uri.equals(picker.fileURL));
+    if (!calendar) {
+      calendar = cal.manager.createCalendar("ics", picker.fileURL);
+
+      // Strip ".ics" from filename for use as calendar name.
+      let prettyName = picker.fileURL.spec.match(/([^/:]+)\.ics$/);
+      if (prettyName) {
+        calendar.name = decodeURIComponent(prettyName[1]);
+      } else {
+        calendar.name = cal.l10n.getCalString("untitledCalendarName");
+      }
+
+      cal.manager.registerCalendar(calendar);
+    }
+
+    let calendarList = document.getElementById("calendar-list");
+    for (let index = 0; index < calendarList.rowCount; index++) {
+      if (calendarList.rows[index].getAttribute("calendar-id") == calendar.id) {
+        calendarList.selectedIndex = index;
+        break;
+      }
+    }
+  });
 }
 
 /**
  * Calendar Offline Manager
  */
 var calendarOfflineManager = {
-  QueryInterface: ChromeUtils.generateQI([Ci.nsIObserver]),
+  QueryInterface: ChromeUtils.generateQI(["nsIObserver"]),
 
   init() {
     if (this.initialized) {
