@@ -8,6 +8,8 @@
 
 //                           The calendar view class hierarchy.
 //
+//                                 CalendarFilteredViewMixin
+//                                             |
 //                                     CalendarBaseView
 //                                     /               \
 //             CalendarMultidayBaseView                CalendarMonthBaseView
@@ -36,7 +38,7 @@
 
     goToDay(date) {
       if (!date) {
-        this.refresh();
+        this.relayout();
         return;
       }
       const timezoneDate = date.getInTimezone(this.timezone);
@@ -74,11 +76,11 @@
       this.displayDaysOff = !this.mWorkdaysOnly;
 
       if (!date) {
-        this.refresh();
+        this.relayout();
         return;
       }
       date = date.getInTimezone(this.timezone);
-      const weekStart = cal.getWeekInfoService().getStartOfWeek(date);
+      const weekStart = cal.weekInfoService.getStartOfWeek(date);
       const weekEnd = weekStart.clone();
       weekEnd.day += 6;
       this.setDateRange(weekStart, weekEnd);
@@ -164,7 +166,7 @@
         // start of the week of the day that we're centering around
         // adjusted for the day the week starts on and the number
         // of previous weeks we're supposed to display.
-        const dayStart = cal.getWeekInfoService().getStartOfWeek(date);
+        const dayStart = cal.weekInfoService.getStartOfWeek(date);
         dayStart.day -= 7 * Services.prefs.getIntPref("calendar.previousweeks.inview", 0);
 
         // The last day we're supposed to show.
@@ -173,7 +175,7 @@
         this.setDateRange(dayStart, dayEnd);
         this.selectedDay = date;
       } else {
-        this.refresh();
+        this.relayout();
       }
     }
 
@@ -225,6 +227,9 @@
       this.displayDaysOff = !this.mWorkdaysOnly;
 
       this.showDate(date ? date.getInTimezone(this.timezone) : null);
+      if (!date) {
+        this.setDateBoxRelations();
+      }
     }
 
     getRangeDescription() {
