@@ -2,18 +2,18 @@
  * License, v. 2.0. If a copy of the MPL was not distributed with this
  * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
-/* import-globals-from agenda-listbox-utils.js */
 /* import-globals-from calendar-modes.js */
+/* import-globals-from calendar-tabs.js */
 /* import-globals-from calendar-views-utils.js */
 
-/* globals switchCalendarView */
-
 var { cal } = ChromeUtils.import("resource:///modules/calendar/calUtils.jsm");
+var { Services } = ChromeUtils.import("resource://gre/modules/Services.jsm");
 
 /**
  * Namespace object to hold functions related to the today pane.
  */
 var TodayPane = {
+  isLoaded: false,
   paneViews: null,
   start: null,
   cwlabel: null,
@@ -31,7 +31,7 @@ var TodayPane = {
    * Load Handler, sets up the today pane controls.
    */
   async onLoad() {
-    await agendaListbox.init();
+    this.isLoaded = true;
 
     TodayPane.paneViews = [
       cal.l10n.getCalString("eventsandtasks"),
@@ -39,7 +39,8 @@ var TodayPane = {
       cal.l10n.getCalString("eventsonly"),
     ];
 
-    TodayPane.setShortWeekdays();
+    this.agenda = document.getElementById("agenda");
+
     TodayPane.updateDisplay();
     TodayPane.updateSplitterState();
     TodayPane.previousMode = gCurrentMode;
@@ -73,6 +74,9 @@ var TodayPane = {
    * views. (event+task, task only, event only)
    */
   updateDisplay() {
+    if (!this.isLoaded) {
+      return;
+    }
     let agendaIsVisible = document.getElementById("agenda-panel").isVisible(gCurrentMode);
     let todoIsVisible = document.getElementById("todo-tab-panel").isVisible(gCurrentMode);
     let index = 2;
@@ -95,20 +99,18 @@ var TodayPane = {
     todayHeader.setAttribute("index", index);
     todayHeader.setAttribute("value", this.paneViews[index]);
     let todayPaneSplitter = document.getElementById("today-pane-splitter");
-    setBooleanAttribute(todayPaneSplitter, "hidden", index != 0);
+    todayPaneSplitter.hidden = index != 0;
     let todayIsVisible = document.getElementById("today-pane-panel").isVisible();
 
     // Disable or enable the today pane menuitems that have an attribute
     // name="minidisplay" depending on the visibility of elements.
-    let menu = document.getElementById("ltnTodayPaneMenuPopup");
-    if (menu) {
-      setAttributeToChildren(
-        menu,
-        "disabled",
-        !todayIsVisible || !agendaIsVisible,
-        "name",
-        "minidisplay"
-      );
+    let menupopup = document.getElementById("calTodayPaneMenuPopup");
+    if (menupopup) {
+      for (let child of menupopup.children) {
+        if (child.getAttribute("name") == "minidisplay") {
+          child.disabled = !todayIsVisible || !agendaIsVisible;
+        }
+      }
     }
 
     if (todayIsVisible) {
@@ -122,7 +124,7 @@ var TodayPane = {
       }
       if (todoIsVisible) {
         // Add listener to update the date filters.
-        getViewDeck().addEventListener("dayselect", event => {
+        getViewBox().addEventListener("dayselect", event => {
           this.updateCalendarToDoUnifinder();
         });
         this.updateCalendarToDoUnifinder();
@@ -139,13 +141,17 @@ var TodayPane = {
    */
   updateCalendarToDoUnifinder(filter) {
     let tree = document.getElementById("unifinder-todo-tree");
+    if (!tree.hasBeenVisible) {
+      tree.hasBeenVisible = true;
+      tree.refresh();
+    }
 
     // Set up hiding completed tasks for the unifinder-todo tree
     filter = filter || tree.getAttribute("filterValue") || "throughcurrent";
     tree.setAttribute("filterValue", filter);
 
     document
-      .querySelectorAll('menuitem[command="calendar_task_filter_todaypane_command"][type="radio"]')
+      .querySelectorAll("#task-context-menu-filter-todaypane-popup > menuitem")
       .forEach(item => {
         if (item.getAttribute("value") == filter) {
           item.setAttribute("checked", "true");
@@ -177,7 +183,7 @@ var TodayPane = {
     if (aEvent.button == 0) {
       if (aEvent.target.id == "datevalue-label") {
         switchCalendarView("day", true);
-      } else if (aEvent.target.parentNode.id == "weekdayNameContainer") {
+      } else if (aEvent.target.id == "weekdayNameLabel") {
         switchCalendarView("day", true);
       } else if (aEvent.target.id == "currentWeek-label") {
         switchCalendarView("week", true);
@@ -186,9 +192,7 @@ var TodayPane = {
       } else {
         return;
       }
-      let title = document.getElementById("calendar-tab-button").getAttribute("tooltiptext");
-      document.getElementById("tabmail").openTab("calendar", { title });
-      currentView().goToDay(agendaListbox.today.start);
+      document.getElementById("tabmail").openTab("calendar");
     }
   },
 
@@ -356,18 +360,6 @@ var TodayPane = {
     this.updateDisplay();
   },
 
-  /**
-   * Shows short weekday names in the weekdayNameContainer
-   */
-  setShortWeekdays() {
-    let weekdisplaydeck = document.getElementById("weekdayNameContainer");
-    let children = weekdisplaydeck.children;
-
-    for (let i = 0; i < children.length; i++) {
-      children[i].setAttribute("value", cal.l10n.getDateFmtString(`day.${i + 1}.Mmm`));
-    }
-  },
-
   /**
    * Sets the shown date from a JSDate.
    *
@@ -376,6 +368,7 @@ var TodayPane = {
   setDaywithjsDate(aNewDate) {
     let newdatetime = cal.dtz.jsDateToDateTime(aNewDate, cal.dtz.floating);
     newdatetime = newdatetime.getInTimezone(cal.dtz.defaultTimezone);
+    newdatetime.hour = newdatetime.minute = newdatetime.second = 0;
     this.setDay(newdatetime, true);
   },
 
@@ -401,12 +394,9 @@ var TodayPane = {
     let daylabel = document.getElementById("datevalue-label");
     daylabel.value = this.start.day;
 
-    // Wait until after the initialisation of #weekdayNameContainer,
-    // to avoid its selectedIndex being reset to the wrong value.
-    setTimeout(() => {
-      let weekdaylabel = document.getElementById("weekdayNameContainer");
-      weekdaylabel.selectedIndex = this.start.weekday;
-    }, 0);
+    document
+      .getElementById("weekdayNameLabel")
+      .setAttribute("value", cal.l10n.getDateFmtString(`day.${this.start.weekday + 1}.Mmm`));
 
     let monthnamelabel = document.getElementById("monthNameContainer");
     monthnamelabel.value =
@@ -416,7 +406,7 @@ var TodayPane = {
     currentweeklabel.value =
       cal.l10n.getCalString("shortcalendarweek") +
       " " +
-      cal.getWeekInfoService().getWeekTitle(this.start);
+      cal.weekInfoService.getWeekTitle(this.start);
 
     if (!aDontUpdateMinimonth) {
       try {
@@ -456,7 +446,7 @@ var TodayPane = {
    * start date.
    */
   updatePeriod() {
-    agendaListbox.refreshPeriodDates(this.start.clone());
+    this.agenda.update(this.start);
     if (document.getElementById("todo-tab-panel").isVisible()) {
       this.updateCalendarToDoUnifinder();
     }
@@ -471,11 +461,7 @@ var TodayPane = {
     document.getElementById("today-minimonth-box").setVisible(aSection == "minimonth");
     document.getElementById("mini-day-box").setVisible(aSection == "miniday");
     document.getElementById("today-none-box").setVisible(aSection == "none");
-    setBooleanAttribute(
-      document.getElementById("today-minimonth"),
-      "freebusy",
-      aSection == "minimonth"
-    );
+    document.getElementById("today-minimonth").setAttribute("freebusy", aSection == "minimonth");
   },
 
   /**
@@ -540,13 +526,10 @@ var TodayPane = {
    * Checks if the todayPaneStatusLabel should be hidden.
    */
   showTodayPaneStatusLabel() {
-    let attributeValue =
-      Services.prefs.getBoolPref("calendar.view.showTodayPaneStatusLabel", true) && "false";
-    setElementValue(
-      document.getElementById("calendar-status-todaypane-button"),
-      !attributeValue,
-      "hideLabel"
-    );
+    let hideLabel = !Services.prefs.getBoolPref("calendar.view.showTodayPaneStatusLabel", true);
+    document
+      .getElementById("calendar-status-todaypane-button")
+      .toggleAttribute("hideLabel", hideLabel);
   },
 };
 
