@@ -4,14 +4,11 @@ var { cal } = ChromeUtils.import("resource:///modules/calendar/calUtils.jsm");
 var { MailServices } = ChromeUtils.import("resource:///modules/MailServices.jsm");
 
 add_task(async () => {
-  let manager = cal.getCalendarManager();
-  let calendar = manager.createCalendar("memory", Services.io.newURI("moz-memory-calendar://"));
+  let calendar = CalendarTestUtils.createCalendar("Mochitest", "memory");
   calendar.name = "Mochitest";
   calendar.setProperty("organizerId", "mailto:mochitest@invalid");
-  manager.registerCalendar(calendar);
 
-  let freeBusyService = cal.getFreeBusyService();
-  freeBusyService.addProvider(freeBusyProvider);
+  cal.freeBusyService.addProvider(freeBusyProvider);
 
   let book = MailServices.ab.getDirectoryFromId(
     MailServices.ab.newAddressBook("Mochitest", null, 101)
@@ -51,8 +48,8 @@ add_task(async () => {
   };
 
   registerCleanupFunction(async () => {
-    manager.unregisterCalendar(calendar);
-    freeBusyService.removeProvider(freeBusyProvider);
+    CalendarTestUtils.removeCalendar(calendar);
+    cal.freeBusyService.removeProvider(freeBusyProvider);
     MailServices.ab.deleteAddressBook(book.URI);
   });
 
@@ -260,14 +257,11 @@ add_task(async () => {
 });
 
 add_task(async () => {
-  let manager = cal.getCalendarManager();
-  let calendar = manager.createCalendar("memory", Services.io.newURI("moz-memory-calendar://"));
-  calendar.name = "Mochitest";
+  let calendar = CalendarTestUtils.createCalendar("Mochitest", "memory");
   calendar.setProperty("organizerId", "mailto:mochitest@invalid");
-  manager.registerCalendar(calendar);
 
   registerCleanupFunction(async () => {
-    manager.unregisterCalendar(calendar);
+    CalendarTestUtils.removeCalendar(calendar);
   });
 
   let defaults = {
@@ -283,48 +277,96 @@ add_task(async () => {
     let attendeesDocument = attendeesWindow.document;
 
     let days = attendeesDocument.querySelectorAll("calendar-day");
-    Assert.equal(days.length, 5);
+    Assert.equal(days.length, 16);
     Assert.equal(days[0].date.icalString, expectedFirst);
-    Assert.equal(days[4].date.icalString, expectedLast);
+    Assert.equal(days[15].date.icalString, expectedLast);
 
     await closeAttendeesWindow(attendeesWindow);
   }
 
+  // With the management of the reduced days or not, the format of the dates is different according to the cases.
+  // In case of a reduced day, the day format will include the start hour of the day (defined by calendar.view.daystarthour).
+  // In the case of a full day, we keep the behavior similar to before.
+
+  //Full day tests
+  await testDays(
+    cal.createDateTime("20100403T020000"),
+    cal.createDateTime("20100403T030000"),
+    "20100403",
+    "20100418"
+  );
+  for (let i = -2; i < 0; i++) {
+    await testDays(
+      fromToday({ days: i, hours: 2 }),
+      fromToday({ days: i, hours: 3 }),
+      fromToday({ days: i }).icalString.substring(0, 8),
+      fromToday({ days: i + 15 }).icalString.substring(0, 8)
+    );
+  }
+  for (let i = 0; i < 3; i++) {
+    await testDays(
+      fromToday({ days: i, hours: 2 }),
+      fromToday({ days: i, hours: 3 }),
+      fromToday({ days: 0 }).icalString.substring(0, 8),
+      fromToday({ days: 15 }).icalString.substring(0, 8)
+    );
+  }
+  for (let i = 3; i < 5; i++) {
+    await testDays(
+      fromToday({ days: i, hours: 2 }),
+      fromToday({ days: i, hours: 3 }),
+      fromToday({ days: i - 2 }).icalString.substring(0, 8),
+      fromToday({ days: i + 13 }).icalString.substring(0, 8)
+    );
+  }
+  await testDays(
+    cal.createDateTime("20300403T020000"),
+    cal.createDateTime("20300403T030000"),
+    "20300401",
+    "20300416"
+  );
+
+  // Reduced day tests
+  let dayStartHour = Services.prefs.getIntPref("calendar.view.daystarthour", 8).toString();
+  if (dayStartHour.length == 1) {
+    dayStartHour = "0" + dayStartHour;
+  }
+
   await testDays(
     cal.createDateTime("20100403T120000"),
     cal.createDateTime("20100403T130000"),
-    "20100403",
-    "20100407"
+    "20100403T" + dayStartHour + "0000Z",
+    "20100418T" + dayStartHour + "0000Z"
   );
   for (let i = -2; i < 0; i++) {
     await testDays(
       fromToday({ days: i, hours: 12 }),
       fromToday({ days: i, hours: 13 }),
-      fromToday({ days: i }).icalString.substring(0, 8),
-      fromToday({ days: i + 4 }).icalString.substring(0, 8)
+      fromToday({ days: i }).icalString.substring(0, 8) + "T" + dayStartHour + "0000Z",
+      fromToday({ days: i + 15 }).icalString.substring(0, 8) + "T" + dayStartHour + "0000Z"
     );
   }
   for (let i = 0; i < 3; i++) {
     await testDays(
       fromToday({ days: i, hours: 12 }),
       fromToday({ days: i, hours: 13 }),
-      fromToday({ days: 0 }).icalString.substring(0, 8),
-      fromToday({ days: 4 }).icalString.substring(0, 8)
+      fromToday({ days: 0 }).icalString.substring(0, 8) + "T" + dayStartHour + "0000Z",
+      fromToday({ days: 15 }).icalString.substring(0, 8) + "T" + dayStartHour + "0000Z"
     );
   }
   for (let i = 3; i < 5; i++) {
     await testDays(
       fromToday({ days: i, hours: 12 }),
       fromToday({ days: i, hours: 13 }),
-      fromToday({ days: i - 2 }).icalString.substring(0, 8),
-      fromToday({ days: i + 2 }).icalString.substring(0, 8)
+      fromToday({ days: i - 2 }).icalString.substring(0, 8) + "T" + dayStartHour + "0000Z",
+      fromToday({ days: i + 13 }).icalString.substring(0, 8) + "T" + dayStartHour + "0000Z"
     );
   }
   await testDays(
     cal.createDateTime("20300403T120000"),
     cal.createDateTime("20300403T130000"),
-    "20300401",
-    "20300405"
+    "20300401T" + dayStartHour + "0000Z",
+    "20300416T" + dayStartHour + "0000Z"
   );
 });
 
