@@ -162,7 +162,7 @@ class CalDavServer {
 
   resetClient(client) {
     MockConflictPrompt.unregister();
-    cal.getCalendarManager().unregisterCalendar(client);
+    cal.manager.unregisterCalendar(client);
   }
 
   waitForLoad(aCalendar) {
@@ -185,8 +185,7 @@ class CalDavServer {
 
   getClient() {
     let uri = this.uri("/calendars/xpcshell/events");
-    let calmgr = cal.getCalendarManager();
-    let client = calmgr.createCalendar("caldav", uri);
+    let client = cal.manager.createCalendar("caldav", uri);
     let uclient = client.wrappedJSObject;
     client.name = "xpcshell";
     client.setProperty("cache.enabled", true);
@@ -202,9 +201,9 @@ class CalDavServer {
       });
     });
 
-    calmgr.registerCalendar(client);
+    cal.manager.registerCalendar(client);
 
-    let cachedCalendar = calmgr.getCalendarById(client.id);
+    let cachedCalendar = cal.manager.getCalendarById(client.id);
     return this.waitForLoad(cachedCalendar);
   }
 
@@ -491,11 +490,11 @@ function run_test() {
   do_get_profile();
   do_test_pending();
 
-  cal.getCalendarManager().startup({
+  cal.manager.startup({
     onResult() {
       gServer = new CalDavServer("xpcshell@example.com");
       gServer.start();
-      cal.getTimezoneService().startup({
+      cal.timezoneService.startup({
         onResult() {
           run_next_test();
           do_test_finished();
@@ -919,14 +918,7 @@ add_task(async function test_freebusy_request() {
   equal(results.headers.get("Originator"), "mailto:xpcshell@example.com");
   equal(results.headers.get("Recipient"), "mailto:recipient@example.com");
 
-  // The following assertion succeeds but causes "too much recursion" with
-  // icaljs. Assert.jsm calls JSON.stringify on the arguments to log the result
-  // of the assertion, and that causes the recursion error. (Then Assert.jsm
-  // just falls back to using toString instead.) See bug 1546606 starting at
-  // comment 23 and ical.js issue #410.
   let first = response.firstRecipient;
-  strictEqual(first, response.data["mailto:recipient1@example.com"]);
-
   equal(first.status, "2.0;Success");
   deepEqual(
     first.intervals.map(interval => interval.type),
@@ -945,12 +937,8 @@ add_task(async function test_freebusy_request() {
 
 add_task(async function test_caldav_client() {
   let client = await gServer.getClient();
-  let pclient = cal.async.promisifyCalendar(client);
+  let items = await client.getItemsAsArray(Ci.calICalendar.ITEM_FILTER_ALL_ITEMS, 0, null, null);
 
-  // TODO: Fix the problem that is causing this test to fail when the next line is removed.
-  await new Promise(resolve => executeSoon(resolve));
-
-  let items = await pclient.getAllItems();
   equal(items.length, 1);
   equal(items[0].title, "会議");
 });
