@@ -23,53 +23,25 @@ add_task(async function calendarInfo_test() {
     {
       input: { locale: "en-US" },
       expected: {
-        properties: [
-          "firstDayOfWeek",
-          "minDays",
-          "weekendStart",
-          "weekendEnd",
-          "calendar",
-          "locale",
-        ],
+        properties: ["firstDayOfWeek", "minDays", "weekend", "calendar", "locale"],
       },
     },
     {
       input: { locale: "EN-US" },
       expected: {
-        properties: [
-          "firstDayOfWeek",
-          "minDays",
-          "weekendStart",
-          "weekendEnd",
-          "calendar",
-          "locale",
-        ],
+        properties: ["firstDayOfWeek", "minDays", "weekend", "calendar", "locale"],
       },
     },
     {
       input: { locale: "et" },
       expected: {
-        properties: [
-          "firstDayOfWeek",
-          "minDays",
-          "weekendStart",
-          "weekendEnd",
-          "calendar",
-          "locale",
-        ],
+        properties: ["firstDayOfWeek", "minDays", "weekend", "calendar", "locale"],
       },
     },
     {
       input: { locale: null }, // this also would trigger caching tests
       expected: {
-        properties: [
-          "firstDayOfWeek",
-          "minDays",
-          "weekendStart",
-          "weekendEnd",
-          "calendar",
-          "locale",
-        ],
+        properties: ["firstDayOfWeek", "minDays", "weekend", "calendar", "locale"],
       },
     },
   ];
