@@ -11,6 +11,7 @@
 // Wrap in a block to prevent leaking to window scope.
 {
   let { Services } = ChromeUtils.import("resource://gre/modules/Services.jsm");
+  var { IMServices } = ChromeUtils.import("resource:///modules/IMServices.jsm");
   let { ChatIcons } = ChromeUtils.import("resource:///modules/chatIcons.jsm");
   const LazyModules = {};
 
@@ -91,6 +92,8 @@
       let htmlTooltip = this.querySelector(".htmlTooltip");
       htmlTooltip.hidden = true;
 
+      this.hasBestAvatar = false;
+
       // We have a few cases that have special behavior. These are richlistitems
       // and have tooltip="<myid>".
       let item = this.triggerNode.closest(
@@ -131,14 +134,10 @@
       // some more general checks.
       let elt = this.triggerNode;
       let classList = elt.classList;
-      if (
-        classList.contains("ib-nick") ||
-        classList.contains("ib-sender") ||
-        classList.contains("ib-person")
-      ) {
+      // ib-sender nicks are handled with _originalMsg if possible
+      if (classList.contains("ib-nick") || classList.contains("ib-person")) {
         let conv = getBrowser()._conv;
-        // ib-sender nicks are handled with _originalMsg
-        if (conv.isChat && !classList.contains("ib-sender")) {
+        if (conv.isChat) {
           return this.updateTooltipFromParticipant(elt.textContent, conv);
         }
         if (!conv.isChat && elt.textContent == conv.name) {
@@ -146,6 +145,9 @@
         }
       }
 
+      let sender = elt.textContent;
+      let overrideAvatar = undefined;
+
       // Are we over a message?
       for (let node = elt; node; node = node.parentNode) {
         if (!node._originalMsg) {
@@ -153,13 +155,9 @@
         }
         // Nick, build tooltip with original who information from message
         if (classList.contains("ib-sender")) {
-          let conv = getBrowser()._conv;
-          if (conv.isChat) {
-            return this.updateTooltipFromParticipant(
-              node._originalMsg.who,
-              conv
-            );
-          }
+          sender = node._originalMsg.who;
+          overrideAvatar = node._originalMsg.iconURL;
+          break;
         }
         // It's a message, so add a date/time tooltip.
         let date = new Date(node._originalMsg.time * 1000);
@@ -190,6 +188,21 @@
         break;
       }
 
+      if (classList.contains("ib-sender")) {
+        let conv = getBrowser()._conv;
+        if (conv.isChat) {
+          return this.updateTooltipFromParticipant(
+            sender,
+            conv,
+            undefined,
+            overrideAvatar
+          );
+        }
+        if (!conv.isChat && elt.textContent == conv.name) {
+          return this.updateTooltipFromConversation(conv, overrideAvatar);
+        }
+      }
+
       largeTooltip.hidden = true;
       // Show the title in the tooltip
       if (showHTMLTooltip) {
@@ -285,16 +298,32 @@
       }
     }
 
-    addRow(aLabel, aValue) {
+    /**
+     * Add a row to the tooltip's table
+     *
+     * @param {string} aLabel - Label for the table row.
+     * @param {string} aValue - Value for the table row.
+     * @param {{label: boolean, value: boolean}} [l10nIds] - Treat the label
+     *   and value as l10n IDs
+     */
+    addRow(aLabel, aValue, l10nIds = { label: false, value: false }) {
       let description;
       let row = [...this.table.querySelectorAll("tr")].find(row => {
-        return row.querySelector("th").textContent == aLabel;
+        let th = row.querySelector("th");
+        if (l10nIds?.label) {
+          return th.dataset.l10nId == aLabel;
+        }
+        return th.textContent == aLabel;
       });
       if (!row) {
         // Create a new row for this label.
         row = document.createElementNS("http://www.w3.org/1999/xhtml", "tr");
         let th = document.createElementNS("http://www.w3.org/1999/xhtml", "th");
-        th.textContent = aLabel;
+        if (l10nIds?.label) {
+          document.l10n.setAttributes(th, aLabel);
+        } else {
+          th.textContent = aLabel;
+        }
         th.setAttribute("valign", "top");
         row.appendChild(th);
         description = document.createElementNS(
@@ -307,7 +336,11 @@
         // Row with this label already exists - just update.
         description = row.querySelector("td");
       }
-      description.textContent = aValue;
+      if (l10nIds?.value) {
+        document.l10n.setAttributes(description, aValue);
+      } else {
+        description.textContent = aValue;
+      }
     }
 
     addSeparator() {
@@ -372,8 +405,10 @@
      *
      * @param {prplIAccountBuddy} aBuddy - The buddy to generate the conversation.
      * @param {imIConversation} [aConv] - A conversation associated with this buddy.
+     * @param {string} [overrideAvatar] - URL for the user avatar to use
+     *  instead.
      */
-    updateTooltipFromBuddy(aBuddy, aConv) {
+    updateTooltipFromBuddy(aBuddy, aConv, overrideAvatar) {
       this.buddy = aBuddy;
 
       this.reset();
@@ -384,8 +419,12 @@
       this.setProtocolIcon(account.protocol);
       // If a conversation is provided, use the icon from it. Otherwise, use the
       // buddy icon filename.
-      if (aConv) {
+      if (overrideAvatar) {
+        this.setUserIcon(overrideAvatar, true);
+        this.hasBestAvatar = true;
+      } else if (aConv && !aConv.isChat) {
         this.setUserIcon(aConv.convIconFilename, true);
+        this.hasBestAvatar = true;
       } else {
         this.setUserIcon(aBuddy.buddyIconFilename, true);
       }
@@ -402,10 +441,20 @@
 
       this.addRow(this.bundle.GetStringFromName("buddy.account"), account.name);
 
+      if (aBuddy.canVerifyIdentity) {
+        const identityStatus = aBuddy.identityVerified
+          ? "chat-buddy-identity-status-verified"
+          : "chat-buddy-identity-status-unverified";
+        this.addRow("chat-buddy-identity-status", identityStatus, {
+          label: true,
+          value: true,
+        });
+      }
+
       // Add encryption status.
       if (this.triggerNode.classList.contains("message-encrypted")) {
         this.addRow(
-          this.bundle.GetStringFromName("otr.tag"),
+          this.bundle.GetStringFromName("encryption.tag"),
           this.bundle.GetStringFromName("message.status")
         );
       }
@@ -435,7 +484,9 @@
             this.setMessage(LazyModules.Status.toLabel(statusType, elt.value));
             break;
           case Ci.prplITooltipInfo.icon:
-            this.setUserIcon(elt.value);
+            if (!this.hasBestAvatar) {
+              this.setUserIcon(elt.value);
+            }
             break;
         }
       }
@@ -445,18 +496,25 @@
      * Regenerate the tooltip based on a conversation.
      *
      * @param {imIConversation} aConv - The conversation to generate the tooltip from.
+     * @param {string} [overrideAvatar] - URL for the user avatar to use
+     *  instead if the conversation is a direct conversation.
      */
-    updateTooltipFromConversation(aConv) {
+    updateTooltipFromConversation(aConv, overrideAvatar) {
       if (!aConv.isChat && aConv.buddy) {
-        return this.updateTooltipFromBuddy(aConv.buddy, aConv);
+        return this.updateTooltipFromBuddy(aConv.buddy, aConv, overrideAvatar);
       }
 
       this.reset();
       this.setAttribute("displayname", aConv.name);
       let account = aConv.account;
       this.setProtocolIcon(account.protocol);
-      // Set the icon, potentially showing a fallback icon if this is an IM.
-      this.setUserIcon(aConv.convIconFilename, !aConv.isChat);
+      if (overrideAvatar && !aConv.isChat) {
+        this.setUserIcon(overrideAvatar, true);
+        this.hasBestAvatar = true;
+      } else {
+        // Set the icon, potentially showing a fallback icon if this is an IM.
+        this.setUserIcon(aConv.convIconFilename, !aConv.isChat);
+      }
       if (aConv.isChat) {
         if (!account.connected || aConv.left) {
           this.setStatusIcon("chat-left");
@@ -478,7 +536,18 @@
       return true;
     }
 
-    updateTooltipFromParticipant(aNick, aConv, aParticipant) {
+    /**
+     * Set the tooltip details based on a conversation participant.
+     *
+     * @param {string} aNick - Nick of the user this tooltip is for.
+     * @param {prplIConversation} aConv - Conversation this tooltip is shown
+     *  in.
+     * @param {prplIConvChatBuddy} [aParticipant] - Participant to use instead
+     *  of looking it up in the conversation by the passed nick.
+     * @param {string} [overrideAvatar] - URL for the user avatar to use
+     *  instead.
+     */
+    updateTooltipFromParticipant(aNick, aConv, aParticipant, overrideAvatar) {
       if (!aConv.target) {
         return false; // We're viewing a log.
       }
@@ -492,12 +561,16 @@
       // contact, we require at least that the normalizedChatBuddyName of
       // the nick is normalized like a normalizedName for contacts.
       if (normalizedNick == account.normalize(normalizedNick)) {
-        let accountBuddy = Services.contacts.getAccountBuddyByNameAndAccount(
+        let accountBuddy = IMServices.contacts.getAccountBuddyByNameAndAccount(
           normalizedNick,
           account
         );
         if (accountBuddy) {
-          return this.updateTooltipFromBuddy(accountBuddy);
+          return this.updateTooltipFromBuddy(
+            accountBuddy,
+            aConv,
+            overrideAvatar
+          );
         }
       }
 
@@ -506,7 +579,20 @@
       this.setProtocolIcon(account.protocol);
       this.setStatusIcon("unknown");
       this.setMessage(LazyModules.Status.toLabel("unknown"));
-      this.setUserIcon(aParticipant?.buddyIconFilename, true);
+      this.setUserIcon(overrideAvatar ?? aParticipant?.buddyIconFilename, true);
+      if (overrideAvatar) {
+        this.hasBestAvatar = true;
+      }
+
+      if (aParticipant.canVerifyIdentity) {
+        const identityStatus = aParticipant.identityVerified
+          ? "chat-buddy-identity-status-verified"
+          : "chat-buddy-identity-status-unverified";
+        this.addRow("chat-buddy-identity-status", identityStatus, {
+          label: true,
+          value: true,
+        });
+      }
 
       this.requestBuddyInfo(account, normalizedNick);
       return true;
