@@ -1,9 +1,8 @@
-/* -*- Mode: JavaScript; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
-/* This Source Code Form is subject to the terms of the Mozilla Public
- * License, v. 2.0. If a copy of the MPL was not distributed with this
- * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
+#filter dumbComments emptyLines substitution
 
-#filter substitution
+// This Source Code Form is subject to the terms of the Mozilla Public
+// License, v. 2.0. If a copy of the MPL was not distributed with this
+// file, You can obtain one at http://mozilla.org/MPL/2.0/.
 
 #ifdef XP_UNIX
 #ifndef XP_MACOSX
@@ -60,15 +59,9 @@ pref("app.update.log", false);
 pref("app.update.staging.enabled", true);
 
 // Update service URL:
-pref("app.update.url", "https://aus.thunderbird.net/update/6/%PRODUCT%/%VERSION%/%BUILD_ID%/%BUILD_TARGET%/%LOCALE%/%CHANNEL%/%OS_VERSION%/%SYSTEM_CAPABILITIES%/%DISTRIBUTION%/%DISTRIBUTION_VERSION%/update.xml");
-
-// URL user can browse to manually if for some reason all update installation
-// attempts fail.
-pref("app.update.url.manual", "https://www.thunderbird.net");
-// A default value for the "More information about this update" link
-// supplied in the "An update is available" page of the update wizard.
-pref("app.update.url.details", "https://www.thunderbird.net/%LOCALE%/%APP%/releases/");
-
+// app.update.url was removed in Bug 1630041
+// app.update.url.manual is in branding section
+// app.update.url.details is in branding section
 // app.update.promptWaitTime is in branding section
 
 // Whether or not to attempt using the service for updates.
@@ -84,6 +77,15 @@ pref("app.update.BITS.enabled", false);
 // Release notes URL
 pref("app.releaseNotesURL", "https://live.thunderbird.net/%APP%/releasenotes?locale=%LOCALE%&version=%VERSION%&channel=%CHANNEL%&os=%OS%&buildid=%APPBUILDID%");
 
+#ifdef XP_MACOSX
+  // If set to true, Thunderbird will automatically restart if it is left
+  // running with no windows open.
+  pref("app.update.noWindowAutoRestart.enabled", true);
+  // How long to wait after all windows are closed before restarting,
+  // in milliseconds. 5 min = 300000 ms.
+  pref("app.update.noWindowAutoRestart.delayMs", 300000);
+#endif
+
 // URL for "Learn More" for DataCollection
 pref("toolkit.datacollection.infoURL",
      "https://www.mozilla.org/thunderbird/legal/privacy/#telemetry");
@@ -104,10 +106,14 @@ pref("datareporting.policy.firstRunURL", "https://www.mozilla.org/thunderbird/le
 #endif
 
 // Base URL for web-based support pages.
-pref("app.support.baseURL", "https://support.thunderbird.net/%LOCALE%/%APP%/%APPBUILDID%/");
+pref("app.support.baseURL", "https://support.thunderbird.net/%APP%/%VERSION%/%OS%/%LOCALE%/");
 
 // Base url for web-based feedback pages.
-pref("app.feedback.baseURL", "https://input.mozilla.org/%LOCALE%/feedback/%APP%/%VERSION%/");
+pref("app.feedback.baseURL", "https://connect.mozilla.org/");
+
+// Allows using Thundebird without a configured email account, blocking the
+// account setup prompt at startup.
+pref("app.use_without_mail_account", false);
 
 // Show error messages in error console.
 pref("javascript.options.showInConsole", true);
@@ -151,11 +157,6 @@ pref("extensions.startupScanScopes", 4);
 // Gecko Profiler
 pref("extensions.geckoProfiler.acceptedExtensionIds", "geckoprofiler@mozilla.com,quantum-foxfooding@mozilla.com,raptor@mozilla.org");
 
-// Add-on content security policies.
-pref("extensions.webextensions.base-content-security-policy", "script-src 'self' https://* moz-extension: blob: filesystem: 'unsafe-eval' 'unsafe-inline'; object-src 'self' https://* moz-extension: blob: filesystem:;");
-pref("extensions.webextensions.default-content-security-policy", "script-src 'self'; object-src 'self';");
-
-
 // Allow "legacy" XUL/XPCOM extensions.
 pref("extensions.legacy.enabled", true);
 
@@ -165,11 +166,11 @@ pref("extensions.getAddons.maxResults", 15);
 pref("extensions.getAddons.get.url", "https://services.addons.thunderbird.net/api/v3/addons/search/?guid=%IDS%&lang=%LOCALE%");
 pref("extensions.getAddons.compatOverides.url", "https://services.addons.thunderbird.net/api/v3/addons/compat-override/?guid=%IDS%&lang=%LOCALE%");
 pref("extensions.getAddons.link.url", "https://addons.thunderbird.net/%LOCALE%/%APP%/");
+pref("browser.dictionaries.download.url", "https://addons.thunderbird.net/%LOCALE%/%APP%/language-tools/");
 pref("extensions.getAddons.recommended.url", "https://services.addons.thunderbird.net/%LOCALE%/%APP%/api/%API_VERSION%/list/recommended/all/%MAX_RESULTS%/%OS%/%VERSION%?src=thunderbird");
-pref("extensions.getAddons.search.browseURL", "https://addons.thunderbird.net/%LOCALE%/%APP%/search/?q=%TERMS%");
+pref("extensions.getAddons.search.browseURL", "https://addons.thunderbird.net/%LOCALE%/%APP%/search/?q=%TERMS%&appver=%VERSION%&platform=%OS%");
 pref("extensions.getAddons.search.url", "https://services.addons.thunderbird.net/%LOCALE%/%APP%/api/%API_VERSION%/search/%TERMS%/all/%MAX_RESULTS%/%OS%/%VERSION%/%COMPATIBILITY_MODE%?src=thunderbird");
 pref("extensions.webservice.discoverURL", "https://services.addons.thunderbird.net/%LOCALE%/%APP%/discovery/pane/%VERSION%/%OS%");
-pref("extensions.getAddons.siteRegExp", "^https://.*addons\\.thunderbird\\.net");
 pref("extensions.getAddons.langpacks.url", "https://services.addons.thunderbird.net/api/v3/addons/language-tools/?app=thunderbird&type=language&appversion=%VERSION%");
 pref("extensions.getAddons.discovery.api_url", "https://services.addons.thunderbird.net/api/v4/discovery/?lang=%LOCALE%&edition=%DISTRIBUTION%");
 
@@ -177,13 +178,22 @@ pref("extensions.getAddons.discovery.api_url", "https://services.addons.thunderb
 pref("extensions.blocklist.detailsURL", "https://blocked.cdn.mozilla.net/");
 pref("extensions.blocklist.itemURL", "https://blocked.cdn.mozilla.net/%blockID%.html");
 
+// Remote settings preferences
+pref("services.settings.server", "https://thunderbird-settings.thunderbird.net/v1");
+pref("services.settings.default_bucket", "thunderbird");
+pref("security.content.signature.root_hash", "[CONTENT SIGNING DISABLED - see bug 1612380]");
+
 // Show new install UI with permission lists
-pref("extensions.webextPermissionPrompts", true);
+pref("extensions.webextOptionalPermissionPrompts", true);
 
 // 1 = allow "Man In The Middle" (local proxy, web filter, etc.) for certificate
 //     pinning checks.
 pref("security.cert_pinning.enforcement_level", 1);
 
+// Whether to use client certificates stored in OS certificate storage.
+// This does not work for S/MIME. See bug 1726442.
+pref("security.osclientcerts.autoload", false);
+
 // Symmetric (can be overridden by individual extensions) update preferences.
 // e.g.
 //  extensions.{GUID}.update.enabled
@@ -203,9 +213,18 @@ pref("extensions.dss.switchPending", false);    // Non-dynamic switch pending af
 
 // Don't show recommendations on the extension and theme list views.
 pref("extensions.htmlaboutaddons.recommendations.enabled", false);
+// Don't allow content scripts on these web sites
+pref("extensions.webextensions.restrictedDomains", "accounts-static.cdn.mozilla.net,accounts.firefox.com,addons.cdn.mozilla.net,addons.mozilla.org,api.accounts.firefox.com,content.cdn.mozilla.net,discovery.addons.mozilla.org,install.mozilla.org,oauth.accounts.firefox.com,profile.accounts.firefox.com,support.mozilla.org,sync.services.mozilla.com,addons.thunderbird.net");
+
+// Define Thunderbird specific add-on related URLs (not used in toolkit code).
+pref("extensions.canonicalAddonServer.url", "https://addons.thunderbird.net");
+pref("extensions.alternativeAddonSearch.url", "https://extension-finder.thunderbird.net");
 
 pref("lightweightThemes.update.enabled", true);
 
+// Use for in-content pages the theme's toolbar color scheme when none is set in the theme.
+pref("browser.theme.unified-color-scheme", true);
+
 // Built-in default permissions.
 pref("permissions.manager.defaultsUrl", "resource://app/defaults/permissions");
 
@@ -244,6 +263,13 @@ pref("browser.preferences.animateFadeIn", false);
 #endif
 pref("browser.preferences.search", true);
 
+// Whether the results panel should be kept open during IME composition.
+// The default value is false because some IME open a picker panel, and we end
+// up with two panels on top of each other. Since for now we can't detect that
+// we leave this choice to the user, hopefully in the future this can be flipped
+// for everyone.
+pref("browser.urlbar.keepPanelOpenDuringImeComposition", false);
+
 pref("accessibility.typeaheadfind", false);
 pref("accessibility.typeaheadfind.timeout", 5000);
 pref("accessibility.typeaheadfind.linksonly", false);
@@ -269,13 +295,16 @@ pref("mail.showCondensedAddresses", true); // show the friendly display name for
 
 pref("mailnews.attachments.display.start_expanded", false);
 // hidden pref for changing how we present attachments in the message pane
-pref("mailnews.attachments.display.view", 0);
 pref("mail.pane_config.dynamic",            0);
 pref("mailnews.reuse_thread_window2",     true);
 pref("editor.singleLine.pasteNewlines", 4);  // substitute commas for new lines in single line text boxes
 pref("editor.CR_creates_new_p", true);
 pref("mail.compose.default_to_paragraph", true);
 
+// If true, when pasting a URL, paste the Open Graph / Twitter Card details
+// we can extract from the URL instead.
+pref("mail.compose.add_link_preview", false);
+
 // hidden pref to ensure a certain number of headers in the message pane
 // to avoid the height of the header area from changing when headers are present / not present
 pref("mailnews.headers.minNumHeaders", 0); // 0 means we ignore this pref
@@ -305,10 +334,14 @@ pref("network.auth.non-web-content-triggered-resources-http-auth-allow", true);
 // 0=as attachment 2=default forward as inline with attachments
 pref("mail.forward_message_mode", 2);
 
-// 0=ask, 1=plain, 2=html, 3=both
-pref("mail.default_html_action", 3);
+pref("mailnews.send.loglevel", "Warn");
+
+pref("mailnews.nntp.jsmodule", true);
+
+pref("mailnews.pop3.jsmodule", true);
+
+pref("mail.import.in_new_tab", true);
 
-/////////////////////////////////////////////////////////////////
 // End core mailnews.js pref overrides
 /////////////////////////////////////////////////////////////////
 
@@ -343,24 +376,52 @@ pref("network.http.speculative-parallel-limit", 0);
 // list in nsMsgContentPolicy::IsExposedProtocol.
 pref("network.protocol-handler.expose-all", false);
 pref("network.protocol-handler.expose.mailto", true);
+pref("network.protocol-handler.expose.mid", true);
 pref("network.protocol-handler.expose.news", true);
 pref("network.protocol-handler.expose.snews", true);
 pref("network.protocol-handler.expose.nntp", true);
 pref("network.protocol-handler.expose.imap", true);
-pref("network.protocol-handler.expose.addbook", true);
 pref("network.protocol-handler.expose.pop", true);
 pref("network.protocol-handler.expose.mailbox", true);
 // Although we allow these to be exposed internally, there are various places
 // (e.g. message pane) where we may divert them out to external applications.
 pref("network.protocol-handler.expose.about", true);
+pref("network.protocol-handler.expose.blob", true);
+pref("network.protocol-handler.expose.data", true);
+pref("network.protocol-handler.expose.file", true);
 pref("network.protocol-handler.expose.http", true);
 pref("network.protocol-handler.expose.https", true);
+pref("network.protocol-handler.expose.javascript", true);
+pref("network.protocol-handler.expose.moz-extension", true);
 
 // suppress external-load warning for standard browser schemes
 pref("network.protocol-handler.warn-external.http", false);
 pref("network.protocol-handler.warn-external.https", false);
 pref("network.protocol-handler.warn-external.ftp", false);
 
+// prevent web pages from registering mailnews protocol handlers
+pref("network.protocol-handler.external.cid", false);
+pref("network.protocol-handler.external.mid", false);
+pref("network.protocol-handler.external.mailto", false);
+pref("network.protocol-handler.external.imap", false);
+pref("network.protocol-handler.external.imap-message", false);
+pref("network.protocol-handler.external.pop", false);
+pref("network.protocol-handler.external.pop3", false);
+pref("network.protocol-handler.external.mailbox", false);
+pref("network.protocol-handler.external.mailbox-message", false);
+pref("network.protocol-handler.external.smtp", false);
+pref("network.protocol-handler.external.smtps", false);
+pref("network.protocol-handler.external.nntp", false);
+pref("network.protocol-handler.external.news", false);
+pref("network.protocol-handler.external.news-message", false);
+pref("network.protocol-handler.external.snews", false);
+pref("network.protocol-handler.external.ldap", false);
+pref("network.protocol-handler.external.ldaps", false);
+pref("network.protocol-handler.external.webcal", false);
+pref("network.protocol-handler.external.webcals", false);
+pref("network.protocol-handler.external.moz-cal-handle-itip", false);
+pref("network.protocol-handler.external.smile", false);
+
 pref("network.hosts.smtp_server",           "mail");
 pref("network.hosts.pop_server",            "mail");
 
@@ -374,14 +435,16 @@ pref("security.warn_leaving_secure", false);
 pref("security.warn_viewing_mixed", false);
 pref("security.aboutcertificate.enabled", true);
 
-// Prompt for the master password prior to opening application windows,
+// Don't automatically cleanup intermediate CA certificates that core
+// has on a preloaded list, it causes S/MIME failures. (Bug 1777336)
+pref("security.intermediate_preloading_healer.enabled", false);
+
+// Don't show a prompt for external applications.
+pref("security.external_protocol_requires_permission", false);
+
+// Prompt for the primary password prior to opening application windows,
 // to avoid the race that triggers multiple prompts (see bug 177175).
-#ifdef XP_MACOSX
-// disabled because of platform specific bug 1612456
-pref("security.prompt_for_master_password_on_startup", false);
-#else
 pref("security.prompt_for_master_password_on_startup", true);
-#endif
 
 pref("general.config.obscure_value", 0); // for MCD .cfg files
 
@@ -389,6 +452,7 @@ pref("browser.display.auto_quality_min_font_size", 0);
 
 pref("view_source.syntax_highlight", false);
 
+pref("dom.serviceWorkers.enabled", true);
 
 /////////////////////////////////////////////////////////////////
 // End core all.js pref overrides
@@ -416,6 +480,12 @@ pref("browser.download.manager.addToRecentDocs", true);
 pref("browser.helperApps.deleteTempFileOnExit", true);
 #endif
 
+// Not used in Thunderbird.
+#ifdef NIGHTLY_BUILD
+pref("browser.startup.homepage.abouthome_cache.enabled", true);
+#else
+pref("browser.startup.homepage.abouthome_cache.enabled", false);
+#endif
 // Whether to start the private browsing mode at application startup. Not used in Thunderbird.
 pref("browser.privatebrowsing.autostart", false);
 
@@ -463,13 +533,6 @@ pref("mail.tabs.tabClipWidth", 140);
 pref("mail.tabs.autoHide", false);
 pref("mail.tabs.closeWindowWithLastTab", true);
 
-// Where to show tab close buttons:
-// 0 - active tab only
-// 1 - all tabs until tabClipWidth is reached, then active tab only
-// 2 - no close buttons
-// 3 - at the end of the tabstrip
-pref("mail.tabs.closeButtons", 1);
-
 // Allow the tabs to be in the titlebar on supported systems
 #ifdef UNIX_BUT_NOT_MAC
 pref("mail.tabs.drawInTitlebar", false);
@@ -481,9 +544,6 @@ pref("mail.tabs.drawInTitlebar", true);
 // will only be shown if browser.tabs.drawInTitlebar is true.
 pref("mail.tabs.extraDragSpace", false);
 
-// Disable ParentProcessDocumentChannel.
-pref("browser.tabs.documentchannel.ppdc", false);
-
 // The breakpad report server to link to in about:crashes
 pref("breakpad.reportURL", "https://crash-stats.mozilla.com/report/index/");
 
@@ -500,16 +560,10 @@ pref("mail.spotlight.firstRunDone", false);
 
 // -- Windows Search/Spotlight logging options
 #ifdef XP_WIN
-// Should we output warnings and errors to the "error console"?
-pref("mail.winsearch.logging.console", false);
-// Should we output all output levels to stdout via dump?
-pref("mail.winsearch.logging.dump", false);
+pref("mail.winsearch.loglevel", "Warn");
 #else
 #ifdef XP_MACOSX
-// Should we output warnings and errors to the "error console"?
-pref("mail.spotlight.logging.console", false);
-// Should we output all output levels to stdout via dump?
-pref("mail.spotlight.logging.dump", false);
+pref("mail.spotlight.loglevel", "Warn");
 #endif
 #endif
 
@@ -520,12 +574,6 @@ pref("toolbar.customization.usesheet", true);
 pref("toolbar.customization.usesheet", false);
 #endif
 
-// Whether to enable experimental CardDAV support.
-pref("mail.addr_book.carddav.enabled", false);
-
-// Number of recipient rows shown by default
-pref("mail.compose.addresswidget.numRowsShownDefault", 3);
-
 // Start compositions with (empty) attachment pane showing
 pref("mail.compose.show_attachment_pane", false);
 // Check for missing attachments?
@@ -548,15 +596,6 @@ pref("mail.compose.big_attachments.insert_notification", true);
 // While false, display information about editing sending identity in compose.
 pref("mail.compose.warned_about_customize_from", false);
 
-// Instrumentation is currently unfinished, do not enable it.
-// Set this to false to prevent instrumentation from happening, e.g., user
-// has opted out, or an enterprise wants to disable it from the get go.
-pref("mail.instrumentation.askUser", true);
-pref("mail.instrumentation.userOptedIn", false);
-pref("mail.instrumentation.postUrl", "https://www.mozilla.org/instrumentation");
-// not sure how this will be formatted - would be nice to make it extensible.
-pref("mail.instrumentation.lastNotificationSent", "");
-
 pref("browser.formfill.enable", true);
 
 // Disable autoplay as we don't handle audio elements in emails very well.
@@ -575,8 +614,6 @@ pref("gloda.facetview.sortby", 2);
 
 // Enable gloda by default!
 pref("mailnews.database.global.indexer.enabled", true);
-// Show gloda errors in the error console
-pref("mailnews.database.global.logging.console", true);
 // Limit the number of gloda message results
 pref("mailnews.database.global.search.msg.limit", 1000);
 
@@ -590,6 +627,34 @@ pref("font.default.x-western", "sans-serif");
 pref("font.default.x-cyrillic", "sans-serif");
 pref("font.default.el", "sans-serif");
 
+#ifdef XP_WIN
+pref("font.name.monospace.x-unicode", "Consolas");
+pref("font.name.sans-serif.x-unicode", "Calibri");
+pref("font.name.serif.x-unicode", "Cambria");
+pref("font.size.monospace.x-unicode", 14);
+pref("font.size.variable.x-unicode", 17);
+
+pref("font.name.monospace.x-western", "Consolas");
+pref("font.name.sans-serif.x-western", "Calibri");
+pref("font.name.serif.x-western", "Cambria");
+pref("font.size.monospace.x-western", 14);
+pref("font.size.variable.x-western", 17);
+
+pref("font.name.monospace.x-cyrillic", "Consolas");
+pref("font.name.sans-serif.x-cyrillic", "Calibri");
+pref("font.name.serif.x-cyrillic", "Cambria");
+pref("font.size.monospace.x-cyrillic", 14);
+pref("font.size.variable.x-cyrillic", 17);
+
+pref("font.name.monospace.el", "Consolas");
+pref("font.name.sans-serif.el", "Calibri");
+pref("font.name.serif.el", "Cambria");
+pref("font.size.monospace.el", 14);
+pref("font.size.variable.el", 17);
+
+pref("mail.font.windows.version", 2);
+#endif
+
 #ifdef XP_MACOSX
 pref("font.name.sans-serif.x-unicode", "Lucida Grande");
 pref("font.name.monospace.x-unicode", "Menlo");
@@ -645,12 +710,7 @@ pref("font.name-list.sans-serif.el", "sans-serif");
 pref("font.name-list.monospace.el", "monospace");
 #endif
 
-pref("mail.font.windows.version", 0);
-
-// What level of warning should we send to the error console?
-pref("mail.wizard.logging.console", "None");
-// What level of warning should we send to stdout via dump?
-pref("mail.wizard.logging.dump", "None");
+pref("mail.setup.loglevel", "Warn");
 
 // Handle links targeting new windows (from within content tabs)
 // These are the values that Firefox can be set to:
@@ -672,8 +732,12 @@ pref("browser.link.open_newwindow.restriction", 0);
 
 pref("browser.tabs.loadDivertedInBackground", false);
 
-// No e10s in Thunderbird for now.
-pref("browser.tabs.remote.autostart", false);
+// Enable multi-process.
+pref("browser.tabs.remote.autostart", true);
+pref("browser.tabs.remote.desktopbehavior", true);
+pref("extensions.webextensions.remote", true);
+
+pref("extensions.webextensions.background-delayed-startup", true);
 
 // Browser icon prefs
 pref("browser.chrome.site_icons", true);
@@ -727,7 +791,6 @@ pref("mail.taskbar.lists.tasks.enabled", true);
 pref("mail.provider.providerList", "https://broker.thunderbird.net/provider/list");
 pref("mail.provider.suggestFromName", "https://broker.thunderbird.net/provider/suggest");
 pref("mail.provider.enabled", true);
-pref("mail.provider.realname", "");
 
 pref("mail.chat.enabled", true);
 // Whether to show chat notifications or not.
@@ -751,12 +814,20 @@ pref("chat.otr.default.allowMsgLog", true);
 
 // BigFiles
 pref("mail.cloud_files.enabled", true);
-pref("mail.cloud_files.inserted_urls.footer.link", "https://www.thunderbird.net");
 pref("mail.cloud_files.learn_more_url", "https://support.thunderbird.net/kb/filelink-large-attachments");
 
 // Ignore threads
 pref("mail.ignore_thread.learn_more_url", "https://support.thunderbird.net/kb/ignore-threads");
 
+// Density control for the entire Thunderbird UI.
+// The possible values are 0=compact, 1=normal, 2=touch.
+pref("mail.uidensity", 0);
+
+// Font size control for the entire Thunderbird UI. The value represents the
+// pixel value which will be applied as inline style to the root element of the
+// page (e.g.: 14 = font-size: 14px)
+pref("mail.uifontsize", 0);
+
 // Sanitize dialog window
 pref("privacy.cpd.history", true);
 pref("privacy.cpd.cookies", true);
@@ -773,6 +844,10 @@ pref("privacy.sanitize.timeSpan", 1);
 // Enable Contextual Identity Containers
 pref("privacy.userContext.enabled", false);
 
+// Set to true to add toggles to the WebRTC indicator for globally
+// muting the camera and microphone.
+pref("privacy.webrtc.globalMuteToggles", false);
+
 // If set to true, Thunderbird will collapse the main menu for new profiles
 // (or, more precisely, profiles that start with no accounts created).
 pref("mail.main_menu.collapse_by_default", true);
@@ -792,16 +867,23 @@ pref("security.webauth.u2f", true);
 // Use OS date and time settings by default.
 pref("intl.regional_prefs.use_os_locales", true);
 
-// Multi-lingual preferences.
-// Let the user select a different language for the UI.
+// Multi-lingual preferences:
+//  *.enabled - Are langpacks available for the build of Firefox?
+//  *.downloadEnabled - Langpacks are allowed to be downloaded from ATN. ATN only serves
+//      langpacks for release and beta. There is no release-only define, so we also enable
+//      it for beta.
+//  *.liveReload - Switching a langpack will change the language without a restart.
+//  *.liveReloadBidirectional - Allows switching when moving between LTR and RTL
+//      languages without a full restart.
 pref("intl.multilingual.enabled", true);
-
-// ATN only serves language packs for release.
-// There is no release-only define, so we also enable it for beta.
 #if defined(RELEASE_OR_BETA)
 pref("intl.multilingual.downloadEnabled", true);
+pref("intl.multilingual.liveReload", false);
+pref("intl.multilingual.liveReloadBidirectional", false);
 #else
 pref("intl.multilingual.downloadEnabled", false);
+pref("intl.multilingual.liveReload", false);
+pref("intl.multilingual.liveReloadBidirectional", false);
 #endif
 
 // if true, use full page zoom instead of text zoom
@@ -827,45 +909,36 @@ pref("devtools.toolbox.zoomValue", "1");
 pref("devtools.toolbox.splitconsoleEnabled", false);
 pref("devtools.toolbox.splitconsoleHeight", 100);
 pref("devtools.toolbox.tabsOrder", "");
+pref("devtools.netmonitor.features.newEditAndResend", false);
 
-// The fission pref for enabling the "Omniscient Browser Toolbox", which will
+// The fission pref for enabling the "Multiprocess Browser Toolbox", which will
 // make it possible to debug anything in Firefox (See Bug 1570639 for more
 // information).
-// ⚠ This is a work in progress. Expect weirdness when the pref is enabled. ⚠
+#if defined(NIGHTLY_BUILD)
+pref("devtools.browsertoolbox.fission", true);
+#else
 pref("devtools.browsertoolbox.fission", false);
-
-// The fission pref for enabling Fission frame debugging directly from the
-// regular web/content toolbox.
-// ⚠ This is a work in progress. Expect weirdness when the pref is enabled. ⚠
-pref("devtools.contenttoolbox.fission", false);
-
-// This pref is also related to fission, but not only. It allows the toolbox
-// to stay open even if the debugged tab switches to another process.
-// It can happen between two documents, one running in the parent process like
-// about:sessionrestore and another one running in the content process like
-// any web page. Or between two distinct domain when running with fission turned
-// on. See bug 1565263.
-// ⚠ This is a work in progress. Expect weirdness when the pref is flipped on ⚠
-pref("devtools.target-switching.enabled", false);
+#endif
 
 // Toolbox Button preferences
 pref("devtools.command-button-pick.enabled", true);
 pref("devtools.command-button-frames.enabled", true);
 pref("devtools.command-button-splitconsole.enabled", true);
-pref("devtools.command-button-paintflashing.enabled", false);
 pref("devtools.command-button-responsive.enabled", true);
 pref("devtools.command-button-screenshot.enabled", false);
 pref("devtools.command-button-rulers.enabled", false);
 pref("devtools.command-button-measure.enabled", false);
 pref("devtools.command-button-noautohide.enabled", false);
+pref("devtools.command-button-errorcount.enabled", true);
 #ifndef MOZILLA_OFFICIAL
-  pref("devtools.command-button-fission-prefs.enabled", true);
+  pref("devtools.command-button-experimental-prefs.enabled", true);
 #endif
 
 // Inspector preferences
 // Enable the Inspector
 pref("devtools.inspector.enabled", true);
 // What was the last active sidebar in the inspector
+pref("devtools.inspector.selectedSidebar", "layoutview");
 pref("devtools.inspector.activeSidebar", "layoutview");
 pref("devtools.inspector.remote", false);
 
@@ -881,12 +954,12 @@ pref("devtools.inspector.imagePreviewTooltipSize", 300);
 pref("devtools.inspector.showUserAgentStyles", false);
 // Show native anonymous content and user agent shadow roots
 pref("devtools.inspector.showAllAnonymousContent", false);
-// Enable the new Rules View
-pref("devtools.inspector.new-rulesview.enabled", false);
+// Enable the inline CSS compatibility warning in inspector rule view
+pref("devtools.inspector.ruleview.inline-compatibility-warning.enabled", false);
 // Enable the compatibility tool in the inspector.
-pref("devtools.inspector.compatibility.enabled", false);
+pref("devtools.inspector.compatibility.enabled", true);
 // Enable color scheme simulation in the inspector.
-pref("devtools.inspector.color-scheme-simulation.enabled", false);
+pref("devtools.inspector.color-scheme-simulation.enabled", true);
 
 // Grid highlighter preferences
 pref("devtools.gridinspector.gridOutlineMaxColumns", 50);
@@ -944,38 +1017,11 @@ pref("devtools.memory.max-retaining-paths", 10);
 
 // Enable the Performance tools
 pref("devtools.performance.enabled", true);
-
-// The default Performance UI settings
-pref("devtools.performance.memory.sample-probability", "0.05");
-// Can't go higher than this without causing internal allocation overflows while
-// serializing the allocations data over the RDP.
-pref("devtools.performance.memory.max-log-length", 125000);
-pref("devtools.performance.timeline.hidden-markers",
-  "[\"Composite\",\"CompositeForwardTransaction\"]");
-pref("devtools.performance.profiler.buffer-size", 10000000);
-pref("devtools.performance.profiler.sample-frequency-hz", 1000);
-pref("devtools.performance.ui.invert-call-tree", true);
-pref("devtools.performance.ui.invert-flame-graph", false);
-pref("devtools.performance.ui.flatten-tree-recursion", true);
-pref("devtools.performance.ui.show-platform-data", false);
-pref("devtools.performance.ui.show-idle-blocks", true);
-pref("devtools.performance.ui.enable-memory", false);
-pref("devtools.performance.ui.enable-allocations", false);
-pref("devtools.performance.ui.enable-framerate", true);
-pref("devtools.performance.ui.show-jit-optimizations", false);
-pref("devtools.performance.ui.show-triggers-for-gc-types",
-  "TOO_MUCH_MALLOC ALLOC_TRIGGER LAST_DITCH EAGER_ALLOC_TRIGGER");
-
-// Temporary pref disabling memory flame views
-// TODO remove once we have flame charts via bug 1148663
-pref("devtools.performance.ui.enable-memory-flame", false);
-
-// Enable experimental options in the UI only in Nightly
-#if defined(NIGHTLY_BUILD)
-  pref("devtools.performance.ui.experimental", true);
-#else
-  pref("devtools.performance.ui.experimental", false);
-#endif
+// But not the pop-up.
+pref("devtools.performance.popup.feature-flag", false);
+// Override the default preset, which is "web-developer" on beta and release.
+pref("devtools.performance.recording.preset", "firefox-platform");
+pref("devtools.performance.recording.preset.remote", "firefox-platform");
 
 // The default cache UI setting
 pref("devtools.cache.disabled", false);
@@ -992,6 +1038,13 @@ pref("devtools.netmonitor.features.requestBlocking", true);
 // Enable the Application panel
 pref("devtools.application.enabled", false);
 
+// Enable the custom formatters feature
+// TODO remove once the custom formatters feature is stable (see bug 1734614)
+pref("devtools.custom-formatters", false);
+// This preference represents the user's choice to enable the custom formatters feature.
+// While the preference above will be removed once the feature is stable, this one is menat to stay.
+pref("devtools.custom-formatters.enabled", false);
+
 // The default Network Monitor UI settings
 pref("devtools.netmonitor.panes-network-details-width", 550);
 pref("devtools.netmonitor.panes-network-details-height", 450);
@@ -1033,28 +1086,6 @@ pref("devtools.netmonitor.audits.slow", 500);
 // Disable the EventSource Inspector.
 pref("devtools.netmonitor.features.serverSentEvents", false);
 
-// Save request/response bodies yes/no.
-pref("devtools.netmonitor.saveRequestAndResponseBodies", true);
-
-// The default Network monitor HAR export setting
-pref("devtools.netmonitor.har.defaultLogDir", "");
-pref("devtools.netmonitor.har.defaultFileName", "%hostname_Archive [%date]");
-pref("devtools.netmonitor.har.jsonp", false);
-pref("devtools.netmonitor.har.jsonpCallback", "");
-pref("devtools.netmonitor.har.includeResponseBodies", true);
-pref("devtools.netmonitor.har.compress", false);
-pref("devtools.netmonitor.har.forceExport", false);
-pref("devtools.netmonitor.har.pageLoadedTimeout", 1500);
-pref("devtools.netmonitor.har.enableAutoExportToFile", false);
-
-pref("devtools.netmonitor.features.webSockets", true);
-
-// netmonitor audit
-pref("devtools.netmonitor.audits.slow", 500);
-
-// Disable the EventSource Inspector.
-pref("devtools.netmonitor.features.serverSentEvents", false);
-
 // Enable the Storage Inspector
 pref("devtools.storage.enabled", true);
 
@@ -1088,7 +1119,11 @@ pref("devtools.webconsole.filter.netxhr", false);
 
 // Webconsole autocomplete preference
 pref("devtools.webconsole.input.autocomplete",true);
-pref("devtools.webconsole.input.context", false);
+#ifdef NIGHTLY_BUILD
+  pref("devtools.webconsole.input.context", true);
+#else
+  pref("devtools.webconsole.input.context", false);
+#endif
 
 // Set to true to eagerly show the results of webconsole terminal evaluations
 // when they don't have side effects.
@@ -1136,9 +1171,6 @@ pref("devtools.browserconsole.input.editorWidth", 0);
 // Display an onboarding UI for the Editor mode.
 pref("devtools.webconsole.input.editorOnboarding", true);
 
-// Disable the new performance recording panel by default
-pref("devtools.performance.new-panel-enabled", false);
-
 // Enable message grouping in the console, true by default
 pref("devtools.webconsole.groupWarningMessages", true);
 
@@ -1184,13 +1216,8 @@ pref("devtools.responsive.reloadConditions.userAgent", false);
 pref("devtools.responsive.reloadNotification.enabled", true);
 // Whether or not touch simulation is enabled.
 pref("devtools.responsive.touchSimulation.enabled", false);
-// Whether or not meta viewport is enabled, if and only if touchSimulation
-// is also enabled.
-pref("devtools.responsive.metaViewport.enabled", false);
 // The user agent of the viewport.
 pref("devtools.responsive.userAgent", "");
-// Whether or not the RDM UI is embedded in the browser.
-pref("devtools.responsive.browserUI.enabled", false);
 
 // Show the custom user agent input in Nightly builds.
 #if defined(NIGHTLY_BUILD)
@@ -1238,12 +1265,10 @@ pref("devtools.debugger.features.async-live-stacks", false);
 // about:devtools-toolbox tabs unusable by mistake.
 pref("devtools.popup.disable_autohide", false);
 
-// Visibility switch preference for the WhatsNew panel.
-pref("devtools.whatsnew.enabled", true);
-
-// Temporary preference to fully disable the WhatsNew panel on any target.
-// Should be removed in https://bugzilla.mozilla.org/show_bug.cgi?id=1596037
-pref("devtools.whatsnew.feature-enabled", true);
+// Enable overflow debugging in the inspector.
+pref("devtools.overflow.debugging.enabled", true);
+// Enable drag to edit properties in the inspector rule view.
+pref("devtools.inspector.draggable_properties", true);
 
 // Telemetry settings.
 
@@ -1272,15 +1297,48 @@ pref("toolkit.telemetry.bhrPing.enabled", true);
   pref("toolkit.telemetry.ecosystemtelemetry.enabled", false);
 #endif
 
-// Coverage ping is disabled by default.
-pref("toolkit.coverage.enabled", false);
-pref("toolkit.coverage.endpoint.base", "https://coverage.thunderbird.net");
-
 #ifdef XP_WIN
 pref("mail.minimizeToTray", false);
 #endif
 
 pref("prompts.defaultModalType", 3);
+pref("prompts.contentPromptSubDialog", false);
 
 // The URL for the privacy policy related to recommended extensions.
 pref("extensions.recommendations.privacyPolicyUrl", "https://www.mozilla.org/en-US/privacy/thunderbird/#addons");
+
+// Used by pdf.js to know the first time Thunderbird is run with it installed
+// so it can become the default pdf viewer.
+pref("pdfjs.firstRun", true);
+// The values of preferredAction and alwaysAskBeforeHandling before pdf.js
+// became the default.
+pref("pdfjs.previousHandler.preferredAction", 0);
+pref("pdfjs.previousHandler.alwaysAskBeforeHandling", false);
+
+pref("mail.activity.loglevel", "Warn");
+
+// The number of public recipients before we offer BCC addressing.
+pref("mail.compose.warn_public_recipients.threshold", 15);
+
+// Indicates whether to show an alert before send if no action taken while the
+// too many public recipients notification is shown.
+pref("mail.compose.warn_public_recipients.aggressive", false);
+
+// Temporary pref while both 3-pane UIs exist.
+// Opens folders in about:3pane and messages in about:message if true.
+pref("mail.useNewMailTabs", false);
+
+// The URL of most things that can be printed is useless information.
+// Hide it and move the title to the center.
+pref("print.print_headerleft", "");
+pref("print.print_headercenter", "&T");
+pref("print.print_headerright", "");
+
+// Enable Masonry Layout for AddressBook.
+pref("layout.css.grid-template-masonry-value.enabled", true);
+
+// End of year donation appeal.
+pref("app.donation.eoy.version", 1);
+pref("app.donation.eoy.version.viewed", 0);
+pref("app.donation.eoy.url", "https://www.thunderbird.net/thunderbird/102.0/eoy/");
+
