@@ -1,15 +1,13 @@
-/* -*- Mode: Java; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
 /* This Source Code Form is subject to the terms of the Mozilla Public
  * License, v. 2.0. If a copy of the MPL was not distributed with this
  * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
+/* import-globals-from mailCore.js */
+
 var { Services } = ChromeUtils.import("resource://gre/modules/Services.jsm");
 var { MailServices } = ChromeUtils.import(
   "resource:///modules/MailServices.jsm"
 );
-var { fixIterator } = ChromeUtils.import(
-  "resource:///modules/iteratorUtils.jsm"
-);
 
 var editContactInlineUI = {
   _overlayLoaded: false,
@@ -103,22 +101,24 @@ var editContactInlineUI = {
 
   _doShowEditContactPanel(aAnchorElement, aPosition) {
     this._blockCommands(); // un-done in the popuphiding handler.
-    var bundle = document.getElementById("bundle_editContact");
+    var bundle = Services.strings.createBundle(
+      "chrome://messenger/locale/editContactOverlay.properties"
+    );
 
     // Is this address book writeable?
     this._writeable = !this._cardDetails.book.readOnly;
     var type = this._writeable ? "edit" : "view";
 
     // Update the labels accordingly.
-    document.getElementById("editContactPanelTitle").value = bundle.getString(
-      type + "Title"
-    );
+    document.getElementById(
+      "editContactPanelTitle"
+    ).textContent = bundle.GetStringFromName(type + "Title");
     document.getElementById(
       "editContactPanelEditDetailsButton"
-    ).label = bundle.getString(type + "DetailsLabel");
+    ).label = bundle.GetStringFromName(type + "DetailsLabel");
     document.getElementById(
       "editContactPanelEditDetailsButton"
-    ).accessKey = bundle.getString(type + "DetailsAccessKey");
+    ).accessKey = bundle.GetStringFromName(type + "DetailsAccessKey");
 
     // We don't need a delete button for a read only card.
     document.getElementById(
@@ -138,9 +138,9 @@ var editContactInlineUI = {
 
     // Fill in the card details
     nameElement.value = this._cardDetails.card.displayName;
-    document.getElementById(
-      "editContactEmail"
-    ).value = aAnchorElement.getAttribute("emailAddress");
+    document.getElementById("editContactEmail").value =
+      aAnchorElement.getAttribute("emailAddress") ||
+      aAnchorElement.emailAddress;
 
     document.getElementById(
       "editContactAddressBookList"
@@ -156,7 +156,7 @@ var editContactInlineUI = {
           continue;
         }
 
-        for (let card of fixIterator(list.childCards, Ci.nsIAbCard)) {
+        for (let card of list.childCards) {
           if (card.primaryEmail == this._cardDetails.card.primaryEmail) {
             inMailList = true;
             break;
@@ -181,13 +181,7 @@ var editContactInlineUI = {
 
   editDetails() {
     this.saveChanges();
-
-    window.openDialog(
-      "chrome://messenger/content/addressbook/abEditCardDialog.xhtml",
-      "",
-      "chrome,modal,resizable=no,centerscreen",
-      { abURI: this._cardDetails.book.URI, card: this._cardDetails.card }
-    );
+    toAddressBook({ action: "edit", card: this._cardDetails.card });
   },
 
   deleteContact() {
@@ -199,12 +193,14 @@ var editContactInlineUI = {
     // Hide before the dialog or the panel takes the first click.
     this.panel.hidePopup();
 
-    var bundle = document.getElementById("bundle_editContact");
+    var bundle = Services.strings.createBundle(
+      "chrome://messenger/locale/editContactOverlay.properties"
+    );
     if (
       !Services.prompt.confirm(
         window,
-        bundle.getString("deleteContactTitle"),
-        bundle.getString("deleteContactMessage")
+        bundle.GetStringFromName("deleteContactTitle"),
+        bundle.GetStringFromName("deleteContactMessage")
       )
     ) {
       // XXX Would be nice to bring the popup back up here.
