@@ -12,7 +12,6 @@
  *  get it wrong and it may eventually want to migrate.
  */
 
-var { Log4Moz } = ChromeUtils.import("resource:///modules/gloda/Log4moz.jsm");
 var { PluralForm } = ChromeUtils.import(
   "resource://gre/modules/PluralForm.jsm"
 );
@@ -21,10 +20,6 @@ var { MailServices } = ChromeUtils.import(
   "resource:///modules/MailServices.jsm"
 );
 var { TagUtils } = ChromeUtils.import("resource:///modules/TagUtils.jsm");
-var { logObject, logException } = ChromeUtils.import(
-  "resource:///modules/ErrUtils.jsm"
-);
-
 var { Gloda } = ChromeUtils.import("resource:///modules/gloda/GlodaPublic.jsm");
 var { FacetDriver, FacetUtils } = ChromeUtils.import(
   "resource:///modules/gloda/Facet.jsm"
@@ -112,15 +107,16 @@ const QueryExplanation = {
           const tag = constraint[2];
           const tagNode = document.createElement("span");
           const color = MailServices.tags.getColorForKey(tag.key);
-          let textColor = "black";
-          if (!TagUtils.isColorContrastEnough(color)) {
-            textColor = "white";
-          }
           tagNode.setAttribute("class", "message-tag");
-          tagNode.setAttribute(
-            "style",
-            "color: " + textColor + "; background-color: " + color + ";"
-          );
+          if (color) {
+            let textColor = !TagUtils.isColorContrastEnough(color)
+              ? "white"
+              : "black";
+            tagNode.setAttribute(
+              "style",
+              "color: " + textColor + "; background-color: " + color + ";"
+            );
+          }
           tagNode.textContent = tag.tag;
           spanify(tagLabel, "explanation-query-tagged");
           this.node.appendChild(tagNode);
@@ -128,7 +124,7 @@ const QueryExplanation = {
       }
       label = label + constraintStrings.join(", "); // XXX l10n?
     } catch (e) {
-      logException(e);
+      Cu.reportError(e);
     }
   },
 };
@@ -487,7 +483,7 @@ var FacetContext = {
       this._sortBy = val;
       this.build(this._sieveAll());
     } catch (e) {
-      logException(e);
+      Cu.reportError(e);
     }
   },
   /**
@@ -651,8 +647,7 @@ var FacetContext = {
             try {
               explicitBinding.build(true);
             } catch (e) {
-              logObject(explicitBinding);
-              logException(e);
+              Cu.reportError(e);
             }
             explicitBinding.removeAttribute("uninitialized");
           }
@@ -767,7 +762,7 @@ var FacetContext = {
         this._hideTimeline(false);
       }
     } catch (e) {
-      logException(e);
+      Cu.reportError(e);
     }
   },
 
