@@ -22,23 +22,32 @@ var { Services } = ChromeUtils.import("resource://gre/modules/Services.jsm");
 var { MailServices } = ChromeUtils.import(
   "resource:///modules/MailServices.jsm"
 );
-var { MailUtils } = ChromeUtils.import("resource:///modules/MailUtils.jsm");
 var { PluralForm } = ChromeUtils.import(
   "resource://gre/modules/PluralForm.jsm"
 );
+ChromeUtils.defineModuleGetter(
+  this,
+  "ConversationOpener",
+  "resource:///modules/ConversationOpener.jsm"
+);
+ChromeUtils.defineModuleGetter(
+  this,
+  "MailUtils",
+  "resource:///modules/MailUtils.jsm"
+);
 
-// Controller object for folder pane
+// Controller object for folder pane.
 var FolderPaneController = {
-  get notificationbox() {
-    delete this.notificationbox;
-
-    let newNotificationBox = new MozElements.NotificationBox(element => {
-      element.setAttribute("flex", "1");
-      element.setAttribute("notificationside", "bottom");
-      document.getElementById("messenger-notification-footer").append(element);
-    });
-
-    return (this.notificationbox = newNotificationBox);
+  get notificationBox() {
+    if (!this._notificationBox) {
+      this._notificationBox = new MozElements.NotificationBox(element => {
+        element.setAttribute("notificationside", "bottom");
+        document
+          .getElementById("messenger-notification-footer")
+          .append(element);
+      });
+    }
+    return this._notificationBox;
   },
 
   supportsCommand(command) {
@@ -152,6 +161,7 @@ var DefaultController = {
       case "button_forward":
       case "cmd_forwardInline":
       case "cmd_forwardAttachment":
+      case "cmd_redirect":
       case "cmd_editAsNew":
       case "cmd_editDraftMsg":
       case "cmd_newMsgFromTemplate":
@@ -204,8 +214,6 @@ var DefaultController = {
       case "cmd_openConversation":
       case "button_print":
       case "cmd_print":
-      case "cmd_printpreview":
-      case "cmd_printSetup":
       case "cmd_saveAsFile":
       case "cmd_saveAsTemplate":
       case "cmd_properties":
@@ -262,6 +270,7 @@ var DefaultController = {
       case "cmd_fullZoomEnlarge":
       case "cmd_fullZoomReset":
       case "cmd_fullZoomToggle":
+      case "cmd_fontSizeReset":
       case "cmd_viewAllHeader":
       case "cmd_viewNormalHeader":
       case "cmd_stop":
@@ -270,6 +279,10 @@ var DefaultController = {
       case "cmd_killThread":
       case "cmd_killSubthread":
         return true;
+      case "cmd_fontSizeReduce":
+        return UIFontSize.size > UIFontSize.MIN_VALUE;
+      case "cmd_fontSizeEnlarge":
+        return UIFontSize.size < UIFontSize.MAX_VALUE;
       case "cmd_downloadFlagged":
       case "cmd_downloadSelected":
       case "cmd_synchronizeOffline":
@@ -341,7 +354,7 @@ var DefaultController = {
       case "button_showconversation":
         return (
           gFolderDisplay.selectedCount == 1 &&
-          gConversationOpener.isSelectedMessageIndexed()
+          ConversationOpener.isMessageIndexed(gFolderDisplay.selectedMessage)
         );
       case "cmd_saveAsFile":
         return gFolderDisplay.selectedCount > 0;
@@ -362,16 +375,20 @@ var DefaultController = {
       case "button_forward":
       case "cmd_forwardInline":
       case "cmd_forwardAttachment":
+      case "cmd_redirect":
       case "cmd_editAsNew":
       case "cmd_editDraftMsg":
       case "cmd_newMsgFromTemplate":
       case "cmd_editTemplateMsg":
       case "cmd_openMessage":
       case "button_print":
-      case "cmd_print":
       case "cmd_viewPageSource":
       case "cmd_reload":
       case "cmd_applyFiltersToSelection":
+        if (!CanComposeMessages()) {
+          return false;
+        }
+
         let numSelected = gFolderDisplay.selectedCount;
         if (command == "cmd_applyFiltersToSelection") {
           var whichText = "valueMessage";
@@ -417,16 +434,10 @@ var DefaultController = {
           return true;
         }
         return false;
-      case "cmd_printpreview":
-        if (gFolderDisplay.selectedCount == 1) {
-          return gFolderDisplay.getCommandStatus(
-            Ci.nsMsgViewCommandType.cmdRequiringMsgBody
-          );
-        }
-        return false;
+      case "cmd_print":
+        return gFolderDisplay.selectedCount >= 1;
       case "cmd_newMessage":
         return CanComposeMessages();
-      case "cmd_printSetup":
       case "cmd_viewAllHeader":
       case "cmd_viewNormalHeader":
         return true;
@@ -479,8 +490,9 @@ var DefaultController = {
       case "cmd_tag8":
       case "cmd_tag9":
       case "cmd_toggleRead":
-      case "cmd_markThreadAsRead":
         return gFolderDisplay.selectedCount > 0;
+      case "cmd_markThreadAsRead":
+        return gFolderDisplay.canMarkThreadAsRead;
       case "cmd_markAsRead":
         return CanMarkMsgAsRead(true);
       case "cmd_markAsUnread":
@@ -534,9 +546,8 @@ var DefaultController = {
           return true;
         }
 
-        // Otherwise, only allow searching if we're showing the message pane
-        // and have more than one message selected.
-        return !IsMessagePaneCollapsed() && gFolderDisplay.selectedCount == 1;
+        // Otherwise, only allow searching if we're showing the message pane.
+        return !IsMessagePaneCollapsed();
       case "cmd_search":
         return MailServices.accounts.accounts.length > 0;
       case "cmd_selectAll":
@@ -609,8 +620,7 @@ var DefaultController = {
           return (
             !folder.isServer &&
             !folder.getFlag(Ci.nsMsgFolderFlags.Virtual) &&
-            (folder.server.type != "imap" ||
-              folder.server.canCompactFoldersOnServer) &&
+            folder.server.canCompactFoldersOnServer &&
             folder.isCommandEnabled("button_compact")
           );
         };
@@ -620,8 +630,7 @@ var DefaultController = {
         let folders = gFolderTreeView.getSelectedFolders();
         let canCompactAll = function(folder) {
           return (
-            (folder.server.type != "imap" ||
-              folder.server.canCompactFoldersOnServer) &&
+            folder.server.canCompactFoldersOnServer &&
             folder.isCommandEnabled("cmd_compactFolder")
           );
         };
@@ -650,7 +659,7 @@ var DefaultController = {
             return false;
           }
         }
-        let targetURI = Services.prefs.getCharPref(
+        let targetURI = Services.prefs.getStringPref(
           "mail.last_msg_movecopy_target_uri"
         );
         if (!targetURI) {
@@ -671,6 +680,11 @@ var DefaultController = {
           return true;
         }
         return IsFolderSelected() && !IsMessagePaneCollapsed();
+      case "cmd_fontSizeReduce":
+        return UIFontSize.size > UIFontSize.MIN_VALUE;
+      case "cmd_fontSizeEnlarge":
+        return UIFontSize.size < UIFontSize.MAX_VALUE;
+      case "cmd_fontSizeReset":
       case "cmd_chat":
         return true;
       case "cmd_newFolder":
@@ -734,6 +748,9 @@ var DefaultController = {
       case "cmd_forwardAttachment":
         MsgForwardAsAttachment(null);
         break;
+      case "cmd_redirect":
+        MsgRedirectMessage(null);
+        break;
       case "cmd_editAsNew":
         MsgEditMessageAsNew(null);
         break;
@@ -790,7 +807,7 @@ var DefaultController = {
               false
             );
           } else {
-            FolderPaneController.notificationbox.removeTransientNotifications();
+            FolderPaneController.notificationBox.removeTransientNotifications();
           }
         }
         // kill thread kills the thread and then does a next unread
@@ -804,7 +821,7 @@ var DefaultController = {
               true
             );
           } else {
-            FolderPaneController.notificationbox.removeTransientNotifications();
+            FolderPaneController.notificationBox.removeTransientNotifications();
           }
         }
         GoNextMessage(Ci.nsMsgNavigationType.toggleSubthreadKilled, true);
@@ -907,18 +924,12 @@ var DefaultController = {
         return;
       case "cmd_openConversation":
       case "button_showconversation":
-        gConversationOpener.openConversationForMessages(
+        new ConversationOpener(window).openConversationForMessages(
           gFolderDisplay.selectedMessages
         );
         return;
-      case "cmd_printSetup":
-        PrintUtils.showPageSetup();
-        return;
       case "cmd_print":
-        PrintEnginePrint();
-        return;
-      case "cmd_printpreview":
-        PrintEnginePrintPreview();
+        PrintSelectedMessages();
         return;
       case "cmd_saveAsFile":
         MsgSaveAsFile();
@@ -1068,7 +1079,7 @@ var DefaultController = {
         break;
       case "cmd_moveToFolderAgain":
         var folder = MailUtils.getOrCreateFolder(
-          Services.prefs.getCharPref("mail.last_msg_movecopy_target_uri")
+          Services.prefs.getStringPref("mail.last_msg_movecopy_target_uri")
         );
         if (Services.prefs.getBoolPref("mail.last_msg_movecopy_was_move")) {
           MsgMoveMessage(folder);
@@ -1106,6 +1117,15 @@ var DefaultController = {
       case "cmd_fullZoomToggle":
         ZoomManager.toggleZoom();
         break;
+      case "cmd_fontSizeReduce":
+        UIFontSize.reduceSize();
+        break;
+      case "cmd_fontSizeReset":
+        UIFontSize.resetSize();
+        break;
+      case "cmd_fontSizeEnlarge":
+        UIFontSize.increaseSize();
+        break;
       case "cmd_chat":
         showChatTab();
         break;
@@ -1136,7 +1156,7 @@ var DefaultController = {
  *                   ignore thread
  */
 function ShowIgnoredMessageNotification(aMsgs, aSubthreadOnly) {
-  let notifyBox = FolderPaneController.notificationbox;
+  let notifyBox = FolderPaneController.notificationBox;
   notifyBox.removeTransientNotifications(); // don't want to pile these up
 
   let bundle = Services.strings.createBundle(
@@ -1203,10 +1223,12 @@ function ShowIgnoredMessageNotification(aMsgs, aSubthreadOnly) {
     let text = ignoredThreadText.replace("#1", subj);
 
     notifyBox.appendNotification(
-      text,
       "ignoreThreadInfo",
+      {
+        label: text,
+        priority: notifyBox.PRIORITY_INFO_MEDIUM,
+      },
       null,
-      notifyBox.PRIORITY_INFO_MEDIUM,
       buttons
     );
   } else {
@@ -1218,10 +1240,11 @@ function ShowIgnoredMessageNotification(aMsgs, aSubthreadOnly) {
       nbrOfThreads
     );
     notifyBox.appendNotification(
-      text,
       "ignoreThreadsInfo",
-      null,
-      notifyBox.PRIORITY_INFO_MEDIUM,
+      {
+        label: text,
+        priority: notifyBox.PRIORITY_INFO_MEDIUM,
+      },
       buttons
     );
   }
@@ -1277,6 +1300,13 @@ function FocusRingUpdate_Mail() {
 }
 
 function RestoreFocusAfterHdrButton() {
+  // Note: Some callers call this function *after* asynchronous functions like
+  // save-as which will trigger a save dialog. As those dialogs are typically
+  // slower to load, in reality this function may set focus on thread pane
+  // *before* the dialog is shown on screen. This does not seem to cause
+  // problems like dialogs being hidden behind the main window, probably because
+  // they're application-modal OS dialogs and will ensure having focus anyway.
+  //
   // I would love to really restore the focus to the pane that had
   // focus before the user clicked on the hdr button, and gLastFocusedElement
   // would almost do that, except that clicking on the hdr button sets
@@ -1408,36 +1438,80 @@ function SetFocusThreadPaneIfNotOnMessagePane() {
 // 3pane related commands.  Need to go in own file.  Putting here for the moment.
 
 /**
- * Cycle through the various panes in the 3pane window (in reverse if the shift
- * key is being held down).
+ * Cycle through the various panes in the 3pane window.
  *
- * @param event the event that triggered us
+ * @param {Event} event - The keypress DOMEvent.
  */
 function SwitchPaneFocus(event) {
-  let messagePane = GetMessagePane();
-
   // First, build an array of panes to cycle through based on our current state.
   // This will usually be something like [threadPane, messagePane, folderPane].
-  let panes = [GetThreadTree()];
+  let panes = [];
+  let focusedElement;
+  let spacesElement = !gSpacesToolbar.isHidden
+    ? gSpacesToolbar.focusButton
+    : document.getElementById("spacesPinnedButton");
 
-  if (!IsMessagePaneCollapsed()) {
-    panes.push(messagePane);
-  }
+  let { currentTabInfo } = document.getElementById("tabmail");
+  if (currentTabInfo.mode.name == "mail3PaneTab") {
+    let { browser, folderPaneVisible, messagePaneVisible } = currentTabInfo;
+    let {
+      document: contentDocument,
+      folderTree,
+      threadTree,
+      webBrowser,
+      messageBrowser,
+      multiMessageBrowser,
+      accountCentralBrowser,
+    } = browser.contentWindow;
 
-  if (gFolderDisplay.folderPaneVisible) {
-    panes.push(document.getElementById("folderTree"));
+    panes.push(spacesElement);
+
+    if (folderPaneVisible) {
+      panes.push(folderTree);
+    }
+
+    if (accountCentralBrowser.hidden) {
+      panes.push(threadTree);
+    } else {
+      panes.push(accountCentralBrowser);
+    }
+
+    if (messagePaneVisible) {
+      for (let browser of [webBrowser, messageBrowser, multiMessageBrowser]) {
+        if (!browser.hidden) {
+          panes.push(browser);
+        }
+      }
+    }
+
+    focusedElement = contentDocument.activeElement;
+  } else if (currentTabInfo.mode.tabType.name == "mail") {
+    panes.push(spacesElement);
+
+    if (gFolderDisplay.folderPaneVisible) {
+      panes.push(document.getElementById("folderTree"));
+    }
+
+    panes.push(GetThreadTree());
+
+    if (!IsMessagePaneCollapsed()) {
+      panes.push(GetMessagePane());
+    }
+
+    focusedElement = gFolderDisplay.focusedPane;
+  } else {
+    return;
   }
 
   // Find our focused element in the array. If focus is not on one of the main
   // panes (it's probably on the toolbar), then act as if it's on the thread
   // tree.
-  let focusedElement = gFolderDisplay.focusedPane;
   let focusedElementIndex = panes.indexOf(focusedElement);
   if (focusedElementIndex == -1) {
     focusedElementIndex = 0;
   }
 
-  if (event && event.shiftKey) {
+  if (event.shiftKey) {
     focusedElementIndex--;
     if (focusedElementIndex == -1) {
       focusedElementIndex = panes.length - 1;
@@ -1454,7 +1528,7 @@ function SwitchPaneFocus(event) {
   // We need to handle the message pane specially, since focusing it isn't as
   // simple as just calling focus(). See SetFocusMessagePane below for more
   // details.
-  if (newElem == messagePane) {
+  if (newElem == GetMessagePane()) {
     SetFocusMessagePane();
   } else {
     newElem.focus();
@@ -1462,8 +1536,7 @@ function SwitchPaneFocus(event) {
 }
 
 function SetFocusThreadPane() {
-  var threadTree = GetThreadTree();
-  threadTree.focus();
+  GetThreadTree()?.focus();
 }
 
 /**
@@ -1476,46 +1549,19 @@ function SetFocusMessagePane() {
   // pane looking like it has focus.
   GetMessagePane().focus();
   if (gMessageDisplay.singleMessageDisplay) {
-    GetMessagePaneFrame().focus();
+    getMessagePaneBrowser().focus();
   } else {
     document.getElementById("multimessage").focus();
   }
 }
 
-//
-// This function checks if the configured junk mail can be renamed or deleted.
-//
-function CanRenameDeleteJunkMail(aFolderUri) {
-  if (!aFolderUri) {
-    return false;
-  }
-
-  // Go through junk mail settings for all servers and see if the folder is set/used by anyone.
-  try {
-    for (let server of MailServices.accounts.allServers) {
-      var settings = server.spamSettings;
-      // If junk mail control or move junk mail to folder option is disabled then
-      // allow the folder to be removed/renamed since the folder is not used in this case.
-      if (!settings.level || !settings.moveOnSpam) {
-        continue;
-      }
-      if (settings.spamFolderURI == aFolderUri) {
-        return false;
-      }
-    }
-  } catch (ex) {
-    dump("Can't get all servers\n");
-  }
-  return true;
-}
-
 /** Check if this is a folder the user is allowed to delete. */
 function CanDeleteFolder(folder) {
   if (folder.isServer) {
     return false;
   }
 
-  var specialFolder = getSpecialFolderString(folder);
+  var specialFolder = FolderUtils.getSpecialFolderString(folder);
 
   if (
     specialFolder == "Inbox" ||
@@ -1524,10 +1570,50 @@ function CanDeleteFolder(folder) {
     specialFolder == "Sent" ||
     specialFolder == "Templates" ||
     specialFolder == "Outbox" ||
-    (specialFolder == "Junk" && !CanRenameDeleteJunkMail(folder.URI))
+    (specialFolder == "Junk" &&
+      !FolderUtils.canRenameDeleteJunkMail(folder.URI))
   ) {
     return false;
   }
 
   return true;
 }
+
+/** Prints the messages selected in the thread pane. */
+async function PrintSelectedMessages() {
+  if (gFolderDisplay.selectedCount == 1) {
+    if (
+      gMessageDisplay.visible &&
+      gFolderDisplay.selectedMessage == gMessageDisplay.displayedMessage
+    ) {
+      // Use the already displayed message and print preview UI if we can.
+      let messagePaneBrowser = document.getElementById("messagepane");
+      PrintUtils.startPrintWindow(messagePaneBrowser.browsingContext, {});
+    } else {
+      // Load the only message in a hidden browser, then use the print preview UI.
+      let uri = gFolderDisplay.selectedMessageUris[0];
+      let messageService = messenger.messageServiceFromURI(uri);
+      await PrintUtils.loadPrintBrowser(messageService.getUrlForUri(uri).spec);
+      PrintUtils.startPrintWindow(PrintUtils.printBrowser.browsingContext, {});
+    }
+
+    return;
+  }
+
+  // Multiple messages. Get the printer settings, then load the messages into
+  // a hidden browser and print them one at a time.
+  let ps = PrintUtils.getPrintSettings();
+  Cc["@mozilla.org/widget/printdialog-service;1"]
+    .getService(Ci.nsIPrintDialogService)
+    .showPrintDialog(window, false, ps);
+  if (ps.isCancelled) {
+    return;
+  }
+  ps.printSilent = true;
+
+  for (let uri of gFolderDisplay.selectedMessageUris) {
+    let messageService = messenger.messageServiceFromURI(uri);
+    await PrintUtils.loadPrintBrowser(messageService.getUrlForUri(uri).spec);
+    await PrintUtils.printBrowser.browsingContext.print(ps);
+  }
+}
