@@ -1,4 +1,4 @@
-/* -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*-
+/* -*- Mode: JS; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*-
  * This Source Code Form is subject to the terms of the Mozilla Public
  * License, v. 2.0. If a copy of the MPL was not distributed with this
  * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
@@ -12,9 +12,9 @@
  * to be used by all of the main mail windows?
  */
 
-/* import-globals-from ../../../common/src/customizeToolbar.js */
 /* import-globals-from ../../extensions/mailviews/content/msgViewPickerOverlay.js */
 /* import-globals-from commandglue.js */
+/* import-globals-from customizeToolbar.js */
 /* import-globals-from mailWindow.js */
 /* import-globals-from utilityOverlay.js */
 
@@ -25,9 +25,6 @@ var { AppConstants } = ChromeUtils.import(
 var { XPCOMUtils } = ChromeUtils.import(
   "resource://gre/modules/XPCOMUtils.jsm"
 );
-var { CharsetMenu } = ChromeUtils.import(
-  "resource://gre/modules/CharsetMenu.jsm"
-);
 
 XPCOMUtils.defineLazyGetter(this, "gViewSourceUtils", function() {
   let scope = {};
@@ -55,48 +52,6 @@ XPCOMUtils.defineLazyGetter(this, "gViewSourceUtils", function() {
   return scope.gViewSourceUtils;
 });
 
-Services.obs.addObserver(
-  {
-    observe(win) {
-      win.addEventListener(
-        "load",
-        function() {
-          if (
-            this.location.href !=
-            "chrome://devtools/content/webconsole/index.html"
-          ) {
-            return;
-          }
-
-          this.setTimeout(() => {
-            this.gViewSourceUtils = {
-              async viewSource(aArgs) {
-                // Check if external view source is enabled. If so, try it. If it fails,
-                // fallback to internal view source.
-                if (Services.prefs.getBoolPref("view_source.editor.external")) {
-                  try {
-                    await this.openInExternalEditor(aArgs);
-                    return;
-                  } catch (ex) {}
-                }
-
-                window.openDialog(
-                  "chrome://messenger/content/viewSource.xhtml",
-                  "_blank",
-                  "all,dialog=no",
-                  aArgs
-                );
-              },
-            };
-          });
-        },
-        { capture: false, once: true }
-      );
-    },
-  },
-  "chrome-document-global-created"
-);
-
 var gCustomizeSheet = false;
 
 function overlayRestoreDefaultSet() {
@@ -237,12 +192,12 @@ function CustomizeMailToolbar(toolboxId, customizePopupId) {
     }
     if (
       AppConstants.platform != "macosx" &&
-      document.getElementById("mail-toolbar-menubar2")
+      document.getElementById("toolbar-menubar")
     ) {
-      externalToolbars.push(document.getElementById("mail-toolbar-menubar2"));
+      externalToolbars.push(document.getElementById("toolbar-menubar"));
     }
-    if (document.getElementById("folderPane-toolbar")) {
-      externalToolbars.push(document.getElementById("folderPane-toolbar"));
+    if (document.getElementById("folderPaneHeader")) {
+      externalToolbars.push(document.getElementById("folderPaneHeader"));
     }
   }
 
@@ -372,30 +327,39 @@ function MailToolboxCustomizeDone(aEvent, customizePopupId) {
  * example, in the appmenu / Preferences view.  Adds toolbar items to the popup
  * and sets their attributes.
  *
- * @param {Event} event                 Event causing the menu popup to appear.
- * @param {string|string[]} toolboxIds  IDs of toolboxes that contain toolbars.
- * @param {Element} insertPoint         Where to insert menu items.
- * @param {string} [elementName]        What kind of menu item element to use.
- *                                      E.g. "toolbarbutton" for the appmenu.
- * @param {string} [classes]            Classes to set on menu items.
+ * @param {Event} event - Event causing the menu popup to appear.
+ * @param {string|string[]} toolboxIds - IDs of toolboxes that contain toolbars.
+ * @param {Element} insertPoint - Where to insert menu items.
+ * @param {string} elementName - What kind of menu item element to use. E.g.
+ *   "toolbarbutton" for the appmenu.
+ * @param {string} classes - Classes to set on menu items.
+ * @param {boolean} keepOpen - If to force the menu to stay open when clicking
+ *   on this element.
  */
 function onViewToolbarsPopupShowing(
   event,
   toolboxIds,
   insertPoint,
   elementName = "menuitem",
-  classes
+  classes,
+  keepOpen = false
 ) {
   if (!Array.isArray(toolboxIds)) {
     toolboxIds = [toolboxIds];
   }
 
-  const popup =
-    event.target.querySelector(".panel-subview-body") || event.target;
+  let popup = event.target.querySelector(".panel-subview-body") || event.target;
+  // Limit the toolbar menu entries to the first level of context menus.
+  if (
+    popup != event.currentTarget &&
+    event.currentTarget.tagName == "menupopup"
+  ) {
+    return;
+  }
 
   // Remove all collapsible nodes from the menu.
   for (let i = popup.children.length - 1; i >= 0; --i) {
-    const deadItem = popup.children[i];
+    let deadItem = popup.children[i];
 
     if (deadItem.hasAttribute("iscollapsible")) {
       deadItem.remove();
@@ -403,63 +367,69 @@ function onViewToolbarsPopupShowing(
   }
 
   // We insert menuitems before the first child if no insert point is given.
-  const firstMenuItem = insertPoint || popup.firstElementChild;
+  let firstMenuItem = insertPoint || popup.firstElementChild;
 
-  for (const toolboxId of toolboxIds) {
-    const toolbox = document.getElementById(toolboxId);
+  for (let toolboxId of toolboxIds) {
+    let toolbox = document.getElementById(toolboxId);
 
     // We consider child nodes that have a toolbarname attribute.
-    const toolbars = Array.from(toolbox.querySelectorAll("[toolbarname]"));
+    let toolbars = Array.from(toolbox.querySelectorAll("[toolbarname]"));
 
-    // Add the folder pane toolbar to the list of toolbars that can be shown and
-    // hidden.
-    if (toolbox.getAttribute("id") === "mail-toolbox") {
-      if (
-        AppConstants.platform != "macosx" &&
-        document.getElementById("mail-toolbar-menubar2")
-      ) {
-        toolbars.push(document.getElementById("mail-toolbar-menubar2"));
-      }
-      if (document.getElementById("folderPane-toolbar")) {
-        toolbars.push(document.getElementById("folderPane-toolbar"));
+    // On main Window add the folder pane toolbar to the list of toolbars that
+    // can be shown and hidden.
+    if (document.getElementById("tabmail")) {
+      if (toolbox.getAttribute("id") === "mail-toolbox") {
+        if (
+          AppConstants.platform != "macosx" &&
+          document.getElementById("toolbar-menubar")
+        ) {
+          toolbars.push(document.getElementById("toolbar-menubar"));
+        }
+        if (document.getElementById("folderPaneHeader")) {
+          toolbars.push(document.getElementById("folderPaneHeader"));
+        }
       }
     }
 
-    for (const toolbar of toolbars) {
-      const toolbarName = toolbar.getAttribute("toolbarname");
-      if (toolbarName) {
-        const menuItem = document.createXULElement(elementName);
-
-        const hidingAttribute =
-          toolbar.getAttribute("type") == "menubar" ? "autohide" : "collapsed";
-
-        menuItem.setAttribute("type", "checkbox");
-        // Mark this menuitem with an iscollapsible attribute, so we
-        // know we can wipe it out later on.
-        menuItem.setAttribute("iscollapsible", true);
-        menuItem.setAttribute("toolbarid", toolbar.id);
-        menuItem.setAttribute("label", toolbarName);
-        menuItem.setAttribute("accesskey", toolbar.getAttribute("accesskey"));
-        menuItem.setAttribute(
-          "checked",
-          toolbar.getAttribute(hidingAttribute) != "true"
-        );
-        if (classes) {
-          menuItem.setAttribute("class", classes);
-        }
-        popup.insertBefore(menuItem, firstMenuItem);
-
-        menuItem.addEventListener("command", () => {
-          const hidden = toolbar.getAttribute(hidingAttribute) != "true";
-
-          if (hidden) {
-            toolbar.setAttribute(hidingAttribute, "true");
-          } else {
-            toolbar.removeAttribute(hidingAttribute);
-          }
-          Services.xulStore.persist(toolbar, hidingAttribute);
-        });
+    for (let toolbar of toolbars) {
+      let toolbarName = toolbar.getAttribute("toolbarname");
+      if (!toolbarName) {
+        continue;
       }
+
+      let menuItem = document.createXULElement(elementName);
+      let hidingAttribute =
+        toolbar.getAttribute("type") == "menubar" ? "autohide" : "collapsed";
+
+      menuItem.setAttribute("type", "checkbox");
+      // Mark this menuitem with an iscollapsible attribute, so we
+      // know we can wipe it out later on.
+      menuItem.setAttribute("iscollapsible", true);
+      menuItem.setAttribute("toolbarid", toolbar.id);
+      menuItem.setAttribute("label", toolbarName);
+      menuItem.setAttribute("accesskey", toolbar.getAttribute("accesskey"));
+      menuItem.setAttribute(
+        "checked",
+        toolbar.getAttribute(hidingAttribute) != "true"
+      );
+      if (classes) {
+        menuItem.setAttribute("class", classes);
+      }
+      if (keepOpen) {
+        menuItem.setAttribute("closemenu", "none");
+      }
+      popup.insertBefore(menuItem, firstMenuItem);
+
+      menuItem.addEventListener("command", () => {
+        if (toolbar.getAttribute(hidingAttribute) != "true") {
+          toolbar.setAttribute(hidingAttribute, "true");
+          menuItem.removeAttribute("checked");
+        } else {
+          menuItem.setAttribute("checked", true);
+          toolbar.removeAttribute(hidingAttribute);
+        }
+        Services.xulStore.persist(toolbar, hidingAttribute);
+      });
     }
   }
 }
@@ -470,26 +440,24 @@ function toJavaScriptConsole() {
 
 function openAboutDebugging(hash) {
   let url = "about:debugging" + (hash ? "#" + hash : "");
-  document
-    .getElementById("tabmail")
-    .openTab("contentTab", { contentPage: url });
+  document.getElementById("tabmail").openTab("contentTab", { url });
 }
 
 function toOpenWindowByType(inType, uri) {
   var topWindow = Services.wm.getMostRecentWindow(inType);
   if (topWindow) {
     topWindow.focus();
-  } else {
-    window.open(
-      uri,
-      "_blank",
-      "chrome,extrachrome,menubar,resizable,scrollbars,status,toolbar"
-    );
+    return topWindow;
   }
+  return window.open(
+    uri,
+    "_blank",
+    "chrome,extrachrome,menubar,resizable,scrollbars,status,toolbar"
+  );
 }
 
 function toMessengerWindow() {
-  toOpenWindowByType(
+  return toOpenWindowByType(
     "mail:3pane",
     "chrome://messenger/content/messenger.xhtml"
   );
@@ -514,11 +482,72 @@ function focusOnMail(tabNo, event) {
   }
 }
 
-function toAddressBook() {
-  toOpenWindowByType(
-    "mail:addressbook",
-    "chrome://messenger/content/addressbook/addressbook.xhtml"
-  );
+/**
+ * Open the address book and optionally display/edit a card.
+ *
+ * @param {?object} openArgs - Arguments to pass to the address book.
+ *   See `externalAction` in aboutAddressBook.js for details.
+ */
+async function toAddressBook(openArgs) {
+  let messengerWindow = toMessengerWindow();
+  if (messengerWindow.document.readyState != "complete") {
+    await new Promise(resolve => {
+      Services.obs.addObserver(
+        {
+          observe(subject) {
+            if (subject == messengerWindow) {
+              Services.obs.removeObserver(this, "mail-tabs-session-restored");
+              resolve();
+            }
+          },
+        },
+        "mail-tabs-session-restored"
+      );
+    });
+  }
+
+  return new Promise(resolve => {
+    messengerWindow.tabmail.openTab("addressBookTab", {
+      onLoad(event, browser) {
+        if (openArgs) {
+          browser.contentWindow.externalAction(openArgs);
+        }
+        resolve(browser.contentWindow);
+      },
+    });
+    messengerWindow.focus();
+  });
+}
+
+/**
+ * Open the calendar.
+ */
+async function toCalendar() {
+  let messengerWindow = toMessengerWindow();
+  if (messengerWindow.document.readyState != "complete") {
+    await new Promise(resolve => {
+      Services.obs.addObserver(
+        {
+          observe(subject) {
+            if (subject == messengerWindow) {
+              Services.obs.removeObserver(this, "mail-tabs-session-restored");
+              resolve();
+            }
+          },
+        },
+        "mail-tabs-session-restored"
+      );
+    });
+  }
+
+  return new Promise(resolve => {
+    messengerWindow.tabmail.openTab("calendar", {
+      onLoad(event, browser) {
+        resolve(browser.contentWindow);
+      },
+    });
+    messengerWindow.focus();
+  });
 }
 
 function showChatTab() {
@@ -530,11 +559,45 @@ function showChatTab() {
   }
 }
 
-function toImport() {
+/**
+ * Open about:import or importDialog.xhtml.
+ * @param {"start"|"app"|"addressBook"|"calendar"|"export"} [tabId] - The tab
+ *  to open in about:import.
+ */
+function toImport(tabId = "start") {
+  if (Services.prefs.getBoolPref("mail.import.in_new_tab")) {
+    let tab = toMessengerWindow().openTab("contentTab", {
+      url: "about:import",
+      onLoad(event, browser) {
+        if (tabId) {
+          browser.contentWindow.showTab(`tab-${tabId}`, true);
+        }
+      },
+    });
+    // Somehow DOMContentLoaded is called even when about:import is already
+    // open, which resets the active tab. Use setTimeout here as a workaround.
+    setTimeout(
+      () => tab.browser.contentWindow.showTab(`tab-${tabId}`, true),
+      100
+    );
+    return;
+  }
   window.openDialog(
     "chrome://messenger/content/importDialog.xhtml",
     "importDialog",
-    "chrome, modal, titlebar, centerscreen"
+    "chrome,modal,titlebar,centerscreen"
+  );
+}
+
+function toExport() {
+  if (Services.prefs.getBoolPref("mail.import.in_new_tab")) {
+    toImport("export");
+    return;
+  }
+  window.openDialog(
+    "chrome://messenger/content/exportDialog.xhtml",
+    "exportDialog",
+    "chrome,modal,titlebar,centerscreen"
   );
 }
 
@@ -564,7 +627,7 @@ function openAddonsMgr(aView) {
     let browserWindow;
 
     let receivePong = function(aSubject, aTopic, aData) {
-      let browserWin = aSubject.docShell.rootTreeItem.domWindow;
+      let browserWin = aSubject.browsingContext.topChromeWindow;
       if (!emWindow || browserWin == window /* favor the current window */) {
         emWindow = aSubject;
         browserWindow = browserWin;
@@ -587,10 +650,7 @@ function openAddonsMgr(aView) {
 
     // This must be a new load, else the ping/pong would have
     // found the window above.
-    let addonSiteRegExp = Services.prefs.getCharPref(
-      "extensions.getAddons.siteRegExp"
-    );
-    let tab = openContentTab("about:addons", "tab", addonSiteRegExp);
+    let tab = openContentTab("about:addons");
     // Also in `contentTabType.restoreTab` in specialTabs.js.
     tab.browser.droppedLinkHandler = event =>
       tab.browser.contentWindow.gDragDrop.onDrop(event);
@@ -649,16 +709,7 @@ function openIMAccountWizard() {
 }
 
 function openSavedFilesWnd() {
-  let tabmail = document.getElementById("tabmail");
-  let downloadsBrowser = tabmail.getBrowserForDocumentId("aboutDownloads");
-  if (downloadsBrowser) {
-    tabmail.switchToTab(downloadsBrowser);
-  } else {
-    tabmail.openTab("chromeTab", {
-      chromePage: "about:downloads",
-      clickHandler: "specialTabs.aboutClickHandler(event);",
-    });
-  }
+  return openContentTab("about:downloads");
 }
 
 function SetBusyCursor(window, enable) {
@@ -735,8 +786,7 @@ function openAboutSupport() {
   if (mailWindow) {
     mailWindow.focus();
     mailWindow.document.getElementById("tabmail").openTab("contentTab", {
-      contentPage: "about:support",
-      clickHandler: "specialTabs.aboutClickHandler(event);",
+      url: "about:support",
     });
     return;
   }
@@ -748,10 +798,7 @@ function openAboutSupport() {
     null,
     {
       tabType: "contentTab",
-      tabParams: {
-        contentPage: "about:support",
-        clickHandler: "specialTabs.aboutClickHandler(event);",
-      },
+      tabParams: { url: "about:support" },
     }
   );
 }
@@ -784,9 +831,13 @@ function safeModeRestart() {
   let bundle = Services.strings.createBundle(
     "chrome://messenger/locale/messenger.properties"
   );
-  let promptTitle = bundle.GetStringFromName("safeModeRestartPromptTitle");
-  let promptMessage = bundle.GetStringFromName("safeModeRestartPromptMessage");
-  let restartText = bundle.GetStringFromName("safeModeRestartButton");
+  let promptTitle = bundle.GetStringFromName(
+    "troubleshootModeRestartPromptTitle"
+  );
+  let promptMessage = bundle.GetStringFromName(
+    "troubleshootModeRestartPromptMessage"
+  );
+  let restartText = bundle.GetStringFromName("troubleshootModeRestartButton");
   let buttonFlags =
     Services.prompt.BUTTON_POS_0 * Services.prompt.BUTTON_TITLE_IS_STRING +
     Services.prompt.BUTTON_POS_1 * Services.prompt.BUTTON_TITLE_CANCEL +
@@ -808,10 +859,8 @@ function safeModeRestart() {
       Ci.nsIEnvironment
     );
     environment.set("MOZ_SAFE_MODE_RESTART", "1");
-    let { BrowserUtils } = ChromeUtils.import(
-      "resource://gre/modules/BrowserUtils.jsm"
-    );
-    BrowserUtils.restartApplication();
+    let { MailUtils } = ChromeUtils.import("resource:///modules/MailUtils.jsm");
+    MailUtils.restartApplication();
   }
 }
 
@@ -860,14 +909,9 @@ function SanitizeAttachmentDisplayName(aAttachment) {
  * @param {nsIMsgAttachment[]} attachments - The attachments to setup
  */
 function setupDataTransfer(event, attachments) {
-  // For now, disallow drag-and-drop on cloud attachments. In the future, we
-  // should allow this.
   let index = 0;
   for (let attachment of attachments) {
-    if (
-      attachment.contentType == "text/x-moz-deleted" ||
-      attachment.sendViaCloud
-    ) {
+    if (attachment.contentType == "text/x-moz-deleted") {
       return;
     }
 
@@ -879,23 +923,24 @@ function setupDataTransfer(event, attachments) {
 
     // Only add type/filename info for non-file URLs that don't already
     // have it.
-    let info;
+    let info = [];
     if (/(^file:|&filename=)/.test(attachment.url)) {
-      info = attachment.url;
+      info.push(attachment.url);
     } else {
-      info =
+      info.push(
         attachment.url +
-        "&type=" +
-        attachment.contentType +
-        "&filename=" +
-        encodeURIComponent(name);
+          "&type=" +
+          attachment.contentType +
+          "&filename=" +
+          encodeURIComponent(name)
+      );
+    }
+    info.push(name, attachment.size, attachment.contentType, attachment.uri);
+    if (attachment.sendViaCloud) {
+      info.push(attachment.cloudFileAccountKey, attachment.cloudPartHeaderData);
     }
 
-    event.dataTransfer.mozSetDataAt(
-      "text/x-moz-url",
-      info + "\n" + name + "\n" + attachment.size,
-      index
-    );
+    event.dataTransfer.mozSetDataAt("text/x-moz-url", info.join("\n"), index);
     event.dataTransfer.mozSetDataAt(
       "text/x-moz-url-data",
       attachment.url,
@@ -916,6 +961,24 @@ function setupDataTransfer(event, attachments) {
   }
 }
 
+/**
+ * Checks if Thunderbird was launched in safe mode and updates the menu items.
+ */
+function updateTroubleshootMenuItem() {
+  if (Services.appinfo.inSafeMode) {
+    let safeMode = document.getElementById("helpTroubleshootMode");
+    document.l10n.setAttributes(safeMode, "menu-help-exit-troubleshoot-mode");
+
+    let appSafeMode = document.getElementById("appmenu_troubleshootMode");
+    if (appSafeMode) {
+      document.l10n.setAttributes(
+        appSafeMode,
+        "appmenu-help-exit-troubleshoot-mode"
+      );
+    }
+  }
+}
+
 function nsFlavorDataProvider() {}
 
 nsFlavorDataProvider.prototype = {
@@ -974,8 +1037,3 @@ nsFlavorDataProvider.prototype = {
     }
   },
 };
-
-function UpdateCharsetMenu(aCharset, aNode) {
-  var bundle = document.getElementById("charsetBundle");
-  CharsetMenu.update(aNode, bundle.getString(aCharset.toLowerCase()));
-}
