@@ -17,19 +17,23 @@
 // From netError.js
 /* globals retryThis */
 
-var { MailServices } = ChromeUtils.import(
-  "resource:///modules/MailServices.jsm"
-);
-var { appIdleManager } = ChromeUtils.import(
-  "resource:///modules/AppIdleManager.jsm"
-);
 var { Services } = ChromeUtils.import("resource://gre/modules/Services.jsm");
-var { MailUtils } = ChromeUtils.import("resource:///modules/MailUtils.jsm");
 var { XPCOMUtils } = ChromeUtils.import(
   "resource://gre/modules/XPCOMUtils.jsm"
 );
-var { Log4Moz } = ChromeUtils.import("resource:///modules/gloda/Log4moz.jsm");
-var { Gloda } = ChromeUtils.import("resource:///modules/gloda/GlodaPublic.jsm");
+XPCOMUtils.defineLazyModuleGetters(this, {
+  appIdleManager: "resource:///modules/AppIdleManager.jsm",
+  Gloda: "resource:///modules/gloda/GlodaPublic.jsm",
+  MailE10SUtils: "resource:///modules/MailE10SUtils.jsm",
+  MailUtils: "resource:///modules/MailUtils.jsm",
+  UIDensity: "resource:///modules/UIDensity.jsm",
+});
+
+XPCOMUtils.defineLazyScriptGetter(
+  this,
+  "PrintUtils",
+  "chrome://messenger/content/printUtils.js"
+);
 
 // This file stores variables common to mail windows
 var messenger;
@@ -39,12 +43,8 @@ var msgWindow;
 var accountManager;
 
 var gContextMenu;
-var gMailWindowLog = Log4Moz.getConfiguredLogger(
-  "mailWindow",
-  Log4Moz.Level.Debug,
-  Log4Moz.Level.Debug,
-  Log4Moz.Level.Debug
-);
+
+UIDensity.registerWindow(window);
 
 /**
  * Called by messageWindow.xhtml:onunload,  the 'single message display window'.
@@ -85,9 +85,22 @@ function onCopyOrDragStart(e) {
   if (!browser) {
     return;
   }
+
+  // We're only interested if this is in the message content.
   let sourceDoc = browser.contentDocument;
   if (e.target.ownerDocument != sourceDoc) {
-    return; // We're only interested if this is in the message content.
+    return;
+  }
+  let sourceURL = sourceDoc.URL;
+  let protocol = sourceURL.substr(0, sourceURL.indexOf(":")).toLowerCase();
+  if (
+    !(
+      Services.io.getProtocolHandler(protocol) instanceof
+      Ci.nsIMsgMessageFetchPartService
+    )
+  ) {
+    // Can't fetch parts, not a message protocol, don't process.
+    return;
   }
 
   let imgMap = new Map(); // Mapping img.src -> dataURL.
@@ -212,6 +225,67 @@ function CreateMailWindowGlobals() {
   accountManager = MailServices.accounts;
 }
 
+function toggleCaretBrowsing() {
+  const enabledPref = "accessibility.browsewithcaret_shortcut.enabled";
+  const warnPref = "accessibility.warn_on_browsewithcaret";
+  const caretPref = "accessibility.browsewithcaret";
+
+  if (!Services.prefs.getBoolPref(enabledPref)) {
+    return;
+  }
+
+  let useCaret = Services.prefs.getBoolPref(caretPref, false);
+  let warn = Services.prefs.getBoolPref(warnPref, true);
+  if (!warn || useCaret) {
+    // Toggle immediately.
+    try {
+      Services.prefs.setBoolPref(caretPref, !useCaret);
+    } catch (ex) {}
+    return;
+  }
+
+  // Async prompt.
+  document.l10n
+    .formatValues([
+      { id: "caret-browsing-prompt-title" },
+      { id: "caret-browsing-prompt-text" },
+      { id: "caret-browsing-prompt-check-text" },
+    ])
+    .then(([title, promptText, checkText]) => {
+      let checkValue = { value: false };
+
+      useCaret =
+        0 ===
+        Services.prompt.confirmEx(
+          window,
+          title,
+          promptText,
+          Services.prompt.STD_YES_NO_BUTTONS |
+            Services.prompt.BUTTON_POS_1_DEFAULT,
+          null,
+          null,
+          null,
+          checkText,
+          checkValue
+        );
+
+      if (checkValue.value) {
+        if (useCaret) {
+          try {
+            Services.prefs.setBoolPref(warnPref, false);
+          } catch (ex) {}
+        } else {
+          try {
+            Services.prefs.setBoolPref(enabledPref, false);
+          } catch (ex) {}
+        }
+      }
+      try {
+        Services.prefs.setBoolPref(caretPref, useCaret);
+      } catch (ex) {}
+    });
+}
+
 function InitMsgWindow() {
   msgWindow.windowCommands = new nsMsgWindowCommands();
   // set the domWindow before setting the status feedback and header sink objects
@@ -219,9 +293,17 @@ function InitMsgWindow() {
   msgWindow.statusFeedback = statusFeedback;
   msgWindow.msgHeaderSink = messageHeaderSink;
   MailServices.mailSession.AddMsgWindow(msgWindow);
-  let messagepane = document.getElementById("messagepane");
-  messagepane.docShell.allowAuth = false;
-  messagepane.docShell.allowDNSPrefetch = false;
+  let messagepane = getMessagePaneBrowser();
+  if (messagepane.docShell) {
+    messagepane.docShell.allowAuth = false;
+    messagepane.docShell.allowDNSPrefetch = false;
+  }
+  let multimessagepane = document.getElementById("multimessage");
+  // The multimessage pane is not available in the standalone message window.
+  if (multimessagepane?.docShell) {
+    multimessagepane.docShell.allowAuth = false;
+    multimessagepane.docShell.allowDNSPrefetch = false;
+  }
   msgWindow.rootDocShell.allowAuth = true;
   msgWindow.rootDocShell.appType = Ci.nsIDocShell.APP_TYPE_MAIL;
   // Ensure we don't load xul error pages into the main window
@@ -243,11 +325,28 @@ function InitMsgWindow() {
     });
   });
 
-  // Run menubar initialization first, to avoid TabsInTitlebar code picking
-  // up mutations from it and causing a reflow.
-  if (AppConstants.platform != "macosx") {
-    AutoHideMenubar.init();
-  }
+  let keypressListener = {
+    handleEvent: event => {
+      if (event.defaultPrevented) {
+        return;
+      }
+
+      switch (event.code) {
+        case "F7":
+          // shift + F7 is the default DevTools shortcut for the Style Editor.
+          if (!event.shiftKey) {
+            toggleCaretBrowsing();
+          }
+          break;
+      }
+    },
+  };
+  Services.els.addSystemEventListener(
+    document,
+    "keypress",
+    keypressListener,
+    false
+  );
 }
 
 // We're going to implement our status feedback for the mail window in JS now.
@@ -597,6 +696,7 @@ function loadStartPage(aForce) {
     return;
   }
 
+  let messagePane = getMessagePaneBrowser();
   gMessageDisplay.singleMessageDisplay = true;
   gMessageNotificationBar.clearMsgNotifications();
   let startpage = Services.urlFormatter.formatURLPref(
@@ -604,13 +704,13 @@ function loadStartPage(aForce) {
   );
   if (startpage) {
     try {
-      let uri = Services.uriFixup.createFixupURI(startpage, 0);
-      GetMessagePaneFrame().location.href = uri.spec;
+      let { preferredURI } = Services.uriFixup.getFixupURIInfo(startpage, 0);
+      MailE10SUtils.loadURI(messagePane, preferredURI.spec);
     } catch (e) {
       Cu.reportError(e);
     }
   } else {
-    GetMessagePaneFrame().location.href = "about:blank";
+    ClearMessagePane();
   }
 }
 
@@ -695,15 +795,62 @@ function nsBrowserAccess() {}
 nsBrowserAccess.prototype = {
   QueryInterface: ChromeUtils.generateQI(["nsIBrowserDOMWindow"]),
 
-  // The following function may be called during account creation, it is called by
-  // the test browser_newmailaccount.js::test_window_open_link_opening_behaviour.
+  _openURIInNewTab(
+    aURI,
+    aReferrerInfo,
+    aIsExternal,
+    aOpenWindowInfo = null,
+    aTriggeringPrincipal = null,
+    aCsp = null,
+    aSkipLoad = false,
+    aMessageManagerGroup = null
+  ) {
+    let win, needToFocusWin;
+
+    // Try the current window. If we're in a popup, fall back on the most
+    // recent browser window.
+    if (!window.document.documentElement.getAttribute("chromehidden")) {
+      win = window;
+    } else {
+      win = getMostRecentMailWindow();
+      needToFocusWin = true;
+    }
+
+    if (!win) {
+      // we couldn't find a suitable window, a new one needs to be opened.
+      return null;
+    }
+
+    let loadInBackground = Services.prefs.getBoolPref(
+      "browser.tabs.loadDivertedInBackground"
+    );
+
+    let tabmail = win.document.getElementById("tabmail");
+    let newTab = tabmail.openTab("contentTab", {
+      background: loadInBackground,
+      csp: aCsp,
+      linkHandler: aMessageManagerGroup,
+      openWindowInfo: aOpenWindowInfo,
+      referrerInfo: aReferrerInfo,
+      skipLoad: aSkipLoad,
+      triggeringPrincipal: aTriggeringPrincipal,
+      url: aURI ? aURI.spec : "about:blank",
+    });
+
+    if (needToFocusWin || (!loadInBackground && aIsExternal)) {
+      win.focus();
+    }
+
+    return newTab.browser;
+  },
+
   createContentWindow(
     aURI,
     aOpenWindowInfo,
     aWhere,
     aFlags,
-    aTriggeringPrincipal = null,
-    aCsp = null
+    aTriggeringPrincipal,
+    aCsp
   ) {
     return this.getContentWindowOrOpenURI(
       null,
@@ -716,21 +863,30 @@ nsBrowserAccess.prototype = {
     );
   },
 
-  openURI(
-    aURI,
-    aOpener,
-    aWhere,
-    aFlags,
-    aTriggeringPrincipal = null,
-    aCsp = null
-  ) {
+  createContentWindowInFrame(aURI, aParams, aWhere, aFlags, aName) {
+    // Passing a null-URI to only create the content window,
+    // and pass true for aSkipLoad to prevent loading of
+    // about:blank
+    return this.getContentWindowOrOpenURIInFrame(
+      aURI,
+      aParams,
+      aWhere,
+      aFlags,
+      aName,
+      false
+    );
+  },
+
+  openURI(aURI, aOpenWindowInfo, aWhere, aFlags, aTriggeringPrincipal, aCsp) {
     if (!aURI) {
-      Cu.reportError("openURI should only be called with a valid URI");
-      throw Components.Exception("", Cr.NS_ERROR_FAILURE);
+      throw Components.Exception(
+        "openURI should only be called with a valid URI",
+        Cr.NS_ERROR_FAILURE
+      );
     }
     return this.getContentWindowOrOpenURI(
       aURI,
-      aOpener,
+      aOpenWindowInfo,
       aWhere,
       aFlags,
       aTriggeringPrincipal,
@@ -739,6 +895,17 @@ nsBrowserAccess.prototype = {
     );
   },
 
+  openURIInFrame(aURI, aParams, aWhere, aFlags, aName) {
+    return this.getContentWindowOrOpenURIInFrame(
+      aURI,
+      aParams,
+      aWhere,
+      aFlags,
+      aName,
+      false
+    );
+  },
+
   getContentWindowOrOpenURI(
     aURI,
     aOpenWindowInfo,
@@ -748,8 +915,23 @@ nsBrowserAccess.prototype = {
     aCsp,
     aSkipLoad
   ) {
-    const nsIBrowserDOMWindow = Ci.nsIBrowserDOMWindow;
-    let isExternal = !!(aFlags & nsIBrowserDOMWindow.OPEN_EXTERNAL);
+    if (aWhere == Ci.nsIBrowserDOMWindow.OPEN_PRINT_BROWSER) {
+      let browser = PrintUtils.handleStaticCloneCreatedForPrint(
+        aOpenWindowInfo
+      );
+      return browser ? browser.browsingContext : null;
+    }
+
+    let isExternal = !!(aFlags & Ci.nsIBrowserDOMWindow.OPEN_EXTERNAL);
+
+    if (aOpenWindowInfo && isExternal) {
+      throw Components.Exception(
+        "nsBrowserAccess.openURI did not expect aOpenWindowInfo to be " +
+          "passed if the context is OPEN_EXTERNAL.",
+        Cr.NS_ERROR_FAILURE
+      );
+    }
+
     if (isExternal && aURI && aURI.schemeIs("chrome")) {
       Services.console.logStringMessage(
         "use -chrome command-line option to load external chrome urls\n"
@@ -757,100 +939,107 @@ nsBrowserAccess.prototype = {
       return null;
     }
 
-    let loadflags = isExternal
-      ? Ci.nsIWebNavigation.LOAD_FLAGS_FROM_EXTERNAL
-      : Ci.nsIWebNavigation.LOAD_FLAGS_NONE;
+    const ReferrerInfo = Components.Constructor(
+      "@mozilla.org/referrer-info;1",
+      "nsIReferrerInfo",
+      "init"
+    );
 
-    if (aWhere != nsIBrowserDOMWindow.OPEN_NEWTAB) {
+    let referrerInfo;
+    if (aFlags & Ci.nsIBrowserDOMWindow.OPEN_NO_REFERRER) {
+      referrerInfo = new ReferrerInfo(Ci.nsIReferrerInfo.EMPTY, false, null);
+    } else if (
+      aOpenWindowInfo &&
+      aOpenWindowInfo.parent &&
+      aOpenWindowInfo.parent.window
+    ) {
+      referrerInfo = new ReferrerInfo(
+        aOpenWindowInfo.parent.window.document.referrerInfo.referrerPolicy,
+        true,
+        makeURI(aOpenWindowInfo.parent.window.location.href)
+      );
+    } else {
+      referrerInfo = new ReferrerInfo(Ci.nsIReferrerInfo.EMPTY, true, null);
+    }
+
+    if (aWhere != Ci.nsIBrowserDOMWindow.OPEN_NEWTAB) {
       Services.console.logStringMessage(
         "Opening a URI in something other than a new tab is not supported, opening in new tab instead"
       );
     }
 
-    let win, needToFocusWin;
-
-    // Try the current window. If we're in a popup, fall back on the most
-    // recent browser window.
-    if (!window.document.documentElement.getAttribute("chromehidden")) {
-      win = window;
-    } else {
-      win = getMostRecentMailWindow();
-      needToFocusWin = true;
-    }
-
-    if (!win) {
-      throw new Error("Couldn't get a suitable window for openURI");
-    }
-
-    let loadInBackground = Services.prefs.getBoolPref(
-      "browser.tabs.loadDivertedInBackground"
+    let browser = this._openURIInNewTab(
+      aURI,
+      referrerInfo,
+      isExternal,
+      aOpenWindowInfo,
+      aTriggeringPrincipal,
+      aCsp,
+      aSkipLoad,
+      aOpenWindowInfo.openerBrowser?.getAttribute("messagemanagergroup")
     );
 
-    let tabmail = win.document.getElementById("tabmail");
-    let clickHandler = null;
-    let browser = tabmail.getBrowserForDocument(window.content);
-    if (browser) {
-      clickHandler = browser.clickHandler;
-    }
-
-    let newTab = tabmail.openTab("contentTab", {
-      contentPage: "about:blank",
-      background: loadInBackground,
-      openWindowInfo: aOpenWindowInfo,
-      clickHandler,
-      skipLoad: aSkipLoad,
-    });
-
-    let docShell = newTab.browser.docShell;
-    let newWindow = docShell.domWindow;
-    let browsingContext = docShell.browsingContext;
-    try {
-      if (aURI) {
-        let referrer = null;
-        if (aOpenWindowInfo) {
-          let location = aOpenWindowInfo.parent.window.location;
-          referrer = Services.io.newURI(location);
-        }
-        newWindow
-          .getInterface(Ci.nsIWebNavigation)
-          .loadURI(
-            aURI.spec,
-            loadflags,
-            referrer,
-            null,
-            null,
-            aTriggeringPrincipal
-          );
-      }
-      if (needToFocusWin || (!loadInBackground && isExternal)) {
-        newWindow.focus();
-      }
-    } catch (e) {}
-    return browsingContext;
+    return browser ? browser.browsingContext : null;
   },
 
-  isTabContentWindow(aWindow) {
-    return false;
+  getContentWindowOrOpenURIInFrame(
+    aURI,
+    aParams,
+    aWhere,
+    aFlags,
+    aName,
+    aSkipLoad
+  ) {
+    if (aWhere == Ci.nsIBrowserDOMWindow.OPEN_PRINT_BROWSER) {
+      return PrintUtils.handleStaticCloneCreatedForPrint(
+        aParams.openWindowInfo
+      );
+    }
+
+    if (aWhere != Ci.nsIBrowserDOMWindow.OPEN_NEWTAB) {
+      Services.console.logStringMessage(
+        "Error: openURIInFrame can only open in new tabs or print"
+      );
+      return null;
+    }
+
+    let isExternal = !!(aFlags & Ci.nsIBrowserDOMWindow.OPEN_EXTERNAL);
+
+    return this._openURIInNewTab(
+      aURI,
+      aParams.referrerInfo,
+      isExternal,
+      aParams.openWindowInfo,
+      aParams.triggeringPrincipal,
+      aParams.csp,
+      aSkipLoad,
+      aParams.openerBrowser?.getAttribute("messagemanagergroup")
+    );
+  },
+
+  canClose() {
+    return true;
+  },
+
+  get tabCount() {
+    return document.getElementById("tabmail").tabInfo.length;
   },
 };
 
-function MailSetCharacterSet(aEvent) {
-  if (aEvent.target.hasAttribute("charset")) {
-    msgWindow.mailCharacterSet = aEvent.target.getAttribute("charset");
-    msgWindow.charsetOverride = true;
-    gMessageDisplay.keyForCharsetOverride =
-      "messageKey" in gMessageDisplay.displayedMessage
-        ? gMessageDisplay.displayedMessage.messageKey
-        : null;
-  }
-  messenger.setDocumentCharset(msgWindow.mailCharacterSet);
+function MailSetCharacterSet() {
+  msgWindow.charsetOverride = true;
+  gMessageDisplay.keyForCharsetOverride =
+    "messageKey" in gMessageDisplay.displayedMessage
+      ? gMessageDisplay.displayedMessage.messageKey
+      : null;
+  messenger.forceDetectDocumentCharset();
 }
 
 /**
  * Called from the extensions manager to open an add-on options XUL document.
  * Only the "open in tab" option is supported, so that's what we'll do here.
  */
-function switchToTabHavingURI(aURI, aOpenNew, aOpenParams) {
+function switchToTabHavingURI(aURI, aOpenNew, aOpenParams = {}) {
   let tabmail = document.getElementById("tabmail");
   let matchingIndex = -1;
   if (tabmail) {
@@ -890,12 +1079,7 @@ function switchToTabHavingURI(aURI, aOpenNew, aOpenParams) {
   }
 
   if (aOpenNew) {
-    // Open a new tab, keeping links from the new tab in Thunderbird if the regexp is set.
-    if (aOpenParams && "handlerRegExp" in aOpenParams) {
-      openContentTab(aURI, "tab", aOpenParams.handlerRegExp);
-    } else {
-      openContentTab(aURI, "tab");
-    }
+    tabmail.openTab("contentTab", { ...aOpenParams, url: aURI });
   }
 
   return false;
