@@ -18,11 +18,16 @@ var { DisplayNameUtils } = ChromeUtils.import(
 var { MailServices } = ChromeUtils.import(
   "resource:///modules/MailServices.jsm"
 );
-var { TagUtils } = ChromeUtils.import("resource:///modules/TagUtils.jsm");
 
 var { PluralStringFormatter, makeFriendlyDateAgo } = ChromeUtils.import(
   "resource:///modules/TemplateUtils.jsm"
 );
+var { Services } = ChromeUtils.import("resource://gre/modules/Services.jsm");
+
+XPCOMUtils.defineLazyModuleGetters(this, {
+  MessageArchiver: "resource:///modules/MessageArchiver.jsm",
+  TagUtils: "resource:///modules/TagUtils.jsm",
+});
 
 var gMessenger = Cc["@mozilla.org/messenger;1"].createInstance(Ci.nsIMessenger);
 
@@ -93,10 +98,6 @@ LimitIterator.prototype[ITERATOR_SYMBOL] = function*() {
  */
 function MultiMessageSummary() {
   this._summarizers = {};
-
-  // Hook into the resize event on the header to make the #content node shift
-  // down as it reflows.
-  window.addEventListener("resize", this._adjustHeadingSize.bind(this));
 }
 
 MultiMessageSummary.prototype = {
@@ -131,6 +132,7 @@ MultiMessageSummary.prototype = {
    * Clear all the content from the summary.
    */
   clear() {
+    this._selectCallback = null;
     this._listener = null;
     this._glodaQuery = null;
     this._msgNodes = {};
@@ -142,8 +144,7 @@ MultiMessageSummary.prototype = {
     }
 
     // Clear the notice.
-    let notice = document.getElementById("notice");
-    notice.classList.add("hidden");
+    document.getElementById("notice").textContent = "";
   },
 
   /**
@@ -151,12 +152,16 @@ MultiMessageSummary.prototype = {
    *
    * @param aType       The type of summary to perform (e.g. 'multimessage').
    * @param aMessages   The messages to summarize.
+   * @param aDBView     The current DB view.
+   * @param aSelectCallback  Called with an array of nsIMsgHdrs when one of
+   *                    a summarized message is clicked on.
    * @param [aListener] A listener to be notified when the summary starts and
    *                    finishes.
    */
-  summarize(aType, aMessages, aListener) {
+  summarize(aType, aMessages, aDBView, aSelectCallback, aListener) {
     this.clear();
 
+    this._selectCallback = aSelectCallback;
     this._listener = aListener;
     if (this._listener) {
       this._listener.onLoadStarted();
@@ -164,8 +169,20 @@ MultiMessageSummary.prototype = {
 
     // Enable/disable the archive button as appropriate.
     let archiveBtn = document.getElementById("hdrArchiveButton");
-    archiveBtn.collapsed = !window.top.gFolderDisplay
-      .canArchiveSelectedMessages;
+    archiveBtn.collapsed = !MessageArchiver.canArchive(aMessages);
+
+    // Set archive and delete button listeners.
+    let msgWindow = window.browsingContext.topChromeWindow;
+    archiveBtn.onclick = event => {
+      if (event.button == 0) {
+        msgWindow.MsgArchiveSelectedMessages(null);
+      }
+    };
+    document.getElementById("hdrTrashButton").onclick = event => {
+      if (event.button == 0) {
+        msgWindow.goDoCommand("cmd_delete");
+      }
+    };
 
     let summarizer = this._summarizers[aType];
     if (!summarizer) {
@@ -173,7 +190,7 @@ MultiMessageSummary.prototype = {
     }
 
     let messages = new LimitIterator(aMessages, this.kMaxMessages);
-    let summarizedMessages = summarizer.summarize(messages);
+    let summarizedMessages = summarizer.summarize(messages, aDBView);
 
     // Stash somewhere so it doesn't get GC'ed.
     this._glodaQuery = Gloda.getMessageCollectionForHeaders(
@@ -194,8 +211,6 @@ MultiMessageSummary.prototype = {
     let subtitleNode = document.getElementById("summary_subtitle");
     titleNode.textContent = title || "";
     subtitleNode.textContent = subtitle || "";
-
-    this._adjustHeadingSize();
   },
 
   /**
@@ -270,9 +285,7 @@ MultiMessageSummary.prototype = {
       subjectNode.classList.add("subject", "primary_header", "link");
       subjectNode.textContent =
         message.mime2DecodedSubject || formatString("noSubject");
-      subjectNode.addEventListener("click", function() {
-        window.top.gFolderDisplay.selectMessages(thread);
-      });
+      subjectNode.addEventListener("click", () => this._selectCallback(thread));
       itemHeaderNode.appendChild(subjectNode);
 
       if (thread && thread.length > 1) {
@@ -306,9 +319,8 @@ MultiMessageSummary.prototype = {
       itemHeaderNode.appendChild(dateNode);
 
       authorNode.classList.add("primary_header", "link");
-      authorNode.addEventListener("click", function() {
-        window.top.gFolderDisplay.selectMessage(message);
-        window.top.document.getElementById("messagepane").focus();
+      authorNode.addEventListener("click", () => {
+        this._selectCallback([message]);
       });
       itemHeaderNode.appendChild(authorNode);
     }
@@ -364,7 +376,6 @@ MultiMessageSummary.prototype = {
   showNotice(aNoticeText) {
     let notice = document.getElementById("notice");
     notice.textContent = aNoticeText;
-    notice.classList.remove("hidden");
   },
 
   /**
@@ -399,17 +410,18 @@ MultiMessageSummary.prototype = {
 
     for (let tag of sortedTags) {
       let tagNode = document.createElement("span");
-      let color = MailServices.tags.getColorForKey(tag.key);
-      let textColor = "black";
-      if (!TagUtils.isColorContrastEnough(color)) {
-        textColor = "white";
-      }
 
       tagNode.className = "tag";
-      tagNode.setAttribute(
-        "style",
-        "color: " + textColor + "; background-color: " + color + ";"
-      );
+      let color = MailServices.tags.getColorForKey(tag.key);
+      if (color) {
+        let textColor = !TagUtils.isColorContrastEnough(color)
+          ? "white"
+          : "black";
+        tagNode.setAttribute(
+          "style",
+          "color: " + textColor + "; background-color: " + color + ";"
+        );
+      }
       tagNode.dataset.tag = tag.tag;
       tagNode.textContent = tag.tag;
       aTagsNode.appendChild(tagNode);
@@ -437,22 +449,6 @@ MultiMessageSummary.prototype = {
     ]);
   },
 
-  /**
-   * Adjust the position of the top of the main content so that it fits below
-   * the heading.
-   */
-  _adjustHeadingSize() {
-    let content = document.getElementById("content");
-    let heading = document.getElementById("heading");
-    let buttonbox = document.getElementById("header-view-toolbox");
-
-    content.style.top =
-      Math.max(
-        buttonbox.getBoundingClientRect().height,
-        heading.getBoundingClientRect().height
-      ) + "px";
-  },
-
   // These are listeners for the gloda collections.
   onItemsAdded(aItems) {},
   onItemsModified(aItems) {
@@ -563,7 +559,7 @@ ThreadSummarizer.prototype = {
    * @param aMessages A LimitIterator of the messages to summarize.
    * @return An array of the messages actually summarized.
    */
-  summarize(aMessages) {
+  summarize(aMessages, aDBView) {
     let messageList = document.getElementById("message_list");
 
     // Remove all ignored messages from summarization.
@@ -665,10 +661,10 @@ MultipleSelectionSummarizer.prototype = {
    *
    * @param aMessages The messages to summarize.
    */
-  summarize(aMessages) {
+  summarize(aMessages, aDBView) {
     let messageList = document.getElementById("message_list");
 
-    let threads = this._buildThreads(aMessages);
+    let threads = this._buildThreads(aMessages, aDBView);
     let threadsCount = threads.length;
 
     // Set the heading based on the number of messages & threads.
@@ -726,14 +722,12 @@ MultipleSelectionSummarizer.prototype = {
    * @param aMessages The messages to group.
    * @return An array of arrays of messages, grouped by thread.
    */
-  _buildThreads(aMessages) {
+  _buildThreads(aMessages, aDBView) {
     // First, we group the messages in threads and count the threads.
     let threads = [];
     let threadMap = {};
     for (let msgHdr of aMessages) {
-      let viewThreadId = window.top.gFolderDisplay.view.dbView.getThreadContainingMsgHdr(
-        msgHdr
-      ).threadKey;
+      let viewThreadId = aDBView.getThreadContainingMsgHdr(msgHdr).threadKey;
       if (!(viewThreadId in threadMap)) {
         threadMap[viewThreadId] = threads.length;
         threads.push([msgHdr]);
