@@ -8,6 +8,8 @@ add_task(async function test_unsigned() {
     set: [
       ["extensions.webapi.testing", true],
       ["extensions.install.requireBuiltInCerts", false],
+      // Relax the user input requirements while running this test.
+      ["xpinstall.userActivation.required", false],
     ],
   });
 
@@ -16,7 +18,7 @@ add_task(async function test_unsigned() {
   registerCleanupFunction(() => PermissionTestUtils.remove(testURI, "install"));
 
   let tab = openContentTab("about:blank");
-  await BrowserTestUtils.loadURI(
+  BrowserTestUtils.loadURI(
     tab.linkedBrowser,
     `${BASE}/file_install_extensions.html`
   );
