@@ -24,9 +24,8 @@ ChromeUtils.defineModuleGetter(
   "resource://gre/modules/Extension.jsm"
 );
 
-ChromeUtils.import(
-  "resource://testing-common/CustomizableUITestUtils.jsm",
-  this
+var { CustomizableUITestUtils } = ChromeUtils.import(
+  "resource://testing-common/CustomizableUITestUtils.jsm"
 );
 let gCUITestUtils = new CustomizableUITestUtils(window);
 
@@ -153,14 +152,15 @@ async function waitForUpdate(addon) {
   return newAddon;
 }
 
+function waitAboutAddonsViewLoaded(doc) {
+  return BrowserTestUtils.waitForEvent(doc, "view-loaded");
+}
+
 /**
  * Trigger an action from the page options menu.
  */
 function triggerPageOptionsAction(win, action) {
-  win
-    .getHtmlBrowser()
-    .contentDocument.querySelector(`#page-options [action="${action}"]`)
-    .click();
+  win.document.querySelector(`#page-options [action="${action}"]`).click();
 }
 
 function isDefaultIcon(icon) {
@@ -189,6 +189,7 @@ function isDefaultIcon(icon) {
  *        The message to be emitted as part of the actual test.
  */
 function checkPermissionString(string, key, param, msg) {
+  console.log(key);
   let localizedString = param
     ? gBrowserBundle.formatStringFromName(key, [param])
     : gBrowserBundle.GetStringFromName(key);
@@ -220,11 +221,21 @@ function checkPermissionString(string, key, param, msg) {
  *        in this array is itself a 2-element array with the string key
  *        for the item (e.g., "webextPerms.description.foo") and an
  *        optional formatting parameter.
+ * @param {boolean} sideloaded
+ *        Whether the notification is for a sideloaded extenion.
+ * @param {boolean} [warning]
+ *        Whether the experiments warning should be visible.
  */
-function checkNotification(panel, checkIcon, permissions, warning = false) {
+function checkNotification(
+  panel,
+  checkIcon,
+  permissions,
+  sideloaded,
+  warning = false
+) {
   let icon = panel.getAttribute("icon");
   let ul = document.getElementById("addon-webext-perm-list");
-  let header = document.getElementById("addon-webext-perm-intro");
+  let singleDataEl = document.getElementById("addon-webext-perm-single-entry");
   let experimentWarning = document.getElementById(
     "addon-webext-experiment-warning"
   );
@@ -241,49 +252,59 @@ function checkNotification(panel, checkIcon, permissions, warning = false) {
     is(icon, checkIcon, "Notification icon is correct");
   }
 
-  is(
-    ul.childElementCount,
-    permissions.length,
-    `Permissions list has ${permissions.length} entries`
-  );
-  if (!permissions.length) {
-    is(header.getAttribute("hidden"), "true", "Permissions header is hidden");
-    is(
-      learnMoreLink.getAttribute("hidden"),
-      "true",
-      "Permissions learn more is hidden"
-    );
-  } else {
-    is(header.getAttribute("hidden"), "", "Permissions header is visible");
-    is(
-      learnMoreLink.getAttribute("hidden"),
-      "",
-      "Permissions learn more is visible"
-    );
+  let description = panel.querySelector(".popup-notification-description")
+    .textContent;
+  let expectedDescription = "webextPerms.header";
+  if (permissions.length) {
+    expectedDescription += "WithPerms";
   }
+  if (sideloaded) {
+    expectedDescription = "webextPerms.sideloadHeader";
+  }
+  checkPermissionString(
+    description,
+    expectedDescription,
+    undefined,
+    `Description is the expected one`
+  );
+  is(
+    learnMoreLink.hidden,
+    !permissions.length,
+    "Permissions learn more is hidden if there are no permissions"
+  );
 
-  for (let i in permissions) {
-    let [key, param] = permissions[i];
-    checkPermissionString(
-      ul.children[i].textContent,
-      key,
-      param,
-      `Permission number ${i + 1} is correct`
+  if (!permissions.length) {
+    ok(ul.hidden, "Permissions list is hidden");
+    ok(singleDataEl.hidden, "Single permission data entry is hidden");
+    ok(
+      !(ul.childElementCount || singleDataEl.textContent),
+      "Permission list and single permission element have no entries"
     );
+  } else if (permissions.length === 1) {
+    ok(ul.hidden, "Permissions list is hidden");
+    ok(!ul.childElementCount, "Permission list has no entries");
+    ok(singleDataEl.textContent, "Single permission data label has been set");
+  } else {
+    ok(singleDataEl.hidden, "Single permission data entry is hidden");
+    ok(
+      !singleDataEl.textContent,
+      "Single permission data label has not been set"
+    );
+    for (let i in permissions) {
+      let [key, param] = permissions[i];
+      checkPermissionString(
+        ul.children[i].textContent,
+        key,
+        param,
+        `Permission number ${i + 1} is correct`
+      );
+    }
   }
 
   if (warning) {
-    is(
-      experimentWarning.getAttribute("hidden"),
-      "",
-      "Experiments warning is visible"
-    );
+    is(experimentWarning.hidden, false, "Experiments warning is visible");
   } else {
-    is(
-      experimentWarning.getAttribute("hidden"),
-      "true",
-      "Experiments warning is hidden"
-    );
+    is(experimentWarning.hidden, true, "Experiments warning is hidden");
   }
 }
 
@@ -323,6 +344,9 @@ async function testInstallMethod(installFn, telemetryBase) {
 
   async function runOnce(filename, cancel) {
     let tab = openContentTab("about:blank");
+    if (tab.browser.webProgress.isLoadingDocument) {
+      await BrowserTestUtils.browserLoaded(tab.browser);
+    }
 
     let installPromise = new Promise(resolve => {
       let listener = {
@@ -367,7 +391,7 @@ async function testInstallMethod(installFn, telemetryBase) {
         ["webextPerms.description.nativeMessaging"],
         // The below permissions are deliberately in this order as permissions
         // are sorted alphabetically by the permission string to match AMO.
-        ["webextPerms.description.accountsRead"],
+        ["webextPerms.description.accountsRead2"],
         ["webextPerms.description.tabs"],
       ]);
     } else if (filename == NO_PERMS_XPI) {
@@ -402,7 +426,8 @@ async function testInstallMethod(installFn, telemetryBase) {
       await addon.uninstall();
     }
 
-    document.getElementById("tabmail").closeTab(tab);
+    let tabmail = document.getElementById("tabmail");
+    tabmail.closeOtherTabs(tabmail.tabInfo[0]);
   }
 
   // A few different tests for each installation method:
@@ -477,16 +502,13 @@ async function interactiveUpdateTest(autoUpdate, checkFn) {
     if (manualUpdatePromise) {
       await manualUpdatePromise;
 
-      let doc = win.getHtmlBrowser().contentDocument;
+      let doc = win.document;
       if (win.gViewController.currentViewId !== "addons://updates/available") {
         let showUpdatesBtn = doc.querySelector("addon-updates-message").button;
         await TestUtils.waitForCondition(() => {
           return !showUpdatesBtn.hidden;
         }, "Wait for show updates button");
-        let viewChanged = BrowserTestUtils.waitForEvent(
-          win.document,
-          "ViewChanged"
-        );
+        let viewChanged = waitAboutAddonsViewLoaded(doc);
         showUpdatesBtn.click();
         await viewChanged;
       }
@@ -510,7 +532,7 @@ async function interactiveUpdateTest(autoUpdate, checkFn) {
 
   let win = await openAddonsMgr("addons://list/extension");
 
-  await BrowserTestUtils.waitForEvent(win.document, "ViewChanged");
+  await waitAboutAddonsViewLoaded(win.document);
 
   // Trigger an update check
   let popupPromise = promisePopupNotificationShown("addon-webext-permissions");
