@@ -5,7 +5,8 @@
 /* This file is a copy of mozilla/toolkit/content/aboutSupport.js with
    modifications for TB. */
 
-/* globals AboutSupportPlatform, populateAccountsSection, sendViaEmail */
+/* globals AboutSupportPlatform, populateAccountsSection, sendViaEmail
+    populateCalendarsSection, populateChatSection */
 
 "use strict";
 
@@ -23,6 +24,12 @@ var { AppConstants } = ChromeUtils.import(
   "resource://gre/modules/AppConstants.jsm"
 );
 
+ChromeUtils.defineModuleGetter(
+  this,
+  "DownloadUtils",
+  "resource://gre/modules/DownloadUtils.jsm"
+);
+
 ChromeUtils.defineModuleGetter(
   this,
   "PluralForm",
@@ -33,6 +40,11 @@ ChromeUtils.defineModuleGetter(
   "PlacesDBUtils",
   "resource://gre/modules/PlacesDBUtils.jsm"
 );
+ChromeUtils.defineModuleGetter(
+  this,
+  "ProcessType",
+  "resource://gre/modules/ProcessType.jsm"
+);
 
 // added for TB
 /* Node classes. All of these are mutually exclusive. */
@@ -59,6 +71,17 @@ window.addEventListener("load", function onload(event) {
     });
     populateActionBox();
     setupEventListeners();
+
+    let hasWinPackageId = false;
+    try {
+      hasWinPackageId = Services.sysinfo.getProperty("hasWinPackageId");
+    } catch (_ex) {
+      // The hasWinPackageId property doesn't exist; assume it would be false.
+    }
+    if (hasWinPackageId) {
+      $("update-dir-row").hidden = true;
+      $("update-history-row").hidden = true;
+    }
   } catch (e) {
     Cu.reportError(
       "stack of load error for about:support: " + e + ": " + e.stack
@@ -66,11 +89,25 @@ window.addEventListener("load", function onload(event) {
   }
   // added for TB
   populateAccountsSection();
+  populateCalendarsSection();
+  populateChatSection();
   document
     .getElementById("check-show-private-data")
     .addEventListener("change", () => onShowPrivateDataChange());
 });
 
+function prefsTable(data) {
+  return sortedArrayFromObject(data).map(function([name, value]) {
+    return $.new("tr", [
+      $.new("td", name, "pref-name"),
+      // Very long preference values can cause users problems when they
+      // copy and paste them into some text editors.  Long values generally
+      // aren't useful anyway, so truncate them to a reasonable length.
+      $.new("td", String(value).substr(0, 120), "pref-value"),
+    ]);
+  });
+}
+
 // Fluent uses lisp-case IDs so this converts
 // the SentenceCase info IDs to lisp-case.
 const FLUENT_IDENT_REGEX = /^[a-zA-Z][a-zA-Z0-9_-]*$/;
@@ -92,6 +129,14 @@ var snapshotFormatters = {
     $("application-box").textContent = data.name;
     $("useragent-box").textContent = data.userAgent;
     $("os-box").textContent = data.osVersion;
+    if (data.osTheme) {
+      $("os-theme-box").textContent = data.osTheme;
+    } else {
+      $("os-theme-row").hidden = true;
+    }
+    if (AppConstants.platform == "macosx") {
+      $("rosetta-box").textContent = data.rosetta;
+    }
     $("binary-box").textContent = Services.dirsvc.get(
       "XREExeF",
       Ci.nsIFile
@@ -131,20 +176,19 @@ var snapshotFormatters = {
       );
     } catch (e) {}
 
-    let statusTextId = "multi-process-status-unknown";
+    const STATUS_STRINGS = {
+      experimentControl: "fission-status-experiment-control",
+      experimentTreatment: "fission-status-experiment-treatment",
+      disabledByE10sEnv: "fission-status-disabled-by-e10s-env",
+      enabledByEnv: "fission-status-enabled-by-env",
+      enabledByDefault: "fission-status-enabled-by-default",
+      disabledByDefault: "fission-status-disabled-by-default",
+      enabledByUserPref: "fission-status-enabled-by-user-pref",
+      disabledByUserPref: "fission-status-disabled-by-user-pref",
+      disabledByE10sOther: "fission-status-disabled-by-e10s-other",
+    };
 
-    // Whitelist of known values with string descriptions:
-    switch (data.autoStartStatus) {
-      case 0:
-      case 1:
-      case 2:
-      case 4:
-      case 6:
-      case 7:
-      case 8:
-        statusTextId = "multi-process-status-" + data.autoStartStatus;
-        break;
-    }
+    let statusTextId = STATUS_STRINGS[data.fissionDecisionStatus];
 
     document.l10n.setAttributes(
       $("multiprocess-box-process-count"),
@@ -154,7 +198,15 @@ var snapshotFormatters = {
         totalWindows: data.numTotalWindows,
       }
     );
-    document.l10n.setAttributes($("multiprocess-box-status"), statusTextId);
+    document.l10n.setAttributes(
+      $("fission-box-process-count"),
+      "fission-windows",
+      {
+        fissionWindows: data.numFissionWindows,
+        totalWindows: data.numTotalWindows,
+      }
+    );
+    document.l10n.setAttributes($("fission-box-status"), statusTextId);
 
     if (Services.policies) {
       let policiesStrId = "";
@@ -209,6 +261,17 @@ var snapshotFormatters = {
 
     $("safemode-box").textContent = data.safeMode;
 
+    const formatHumanReadableBytes = (elem, bytes) => {
+      let size = DownloadUtils.convertByteUnits(bytes);
+      document.l10n.setAttributes(elem, "app-basics-data-size", {
+        value: size[0],
+        unit: size[1],
+      });
+    };
+
+    formatHumanReadableBytes($("memory-size-box"), data.memorySizeBytes);
+    formatHumanReadableBytes($("disk-available-box"), data.diskAvailableBytes);
+
     // added for TB
     // Add profile path as private info into the page.
     let currProfD = Services.dirsvc.get("ProfD", Ci.nsIFile);
@@ -230,13 +293,15 @@ var snapshotFormatters = {
     let fsType;
     try {
       fsType = AboutSupportPlatform.getFileSystemType(currProfD);
-      let bundle = Services.strings.createBundle(
-        "chrome://messenger/locale/aboutSupportMail.properties"
-      );
-      let fsText = bundle.GetStringFromName("fsType." + fsType);
-      let fsTextNode = document.createElement("span");
-      fsTextNode.textContent = fsText;
-      profElem.appendChild(fsTextNode);
+      if (fsType) {
+        let bundle = Services.strings.createBundle(
+          "chrome://messenger/locale/aboutSupportMail.properties"
+        );
+        let fsText = bundle.GetStringFromName("fsType." + fsType);
+        let fsTextNode = document.createElement("span");
+        fsTextNode.textContent = fsText;
+        profElem.appendChild(fsTextNode);
+      }
     } catch (x) {
       Cu.reportError(x);
     }
@@ -309,15 +374,16 @@ var snapshotFormatters = {
     );
   },
 
-  extensions(data) {
+  addons(data) {
     $.append(
-      $("extensions-tbody"),
-      data.map(function(extension) {
+      $("addons-tbody"),
+      data.map(function(addon) {
         return $.new("tr", [
-          $.new("td", extension.name),
-          $.new("td", extension.version),
-          $.new("td", extension.isActive),
-          $.new("td", extension.id),
+          $.new("td", addon.name),
+          $.new("td", addon.type),
+          $.new("td", addon.version),
+          $.new("td", addon.isActive),
+          $.new("td", addon.id),
         ]);
       })
     );
@@ -349,11 +415,8 @@ var snapshotFormatters = {
 
   async processes(data) {
     async function buildEntry(name, value) {
-      let entryName =
-        (await document.l10n.formatValue(
-          `process-type-${name.toLowerCase()}`
-        )) || name;
-
+      const fluentName = ProcessType.fluentNameFromProcessTypeString(name);
+      let entryName = (await document.l10n.formatValue(fluentName)) || name;
       $("processes-tbody").appendChild(
         $.new("tr", [$.new("td", entryName), $.new("td", value)])
       );
@@ -382,30 +445,43 @@ var snapshotFormatters = {
     }
   },
 
-  modifiedPreferences(data) {
+  environmentVariables(data) {
+    if (!data) {
+      return;
+    }
     $.append(
-      $("prefs-tbody"),
-      sortedArrayFromObject(data).map(function([name, value]) {
+      $("environment-variables-tbody"),
+      Object.entries(data).map(([name, value]) => {
         return $.new("tr", [
           $.new("td", name, "pref-name"),
-          // Very long preference values can cause users problems when they
-          // copy and paste them into some text editors.  Long values generally
-          // aren't useful anyway, so truncate them to a reasonable length.
-          $.new("td", String(value).substr(0, 120), "pref-value"),
+          $.new("td", value, "pref-value"),
         ]);
       })
     );
   },
 
+  modifiedPreferences(data) {
+    $.append($("prefs-tbody"), prefsTable(data));
+  },
+
   lockedPreferences(data) {
-    $.append(
-      $("locked-prefs-tbody"),
-      sortedArrayFromObject(data).map(function([name, value]) {
-        return $.new("tr", [
-          $.new("td", name, "pref-name"),
-          $.new("td", String(value).substr(0, 120), "pref-value"),
-        ]);
-      })
+    $.append($("locked-prefs-tbody"), prefsTable(data));
+  },
+
+  printingPreferences(data) {
+    if (AppConstants.platform == "android") {
+      return;
+    }
+    const tbody = $("support-printing-prefs-tbody");
+    $.append(tbody, prefsTable(data));
+    $("support-printing-clear-settings-button").addEventListener(
+      "click",
+      function() {
+        for (let name in data) {
+          Services.prefs.clearUserPref(name);
+        }
+        tbody.textContent = "";
+      }
     );
   },
 
@@ -612,9 +688,6 @@ var snapshotFormatters = {
     let compositor = "";
     if (data.windowLayerManagerRemote) {
       compositor = data.windowLayerManagerType;
-      if (data.windowUsingAdvancedLayers) {
-        compositor += " (Advanced Layers)";
-      }
     } else {
       let noOMTCString = await document.l10n.formatValue("main-thread-no-omtc");
       compositor = "BasicLayers (" + noOMTCString + ")";
@@ -625,7 +698,6 @@ var snapshotFormatters = {
     delete data.numTotalWindows;
     delete data.numAcceleratedWindows;
     delete data.numAcceleratedWindowsMessage;
-    delete data.windowUsingAdvancedLayers;
 
     addRow(
       "features",
@@ -660,9 +732,6 @@ var snapshotFormatters = {
       ["windowProtocol", "graphics-window-protocol"],
       ["desktopEnvironment", "graphics-desktop-environment"],
       "usesTiling",
-      "contentUsesTiling",
-      "offMainThreadPaintEnabled",
-      "offMainThreadPaintWorkerCount",
       "targetFrameRate",
     ];
     for (let feature of featureKeys) {
@@ -736,39 +805,30 @@ var snapshotFormatters = {
     let featureLog = data.featureLog;
     delete data.featureLog;
 
-    let features = [];
-    for (let feature of featureLog.features) {
-      // Only add interesting decisions - ones that were not automatic based on
-      // all.js/gfxPrefs defaults.
-      if (feature.log.length > 1 || feature.log[0].status != "available") {
-        features.push(feature);
-      }
-    }
-
-    if (features.length) {
-      for (let feature of features) {
+    if (featureLog.features.length) {
+      for (let feature of featureLog.features) {
         let trs = [];
         for (let entry of feature.log) {
-          if (entry.type == "default" && entry.status == "available") {
-            continue;
-          }
-
           let contents;
-          if (entry.message.length > 0 && entry.message[0] == "#") {
+          if (!entry.hasOwnProperty("message")) {
+            // This is a default entry.
+            contents = entry.status + " by " + entry.type;
+          } else if (entry.message.length && entry.message[0] == "#") {
             // This is a failure ID. See nsIGfxInfo.idl.
             let m = /#BLOCKLIST_FEATURE_FAILURE_BUG_(\d+)/.exec(entry.message);
             if (m) {
               let bugSpan = $.new("span");
-              document.l10n.setAttributes(bugSpan, "blocklisted-bug");
 
               let bugHref = $.new("a");
               bugHref.href =
                 "https://bugzilla.mozilla.org/show_bug.cgi?id=" + m[1];
-              document.l10n.setAttributes(bugHref, "bug-link", {
+              bugHref.setAttribute("data-l10n-name", "bug-link");
+              bugSpan.append(bugHref);
+              document.l10n.setAttributes(bugSpan, "support-blocklisted-bug", {
                 bugNumber: m[1],
               });
 
-              contents = [bugSpan, bugHref];
+              contents = [bugSpan];
             } else {
               let unknownFailure = $.new("span");
               document.l10n.setAttributes(unknownFailure, "unknown-failure", {
@@ -1027,7 +1087,7 @@ var snapshotFormatters = {
   },
 
   remoteAgent(data) {
-    if (!AppConstants.ENABLE_REMOTE_AGENT) {
+    if (!AppConstants.ENABLE_WEBDRIVER) {
       return;
     }
     $("remote-debugging-accepting-connections").textContent = data.listening;
@@ -1344,7 +1404,7 @@ Serializer.prototype = {
   },
 
   set _currentLine(val) {
-    return (this._lines[this._lines.length - 1] = val);
+    this._lines[this._lines.length - 1] = val;
   },
 
   _serializeElement(elem) {
@@ -1514,11 +1574,9 @@ function openProfileDirectory() {
 function populateActionBox() {
   if (ResetProfile.resetSupported()) {
     $("reset-box").style.display = "block";
-    $("action-box").style.display = "block";
   }
   if (!Services.appinfo.inSafeMode && AppConstants.platform !== "android") {
     $("safe-mode-box").style.display = "block";
-    $("action-box").style.display = "block";
 
     if (Services.policies && !Services.policies.isAllowed("safeMode")) {
       $("restart-in-safe-mode-button").setAttribute("disabled", "true");
@@ -1544,9 +1602,12 @@ function safeModeRestart() {
 
 // Added for TB.
 function onShowPrivateDataChange() {
-  document.getElementById(
-    "about-support-private"
-  ).disabled = document.getElementById("check-show-private-data").checked;
+  document
+    .getElementById("contents")
+    .classList.toggle(
+      "show-private-data",
+      document.getElementById("check-show-private-data").checked
+    );
 }
 
 /**
@@ -1561,7 +1622,43 @@ function setupEventListeners() {
     });
   }
 */
-  let button = $("restart-in-safe-mode-button");
+  let button = $("clear-startup-cache-button");
+  if (button) {
+    button.addEventListener("click", async function(event) {
+      const [
+        promptTitle,
+        promptBody,
+        restartButtonLabel,
+      ] = await document.l10n.formatValues([
+        { id: "startup-cache-dialog-title2" },
+        { id: "startup-cache-dialog-body2" },
+        { id: "restart-button-label" },
+      ]);
+      const buttonFlags =
+        Services.prompt.BUTTON_POS_0 * Services.prompt.BUTTON_TITLE_IS_STRING +
+        Services.prompt.BUTTON_POS_1 * Services.prompt.BUTTON_TITLE_CANCEL +
+        Services.prompt.BUTTON_POS_0_DEFAULT;
+      const result = Services.prompt.confirmEx(
+        window.docShell.chromeEventHandler.ownerGlobal,
+        promptTitle,
+        promptBody,
+        buttonFlags,
+        restartButtonLabel,
+        null,
+        null,
+        null,
+        {}
+      );
+      if (result !== 0) {
+        return;
+      }
+      Services.appinfo.invalidateCachesOnRestart();
+      Services.startup.quit(
+        Ci.nsIAppStartup.eRestart | Ci.nsIAppStartup.eAttemptQuit
+      );
+    });
+  }
+  button = $("restart-in-safe-mode-button");
   if (button) {
     button.addEventListener("click", function(event) {
       if (
@@ -1597,7 +1694,7 @@ function setupEventListeners() {
     button = $("show-update-history-button");
     if (button) {
       button.addEventListener("click", function(event) {
-        window.docShell.rootTreeItem.domWindow.openDialog(
+        window.browsingContext.topChromeWindow.openDialog(
           "chrome://mozapps/content/update/history.xhtml",
           "Update:History",
           "centerscreen,resizable=no,titlebar,modal"
