@@ -1,4 +1,3 @@
-/* -*- Mode: Java; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
 /* This Source Code Form is subject to the terms of the Mozilla Public
  * License, v. 2.0. If a copy of the MPL was not distributed with this
  * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
@@ -8,10 +7,6 @@
 /* import-globals-from ComposerCommands.js */
 /* import-globals-from editorUtilities.js */
 
-var { GetNextUntitledValue } = ChromeUtils.import(
-  "resource:///modules/editorUtilities.jsm"
-);
-var { Async } = ChromeUtils.import("resource://services-common/async.js");
 var { AppConstants } = ChromeUtils.import(
   "resource://gre/modules/AppConstants.jsm"
 );
@@ -38,8 +33,7 @@ const kDisplayModeTabIDS = [
   "SourceModeButton",
   "PreviewModeButton",
 ];
-const kNormalStyleSheet =
-  "chrome://messenger/content/messengercompose/EditorContent.css";
+const kNormalStyleSheet = "chrome://messenger/skin/shared/editorContent.css";
 const kContentEditableStyleSheet = "resource://gre/res/contenteditable.css";
 
 var kTextMimeType = "text/plain";
@@ -439,6 +433,17 @@ async function CheckAndSaveDocument(command, allowDontSave) {
 
 // --------------------------- Text style ---------------------------
 
+function editorSetParagraphState(state) {
+  if (state === "") {
+    // Corresponds to body text. Has no corresponding formatBlock value.
+    goDoCommandParams("cmd_paragraphState", "");
+  } else {
+    GetCurrentEditor().document.execCommand("formatBlock", false, state);
+  }
+  document.getElementById("cmd_paragraphState").setAttribute("state", state);
+  onParagraphFormatChange();
+}
+
 function onParagraphFormatChange() {
   let paraMenuList = document.getElementById("ParagraphSelect");
   if (!paraMenuList) {
@@ -459,10 +464,8 @@ function onParagraphFormatChange() {
     paraMenuList.setAttribute("label", GetString("Mixed"));
   } else {
     var menuPopup = document.getElementById("ParagraphPopup");
-    var menuItems = menuPopup.children;
-    for (var i = 0; i < menuItems.length; i++) {
-      var menuItem = menuItems.item(i);
-      if ("value" in menuItem && menuItem.value == state) {
+    for (let menuItem of menuPopup.children) {
+      if (menuItem.value === state) {
         paraMenuList.selectedItem = menuItem;
         break;
       }
@@ -470,6 +473,12 @@ function onParagraphFormatChange() {
   }
 }
 
+function editorRemoveTextStyling() {
+  GetCurrentEditor().document.execCommand("removeFormat", false, null);
+  // After removing the formatting, update the full styling command set.
+  window.updateCommands("style");
+}
+
 /**
  * Selects the current font face in the menulist.
  */
@@ -661,35 +670,26 @@ function onFontFaceChange() {
   fontFaceMenuList.selectedItem = foundFont;
 }
 
+/**
+ * Changes the font size for the selection or at the insertion point. This
+ * requires an integer from 1-7 as a value argument (x-small - xxx-large)
+ *
+ * @param {"1"|"2"|"3"|"4"|"5"|"6"|"7"} size - The font size.
+ */
 function EditorSetFontSize(size) {
-  if (size == "0" || size == "normal" || size == "medium") {
+  // For normal/medium size (that is 3), we clear size.
+  if (size == "3") {
     EditorRemoveTextProperty("font", "size");
     // Also remove big and small,
     //  else it will seem like size isn't changing correctly
     EditorRemoveTextProperty("small", "");
     EditorRemoveTextProperty("big", "");
   } else {
-    // Temp: convert from new CSS size strings to old HTML size strings
-    switch (size) {
-      case "xx-small":
-      case "x-small":
-        size = "-2";
-        break;
-      case "small":
-        size = "-1";
-        break;
-      case "large":
-        size = "+1";
-        break;
-      case "x-large":
-        size = "+2";
-        break;
-      case "xx-large":
-        size = "+3";
-        break;
-    }
-    EditorSetTextProperty("font", "size", size);
+    GetCurrentEditor().document.execCommand("fontSize", false, size);
   }
+  // Enable or Disable the toolbar buttons according to the font size.
+  goUpdateCommand("cmd_decreaseFontStep");
+  goUpdateCommand("cmd_increaseFontStep");
   gContentWindow.focus();
 }
 
@@ -849,121 +849,27 @@ function createFontFaceMenuitem(aFontLabel, aFontName, aMenuPopup) {
 
 /**
  * Helper function
+ *
+ * @see https://dvcs.w3.org/hg/editing/raw-file/tip/editing.html#legacy-font-size-for
  */
-function getFontSizeIndex() {
-  var firstHas = { value: false };
-  var anyHas = { value: false };
-  var allHas = { value: false };
-
-  var fontSize = EditorGetTextProperty(
-    "font",
-    "size",
-    null,
-    firstHas,
-    anyHas,
-    allHas
-  );
-
-  // If the element has no size attribute and no size was found at all,
-  // we assume "medium" size. This is highly problematic since
-  // CSS sizes are not recognised and will show as "medium" as well.
-  // Currently we can't distinguish between "no attribute" which
-  // can imply "medium" and "CSS attribute present" which should not
-  // imply "medium".
-  if (!anyHas.value) {
-    return 2;
+function getLegacyFontSize() {
+  let fontSize = GetCurrentEditor().document.queryCommandValue("fontSize");
+  // If one selects all the texts in the editor and deletes it, the editor
+  // will return null fontSize. We will set it to default value then.
+  if (!fontSize) {
+    fontSize = Services.prefs.getCharPref("msgcompose.font_size", "3");
   }
-
-  // Mixed selection.
-  if (!allHas.value) {
-    return -1;
-  }
-
-  switch (fontSize) {
-    case "-3":
-    case "-2":
-    case "0":
-    case "1":
-      // x-small.
-      return 0;
-    case "-1":
-    case "2":
-      // small.
-      return 1;
-    case "3":
-      // medium.
-      return 2;
-    case "+1":
-    case "4":
-      // large.
-      return 3;
-    case "+2":
-    case "5":
-      // x-large.
-      return 4;
-    case "+3":
-    case "+4":
-    case "6":
-    case "7":
-      // xx-large.
-      return 5;
-  }
-
-  // We shouldn't get here. All the selection has a value we don't understand.
-  return -1;
+  return fontSize;
 }
 
-function initFontSizeMenu(menuPopup, fullMenu) {
+function initFontSizeMenu(menuPopup) {
   if (menuPopup) {
-    var children = menuPopup.children;
-    if (!children) {
-      return;
-    }
-
-    // Fixed size items start after menu separator depending on whether it is
-    // a full menu.
-    var menuIndex = fullMenu ? 3 : 0;
-
-    var setIndex = getFontSizeIndex();
-    if (setIndex >= 0) {
-      children[menuIndex + setIndex].setAttribute("checked", true);
-    } else {
-      // In case of mixed, clear all items.
-      for (var i = menuIndex; i < children.length; i++) {
-        children[i].setAttribute("checked", false);
+    let fontSize = getLegacyFontSize();
+    for (let menuitem of menuPopup.children) {
+      if (menuitem.getAttribute("value") == fontSize) {
+        menuitem.setAttribute("checked", true);
       }
     }
-
-    // Some configurations might not have the "small/big" indicator as
-    // last item. If there is no indicator, we are done.
-    if (!menuPopup.lastElementChild.id.includes("smallBigInfo")) {
-      return;
-    }
-
-    // While it would be better to show the number of levels,
-    // at least this tells user if either of them are set.
-    var firstHas = { value: false };
-    var anyHas = { value: false };
-    var allHas = { value: false };
-
-    // Show "small"/"big" indicator.
-    var htmlInfo = "";
-    EditorGetTextProperty("small", "", "", firstHas, anyHas, allHas);
-    if (anyHas.value) {
-      htmlInfo = "<small>";
-    }
-    EditorGetTextProperty("big", "", "", firstHas, anyHas, allHas);
-    if (anyHas.value) {
-      htmlInfo += "<big>";
-    }
-
-    if (htmlInfo) {
-      menuPopup.lastElementChild.hidden = false;
-      menuPopup.lastElementChild.setAttribute("label", "HTML: " + htmlInfo);
-      menuPopup.lastElementChild.setAttribute("checked", true);
-    } else {
-      menuPopup.lastElementChild.hidden = true;
-    }
   }
 }
 
@@ -997,7 +903,7 @@ function ChangeButtonColor(commandID, id, defaultColor) {
         color = defaultColor;
       }
 
-      button.setAttribute("style", "background-color:" + color + " !important");
+      button.style.backgroundColor = color;
     }
   }
 }
@@ -1228,7 +1134,11 @@ function EditorSelectColor(colorType, mouseEvent) {
   if (gColorObj.Type == "Text") {
     if (currentColor != gColorObj.TextColor) {
       if (gColorObj.TextColor) {
-        EditorSetTextProperty("font", "color", gColorObj.TextColor);
+        GetCurrentEditor().document.execCommand(
+          "foreColor",
+          false,
+          gColorObj.TextColor
+        );
       } else {
         EditorRemoveTextProperty("font", "color");
       }
@@ -1238,7 +1148,11 @@ function EditorSelectColor(colorType, mouseEvent) {
   } else if (gColorObj.Type == "Highlight") {
     if (currentColor != gColorObj.HighlightColor) {
       if (gColorObj.HighlightColor) {
-        EditorSetTextProperty("font", "bgcolor", gColorObj.HighlightColor);
+        GetCurrentEditor().document.execCommand(
+          "backColor",
+          false,
+          gColorObj.HighlightColor
+        );
       } else {
         EditorRemoveTextProperty("font", "bgcolor");
       }
@@ -1351,44 +1265,36 @@ function GetParentTableCell(element) {
 }
 
 function EditorDblClick(event) {
-  // We check event.explicitOriginalTarget here because .target will never
-  // be a textnode (bug 193689)
-  if (event.explicitOriginalTarget) {
-    // Only bring up properties if clicked on an element or selected link
-    var element;
+  // Only bring up properties if clicked on an element or selected link
+  let element = event.target;
+  //  We use "href" instead of "a" to not be fooled by named anchor
+  if (!element) {
     try {
-      element = event.explicitOriginalTarget;
+      element = GetCurrentEditor().getSelectedElement("href");
     } catch (e) {}
+  }
 
-    //  We use "href" instead of "a" to not be fooled by named anchor
-    if (!element) {
-      try {
-        element = GetCurrentEditor().getSelectedElement("href");
-      } catch (e) {}
-    }
-
-    // Don't fire for body/p and other block elements.
-    // It's common that people try to double-click
-    // to select a word, but the click hits an empty area.
-    if (
-      element &&
-      ![
-        "body",
-        "p",
-        "h1",
-        "h2",
-        "h3",
-        "h4",
-        "h5",
-        "h6",
-        "blockquote",
-        "div",
-        "pre",
-      ].includes(element.nodeName.toLowerCase())
-    ) {
-      goDoCommand("cmd_objectProperties");
-      event.preventDefault();
-    }
+  // Don't fire for body/p and other block elements.
+  // It's common that people try to double-click
+  // to select a word, but the click hits an empty area.
+  if (
+    element &&
+    ![
+      "body",
+      "p",
+      "h1",
+      "h2",
+      "h3",
+      "h4",
+      "h5",
+      "h6",
+      "blockquote",
+      "div",
+      "pre",
+    ].includes(element.nodeName.toLowerCase())
+  ) {
+    goDoCommand("cmd_objectProperties");
+    event.preventDefault();
   }
 }
 
@@ -1947,13 +1853,6 @@ function initFontStyleMenu(menuPopup) {
   }
 }
 
-// --------------------------------------------------------------------
-function onButtonUpdate(button, commmandID) {
-  var commandNode = document.getElementById(commmandID);
-  var state = commandNode.getAttribute("state");
-  button.checked = state == "true";
-}
-
 // -----------------------------------------------------------------------------------
 function IsSpellCheckerInstalled() {
   return true; // Always installed.
@@ -2291,31 +2190,26 @@ function GetNumberOfContiguousSelectedRows() {
   }
 
   var rows = 0;
-  try {
-    var editor = GetCurrentTableEditor();
-    var rowObj = { value: 0 };
-    var colObj = { value: 0 };
-    var cell = editor.getFirstSelectedCellInTable(rowObj, colObj);
-    if (!cell) {
-      return 0;
+  var editor = GetCurrentTableEditor();
+  var rowObj = { value: 0 };
+  var colObj = { value: 0 };
+  var cell = editor.getFirstSelectedCellInTable(rowObj, colObj);
+  if (!cell) {
+    return 0;
+  }
+
+  // We have at least one row
+  rows++;
+
+  var lastIndex = rowObj.value;
+  for (let cell of editor.getSelectedCells()) {
+    editor.getCellIndexes(cell, rowObj, colObj);
+    var index = rowObj.value;
+    if (index == lastIndex + 1) {
+      lastIndex = index;
+      rows++;
     }
-
-    // We have at least one row
-    rows++;
-
-    var lastIndex = rowObj.value;
-    do {
-      cell = editor.getNextSelectedCell({ value: 0 });
-      if (cell) {
-        editor.getCellIndexes(cell, rowObj, colObj);
-        var index = rowObj.value;
-        if (index == lastIndex + 1) {
-          lastIndex = index;
-          rows++;
-        }
-      }
-    } while (cell);
-  } catch (e) {}
+  }
 
   return rows;
 }
@@ -2326,31 +2220,27 @@ function GetNumberOfContiguousSelectedColumns() {
   }
 
   var columns = 0;
-  try {
-    var editor = GetCurrentTableEditor();
-    var colObj = { value: 0 };
-    var rowObj = { value: 0 };
-    var cell = editor.getFirstSelectedCellInTable(rowObj, colObj);
-    if (!cell) {
-      return 0;
+
+  var editor = GetCurrentTableEditor();
+  var colObj = { value: 0 };
+  var rowObj = { value: 0 };
+  var cell = editor.getFirstSelectedCellInTable(rowObj, colObj);
+  if (!cell) {
+    return 0;
+  }
+
+  // We have at least one column
+  columns++;
+
+  var lastIndex = colObj.value;
+  for (let cell of editor.getSelectedCells()) {
+    editor.getCellIndexes(cell, rowObj, colObj);
+    var index = colObj.value;
+    if (index == lastIndex + 1) {
+      lastIndex = index;
+      columns++;
     }
-
-    // We have at least one column
-    columns++;
-
-    var lastIndex = colObj.value;
-    do {
-      cell = editor.getNextSelectedCell({ value: 0 });
-      if (cell) {
-        editor.getCellIndexes(cell, rowObj, colObj);
-        var index = colObj.value;
-        if (index == lastIndex + 1) {
-          lastIndex = index;
-          columns++;
-        }
-      }
-    } while (cell);
-  } catch (e) {}
+  }
 
   return columns;
 }
