@@ -11,6 +11,15 @@ var { BrowserUtils } = ChromeUtils.import(
 var { ExtensionParent } = ChromeUtils.import(
   "resource://gre/modules/ExtensionParent.jsm"
 );
+var { MailE10SUtils } = ChromeUtils.import(
+  "resource:///modules/MailE10SUtils.jsm"
+);
+
+ChromeUtils.defineModuleGetter(
+  this,
+  "PromiseUtils",
+  "resource://gre/modules/PromiseUtils.jsm"
+);
 
 var gContextMenu;
 
@@ -19,7 +28,20 @@ var gContextMenu;
 function loadRequestedUrl() {
   let browser = document.getElementById("requestFrame");
   browser.addProgressListener(reporterListener, Ci.nsIWebProgress.NOTIFY_ALL);
-  browser.addEventListener("DOMTitleChanged", () => gBrowser.updateTitlebar());
+  browser.addEventListener(
+    "DOMWindowClose",
+    () => {
+      if (browser.getAttribute("allowscriptstoclose") == "true") {
+        window.close();
+      }
+    },
+    true
+  );
+  browser.addEventListener(
+    "pagetitlechanged",
+    () => gBrowser.updateTitlebar(),
+    true
+  );
 
   // This window does double duty. If window.arguments[0] is a string, it's
   // probably being called by browser.identity.launchWebAuthFlowInParent.
@@ -29,14 +51,16 @@ function loadRequestedUrl() {
   // which is consistent with Firefox behaviour.
 
   if (typeof window.arguments[0] == "string") {
-    browser.src = window.arguments[0];
+    MailE10SUtils.loadURI(browser, window.arguments[0]);
   } else {
     if (window.arguments[1].wrappedJSObject.allowScriptsToClose) {
-      browser.contentWindow.windowUtils.allowScriptsToClose();
+      browser.setAttribute("allowscriptstoclose", "true");
     }
     ExtensionParent.apiManager.emit("extension-browser-inserted", browser);
-    browser.src =
-      window.arguments[1].wrappedJSObject.tabs[0].tabParams.contentPage;
+    MailE10SUtils.loadURI(
+      browser,
+      window.arguments[1].wrappedJSObject.tabs[0].tabParams.url
+    );
   }
 }
 
@@ -52,9 +76,8 @@ var gBrowser = {
     return this.selectedBrowser.webNavigation;
   },
   updateTitlebar() {
-    let docTitle = browser.contentDocument.title
-      ? browser.contentDocument.title.trim()
-      : "";
+    let docTitle =
+      browser.browsingContext?.currentWindowGlobal?.documentTitle?.trim() || "";
     let docElement = document.documentElement;
     // If the document title is blank, add the default title.
     if (!docTitle) {
@@ -75,6 +98,9 @@ var gBrowser = {
     document.title = docTitle;
     document.dispatchEvent(new Event("extension-window-title-changed"));
   },
+  getTabForBrowser(browser) {
+    return null;
+  },
 };
 
 this.__defineGetter__("browser", getBrowser);
@@ -82,12 +108,37 @@ this.__defineGetter__("browser", getBrowser);
 function getBrowser() {
   return gBrowser.selectedBrowser;
 }
-function mailContextOnContextMenu(event) {
-  document.getElementById("mailContext").target =
-    event.composedTarget || event.originalTarget;
-}
 function fillMailContextMenu(event) {
   gContextMenu = new nsContextMenu(event.target, event.shiftKey);
   return gContextMenu.shouldDisplay;
 }
 function mailContextOnPopupHiding() {}
+
+var gBrowserInit = {
+  onDOMContentLoaded() {
+    let initiallyFocusedElement = document.commandDispatcher.focusedElement;
+    let promise = gBrowser.selectedBrowser.isRemoteBrowser
+      ? PromiseUtils.defer().promise
+      : Promise.resolve();
+
+    promise.then(() => {
+      // If focus didn't move while we were waiting, we're okay to move to
+      // the browser.
+      if (
+        document.commandDispatcher.focusedElement == initiallyFocusedElement
+      ) {
+        gBrowser.selectedBrowser.focus();
+      }
+    });
+  },
+};
+
+// The listener of DOMContentLoaded must be set on window, rather than
+// document, because the window can go away before the event is fired.
+// In that case, we don't want to initialize anything, otherwise we
+// may be leaking things because they will never be destroyed after.
+window.addEventListener(
+  "DOMContentLoaded",
+  gBrowserInit.onDOMContentLoaded.bind(gBrowserInit),
+  { once: true }
+);
