@@ -2,8 +2,6 @@
  * License, v. 2.0. If a copy of the MPL was not distributed with this
  * file, you can obtain one at http://mozilla.org/MPL/2.0/. */
 
-/* globals gFolderDisplay, gFolderTreeView, MsgOpenNewWindowForMessage */
-
 let { BrowserTestUtils } = ChromeUtils.import(
   "resource://testing-common/BrowserTestUtils.jsm"
 );
@@ -11,45 +9,25 @@ let { BrowserTestUtils } = ChromeUtils.import(
 add_task(async () => {
   let files = {
     "background.js": async () => {
-      // Address book window.
+      // Message compose window.
 
       let createdWindowPromise = window.waitForEvent("windows.onCreated");
-      await browser.addressBooks.openUI();
+      await browser.compose.beginNew();
       let [createdWindow] = await createdWindowPromise;
-      browser.test.assertEq("addressBook", createdWindow.type);
+      browser.test.assertEq("messageCompose", createdWindow.type);
 
       let windowDetail = await browser.windows.get(createdWindow.id, {
         populate: true,
       });
-      browser.test.assertEq("addressBook", windowDetail.type);
+      browser.test.assertEq("messageCompose", windowDetail.type);
       browser.test.assertEq(1, windowDetail.tabs.length);
+      browser.test.assertEq("messageCompose", windowDetail.tabs[0].type);
       // These three properties should not be present, but not fail either.
       browser.test.assertEq(undefined, windowDetail.tabs[0].favIconUrl);
       browser.test.assertEq(undefined, windowDetail.tabs[0].title);
       browser.test.assertEq(undefined, windowDetail.tabs[0].url);
 
       let removedWindowPromise = window.waitForEvent("windows.onRemoved");
-      await browser.addressBooks.closeUI();
-      await removedWindowPromise;
-
-      // Message compose window.
-
-      createdWindowPromise = window.waitForEvent("windows.onCreated");
-      await browser.compose.beginNew();
-      [createdWindow] = await createdWindowPromise;
-      browser.test.assertEq("messageCompose", createdWindow.type);
-
-      windowDetail = await browser.windows.get(createdWindow.id, {
-        populate: true,
-      });
-      browser.test.assertEq("messageCompose", windowDetail.type);
-      browser.test.assertEq(1, windowDetail.tabs.length);
-      // These three properties should not be present, but not fail either.
-      browser.test.assertEq(undefined, windowDetail.tabs[0].favIconUrl);
-      browser.test.assertEq(undefined, windowDetail.tabs[0].title);
-      browser.test.assertEq(undefined, windowDetail.tabs[0].url);
-
-      removedWindowPromise = window.waitForEvent("windows.onRemoved");
       await browser.tabs.remove(windowDetail.tabs[0].id);
       await removedWindowPromise;
 
@@ -65,6 +43,7 @@ add_task(async () => {
       });
       browser.test.assertEq("messageDisplay", windowDetail.type);
       browser.test.assertEq(1, windowDetail.tabs.length);
+      browser.test.assertEq("messageDisplay", windowDetail.tabs[0].type);
       browser.test.assertEq("about:blank", windowDetail.tabs[0].url);
       // These properties should not be present, but not fail either.
       browser.test.assertEq(undefined, windowDetail.tabs[0].favIconUrl);
@@ -96,15 +75,10 @@ add_task(async () => {
   }
   createMessages(subFolders.test1, 1);
 
-  gFolderTreeView.selectFolder(subFolders.test1);
-  gFolderDisplay.selectViewIndex(0);
-
   await extension.startup();
 
   await extension.awaitMessage("openMessage");
-  let newWindowPromise = BrowserTestUtils.domWindowOpened();
-  MsgOpenNewWindowForMessage();
-  let newWindow = await newWindowPromise;
+  let newWindow = await openMessageInWindow([...subFolders.test1.messages][0]);
 
   await extension.awaitMessage("closeMessage");
   newWindow.close();
