@@ -6,7 +6,8 @@
 /* globals statusSelector */
 /* globals MsgAccountManager */
 
-var { Services } = ChromeUtils.import("resource:///modules/imServices.jsm");
+var { Services } = ChromeUtils.import("resource://gre/modules/Services.jsm");
+var { IMServices } = ChromeUtils.import("resource:///modules/IMServices.jsm");
 var { MailServices } = ChromeUtils.import(
   "resource:///modules/MailServices.jsm"
 );
@@ -45,13 +46,12 @@ var gAccountManager = {
   _connectedLabelInterval: 0,
 
   get msgNotificationBar() {
-    delete this._msgNotificationBar;
-
-    let newNotificationBox = new MozElements.NotificationBox(element => {
-      document.getElementById("accounts-notification-box").prepend(element);
-    });
-
-    return (this._msgNotificationBar = newNotificationBox);
+    if (!this._notificationBox) {
+      this._notificationBox = new MozElements.NotificationBox(element => {
+        document.getElementById("accounts-notification-box").prepend(element);
+      });
+    }
+    return this._notificationBox;
   },
 
   load() {
@@ -60,7 +60,7 @@ var gAccountManager = {
       () => {
         this.accountList = document.getElementById("accountlist");
         let defaultID;
-        Services.core.init(); // ensure the imCore is initialized.
+        IMServices.core.init(); // ensure the imCore is initialized.
         for (let acc of this.getAccounts()) {
           let elt = document.createXULElement("richlistitem", {
             is: "chat-account-richlistitem",
@@ -85,7 +85,8 @@ var gAccountManager = {
           }, 0);
         } else {
           // we have accounts, show the list
-          document.getElementById("accountsDesk").selectedIndex = 1;
+          document.getElementById("noAccountScreen").hidden = true;
+          document.getElementById("accounts-notification-box").hidden = false;
 
           // ensure an account is selected
           if (defaultID) {
@@ -124,7 +125,7 @@ var gAccountManager = {
       if (oldItem.id != acc.id) {
         let accElt = document.getElementById(acc.id);
         accountList.insertBefore(accElt, oldItem);
-        accElt.restoreItems();
+        accElt.refreshState();
       }
       ++i;
     }
@@ -175,7 +176,8 @@ var gAccountManager = {
     let account = aObject.QueryInterface(Ci.imIAccount);
 
     if (aTopic == "account-added") {
-      document.getElementById("accountsDesk").selectedIndex = 1;
+      document.getElementById("noAccountScreen").hidden = true;
+      document.getElementById("accounts-notification-box").hidden = false;
       let elt = document.createXULElement("richlistitem", {
         is: "chat-account-richlistitem",
       });
@@ -200,7 +202,8 @@ var gAccountManager = {
       elt.remove();
       var count = this.accountList.getRowCount();
       if (!count) {
-        document.getElementById("accountsDesk").selectedIndex = 0;
+        document.getElementById("noAccountScreen").hidden = false;
+        document.getElementById("accounts-notification-box").hidden = true;
         return;
       }
       if (selectedIndex == count) {
@@ -211,7 +214,7 @@ var gAccountManager = {
       document.getElementById(account.id).build(account);
       this.disableCommandItems();
     } else if (aTopic == "account-connect-progress") {
-      document.getElementById(account.id).updateConnectionState();
+      document.getElementById(account.id).updateConnectingProgress();
     } else if (aTopic == "account-connect-error") {
       document.getElementById(account.id).updateConnectionError();
       // See NSSErrorsService::ErrorIsOverridable.
@@ -250,15 +253,7 @@ var gAccountManager = {
           // Probably disconnecting a removed account.
           return;
         }
-
-        if (aTopic == "account-connecting") {
-          elt.removeAttribute("error");
-          elt.updateConnectionState();
-        } else if (aTopic == "account-connected") {
-          elt.refreshConnectedLabel();
-        }
-
-        elt.setAttribute("state", stateEvents[aTopic]);
+        elt.refreshState(stateEvents[aTopic]);
       }
     }
   },
@@ -437,23 +432,18 @@ var gAccountManager = {
       (account.disconnected &&
         account.connectionErrorReason == Ci.imIAccount.ERROR_UNKNOWN_PRPL);
 
-    let disabledItems = [
-      ["connect", isCommandDisabled],
-      ["disconnect", isCommandDisabled],
-      ["moveup", accountList.selectedIndex == 0],
-      ["movedown", accountList.selectedIndex == accountList.itemCount - 1],
-    ];
-    for (let [name, state] of disabledItems) {
+    let disabledItems = ["connect", "disconnect"];
+    for (let name of disabledItems) {
       let elt = document.getElementById("cmd_" + name);
-      if (state) {
+      if (isCommandDisabled) {
         elt.setAttribute("disabled", "true");
       } else {
         elt.removeAttribute("disabled");
       }
     }
   },
-  onContextMenuShowing() {
-    let targetElt = document.popupNode.closest(
+  onContextMenuShowing(event) {
+    let targetElt = event.target.triggerNode.closest(
       'richlistitem[is="chat-account-richlistitem"]'
     );
     document.querySelectorAll(".im-context-account-item").forEach(e => {
@@ -500,18 +490,6 @@ var gAccountManager = {
     if (!this.selectedItem) {
       return;
     }
-
-    if (
-      event.shiftKey &&
-      (event.keyCode == event.DOM_VK_DOWN || event.keyCode == event.DOM_VK_UP)
-    ) {
-      let offset = event.keyCode == event.DOM_VK_DOWN ? 1 : -1;
-      gAccountManager.moveCurrentItem(offset);
-      event.stopPropagation();
-      event.preventDefault();
-      return;
-    }
-
     // As we stop propagation, the default action applies to the richlistbox
     // so that the selected account is changed with this default action
     if (event.keyCode == event.DOM_VK_DOWN) {
@@ -531,7 +509,7 @@ var gAccountManager = {
     }
 
     if (event.keyCode == event.DOM_VK_RETURN) {
-      let target = event.originalTarget;
+      let target = event.target;
       if (
         target.localName != "checkbox" &&
         (target.localName != "button" ||
@@ -542,35 +520,8 @@ var gAccountManager = {
     }
   },
 
-  moveCurrentItem(aOffset) {
-    let accountList = this.accountList;
-    if (!aOffset || !accountList.selectedItem) {
-      return;
-    }
-
-    // Create the new preference value from the richlistbox list
-    let items = accountList.itemChildren;
-    let selectedID = accountList.selectedItem.id;
-    let array = [];
-    for (let i in items) {
-      if (items[i].id != selectedID) {
-        array.push(items[i].id);
-      }
-    }
-
-    let newIndex = accountList.selectedIndex + aOffset;
-    if (newIndex < 0) {
-      newIndex = 0;
-    } else if (newIndex >= accountList.itemCount) {
-      newIndex = accountList.itemCount - 1;
-    }
-    array.splice(newIndex, 0, selectedID);
-
-    Services.prefs.setCharPref("messenger.accounts", array.join(","));
-  },
-
   *getAccounts() {
-    for (let account of Services.accounts.getAccounts()) {
+    for (let account of IMServices.accounts.getAccounts()) {
       yield account;
     }
   },
@@ -582,7 +533,7 @@ var gAccountManager = {
   },
 
   setAutoLoginNotification() {
-    var as = Services.accounts;
+    var as = IMServices.accounts;
     var autoLoginStatus = as.autoLoginStatus;
     let isOffline = false;
     let crashCount = 0;
@@ -596,14 +547,14 @@ var gAccountManager = {
     }
 
     if (autoLoginStatus == as.AUTOLOGIN_ENABLED && crashCount == 0) {
-      let status = Services.core.globalUserStatus.statusType;
+      let status = IMServices.core.globalUserStatus.statusType;
       this.setOffline(isOffline || status == Ci.imIStatusInfo.STATUS_OFFLINE);
       return;
     }
 
     var bundle = document.getElementById("accountsBundle");
     let box = this.msgNotificationBar;
-    var priority = box.PRIORITY_INFO_HIGH;
+    var prio = box.PRIORITY_INFO_HIGH;
     var connectNowButton = {
       accessKey: bundle.getString(
         "accountsManager.notification.button.accessKey"
@@ -611,51 +562,61 @@ var gAccountManager = {
       callback: this.processAutoLogin,
       label: bundle.getString("accountsManager.notification.button.label"),
     };
-    var label;
+    var barLabel;
 
     switch (autoLoginStatus) {
       case as.AUTOLOGIN_USER_DISABLED:
-        label = bundle.getString(
+        barLabel = bundle.getString(
           "accountsManager.notification.userDisabled.label"
         );
         break;
 
       case as.AUTOLOGIN_SAFE_MODE:
-        label = bundle.getString("accountsManager.notification.safeMode.label");
+        barLabel = bundle.getString(
+          "accountsManager.notification.safeMode.label"
+        );
         break;
 
       case as.AUTOLOGIN_START_OFFLINE:
-        label = bundle.getString(
+        barLabel = bundle.getString(
           "accountsManager.notification.startOffline.label"
         );
         isOffline = true;
         break;
 
       case as.AUTOLOGIN_CRASH:
-        label = bundle.getString("accountsManager.notification.crash.label");
-        priority = box.PRIORITY_WARNING_MEDIUM;
+        barLabel = bundle.getString("accountsManager.notification.crash.label");
+        prio = box.PRIORITY_WARNING_MEDIUM;
         break;
 
       /* One or more accounts made the application crash during their connection.
          If none, this function has already returned */
       case as.AUTOLOGIN_ENABLED:
-        label = bundle.getString(
+        barLabel = bundle.getString(
           "accountsManager.notification.singleCrash.label"
         );
-        label = PluralForm.get(crashCount, label).replace("#1", crashCount);
-        priority = box.PRIORITY_WARNING_MEDIUM;
+        barLabel = PluralForm.get(crashCount, barLabel).replace(
+          "#1",
+          crashCount
+        );
+        prio = box.PRIORITY_WARNING_MEDIUM;
         connectNowButton.callback = this.processCrashedAccountsLogin;
         break;
 
       default:
-        label = bundle.getString("accountsManager.notification.other.label");
+        barLabel = bundle.getString("accountsManager.notification.other.label");
     }
-    let status = Services.core.globalUserStatus.statusType;
+    let status = IMServices.core.globalUserStatus.statusType;
     this.setOffline(isOffline || status == Ci.imIStatusInfo.STATUS_OFFLINE);
 
-    box.appendNotification(label, "autologinStatus", null, priority, [
-      connectNowButton,
-    ]);
+    box.appendNotification(
+      "autologinStatus",
+      {
+        label: barLabel,
+        priority: prio,
+      },
+      [connectNowButton]
+    );
   },
   processAutoLogin() {
     var ioService = Services.io;
@@ -664,7 +625,7 @@ var gAccountManager = {
       ioService.offline = false;
     }
 
-    Services.accounts.processAutoLogin();
+    IMServices.accounts.processAutoLogin();
 
     gAccountManager.accountList.selectedItem.setFocus();
   },
@@ -699,159 +660,6 @@ var gAccountManager = {
   },
 };
 
-var gAMDragAndDrop = {
-  ACCOUNT_MIME_TYPE: "application/x-moz-richlistitem",
-  // Size of the scroll zone on the top and on the bottom of the account list
-  MAGIC_SCROLL_HEIGHT: 20,
-
-  // A preference already exists to define scroll speed, let's use it.
-  get SCROLL_SPEED() {
-    delete this.SCROLL_SPEED;
-    this.SCROLL_SPEED = Services.prefs.getIntPref(
-      "toolkit.scrollbox.scrollIncrement",
-      20
-    );
-    return this.SCROLL_SPEED;
-  },
-
-  onDragStart(event) {
-    let target = event.target;
-    if (target.nodeType != Node.ELEMENT_NODE) {
-      target = target.parentNode;
-    }
-    let accountElement = target.closest(
-      'richlistitem[is="chat-account-richlistitem"]'
-    );
-    if (!accountElement) {
-      return;
-    }
-    if (gAccountManager.accountList.itemCount == 1) {
-      throw new Error("Can't drag while there is only one account!");
-    }
-
-    event.dataTransfer.effectAllowed = "move";
-    event.dataTransfer.dropEffect = "move";
-    event.dataTransfer.addElement(accountElement);
-    event.dataTransfer.setData(this.ACCOUNT_MIME_TYPE, accountElement);
-    event.stopPropagation();
-  },
-
-  onDragOver(event) {
-    let target = event.target;
-    if (target.nodeType != Node.ELEMENT_NODE) {
-      target = target.parentNode;
-    }
-    let accountElement = target.closest(
-      'richlistitem[is="chat-account-richlistitem"]'
-    );
-    if (!accountElement) {
-      return;
-    }
-
-    // Auto scroll the account list if we are dragging at the top/bottom
-    this.checkForMagicScroll(event.clientY);
-
-    // The hovered element has changed, change the border too
-    if ("_accountElement" in this && this._accountElement != accountElement) {
-      this.cleanBorders();
-    }
-
-    event.dataTransfer.dropEffect = "move";
-
-    if (
-      event.clientY <
-      accountElement.getBoundingClientRect().top +
-        accountElement.clientHeight / 2
-    ) {
-      // we don't want the previous item to show its default bottom-border
-      let previousItem = accountElement.previousElementSibling;
-      if (previousItem) {
-        previousItem.style.borderBottom = "none";
-      }
-      accountElement.setAttribute("dragover", "up");
-    } else {
-      if (
-        "_accountElement" in this &&
-        this._accountElement == accountElement &&
-        accountElement.getAttribute("dragover") == "up"
-      ) {
-        this.cleanBorders();
-      }
-      accountElement.setAttribute("dragover", "down");
-    }
-
-    this._accountElement = accountElement;
-    event.stopPropagation();
-    event.preventDefault();
-  },
-
-  cleanBorders(isEnd) {
-    if (!this._accountElement) {
-      return;
-    }
-    this._accountElement.removeAttribute("dragover");
-    // reset the border of the previous element
-    let previousItem = this._accountElement.previousElementSibling;
-    if (previousItem) {
-      if (
-        isEnd &&
-        !previousItem.style.borderBottom &&
-        previousItem.previousElementSibling
-      ) {
-        previousItem = previousItem.previousElementSibling;
-      }
-      previousItem.style.borderBottom = "";
-    }
-
-    if (isEnd) {
-      delete this._accountElement;
-    }
-  },
-
-  checkForMagicScroll(clientY) {
-    let accountList = gAccountManager.accountList;
-    let listSize = accountList.getBoundingClientRect();
-    let direction = 1;
-    if (clientY < listSize.top + this.MAGIC_SCROLL_HEIGHT) {
-      direction = -1;
-    } else if (clientY < listSize.bottom - this.MAGIC_SCROLL_HEIGHT) {
-      // We are not on a scroll zone
-      return;
-    }
-
-    accountList.scrollTop += direction * this.SCROLL_SPEED;
-  },
-
-  onDrop(event) {
-    let target = event.target;
-    if (target.nodeType != Node.ELEMENT_NODE) {
-      target = target.parentNode;
-    }
-    let accountElement = target.closest(
-      'richlistitem[is="chat-account-richlistitem"]'
-    );
-    if (!accountElement) {
-      return;
-    }
-
-    // compute the destination
-    let accountList = gAccountManager.accountList;
-    let offset =
-      accountList.getIndexOfItem(accountElement) - accountList.selectedIndex;
-    let isDroppingAbove =
-      event.clientY <
-      accountElement.getBoundingClientRect().top +
-        accountElement.clientHeight / 2;
-    if (offset > 0) {
-      offset -= isDroppingAbove;
-    } else {
-      offset += !isDroppingAbove;
-    }
-    gAccountManager.moveCurrentItem(offset);
-    event.stopPropagation();
-  },
-};
-
 window.addEventListener("DOMContentLoaded", () => {
   gAccountManager.load();
 });
