@@ -3,7 +3,6 @@
  * License, v. 2.0. If a copy of the MPL was not distributed with this
  * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
-#include "nsDataHashtable.h"
 #include "nsMailProfileMigratorUtils.h"
 #include "nsDirectoryServiceDefs.h"
 #include "nsIMsgAccountManager.h"
@@ -11,7 +10,6 @@
 #include "nsISmtpService.h"
 #include "nsIPrefLocalizedString.h"
 #include "nsIPrefService.h"
-#include "nsArrayUtils.h"
 #include "nsISupportsPrimitives.h"
 #include "nsMsgBaseCID.h"
 #include "nsMsgCompCID.h"
@@ -23,22 +21,27 @@
 #include "mozilla/ArrayUtils.h"
 #include "nsIFile.h"
 
+#include "nsAbBaseCID.h"
+#include "nsIAbManager.h"
+#include "nsIAbDirectory.h"
+#include "../../../../mailnews/import/src/MorkImport.h"
+
 // Mail specific folder paths
-#define MAIL_DIR_50_NAME NS_LITERAL_STRING("Mail")
-#define IMAP_MAIL_DIR_50_NAME NS_LITERAL_STRING("ImapMail")
-#define NEWS_DIR_50_NAME NS_LITERAL_STRING("News")
+#define MAIL_DIR_50_NAME u"Mail"_ns
+#define IMAP_MAIL_DIR_50_NAME u"ImapMail"_ns
+#define NEWS_DIR_50_NAME u"News"_ns
 
 ///////////////////////////////////////////////////////////////////////////////
 // nsSeamonkeyProfileMigrator
-#define FILE_NAME_JUNKTRAINING NS_LITERAL_STRING("training.dat")
-#define FILE_NAME_PERSONALDICTIONARY NS_LITERAL_STRING("persdict.dat")
-#define FILE_NAME_PERSONAL_ADDRESSBOOK NS_LITERAL_STRING("abook.mab")
-#define FILE_NAME_MAILVIEWS NS_LITERAL_STRING("mailviews.dat")
-#define FILE_NAME_CERT9DB NS_LITERAL_STRING("cert9.db")
-#define FILE_NAME_KEY4DB NS_LITERAL_STRING("key4.db")
-#define FILE_NAME_SECMODDB NS_LITERAL_STRING("secmod.db")
-#define FILE_NAME_PREFS NS_LITERAL_STRING("prefs.js")
-#define FILE_NAME_USER_PREFS NS_LITERAL_STRING("user.js")
+#define FILE_NAME_JUNKTRAINING u"training.dat"_ns
+#define FILE_NAME_PERSONALDICTIONARY u"persdict.dat"_ns
+#define FILE_NAME_PERSONAL_ADDRESSBOOK u"abook.mab"_ns
+#define FILE_NAME_MAILVIEWS u"mailviews.dat"_ns
+#define FILE_NAME_CERT9DB u"cert9.db"_ns
+#define FILE_NAME_KEY4DB u"key4.db"_ns
+#define FILE_NAME_SECMODDB u"secmod.db"_ns
+#define FILE_NAME_PREFS u"prefs.js"_ns
+#define FILE_NAME_USER_PREFS u"user.js"_ns
 
 struct PrefBranchStruct {
   char* prefName;
@@ -243,9 +246,9 @@ nsresult nsSeamonkeyProfileMigrator::FillProfileDataFromSeamonkeyRegistry() {
   NS_ENSURE_TRUE(newSeamonkeyData, NS_ERROR_FAILURE);
 
 #ifdef EXTRA_PREPEND
-  newSeamonkeyData->Append(NS_LITERAL_STRING(EXTRA_PREPEND));
+  newSeamonkeyData->Append(NS_LITERAL_STRING_FROM_CSTRING(EXTRA_PREPEND));
 #endif
-  newSeamonkeyData->Append(NS_LITERAL_STRING(NEW_FOLDER));
+  newSeamonkeyData->Append(NS_LITERAL_STRING_FROM_CSTRING(NEW_FOLDER));
 
   nsresult rv = GetProfileDataFromProfilesIni(newSeamonkeyData, mProfileNames,
                                               mProfileLocations);
@@ -266,8 +269,6 @@ static nsSeamonkeyProfileMigrator::PrefTransform gTransforms[] = {
     MAKESAMETYPEPREFTRANSFORM("mail.forward_message_mode", Int),
     MAKESAMETYPEPREFTRANSFORM("mail.SpellCheckBeforeSend", Bool),
     MAKESAMETYPEPREFTRANSFORM("mail.warn_on_send_accel_key", Bool),
-    MAKESAMETYPEPREFTRANSFORM("mailnews.html_domains", String),
-    MAKESAMETYPEPREFTRANSFORM("mailnews.plaintext_domains", String),
     MAKESAMETYPEPREFTRANSFORM("mailnews.headers.showUserAgent", Bool),
     MAKESAMETYPEPREFTRANSFORM("mailnews.headers.showOrganization", Bool),
     MAKESAMETYPEPREFTRANSFORM("mail.biff.play_sound", Bool),
@@ -326,8 +327,9 @@ nsresult nsSeamonkeyProfileMigrator::TransformPreferences(
 
   static const char* branchNames[] = {
       // Keep the three below first, or change the indexes below
-      "mail.identity.",   "mail.server.",     "ldap_2.",       "mail.account.",
-      "mail.smtpserver.", "mailnews.labels.", "mailnews.tags."};
+      "mail.identity.", "mail.server.",     "ldap_2.servers.",
+      "mail.account.",  "mail.smtpserver.", "mailnews.labels.",
+      "mailnews.tags."};
 
   // read in the various pref branch trees for accounts, identities, servers,
   // etc.
@@ -346,7 +348,7 @@ nsresult nsSeamonkeyProfileMigrator::TransformPreferences(
   // the new prefs.js
   CopyMailFolders(branches[1], psvc);
 
-  CopyAddressBookDirectories(branches[2], psvc);
+  TransformAddressbooksForImport(psvc, branches[2], true);
 
   // Now that we have all the pref data in memory, load the target pref file,
   // and write it back out.
@@ -368,95 +370,6 @@ nsresult nsSeamonkeyProfileMigrator::TransformPreferences(
   return NS_OK;
 }
 
-/**
- * Copy .mab (Mork files) from Seamonkey profile to Thunderbird profile. The
- * restarting after migration will convert .mab to .sqlite, so we don't need to
- * read and convert the .mab here.
- */
-nsresult nsSeamonkeyProfileMigrator::CopyAddressBookDirectories(
-    PBStructArray& aLdapServers, nsIPrefService* aPrefService) {
-  // each server has a pref ending with .filename. The value of that pref points
-  // to a profile which we need to migrate.
-  nsAutoString index;
-  index.AppendInt(nsIMailProfileMigrator::ADDRESSBOOK_DATA);
-  NOTIFY_OBSERVERS(MIGRATION_ITEMBEFOREMIGRATE, index.get());
-
-  for (auto pref : aLdapServers) {
-    nsDependentCString prefName(pref->prefName);
-
-    if (StringEndsWith(prefName, nsDependentCString(".filename"))) {
-      nsCString fileName(pref->stringValue);
-      nsCString newFileName(pref->stringValue);
-      if (StringBeginsWith(prefName, nsDependentCString("pab")) ||
-          StringBeginsWith(prefName, nsDependentCString("history"))) {
-        // By default, ldap_2.servers.pab.filename is abook.mab on Seamonkey,
-        // but when read from Thunderbird, pref->prefName becomes abook.sqlite.
-        // It's because Seamonkey and Thunderbird have different default values.
-        // CopyFile won't work since abook.sqlite doesn't exist in Seamonkey's
-        // profile.
-        nsCOMPtr<nsIFile> sourceAddrbook;
-        mSourceProfile->Clone(getter_AddRefs(sourceAddrbook));
-        sourceAddrbook->Append(NS_ConvertUTF8toUTF16(fileName));
-        bool exists;
-        sourceAddrbook->Exists(&exists);
-
-        if (StringEndsWith(fileName, nsDependentCString(".sqlite"))) {
-          if (exists) {
-            // This means Seamonkey has started using SQLite for addressbook, to
-            // prevent overwriting pab.sqlite and history.sqlite, do nothing.
-            continue;
-          } else {
-            // If abook.sqlite doesn't exist in Seamonkey's profile, but
-            // abook.mab does, we copy abook.mab into Thunderbird's profile, and
-            // on restarting, abook.mab will be merged into abook.sqlite.
-            fileName.Cut(fileName.Length() - strlen("sqlite"),
-                         strlen("sqlite"));
-            fileName.Append("mab");
-            mSourceProfile->Clone(getter_AddRefs(sourceAddrbook));
-            sourceAddrbook->Append(NS_ConvertUTF8toUTF16(fileName));
-            sourceAddrbook->Exists(&exists);
-            if (exists) {
-              newFileName = fileName;
-            }
-          }
-        }
-      } else if (StringEndsWith(newFileName, nsDependentCString(".mab"))) {
-        // For user created addressbook, say "abook-1.mab" in Seamonkey, it's
-        // possible there is a "abook-1.sqlite" in Thunderbird. To avoid
-        // overwriting, we start from abook-1.sqlite, then use CreateUnique to
-        // get a unique filename, say "abook-N.sqlite", then change the file
-        // extension to get "abook-N.mab" and use it in pref->stringValue and
-        // CopyFile.
-        newFileName.Cut(newFileName.Length() - strlen("mab"), strlen("mab"));
-        newFileName.Append("sqlite");
-        nsCOMPtr<nsIFile> targetAddrbook;
-        mTargetProfile->Clone(getter_AddRefs(targetAddrbook));
-        targetAddrbook->Append(NS_ConvertUTF8toUTF16(newFileName));
-        nsresult rv =
-            targetAddrbook->CreateUnique(nsIFile::NORMAL_FILE_TYPE, 0600);
-        NS_ENSURE_SUCCESS(rv, rv);
-
-        nsString leafName;
-        targetAddrbook->GetLeafName(leafName);
-        newFileName.Assign(NS_ConvertUTF16toUTF8(leafName));
-        newFileName.Cut(newFileName.Length() - strlen("sqlite"),
-                        strlen("sqlite"));
-        newFileName.Append("mab");
-        targetAddrbook->RenameTo(mTargetProfile,
-                                 NS_ConvertUTF8toUTF16(newFileName));
-        pref->stringValue = moz_xstrdup(newFileName.get());
-      }
-
-      CopyFile(NS_ConvertUTF8toUTF16(fileName),
-               NS_ConvertUTF8toUTF16(newFileName));
-    }
-  }
-
-  NOTIFY_OBSERVERS(MIGRATION_ITEMAFTERMIGRATE, index.get());
-
-  return NS_OK;
-}
-
 nsresult nsSeamonkeyProfileMigrator::CopySignatureFiles(
     PBStructArray& aIdentities, nsIPrefService* aPrefService) {
   nsresult rv = NS_OK;
@@ -521,7 +434,7 @@ nsresult nsSeamonkeyProfileMigrator::CopyMailFolders(
     PrefBranchStruct* pref = aMailServers.ElementAt(i);
     nsDependentCString prefName(pref->prefName);
 
-    if (StringEndsWith(prefName, NS_LITERAL_CSTRING(".directory-rel"))) {
+    if (StringEndsWith(prefName, ".directory-rel"_ns)) {
       // When the directories are modified below, we may change the .directory
       // pref. As we don't have a pref branch to modify at this stage and set
       // up the relative folders properly, we'll just remove all the
@@ -685,7 +598,7 @@ nsresult nsSeamonkeyProfileMigrator::ImportPreferences(uint16_t aItems) {
   // base later.
   nsCOMPtr<nsIFile> targetPrefsFile;
   mTargetProfile->Clone(getter_AddRefs(targetPrefsFile));
-  targetPrefsFile->Append(FILE_NAME_PREFS + NS_LITERAL_STRING(".orig"));
+  targetPrefsFile->Append(FILE_NAME_PREFS + u".orig"_ns);
   rv = psvc->SavePrefFile(targetPrefsFile);
   NS_ENSURE_SUCCESS(rv, rv);
 
@@ -744,10 +657,8 @@ nsresult nsSeamonkeyProfileMigrator::ImportPreferences(uint16_t aItems) {
   WriteBranch(branchNames[1], psvc, sourceBranches[1], false);
   CopyMailFolders(sourceBranches[1], psvc);
 
-  TransformAddressbooksForImport(psvc, sourceBranches[6]);
-  // CopyAddressBookDirectories requires ldap_2.servers.newName branch exists.
-  WriteBranch(branchNames[6], psvc, sourceBranches[6], false);
-  CopyAddressBookDirectories(sourceBranches[6], psvc);
+  // TransformAddressbooksForImport writes the branch and migrates the files.
+  TransformAddressbooksForImport(psvc, sourceBranches[6], false);
 
   for (uint32_t i = 0; i < MOZ_ARRAY_LENGTH(branchNames); i++)
     WriteBranch(branchNames[i], psvc, sourceBranches[i]);
@@ -788,7 +699,7 @@ nsresult nsSeamonkeyProfileMigrator::TransformIdentitiesForImport(
       NS_ENSURE_SUCCESS(rv, rv);
 
       identity->GetKey(newKey);
-      keyHashTable.Put(key, newKey);
+      keyHashTable.InsertOrUpdate(key, newKey);
     }
 
     // Replace the prefName with the new key.
@@ -811,7 +722,7 @@ nsresult nsSeamonkeyProfileMigrator::TransformMailAccountsForImport(
     nsIMsgAccountManager* accountManager,
     PrefKeyHashTable& identityKeyHashTable,
     PrefKeyHashTable& serverKeyHashTable) {
-  nsDataHashtable<nsCStringHashKey, nsCString> keyHashTable;
+  nsTHashMap<nsCStringHashKey, nsCString> keyHashTable;
   nsTArray<nsCString> newKeys;
 
   for (auto pref : aAccounts) {
@@ -840,7 +751,7 @@ nsresult nsSeamonkeyProfileMigrator::TransformMailAccountsForImport(
     if (!keyHashTable.Get(key, &newKey)) {
       accountManager->GetUniqueAccountKey(newKey);
       newKeys.AppendElement(newKey);
-      keyHashTable.Put(key, newKey);
+      keyHashTable.InsertOrUpdate(key, newKey);
     }
 
     // Replace the prefName with the new key.
@@ -902,7 +813,7 @@ nsresult nsSeamonkeyProfileMigrator::TransformMailServersForImport(
         accountManager->GetUniqueServerKey(newKey);
       } while (newKeys.Contains(newKey));
       newKeys.AppendElement(newKey);
-      keyHashTable.Put(key, newKey);
+      keyHashTable.InsertOrUpdate(key, newKey);
     }
 
     prefName.Assign(moz_xstrdup(newKey.get()));
@@ -962,7 +873,7 @@ nsresult nsSeamonkeyProfileMigrator::TransformSmtpServersForImport(
       server->GetKey(&str);
       newKey.Assign(str);
       newKeys.AppendElement(newKey);
-      keyHashTable.Put(key, newKey);
+      keyHashTable.InsertOrUpdate(key, newKey);
     }
 
     // Replace the prefName with the new key.
@@ -981,44 +892,50 @@ nsresult nsSeamonkeyProfileMigrator::TransformSmtpServersForImport(
  * Transform ldap_2.servers branch.
  */
 nsresult nsSeamonkeyProfileMigrator::TransformAddressbooksForImport(
-    nsIPrefService* aPrefService, PBStructArray& aAddressbooks) {
-  nsDataHashtable<nsCStringHashKey, nsCString> keyHashTable;
+    nsIPrefService* aPrefService, PBStructArray& aAddressbooks, bool aReplace) {
+  nsTHashMap<nsCStringHashKey, nsCString> keyHashTable;
+  nsTHashMap<nsCStringHashKey, nsCString> pendingMigrations;
   nsTArray<nsCString> newKeys;
+  nsresult rv;
+
+  nsCOMPtr<nsIPrefBranch> branch;
+  rv = aPrefService->GetBranch("ldap_2.servers.", getter_AddRefs(branch));
+  NS_ENSURE_SUCCESS(rv, rv);
 
   for (auto pref : aAddressbooks) {
     nsDependentCString prefName(pref->prefName);
     nsTArray<nsCString> keys;
     ParseString(prefName, '.', keys);
     auto key = keys[0];
-    if (key == "default" || key == "pab" || key == "history") {
+    if (key == "default") {
       continue;
     }
 
-    // For every addressbook, create a new one to avoid conflicts.
     nsCString newKey;
-    if (!keyHashTable.Get(key, &newKey)) {
-      nsCOMPtr<nsIPrefBranch> branch;
-      nsresult rv =
-          aPrefService->GetBranch("ldap_2.servers.", getter_AddRefs(branch));
-      NS_ENSURE_SUCCESS(rv, rv);
-      uint32_t uniqueCount = 0;
+    if (aReplace) {
+      newKey.Assign(key);
+    } else {
+      // For every addressbook, create a new one to avoid conflicts.
+      if (!keyHashTable.Get(key, &newKey)) {
+        uint32_t uniqueCount = 0;
 
-      while (true) {
-        nsAutoCString filenameKey;
-        nsAutoCString filename;
-        filenameKey.Assign(key);
-        filenameKey.AppendInt(++uniqueCount);
-        filenameKey.AppendLiteral(".filename");
-        nsresult rv = branch->GetCharPref(filenameKey.get(), filename);
-        if (NS_FAILED(rv)) {
-          newKey.Assign(key);
-          newKey.AppendInt(uniqueCount);
-          (void)branch->SetCharPref(filenameKey.get(),
-                                    nsDependentCString("placeholder"));
-          break;
+        while (true) {
+          nsAutoCString filenameKey;
+          nsAutoCString filename;
+          filenameKey.Assign(key);
+          filenameKey.AppendInt(++uniqueCount);
+          filenameKey.AppendLiteral(".filename");
+          nsresult rv = branch->GetCharPref(filenameKey.get(), filename);
+          if (NS_FAILED(rv)) {
+            newKey.Assign(key);
+            newKey.AppendInt(uniqueCount);
+            (void)branch->SetCharPref(filenameKey.get(),
+                                      nsDependentCString("placeholder"));
+            break;
+          }
         }
+        keyHashTable.InsertOrUpdate(key, newKey);
       }
-      keyHashTable.Put(key, newKey);
     }
 
     // Replace the prefName with the new key.
@@ -1026,10 +943,120 @@ nsresult nsSeamonkeyProfileMigrator::TransformAddressbooksForImport(
     for (uint32_t j = 1; j < keys.Length(); j++) {
       prefName.Append('.');
       prefName.Append(keys[j]);
+
+      if (j == 1) {
+        if (keys[j].Equals("dirType")) {
+          // Make sure we have the right type of directory.
+          pref->intValue = 101;
+        } else if (!aReplace && keys[j].Equals("description") &&
+                   !strcmp(pref->stringValue,
+                           "chrome://messenger/locale/addressbook/"
+                           "addressBook.properties")) {
+          // We're importing the default directories, which have localized
+          // names. The names are tied to the pref's name, which we are
+          // changing, so the localization will fail. Instead, do the
+          // localization here and assign it to the directory being copied.
+          nsCOMPtr<nsIPrefLocalizedString> localizedString;
+          rv = branch->GetComplexValue(pref->prefName,
+                                       NS_GET_IID(nsIPrefLocalizedString),
+                                       getter_AddRefs(localizedString));
+          if (NS_SUCCEEDED(rv)) {
+            nsString localizedValue;
+            localizedString->GetData(localizedValue);
+            pref->stringValue =
+                moz_xstrdup(NS_ConvertUTF16toUTF8(localizedValue).get());
+          }
+        } else if (keys[j].Equals("filename")) {
+          // Update the prefs for the new filename of the directory.
+          nsCString oldFileName(pref->stringValue);
+          nsCString newFileName(pref->stringValue);
+
+          if (StringEndsWith(newFileName, nsCString("mab"))) {
+            newFileName.Cut(newFileName.Length() - strlen("mab"),
+                            strlen("mab"));
+            newFileName.Append("sqlite");
+            pref->stringValue = moz_xstrdup(newFileName.get());
+          }
+
+          if (!aReplace) {
+            // Find an unused filename in the destination directory.
+            nsCOMPtr<nsIFile> targetAddrbook;
+            mTargetProfile->Clone(getter_AddRefs(targetAddrbook));
+            targetAddrbook->Append(NS_ConvertUTF8toUTF16(newFileName));
+            nsresult rv =
+                targetAddrbook->CreateUnique(nsIFile::NORMAL_FILE_TYPE, 0600);
+            NS_ENSURE_SUCCESS(rv, rv);
+
+            nsString leafName;
+            targetAddrbook->GetLeafName(leafName);
+
+            pref->stringValue =
+                moz_xstrdup(NS_ConvertUTF16toUTF8(leafName).get());
+          }
+
+          if (StringEndsWith(oldFileName, nsCString("sqlite"))) {
+            nsCOMPtr<nsIFile> oldFile;
+            mSourceProfile->Clone(getter_AddRefs(oldFile));
+            oldFile->Append(NS_ConvertUTF8toUTF16(oldFileName));
+            bool exists = false;
+            oldFile->Exists(&exists);
+            if (exists) {
+              // The source directory already has SQLite directories.
+              // Just copy them.
+              CopyFile(NS_ConvertUTF8toUTF16(oldFileName),
+                       NS_ConvertUTF8toUTF16(newFileName));
+              continue;
+            }
+
+            oldFileName.Cut(oldFileName.Length() - strlen("sqlite"),
+                            strlen("sqlite"));
+            oldFileName.Append("mab");
+          }
+
+          // Store the directories to be migrated for later.
+          pendingMigrations.InsertOrUpdate(newKey, oldFileName);
+        }
+      }
     }
     pref->prefName = moz_xstrdup(prefName.get());
   }
 
+  // Write out the preferences and ask the address book manager to reload.
+  // This initializes the directories using the new prefs we've just set up.
+  WriteBranch("ldap_2.servers.", aPrefService, aAddressbooks, false);
+  NOTIFY_OBSERVERS("addrbook-reload", nullptr);
+
+  // Do the migration.
+  for (auto iter = pendingMigrations.Iter(); !iter.Done(); iter.Next()) {
+    nsCString dirPrefId = "ldap_2.servers."_ns;
+    dirPrefId.Append(iter.Key());
+    MigrateMABFile(dirPrefId, iter.UserData());
+  }
+
+  return NS_OK;
+}
+
+nsresult nsSeamonkeyProfileMigrator::MigrateMABFile(
+    const nsCString& aDirPrefId, const nsCString& aSourceFileName) {
+  nsCOMPtr<nsIFile> sourceFile;
+  mSourceProfile->Clone(getter_AddRefs(sourceFile));
+
+  sourceFile->Append(NS_ConvertUTF8toUTF16(aSourceFileName));
+  bool exists = false;
+  sourceFile->Exists(&exists);
+  if (!exists) return NS_OK;
+
+  nsresult rv;
+
+  nsCOMPtr<nsIAbManager> abManager(do_GetService(NS_ABMANAGER_CONTRACTID, &rv));
+  NS_ENSURE_SUCCESS(rv, rv);
+
+  nsCOMPtr<nsIAbDirectory> directory;
+  rv = abManager->GetDirectoryFromId(aDirPrefId, getter_AddRefs(directory));
+  NS_ENSURE_SUCCESS(rv, NS_OK);
+
+  rv = ReadMABToDirectory(sourceFile, directory);
+
   return NS_OK;
 }
 
@@ -1071,8 +1098,10 @@ void nsSeamonkeyProfileMigrator::ReadBranch(const char* branchName,
             "nsNetscapeProfileMigratorBase::ReadBranch");
         break;
     }
-
-    if (NS_SUCCEEDED(rv)) aPrefs.AppendElement(prefBranch);
+    if (NS_SUCCEEDED(rv))
+      aPrefs.AppendElement(prefBranch);
+    else
+      delete prefBranch;
   }
 }
 
