@@ -6,7 +6,6 @@
 "use strict";
 
 /* import-globals-from preferences.js */
-/* import-globals-from subdialogs.js */
 
 var { InlineSpellChecker } = ChromeUtils.import(
   "resource://gre/modules/InlineSpellChecker.jsm"
@@ -16,6 +15,10 @@ var { InlineSpellChecker } = ChromeUtils.import(
 var { cloudFileAccounts } = ChromeUtils.import(
   "resource:///modules/cloudFileAccounts.jsm"
 );
+var { E10SUtils } = ChromeUtils.import("resource://gre/modules/E10SUtils.jsm");
+var { ExtensionParent } = ChromeUtils.import(
+  "resource://gre/modules/ExtensionParent.jsm"
+);
 var { Services } = ChromeUtils.import("resource://gre/modules/Services.jsm");
 
 Preferences.addAll([
@@ -42,16 +45,17 @@ Preferences.addAll([
   { id: "mail.compose.default_to_paragraph", type: "bool" },
   { id: "mail.compose.big_attachments.notify", type: "bool" },
   { id: "mail.compose.big_attachments.threshold_kb", type: "int" },
+  { id: "mail.default_send_format", type: "int" },
+  { id: "mail.compose.add_link_preview", type: "bool" },
 ]);
 
 var gComposePane = {
   mSpellChecker: null,
-  mDictCount: 0,
 
   init() {
     this.enableAutocomplete();
 
-    this.initLanguageMenu();
+    this.initLanguages();
 
     this.populateFonts();
 
@@ -79,14 +83,10 @@ var gComposePane = {
     gCloudFile.init();
   },
 
-  sendOptionsDialog() {
-    gSubDialog.open("chrome://messenger/content/preferences/sendoptions.xhtml");
-  },
-
   attachmentReminderOptionsDialog() {
     gSubDialog.open(
       "chrome://messenger/content/preferences/attachmentReminder.xhtml",
-      "resizable=no"
+      { features: "resizable=no" }
     );
   },
 
@@ -201,44 +201,54 @@ var gComposePane = {
     }
   },
 
-  initLanguageMenu() {
-    var languageMenuList = document.getElementById("languageMenuList");
+  async initLanguages() {
+    let languageList = document.getElementById("dictionaryList");
     this.mSpellChecker = Cc["@mozilla.org/spellchecker/engine;1"].getService(
       Ci.mozISpellCheckingEngine
     );
 
-    // Get the list of dictionaries from
-    // the spellchecker.
+    // Get the list of dictionaries from the spellchecker.
 
-    var dictList = this.mSpellChecker.getDictionaryList();
-    var count = dictList.length;
+    let dictList = this.mSpellChecker.getDictionaryList();
 
-    // if we don't have any dictionaries installed, disable the menu list
-    languageMenuList.disabled = !count;
-
-    // If dictionary count hasn't changed then no need to update the menu.
-    if (this.mDictCount == count) {
-      return;
-    }
-
-    // Store current dictionary count.
-    this.mDictCount = count;
-
-    var inlineSpellChecker = new InlineSpellChecker();
-    var sortedList = inlineSpellChecker.sortDictionaryList(dictList);
-
-    // Remove any languages from the list.
-    languageMenuList.removeAllItems();
-
-    // append the dictionaries to the menu list...
-    for (var i = 0; i < count; i++) {
-      languageMenuList.appendItem(
-        sortedList[i].displayName,
-        sortedList[i].localeCode
-      );
-    }
-
-    languageMenuList.setInitialSelection();
+    // HACK: calling sortDictionaryList may fail the first time due to
+    // synchronous loading of the .ftl files. If we load the files and wait
+    // for a known value asynchronously, no such failure will happen.
+    await new Localization([
+      "toolkit/intl/languageNames.ftl",
+      "toolkit/intl/regionNames.ftl",
+    ]).formatValue("language-name-en");
+    let sortedList = new InlineSpellChecker().sortDictionaryList(dictList);
+    let activeDictionaries = Services.prefs
+      .getCharPref("spellchecker.dictionary")
+      .split(",");
+    let template = document.getElementById("dictionaryListItem");
+    languageList.replaceChildren(
+      ...sortedList.map(({ displayName, localeCode }) => {
+        let item = template.content.cloneNode(true).firstElementChild;
+        item.querySelector(".checkbox-label").textContent = displayName;
+        let input = item.querySelector("input");
+        input.setAttribute("value", localeCode);
+        input.addEventListener("change", event => {
+          let language = event.target.value;
+          let dicts = Services.prefs
+            .getCharPref("spellchecker.dictionary")
+            .split(",")
+            .filter(Boolean);
+          if (!event.target.checked) {
+            dicts = dicts.filter(item => item != language);
+          } else {
+            dicts.push(language);
+          }
+          Services.prefs.setCharPref(
+            "spellchecker.dictionary",
+            dicts.join(",")
+          );
+        });
+        input.checked = activeDictionaries.includes(localeCode);
+        return item;
+      })
+    );
   },
 
   populateFonts() {
@@ -327,7 +337,6 @@ var gCloudFile = {
   _buttonContainer: null,
   _listContainer: null,
   _settings: null,
-  _settingsDeck: null,
   _tabpanel: null,
   _settingsPanelWrap: null,
   _defaultPanel: null,
@@ -350,7 +359,6 @@ var gCloudFile = {
     this._removeAccountButton = document.getElementById(
       "removeCloudFileAccount"
     );
-    this._settingsDeck = document.getElementById("cloudFileSettingsDeck");
     this._defaultPanel = document.getElementById("cloudFileDefaultPanel");
     this._settingsPanelWrap = document.getElementById(
       "cloudFileSettingsWrapper"
@@ -385,8 +393,7 @@ var gCloudFile = {
   _onAccountConfigured(event, account) {
     for (let item of this._list.children) {
       if (item.value == account.accountKey) {
-        item.querySelector("image.configuredWarning").hidden =
-          account.configured;
+        item.querySelector(".configuredWarning").hidden = account.configured;
       }
     }
   },
@@ -411,7 +418,8 @@ var gCloudFile = {
       // If the provider is unregistered, getAccount returns null.
       if (!cloudFileAccounts.getAccount(item.value)) {
         if (item.hasAttribute("selected")) {
-          this._settingsDeck.selectedPanel = this._defaultPanel;
+          this._defaultPanel.hidden = false;
+          this._settingsPanelWrap.hidden = true;
           if (this._settings) {
             this._settings.remove();
           }
@@ -441,17 +449,16 @@ var gCloudFile = {
 
   makeRichListItemForAccount(aAccount) {
     let rli = document.createXULElement("richlistitem");
-    rli.value = aAccount.accountKey;
     rli.setAttribute("align", "center");
     rli.classList.add("cloudfileAccount", "input-container");
     rli.setAttribute("value", aAccount.accountKey);
 
+    let icon = document.createElement("img");
+    icon.classList.add("typeIcon");
     if (aAccount.iconURL) {
-      rli.style.listStyleImage = "url('" + aAccount.iconURL + "')";
+      icon.setAttribute("src", aAccount.iconURL);
     }
-
-    let icon = document.createXULElement("image");
-    icon.setAttribute("class", "typeIcon");
+    icon.setAttribute("alt", "");
     rli.appendChild(icon);
 
     let label = document.createXULElement("label");
@@ -471,11 +478,12 @@ var gCloudFile = {
     input.addEventListener("keypress", this);
     rli.appendChild(input);
 
-    let warningIcon = document.createXULElement("image");
+    let warningIcon = document.createElement("img");
     warningIcon.setAttribute("class", "configuredWarning typeIcon");
     warningIcon.setAttribute("src", "chrome://global/skin/icons/warning.svg");
+    // "title" provides the accessible name, not "alt".
     warningIcon.setAttribute(
-      "tooltiptext",
+      "title",
       this._strings.GetStringFromName("notConfiguredYet")
     );
     if (aAccount.configured) {
@@ -552,7 +560,8 @@ var gCloudFile = {
     let selection = this._list.selectedItem;
     this._removeAccountButton.disabled = !selection;
     if (!selection) {
-      this._settingsDeck.selectedPanel = this._defaultPanel;
+      this._defaultPanel.hidden = false;
+      this._settingsPanelWrap.hidden = true;
       if (this._settings) {
         this._settings.remove();
       }
@@ -564,27 +573,54 @@ var gCloudFile = {
 
   _showAccountInfo(aAccountKey) {
     let account = cloudFileAccounts.getAccount(aAccountKey);
-    this._settingsDeck.selectedPanel = this._settingsPanelWrap;
+    this._defaultPanel.hidden = true;
+    this._settingsPanelWrap.hidden = false;
 
     let url = account.managementURL + `?accountId=${account.accountKey}`;
 
-    let iframe = document.createXULElement("iframe");
-    iframe.setAttribute("flex", "1");
-    // allows keeping dialog background color without hoops
-    iframe.setAttribute("transparent", "true");
+    let browser = document.createXULElement("browser");
+    browser.setAttribute("type", "content");
+    browser.setAttribute("remote", "true");
+    browser.setAttribute("remoteType", E10SUtils.EXTENSION_REMOTE_TYPE);
+    browser.setAttribute("forcemessagemanager", "true");
+    if (account.extension) {
+      browser.setAttribute(
+        "initialBrowsingContextGroupId",
+        account.extension.policy.browsingContextGroupId
+      );
+    }
+    browser.setAttribute("disableglobalhistory", "true");
+    browser.setAttribute("messagemanagergroup", "webext-browsers");
+    browser.setAttribute("autocompletepopup", "PopupAutoComplete");
+    browser.setAttribute("selectmenulist", "ContentSelectDropdown");
 
-    let type = url.startsWith("chrome:") ? "chrome" : "content";
-    iframe.setAttribute("type", type);
-    iframe.setAttribute("src", url);
+    browser.setAttribute("flex", "1");
+    // Allows keeping dialog background color without hoops.
+    browser.setAttribute("transparent", "true");
 
-    // If we have a past iframe, we replace it. Else append
-    // to the wrapper.
+    // If we have a past browser, we replace it. Else append to the wrapper.
     if (this._settings) {
       this._settings.remove();
     }
 
-    this._settingsPanelWrap.appendChild(iframe);
-    this._settings = iframe;
+    this._settingsPanelWrap.appendChild(browser);
+    this._settings = browser;
+
+    ExtensionParent.apiManager.emit("extension-browser-inserted", browser);
+    browser.messageManager.loadFrameScript(
+      "chrome://extensions/content/ext-browser-content.js",
+      false,
+      true
+    );
+
+    let options = account.browserStyle
+      ? { stylesheets: ExtensionParent.extensionStylesheets }
+      : {};
+    browser.messageManager.sendAsyncMessage("Extension:InitBrowser", options);
+
+    browser.loadURI(url, {
+      triggeringPrincipal: Services.scriptSecurityManager.getSystemPrincipal(),
+    });
   },
 
   onListOverflow() {
@@ -633,7 +669,8 @@ var gCloudFile = {
         "richlistitem[value='" + accountKey + "']"
       );
       rli.remove();
-      this._settingsDeck.selectedPanel = this._defaultPanel;
+      this._defaultPanel.hidden = false;
+      this._settingsPanelWrap.hidden = true;
       if (this._settings) {
         this._settings.remove();
       }
