@@ -8,9 +8,8 @@
 /* import-globals-from downloads.js */
 /* import-globals-from privacy.js */
 /* import-globals-from chat.js */
-/* import-globals-from subdialogs.js */
 /* import-globals-from findInPage.js */
-/* globals gLightningPane */
+/* globals gCalendarPane */
 
 var { Services } = ChromeUtils.import("resource://gre/modules/Services.jsm");
 var { AppConstants } = ChromeUtils.import(
@@ -25,6 +24,8 @@ var { ExtensionSupport } = ChromeUtils.import(
 var { calendarDeactivator } = ChromeUtils.import(
   "resource:///modules/calendar/calCalendarDeactivator.jsm"
 );
+var { UIDensity } = ChromeUtils.import("resource:///modules/UIDensity.jsm");
+var { UIFontSize } = ChromeUtils.import("resource:///modules/UIFontSize.jsm");
 
 var paneDeck = document.getElementById("paneDeck");
 var defaultPane = "paneGeneral";
@@ -35,6 +36,55 @@ ChromeUtils.defineModuleGetter(
   "resource://gre/modules/AddonManager.jsm"
 );
 
+XPCOMUtils.defineLazyGetter(this, "gSubDialog", function() {
+  const { SubDialogManager } = ChromeUtils.import(
+    "resource://gre/modules/SubDialog.jsm"
+  );
+  return new SubDialogManager({
+    dialogStack: document.getElementById("dialogStack"),
+    dialogTemplate: document.getElementById("dialogTemplate"),
+    dialogOptions: {
+      styleSheets: [
+        "chrome://messenger/skin/preferences/dialog.css",
+        "chrome://messenger/skin/preferences/preferences.css",
+      ],
+      resizeCallback: ({ title, frame }) => {
+        UIFontSize.registerWindow(frame.contentWindow);
+
+        // Search within main document and highlight matched keyword.
+        gSearchResultsPane.searchWithinNode(title, gSearchResultsPane.query);
+
+        // Search within sub-dialog document and highlight matched keyword.
+        gSearchResultsPane.searchWithinNode(
+          frame.contentDocument.firstElementChild,
+          gSearchResultsPane.query
+        );
+
+        // Creating tooltips for all the instances found
+        for (let node of gSearchResultsPane.listSearchTooltips) {
+          if (!node.tooltipNode) {
+            gSearchResultsPane.createSearchTooltip(
+              node,
+              gSearchResultsPane.query
+            );
+          }
+        }
+
+        // Resize the dialog to fit the content with edited font size.
+        requestAnimationFrame(() => {
+          let dialogs = frame.ownerGlobal.gSubDialog._dialogs;
+          let dialog = dialogs.find(
+            d => d._frame.contentDocument == frame.contentDocument
+          );
+          if (dialog) {
+            UIFontSize.resizeSubDialog(dialog);
+          }
+        });
+      },
+    },
+  });
+});
+
 document.addEventListener("DOMContentLoaded", init, { once: true });
 
 var gCategoryInits = new Map();
@@ -69,9 +119,9 @@ function register_module(categoryName, categoryObject) {
         document.l10n.resumeObserving();
 
         // Asks Preferences to update the attribute value of the entire
-        // document again (this can be simplified if we could seperate the
+        // document again (this can be simplified if we could separate the
         // preferences of each pane.)
-        Preferences.updateAllElements();
+        Preferences.queueUpdateOfAllElements();
       }
       categoryObject.init();
       this.inited = true;
@@ -82,11 +132,10 @@ function register_module(categoryName, categoryObject) {
 function init() {
   Preferences.forceEnableInstantApply();
 
-  gSubDialog.init();
   register_module("paneGeneral", gGeneralPane);
   register_module("paneCompose", gComposePane);
   register_module("panePrivacy", gPrivacyPane);
-  register_module("paneCalendar", gLightningPane);
+  register_module("paneCalendar", gCalendarPane);
   register_module("paneSearchResults", gSearchResultsPane);
   if (Services.prefs.getBoolPref("mail.chat.enabled")) {
     register_module("paneChat", gChatPane);
@@ -123,10 +172,13 @@ function init() {
   window.addEventListener("hashchange", onHashChange);
   let lastSelected = Services.xulStore.getValue(
     "about:preferences",
-    "MailPreferences",
+    "paneDeck",
     "lastSelected"
   );
   gotoPref(lastSelected);
+
+  UIDensity.registerWindow(window);
+  UIFontSize.registerWindow(window);
 }
 
 function onHashChange() {
@@ -230,10 +282,10 @@ async function gotoPref(aCategory) {
 
   document.dispatchEvent(new CustomEvent("paneSelected", { bubbles: true }));
   document.getElementById("preferencesContainer").scrollTo(0, 0);
-  document.documentElement.setAttribute("lastSelected", category);
+  document.getElementById("paneDeck").setAttribute("lastSelected", category);
   Services.xulStore.setValue(
     "about:preferences",
-    "MailPreferences",
+    "paneDeck",
     "lastSelected",
     category
   );
