@@ -13,8 +13,6 @@ var { ExtensionParent } = ChromeUtils.import(
   "resource://gre/modules/ExtensionParent.jsm"
 );
 
-var gPrefTab = null;
-
 /**
  * A tab to show Preferences.
  */
@@ -43,31 +41,26 @@ var preferencesTabType = {
     },
   },
 
-  initialize() {
-    let tabmail = document.getElementById("tabmail");
-    tabmail.registerTabType(this);
-  },
-
   shouldSwitchTo(aArgs) {
-    if (!gPrefTab) {
+    if (!this.tab) {
       return -1;
     }
-    gPrefTab.browser.contentWindow.selectPrefPane(
+    this.tab.browser.contentWindow.selectPrefPane(
       aArgs.paneID,
       aArgs.scrollPaneTo,
       aArgs.otherArgs
     );
-    return document.getElementById("tabmail").tabInfo.indexOf(gPrefTab);
+    return document.getElementById("tabmail").tabInfo.indexOf(this.tab);
   },
 
   closeTab(aTab) {
-    gPrefTab = null;
+    this.tab = null;
   },
 
   openTab(aTab, aArgs) {
-    if (!("contentPage" in aArgs)) {
-      throw new Error("contentPage must be specified");
-    }
+    aTab.tabNode.setIcon(
+      "chrome://messenger/skin/icons/new/compact/settings.svg"
+    );
 
     // First clone the page and set up the basics.
     let clone = document
@@ -86,18 +79,20 @@ var preferencesTabType = {
       "id",
       "preferencesTabBrowser" + this.lastBrowserId
     );
+    aTab.browser.setAttribute("autocompletepopup", "PopupAutoComplete");
     aTab.browser.addEventListener("DOMLinkAdded", DOMLinkHandler);
 
-    aTab.findbar = aTab.panel.querySelector("findbar");
+    aTab.findbar = document.createXULElement("findbar");
     aTab.findbar.setAttribute(
       "browserid",
       "preferencesTabBrowser" + this.lastBrowserId
     );
+    aTab.panel.appendChild(aTab.findbar);
 
     // Default to reload being disabled.
     aTab.reloadEnabled = false;
 
-    aTab.url = aArgs.contentPage;
+    aTab.url = "about:preferences";
     aTab.paneID = aArgs.paneID;
     aTab.scrollPaneTo = aArgs.scrollPaneTo;
     aTab.otherArgs = aArgs.otherArgs;
@@ -140,9 +135,9 @@ var preferencesTabType = {
       triggeringPrincipal: Services.scriptSecurityManager.getSystemPrincipal(),
       postData: aArgs.postData || null,
     };
-    aTab.browser.loadURI(aArgs.contentPage, params);
+    aTab.browser.loadURI("about:preferences", params);
 
-    gPrefTab = aTab;
+    this.tab = aTab;
     this.lastBrowserId++;
   },
 
@@ -152,7 +147,6 @@ var preferencesTabType = {
     }
 
     return {
-      tabURI: aTab.url,
       paneID: aTab.paneID,
       scrollPaneTo: aTab.scrollPaneTo,
       otherArgs: aTab.otherArgs,
@@ -161,7 +155,6 @@ var preferencesTabType = {
 
   restoreTab(aTabmail, aPersistedState) {
     aTabmail.openTab("preferencesTab", {
-      contentPage: aPersistedState.tabURI,
       paneID: aPersistedState.paneID,
       scrollPaneTo: aPersistedState.scrollPaneTo,
       otherArgs: aPersistedState.otherArgs,
