@@ -7,17 +7,17 @@
 "use strict";
 
 /*globally available Thunderbird variables/object/functions: */
-/*global gMsgCompose: false, gNotification: false */
+/*global gMsgCompose: false, gComposeNotification: false */
 /*global UpdateAttachmentBucket: false, gContentChanged: true */
 /*global AddAttachments: false, AddAttachment: false, ChangeAttachmentBucketVisibility: false, GetResourceFromUri: false */
-/*global Recipients2CompFields: false, Attachments2CompFields: false, DetermineConvertibility: false, gWindowLocked: false */
+/*global Recipients2CompFields: false, Attachments2CompFields: false, gWindowLocked: false */
 /*global CommandUpdate_MsgCompose: false, gSMFields: false, setSecuritySettings: false, getCurrentAccountKey: false */
 /*global Sendlater3Composing: false, gCurrentIdentity: false, showMessageComposeSecurityStatus: false */
 /*global gSendEncrypted: true, gOptionalEncryption: true, gSendSigned: true, gSelectedTechnologyIsPGP: true */
 /*global gIsRelatedToEncryptedOriginal: true, gIsRelatedToSignedOriginal: true, gAttachMyPublicPGPKey: true */
-/*global setEncSigStatusUI: false, gEncryptedURIService: false */
-
-/* import-globals-from ../BondOpenPGP.jsm */
+/*global gEncryptSubject: true, setEncSigStatusUI: false, gEncryptedURIService: false */
+/* global updateE2eeOptions: true */
+/* global gAttachmentBucket: true */
 
 var { Services } = ChromeUtils.import("resource://gre/modules/Services.jsm");
 var { MailServices } = ChromeUtils.import(
@@ -33,33 +33,15 @@ var EnigmailFuncs = ChromeUtils.import(
 var { EnigmailLog } = ChromeUtils.import(
   "chrome://openpgp/content/modules/log.jsm"
 );
-var EnigmailPrefs = ChromeUtils.import(
-  "chrome://openpgp/content/modules/prefs.jsm"
-).EnigmailPrefs;
-var { EnigmailOS } = ChromeUtils.import(
-  "chrome://openpgp/content/modules/os.jsm"
-);
 var EnigmailArmor = ChromeUtils.import(
   "chrome://openpgp/content/modules/armor.jsm"
 ).EnigmailArmor;
-var EnigmailLocale = ChromeUtils.import(
-  "chrome://openpgp/content/modules/locale.jsm"
-).EnigmailLocale;
-var EnigmailFiles = ChromeUtils.import(
-  "chrome://openpgp/content/modules/files.jsm"
-).EnigmailFiles;
 var EnigmailData = ChromeUtils.import(
   "chrome://openpgp/content/modules/data.jsm"
 ).EnigmailData;
-var { EnigmailApp } = ChromeUtils.import(
-  "chrome://openpgp/content/modules/app.jsm"
-);
 var EnigmailDialog = ChromeUtils.import(
   "chrome://openpgp/content/modules/dialog.jsm"
 ).EnigmailDialog;
-var EnigmailTimer = ChromeUtils.import(
-  "chrome://openpgp/content/modules/timer.jsm"
-).EnigmailTimer;
 var EnigmailWindows = ChromeUtils.import(
   "chrome://openpgp/content/modules/windows.jsm"
 ).EnigmailWindows;
@@ -78,15 +60,9 @@ var EnigmailDecryption = ChromeUtils.import(
 var EnigmailEncryption = ChromeUtils.import(
   "chrome://openpgp/content/modules/encryption.jsm"
 ).EnigmailEncryption;
-var EnigmailClipboard = ChromeUtils.import(
-  "chrome://openpgp/content/modules/clipboard.jsm"
-).EnigmailClipboard;
 var EnigmailWkdLookup = ChromeUtils.import(
   "chrome://openpgp/content/modules/wkdLookup.jsm"
 ).EnigmailWkdLookup;
-var EnigmailAutocrypt = ChromeUtils.import(
-  "chrome://openpgp/content/modules/autocrypt.jsm"
-).EnigmailAutocrypt;
 var EnigmailMime = ChromeUtils.import(
   "chrome://openpgp/content/modules/mime.jsm"
 ).EnigmailMime;
@@ -99,9 +75,12 @@ var EnigmailMimeEncrypt = ChromeUtils.import(
 const { EnigmailCryptoAPI } = ChromeUtils.import(
   "chrome://openpgp/content/modules/cryptoAPI.jsm"
 );
+const { OpenPGPAlias } = ChromeUtils.import(
+  "chrome://openpgp/content/modules/OpenPGPAlias.jsm"
+);
 var { jsmime } = ChromeUtils.import("resource:///modules/jsmime.jsm");
 
-var l10n = new Localization(["messenger/openpgp/openpgp.ftl"], true);
+var l10nOpenPGP = new Localization(["messenger/openpgp/openpgp.ftl"]);
 
 // Account encryption policy values:
 // const kEncryptionPolicy_Never = 0;
@@ -150,10 +129,6 @@ Enigmail.msg = {
   /* timeout when entering something into the address field */
 
   async composeStartup() {
-    if (!BondOpenPGP.isEnabled()) {
-      return;
-    }
-
     EnigmailLog.DEBUG(
       "enigmailMsgComposeOverlay.js: Enigmail.msg.composeStartup\n"
     );
@@ -175,39 +150,7 @@ Enigmail.msg = {
       true
     );
 
-    // Relabel SMIME button and menu item
-    //var smimeButton = document.getElementById("button-security");
-    //let toolbar = document.getElementById("composeToolbar2");
-
-    /*
-    if (smimeButton) {
-      smimeButton.setAttribute("label", "S/MIME");
-      if (toolbar && toolbar.getAttribute("currentset").length === 0) {
-        // remove S/MIME button if the toolbar is displaying the default set
-        toolbar.removeChild(smimeButton);
-      }
-    }
-    */
-
-    /*
-    let numCerts = EnigmailFuncs.getNumOfX509Certs();
-    this.addrOnChangeTimeout = Math.max((numCerts - 250) * 2, 250);
-    EnigmailLog.DEBUG(`enigmailMsgComposeOverlay.js: composeStartup: numCerts=${numCerts}; setting timeout to ${this.addrOnChangeTimeout}\n`);
-    */
-
-    // TODO implement migration code for old prefs, possibly in configure.jsm
-    // Use a new pref identityEnigmailPrefsMigrated, default false.
-    // Only if we're doing this for the first time for an identity,
-    // try to read old prefs and if found, store as new prefs,
-    // then set identityEnigmailPrefsMigrated=true
-    // Enigmail account prefs that need to be handled:
-    // - defaultEncryptionPolicy
-    // - autoSendEncrypted
-    // - defaultSigningPolicy
-    // possible values to check:
-    // - Enigmail.msg.wasEnigmailAddOnInstalled()
-    // - Enigmail.msg.wasEnigmailEnabledForIdentity
-    // - this.identity.getIntAttribute("mimePreferOpenPGP") > 0
+    await OpenPGPAlias.load().catch(Cu.reportError);
 
     Enigmail.msg.composeOpen();
     //Enigmail.msg.processFinalState();
@@ -223,7 +166,7 @@ Enigmail.msg = {
     EnigmailLog.DEBUG("enigmailMsgComposeOverlay.js: Enigmail.msg.handleClick\n");
     switch (event.button) {
       case 2:
-        // do not process the event any futher
+        // do not process the event any further
         // needed on Windows to prevent displaying the context menu
         event.preventDefault();
         this.doPgpButton();
@@ -253,7 +196,7 @@ Enigmail.msg = {
       switch (key) {
         case 'sign':
           if (preferSmimeByDefault) {
-            res = (this.identity.getBoolAttribute("sign_mail"));
+            res = (this.identity.signMail);
           }
           else {
             res = (this.identity.getIntAttribute("defaultSigningPolicy") > 0);
@@ -261,7 +204,7 @@ Enigmail.msg = {
           break;
         case 'encrypt':
           if (preferSmimeByDefault) {
-            res = (this.identity.getIntAttribute("encryptionpolicy") > 0);
+            res = (this.identity.encryptionPolicy > 0);
           }
           else {
             res = (this.identity.getIntAttribute("defaultEncryptionPolicy") > 0);
@@ -274,7 +217,7 @@ Enigmail.msg = {
           res = this.identity.getBoolAttribute(key);
           break;
         case 'attachPgpKey':
-          res = this.identity.getBoolAttribute(key);
+          res = this.identity.attachPgpKey;
           break;
       }
       //EnigmailLog.DEBUG("enigmailMsgComposeOverlay.js: Enigmail.msg.getAccDefault:   "+key+"="+res+"\n");
@@ -283,10 +226,10 @@ Enigmail.msg = {
     else if (Enigmail.msg.isSmimeEnabled()) {
       switch (key) {
         case 'sign':
-          res = this.identity.getBoolAttribute("sign_mail");
+          res = this.identity.signMail;
           break;
         case 'encrypt':
-          res = (this.identity.getIntAttribute("encryptionpolicy") > 0);
+          res = (this.identity.encryptionPolicy > 0);
           break;
         default:
           res = false;
@@ -348,7 +291,7 @@ Enigmail.msg = {
 
     if (!id.getUnicharAttribute("signing_cert_name")) return false;
 
-    return id.getBoolAttribute("sign_mail");
+    return id.signMail;
   },
   */
 
@@ -485,11 +428,11 @@ Enigmail.msg = {
         case EnigmailConstants.ENIG_UNDEF:
           break;
         case EnigmailConstants.ENIG_ALWAYS:
-          gSendEncrypted = true;
+          updateE2eeOptions(true);
           break;
         case EnigmailConstants.ENIG_NEVER:
         default:
-          gSendEncrypted = false;
+          updateE2eeOptions(false);
           break;
       }
       gOptionalEncryption = false;
@@ -551,7 +494,8 @@ Enigmail.msg = {
     switch (gMsgCompose.type) {
       case CT.ForwardInline:
       case CT.ForwardAsAttachment:
-        prefix = this.getMailPref("mail.forward_subject_prefix") + ": ";
+        prefix =
+          Services.prefs.getStringPref("mail.forward_subject_prefix") + ": ";
         break;
       case CT.Draft:
         isReply = Boolean(flags & Ci.nsMsgMessageFlags.HasRe);
@@ -646,7 +590,9 @@ Enigmail.msg = {
   continueComposeOpenWithMimeTree(msgUri, msgHdr, mimeMsg) {
     let selectedElement = document.activeElement;
 
-    let msgIsDraft = !!gMsgCompose.compFields.draftId;
+    let msgIsDraft =
+      gMsgCompose.type === Ci.nsIMsgCompType.Draft ||
+      gMsgCompose.type === Ci.nsIMsgCompType.Template;
 
     if (!gSendEncrypted || msgIsDraft) {
       let useEncryptionUnlessWeHaveDraftInfo = false;
@@ -691,7 +637,7 @@ Enigmail.msg = {
             EnigmailLog.DEBUG(
               "originalMsgURI=" + gMsgCompose.originalMsgURI + "\n"
             );
-            gSendEncrypted = true;
+            updateE2eeOptions(true);
             gSelectedTechnologyIsPGP = true;
             useEncryptionUnlessWeHaveDraftInfo = false;
             usePGPUnlessWeKnowOtherwise = false;
@@ -702,7 +648,7 @@ Enigmail.msg = {
       }
 
       if (useEncryptionUnlessWeHaveDraftInfo) {
-        gSendEncrypted = true;
+        updateE2eeOptions(true);
       }
       if (gSendEncrypted && !obtainedDraftFlagsObj.value) {
         gSendSigned = true;
@@ -712,7 +658,6 @@ Enigmail.msg = {
       } else if (useSMIMEUnlessWeKnowOtherwise) {
         gSelectedTechnologyIsPGP = false;
       }
-      setEncSigStatusUI();
     }
 
     // check for attached signature files and remove them
@@ -762,69 +707,7 @@ Enigmail.msg = {
     return baseAttachment;
   },
 
-  initRadioMenu(prefName, optionIds) {
-    EnigmailLog.DEBUG(
-      "enigmailMessengerOverlay.js: Enigmail.msg.initRadioMenu: " +
-        prefName +
-        "\n"
-    );
-
-    var prefValue = EnigmailPrefs.getPref(prefName);
-
-    if (prefValue >= optionIds.length) {
-      return;
-    }
-
-    var menuItem = document.getElementById("enigmail_" + optionIds[prefValue]);
-    if (menuItem) {
-      menuItem.setAttribute("checked", "true");
-    }
-  },
-
-  /*
-  toggleAttachOwnKey: function() {
-    EnigmailLog.DEBUG("enigmailMsgComposeOverlay.js: Enigmail.msg.toggleAttachOwnKey\n");
-    EnigmailCore.getService(window); // make sure Enigmail is loaded and working
-
-    gAttachMyPublicPGPKey = !gAttachMyPublicPGPKey;
-
-    //this.setOwnKeyStatus();
-  },
-  */
-
-  /***
-   * set broadcaster to display whether the own key is attached or not
-   */
-
-  /*
-  setOwnKeyStatus: function() {
-    return;
-    let bc = document.getElementById("enigmail-bc-attach");
-    let attachIcon = document.getElementById("button-enigmail-attach");
-
-    if (this.allowAttachOwnKey() === 0) {
-      this.statusAttachOwnKey = EnigmailLocale.getString("attachOwnKeyDisabled");
-    }
-    else {
-      if (gAttachMyPublicPGPKey) {
-        bc.setAttribute("addPubkey", "true");
-        bc.setAttribute("checked", "true");
-        this.statusAttachOwnKey = EnigmailLocale.getString("attachOwnKeyYes");
-      }
-      else {
-        bc.setAttribute("addPubkey", "false");
-        bc.removeAttribute("checked");
-        this.statusAttachOwnKey = EnigmailLocale.getString("attachOwnKeyNo");
-      }
-    }
-
-    if (attachIcon)
-      attachIcon.setAttribute("tooltiptext", this.statusAttachOwnKey);
-
-  },
-  */
-
-  attachOwnKey(id) {
+  async attachOwnKey(id) {
     EnigmailLog.DEBUG(
       "enigmailMsgComposeOverlay.js: Enigmail.msg.attachOwnKey: " + id + "\n"
     );
@@ -842,10 +725,9 @@ Enigmail.msg = {
 
     if (!this.attachOwnKeyObj.attachedKey) {
       let hex = "0x" + id;
-      let myID = [hex];
-      let allIds = myID.concat(revokedIDs);
-      var attachedObj = this.extractAndAttachKey(
-        allIds,
+      var attachedObj = await this.extractAndAttachKey(
+        hex,
+        revokedIDs,
         gCurrentIdentity.email,
         true,
         true // one key plus revocations
@@ -857,37 +739,11 @@ Enigmail.msg = {
     }
   },
 
-  async attachKey() {
-    EnigmailLog.DEBUG(
-      "enigmailMsgComposeOverlay.js: Enigmail.msg.attachKey: \n"
-    );
-
-    var resultObj = {};
-    var inputObj = {};
-    inputObj.dialogHeader = await document.l10n.formatValue("keys-to-export");
-    inputObj.options = "multisel,allowexpired,nosending";
-    window.openDialog(
-      "chrome://openpgp/content/ui/enigmailKeySelection.xhtml",
-      "",
-      "dialog,modal,centerscreen,resizable",
-      inputObj,
-      resultObj
-    );
-    try {
-      if (resultObj.cancelled) {
-        return;
-      }
-      this.extractAndAttachKey(resultObj.userList, null, true, false);
-    } catch (ex) {
-      // cancel pressed -> do nothing
-    }
-  },
-
-  extractAndAttachKey(
-    uidArray,
+  async extractAndAttachKey(
+    primaryId,
+    revokedIds,
     emailForFilename,
-    warnOnError,
-    additionalKeysAreRevocations
+    warnOnError
   ) {
     EnigmailLog.DEBUG(
       "enigmailMsgComposeOverlay.js: Enigmail.msg.extractAndAttachKey: \n"
@@ -897,23 +753,7 @@ Enigmail.msg = {
       return null;
     }
 
-    var tmpDir = EnigmailFiles.getTempDir();
-    var tmpFile;
-    try {
-      tmpFile = Cc[LOCAL_FILE_CONTRACTID].createInstance(Ci.nsIFile);
-      tmpFile.initWithPath(tmpDir);
-      if (!(tmpFile.isDirectory() && tmpFile.isWritable())) {
-        document.l10n.formatValue("no-temp-dir").then(value => {
-          EnigmailDialog.alert(window, value);
-        });
-        return null;
-      }
-    } catch (ex) {
-      EnigmailLog.writeException(
-        "enigmailMsgComposeOverlay.js: Enigmail.msg.extractAndAttachKey",
-        ex
-      );
-    }
+    var tmpFile = Services.dirsvc.get("TmpD", Ci.nsIFile);
     tmpFile.append("key.asc");
     tmpFile.createUnique(Ci.nsIFile.NORMAL_FILE_TYPE, 0o600);
 
@@ -921,9 +761,10 @@ Enigmail.msg = {
     var exitCodeObj = {};
     var errorMsgObj = {};
 
-    EnigmailKeyRing.extractKey(
-      false,
-      uidArray,
+    await EnigmailKeyRing.extractPublicKeys(
+      [], // full
+      [primaryId], // reduced
+      revokedIds, // minimal
       tmpFile,
       exitCodeObj,
       errorMsgObj
@@ -936,48 +777,39 @@ Enigmail.msg = {
     }
 
     // create attachment
-    var ioServ = Services.io;
-    var tmpFileURI = ioServ.newFileURI(tmpFile);
+    var tmpFileURI = Services.io.newFileURI(tmpFile);
     var keyAttachment = Cc[
       "@mozilla.org/messengercompose/attachment;1"
     ].createInstance(Ci.nsIMsgAttachment);
     keyAttachment.url = tmpFileURI.spec;
-    if (
-      (uidArray.length == 1 || additionalKeysAreRevocations) &&
-      uidArray[0].search(/^(0x)?[a-fA-F0-9]+$/) === 0
-    ) {
-      keyAttachment.name = uidArray[0].substr(-16, 16);
-      if (keyAttachment.name.search(/^0x/) < 0) {
-        keyAttachment.name = "0x" + keyAttachment.name;
-      }
-      let withRevSuffix = "";
-      if (uidArray.length > 1 && additionalKeysAreRevocations) {
-        withRevSuffix = "_and_old_rev";
-      }
-      // let normalizedEmail = emailForFilename.replace(" ", "_");
-      // emailForFilename is currently unused
-      keyAttachment.name =
-        "OpenPGP_" + keyAttachment.name + withRevSuffix + ".asc";
-    } else {
-      keyAttachment.name = "pgpkeys.asc";
+    keyAttachment.name = primaryId.substr(-16, 16);
+    if (keyAttachment.name.search(/^0x/) < 0) {
+      keyAttachment.name = "0x" + keyAttachment.name;
     }
+    let withRevSuffix = "";
+    if (revokedIds && revokedIds.length) {
+      withRevSuffix = "_and_old_rev";
+    }
+    keyAttachment.name =
+      "OpenPGP_" + keyAttachment.name + withRevSuffix + ".asc";
     keyAttachment.temporary = true;
     keyAttachment.contentType = "application/pgp-keys";
+    keyAttachment.size = tmpFile.fileSize;
 
-    // add attachment to msg
-    this.addAttachment(keyAttachment);
+    if (
+      !gAttachmentBucket.itemChildren.find(
+        item => item.attachment.name == keyAttachment.name
+      )
+    ) {
+      await this.addAttachment(keyAttachment);
+    }
 
-    // XXX TODO: ChangeAttachmentBucketVisibility() doesn't exist.
-    try {
-      // TB only
-      ChangeAttachmentBucketVisibility(false);
-    } catch (ex) {}
     gContentChanged = true;
     return keyAttachment;
   },
 
   addAttachment(attachment) {
-    AddAttachments([attachment]);
+    return AddAttachments([attachment]);
   },
 
   removeAttachedKey() {
@@ -985,34 +817,27 @@ Enigmail.msg = {
       "enigmailMsgComposeOverlay.js: Enigmail.msg.removeAttachedKey: \n"
     );
 
-    var bucketList = document.getElementById("attachmentBucket");
-    var node = bucketList.firstChild;
+    let bucketList = document.getElementById("attachmentBucket");
+    let node = bucketList.firstElementChild;
 
-    if (
-      bucketList &&
-      bucketList.hasChildNodes() &&
-      this.attachOwnKeyObj.attachedObj
-    ) {
-      // undo attaching own key
+    if (bucketList.itemCount && this.attachOwnKeyObj.attachedObj) {
+      // Undo attaching own key.
       while (node) {
         if (node.attachment.url == this.attachOwnKeyObj.attachedObj.url) {
           node = bucketList.removeChild(node);
-          // Let's release the attachment object held by the node else it won't go away until the window is destroyed
+          // Let's release the attachment object held by the node else it won't
+          // go away until the window is destroyed.
           node.attachment = null;
           this.attachOwnKeyObj.attachedObj = null;
           this.attachOwnKeyObj.attachedKey = null;
-          node = null; // exit loop
+          node = null; // exit loop.
         } else {
           node = node.nextSibling;
         }
       }
-      if (!bucketList.hasChildNodes()) {
-        // XXX TODO: ChangeAttachmentBucketVisibility() doesn't exist.
-        try {
-          // TB only
-          ChangeAttachmentBucketVisibility(true);
-        } catch (ex) {}
-      }
+
+      // Update the visibility of the attachment pane.
+      UpdateAttachmentBucket(bucketList.itemCount);
     }
   },
 
@@ -1206,7 +1031,7 @@ Enigmail.msg = {
         //gSMFields.signMessage = true;
 
       // smime policy
-      //if (this.identity.getIntAttribute("encryptionpolicy") > 0)
+      //if (this.identity.encryptionPolicy > 0)
 
       // update the S/MIME GUI elements
       try {
@@ -1259,11 +1084,13 @@ Enigmail.msg = {
         this.addRecipients(toAddrList, recList);
       }
 
-      // We don't need the returned array of valid keys
-      await Enigmail.hlp.validKeysForAllRecipients(
-        toAddrList.join(", "),
-        detailsObj
-      );
+      let addresses = [];
+      try {
+        addresses = EnigmailFuncs.stripEmail(toAddrList.join(", ")).split(",");
+      } catch (ex) {}
+
+      // Resolve all the email addresses if possible.
+      await EnigmailKeyRing.getValidKeysForAllRecipients(addresses, detailsObj);
       //this.autoPgpEncryption = (validKeyList !== null);
     }
 
@@ -1356,10 +1183,7 @@ Enigmail.msg = {
 
     draftStatus += gAttachMyPublicPGPKey ? "1" : "0";
 
-    draftStatus +=
-      doEncrypt && gCurrentIdentity.getBoolAttribute("protectSubject")
-        ? "1"
-        : "0";
+    draftStatus += gSendEncrypted && gEncryptSubject ? "1" : "0";
 
     this.setAdditionalHeader("X-Enigmail-Draft-Status", draftStatus);
   },
@@ -1434,7 +1258,7 @@ Enigmail.msg = {
    * @wrapresultObj.usePpgMime, true if message send option was changed to PGP/MIME, else false
    */
 
-  wrapInLine(wrapresultObj) {
+  async wrapInLine(wrapresultObj) {
     EnigmailLog.DEBUG("enigmailMsgComposeOverlay.js: WrapInLine\n");
     wrapresultObj.cancelled = false;
     wrapresultObj.usePpgMime = false;
@@ -1443,21 +1267,18 @@ Enigmail.msg = {
       var editor = gMsgCompose.editor.QueryInterface(Ci.nsIEditorMailSupport);
       var encoderFlags = dce.OutputFormatted | dce.OutputLFLineBreak;
 
-      var wrapWidth = this.getMailPref("mailnews.wraplength");
+      var wrapWidth = Services.prefs.getIntPref("mailnews.wraplength");
       if (wrapWidth > 0 && wrapWidth < 68 && editor.wrapWidth > 0) {
         if (
           EnigmailDialog.confirmDlg(
             window,
-            l10n.formatValueSync("minimal-line-wrapping", {
+            await l10nOpenPGP.formatValue("minimal-line-wrapping", {
               width: wrapWidth,
             })
           )
         ) {
           wrapWidth = 68;
-          EnigmailPrefs.getPrefRoot().setIntPref(
-            "mailnews.wraplength",
-            wrapWidth
-          );
+          Services.prefs.setIntPref("mailnews.wraplength", wrapWidth);
         }
       }
 
@@ -1550,10 +1371,13 @@ Enigmail.msg = {
   async saveDraftMessage(senderKeyIsGnuPG) {
     EnigmailLog.DEBUG("enigmailMsgComposeOverlay.js: saveDraftMessage()\n");
 
-    // If we have an encryption key, then always save drafts encrypted,
-    // as a precaution. This is independent from the final decision
-    // of sending the message encrypted or not.
-    let doEncrypt = Enigmail.msg.isEnigmailEnabledForIdentity();
+    // If we have an encryption key configured, then encrypt saved
+    // drafts by default, as a precaution. This is independent from the
+    // final decision of sending the message encrypted or not.
+    // However, we allow the user to disable encrypted drafts.
+    let doEncrypt =
+      Enigmail.msg.isEnigmailEnabledForIdentity() &&
+      gCurrentIdentity.autoEncryptDrafts;
 
     this.setDraftStatus(doEncrypt);
 
@@ -1576,7 +1400,7 @@ Enigmail.msg = {
       EnigmailConstants.SEND_ENCRYPT_TO_SELF |
       EnigmailConstants.SAVE_MESSAGE;
 
-    if (gCurrentIdentity.getBoolAttribute("protectSubject")) {
+    if (gEncryptSubject) {
       sendFlags |= EnigmailConstants.ENCRYPT_SUBJECT;
     }
     if (senderKeyIsGnuPG) {
@@ -1668,7 +1492,7 @@ Enigmail.msg = {
         break;
       default:
         if (gAttachMyPublicPGPKey) {
-          this.attachOwnKey();
+          await this.attachOwnKey();
           Attachments2CompFields(gMsgCompose.compFields); // update list of attachments
         }
     }
@@ -1735,7 +1559,7 @@ Enigmail.msg = {
       f |= EnigmailConstants.SEND_TWO_MIME_LAYERS;
     }
 
-    if (gCurrentIdentity.getBoolAttribute("protectSubject")) {
+    if (gSendEncrypted && gEncryptSubject) {
       f |= EnigmailConstants.ENCRYPT_SUBJECT;
     }
 
@@ -1775,7 +1599,7 @@ Enigmail.msg = {
     return newSecurityInfo;
   },
 
-  determineMsgRecipients(sendFlags) {
+  async determineMsgRecipients(sendFlags) {
     EnigmailLog.DEBUG(
       "enigmailMsgComposeOverlay.js: Enigmail.msg.determineMsgRecipients: currentId=" +
         gCurrentIdentity +
@@ -1799,7 +1623,7 @@ Enigmail.msg = {
     let msgCompFields = gMsgCompose.compFields;
     let newsgroups = msgCompFields.newsgroups;
 
-    if (EnigmailPrefs.getPref("encryptToSelf")) {
+    if (Services.prefs.getBoolPref("temp.openpgp.encryptToSelf")) {
       optSendFlags |= EnigmailConstants.SEND_ENCRYPT_TO_SELF;
     }
 
@@ -1828,55 +1652,28 @@ Enigmail.msg = {
       this.addRecipients(toAddrList, recList);
     }
 
-    // special handling of bcc:
-    // - note: bcc and encryption is a problem
-    // - but bcc to the sender is fine
+    // We allow sending to BCC recipients, we assume the user interface
+    // has warned the user that there is no privacy of BCC recipients.
     if (msgCompFields.bcc.length > 0) {
       recList = splitRecipients(msgCompFields.bcc, true, arrLen);
-
-      var bccLC = "";
-      try {
-        bccLC = EnigmailFuncs.stripEmail(msgCompFields.bcc).toLowerCase();
-      } catch (ex) {}
-      EnigmailLog.DEBUG(
-        "enigmailMsgComposeOverlay.js: Enigmail.msg.determineMsgRecipients: BCC: " +
-          bccLC +
-          "\n"
-      );
-
-      // It's equal, if self's email address is the only entry in BCC.
-      var selfBCC =
-        gCurrentIdentity.email && gCurrentIdentity.email.toLowerCase() == bccLC;
-
-      if (selfBCC) {
-        EnigmailLog.DEBUG(
-          "enigmailMsgComposeOverlay.js: Enigmail.msg.determineMsgRecipients: Self BCC\n"
-        );
-        this.addRecipients(toAddrList, recList);
-      } else if (sendFlags & EnigmailConstants.SEND_ENCRYPTED) {
-        EnigmailDialog.alert(
-          window,
-          l10n.formatValueSync("sending-hidden-rcpt")
-        );
-        return false;
-      }
+      this.addRecipients(bccAddrList, recList);
     }
 
     if (newsgroups) {
       toAddrList.push(newsgroups);
 
       if (sendFlags & EnigmailConstants.SEND_ENCRYPTED) {
-        if (!EnigmailPrefs.getPref("encryptToNews")) {
+        if (!Services.prefs.getBoolPref("temp.openpgp.encryptToNews")) {
           document.l10n.formatValue("sending-news").then(value => {
             EnigmailDialog.alert(window, value);
           });
           return false;
         } else if (
-          !EnigmailDialog.confirmPref(
+          !EnigmailDialog.confirmBoolPref(
             window,
-            l10n.formatValueSync("send-to-news-warning"),
-            "warnOnSendingNewsgroups",
-            l10n.formatValueSync("msg-compose-button-send")
+            await l10nOpenPGP.formatValue("send-to-news-warning"),
+            "temp.openpgp.warnOnSendingNewsgroups",
+            await l10nOpenPGP.formatValue("msg-compose-button-send")
           )
         ) {
           return false;
@@ -1938,7 +1735,7 @@ Enigmail.msg = {
         gMsgCompose.compFields.subject = "";
       }
 
-      if (EnigmailPrefs.getPref("protectReferencesHdr")) {
+      if (Services.prefs.getBoolPref("temp.openpgp.protectReferencesHdr")) {
         gMsgCompose.compFields.references = "";
       }
     }
@@ -2048,32 +1845,11 @@ Enigmail.msg = {
       }
     }
 
-    let cannotEncryptMissingInfo = "";
+    let cannotEncryptMissingInfo = false;
     if (gSendEncrypted) {
       let canEncryptDetails = await this.determineSendFlags();
       if (canEncryptDetails.errArray.length != 0) {
-        let isFirst = true;
-        let allProblems = "";
-        for (let obj of canEncryptDetails.errArray) {
-          if (isFirst) {
-            isFirst = false;
-          } else {
-            allProblems += ", ";
-          }
-          allProblems += obj.addr;
-        }
-
-        cannotEncryptMissingInfo = l10n.formatValueSync(
-          "cannot-encrypt-because-missing",
-          {
-            problem: allProblems,
-          }
-        );
-
-        // Delay showing the alert until after determineMsgRecipients()
-        // has checked that BCC recipients aren't an issue.
-        // A BCC problem should be notified first. Only afterwards we
-        // should tell the user to get missing keys.
+        cannotEncryptMissingInfo = true;
       }
     }
 
@@ -2092,15 +1868,14 @@ Enigmail.msg = {
       this.modifiedAttach = null;
 
       // fill fromAddr, toAddrList, bcc etc
-      let rcpt = this.determineMsgRecipients(sendFlags);
+      let rcpt = await this.determineMsgRecipients(sendFlags);
       if (typeof rcpt === "boolean") {
         return rcpt;
       }
       sendFlags = rcpt.sendFlags;
 
       if (cannotEncryptMissingInfo) {
-        EnigmailDialog.alert(window, cannotEncryptMissingInfo);
-        showMessageComposeSecurityStatus();
+        showMessageComposeSecurityStatus(true);
         return false;
       }
 
@@ -2114,7 +1889,7 @@ Enigmail.msg = {
       let keyMap = {};
 
       if (gAttachMyPublicPGPKey) {
-        this.attachOwnKey(senderKeyId);
+        await this.attachOwnKey(senderKeyId);
       }
 
       /*
@@ -2150,7 +1925,7 @@ Enigmail.msg = {
       ) {
         var wrapresultObj = {};
 
-        this.wrapInLine(wrapresultObj);
+        await this.wrapInLine(wrapresultObj);
 
         if (wrapresultObj.usePpgMime) {
           sendFlags |= EnigmailConstants.SEND_PGP_MIME;
@@ -2190,7 +1965,7 @@ Enigmail.msg = {
           bucketList: document.getElementById("attachmentBucket"),
         };
 
-        if (!this.signInline(sendInfo)) {
+        if (!(await this.signInline(sendInfo))) {
           return false;
         }
       }
@@ -2242,7 +2017,7 @@ Enigmail.msg = {
     return true;
   },
 
-  signInline(sendInfo) {
+  async signInline(sendInfo) {
     // sign message using inline-PGP
 
     if (sendInfo.sendFlags & ENCRYPT) {
@@ -2263,26 +2038,21 @@ Enigmail.msg = {
       return false;
     }
 
-    try {
-      if (this.getMailPref("mail.strictly_mime")) {
-        if (
-          EnigmailDialog.confirmPref(
-            window,
-            l10n.formatValueSync("quoted-printable-warn"),
-            "quotedPrintableWarn"
-          )
-        ) {
-          EnigmailPrefs.getPrefRoot().setBoolPref("mail.strictly_mime", false);
-        }
+    if (Services.prefs.getBoolPref("mail.strictly_mime")) {
+      if (
+        EnigmailDialog.confirmIntPref(
+          window,
+          await l10nOpenPGP.formatValue("quoted-printable-warn"),
+          "temp.openpgp.quotedPrintableWarn"
+        )
+      ) {
+        Services.prefs.setBoolPref("mail.strictly_mime", false);
       }
-    } catch (ex) {}
-
-    var sendFlowed;
-    try {
-      sendFlowed = this.getMailPref("mailnews.send_plaintext_flowed");
-    } catch (ex) {
-      sendFlowed = true;
     }
+
+    var sendFlowed = Services.prefs.getBoolPref(
+      "mailnews.send_plaintext_flowed"
+    );
     var encoderFlags = dce.OutputFormatted | dce.OutputLFLineBreak;
 
     // plaintext: Wrapping code has been moved to superordinate function prepareSendMsg to enable interactive format switch
@@ -2377,14 +2147,6 @@ Enigmail.msg = {
         if (sendInfo.sendFlags & SIGN) {
           // Encryption/signing failed
 
-          /*if (statusFlagsObj.statusMsg) {
-            // check if own key is invalid
-            let s = new RegExp("^(\\[GNUPG:\\] )?INV_(RECP|SGNR) [0-9]+ \\<?" + sendInfo.fromAddr + "\\>?", "m");
-            if (statusFlagsObj.statusMsg.search(s) >= 0) {
-              errorMsgObj.value += "\n\n" + EnigmailLocale.getString("keyError.resolutionAction");
-            }
-          }*/
-
           this.sendAborted(window, errorMsgObj);
           return false;
         }
@@ -2442,49 +2204,13 @@ Enigmail.msg = {
     }
   },
 
-  getMailPref(prefName) {
-    let prefRoot = EnigmailPrefs.getPrefRoot();
-
-    var prefValue = null;
-    try {
-      var prefType = prefRoot.getPrefType(prefName);
-      // Get pref value
-      switch (prefType) {
-        case prefRoot.PREF_BOOL:
-          prefValue = prefRoot.getBoolPref(prefName);
-          break;
-
-        case prefRoot.PREF_INT:
-          prefValue = prefRoot.getIntPref(prefName);
-          break;
-
-        case prefRoot.PREF_STRING:
-          prefValue = prefRoot.getCharPref(prefName);
-          break;
-
-        default:
-          prefValue = undefined;
-          break;
-      }
-    } catch (ex) {
-      // Failed to get pref value
-      EnigmailLog.ERROR(
-        "enigmailMsgComposeOverlay.js: Enigmail.msg.getMailPref: unknown prefName:" +
-          prefName +
-          " \n"
-      );
-    }
-
-    return prefValue;
-  },
-
   messageSendCheck() {
     EnigmailLog.DEBUG(
       "enigmailMsgComposeOverlay.js: Enigmail.msg.messageSendCheck\n"
     );
 
     try {
-      var warn = this.getMailPref("mail.warn_on_send_accel_key");
+      var warn = Services.prefs.getBoolPref("mail.warn_on_send_accel_key");
 
       if (warn) {
         var checkValue = {
@@ -2509,10 +2235,7 @@ Enigmail.msg = {
           return false;
         }
         if (checkValue.value) {
-          EnigmailPrefs.getPrefRoot().setBoolPref(
-            "mail.warn_on_send_accel_key",
-            false
-          );
+          Services.prefs.setBoolPref("mail.warn_on_send_accel_key", false);
         }
       }
     } catch (ex) {}
@@ -2614,8 +2337,6 @@ Enigmail.msg = {
 
       try {
         this.modifyCompFields();
-        //bc.setAttribute("disabled", "true");
-
         const cApi = EnigmailCryptoAPI();
         let encryptResult = cApi.sync(this.prepareSendMsg(sendMsgType));
         if (!encryptResult) {
@@ -2624,9 +2345,11 @@ Enigmail.msg = {
           event.stopPropagation();
         }
       } catch (ex) {
-        console.debug(ex);
+        Cu.reportError("GenericSendMessage FAILED: " + ex);
+        this.resetUpdatedFields();
+        event.preventDefault();
+        event.stopPropagation();
       }
-      //bc.removeAttribute("disabled");
     } else {
       EnigmailLog.DEBUG(
         "enigmailMsgComposeOverlay.js: Enigmail.msg.sendMessageListener: sending in progress - autosave aborted\n"
@@ -2637,29 +2360,7 @@ Enigmail.msg = {
     this.sendProcess = false;
   },
 
-  toggleAttribute(attrName) {
-    EnigmailLog.DEBUG(
-      "enigmailMsgComposeOverlay.js: Enigmail.msg.toggleAttribute('" +
-        attrName +
-        "')\n"
-    );
-
-    var oldValue = EnigmailPrefs.getPref(attrName);
-    EnigmailPrefs.setPref(attrName, !oldValue);
-  },
-
-  toggleAccountAttr(attrName) {
-    EnigmailLog.DEBUG(
-      "enigmailMsgComposeOverlay.js: Enigmail.msg.toggleAccountAttr('" +
-        attrName +
-        "')\n"
-    );
-
-    var oldValue = gCurrentIdentity.getBoolAttribute(attrName);
-    gCurrentIdentity.setBoolAttribute(attrName, !oldValue);
-  },
-
-  decryptQuote(interactive) {
+  async decryptQuote(interactive) {
     EnigmailLog.DEBUG(
       "enigmailMsgComposeOverlay.js: Enigmail.msg.decryptQuote: " +
         interactive +
@@ -2768,19 +2469,6 @@ Enigmail.msg = {
     // Encode ciphertext from unicode to charset
     var cipherText = EnigmailData.convertFromUnicode(pgpBlock, charset);
 
-    if (
-      !this.getMailPref("mailnews.reply_in_default_charset") &&
-      blockType == "MESSAGE"
-    ) {
-      // set charset according to PGP block, if available (encrypted messages only)
-      let armorHeaders = EnigmailArmor.getArmorHeaders(cipherText);
-
-      if ("charset" in armorHeaders) {
-        charset = armorHeaders.charset;
-        gMsgCompose.SetDocumentCharset(charset);
-      }
-    }
-
     // Decrypt message
     var signatureObj = {};
     signatureObj.value = "";
@@ -2801,6 +2489,7 @@ Enigmail.msg = {
       window,
       uiFlags,
       cipherText,
+      null, // date
       signatureObj,
       exitCodeObj,
       statusFlagsObj,
@@ -2817,7 +2506,7 @@ Enigmail.msg = {
       "\n"
     );
 
-    //if (EnigmailPrefs.getPref("keepSettingsForReply")) {
+    //if (Services.prefs.getBoolPref("temp.openpgp.keepSettingsForReply")) {
     if (statusFlagsObj.value & EnigmailConstants.DECRYPTION_OKAY) {
       //this.setSendMode('encrypt');
 
@@ -2825,9 +2514,8 @@ Enigmail.msg = {
       // automatic enabling encryption currently depends on
       // adjustSignEncryptAfterIdentityChanged to be always reached
       gIsRelatedToEncryptedOriginal = true;
-      gSendEncrypted = true;
+      updateE2eeOptions(gSendEncrypted);
       gSendSigned = true;
-      setEncSigStatusUI();
     }
     //}
 
@@ -2871,7 +2559,9 @@ Enigmail.msg = {
     }
 
     const nsIMsgCompType = Ci.nsIMsgCompType;
-    var doubleDashSeparator = EnigmailPrefs.getPref("doubleDashSeparator");
+    var doubleDashSeparator = Services.prefs.getBoolPref(
+      "temp.openpgp.doubleDashSeparator"
+    );
     if (
       gMsgCompose.type != nsIMsgCompType.Template &&
       gMsgCompose.type != nsIMsgCompType.Draft &&
@@ -2889,17 +2579,6 @@ Enigmail.msg = {
       }
     }
 
-    var clipBoard = Services.clipboard;
-    var data;
-    if (clipBoard.supportsSelectionClipboard()) {
-      // get the clipboard contents for selected text (X11)
-      data = EnigmailClipboard.getClipboardContent(
-        window,
-        Ci.nsIClipboard.kSelectionClipboard
-      );
-    }
-
-    // Replace encrypted quote with decrypted quote (destroys selection clipboard on X11)
     this.editorSelectAll();
 
     //EnigmailLog.DEBUG("enigmailMsgComposeOverlay.js: Enigmail.msg.decryptQuote: plainText='"+plainText+"'\n");
@@ -2924,14 +2603,6 @@ Enigmail.msg = {
       this.checkInlinePgpReply(head, tail);
     }
 
-    if (clipBoard.supportsSelectionClipboard()) {
-      // restore the clipboard contents for selected text (X11)
-      EnigmailClipboard.setClipboardContent(
-        data,
-        clipBoard.kSelectionClipboard
-      );
-    }
-
     if (interactive) {
       return;
     }
@@ -2951,8 +2622,6 @@ Enigmail.msg = {
         "\n"
     );
 
-    var nsISelectionController = Ci.nsISelectionController;
-
     if (this.editor.selectionController) {
       var selection = this.editor.selectionController;
       selection.completeMove(false, false); // go to start;
@@ -2993,8 +2662,8 @@ Enigmail.msg = {
       }
 
       this.editor.selectionController.scrollSelectionIntoView(
-        nsISelectionController.SELECTION_NORMAL,
-        nsISelectionController.SELECTION_ANCHOR_REGION,
+        Ci.nsISelectionController.SELECTION_NORMAL,
+        Ci.nsISelectionController.SELECTION_ANCHOR_REGION,
         true
       );
     }
@@ -3101,21 +2770,17 @@ Enigmail.msg = {
    *                              if null or "", then the Detail button will no be displayed.
    */
   async notifyUser(priority, msgText, messageId, detailsText) {
-    let notif = document.getElementById("attachmentNotificationBox");
-    if (!notif) {
-      notif = gNotification.notificationbox;
-    }
     let prio;
 
     switch (priority) {
       case 1:
-        prio = notif.PRIORITY_CRITICAL_MEDIUM;
+        prio = gComposeNotification.PRIORITY_CRITICAL_MEDIUM;
         break;
       case 3:
-        prio = notif.PRIORITY_INFO_MEDIUM;
+        prio = gComposeNotification.PRIORITY_INFO_MEDIUM;
         break;
       default:
-        prio = notif.PRIORITY_WARNING_MEDIUM;
+        prio = gComposeNotification.PRIORITY_WARNING_MEDIUM;
     }
 
     let buttonArr = [];
@@ -3134,7 +2799,14 @@ Enigmail.msg = {
         },
       });
     }
-    notif.appendNotification(msgText, messageId, null, prio, buttonArr);
+    gComposeNotification.appendNotification(
+      messageId,
+      {
+        label: msgText,
+        priority: prio,
+      },
+      buttonArr
+    );
   },
 
   /**
@@ -3292,21 +2964,26 @@ Enigmail.composeStateListener = {
     }
     */
 
-    if (!isEditable || isEmpty) {
-      return;
+    if (isEditable && !isEmpty) {
+      let msgHdr = Enigmail.msg.getMsgHdr();
+      if (msgHdr) {
+        Enigmail.msg.setOriginalSubject(msgHdr.subject, msgHdr.flags, true);
+      }
+      Enigmail.msg.fixMessageSubject();
+
+      if (!Enigmail.msg.timeoutId && !Enigmail.msg.dirty) {
+        Enigmail.msg.timeoutId = setTimeout(function() {
+          Enigmail.msg.decryptQuote(false);
+        }, 0);
+      }
     }
 
-    let msgHdr = Enigmail.msg.getMsgHdr();
-    if (msgHdr) {
-      Enigmail.msg.setOriginalSubject(msgHdr.subject, msgHdr.flags, true);
-    }
-    Enigmail.msg.fixMessageSubject();
-
-    if (!Enigmail.msg.timeoutId && !Enigmail.msg.dirty) {
-      Enigmail.msg.timeoutId = EnigmailTimer.setTimeout(function() {
-        Enigmail.msg.decryptQuote(false);
-      }, 0);
-    }
+    // This must be called by the last registered NotifyComposeBodyReady()
+    // stateListener. We need this in order to know when the entire init
+    // sequence of the composeWindow has finished, so the WebExtension compose
+    // API can do its final modifications.
+    window.composeEditorReady = true;
+    window.dispatchEvent(new CustomEvent("compose-editor-ready"));
   },
 
   SaveInFolderDone(folderURI) {
