@@ -6,87 +6,34 @@
 
 "use strict";
 
-/* Globals from Thunderbird: */
-/* global gFolderDisplay: false, currentAttachments: false, gCryptoContainer: false, gSignedUINode: false, gEncryptedUINode: false */
-/* global gDBView: false, msgWindow: false, messageHeaderSink: false, gMessageListeners: false, findEmailNodeFromPopupNode: true */
+/* global setMessageEncryptionStateButton */
+/* global gFolderDisplay: false, currentAttachments: false */
+/* global gDBView: false, msgWindow: false, messageHeaderSink: false, gMessageListeners: false */
 /* global gExpandedHeaderView: false, CanDetachAttachments: true, gEncryptedURIService: false, FillAttachmentListPopup: false */
-/* global attachmentList: false, MailOfflineMgr: false, currentHeaderData: false, ContentTypeIsSMIME: false */
+/* global attachmentList: false, currentHeaderData: false */
 
-/* import-globals-from ../BondOpenPGP.jsm */
+var { XPCOMUtils } = ChromeUtils.import(
+  "resource://gre/modules/XPCOMUtils.jsm"
+);
+var { Services } = ChromeUtils.import("resource://gre/modules/Services.jsm");
 
-var EnigmailCore = ChromeUtils.import(
-  "chrome://openpgp/content/modules/core.jsm"
-).EnigmailCore;
-var EnigmailFuncs = ChromeUtils.import(
-  "chrome://openpgp/content/modules/funcs.jsm"
-).EnigmailFuncs;
-var EnigmailVerify = ChromeUtils.import(
-  "chrome://openpgp/content/modules/mimeVerify.jsm"
-).EnigmailVerify;
-var { EnigmailLog } = ChromeUtils.import(
-  "chrome://openpgp/content/modules/log.jsm"
-);
-var EnigmailPrefs = ChromeUtils.import(
-  "chrome://openpgp/content/modules/prefs.jsm"
-).EnigmailPrefs;
-var EnigmailLocale = ChromeUtils.import(
-  "chrome://openpgp/content/modules/locale.jsm"
-).EnigmailLocale;
-var EnigmailWindows = ChromeUtils.import(
-  "chrome://openpgp/content/modules/windows.jsm"
-).EnigmailWindows;
-var EnigmailDialog = ChromeUtils.import(
-  "chrome://openpgp/content/modules/dialog.jsm"
-).EnigmailDialog;
-var EnigmailTime = ChromeUtils.import(
-  "chrome://openpgp/content/modules/time.jsm"
-).EnigmailTime;
-var { EnigmailGpg } = ChromeUtils.import(
-  "chrome://openpgp/content/modules/gpg.jsm"
-);
-var { EnigmailKey } = ChromeUtils.import(
-  "chrome://openpgp/content/modules/key.jsm"
-);
-var EnigmailKeyRing = ChromeUtils.import(
-  "chrome://openpgp/content/modules/keyRing.jsm"
-).EnigmailKeyRing;
-var EnigmailURIs = ChromeUtils.import(
-  "chrome://openpgp/content/modules/uris.jsm"
-).EnigmailURIs;
-var EnigmailConstants = ChromeUtils.import(
-  "chrome://openpgp/content/modules/constants.jsm"
-).EnigmailConstants;
-var EnigmailData = ChromeUtils.import(
-  "chrome://openpgp/content/modules/data.jsm"
-).EnigmailData;
-var EnigmailClipboard = ChromeUtils.import(
-  "chrome://openpgp/content/modules/clipboard.jsm"
-).EnigmailClipboard;
-var EnigmailStdlib = ChromeUtils.import(
-  "chrome://openpgp/content/modules/stdlib.jsm"
-).EnigmailStdlib;
-/*
-var EnigmailWks = ChromeUtils.import(
-  "chrome://openpgp/content/modules/webKey.jsm"
-).EnigmailWks;
-*/
-var EnigmailMime = ChromeUtils.import(
-  "chrome://openpgp/content/modules/mime.jsm"
-).EnigmailMime;
-var EnigmailMsgRead = ChromeUtils.import(
-  "chrome://openpgp/content/modules/msgRead.jsm"
-).EnigmailMsgRead;
-var EnigmailSingletons = ChromeUtils.import(
-  "chrome://openpgp/content/modules/singletons.jsm"
-).EnigmailSingletons;
-/*
-var EnigmailAutocrypt = ChromeUtils.import(
-  "chrome://openpgp/content/modules/autocrypt.jsm"
-).EnigmailAutocrypt;
-*/
-var EnigmailCompat = ChromeUtils.import(
-  "chrome://openpgp/content/modules/compat.jsm"
-).EnigmailCompat;
+XPCOMUtils.defineLazyModuleGetters(this, {
+  EnigmailConstants: "chrome://openpgp/content/modules/constants.jsm",
+  EnigmailCore: "chrome://openpgp/content/modules/core.jsm",
+  EnigmailData: "chrome://openpgp/content/modules/data.jsm",
+  EnigmailDialog: "chrome://openpgp/content/modules/dialog.jsm",
+  EnigmailFuncs: "chrome://openpgp/content/modules/funcs.jsm",
+  EnigmailKey: "chrome://openpgp/content/modules/key.jsm",
+  EnigmailKeyRing: "chrome://openpgp/content/modules/keyRing.jsm",
+  EnigmailLog: "chrome://openpgp/content/modules/log.jsm",
+  EnigmailMime: "chrome://openpgp/content/modules/mime.jsm",
+  EnigmailMsgRead: "chrome://openpgp/content/modules/msgRead.jsm",
+  EnigmailSingletons: "chrome://openpgp/content/modules/singletons.jsm",
+  EnigmailURIs: "chrome://openpgp/content/modules/uris.jsm",
+  EnigmailVerify: "chrome://openpgp/content/modules/mimeVerify.jsm",
+  EnigmailWindows: "chrome://openpgp/content/modules/windows.jsm",
+  // EnigmailWks: "chrome://openpgp/content/modules/webKey.jsm",
+});
 
 if (!Enigmail) {
   var Enigmail = {};
@@ -97,18 +44,26 @@ Enigmail.hdrView = {
   lastEncryptedUri: null,
   flexbuttonAction: null,
 
+  msgSignedStateString: null,
+  msgEncryptedStateString: null,
   msgSignatureState: EnigmailConstants.MSG_SIG_NONE,
   msgEncryptionState: EnigmailConstants.MSG_ENC_NONE,
   msgSignatureKeyId: "",
   msgEncryptionKeyId: null,
   msgEncryptionAllKeyIds: null,
+  msgHasKeyAttached: false,
+
+  alreadyWrappedCDA: false,
 
   reset() {
+    this.msgSignedStateString = null;
+    this.msgEncryptedStateString = null;
     this.msgSignatureState = EnigmailConstants.MSG_SIG_NONE;
     this.msgEncryptionState = EnigmailConstants.MSG_ENC_NONE;
     this.msgSignatureKeyId = "";
     this.msgEncryptionKeyId = null;
     this.msgEncryptionAllKeyIds = null;
+    this.msgHasKeyAttached = false;
     for (let value of ["decryptionFailed", "brokenExchange"]) {
       Enigmail.msg.removeNotification(value);
     }
@@ -120,13 +75,16 @@ Enigmail.hdrView = {
     // THE FOLLOWING OVERRIDES CODE IN msgHdrViewOverlay.js
     // which wouldn't work otherwise
 
-    this.origCanDetachAttachments = CanDetachAttachments;
-    CanDetachAttachments = function() {
-      return (
-        Enigmail.hdrView.origCanDetachAttachments() &&
-        Enigmail.hdrView.enigCanDetachAttachments()
-      );
-    };
+    if (!this.alreadyWrappedCDA) {
+      this.alreadyWrappedCDA = true;
+      this.origCanDetachAttachments = CanDetachAttachments;
+      CanDetachAttachments = function() {
+        return (
+          Enigmail.hdrView.origCanDetachAttachments() &&
+          Enigmail.hdrView.enigCanDetachAttachments()
+        );
+      };
+    }
 
     this.msgHdrViewLoad();
 
@@ -249,7 +207,7 @@ Enigmail.hdrView = {
       );
     }
 
-    if (!replaceUid) {
+    if (!replaceUid && userId) {
       replaceUid = userId.replace(/\n.*$/gm, "");
     }
 
@@ -266,7 +224,7 @@ Enigmail.hdrView = {
     }
 
     if (userId && replaceUid) {
-      // no EnigConvertGpgToUnicode() here; strings are already UTF-8
+      // no EnigmailData.convertGpgToUnicode here; strings are already UTF-8
       replaceUid = replaceUid.replace(/\\[xe]3a/gi, ":");
       errorMsg = errorMsg.replace(userId, replaceUid);
     }
@@ -276,7 +234,7 @@ Enigmail.hdrView = {
     if (exitCode == EnigmailConstants.POSSIBLE_PGPMIME) {
       exitCode = 0;
     } else if (errorMsg) {
-      // no EnigConvertGpgToUnicode() here; strings are already UTF-8
+      // no EnigmailData.convertGpgToUnicode here; strings are already UTF-8
       errorLines = errorMsg.split(/\r?\n/);
     }
 
@@ -378,8 +336,8 @@ Enigmail.hdrView = {
         }
 
         Enigmail.hdrView.displayFlexAction(
-          EnigmailLocale.getString("wksConfirmationReq"),
-          EnigmailLocale.getString("wksConfirmationReq.button.label"),
+          "Web Key Directory Confirmation Request",
+          "Confirm Request",
           "wks-request"
         );
         Enigmail.hdrView.displayWksMessage();
@@ -404,86 +362,27 @@ Enigmail.hdrView = {
       let bodyElement = document.getElementById("messagepane");
       bodyElement.setAttribute("collapsed", true);
       enigMsgPane.removeAttribute("collapsed");
-      enigMsgPane.textContent = EnigmailLocale.getString(
-        "wksConfirmationReq.message"
+      enigMsgPane.textContent = "This message has been sent by your email provider to confirm deployment of your OpenPGP public key\nin their Web Key Directory.\nProviding your public key helps others to discover your key and thus being able to encrypt messages to you.\n\nIf you want to deploy your key in the Web Key Directory now, please click on the button "Confirm Request" in the status bar.\nOtherwise, simply ignore this message."
       );
     }
   },
   */
 
-  /**
-   * Try to import an autocrypt header from an uncertain signature
-   * (i.e. the sender's key is not available)
-   */
-  /*
-  tryImportAutocryptHeader() {
-    EnigmailLog.DEBUG(
-      "enigmailMsgHdrViewOverlay.js: tryImportAutocryptHeader()\n"
-    );
-
-    if (!("autocrypt" in currentHeaderData)) {
-      return;
-    }
-    if (!Enigmail.msg.isAutocryptEnabled()) {
-      return;
-    }
-    if (!("from" in currentHeaderData)) {
-      return;
-    }
-
-    let fromEmail = "";
-    try {
-      fromEmail = EnigmailFuncs.stripEmail(
-        currentHeaderData.from.headerValue
-      ).toLowerCase();
-    } catch (ex) {
-      console.debug(ex);
-    }
-
-    let keys = EnigmailKeyRing.getKeysByEmail(fromEmail, true);
-    if (keys.length > 0) {
-      return;
-    }
-
-    EnigmailAutocrypt.importAutocryptKeys([fromEmail]).then(foundKeys => {
-      EnigmailLog.DEBUG(
-        "enigmailMsgComposeOverlay.js: tryImportAutocryptHeader: got " +
-          foundKeys.length +
-          " autocrypt keys\n"
-      );
-      if (foundKeys.length > 0) {
-        let k = EnigmailKeyRing.getKeyById(Enigmail.msg.securityInfo.keyId);
-        if (k) {
-          gDBView.reloadMessageWithAllParts();
-        }
-      }
-    });
-  },
-  */
-
   async displayStatusBar() {
     let secInfo = Enigmail.msg.securityInfo;
     let statusFlags = secInfo.statusFlags;
     let extStatusFlags =
       "extStatusFlags" in secInfo ? secInfo.extStatusFlags : 0;
-    let cryptoContainer, encryptedUINode, signedUINode;
 
-    cryptoContainer = gCryptoContainer;
-    signedUINode = gSignedUINode;
-    encryptedUINode = gEncryptedUINode;
-
-    cryptoContainer.collapsed = false;
-    cryptoContainer.setAttribute("tech", "OpenPGP");
-    document
-      .getElementById("encryptionTechBtn")
-      .querySelector("span").textContent = "OpenPGP";
+    let signed;
+    let encrypted;
 
     if (
       statusFlags &
       (EnigmailConstants.DECRYPTION_FAILED |
         EnigmailConstants.DECRYPTION_INCOMPLETE)
     ) {
-      encryptedUINode.setAttribute("encrypted", "notok");
+      encrypted = "notok";
       let unhideBar = false;
       let infoId;
       if (statusFlags & EnigmailConstants.NO_SECKEY) {
@@ -501,26 +400,26 @@ Enigmail.hdrView = {
 
       if (unhideBar) {
         Enigmail.msg.notificationBox.appendNotification(
-          await document.l10n.formatValue(infoId),
           "decryptionFailed",
-          "chrome://global/skin/icons/warning.svg",
-          Enigmail.msg.notificationBox.PRIORITY_CRITICAL_MEDIUM,
+          {
+            label: await document.l10n.formatValue(infoId),
+            image: "chrome://global/skin/icons/warning.svg",
+            priority: Enigmail.msg.notificationBox.PRIORITY_CRITICAL_MEDIUM,
+          },
           null
         );
       }
 
       this.msgSignatureState = EnigmailConstants.MSG_SIG_NONE;
-      encryptedUINode.hidden = false;
     } else if (statusFlags & EnigmailConstants.DECRYPTION_OKAY) {
       EnigmailURIs.rememberEncryptedUri(this.lastEncryptedMsgKey);
-      encryptedUINode.setAttribute("encrypted", "ok");
+      encrypted = "ok";
       this.msgEncryptionState = EnigmailConstants.MSG_ENC_OK;
       if (secInfo.xtraStatus && secInfo.xtraStatus == "buggyMailFormat") {
         console.log(
           await document.l10n.formatValue("decrypted-msg-with-format-error")
         );
       }
-      encryptedUINode.hidden = false;
     }
 
     if (
@@ -535,26 +434,22 @@ Enigmail.hdrView = {
       } else {
         this.msgSignatureState = EnigmailConstants.MSG_SIG_INVALID;
       }
-      signedUINode.setAttribute("signed", "notok");
-      signedUINode.hidden = false;
+      signed = "notok";
     } else if (statusFlags & EnigmailConstants.GOOD_SIGNATURE) {
-      let val;
       if (statusFlags & EnigmailConstants.TRUSTED_IDENTITY) {
         this.msgSignatureState = EnigmailConstants.MSG_SIG_VALID_KEY_VERIFIED;
-        val = "verified";
+        signed = "verified";
       } else if (extStatusFlags & EnigmailConstants.EXT_SELF_IDENTITY) {
-        val = "ok";
+        signed = "ok";
         this.msgSignatureState = EnigmailConstants.MSG_SIG_VALID_SELF;
       } else {
-        val = "unverified";
+        signed = "unverified";
         this.msgSignatureState = EnigmailConstants.MSG_SIG_VALID_KEY_UNVERIFIED;
       }
-      signedUINode.setAttribute("signed", val);
-      signedUINode.hidden = false;
     } else if (statusFlags & EnigmailConstants.UNCERTAIN_SIGNATURE) {
-      let val = "unknown";
+      signed = "unknown";
       if (statusFlags & EnigmailConstants.INVALID_RECIPIENT) {
-        val = "mismatch";
+        signed = "mismatch";
         this.msgSignatureState =
           EnigmailConstants.MSG_SIG_UNCERTAIN_UID_MISMATCH;
       } else if (statusFlags & EnigmailConstants.NO_PUBKEY) {
@@ -565,16 +460,67 @@ Enigmail.hdrView = {
         this.msgSignatureState =
           EnigmailConstants.MSG_SIG_UNCERTAIN_KEY_NOT_ACCEPTED;
       }
-      signedUINode.setAttribute("signed", val);
-      signedUINode.hidden = false;
     }
     // (statusFlags & EnigmailConstants.INLINE_KEY) ???
 
+    if (encrypted) {
+      this.msgEncryptedStateString = encrypted;
+    }
+    if (signed) {
+      this.msgSignedStateString = signed;
+    }
+    this.updateEncryptionStateButton();
+
     /*
     // special handling after trying to fix buggy mail format (see buggyExchangeEmailContent in code)
     if (secInfo.xtraStatus && secInfo.xtraStatus == "buggyMailFormat") {
     }
     */
+
+    if (encrypted) {
+      // For telemetry purposes.
+      window.dispatchEvent(
+        new CustomEvent("secureMsgLoaded", {
+          detail: {
+            key: "encrypted-openpgp",
+            data: encrypted,
+          },
+        })
+      );
+    }
+    if (signed) {
+      window.dispatchEvent(
+        new CustomEvent("secureMsgLoaded", {
+          detail: {
+            key: "signed-openpgp",
+            data: signed,
+          },
+        })
+      );
+    }
+  },
+
+  /**
+   * Should be called as soon as it is known that the message has
+   * an OpenPGP key attached.
+   */
+  notifyHasKeyAttached() {
+    this.msgHasKeyAttached = true;
+    this.updateEncryptionStateButton();
+  },
+
+  /**
+   * Should be called whenever more information about the OpenPGP
+   * message state became available, such as encryption or signature
+   * status, or the availability of an attached key.
+   */
+  updateEncryptionStateButton() {
+    setMessageEncryptionStateButton(
+      "OpenPGP",
+      this.msgEncryptedStateString,
+      this.msgSignedStateString,
+      this.msgHasKeyAttached
+    );
   },
 
   editKeyExpiry() {
@@ -679,25 +625,11 @@ Enigmail.hdrView = {
     }
   },
 
-  messageLoad(event) {
+  async messageLoad(event) {
     EnigmailLog.DEBUG("enigmailMsgHdrViewOverlay.js: this.messageLoad\n");
 
-    Enigmail.msg.messageAutoDecrypt();
-    Enigmail.msg.handleAttchmentEvent();
-  },
-
-  showPhoto() {
-    if (!Enigmail.msg.securityInfo) {
-      return;
-    }
-
-    let key = EnigmailKeyRing.getKeyById(Enigmail.msg.securityInfo.keyId);
-
-    EnigmailWindows.showPhoto(
-      window,
-      key.keyId,
-      Enigmail.msg.securityInfo.userId
-    );
+    await Enigmail.msg.messageAutoDecrypt();
+    Enigmail.msg.handleAttachmentEvent();
   },
 
   dispKeyDetails() {
@@ -723,97 +655,82 @@ Enigmail.hdrView = {
   },
 
   onShowAttachmentContextMenu(event) {
-    EnigmailLog.DEBUG(
-      "enigmailMsgHdrViewOverlay.js: this.onShowAttachmentContextMenu\n"
-    );
+    let contextMenu = document.getElementById("attachmentItemContext");
+    let separator = document.getElementById("openpgpCtxItemsSeparator");
+    let decryptOpenMenu = document.getElementById("enigmail_ctxDecryptOpen");
+    let decryptSaveMenu = document.getElementById("enigmail_ctxDecryptSave");
+    let importMenu = document.getElementById("enigmail_ctxImportKey");
+    let verifyMenu = document.getElementById("enigmail_ctxVerifyAtt");
 
-    let contextMenu, selectedAttachments;
-    // Thunderbird
-    contextMenu = document.getElementById("attachmentItemContext");
-    selectedAttachments = contextMenu.attachments;
+    if (contextMenu.attachments.length == 1) {
+      let attachment = contextMenu.attachments[0];
 
-    var decryptOpenMenu = document.getElementById("enigmail_ctxDecryptOpen");
-    var decryptSaveMenu = document.getElementById("enigmail_ctxDecryptSave");
-    var importMenu = document.getElementById("enigmail_ctxImportKey");
-    var verifyMenu = document.getElementById("enigmail_ctxVerifyAtt");
-
-    if (BondOpenPGP.isEnabled() && selectedAttachments.length > 0) {
-      this.enableContextMenuEntries(
-        selectedAttachments[0],
-        decryptOpenMenu,
-        decryptSaveMenu,
-        importMenu,
-        verifyMenu
-      );
-    } else {
-      //openMenu.setAttribute("disabled", true); /* global openMenu: false */
-      //saveMenu.setAttribute("disabled", true); /* global saveMenu: false */
-      decryptOpenMenu.setAttribute("disabled", true);
-      decryptSaveMenu.setAttribute("disabled", true);
-      importMenu.setAttribute("disabled", true);
-      verifyMenu.setAttribute("disabled", true);
-    }
-  },
-
-  enableContextMenuEntries(
-    attachment,
-    decryptOpenMenu,
-    decryptSaveMenu,
-    importMenu,
-    verifyMenu
-  ) {
-    if (attachment.contentType.search(/^application\/pgp-keys/i) === 0) {
-      importMenu.removeAttribute("disabled");
-      decryptOpenMenu.setAttribute("disabled", true);
-      decryptSaveMenu.setAttribute("disabled", true);
-      verifyMenu.setAttribute("disabled", true);
-    } else if (Enigmail.msg.checkEncryptedAttach(attachment)) {
-      if (
-        (typeof attachment.name !== "undefined" &&
-          attachment.name.match(/\.asc\.(gpg|pgp)$/i)) ||
-        (typeof attachment.displayName !== "undefined" &&
-          attachment.displayName.match(/\.asc\.(gpg|pgp)$/i))
-      ) {
-        importMenu.removeAttribute("disabled");
-      } else {
-        importMenu.setAttribute("disabled", true);
-      }
-      decryptOpenMenu.removeAttribute("disabled");
-      decryptSaveMenu.removeAttribute("disabled");
-      if (
+      if (/^application\/pgp-keys/i.test(attachment.contentType)) {
+        importMenu.hidden = false;
+        decryptOpenMenu.hidden = true;
+        decryptSaveMenu.hidden = true;
+        verifyMenu.hidden = true;
+      } else if (Enigmail.msg.checkEncryptedAttach(attachment)) {
+        if (
+          (typeof attachment.name !== "undefined" &&
+            attachment.name.match(/\.asc\.(gpg|pgp)$/i)) ||
+          (typeof attachment.displayName !== "undefined" &&
+            attachment.displayName.match(/\.asc\.(gpg|pgp)$/i))
+        ) {
+          importMenu.hidden = false;
+        } else {
+          importMenu.hidden = true;
+        }
+        decryptOpenMenu.hidden = false;
+        decryptSaveMenu.hidden = false;
+        if (
+          EnigmailMsgRead.checkSignedAttachment(
+            attachment,
+            null,
+            currentAttachments
+          )
+        ) {
+          verifyMenu.hidden = false;
+        } else {
+          verifyMenu.hidden = true;
+        }
+        if (typeof attachment.displayName == "undefined") {
+          if (!attachment.name) {
+            attachment.name = "message.pgp";
+          }
+        } else if (!attachment.displayName) {
+          attachment.displayName = "message.pgp";
+        }
+      } else if (
         EnigmailMsgRead.checkSignedAttachment(
           attachment,
           null,
           currentAttachments
         )
       ) {
-        verifyMenu.removeAttribute("disabled");
+        importMenu.hidden = true;
+        decryptOpenMenu.hidden = true;
+        decryptSaveMenu.hidden = true;
+
+        verifyMenu.hidden = false;
       } else {
-        verifyMenu.setAttribute("disabled", true);
+        importMenu.hidden = true;
+        decryptOpenMenu.hidden = true;
+        decryptSaveMenu.hidden = true;
+        verifyMenu.hidden = true;
       }
-      if (typeof attachment.displayName == "undefined") {
-        if (!attachment.name) {
-          attachment.name = "message.pgp";
-        }
-      } else if (!attachment.displayName) {
-        attachment.displayName = "message.pgp";
-      }
-    } else if (
-      EnigmailMsgRead.checkSignedAttachment(
-        attachment,
-        null,
-        currentAttachments
-      )
-    ) {
-      importMenu.setAttribute("disabled", true);
-      decryptOpenMenu.setAttribute("disabled", true);
-      decryptSaveMenu.setAttribute("disabled", true);
-      verifyMenu.removeAttribute("disabled");
+
+      separator.hidden =
+        decryptOpenMenu.hidden &&
+        decryptSaveMenu.hidden &&
+        importMenu.hidden &&
+        verifyMenu.hidden;
     } else {
-      importMenu.setAttribute("disabled", true);
-      decryptOpenMenu.setAttribute("disabled", true);
-      decryptSaveMenu.setAttribute("disabled", true);
-      verifyMenu.setAttribute("disabled", true);
+      decryptOpenMenu.hidden = true;
+      decryptSaveMenu.hidden = true;
+      importMenu.hidden = true;
+      verifyMenu.hidden = true;
+      separator.hidden = true;
     }
   },
 
@@ -929,6 +846,12 @@ Enigmail.hdrView = {
     EnigmailSingletons.messageReader = this.headerPane;
   },
 
+  clearWindowCallback() {
+    if (EnigmailSingletons.messageReader == this.headerPane) {
+      EnigmailSingletons.messageReader = null;
+    }
+  },
+
   headerPane: {
     isCurrentMessage(uri) {
       let uriSpec = uri ? uri.spec : null;
@@ -939,33 +862,24 @@ Enigmail.hdrView = {
           "\n"
       );
 
-      if (!uriSpec || uriSpec.search(/^enigmail:/) === 0) {
-        // we cannot compare if no URI given or if URI is Enigmail-internal;
-        // therefore assuming it's the current message
+      if (!uriSpec) {
+        // We cannot compare if no URI, => assume it's the current message.
         return true;
       }
 
       let msgUriSpec = Enigmail.msg.getCurrentMsgUriSpec();
-
-      let currUrl = EnigmailCompat.getUrlFromUriSpec(msgUriSpec);
+      let currUrl = EnigmailFuncs.getUrlFromUriSpec(msgUriSpec);
       if (!currUrl) {
-        EnigmailLog.DEBUG(
-          "enigmailMsgHdrViewOverlay.js: EnigMimeHeaderSink.isCurrentMessage: could not determine URL\n"
-        );
-        currUrl = {
-          host: "invalid",
-          path: "/message",
-          scheme: "enigmail",
-          spec: "enigmail://invalid/message",
-          schemeIs(s) {
-            return s === this.scheme;
-          },
-        };
+        return false;
       }
 
       let currMsgId = EnigmailURIs.msgIdentificationFromUrl(currUrl);
       let gotMsgId = EnigmailURIs.msgIdentificationFromUrl(uri);
 
+      if (!gotMsgId) {
+        return false;
+      }
+
       EnigmailLog.DEBUG(
         "enigmailMsgHdrViewOverlay.js: EnigMimeHeaderSink.isCurrentMessage: url=" +
           currUrl.spec +
@@ -1017,7 +931,7 @@ Enigmail.hdrView = {
      * Determine if a given mime part number should be displayed.
      * Returns true if one of these conditions is true:
      *  - this is the 1st displayed block of the message
-     *  - the message part displayed corresonds to the decrypted part
+     *  - the message part displayed corresponds to the decrypted part
      *
      * @param mimePartNumber: String - the MIME part number that was decrypted/verified
      * @param uriSpec:        String - the URI spec that is being displayed
@@ -1118,7 +1032,7 @@ Enigmail.hdrView = {
       extraDetails,
       mimePartNumber
     ) {
-      // uriSpec is not used for Enigmail anymore. It is here becaue other addons and pEp rely on it
+      // uriSpec is not used for Enigmail anymore. It is here because other addons and pEp rely on it
 
       EnigmailLog.DEBUG(
         "enigmailMsgHdrViewOverlay.js: updateSecurityStatus: mimePart=" +
@@ -1177,27 +1091,6 @@ Enigmail.hdrView = {
           mimePartNumber
         );
       }
-
-      if (uriSpec && uriSpec.search(/^enigmail:message\//) === 0) {
-        let buttons = [
-          {
-            "l10n-id": "openpgp-broken-exchange-repair",
-            popup: null,
-            callback(aNotification, aButton) {
-              Enigmail.msg.fixBuggyExchangeMail();
-              return false; // Close notification.
-            },
-          },
-        ];
-
-        Enigmail.msg.notificationBox.appendNotification(
-          await document.l10n.formatValue("openpgp-broken-exchange-info"),
-          "brokenExchange",
-          null,
-          Enigmail.msg.notificationBox.PRIORITY_WARNING_MEDIUM,
-          buttons
-        );
-      }
     },
 
     processDecryptionResult(uri, actionType, processData, mimePartNumber) {
@@ -1304,35 +1197,6 @@ Enigmail.hdrView = {
         Enigmail.msg.messageReload(false);
       }
     },
-
-    maxWantedNesting() {
-      EnigmailLog.DEBUG(
-        "enigmailMsgHdrViewOverlay.js: EnigMimeHeaderSink.maxWantedNesting:\n"
-      );
-      return this._smimeHeaderSink.maxWantedNesting();
-    },
-
-    signedStatus(aNestingLevel, aSignatureStatus, aSignerCert) {
-      EnigmailLog.DEBUG(
-        "enigmailMsgHdrViewOverlay.js: EnigMimeHeaderSink.signedStatus:\n"
-      );
-      return this._smimeHeaderSink.signedStatus(
-        aNestingLevel,
-        aSignatureStatus,
-        aSignerCert
-      );
-    },
-
-    encryptionStatus(aNestingLevel, aEncryptionStatus, aRecipientCert) {
-      EnigmailLog.DEBUG(
-        "enigmailMsgHdrViewOverlay.js: EnigMimeHeaderSink.encryptionStatus:\n"
-      );
-      return this._smimeHeaderSink.encryptionStatus(
-        aNestingLevel,
-        aEncryptionStatus,
-        aRecipientCert
-      );
-    },
   },
 
   /*
@@ -1396,3 +1260,4 @@ window.addEventListener(
   "load-enigmail",
   Enigmail.hdrView.hdrViewLoad.bind(Enigmail.hdrView)
 );
+window.addEventListener("unload", () => Enigmail.hdrView.clearWindowCallback());
