@@ -8,13 +8,6 @@
 
 "use strict";
 
-var elib = ChromeUtils.import(
-  "resource://testing-common/mozmill/elementslib.jsm"
-);
-var EventUtils = ChromeUtils.import(
-  "resource://testing-common/mozmill/EventUtils.jsm"
-);
-
 var { close_compose_window, open_compose_with_forward } = ChromeUtils.import(
   "resource://testing-common/mozmill/ComposeHelpers.jsm"
 );
@@ -43,11 +36,10 @@ var { SyntheticPartLeaf, SyntheticPartMultiMixed } = ChromeUtils.import(
 );
 
 var {
+  async_plan_for_new_window,
   close_window,
   plan_for_modal_dialog,
-  plan_for_new_window,
   wait_for_modal_dialog,
-  wait_for_new_window,
 } = ChromeUtils.import("resource://testing-common/mozmill/WindowHelpers.jsm");
 
 var { Services } = ChromeUtils.import("resource://gre/modules/Services.jsm");
@@ -66,8 +58,8 @@ var textAttachment =
 
 var binaryAttachment = textAttachment;
 
-add_task(function setupModule(module) {
-  folder = create_folder("AttachmentA");
+add_setup(async function() {
+  folder = await create_folder("AttachmentA");
 
   var attachedMessage = msgGen.makeMessage({
     body: { body: "I'm an attached email!" },
@@ -181,7 +173,7 @@ add_task(function setupModule(module) {
   }
 
   for (let i = 0; i < messages.length; i++) {
-    add_message_to_folder(folder, create_message(messages[i]));
+    await add_message_to_folder([folder], create_message(messages[i]));
   }
 });
 
@@ -273,7 +265,7 @@ add_task(function test_long_attachment_name() {
  * "beneath" the attached message (as opposed to all attachments for the root
  * message).
  */
-add_task(function test_attached_message_attachments() {
+add_task(async function test_attached_message_attachments() {
   be_in_folder(folder);
 
   select_click_row(5);
@@ -284,11 +276,11 @@ add_task(function test_attached_message_attachments() {
   Assert.equal(mc.e("attachmentList").itemCount, 3);
 
   // Open the attached email.
-  plan_for_new_window("mail:messageWindow");
+  let newWindowPromise = async_plan_for_new_window("mail:messageWindow");
   mc.e("attachmentList")
     .getItemAtIndex(1)
     .attachment.open();
-  let msgc = wait_for_new_window("mail:messageWindow");
+  let msgc = await newWindowPromise;
   wait_for_message_display_completion(msgc, true);
 
   // Make sure we have the expected number of attachments in the attached
@@ -314,7 +306,7 @@ add_task(function test_attachment_name_click() {
   // Ensure the open dialog appears when clicking on the attachment name and
   // that the attachment list doesn't expand.
   plan_for_modal_dialog("unknownContentTypeWindow", function() {});
-  mc.click(mc.eid("attachmentName"));
+  mc.click(mc.e("attachmentName"));
   wait_for_modal_dialog("unknownContentTypeWindow");
   Assert.ok(
     attachmentList.collapsed,
@@ -329,56 +321,84 @@ add_task(function test_attachment_name_click() {
  * @param elementId the id of the element to right click on
  * @param contextMenuId the id of the context menu that should appear
  */
-function subtest_attachment_right_click(elementId, contextMenuId) {
-  mc.rightClick(mc.eid(elementId));
-  wait_for_popup_to_open(mc.e(contextMenuId));
-  close_popup(mc, mc.eid(contextMenuId));
+async function subtest_attachment_right_click(elementId, contextMenuId) {
+  let element = document.getElementById(elementId);
+  let contextMenu = document.getElementById(contextMenuId);
+
+  let shownPromise = BrowserTestUtils.waitForEvent(contextMenu, "popupshown");
+  EventUtils.synthesizeMouseAtCenter(element, { type: "contextmenu" }, window);
+  await shownPromise;
+  let hiddenPromise = BrowserTestUtils.waitForEvent(contextMenu, "popuphidden");
+  contextMenu.hidePopup();
+  await hiddenPromise;
+  await new Promise(resolve => requestAnimationFrame(resolve));
 }
 
-add_task(function test_attachment_right_click_single() {
+add_task(async function test_attachment_right_click_single() {
   be_in_folder(folder);
 
   select_click_row(1);
   assert_selected_and_displayed(1);
 
-  subtest_attachment_right_click("attachmentIcon", "attachmentItemContext");
-  subtest_attachment_right_click("attachmentCount", "attachmentItemContext");
-  subtest_attachment_right_click("attachmentName", "attachmentItemContext");
-  subtest_attachment_right_click("attachmentSize", "attachmentItemContext");
+  await subtest_attachment_right_click(
+    "attachmentIcon",
+    "attachmentItemContext"
+  );
+  await subtest_attachment_right_click(
+    "attachmentCount",
+    "attachmentItemContext"
+  );
+  await subtest_attachment_right_click(
+    "attachmentName",
+    "attachmentItemContext"
+  );
+  await subtest_attachment_right_click(
+    "attachmentSize",
+    "attachmentItemContext"
+  );
 
-  subtest_attachment_right_click(
+  await subtest_attachment_right_click(
     "attachmentToggle",
     "attachment-toolbar-context-menu"
   );
-  subtest_attachment_right_click(
+  await subtest_attachment_right_click(
     "attachmentSaveAllSingle",
     "attachment-toolbar-context-menu"
   );
-  subtest_attachment_right_click(
+  await subtest_attachment_right_click(
     "attachmentBar",
     "attachment-toolbar-context-menu"
   );
 });
 
-add_task(function test_attachment_right_click_multiple() {
+add_task(async function test_attachment_right_click_multiple() {
   be_in_folder(folder);
 
   select_click_row(3);
   assert_selected_and_displayed(3);
 
-  subtest_attachment_right_click("attachmentIcon", "attachmentListContext");
-  subtest_attachment_right_click("attachmentCount", "attachmentListContext");
-  subtest_attachment_right_click("attachmentSize", "attachmentListContext");
+  await subtest_attachment_right_click(
+    "attachmentIcon",
+    "attachmentListContext"
+  );
+  await subtest_attachment_right_click(
+    "attachmentCount",
+    "attachmentListContext"
+  );
+  await subtest_attachment_right_click(
+    "attachmentSize",
+    "attachmentListContext"
+  );
 
-  subtest_attachment_right_click(
+  await subtest_attachment_right_click(
     "attachmentToggle",
     "attachment-toolbar-context-menu"
   );
-  subtest_attachment_right_click(
+  await subtest_attachment_right_click(
     "attachmentSaveAllMultiple",
     "attachment-toolbar-context-menu"
   );
-  subtest_attachment_right_click(
+  await subtest_attachment_right_click(
     "attachmentBar",
     "attachment-toolbar-context-menu"
   );
@@ -392,7 +412,7 @@ add_task(function test_attachment_right_click_multiple() {
  */
 function subtest_attachment_list_toggle(elementId) {
   let attachmentList = mc.e("attachmentList");
-  let element = mc.eid(elementId);
+  let element = mc.e(elementId);
 
   mc.click(element);
   Assert.ok(
@@ -429,10 +449,8 @@ add_task(function test_attachment_list_expansion() {
   // Ensure that clicking the "Save All" button doesn't expand the attachment
   // list.
   mc.click(
-    new elib.Elem(
-      mc
-        .e("attachmentSaveAllSingle")
-        .querySelector(".toolbarbutton-menubutton-dropmarker")
+    mc.window.document.querySelector(
+      "#attachmentSaveAllSingle .toolbarbutton-menubutton-dropmarker"
     )
   );
   Assert.ok(
@@ -461,7 +479,7 @@ add_task(function test_selected_attachments_are_cleared() {
   select_click_row(3);
 
   // Expand the attachment list.
-  mc.click(mc.eid("attachmentToggle"));
+  mc.click(mc.e("attachmentToggle"));
 
   // Select both the attachments.
   let attachmentList = mc.e("attachmentList");
@@ -473,7 +491,7 @@ add_task(function test_selected_attachments_are_cleared() {
 
   // We can just click on the first element, but the second one needs a
   // ctrl-click (or cmd-click for those Mac-heads among us).
-  mc.click(new elib.Elem(attachmentList.children[0]), 5, 5);
+  mc.click(attachmentList.children[0], 5, 5);
   EventUtils.synthesizeMouse(
     attachmentList.children[1],
     5,
@@ -493,7 +511,7 @@ add_task(function test_selected_attachments_are_cleared() {
   select_click_row(2);
 
   // Expand the attachment list again.
-  mc.click(mc.eid("attachmentToggle"));
+  mc.click(mc.e("attachmentToggle"));
 
   Assert.equal(
     attachmentList.selectedItems.length,
@@ -510,10 +528,11 @@ add_task(function test_select_all_attachments_key() {
   select_click_row(3);
 
   // Expand the attachment list.
-  mc.click(mc.eid("attachmentToggle"));
+  mc.click(mc.e("attachmentToggle"));
 
   let attachmentList = mc.e("attachmentList");
-  mc.keypress(new elib.Elem(attachmentList), "a", { accelKey: true });
+  attachmentList.focus();
+  EventUtils.synthesizeKey("a", { accelKey: true }, mc.window);
   Assert.equal(
     attachmentList.selectedItems.length,
     2,
@@ -521,7 +540,7 @@ add_task(function test_select_all_attachments_key() {
   );
 });
 
-add_task(function test_delete_attachment_key() {
+add_task(async function test_delete_attachment_key() {
   be_in_folder(folder);
 
   // First, select the message with two attachments.
@@ -529,24 +548,24 @@ add_task(function test_delete_attachment_key() {
   select_click_row(3);
 
   // Expand the attachment list.
-  mc.click(mc.eid("attachmentToggle"));
-
-  let firstAttachment = new elib.Elem(mc.e("attachmentList").firstElementChild);
+  assert_selected_and_displayed(3);
+  if (mc.e("attachmentList").collapsed) {
+    mc.click(mc.e("attachmentToggle"));
+  }
+  let firstAttachment = mc.e("attachmentList").firstElementChild;
   mc.click(firstAttachment, 5, 5);
 
   // Try deleting with the delete key
-  plan_for_modal_dialog("commonDialogWindow", function(cdc) {
-    cdc.window.document.documentElement.querySelector("dialog").cancelDialog();
-  });
-  mc.keypress(firstAttachment, "VK_DELETE", {});
-  wait_for_modal_dialog("commonDialogWindow");
+  let dialogPromise = BrowserTestUtils.promiseAlertDialog("cancel");
+  firstAttachment.focus();
+  EventUtils.synthesizeKey("VK_DELETE", {}, mc.window);
+  await dialogPromise;
 
   // Try deleting with the shift-delete key combo.
-  plan_for_modal_dialog("commonDialogWindow", function(cdc) {
-    cdc.window.document.documentElement.querySelector("dialog").cancelDialog();
-  });
-  mc.keypress(firstAttachment, "VK_DELETE", { shiftKey: true });
-  wait_for_modal_dialog("commonDialogWindow");
+  dialogPromise = BrowserTestUtils.promiseAlertDialog("cancel");
+  firstAttachment.focus();
+  EventUtils.synthesizeKey("VK_DELETE", { shiftKey: true }, mc.window);
+  await dialogPromise;
 });
 
 add_task(function test_attachments_compose_menu() {
@@ -574,7 +593,9 @@ add_task(function test_attachments_compose_menu() {
       // Walk up the DOM tree and call isCommandEnabled on the first controller
       // that supports "cmd_delete".
       while (element != cwc.window.document) {
-        for (let i = 0; i < element.controllers.getControllerCount(); i++) {
+        // NOTE: html elements (like body) don't have controllers.
+        let numControllers = element.controllers?.getControllerCount() || 0;
+        for (let i = 0; numControllers; i++) {
           let currController = element.controllers.getControllerAt(i);
           if (currController.supportsCommand("cmd_delete")) {
             currController.isCommandEnabled("cmd_delete");
@@ -586,15 +607,20 @@ add_task(function test_attachments_compose_menu() {
     }
   }
 
-  // Click on a portion of the attachmentBucket that will focus it, but not
-  // bring up the file picker
+  // Click on a portion of the attachmentBucket to focus on it. The last
+  // attachment should be selected since we don't handle any action on an empty
+  // bucket, and we always ensure that the last attached file is visible.
   force_focus("attachmentBucket");
+
   Assert.equal(
-    "Remove Attachments",
+    "Remove Attachment",
     cwc.e("cmd_delete").getAttribute("label"),
-    "attachmentBucket is focused!"
+    "attachmentBucket with last attachment is focused!"
   );
 
+  // We opened a message with 2 attachments, so index 1 should be focused.
+  Assert.equal(attachment.selectedIndex, 1, "Last attachment is focused!");
+
   // Select 1 attachment, and
   // focus the subject to see the label change and to execute isCommandEnabled
   attachment.selectedIndex = 0;
@@ -641,14 +667,22 @@ add_task(function test_delete_from_toolbar() {
   select_click_row(3);
 
   // Expand the attachment list.
-  mc.click(mc.eid("attachmentToggle"));
+  assert_selected_and_displayed(3);
+  if (mc.e("attachmentList").collapsed) {
+    mc.click(mc.e("attachmentToggle"));
+  }
 
-  let firstAttachment = new elib.Elem(mc.e("attachmentList").firstElementChild);
+  let firstAttachment = mc.e("attachmentList").firstElementChild;
   mc.click(firstAttachment, 5, 5);
 
   // Make sure clicking the "Delete" toolbar button with an attachment focused
   // deletes the *message*.
   plan_to_wait_for_folder_events("DeleteOrMoveMsgCompleted");
-  mc.click(mc.eid("hdrTrashButton"));
+  mc.click(mc.e("hdrTrashButton"));
   wait_for_folder_events();
 });
+
+registerCleanupFunction(() => {
+  // Remove created folders.
+  folder.deleteSelf(null);
+});
