@@ -8,10 +8,6 @@
 
 "use strict";
 
-var elementslib = ChromeUtils.import(
-  "resource://testing-common/mozmill/elementslib.jsm"
-);
-
 var {
   add_attachments,
   close_compose_window,
@@ -35,13 +31,13 @@ var {
   assert_notification_displayed,
   check_notification_displayed,
   get_notification_button,
+  get_notification,
   wait_for_notification_to_show,
   wait_for_notification_to_stop,
 } = ChromeUtils.import(
   "resource://testing-common/mozmill/NotificationBoxHelpers.jsm"
 );
 var {
-  close_window,
   plan_for_modal_dialog,
   plan_for_new_window,
   plan_for_window_close,
@@ -61,11 +57,11 @@ var kReminderPref = "mail.compose.attachment_reminder";
 var gDrafts;
 var gOutbox;
 
-add_task(function setupModule(module) {
+add_setup(async function() {
   requestLongerTimeout(4);
 
-  gDrafts = get_special_folder(Ci.nsMsgFolderFlags.Drafts, true);
-  gOutbox = get_special_folder(Ci.nsMsgFolderFlags.Queue);
+  gDrafts = await get_special_folder(Ci.nsMsgFolderFlags.Drafts, true);
+  gOutbox = await get_special_folder(Ci.nsMsgFolderFlags.Queue);
 
   Assert.ok(Services.prefs.getBoolPref(kReminderPref));
 });
@@ -131,14 +127,12 @@ function assert_manual_reminder_state(aCwc, aChecked) {
 /**
  * Returns the keywords string currently shown in the notification message.
  *
- * @param aCwc A compose window controller.
+ * @param {MozMillController} cwc - The compose window controller.
  */
-function get_reminder_keywords(aCwc) {
-  assert_automatic_reminder_state(aCwc, true);
-  let nBox = aCwc.e(kBoxId).querySelector(".notificationbox-stack")
-    ._notificationBox;
-  let notification = nBox.getNotificationWithValue(kNotificationId);
-  return notification
+function get_reminder_keywords(cwc) {
+  assert_automatic_reminder_state(cwc, true);
+  let box = get_notification(cwc, kBoxId, kNotificationId);
+  return box.messageText
     .querySelector("#attachmentKeywords")
     .getAttribute("value");
 }
@@ -146,7 +140,7 @@ function get_reminder_keywords(aCwc) {
 /**
  * Test that the attachment reminder works, in general.
  */
-add_task(function test_attachment_reminder_appears_properly() {
+add_task(async function test_attachment_reminder_appears_properly() {
   let cwc = open_compose_new_mail();
 
   // There should be no notification yet.
@@ -162,7 +156,7 @@ add_task(function test_attachment_reminder_appears_properly() {
   // Give the notification time to appear. It shouldn't.
   wait_for_reminder_state(cwc, false);
 
-  cwc.type(cwc.eid("content-frame"), "Seen this cool attachment?");
+  cwc.type(cwc.e("messageEditor"), "Seen this cool attachment?");
 
   // Give the notification time to appear. It should now.
   wait_for_reminder_state(cwc, true);
@@ -170,11 +164,12 @@ add_task(function test_attachment_reminder_appears_properly() {
   // The manual reminder should be disabled yet.
   assert_manual_reminder_state(cwc, false);
 
+  let box = get_notification(cwc, kBoxId, kNotificationId);
   // Click ok to be notified on send if no attachments are attached.
-  cwc.click(
-    new elementslib.Elem(
-      cwc.e(kBoxId).querySelector('button[label="Remind Me Later"]')
-    )
+  EventUtils.synthesizeMouseAtCenter(
+    box.buttonContainer.lastElementChild,
+    {},
+    cwc.window
   );
   wait_for_reminder_state(cwc, false);
 
@@ -182,9 +177,10 @@ add_task(function test_attachment_reminder_appears_properly() {
   assert_manual_reminder_state(cwc, true);
 
   // Now try to send, make sure we get the alert.
-  plan_for_modal_dialog("commonDialogWindow", click_oh_i_did);
-  cwc.click(cwc.eid("button-send"));
-  wait_for_modal_dialog("commonDialogWindow");
+  // Click the "Oh, I Did!" button in the attachment reminder dialog.
+  let dialogPromise = BrowserTestUtils.promiseAlertDialog("extra1");
+  cwc.click(cwc.e("button-send"));
+  await dialogPromise;
 
   // After confirming the reminder the menuitem should get disabled.
   assert_manual_reminder_state(cwc, false);
@@ -196,7 +192,7 @@ add_task(function test_attachment_reminder_appears_properly() {
  * Test that the alert appears normally, but not after closing the
  * notification.
  */
-add_task(function test_attachment_reminder_dismissal() {
+add_task(async function test_attachment_reminder_dismissal() {
   let cwc = open_compose_new_mail();
 
   // There should be no notification yet.
@@ -218,9 +214,10 @@ add_task(function test_attachment_reminder_dismissal() {
 
   // We didn't click the "Remind Me Later" - the alert should pop up
   // on send anyway.
-  plan_for_modal_dialog("commonDialogWindow", click_oh_i_did);
-  cwc.click(cwc.eid("button-send"));
-  wait_for_modal_dialog("commonDialogWindow");
+  // Click the "Oh, I Did!" button in the attachment reminder dialog.
+  let dialogPromise = BrowserTestUtils.promiseAlertDialog("extra1");
+  cwc.click(cwc.e("button-send"));
+  await dialogPromise;
 
   let notification = assert_automatic_reminder_state(cwc, true);
 
@@ -254,13 +251,13 @@ add_task(function test_attachment_reminder_with_attachment() {
 
   // Add an attachment.
   let file = Services.dirsvc.get("ProfD", Ci.nsIFile);
-  file.append("panacea.dat");
+  file.append("prefs.js");
   Assert.ok(
     file.exists(),
-    "The required file panacea.dat was not found in the profile."
+    "The required file prefs.js was not found in the profile."
   );
-  let attachment = [cwc.window.FileToAttachment(file)];
-  cwc.window.AddAttachments(attachment);
+  let attachments = [cwc.window.FileToAttachment(file)];
+  cwc.window.AddAttachments(attachments);
 
   // The notification should hide.
   wait_for_reminder_state(cwc, false);
@@ -315,7 +312,7 @@ add_task(function test_attachment_reminder_aggressive_pref() {
  * Test that clicking "No, Send Now" in the attachment reminder alert
  * works.
  */
-add_task(function test_no_send_now_sends() {
+add_task(async function test_no_send_now_sends() {
   let cwc = open_compose_new_mail();
 
   setup_msg_contents(
@@ -328,9 +325,9 @@ add_task(function test_no_send_now_sends() {
   wait_for_reminder_state(cwc, true);
 
   // Click the send button again, this time choose "No, Send Now".
-  plan_for_modal_dialog("commonDialogWindow", click_no_send_now);
-  cwc.click(cwc.eid("button-send"));
-  wait_for_modal_dialog("commonDialogWindow");
+  let dialogPromise = BrowserTestUtils.promiseAlertDialog("accept");
+  cwc.click(cwc.e("button-send"));
+  await dialogPromise;
 
   // After clicking "Send Now" sending is proceeding, just handle the error.
   click_send_and_handle_send_error(cwc, true);
@@ -346,18 +343,25 @@ add_task(function test_no_send_now_sends() {
  * @param aExpectedState  A boolean specifying what is the expected state
  *                        of the reminder menuitem after the click.
  */
-function click_manual_reminder(aCwc, aExpectedState) {
+async function click_manual_reminder(aCwc, aExpectedState) {
   wait_for_window_focused(aCwc.window);
   let button = aCwc.window.document.getElementById("button-attach");
 
-  aCwc.click(
-    new elementslib.Elem(
-      button.querySelector(".toolbarbutton-menubutton-dropmarker")
-    )
+  let popup = aCwc.e("button-attachPopup");
+  let shownPromise = BrowserTestUtils.waitForEvent(popup, "popupshown");
+  EventUtils.synthesizeMouseAtCenter(
+    button.querySelector(".toolbarbutton-menubutton-dropmarker"),
+    {},
+    aCwc.window
   );
-  aCwc.click_menus_in_sequence(aCwc.e("button-attachPopup"), [
-    { id: "button-attachPopup_remindLaterItem" },
-  ]);
+  await shownPromise;
+  let hiddenPromise = BrowserTestUtils.waitForEvent(popup, "popuphidden");
+  EventUtils.synthesizeMouseAtCenter(
+    aCwc.e("button-attachPopup_remindLaterItem"),
+    {},
+    aCwc.window
+  );
+  await hiddenPromise;
   wait_for_window_focused(aCwc.window);
   assert_manual_reminder_state(aCwc, aExpectedState);
 }
@@ -366,7 +370,7 @@ function click_manual_reminder(aCwc, aExpectedState) {
  * Bug 521128
  * Test proper behaviour of the manual reminder.
  */
-add_task(function test_manual_attachment_reminder() {
+add_task(async function test_manual_attachment_reminder() {
   // Open a sample message with no attachment keywords.
   let cwc = open_compose_new_mail();
   setup_msg_contents(
@@ -377,7 +381,7 @@ add_task(function test_manual_attachment_reminder() {
   );
 
   // Enable the manual reminder.
-  click_manual_reminder(cwc, true);
+  await click_manual_reminder(cwc, true);
   // There should be no attachment notification.
   assert_automatic_reminder_state(cwc, false);
 
@@ -403,10 +407,9 @@ add_task(function test_manual_attachment_reminder() {
   wait_for_notification_to_show(mc, "mail-notification-top", "draftMsgContent");
   // Edit the draft again...
   plan_for_new_window("msgcompose");
+  let box = get_notification(mc, "mail-notification-top", "draftMsgContent");
   // ... by clicking Edit in the draft message notification bar.
-  mc.click(
-    mc.eid("mail-notification-top", { tagName: "button", label: "Edit" })
-  );
+  EventUtils.synthesizeMouseAtCenter(box.buttonContainer.firstElementChild, {});
   cwc = wait_for_compose_window();
 
   // Check the reminder enablement was preserved in the message.
@@ -415,17 +418,18 @@ add_task(function test_manual_attachment_reminder() {
   assert_automatic_reminder_state(cwc, false);
 
   // Now try to send, make sure we get the alert.
-  plan_for_modal_dialog("commonDialogWindow", click_oh_i_did);
-  cwc.click(cwc.eid("button-send"));
-  wait_for_modal_dialog("commonDialogWindow");
+  // Click the "Oh, I Did!" button in the attachment reminder dialog.
+  let dialogPromise = BrowserTestUtils.promiseAlertDialog("extra1");
+  cwc.click(cwc.e("button-send"));
+  await dialogPromise;
 
   // We were alerted once and the manual reminder is automatically turned off.
   assert_manual_reminder_state(cwc, false);
 
   // Enable the manual reminder and disable it again to see if it toggles right.
-  click_manual_reminder(cwc, true);
+  await click_manual_reminder(cwc, true);
   cwc.sleep(2000);
-  click_manual_reminder(cwc, false);
+  await click_manual_reminder(cwc, false);
 
   // Now try to send again, there should be no more alert.
   click_send_and_handle_send_error(cwc);
@@ -440,54 +444,56 @@ add_task(function test_manual_attachment_reminder() {
  * Bug 938759
  * Test hiding of the automatic notification if the manual reminder is set.
  */
-add_task(function test_manual_automatic_attachment_reminder_interaction() {
-  // Open a blank message compose
-  let cwc = open_compose_new_mail();
-  // This one should have the reminder disabled.
-  assert_manual_reminder_state(cwc, false);
-  // There should be no attachment notification.
-  assert_automatic_reminder_state(cwc, false);
+add_task(
+  async function test_manual_automatic_attachment_reminder_interaction() {
+    // Open a blank message compose
+    let cwc = open_compose_new_mail();
+    // This one should have the reminder disabled.
+    assert_manual_reminder_state(cwc, false);
+    // There should be no attachment notification.
+    assert_automatic_reminder_state(cwc, false);
 
-  // Add some attachment keywords.
-  setup_msg_contents(
-    cwc,
-    "test@example.invalid",
-    "Testing manual reminder!",
-    "Expect an attachment here..."
-  );
+    // Add some attachment keywords.
+    setup_msg_contents(
+      cwc,
+      "test@example.invalid",
+      "Testing manual reminder!",
+      "Expect an attachment here..."
+    );
 
-  // The automatic attachment notification should pop up.
-  wait_for_reminder_state(cwc, true);
+    // The automatic attachment notification should pop up.
+    wait_for_reminder_state(cwc, true);
 
-  // Now enable the manual reminder.
-  click_manual_reminder(cwc, true);
-  // The attachment notification should disappear.
-  wait_for_reminder_state(cwc, false);
+    // Now enable the manual reminder.
+    await click_manual_reminder(cwc, true);
+    // The attachment notification should disappear.
+    wait_for_reminder_state(cwc, false);
 
-  // Add some more text so the automatic notification
-  // could potentially show up.
-  setup_msg_contents(cwc, "", "", " and look for your attachment!");
-  // Give the notification time to appear. It shouldn't.
-  wait_for_reminder_state(cwc, false);
+    // Add some more text so the automatic notification
+    // could potentially show up.
+    setup_msg_contents(cwc, "", "", " and look for your attachment!");
+    // Give the notification time to appear. It shouldn't.
+    wait_for_reminder_state(cwc, false);
 
-  // Now disable the manual reminder.
-  click_manual_reminder(cwc, false);
-  // Give the notification time to appear. It shouldn't.
-  wait_for_reminder_state(cwc, false);
+    // Now disable the manual reminder.
+    await click_manual_reminder(cwc, false);
+    // Give the notification time to appear. It shouldn't.
+    wait_for_reminder_state(cwc, false);
 
-  // Add some more text without keywords.
-  setup_msg_contents(cwc, "", "", " No keywords here.");
-  // Give the notification time to appear. It shouldn't.
-  wait_for_reminder_state(cwc, false);
+    // Add some more text without keywords.
+    setup_msg_contents(cwc, "", "", " No keywords here.");
+    // Give the notification time to appear. It shouldn't.
+    wait_for_reminder_state(cwc, false);
 
-  // Add some more text with a new keyword.
-  setup_msg_contents(cwc, "", "", " Do you find it attached?");
-  // Give the notification time to appear. It should now.
-  wait_for_reminder_state(cwc, true);
-  Assert.equal(get_reminder_keywords(cwc), "attachment, attached");
+    // Add some more text with a new keyword.
+    setup_msg_contents(cwc, "", "", " Do you find it attached?");
+    // Give the notification time to appear. It should now.
+    wait_for_reminder_state(cwc, true);
+    Assert.equal(get_reminder_keywords(cwc), "attachment, attached");
 
-  close_compose_window(cwc);
-});
+    close_compose_window(cwc);
+  }
+);
 
 /**
  * Assert if there is any notification in the compose window.
@@ -524,7 +530,7 @@ add_task(function test_attachment_vs_filelink_reminder() {
   let kOfferThreshold = "mail.compose.big_attachments.threshold_kb";
   let maxSize = Services.prefs.getIntPref(kOfferThreshold, 0) * 1024;
   let file = Services.dirsvc.get("ProfD", Ci.nsIFile);
-  file.append("panacea.dat");
+  file.append("prefs.js");
   add_attachments(cwc, Services.io.newFileURI(file).spec, maxSize);
 
   // The filelink attachment proposal should be up but not the attachment
@@ -533,7 +539,7 @@ add_task(function test_attachment_vs_filelink_reminder() {
   assert_automatic_reminder_state(cwc, false);
 
   click_send_and_handle_send_error(cwc);
-  close_window(cwc);
+  close_compose_window(cwc);
 });
 
 /**
@@ -561,7 +567,7 @@ add_task(function test_attachment_reminder_in_subject() {
   Assert.equal(get_reminder_keywords(cwc), "attachment");
 
   // Now clear the subject
-  delete_all_existing(cwc, cwc.eid("msgSubject"));
+  delete_all_existing(cwc, cwc.e("msgSubject"));
 
   // Give the notification time to disappear.
   wait_for_reminder_state(cwc, false);
@@ -595,7 +601,7 @@ add_task(function test_attachment_reminder_in_subject_and_body() {
   Assert.equal(get_reminder_keywords(cwc), "attachment, attached");
 
   // Now clear only the subject
-  delete_all_existing(cwc, cwc.eid("msgSubject"));
+  delete_all_existing(cwc, cwc.e("msgSubject"));
 
   // Give the notification some time. It should not disappear,
   // just reduce the keywords list.
@@ -610,7 +616,7 @@ add_task(function test_attachment_reminder_in_subject_and_body() {
  * Test proper behaviour of attachment reminder when keyword reminding
  * is turned off.
  */
-add_task(function test_disabled_attachment_reminder() {
+add_task(async function test_disabled_attachment_reminder() {
   Services.prefs.setBoolPref(kReminderPref, false);
 
   // Open a sample message with no attachment keywords.
@@ -634,12 +640,12 @@ add_task(function test_disabled_attachment_reminder() {
   wait_for_reminder_state(cwc, false);
 
   // Enable the manual reminder.
-  click_manual_reminder(cwc, true);
+  await click_manual_reminder(cwc, true);
   assert_automatic_reminder_state(cwc, false);
 
   // Disable the manual reminder and the notification should still be hidden
   // even when there are still keywords in the body.
-  click_manual_reminder(cwc, false);
+  await click_manual_reminder(cwc, false);
   assert_automatic_reminder_state(cwc, false);
 
   // There should be no attachment message upon send.
@@ -680,6 +686,7 @@ add_task(function test_reminder_in_draft() {
   plan_for_modal_dialog("commonDialogWindow", click_save_message);
   cwc.window.goDoCommand("cmd_close");
   wait_for_modal_dialog("commonDialogWindow");
+  waitForSaveOperation(cwc);
 
   // The draft message was saved into Local Folders/Drafts.
   be_in_folder(gDrafts);
@@ -689,10 +696,9 @@ add_task(function test_reminder_in_draft() {
   wait_for_notification_to_show(mc, "mail-notification-top", "draftMsgContent");
   // Edit the draft again...
   plan_for_new_window("msgcompose");
+  let box = get_notification(mc, "mail-notification-top", "draftMsgContent");
   // ... by clicking Edit in the draft message notification bar.
-  mc.click(
-    mc.eid("mail-notification-top", { tagName: "button", label: "Edit" })
-  );
+  EventUtils.synthesizeMouseAtCenter(box.buttonContainer.firstElementChild, {});
   cwc = wait_for_compose_window();
 
   // Give the notification time to appear.
@@ -708,7 +714,7 @@ add_task(function test_reminder_in_draft() {
  * Bug 942436
  * Test that the reminder can be turned off for the current message.
  */
-add_task(function test_disabling_attachment_reminder() {
+add_task(async function test_disabling_attachment_reminder() {
   // Open a sample message with attachment keywords.
   let cwc = open_compose_new_mail();
   setup_msg_contents(
@@ -728,10 +734,11 @@ add_task(function test_disabling_attachment_reminder() {
   let disableButton = get_notification_button(cwc, kBoxId, kNotificationId, {
     popup: "reminderBarPopup",
   });
-  cwc.click(new elementslib.Elem(disableButton.querySelector("dropmarker")));
-  cwc.click_menus_in_sequence(cwc.e("reminderBarPopup"), [
-    { id: "disableReminder" },
-  ]);
+  cwc.click(disableButton.querySelector("dropmarker"));
+  await cwc.click_menus_in_sequence(
+    disableButton.closest("button").querySelector("menupopup"),
+    [{ id: "disableReminder" }]
+  );
 
   wait_for_reminder_state(cwc, false);
 
@@ -742,12 +749,12 @@ add_task(function test_disabling_attachment_reminder() {
 
   // Enable the manual reminder.
   // This overrides the previous explicit disabling of any reminder.
-  click_manual_reminder(cwc, true);
+  await click_manual_reminder(cwc, true);
   assert_automatic_reminder_state(cwc, false);
 
   // Disable the manual reminder and the notification should still be hidden
   // even when there are still keywords in the body.
-  click_manual_reminder(cwc, false);
+  await click_manual_reminder(cwc, false);
   assert_automatic_reminder_state(cwc, false);
 
   // Add more keywords to trigger automatic reminder.
@@ -759,10 +766,11 @@ add_task(function test_disabling_attachment_reminder() {
   disableButton = get_notification_button(cwc, kBoxId, kNotificationId, {
     popup: "reminderBarPopup",
   });
-  cwc.click(new elementslib.Elem(disableButton.querySelector("dropmarker")));
-  cwc.click_menus_in_sequence(cwc.e("reminderBarPopup"), [
-    { id: "disableReminder" },
-  ]);
+  cwc.click(disableButton.querySelector("dropmarker"));
+  await cwc.click_menus_in_sequence(
+    disableButton.closest("button").querySelector("menupopup"),
+    [{ id: "disableReminder" }]
+  );
   wait_for_reminder_state(cwc, false);
 
   // Now send the message.
@@ -793,31 +801,11 @@ add_task(function test_disabling_attachment_reminder() {
 function click_send_and_handle_send_error(aController, aAlreadySending) {
   plan_for_modal_dialog("commonDialogWindow", click_ok_on_send_error);
   if (!aAlreadySending) {
-    aController.click(aController.eid("button-send"));
+    aController.click(aController.e("button-send"));
   }
   wait_for_modal_dialog("commonDialogWindow");
 }
 
-/**
- * Click the "Oh, I Did!" button in the attachment reminder dialog.
- */
-function click_oh_i_did(controller) {
-  controller.window.document
-    .querySelector("dialog")
-    .getButton("extra1")
-    .doCommand();
-}
-
-/**
- * Click the "No, Send Now" button in the attachment reminder dialog.
- */
-function click_no_send_now(controller) {
-  controller.window.document
-    .querySelector("dialog")
-    .getButton("accept")
-    .doCommand();
-}
-
 /**
  * Click Ok in the Send Message Error dialog.
  */
