@@ -17,18 +17,21 @@
  *
  *   messenger
  *   gDBView
- *   MsgJunkMailInfo(aCheckFirstUse)
  *   msgWindow
  */
 
 /* globals ClearMessagePane, gDBView, gFolderDisplay, MarkSelectedMessagesRead, messenger,
-   MsgJunkMailInfo, msgWindow, nsMsgViewIndex_None */
+   msgWindow, nsMsgViewIndex_None */
 
 var { Services } = ChromeUtils.import("resource://gre/modules/Services.jsm");
 var { MailServices } = ChromeUtils.import(
   "resource:///modules/MailServices.jsm"
 );
-var { MailUtils } = ChromeUtils.import("resource:///modules/MailUtils.jsm");
+ChromeUtils.defineModuleGetter(
+  this,
+  "MailUtils",
+  "resource:///modules/MailUtils.jsm"
+);
 
 /*
  * determineActionsForJunkMsgs
@@ -77,37 +80,21 @@ function determineActionsForJunkMsgs(aFolder) {
  *
  * Performs required operations on a list of newly-classified junk messages
  *
- * @param aFolder
- *        the folder with messages being marked as junk
- *
- * @param aJunkMsgHdrs
- *        nsIArray containing headers (nsIMsgDBHdr) of new junk messages
- *
- * @param aGoodMsgHdrs
- *        nsIArray containing headers (nsIMsgDBHdr) of new good messages
+ * @param {nsIMsgFolder} aFolder - The folder with messages being marked as
+ *                                 junk.
+ * @param {Array.<nsIMsgDBHdr>} aJunkMsgHdrs - New junk messages.
+ * @param {Array.<nsIMsgDBHdr>} aGoodMsgHdrs - New good messages.
  */
 function performActionsOnJunkMsgs(aFolder, aJunkMsgHdrs, aGoodMsgHdrs) {
   if (aFolder instanceof Ci.nsIMsgImapMailFolder) {
     // need to update IMAP custom flags
     if (aJunkMsgHdrs.length) {
-      var junkMsgKeys = [];
-      for (let i = 0; i < aJunkMsgHdrs.length; i++) {
-        junkMsgKeys[i] = aJunkMsgHdrs.queryElementAt(
-          i,
-          Ci.nsIMsgDBHdr
-        ).messageKey;
-      }
+      let junkMsgKeys = aJunkMsgHdrs.map(hdr => hdr.messageKey);
       aFolder.storeCustomKeywords(null, "Junk", "NonJunk", junkMsgKeys);
     }
 
     if (aGoodMsgHdrs.length) {
-      let goodMsgKeys = [];
-      for (let i = 0; i < aGoodMsgHdrs.length; i++) {
-        goodMsgKeys[i] = aGoodMsgHdrs.queryElementAt(
-          i,
-          Ci.nsIMsgDBHdr
-        ).messageKey;
-      }
+      let goodMsgKeys = aGoodMsgHdrs.map(hdr => hdr.messageKey);
       aFolder.storeCustomKeywords(null, "NonJunk", "Junk", goodMsgKeys);
     }
   }
@@ -119,7 +106,7 @@ function performActionsOnJunkMsgs(aFolder, aJunkMsgHdrs, aGoodMsgHdrs) {
     }
 
     if (actionParams.junkTargetFolder) {
-      MailServices.copy.CopyMessages(
+      MailServices.copy.copyMessages(
         aFolder,
         aJunkMsgHdrs,
         actionParams.junkTargetFolder,
@@ -146,12 +133,8 @@ function performActionsOnJunkMsgs(aFolder, aJunkMsgHdrs, aGoodMsgHdrs) {
 
 function MessageClassifier(aFolder, aTotalMessages) {
   this.mFolder = aFolder;
-  this.mJunkMsgHdrs = Cc["@mozilla.org/array;1"].createInstance(
-    Ci.nsIMutableArray
-  );
-  this.mGoodMsgHdrs = Cc["@mozilla.org/array;1"].createInstance(
-    Ci.nsIMutableArray
-  );
+  this.mJunkMsgHdrs = [];
+  this.mGoodMsgHdrs = [];
   this.mMessages = {};
   this.mMessageQueue = [];
   this.mTotalMessages = aTotalMessages;
@@ -189,7 +172,7 @@ MessageClassifier.prototype = {
         Ci.nsIJunkMailPlugin.IS_HAM_SCORE
       );
       db.setStringProperty(aMsgHdr.messageKey, "junkscoreorigin", "whitelist");
-      this.mGoodMsgHdrs.appendElement(aMsgHdr);
+      this.mGoodMsgHdrs.push(aMsgHdr);
       return;
     }
 
@@ -240,9 +223,9 @@ MessageClassifier.prototype = {
     db.setStringProperty(msgHdr.messageKey, "junkpercent", aJunkPercent);
 
     if (aClassification == nsIJunkMailPlugin.JUNK) {
-      this.mJunkMsgHdrs.appendElement(msgHdr);
+      this.mJunkMsgHdrs.push(msgHdr);
     } else if (aClassification == nsIJunkMailPlugin.GOOD) {
-      this.mGoodMsgHdrs.appendElement(msgHdr);
+      this.mGoodMsgHdrs.push(msgHdr);
     }
 
     var nextMsgURI = this.mMessageQueue.shift();
@@ -307,8 +290,6 @@ function analyzeMessagesForJunk() {
  * @param aAll: true to filter all messages, else filter selection
  */
 function processFolderForJunk(aAll) {
-  MsgJunkMailInfo(true);
-
   let indices;
   if (aAll) {
     // need to expand all threads, so we analyze everything
@@ -376,8 +357,6 @@ function processFolderForJunk(aAll) {
 }
 
 function JunkSelectedMessages(setAsJunk) {
-  MsgJunkMailInfo(true);
-
   // When the user explicitly marks a message as junk, we can mark it as read,
   // too. This is independent of the "markAsReadOnSpam" pref, which applies
   // only to automatically-classified messages.
@@ -403,18 +382,14 @@ function JunkSelectedMessages(setAsJunk) {
  * @returns The number of messages deleted.
  */
 function deleteJunkInFolder() {
-  MsgJunkMailInfo(true);
-
   // use direct folder commands if possible so we don't mess with the selection
   let selectedFolder = gFolderDisplay.displayedFolder;
   if (!selectedFolder.getFlag(Ci.nsMsgFolderFlags.Virtual)) {
-    var junkMsgHdrs = Cc["@mozilla.org/array;1"].createInstance(
-      Ci.nsIMutableArray
-    );
+    let junkMsgHdrs = [];
     for (let msgHdr of gDBView.msgFolder.messages) {
       let junkScore = msgHdr.getStringProperty("junkscore");
       if (junkScore == Ci.nsIJunkMailPlugin.IS_SPAM_SCORE) {
-        junkMsgHdrs.appendElement(msgHdr);
+        junkMsgHdrs.push(msgHdr);
       }
     }
 
