@@ -10,7 +10,8 @@ var { Services } = ChromeUtils.import("resource://gre/modules/Services.jsm");
 var { MailServices } = ChromeUtils.import(
   "resource:///modules/MailServices.jsm"
 );
-var l10n = new Localization(["messenger/accountCentral.ftl"], true);
+var { UIDensity } = ChromeUtils.import("resource:///modules/UIDensity.jsm");
+var { UIFontSize } = ChromeUtils.import("resource:///modules/UIFontSize.jsm");
 
 var gSelectedServer = null;
 var gSelectedFolder = null;
@@ -20,15 +21,16 @@ var gSelectedFolder = null;
  * The folder is passed in via the document URL.
  */
 function OnInit() {
-  document.getElementById("setupTitle").textContent = l10n.formatValueSync(
-    "setup-title",
-    {
-      accounts: MailServices.accounts.accounts.length,
-    }
-  );
+  let el = document.getElementById("setupTitle");
+
+  document.l10n.setAttributes(el, "setup-title", {
+    accounts: MailServices.accounts.accounts.length,
+  });
 
   // Selected folder URI is passed as folderURI argument in the query string.
-  let folderURI = document.location.search.replace("?folderURI=", "");
+  let folderURI = decodeURIComponent(
+    document.location.search.replace("?folderURI=", "")
+  );
   gSelectedFolder = folderURI ? MailUtils.getExistingFolder(folderURI) : null;
   gSelectedServer = gSelectedFolder ? gSelectedFolder.server : null;
 
@@ -51,34 +53,8 @@ function OnInit() {
     document.getElementById("accountFeaturesSection").hidden = true;
   }
 
-  addDonationLink();
-}
-
-/**
- * Append the donation link at the end of the second paragraph.
- */
-function addDonationLink() {
-  let donationLink = document.createElement("a");
-  donationLink.setAttribute(
-    "href",
-    "https://give.thunderbird.net/en-US/?utm_source=start_page_tb_release&utm_medium=referral&utm_content=paragraph_text"
-  );
-  donationLink.classList.add("donation-link");
-  donationLink.setAttribute("tabindex", 0);
-  donationLink.addEventListener("click", event => {
-    openLink(event);
-  });
-  donationLink.addEventListener("keypress", event => {
-    if (event.key == "Enter") {
-      openLink(event);
-    }
-  });
-  donationLink.textContent = l10n.formatValueSync("about-donation");
-
-  let donationParagraph = document.getElementById("donationParagraph");
-  donationParagraph.append(" ");
-  donationParagraph.appendChild(donationLink);
-  donationParagraph.append(".");
+  UIDensity.registerWindow(window);
+  UIFontSize.registerWindow(window);
 }
 
 /**
@@ -193,7 +169,7 @@ function readMessages() {
   }
 
   try {
-    window.parent.OpenInboxForServer(gSelectedServer);
+    window.browsingContext.topChromeWindow.OpenInboxForServer(gSelectedServer);
   } catch (ex) {
     Cu.reportError("Error opening Inbox for server: " + ex + "\n");
   }
@@ -208,36 +184,24 @@ function readMessages() {
  *   'am-smtp.xhtml'
  */
 function viewSettings(selectPage) {
-  window.parent.MsgAccountManager(selectPage, gSelectedServer);
-}
-
-/**
- * Open the newsgroup account wizard.
- */
-function createNewsgroups() {
-  window.parent.msgOpenAccountWizard(function(state) {
-    updateMailPaneUI();
-    let win = getMostRecentMailWindow();
-    if (state && win && win.gFolderTreeView && this.gCurrentAccount) {
-      win.gFolderTreeView.selectFolder(
-        this.gCurrentAccount.incomingServer.rootMsgFolder
-      );
-    }
-  });
+  window.browsingContext.topChromeWindow.MsgAccountManager(
+    selectPage,
+    gSelectedServer
+  );
 }
 
 /**
  * Bring up the search interface for selected account.
  */
 function searchMessages() {
-  window.parent.MsgSearchMessages(gSelectedFolder);
+  window.browsingContext.topChromeWindow.MsgSearchMessages(gSelectedFolder);
 }
 
 /**
  * Open the filters window.
  */
 function createMsgFilters() {
-  window.parent.MsgFilters(null, gSelectedFolder);
+  window.browsingContext.topChromeWindow.MsgFilters(null, gSelectedFolder);
 }
 
 /**
@@ -248,9 +212,11 @@ function subscribe() {
     return;
   }
   if (gSelectedServer.type == "rss") {
-    window.parent.openSubscriptionsDialog(gSelectedServer.rootFolder);
+    window.browsingContext.topChromeWindow.openSubscriptionsDialog(
+      gSelectedServer.rootFolder
+    );
   } else {
-    window.parent.MsgSubscribe(gSelectedFolder);
+    window.browsingContext.topChromeWindow.MsgSubscribe(gSelectedFolder);
   }
 }
 
