@@ -10,15 +10,34 @@
 var { MailUtils } = ChromeUtils.import("resource:///modules/MailUtils.jsm");
 var { Services } = ChromeUtils.import("resource://gre/modules/Services.jsm");
 var { FileUtils } = ChromeUtils.import("resource://gre/modules/FileUtils.jsm");
-var { Log } = ChromeUtils.import("resource://gre/modules/Log.jsm");
-var { allAccountsSorted } = ChromeUtils.import(
-  "resource:///modules/folderUtils.jsm"
+ChromeUtils.defineModuleGetter(
+  this,
+  "FolderUtils",
+  "resource:///modules/FolderUtils.jsm"
 );
 var MailstoreConverter = ChromeUtils.import(
   "resource:///modules/mailstoreConverter.jsm"
 );
+var { AppConstants } = ChromeUtils.import(
+  "resource://gre/modules/AppConstants.jsm"
+);
 
-var log = Log.repository.getLogger("MailStoreConverter");
+// Set platform dependent icon src.
+window.addEventListener("DOMContentLoaded", () => {
+  let src =
+    AppConstants.platform == "linux"
+      ? "moz-icon://stock/gtk-dialog-warning?size=dialog"
+      : "chrome://global/skin/icons/warning-large.png";
+  for (let img of document.querySelectorAll(".infoIcon")) {
+    img.setAttribute("src", src);
+  }
+});
+
+var log = console.createInstance({
+  prefix: "mail.mailstoreconverter",
+  maxLogLevel: "Warn",
+  maxLogLevelPref: "mail.mailstoreconverter.loglevel",
+});
 // {nsIMsgIncomingServer} server for the account to be migrated.
 var gServer;
 // {nsIMsgFolder} account root folder.
@@ -55,7 +74,7 @@ function placeAccountName(aServer) {
   // Account to which other accounts have been deferred.
   let deferredToAccount;
   // Array of all accounts.
-  let accounts = allAccountsSorted(true);
+  let accounts = FolderUtils.allAccountsSorted(true);
 
   for (let account of accounts) {
     if (
@@ -206,14 +225,14 @@ function startContinue(aSelectedStoreType, aResponse) {
   document.getElementById("progress").addEventListener("progress", function(e) {
     document.getElementById("progress").value = e.detail;
     document.getElementById(
-      "progressPrcnt"
+      "progressPercent"
     ).textContent = bundle.formatStringFromName("converterDialog.percentDone", [
       e.detail,
     ]);
   });
 
-  document.getElementById("warning").setAttribute("hidden", "hidden");
-  document.getElementById("progressDiv").removeAttribute("hidden");
+  document.getElementById("warningArea").hidden = true;
+  document.getElementById("progressArea").hidden = false;
 
   // Storing original prefs and root folder path
   // to revert changes in case of error.
@@ -240,9 +259,9 @@ function startContinue(aSelectedStoreType, aResponse) {
    */
   function promiseRejected(aReason) {
     log.error("Conversion to '" + aSelectedStoreType + "' failed: " + aReason);
-    document.getElementById("messageSpan").style.display = "none";
+    document.getElementById("messageSpan").hidden = true;
 
-    document.getElementById("errorSpan").style.display = "block";
+    document.getElementById("errorSpan").hidden = false;
     gResponse.newRootFolder = null;
 
     // Revert prefs.
@@ -279,10 +298,10 @@ function startContinue(aSelectedStoreType, aResponse) {
     }
 
     Services.io.offline = gOriginalOffline;
-    document.getElementById("cancel").style.display = "none";
-    document.getElementById("finish").style.display = "inline-block";
-    document.getElementById("messageSpan").style.display = "none";
-    document.getElementById("completeSpan").style.display = "block";
+    document.getElementById("cancel").hidden = true;
+    document.getElementById("finish").hidden = false;
+    document.getElementById("messageSpan").hidden = true;
+    document.getElementById("completeSpan").hidden = false;
   }
 
   /**
@@ -330,11 +349,7 @@ function startContinue(aSelectedStoreType, aResponse) {
           });
       },
     };
-    gServer.rootFolder.compactAll(
-      urlListener,
-      null,
-      gServer.type == "imap" || gServer.type == "nntp"
-    );
+    gServer.rootFolder.compactAll(urlListener, null);
   } else {
     let pConvert = MailstoreConverter.convertMailStoreTo(
       originalStoreContractID,
