@@ -18,7 +18,7 @@
 #include "nsITreeSelection.h"
 #include "nsIMsgFolder.h"
 #include "nsIMsgThread.h"
-#include "DateTimeFormat.h"
+#include "nsMsgUtils.h"
 #include "nsIImapIncomingServer.h"
 #include "nsIMsgFilterPlugin.h"
 #include "nsIStringBundle.h"
@@ -29,7 +29,8 @@
 #include "nsHashKeys.h"
 #include "nsIMsgCustomColumnHandler.h"
 #include "nsIWeakReferenceUtils.h"
-#include "nsSimpleEnumerator.h"
+#include "nsMsgEnumerator.h"
+
 #define MESSENGER_STRING_URL "chrome://messenger/locale/messenger.properties"
 
 typedef AutoTArray<nsMsgViewIndex, 1> nsMsgViewIndexArray;
@@ -43,7 +44,9 @@ enum eFieldType { kCollationKey, kU32 };
 class MsgViewSortColumnInfo {
  public:
   MsgViewSortColumnInfo(const MsgViewSortColumnInfo& other);
-  MsgViewSortColumnInfo() {}
+  MsgViewSortColumnInfo()
+      : mSortType(nsMsgViewSortType::byNone),
+        mSortOrder(nsMsgViewSortOrder::none) {}
   bool operator==(const MsgViewSortColumnInfo& other) const;
   nsMsgViewSortTypeValue mSortType;
   nsMsgViewSortOrderValue mSortOrder;
@@ -52,7 +55,8 @@ class MsgViewSortColumnInfo {
   nsCOMPtr<nsIMsgCustomColumnHandler> mColHandler;
 };
 
-// Reserve the top 8 bits in the msg flags for the view-only flags.
+// Reserve some bits in the msg flags for the view-only flags.
+// NOTE: this bit space is shared by nsMsgMessageFlags (and labels).
 #define MSG_VIEW_FLAGS 0xEE000000
 #define MSG_VIEW_FLAG_HASCHILDREN 0x40000000
 #define MSG_VIEW_FLAG_DUMMY 0x20000000
@@ -81,6 +85,17 @@ struct IdKey : public IdUint32 {
   nsTArray<uint8_t> key;
 };
 
+class nsMsgDBViewService final : public nsIMsgDBViewService {
+ public:
+  NS_DECL_ISUPPORTS
+  NS_DECL_NSIMSGDBVIEWSERVICE
+
+  nsMsgDBViewService(){};
+
+ protected:
+  ~nsMsgDBViewService(){};
+};
+
 // This is an abstract implementation class.
 // The actual view objects will be instances of sub-classes of this class.
 class nsMsgDBView : public nsIMsgDBView,
@@ -88,6 +103,7 @@ class nsMsgDBView : public nsIMsgDBView,
                     public nsITreeView,
                     public nsIJunkMailClassificationListener {
  public:
+  friend class nsMsgDBViewService;
   nsMsgDBView();
 
   NS_DECL_ISUPPORTS
@@ -108,20 +124,28 @@ class nsMsgDBView : public nsIMsgDBView,
  protected:
   virtual ~nsMsgDBView();
 
-  static nsrefcnt gInstanceCount;
+  static nsString kHighestPriorityString;
+  static nsString kHighPriorityString;
+  static nsString kLowestPriorityString;
+  static nsString kLowPriorityString;
+  static nsString kNormalPriorityString;
 
-  static char16_t* kHighestPriorityString;
-  static char16_t* kHighPriorityString;
-  static char16_t* kLowestPriorityString;
-  static char16_t* kLowPriorityString;
-  static char16_t* kNormalPriorityString;
+  static nsString kReadString;
+  static nsString kRepliedString;
+  static nsString kForwardedString;
+  static nsString kRedirectedString;
+  static nsString kNewString;
 
-  static char16_t* kReadString;
-  static char16_t* kRepliedString;
-  static char16_t* kForwardedString;
-  static char16_t* kNewString;
+  // Used for group views.
+  static nsString kTodayString;
+  static nsString kYesterdayString;
+  static nsString kLastWeekString;
+  static nsString kTwoWeeksAgoString;
+  static nsString kOldMailString;
+  static nsString kFutureDateString;
 
   RefPtr<mozilla::dom::XULTreeElement> mTree;
+  nsCOMPtr<nsIMsgJSTree> mJSTree;
   nsCOMPtr<nsITreeSelection> mTreeSelection;
   // We cache this to determine when to push command status notifications.
   uint32_t mNumSelectedRows;
@@ -156,9 +180,9 @@ class nsMsgDBView : public nsIMsgDBView,
 
   // The default enumerator is over the db, but things like
   // quick search views will enumerate just the displayed messages.
-  virtual nsresult GetMessageEnumerator(nsISimpleEnumerator** enumerator);
+  virtual nsresult GetMessageEnumerator(nsIMsgEnumerator** enumerator);
   // this is a message enumerator that enumerates based on the view contents
-  virtual nsresult GetViewEnumerator(nsISimpleEnumerator** enumerator);
+  virtual nsresult GetViewEnumerator(nsIMsgEnumerator** enumerator);
 
   // Save and Restore Selection are a pair of routines you should
   // use when performing an operation which is going to change the view
@@ -271,7 +295,7 @@ class nsMsgDBView : public nsIMsgDBView,
   virtual nsMsgViewIndex FindKey(nsMsgKey key, bool expand);
   virtual nsresult GetDBForViewIndex(nsMsgViewIndex index, nsIMsgDatabase** db);
   virtual nsCOMArray<nsIMsgFolder>* GetFolders();
-  virtual nsresult GetFolderFromMsgURI(const char* aMsgURI,
+  virtual nsresult GetFolderFromMsgURI(const nsACString& aMsgURI,
                                        nsIMsgFolder** aFolder);
 
   virtual nsresult ListIdsInThread(nsIMsgThread* threadHdr,
@@ -288,28 +312,24 @@ class nsMsgDBView : public nsIMsgDBView,
                                         uint32_t* pNumListed);
   uint32_t GetSize(void) { return (m_keys.Length()); }
 
-  // Notification APIs.
-  void NoteStartChange(nsMsgViewIndex firstlineChanged, int32_t numChanged,
-                       nsMsgViewNotificationCodeValue changeType);
-  void NoteEndChange(nsMsgViewIndex firstlineChanged, int32_t numChanged,
-                     nsMsgViewNotificationCodeValue changeType);
-
   // For commands.
-  virtual nsresult ApplyCommandToIndices(nsMsgViewCommandTypeValue command,
-                                         nsMsgViewIndex* indices,
-                                         int32_t numIndices);
+  virtual nsresult ApplyCommandToIndices(
+      nsMsgViewCommandTypeValue command,
+      nsTArray<nsMsgViewIndex> const& selection);
   virtual nsresult ApplyCommandToIndicesWithFolder(
-      nsMsgViewCommandTypeValue command, nsMsgViewIndex* indices,
-      int32_t numIndices, nsIMsgFolder* destFolder);
-  virtual nsresult CopyMessages(nsIMsgWindow* window, nsMsgViewIndex* indices,
-                                int32_t numIndices, bool isMove,
-                                nsIMsgFolder* destFolder);
-  virtual nsresult DeleteMessages(nsIMsgWindow* window, nsMsgViewIndex* indices,
-                                  int32_t numIndices, bool deleteStorage);
-  nsresult GetHeadersFromSelection(uint32_t* indices, uint32_t numIndices,
-                                   nsIMutableArray* messageArray);
-  virtual nsresult ListCollapsedChildren(nsMsgViewIndex viewIndex,
-                                         nsIMutableArray* messageArray);
+      nsMsgViewCommandTypeValue command,
+      nsTArray<nsMsgViewIndex> const& selection, nsIMsgFolder* destFolder);
+  virtual nsresult CopyMessages(nsIMsgWindow* window,
+                                nsTArray<nsMsgViewIndex> const& selection,
+                                bool isMove, nsIMsgFolder* destFolder);
+  virtual nsresult DeleteMessages(nsIMsgWindow* window,
+                                  nsTArray<nsMsgViewIndex> const& selection,
+                                  bool deleteStorage);
+  nsresult GetHeadersFromSelection(nsTArray<nsMsgViewIndex> const& selection,
+                                   nsTArray<RefPtr<nsIMsgDBHdr>>& hdrs);
+  // ListCollapsedChildren() adds to messageArray (rather than replacing it).
+  virtual nsresult ListCollapsedChildren(
+      nsMsgViewIndex viewIndex, nsTArray<RefPtr<nsIMsgDBHdr>>& messageArray);
 
   nsresult SetMsgHdrJunkStatus(nsIJunkMailPlugin* aJunkPlugin,
                                nsIMsgDBHdr* aMsgHdr,
@@ -328,15 +348,15 @@ class nsMsgDBView : public nsIMsgDBView,
   virtual void OnExtraFlagChanged(nsMsgViewIndex /*index*/,
                                   uint32_t /*extraFlag*/) {}
   virtual void OnHeaderAddedOrDeleted() {}
-  nsresult ToggleWatched(nsMsgViewIndex* indices, int32_t numIndices);
+  nsresult ToggleWatched(nsTArray<nsMsgViewIndex> const& selection);
   nsresult SetThreadWatched(nsIMsgThread* thread, nsMsgViewIndex index,
                             bool watched);
   nsresult SetThreadIgnored(nsIMsgThread* thread, nsMsgViewIndex threadIndex,
                             bool ignored);
   nsresult SetSubthreadKilled(nsIMsgDBHdr* header, nsMsgViewIndex msgIndex,
                               bool ignored);
-  nsresult DownloadForOffline(nsIMsgWindow* window, nsMsgViewIndex* indices,
-                              int32_t numIndices);
+  nsresult DownloadForOffline(nsIMsgWindow* window,
+                              nsTArray<nsMsgViewIndex> const& selection);
   nsresult DownloadFlaggedForOffline(nsIMsgWindow* window);
   nsMsgViewIndex GetThreadFromMsgIndex(nsMsgViewIndex index,
                                        nsIMsgThread** threadHdr);
@@ -383,23 +403,24 @@ class nsMsgDBView : public nsIMsgDBView,
   nsresult MarkThreadRead(nsIMsgThread* threadHdr, nsMsgViewIndex threadIndex,
                           nsTArray<nsMsgKey>& idsMarkedRead, bool bRead);
   bool IsValidIndex(nsMsgViewIndex index);
-  nsresult ToggleIgnored(nsMsgViewIndex* indices, int32_t numIndices,
+  nsresult ToggleIgnored(nsTArray<nsMsgViewIndex> const& selection,
                          nsMsgViewIndex* resultIndex, bool* resultToggleState);
-  nsresult ToggleMessageKilled(nsMsgViewIndex* indices, int32_t numIndices,
+  nsresult ToggleMessageKilled(nsTArray<nsMsgViewIndex> const& selection,
                                nsMsgViewIndex* resultIndex,
                                bool* resultToggleState);
-  bool OfflineMsgSelected(nsMsgViewIndex* indices, int32_t numIndices);
-  bool NonDummyMsgSelected(nsMsgViewIndex* indices, int32_t numIndices);
-  char16_t* GetString(const char16_t* aStringName);
-  nsresult GetPrefLocalizedString(const char* aPrefName, nsString& aResult);
+  bool OfflineMsgSelected(nsTArray<nsMsgViewIndex> const& selection);
+  bool NonDummyMsgSelected(nsTArray<nsMsgViewIndex> const& selection);
+  static void GetString(const char16_t* aStringName, nsAString& aValue);
+  static nsresult GetPrefLocalizedString(const char* aPrefName,
+                                         nsString& aResult);
   nsresult AppendKeywordProperties(const nsACString& keywords,
                                    nsAString& properties, bool* tagAdded);
-  nsresult InitLabelStrings(void);
+  static nsresult InitLabelStrings(void);
   nsresult CopyDBView(nsMsgDBView* aNewMsgDBView,
                       nsIMessenger* aMessengerInstance,
                       nsIMsgWindow* aMsgWindow,
                       nsIMsgDBViewCommandUpdater* aCmdUpdater);
-  void InitializeLiterals();
+  static void InitializeLiterals();
   virtual int32_t FindLevelInThread(nsIMsgDBHdr* msgHdr,
                                     nsMsgViewIndex startOfThread,
                                     nsMsgViewIndex viewIndex);
@@ -410,9 +431,14 @@ class nsMsgDBView : public nsIMsgDBView,
   bool AdjustReadFlag(nsIMsgDBHdr* msgHdr, uint32_t* msgFlags);
   void FreeAll(nsTArray<void*>* ptrs);
   void ClearHdrCache();
+
+  // The message held in each row.
   nsTArray<nsMsgKey> m_keys;
+  // Flags for each row, combining nsMsgMessageFlags and MSG_VIEW_FLAGS.
   nsTArray<uint32_t> m_flags;
+  // Threading level of each row (1=top)
   nsTArray<uint8_t> m_levels;
+
   nsMsgImapDeleteModel mDeleteModel;
 
   // Cache the most recently asked for header and corresponding msgKey.
@@ -467,10 +493,10 @@ class nsMsgDBView : public nsIMsgDBView,
   nsWeakPtr mMsgWindowWeak;
   // We push command update notifications to the UI from this.
   nsCOMPtr<nsIMsgDBViewCommandUpdater> mCommandUpdater;
-  nsCOMPtr<nsIStringBundle> mMessengerStringBundle;
+  static nsCOMPtr<nsIStringBundle> mMessengerStringBundle;
 
   // Used for the preference labels.
-  nsString mLabelPrefDescriptions[PREF_LABELS_MAX];
+  static nsString mLabelPrefDescriptions[PREF_LABELS_MAX];
   nsString mLabelPrefColors[PREF_LABELS_MAX];
 
   // Used to determine when to start and end junk plugin batches.
@@ -479,7 +505,7 @@ class nsMsgDBView : public nsIMsgDBView,
   // These are the headers of the messages in the current
   // batch/series of batches of messages manually marked
   // as junk.
-  nsCOMPtr<nsIMutableArray> mJunkHdrs;
+  nsTArray<RefPtr<nsIMsgDBHdr>> mJunkHdrs;
 
   nsTArray<uint32_t> mIndicesToNoteChange;
 
@@ -514,9 +540,12 @@ class nsMsgDBView : public nsIMsgDBView,
   static nsresult InitDisplayFormats();
 
  private:
-  static mozilla::nsDateFormatSelector m_dateFormatDefault;
-  static mozilla::nsDateFormatSelector m_dateFormatThisWeek;
-  static mozilla::nsDateFormatSelector m_dateFormatToday;
+  static bool m_dateFormatsInitialized;
+  static nsDateFormatSelectorComm m_dateFormatDefault;
+  static nsDateFormatSelectorComm m_dateFormatThisWeek;
+  static nsDateFormatSelectorComm m_dateFormatToday;
+  static nsString m_connectorPattern;
+
   bool ServerSupportsFilterAfterTheFact();
 
   nsresult PerformActionsOnJunkMsgs(bool msgsAreJunk);
@@ -526,21 +555,19 @@ class nsMsgDBView : public nsIMsgDBView,
                                          bool& changeReadState,
                                          nsIMsgFolder** targetFolder);
 
-  class nsMsgViewHdrEnumerator final : public nsSimpleEnumerator {
+  class nsMsgViewHdrEnumerator final : public nsBaseMsgEnumerator {
    public:
-    const nsID& DefaultInterface() override { return NS_GET_IID(nsIMsgDBHdr); }
-
-    // nsISimpleEnumerator methods:
-    NS_DECL_NSISIMPLEENUMERATOR
-
-    // nsMsgThreadEnumerator methods:
     explicit nsMsgViewHdrEnumerator(nsMsgDBView* view);
 
+    // nsIMsgEnumerator support.
+    NS_IMETHOD GetNext(nsIMsgDBHdr** aItem) override;
+    NS_IMETHOD HasMoreElements(bool* aResult) override;
+
     RefPtr<nsMsgDBView> m_view;
     nsMsgViewIndex m_curHdrIndex;
 
    private:
-    ~nsMsgViewHdrEnumerator() override;
+    virtual ~nsMsgViewHdrEnumerator() override;
   };
 };
 
