@@ -26,7 +26,6 @@
 #include "nsIMsgWindow.h"
 #include "nsIMsgFilterService.h"
 #include "nsIMsgProtocolInfo.h"
-#include "nsIMutableArray.h"
 #include "nsIPrefService.h"
 #include "nsIRelativeFilePref.h"
 #include "mozilla/nsRelativeFilePref.h"
@@ -49,8 +48,6 @@
 #include "mozilla/Services.h"
 #include "Services.h"
 #include "nsIMsgFilter.h"
-#include "nsIArray.h"
-#include "nsArrayUtils.h"
 #include "nsIObserverService.h"
 #include "mozilla/Unused.h"
 
@@ -90,6 +87,22 @@ nsMsgIncomingServer::Observe(nsISupports* aSubject, const char* aTopic,
   // When the state of the password manager changes we need to clear the
   // password from the cache in case the user just removed it.
   if (strcmp(aTopic, "passwordmgr-storage-changed") == 0) {
+    // Check that the notification is for this server.
+    nsCOMPtr<nsILoginInfo> loginInfo = do_QueryInterface(aSubject);
+    if (loginInfo) {
+      nsAutoString hostnameInfo;
+      loginInfo->GetHostname(hostnameInfo);
+      nsAutoCString hostname;
+      GetHostName(hostname);
+      nsAutoCString fullName;
+      GetType(fullName);
+      if (fullName.EqualsLiteral("pop3")) {
+        fullName = "mailbox://"_ns + hostname;
+      } else {
+        fullName += "://"_ns + hostname;
+      }
+      if (!fullName.Equals(NS_ConvertUTF16toUTF8(hostnameInfo))) return NS_OK;
+    }
     rv = ForgetSessionPassword();
     NS_ENSURE_SUCCESS(rv, rv);
   } else if (strcmp(aTopic, NS_XPCOM_SHUTDOWN_OBSERVER_ID) == 0) {
@@ -424,7 +437,7 @@ nsMsgIncomingServer::GetFileValue(const char* aRelPrefName,
         new mozilla::nsRelativeFilePref();
     mozilla::Unused << relFilePref->SetFile(*aLocalFile);
     mozilla::Unused << relFilePref->SetRelativeToKey(
-        NS_LITERAL_CSTRING(NS_APP_USER_PROFILE_50_DIR));
+        nsLiteralCString(NS_APP_USER_PROFILE_50_DIR));
 
     rv = mPrefBranch->SetComplexValue(
         aRelPrefName, NS_GET_IID(nsIRelativeFilePref), relFilePref);
@@ -443,7 +456,7 @@ nsMsgIncomingServer::SetFileValue(const char* aRelPrefName,
   nsCOMPtr<nsIRelativeFilePref> relFilePref = new mozilla::nsRelativeFilePref();
   mozilla::Unused << relFilePref->SetFile(aLocalFile);
   mozilla::Unused << relFilePref->SetRelativeToKey(
-      NS_LITERAL_CSTRING(NS_APP_USER_PROFILE_50_DIR));
+      nsLiteralCString(NS_APP_USER_PROFILE_50_DIR));
 
   nsresult rv = mPrefBranch->SetComplexValue(
       aRelPrefName, NS_GET_IID(nsIRelativeFilePref), relFilePref);
@@ -771,8 +784,11 @@ nsMsgIncomingServer::ForgetPassword() {
   // there isn't.
   nsString username;
   for (uint32_t i = 0; i < logins.Length(); ++i) {
-    if (NS_SUCCEEDED(logins[i]->GetUsername(username)) &&
-        username.Equals(serverUsername)) {
+    rv = logins[i]->GetUsername(username);
+    int32_t atPos = serverUsername.FindChar('@');
+    if (NS_SUCCEEDED(rv) &&
+        (username.Equals(serverUsername) ||
+         StringHead(serverUsername, atPos).Equals(username))) {
       // If this fails, just continue, we'll still want to remove the password
       // from our local cache.
       loginMgr->RemoveLogin(logins[i]);
@@ -924,7 +940,15 @@ NS_IMETHODIMP
 nsMsgIncomingServer::ClearAllValues() {
   if (!mPrefBranch) return NS_ERROR_NOT_INITIALIZED;
 
-  return mPrefBranch->DeleteBranch("");
+  nsTArray<nsCString> prefNames;
+  nsresult rv = mPrefBranch->GetChildList("", prefNames);
+  NS_ENSURE_SUCCESS(rv, rv);
+
+  for (auto& prefName : prefNames) {
+    mPrefBranch->ClearUserPref(prefName.get());
+  }
+
+  return NS_OK;
 }
 
 NS_IMETHODIMP
@@ -993,7 +1017,7 @@ nsMsgIncomingServer::GetFilterList(nsIMsgWindow* aMsgWindow,
     rv = mFilterFile->InitWithFile(thisFolder);
     NS_ENSURE_SUCCESS(rv, rv);
 
-    mFilterFile->AppendNative(NS_LITERAL_CSTRING("msgFilterRules.dat"));
+    mFilterFile->AppendNative("msgFilterRules.dat"_ns);
 
     bool fileExists;
     mFilterFile->Exists(&fileExists);
@@ -1003,13 +1027,12 @@ nsMsgIncomingServer::GetFilterList(nsIMsgWindow* aMsgWindow,
       NS_ENSURE_SUCCESS(rv, rv);
       rv = oldFilterFile->InitWithFile(thisFolder);
       NS_ENSURE_SUCCESS(rv, rv);
-      oldFilterFile->AppendNative(NS_LITERAL_CSTRING("rules.dat"));
+      oldFilterFile->AppendNative("rules.dat"_ns);
 
       oldFilterFile->Exists(&fileExists);
       if (fileExists)  // copy rules.dat --> msgFilterRules.dat
       {
-        rv = oldFilterFile->CopyToNative(
-            thisFolder, NS_LITERAL_CSTRING("msgFilterRules.dat"));
+        rv = oldFilterFile->CopyToNative(thisFolder, "msgFilterRules.dat"_ns);
         NS_ENSURE_SUCCESS(rv, rv);
       }
     }
@@ -1097,32 +1120,21 @@ nsMsgIncomingServer::OnUserOrHostNameChanged(const nsACString& oldName,
   // 1. Reset password so that users are prompted for new password for the new
   // user/host.
   int32_t atPos = newName.FindChar('@');
-  // If only username changed and the new name just added a domain
-  // we can keep the password.
-  if (hostnameChanged || (atPos == kNotFound) ||
-      !StringHead(NS_ConvertASCIItoUTF16(newName), atPos)
-           .Equals(NS_ConvertASCIItoUTF16(oldName))) {
+  if (hostnameChanged) {
     ForgetPassword();
   }
 
-  // 2. Let the derived class close all cached connection to the old host.
-  CloseCachedConnections();
-
-  // 3. Notify any listeners for account server changes.
-  nsCOMPtr<nsIMsgAccountManager> accountManager(
-      mozilla::services::GetAccountManager());
-
-  rv = accountManager->NotifyServerChanged(this);
-  NS_ENSURE_SUCCESS(rv, rv);
-
-  // 4. Replace all occurrences of old name in the acct name with the new one.
+  // 2. Replace all occurrences of old name in the acct name with the new one.
   nsString acctName;
   rv = GetPrettyName(acctName);
   NS_ENSURE_SUCCESS(rv, rv);
 
-  // 5. Clear the clientid because the user or host have changed.
+  // 3. Clear the clientid because the user or host have changed.
   SetClientid(EmptyCString());
 
+  // Will be generated again when used.
+  mPrefBranch->ClearUserPref("spamActionTargetAccount");
+
   // If new username contains @ then better do not update the account name.
   if (acctName.IsEmpty() || (!hostnameChanged && (atPos != kNotFound)))
     return NS_OK;
@@ -1132,12 +1144,12 @@ nsMsgIncomingServer::OnUserOrHostNameChanged(const nsACString& oldName,
   // get previous username and hostname
   nsCString userName, hostName;
   if (hostnameChanged) {
-    rv = GetRealUsername(userName);
+    rv = GetUsername(userName);
     NS_ENSURE_SUCCESS(rv, rv);
     hostName.Assign(oldName);
   } else {
     userName.Assign(oldName);
-    rv = GetRealHostName(hostName);
+    rv = GetHostName(hostName);
     NS_ENSURE_SUCCESS(rv, rv);
   }
 
@@ -1166,28 +1178,20 @@ nsMsgIncomingServer::OnUserOrHostNameChanged(const nsACString& oldName,
 
 NS_IMETHODIMP
 nsMsgIncomingServer::SetHostName(const nsACString& aHostname) {
-  return (InternalSetHostName(aHostname, "hostname"));
-}
-
-// SetRealHostName() is called only when the server name is changed from the
-// UI (Account Settings page).  No one should call it in any circumstances.
-NS_IMETHODIMP
-nsMsgIncomingServer::SetRealHostName(const nsACString& aHostname) {
   nsCString oldName;
-  nsresult rv = GetRealHostName(oldName);
+  nsresult rv = GetHostName(oldName);
   NS_ENSURE_SUCCESS(rv, rv);
-  rv = InternalSetHostName(aHostname, "realhostname");
+  rv = InternalSetHostName(aHostname, "hostname");
 
-  // A few things to take care of if we're changing the hostname.
-  if (!aHostname.Equals(oldName, nsCaseInsensitiveCStringComparator))
+  if (!oldName.IsEmpty() &&
+      !aHostname.Equals(oldName, nsCaseInsensitiveCStringComparator))
     rv = OnUserOrHostNameChanged(oldName, aHostname, true);
   return rv;
 }
 
 NS_IMETHODIMP
 nsMsgIncomingServer::GetHostName(nsACString& aResult) {
-  nsresult rv;
-  rv = GetCharValue("hostname", aResult);
+  nsresult rv = GetCharValue("hostname", aResult);
   if (aResult.CountChar(':') == 1) {
     // gack, we need to reformat the hostname - SetHostName will do that
     SetHostName(aResult);
@@ -1197,45 +1201,34 @@ nsMsgIncomingServer::GetHostName(nsACString& aResult) {
 }
 
 NS_IMETHODIMP
-nsMsgIncomingServer::GetRealHostName(nsACString& aResult) {
-  // If 'realhostname' is set (was changed) then use it, otherwise use
-  // 'hostname'
-  nsresult rv;
-  rv = GetCharValue("realhostname", aResult);
-  NS_ENSURE_SUCCESS(rv, rv);
-
-  if (aResult.IsEmpty()) return GetHostName(aResult);
-
-  if (aResult.CountChar(':') == 1) {
-    SetRealHostName(aResult);
-    rv = GetCharValue("realhostname", aResult);
-  }
-
-  return rv;
-}
-
-NS_IMETHODIMP
-nsMsgIncomingServer::GetRealUsername(nsACString& aResult) {
-  // If 'realuserName' is set (was changed) then use it, otherwise use
-  // 'userName'
-  nsresult rv;
-  rv = GetCharValue("realuserName", aResult);
-  NS_ENSURE_SUCCESS(rv, rv);
-  return aResult.IsEmpty() ? GetUsername(aResult) : rv;
-}
-
-NS_IMETHODIMP
-nsMsgIncomingServer::SetRealUsername(const nsACString& aUsername) {
-  // Need to take care of few things if we're changing the username.
+nsMsgIncomingServer::SetUsername(const nsACString& aUsername) {
   nsCString oldName;
-  nsresult rv = GetRealUsername(oldName);
+  nsresult rv = GetUsername(oldName);
   NS_ENSURE_SUCCESS(rv, rv);
-  rv = SetCharValue("realuserName", aUsername);
-  if (!oldName.Equals(aUsername))
+
+  if (!oldName.IsEmpty() && !oldName.Equals(aUsername)) {
+    // If only username changed and the new name just added a domain we can keep
+    // the password.
+    int32_t atPos = aUsername.FindChar('@');
+    if ((atPos == kNotFound) ||
+        !StringHead(NS_ConvertASCIItoUTF16(aUsername), atPos)
+             .Equals(NS_ConvertASCIItoUTF16(oldName))) {
+      ForgetPassword();
+    }
+    rv = SetCharValue("userName", aUsername);
+    NS_ENSURE_SUCCESS(rv, rv);
     rv = OnUserOrHostNameChanged(oldName, aUsername, false);
+  } else {
+    rv = SetCharValue("userName", aUsername);
+  }
   return rv;
 }
 
+NS_IMETHODIMP
+nsMsgIncomingServer::GetUsername(nsACString& aResult) {
+  return GetCharValue("userName", aResult);
+}
+
 #define BIFF_PREF_NAME "check_new_mail"
 
 NS_IMETHODIMP
@@ -1552,7 +1545,6 @@ nsMsgIncomingServer::GetIsSecure(bool* aIsSecure) {
 }
 
 // use the convenience macros to implement the accessors
-NS_IMPL_SERVERPREF_STR(nsMsgIncomingServer, Username, "userName")
 NS_IMPL_SERVERPREF_INT(nsMsgIncomingServer, AuthMethod, "authMethod")
 NS_IMPL_SERVERPREF_INT(nsMsgIncomingServer, BiffMinutes, "check_time")
 NS_IMPL_SERVERPREF_STR(nsMsgIncomingServer, Type, "type")
@@ -1730,23 +1722,15 @@ nsresult nsMsgIncomingServer::ConfigureTemporaryServerSpamFilters(
      */
 
     // get the list of search terms from the filter
-    nsCOMPtr<nsIMutableArray> searchTerms;
-    rv = newFilter->GetSearchTerms(getter_AddRefs(searchTerms));
+    nsTArray<RefPtr<nsIMsgSearchTerm>> searchTerms;
+    rv = newFilter->GetSearchTerms(searchTerms);
     NS_ENSURE_SUCCESS(rv, rv);
-    uint32_t count = 0;
-    searchTerms->Count(&count);
+    uint32_t count = searchTerms.Length();
     if (count > 1)  // don't need to group a single term
     {
       // beginGrouping the first term, and endGrouping the last term
-      nsCOMPtr<nsIMsgSearchTerm> firstTerm(
-          do_QueryElementAt(searchTerms, 0, &rv));
-      NS_ENSURE_SUCCESS(rv, rv);
-      firstTerm->SetBeginsGrouping(true);
-
-      nsCOMPtr<nsIMsgSearchTerm> lastTerm(
-          do_QueryElementAt(searchTerms, count - 1, &rv));
-      NS_ENSURE_SUCCESS(rv, rv);
-      lastTerm->SetEndsGrouping(true);
+      searchTerms[0]->SetBeginsGrouping(true);
+      searchTerms[count - 1]->SetEndsGrouping(true);
     }
 
     // Create a new term, checking if the user set junk status. The term will
@@ -1763,16 +1747,16 @@ nsresult nsMsgIncomingServer::ConfigureTemporaryServerSpamFilters(
     searchTerm->GetValue(getter_AddRefs(searchValue));
     NS_ENSURE_SUCCESS(rv, rv);
     searchValue->SetAttrib(nsMsgSearchAttrib::JunkScoreOrigin);
-    searchValue->SetStr(NS_LITERAL_STRING("user"));
+    searchValue->SetStr(u"user"_ns);
     searchTerm->SetValue(searchValue);
 
-    searchTerms->InsertElementAt(searchTerm, count);
+    newFilter->AppendTerm(searchTerm);
 
     bool moveOnSpam, markAsReadOnSpam;
     spamSettings->GetMoveOnSpam(&moveOnSpam);
     if (moveOnSpam) {
       nsCString spamFolderURI;
-      rv = spamSettings->GetSpamFolderURI(getter_Copies(spamFolderURI));
+      rv = spamSettings->GetSpamFolderURI(spamFolderURI);
       if (NS_SUCCEEDED(rv) && (!spamFolderURI.IsEmpty())) {
         nsCOMPtr<nsIMsgRuleAction> moveAction;
         rv = newFilter->CreateAction(getter_AddRefs(moveAction));
@@ -1834,8 +1818,8 @@ nsresult nsMsgIncomingServer::ConfigureTemporaryReturnReceiptsFilter(
 
   // this is a temporary, internal mozilla filter
   // it will not show up in the UI, it will not be written to disk
-  NS_NAMED_LITERAL_STRING(internalReturnReceiptFilterName,
-                          "mozilla-temporary-internal-MDN-receipt-filter");
+  constexpr auto internalReturnReceiptFilterName =
+      u"mozilla-temporary-internal-MDN-receipt-filter"_ns;
 
   nsCOMPtr<nsIMsgFilter> newFilter;
   rv = filterList->GetFilterNamed(internalReturnReceiptFilterName,
@@ -1864,11 +1848,11 @@ nsresult nsMsgIncomingServer::ConfigureTemporaryReturnReceiptsFilter(
             // we need to use OtherHeader + 1 so nsMsgFilter::GetTerm will
             // return our custom header.
             value->SetAttrib(nsMsgSearchAttrib::OtherHeader + 1);
-            value->SetStr(NS_LITERAL_STRING("multipart/report"));
+            value->SetStr(u"multipart/report"_ns);
             term->SetAttrib(nsMsgSearchAttrib::OtherHeader + 1);
             term->SetOp(nsMsgSearchOp::Contains);
             term->SetBooleanAnd(true);
-            term->SetArbitraryHeader(NS_LITERAL_CSTRING("Content-Type"));
+            term->SetArbitraryHeader("Content-Type"_ns);
             term->SetValue(value);
             newFilter->AppendTerm(term);
           }
@@ -1881,11 +1865,11 @@ nsresult nsMsgIncomingServer::ConfigureTemporaryReturnReceiptsFilter(
             // determine if ::OtherHeader is the best way to do this.
             // see nsMsgSearchOfflineMail::MatchTerms()
             value->SetAttrib(nsMsgSearchAttrib::OtherHeader + 1);
-            value->SetStr(NS_LITERAL_STRING("disposition-notification"));
+            value->SetStr(u"disposition-notification"_ns);
             term->SetAttrib(nsMsgSearchAttrib::OtherHeader + 1);
             term->SetOp(nsMsgSearchOp::Contains);
             term->SetBooleanAnd(true);
-            term->SetArbitraryHeader(NS_LITERAL_CSTRING("Content-Type"));
+            term->SetArbitraryHeader("Content-Type"_ns);
             term->SetValue(value);
             newFilter->AppendTerm(term);
           }
@@ -1909,7 +1893,7 @@ nsMsgIncomingServer::ClearTemporaryReturnReceiptsFilter() {
   if (mFilterList) {
     nsCOMPtr<nsIMsgFilter> mdnFilter;
     nsresult rv = mFilterList->GetFilterNamed(
-        NS_LITERAL_STRING("mozilla-temporary-internal-MDN-receipt-filter"),
+        u"mozilla-temporary-internal-MDN-receipt-filter"_ns,
         getter_AddRefs(mdnFilter));
     if (NS_SUCCEEDED(rv) && mdnFilter)
       return mFilterList->RemoveFilter(mdnFilter);
@@ -2054,14 +2038,13 @@ NS_IMETHODIMP nsMsgIncomingServer::IsNewHdrDuplicate(nsIMsgDBHdr* aNewHdr,
   // err on the side of caution and ignore messages w/o subject or messageid.
   if (subject.IsEmpty() || messageId.IsEmpty()) return NS_OK;
   strHashKey.Append(subject);
-  int32_t hashValue = 0;
-  m_downloadedHdrs.Get(strHashKey, &hashValue);
+  int32_t hashValue = m_downloadedHdrs.Get(strHashKey);
   if (hashValue)
     *aResult = true;
   else {
     // we store the current size of the hash table as the hash
     // value - this allows us to delete older entries.
-    m_downloadedHdrs.Put(strHashKey, ++m_numMsgsDownloaded);
+    m_downloadedHdrs.InsertOrUpdate(strHashKey, ++m_numMsgsDownloaded);
     // Check if hash table is larger than some reasonable size
     // and if is it, iterate over hash table deleting messages
     // with an arrival index < number of msgs downloaded - half the reasonable
@@ -2096,8 +2079,7 @@ nsMsgIncomingServer::SetForcePropertyEmpty(const char* aPropertyName,
                                            bool aValue) {
   nsAutoCString nameEmpty(aPropertyName);
   nameEmpty.AppendLiteral(".empty");
-  return SetCharValue(nameEmpty.get(), aValue ? NS_LITERAL_CSTRING("true")
-                                              : NS_LITERAL_CSTRING(""));
+  return SetCharValue(nameEmpty.get(), aValue ? "true"_ns : ""_ns);
 }
 
 NS_IMETHODIMP
