@@ -47,8 +47,9 @@
 
 using namespace mozilla;
 
-NS_IMPL_ISUPPORTS(nsMsgProtocol, nsIChannel, nsIStreamListener,
-                  nsIRequestObserver, nsIRequest, nsITransportEventSink)
+NS_IMPL_ISUPPORTS_INHERITED(nsMsgProtocol, nsHashPropertyBag, nsIChannel,
+                            nsIStreamListener, nsIRequestObserver, nsIRequest,
+                            nsITransportEventSink)
 
 static char16_t* FormatStringWithHostNameByName(const char16_t* stringName,
                                                 nsIMsgMailNewsUrl* msgUri);
@@ -60,6 +61,7 @@ nsMsgProtocol::nsMsgProtocol(nsIURI* aURL) {
   m_socketIsOpen = false;
   mContentLength = -1;
   m_isChannel = false;
+  mContentDisposition = nsIChannel::DISPOSITION_INLINE;
 
   GetSpecialDirectoryWithFileName(NS_OS_TEMP_DIR, "tempMessage.eml",
                                   getter_AddRefs(m_tempMsgFile));
@@ -128,9 +130,9 @@ nsresult nsMsgProtocol::OpenNetworkSocketWithInfo(
   nsCOMPtr<nsISocketTransport> strans;
   AutoTArray<nsCString, 1> connectionTypeArray;
   if (connectionType) connectionTypeArray.AppendElement(connectionType);
-  rv = socketService->CreateTransport(connectionTypeArray,
-                                      nsDependentCString(aHostName), aGetPort,
-                                      aProxyInfo, getter_AddRefs(strans));
+  rv = socketService->CreateTransport(
+      connectionTypeArray, nsDependentCString(aHostName), aGetPort, aProxyInfo,
+      nullptr, getter_AddRefs(strans));
   if (NS_FAILED(rv)) return rv;
 
   strans->SetSecurityCallbacks(callbacks);
@@ -433,6 +435,11 @@ nsresult nsMsgProtocol::GetPromptDialogFromUrl(nsIMsgMailNewsUrl* aMsgUrl,
 }
 
 nsresult nsMsgProtocol::LoadUrl(nsIURI* aURL, nsISupports* aConsumer) {
+  // nsMsgProtocol implements nsIChannel, and all channels are required to
+  // have non-null loadInfo. So if it's still unset, we've not been correctly
+  // initialised.
+  MOZ_ASSERT(m_loadInfo);
+
   // okay now kick us off to the next state...
   // our first state is a process state so drive the state machine...
   nsresult rv = NS_OK;
@@ -474,8 +481,10 @@ nsresult nsMsgProtocol::LoadUrl(nsIURI* aURL, nsISupports* aConsumer) {
 
         m_request = pump;  // keep a reference to the pump so we can cancel it
 
-        // put us in a state where we are always notified of incoming data
-        rv = pump->AsyncRead(this, nullptr);
+        // Put us in a state where we are always notified of incoming data.
+        // OnDataAvailable() will be called when that happens, which will
+        // pass that data into ProcessProtocolState().
+        rv = pump->AsyncRead(this);
         NS_ASSERTION(NS_SUCCEEDED(rv), "AsyncRead failed");
         m_socketIsOpen = true;  // mark the channel as open
       }
@@ -602,12 +611,14 @@ NS_IMETHODIMP nsMsgProtocol::SetContentCharset(
 
 NS_IMETHODIMP
 nsMsgProtocol::GetContentDisposition(uint32_t* aContentDisposition) {
-  return NS_ERROR_NOT_AVAILABLE;
+  *aContentDisposition = mContentDisposition;
+  return NS_OK;
 }
 
 NS_IMETHODIMP
 nsMsgProtocol::SetContentDisposition(uint32_t aContentDisposition) {
-  return NS_ERROR_NOT_AVAILABLE;
+  mContentDisposition = aContentDisposition;
+  return NS_OK;
 }
 
 NS_IMETHODIMP
@@ -716,7 +727,7 @@ nsMsgProtocol::OnTransportStatus(nsITransport* transport, nsresult status,
   if (mailnewsUrl) {
     nsCOMPtr<nsIMsgIncomingServer> server;
     mailnewsUrl->GetServer(getter_AddRefs(server));
-    if (server) server->GetRealHostName(host);
+    if (server) server->GetHostName(host);
   }
   mProgressEventSink->OnStatus(this, status, NS_ConvertUTF8toUTF16(host).get());
 
@@ -799,7 +810,7 @@ nsresult nsMsgProtocol::PostMessage(nsIURI* url, nsIFile* postFile) {
     /* escape starting periods
      */
     if (line.CharAt(0) == '.') line.Insert('.', 0);
-    line.Append(NS_LITERAL_CSTRING(CRLF));
+    line.AppendLiteral(CRLF);
     outputBuffer.Append(line);
     // test hack by mscott. If our buffer is almost full, then
     // send it off & reset ourselves
@@ -816,7 +827,8 @@ nsresult nsMsgProtocol::PostMessage(nsIURI* url, nsIFile* postFile) {
   return NS_OK;
 }
 
-nsresult nsMsgProtocol::DoGSSAPIStep1(const char* service, const char* username,
+nsresult nsMsgProtocol::DoGSSAPIStep1(const nsACString& service,
+                                      const char* username,
                                       nsCString& response) {
   nsresult rv;
 #ifdef DEBUG_BenB
@@ -826,8 +838,8 @@ nsresult nsMsgProtocol::DoGSSAPIStep1(const char* service, const char* username,
   // if this fails, then it means that we cannot do GSSAPI SASL.
   m_authModule = nsIAuthModule::CreateInstance("sasl-gssapi");
 
-  m_authModule->Init(service, nsIAuthModule::REQ_DEFAULT, nullptr,
-                     NS_ConvertUTF8toUTF16(username).get(), nullptr);
+  m_authModule->Init(service, nsIAuthModule::REQ_DEFAULT, u""_ns,
+                     NS_ConvertUTF8toUTF16(username), u""_ns);
 
   void* outBuf;
   uint32_t outBufLen;
@@ -913,8 +925,8 @@ nsresult nsMsgProtocol::DoNtlmStep1(const nsACString& username,
 
   m_authModule = nsIAuthModule::CreateInstance("ntlm");
 
-  m_authModule->Init(nullptr, 0, nullptr, NS_ConvertUTF8toUTF16(username).get(),
-                     PromiseFlatString(password).get());
+  m_authModule->Init(""_ns, 0, u""_ns, NS_ConvertUTF8toUTF16(username),
+                     PromiseFlatString(password));
 
   void* outBuf;
   uint32_t outBufLen;
@@ -1084,7 +1096,7 @@ nsresult nsMsgFilePostHelper::Init(nsIOutputStream* aOutStream,
   rv = NS_NewInputStreamPump(getter_AddRefs(pump), stream.forget());
   if (NS_FAILED(rv)) return rv;
 
-  rv = pump->AsyncRead(this, nullptr);
+  rv = pump->AsyncRead(this);
   if (NS_FAILED(rv)) return rv;
 
   mPostFileRequest = pump;
@@ -1154,6 +1166,8 @@ nsMsgAsyncWriteProtocol::nsMsgAsyncWriteProtocol(nsIURI* aURL)
   mGenerateProgressNotifications = false;
   mSuspendedReadBytesPostPeriod = 0;
   mFilePostHelper = nullptr;
+  mNumBytesPosted = 0;
+  mFilePostSize = 0;
 }
 
 nsMsgAsyncWriteProtocol::~nsMsgAsyncWriteProtocol() {}
@@ -1474,7 +1488,7 @@ char16_t* FormatStringWithHostNameByName(const char16_t* stringName,
   NS_ENSURE_SUCCESS(rv, nullptr);
 
   nsCString hostName;
-  rv = server->GetRealHostName(hostName);
+  rv = server->GetHostName(hostName);
   NS_ENSURE_SUCCESS(rv, nullptr);
 
   AutoTArray<nsString, 1> params;
