@@ -14,7 +14,6 @@
 #include "nsIMsgMdnGenerator.h"
 #include "nsServiceManagerUtils.h"
 #include "nsMsgMimeCID.h"
-#include "nsArrayEnumerator.h"
 #include "nsMemory.h"
 #include "mozilla/ArrayUtils.h"
 #include "mozilla/mailnews/MimeHeaderParser.h"
@@ -76,20 +75,12 @@ nsMsgCompFields::nsMsgCompFields()
   m_forceMsgEncoding = false;
   m_needToCheckCharset = true;
   m_attachmentReminder = false;
-  m_deliveryFormat = nsIMsgCompSendFormat::AskUser;
-
-  // Get the default charset from pref, use this as a mail charset.
-  nsString charset;
-  NS_GetLocalizedUnicharPreferenceWithDefault(
-      nullptr, "mailnews.send_default_charset", NS_LITERAL_STRING("UTF-8"),
-      charset);
-
-  LossyCopyUTF16toASCII(charset,
-                        m_DefaultCharacterSet);  // Charsets better be ASCII
-  SetCharacterSet(m_DefaultCharacterSet.get());
+  m_deliveryFormat = nsIMsgCompSendFormat::Unset;
 }
 
-nsMsgCompFields::~nsMsgCompFields() {}
+nsMsgCompFields::~nsMsgCompFields() {
+  MOZ_LOG(Compose, mozilla::LogLevel::Debug, ("~nsMsgCompFields()"));
+}
 
 nsresult nsMsgCompFields::SetAsciiHeader(MsgHeaderID header,
                                          const char* value) {
@@ -278,15 +269,6 @@ NS_IMETHODIMP nsMsgCompFields::GetPriority(char** _retval) {
   return *_retval ? NS_OK : NS_ERROR_OUT_OF_MEMORY;
 }
 
-NS_IMETHODIMP nsMsgCompFields::SetCharacterSet(const char* value) {
-  return SetAsciiHeader(MSG_CHARACTER_SET_HEADER_ID, value);
-}
-
-NS_IMETHODIMP nsMsgCompFields::GetCharacterSet(char** _retval) {
-  *_retval = strdup(GetAsciiHeader(MSG_CHARACTER_SET_HEADER_ID));
-  return *_retval ? NS_OK : NS_ERROR_OUT_OF_MEMORY;
-}
-
 NS_IMETHODIMP nsMsgCompFields::SetMessageId(const char* value) {
   return SetAsciiHeader(MSG_MESSAGE_ID_HEADER_ID, value);
 }
@@ -304,22 +286,24 @@ NS_IMETHODIMP nsMsgCompFields::GetTemplateName(nsAString& _retval) {
   return GetUnicodeHeader(MSG_X_TEMPLATE_HEADER_ID, _retval);
 }
 
-NS_IMETHODIMP nsMsgCompFields::SetDraftId(const char* value) {
-  return SetAsciiHeader(MSG_DRAFT_ID_HEADER_ID, value);
+NS_IMETHODIMP nsMsgCompFields::SetDraftId(const nsACString& value) {
+  return SetAsciiHeader(MSG_DRAFT_ID_HEADER_ID,
+                        PromiseFlatCString(value).get());
 }
 
-NS_IMETHODIMP nsMsgCompFields::GetDraftId(char** _retval) {
-  *_retval = strdup(GetAsciiHeader(MSG_DRAFT_ID_HEADER_ID));
-  return *_retval ? NS_OK : NS_ERROR_OUT_OF_MEMORY;
+NS_IMETHODIMP nsMsgCompFields::GetDraftId(nsACString& _retval) {
+  _retval.Assign(GetAsciiHeader(MSG_DRAFT_ID_HEADER_ID));
+  return NS_OK;
 }
 
-NS_IMETHODIMP nsMsgCompFields::SetTemplateId(const char* value) {
-  return SetAsciiHeader(MSG_TEMPLATE_ID_HEADER_ID, value);
+NS_IMETHODIMP nsMsgCompFields::SetTemplateId(const nsACString& value) {
+  return SetAsciiHeader(MSG_TEMPLATE_ID_HEADER_ID,
+                        PromiseFlatCString(value).get());
 }
 
-NS_IMETHODIMP nsMsgCompFields::GetTemplateId(char** _retval) {
-  *_retval = strdup(GetAsciiHeader(MSG_TEMPLATE_ID_HEADER_ID));
-  return *_retval ? NS_OK : NS_ERROR_OUT_OF_MEMORY;
+NS_IMETHODIMP nsMsgCompFields::GetTemplateId(nsACString& _retval) {
+  _retval.Assign(GetAsciiHeader(MSG_TEMPLATE_ID_HEADER_ID));
+  return NS_OK;
 }
 
 NS_IMETHODIMP nsMsgCompFields::SetReturnReceipt(bool value) {
@@ -375,14 +359,15 @@ NS_IMETHODIMP nsMsgCompFields::SetAttachmentReminder(bool value) {
 
 NS_IMETHODIMP nsMsgCompFields::SetDeliveryFormat(int32_t value) {
   switch (value) {
-    case nsIMsgCompSendFormat::AskUser:
+    case nsIMsgCompSendFormat::Auto:
     case nsIMsgCompSendFormat::PlainText:
     case nsIMsgCompSendFormat::HTML:
     case nsIMsgCompSendFormat::Both:
       m_deliveryFormat = value;
       break;
+    case nsIMsgCompSendFormat::Unset:
     default:
-      m_deliveryFormat = nsIMsgCompSendFormat::AskUser;
+      m_deliveryFormat = nsIMsgCompSendFormat::Unset;
   }
 
   return NS_OK;
@@ -448,60 +433,39 @@ NS_IMETHODIMP nsMsgCompFields::GetBodyIsAsciiOnly(bool* _retval) {
 }
 
 NS_IMETHODIMP nsMsgCompFields::SetBody(const nsAString& value) {
-  CopyUTF16toUTF8(value, m_body);
+  m_body = value;
   return NS_OK;
 }
 
 NS_IMETHODIMP nsMsgCompFields::GetBody(nsAString& _retval) {
-  CopyUTF8toUTF16(m_body, _retval);
+  _retval = m_body;
   return NS_OK;
 }
 
-nsresult nsMsgCompFields::SetBody(const char* value) {
-  if (value)
-    m_body = value;
-  else
-    m_body.Truncate();
-  return NS_OK;
-}
-
-const char* nsMsgCompFields::GetBody() { return m_body.get(); }
-
-/* readonly attribute nsISimpleEnumerator attachmentsArray; */
 NS_IMETHODIMP nsMsgCompFields::GetAttachments(
-    nsISimpleEnumerator** aAttachmentsEnum) {
-  return aAttachmentsEnum
-             ? NS_NewArrayEnumerator(aAttachmentsEnum, m_attachments,
-                                     NS_GET_IID(nsIMsgAttachment))
-             : NS_ERROR_NULL_POINTER;
+    nsTArray<RefPtr<nsIMsgAttachment>>& attachments) {
+  attachments = m_attachments.Clone();
+  return NS_OK;
 }
 
-/* void addAttachment (in nsIMsgAttachment attachment); */
 NS_IMETHODIMP nsMsgCompFields::AddAttachment(nsIMsgAttachment* attachment) {
-  int32_t attachmentCount = m_attachments.Count();
-
-  // Don't add twice the same attachment.
-  nsCOMPtr<nsIMsgAttachment> element;
-  bool sameUrl;
-  for (int32_t i = 0; i < attachmentCount; i++) {
-    m_attachments[i]->EqualsUrl(attachment, &sameUrl);
+  // Don't add the same attachment twice.
+  for (nsIMsgAttachment* a : m_attachments) {
+    bool sameUrl;
+    a->EqualsUrl(attachment, &sameUrl);
     if (sameUrl) return NS_OK;
   }
-  m_attachments.AppendObject(attachment);
-
+  m_attachments.AppendElement(attachment);
   return NS_OK;
 }
 
 /* void removeAttachment (in nsIMsgAttachment attachment); */
 NS_IMETHODIMP nsMsgCompFields::RemoveAttachment(nsIMsgAttachment* attachment) {
-  int32_t attachmentCount = m_attachments.Count();
-
-  nsCOMPtr<nsIMsgAttachment> element;
-  bool sameUrl;
-  for (int32_t i = 0; i < attachmentCount; i++) {
+  for (uint32_t i = 0; i < m_attachments.Length(); i++) {
+    bool sameUrl;
     m_attachments[i]->EqualsUrl(attachment, &sameUrl);
     if (sameUrl) {
-      m_attachments.RemoveObjectAt(i);
+      m_attachments.RemoveElementAt(i);
       break;
     }
   }
@@ -509,10 +473,20 @@ NS_IMETHODIMP nsMsgCompFields::RemoveAttachment(nsIMsgAttachment* attachment) {
   return NS_OK;
 }
 
+NS_IMETHODIMP nsMsgCompFields::SetOtherHeaders(
+    const nsTArray<nsString>& headers) {
+  m_otherHeaders = headers.Clone();
+  return NS_OK;
+}
+
+NS_IMETHODIMP nsMsgCompFields::GetOtherHeaders(nsTArray<nsString>& headers) {
+  headers = m_otherHeaders.Clone();
+  return NS_OK;
+}
+
 /* void removeAttachments (); */
 NS_IMETHODIMP nsMsgCompFields::RemoveAttachments() {
   m_attachments.Clear();
-
   return NS_OK;
 }
 
@@ -552,15 +526,10 @@ NS_IMETHODIMP nsMsgCompFields::ConvertBodyToPlainText() {
   nsresult rv = NS_OK;
 
   if (!m_body.IsEmpty()) {
-    nsAutoString body;
-    rv = GetBody(body);
     if (NS_SUCCEEDED(rv)) {
-      bool flowed, delsp, formatted, disallowBreaks;
-      GetSerialiserFlags(GetCharacterSet(), &flowed, &delsp, &formatted,
-                         &disallowBreaks);
-      rv =
-          ConvertBufToPlainText(body, flowed, delsp, formatted, disallowBreaks);
-      if (NS_SUCCEEDED(rv)) rv = SetBody(body);
+      bool flowed, formatted;
+      GetSerialiserFlags(&flowed, &formatted);
+      rv = ConvertBufToPlainText(m_body, flowed, formatted, true);
     }
   }
   return rv;
@@ -579,13 +548,6 @@ NS_IMETHODIMP nsMsgCompFields::SetComposeSecure(
   return NS_OK;
 }
 
-NS_IMETHODIMP nsMsgCompFields::GetDefaultCharacterSet(
-    char** aDefaultCharacterSet) {
-  NS_ENSURE_ARG_POINTER(aDefaultCharacterSet);
-  *aDefaultCharacterSet = ToNewCString(m_DefaultCharacterSet);
-  return *aDefaultCharacterSet ? NS_OK : NS_ERROR_OUT_OF_MEMORY;
-}
-
 NS_IMETHODIMP nsMsgCompFields::GetNeedToCheckCharset(bool* _retval) {
   NS_ENSURE_ARG_POINTER(_retval);
   *_retval = m_needToCheckCharset;
