@@ -53,10 +53,8 @@ class nsMsgCompose : public nsIMsgCompose, public nsSupportsWeakReference {
   bool IsEmbeddedObjectSafe(const char* originalScheme,
                             const char* originalHost, const char* originalPath,
                             mozilla::dom::Element* element);
-  nsresult ResetUrisForEmbeddedObjects();
   nsresult TagEmbeddedObjects(nsIEditor* aEditor);
 
-  nsCString mQuoteCharset;
   nsCString mOriginalMsgURI;  // used so we can mark message disposition flags
                               // after we send the message
 
@@ -68,11 +66,11 @@ class nsMsgCompose : public nsIMsgCompose, public nsSupportsWeakReference {
   bool CheckIncludeSignaturePrefs(nsIMsgIdentity* identity);
   // m_folderName to store the value of the saved drafts folder.
   nsCString m_folderName;
-  void InsertDivWrappedTextAtSelection(const nsAString& aText,
-                                       const nsAString& classStr);
+  MOZ_CAN_RUN_SCRIPT void InsertDivWrappedTextAtSelection(
+      const nsAString& aText, const nsAString& classStr);
 
  protected:
-  nsresult CreateMessage(const char* originalMsgURI, MSG_ComposeType type,
+  nsresult CreateMessage(const nsACString& originalMsgURI, MSG_ComposeType type,
                          nsIMsgCompFields* compFields);
   void CleanUpRecipients(nsString& recipients);
   nsresult GetABDirAndMailLists(const nsACString& aDirUri,
@@ -85,9 +83,9 @@ class nsMsgCompose : public nsIMsgCompose, public nsSupportsWeakReference {
                            nsTArray<nsMsgRecipient>& aListMembers);
   void TagConvertible(mozilla::dom::Element* node, int32_t* _retval);
   void _NodeTreeConvertible(mozilla::dom::Element* node, int32_t* _retval);
-  nsresult MoveToAboveQuote(void);
-  nsresult MoveToBeginningOfDocument(void);
-  nsresult MoveToEndOfDocument(void);
+  MOZ_CAN_RUN_SCRIPT nsresult MoveToAboveQuote(void);
+  MOZ_CAN_RUN_SCRIPT nsresult MoveToBeginningOfDocument(void);
+  MOZ_CAN_RUN_SCRIPT nsresult MoveToEndOfDocument(void);
   nsresult ReplaceFileURLs(nsString& sigData);
   nsresult DataURLForFileURL(const nsAString& aFileURL, nsAString& aDataURL);
 
@@ -135,21 +133,25 @@ class nsMsgCompose : public nsIMsgCompose, public nsSupportsWeakReference {
   nsTObserverArray<nsCOMPtr<nsIMsgComposeStateListener> > mStateListeners;
   nsTObserverArray<nsCOMPtr<nsIMsgSendListener> > mExternalSendListeners;
 
-  bool mInsertingQuotedContent;
+  bool mAllowRemoteContent;
   MSG_DeliverMode mDeliverMode;  // nsIMsgCompDeliverMode long.
 
   friend class QuotingOutputStreamListener;
   friend class nsMsgComposeSendListener;
+
+ private:
+  void DeleteTmpAttachments();
+  bool mTmpAttachmentsDeleted;
 };
 
 ////////////////////////////////////////////////////////////////////////////////////
 // THIS IS THE CLASS THAT IS THE STREAM Listener OF THE HTML OUTPUT
 // FROM LIBMIME. THIS IS FOR QUOTING
 ////////////////////////////////////////////////////////////////////////////////////
-class QuotingOutputStreamListener : public nsIMsgQuotingOutputStreamListener {
+class QuotingOutputStreamListener : public nsIMsgQuotingOutputStreamListener,
+                                    public nsSupportsWeakReference {
  public:
-  QuotingOutputStreamListener(const char* originalMsgURI,
-                              nsIMsgDBHdr* origMsgHdr, bool quoteHeaders,
+  QuotingOutputStreamListener(nsIMsgDBHdr* origMsgHdr, bool quoteHeaders,
                               bool headersOnly, nsIMsgIdentity* identity,
                               nsIMsgQuote* msgQuote, bool charsetFixed,
                               bool quoteOriginal,
@@ -161,9 +163,10 @@ class QuotingOutputStreamListener : public nsIMsgQuotingOutputStreamListener {
   NS_DECL_NSIMSGQUOTINGOUTPUTSTREAMLISTENER
 
   nsresult SetComposeObj(nsIMsgCompose* obj);
-  nsresult ConvertToPlainText(bool formatflowed, bool delsp, bool formatted,
+  nsresult ConvertToPlainText(bool formatflowed, bool formatted,
                               bool disallowBreaks);
-  nsresult InsertToCompose(nsIEditor* aEditor, bool aHTMLEditor);
+  MOZ_CAN_RUN_SCRIPT nsresult InsertToCompose(nsIEditor* aEditor,
+                                              bool aHTMLEditor);
   nsresult AppendToMsgBody(const nsCString& inStr);
 
  private:
