@@ -46,12 +46,12 @@
 #include "mozilla/intl/LineBreaker.h"
 #include "mozilla/Services.h"
 #include "mimemoz2.h"
-#include "nsIArray.h"
-#include "nsArrayUtils.h"
 #include "nsIURIMutator.h"
 #include "mozilla/dom/Element.h"
 #include "mozilla/dom/XULFrameElement.h"
 #include "nsFrameLoader.h"
+#include "nsSmtpUrl.h"
+#include "mozilla/NullPrincipal.h"
 
 #ifdef MSGCOMP_TRACE_PERFORMANCE
 #  include "mozilla/Logging.h"
@@ -64,6 +64,10 @@
 #include "nsIAppStartup.h"
 #include "nsMsgUtils.h"
 #include "nsIPrincipal.h"
+#include "nsIMutableArray.h"
+
+using namespace mozilla;
+using namespace mozilla::dom;
 
 #ifdef XP_WIN
 #  include <windows.h>
@@ -72,7 +76,7 @@
 #endif
 
 #define DEFAULT_CHROME \
-  "chrome://messenger/content/messengercompose/messengercompose.xhtml"
+  "chrome://messenger/content/messengercompose/messengercompose.xhtml"_ns
 
 #define PREF_MAILNEWS_REPLY_QUOTING_SELECTION "mailnews.reply_quoting_selection"
 #define PREF_MAILNEWS_REPLY_QUOTING_SELECTION_MULTI_WORD \
@@ -91,10 +95,10 @@
 #ifdef MSGCOMP_TRACE_PERFORMANCE
 static mozilla::LazyLogModule MsgComposeLogModule("MsgCompose");
 
-static uint32_t GetMessageSizeFromURI(const char* originalMsgURI) {
+static uint32_t GetMessageSizeFromURI(const nsACString& originalMsgURI) {
   uint32_t msgSize = 0;
 
-  if (originalMsgURI && *originalMsgURI) {
+  if (!originalMsgURI.IsEmpty()) {
     nsCOMPtr<nsIMsgDBHdr> originalMsgHdr;
     GetMsgDBHdrFromURI(originalMsgURI, getter_AddRefs(originalMsgHdr));
     if (originalMsgHdr) originalMsgHdr->GetMessageSize(&msgSize);
@@ -177,8 +181,14 @@ nsMsgComposeService::OpenComposeWindowWithParams(const char* chrome,
   msgParamsWrapper->SetDataIID(&NS_GET_IID(nsIMsgComposeParams));
 
   nsCOMPtr<mozIDOMWindowProxy> newWindow;
-  rv = wwatch->OpenWindow(0, chrome && *chrome ? chrome : DEFAULT_CHROME,
-                          "_blank", "all,chrome,dialog=no,status,toolbar",
+  nsAutoCString chromeURL;
+  if (chrome && *chrome) {
+    chromeURL = nsDependentCString(chrome);
+  } else {
+    chromeURL = DEFAULT_CHROME;
+  }
+  rv = wwatch->OpenWindow(0, chromeURL, "_blank"_ns,
+                          "all,chrome,dialog=no,status,toolbar"_ns,
                           msgParamsWrapper, getter_AddRefs(newWindow));
 
   return rv;
@@ -251,8 +261,8 @@ nsMsgComposeService::GetOrigWindowSelection(MSG_ComposeType type,
   NS_ENSURE_SUCCESS(rv, rv);
 
   nsCOMPtr<nsIDocShell> messagePaneDocShell;
-  RefPtr<mozilla::dom::Element> el = rootShell->GetDocument()->GetElementById(
-      NS_LITERAL_STRING("messagepane"));
+  RefPtr<mozilla::dom::Element> el =
+      rootShell->GetDocument()->GetElementById(u"messagepane"_ns);
   RefPtr<mozilla::dom::XULFrameElement> frame =
       mozilla::dom::XULFrameElement::FromNodeOrNull(el);
   NS_ENSURE_TRUE(frame, NS_ERROR_FAILURE);
@@ -283,13 +293,11 @@ nsMsgComposeService::GetOrigWindowSelection(MSG_ComposeType type,
     if (requireMultipleWords) {
       if (selPlain.IsEmpty()) return NS_ERROR_ABORT;
 
-      RefPtr<mozilla::intl::LineBreaker> lineBreaker =
-          mozilla::intl::LineBreaker::Create();
-
       if (NS_SUCCEEDED(rv)) {
         const uint32_t length = selPlain.Length();
         const char16_t* unicodeStr = selPlain.get();
-        int32_t endWordPos = lineBreaker->Next(unicodeStr, length, 0);
+        int32_t endWordPos =
+            mozilla::intl::LineBreaker::Next(unicodeStr, length, 0);
 
         // If there's not even one word, then there's not multiple words
         if (endWordPos == NS_LINEBREAKER_NEED_MORE_TEXT) return NS_ERROR_ABORT;
@@ -305,7 +313,8 @@ nsMsgComposeService::GetOrigWindowSelection(MSG_ComposeType type,
     }
 
     if (!charsOnlyIf.IsEmpty()) {
-      if (MsgFindCharInSet(selPlain, charsOnlyIf.get()) < 0)
+      if (MsgFindCharInSet(NS_ConvertUTF16toUTF8(selPlain), charsOnlyIf.get()) <
+          0)
         return NS_ERROR_ABORT;
     }
   }
@@ -318,7 +327,7 @@ nsMsgComposeService::GetOrigWindowSelection(MSG_ComposeType type,
 
   nsCOMPtr<nsIDocumentEncoder> docEncoder = do_createHTMLCopyEncoder();
 
-  rv = docEncoder->Init(domDocument, NS_LITERAL_STRING("text/html"), 0);
+  rv = docEncoder->Init(domDocument, u"text/html"_ns, 0);
   NS_ENSURE_SUCCESS(rv, rv);
 
   rv = docEncoder->SetSelection(sel);
@@ -348,18 +357,12 @@ nsMsgComposeService::GetOrigWindowSelection(MSG_ComposeType type,
 
 MOZ_CAN_RUN_SCRIPT_FOR_DEFINITION NS_IMETHODIMP
 nsMsgComposeService::OpenComposeWindow(
-    const char* msgComposeWindowURL, nsIMsgDBHdr* origMsgHdr,
-    const char* originalMsgURI, MSG_ComposeType type, MSG_ComposeFormat format,
-    nsIMsgIdentity *aIdentity, const nsACString& from, nsIMsgWindow* aMsgWindow) {
+    const nsACString& msgComposeWindowURL, nsIMsgDBHdr* origMsgHdr,
+    const nsACString& originalMsgURI, MSG_ComposeType type,
+    MSG_ComposeFormat format, nsIMsgIdentity* aIdentity, const nsACString& from,
+    nsIMsgWindow* aMsgWindow, bool suppressReplyQuote) {
   nsresult rv;
 
-  // Check for any reply type that wants to ignore the quote.
-  bool ignoreQuote = false;
-  if (type >= nsIMsgCompType::ReplyIgnoreQuote) {
-    type -= nsIMsgCompType::ReplyIgnoreQuote;
-    ignoreQuote = true;
-  }
-
   nsCOMPtr<nsIMsgIdentity> identity = aIdentity;
   if (!identity) GetDefaultIdentity(getter_AddRefs(identity));
 
@@ -407,7 +410,7 @@ nsMsgComposeService::OpenComposeWindow(
 
       // When doing a reply (except with a template) see if there's a selection
       // that we should quote
-      if (!ignoreQuote &&
+      if (!suppressReplyQuote &&
           (type == nsIMsgCompType::Reply || type == nsIMsgCompType::ReplyAll ||
            type == nsIMsgCompType::ReplyToSender ||
            type == nsIMsgCompType::ReplyToGroup ||
@@ -418,7 +421,7 @@ nsMsgComposeService::OpenComposeWindow(
           pMsgComposeParams->SetHtmlToQuote(selHTML);
       }
 
-      if (originalMsgURI && *originalMsgURI) {
+      if (!originalMsgURI.IsEmpty()) {
         if (type == nsIMsgCompType::NewsPost) {
           nsAutoCString newsURI(originalMsgURI);
           nsAutoCString group;
@@ -461,7 +464,8 @@ nsMsgComposeService::OpenComposeWindow(
 #endif
       }  // end if(mLogComposePerformance)
 
-      rv = OpenComposeWindowWithParams(msgComposeWindowURL, pMsgComposeParams);
+      rv = OpenComposeWindowWithParams(
+          PromiseFlatCString(msgComposeWindowURL).get(), pMsgComposeParams);
     }
   }
   return rv;
@@ -471,7 +475,14 @@ NS_IMETHODIMP nsMsgComposeService::GetParamsForMailto(
     nsIURI* aURI, nsIMsgComposeParams** aParams) {
   nsresult rv = NS_OK;
   if (aURI) {
-    nsCOMPtr<nsIMailtoUrl> aMailtoUrl = do_QueryInterface(aURI, &rv);
+    nsCString spec;
+    aURI->GetSpec(spec);
+
+    nsCOMPtr<nsIURI> url;
+    rv = nsMailtoUrl::NewMailtoURI(spec, nullptr, getter_AddRefs(url));
+    NS_ENSURE_SUCCESS(rv, rv);
+    nsCOMPtr<nsIMailtoUrl> aMailtoUrl = do_QueryInterface(url, &rv);
+
     if (NS_SUCCEEDED(rv)) {
       MSG_ComposeFormat requestedComposeFormat = nsIMsgCompFormat::Default;
       nsCString toPart;
@@ -702,7 +713,7 @@ NS_IMETHODIMP nsMsgTemplateReplyHelper::OnStopRunningUrl(nsIURI* aUrl,
 
   mHdrToReplyTo->GetMime2DecodedSubject(replySubject);
   mTemplateHdr->GetMime2DecodedSubject(templateSubject);
-  nsString subject(NS_LITERAL_STRING("Auto: "));  // RFC 3834 3.1.5.
+  nsString subject(u"Auto: "_ns);  // RFC 3834 3.1.5.
   subject.Append(templateSubject);
   if (!replySubject.IsEmpty()) {
     subject.AppendLiteral(u" (was: ");
@@ -711,12 +722,11 @@ NS_IMETHODIMP nsMsgTemplateReplyHelper::OnStopRunningUrl(nsIURI* aUrl,
   }
 
   compFields->SetSubject(subject);
-  compFields->SetRawHeader("Auto-Submitted", NS_LITERAL_CSTRING("auto-replied"));
+  compFields->SetRawHeader("Auto-Submitted", "auto-replied"_ns);
 
   nsCString charset;
   rv = mTemplateHdr->GetCharset(getter_Copies(charset));
   NS_ENSURE_SUCCESS(rv, rv);
-  compFields->SetCharacterSet(charset.get());
   rv = nsMsgI18NConvertToUnicode(charset, mTemplateBody, body);
   NS_WARNING_ASSERTION(NS_SUCCEEDED(rv),
                        "couldn't convert templ body to unicode");
@@ -731,7 +741,7 @@ NS_IMETHODIMP nsMsgTemplateReplyHelper::OnStopRunningUrl(nsIURI* aUrl,
   pMsgComposeParams->SetFormat(nsIMsgCompFormat::Default);
   pMsgComposeParams->SetIdentity(mIdentity);
   pMsgComposeParams->SetComposeFields(compFields);
-  pMsgComposeParams->SetOriginalMsgURI(msgUri.get());
+  pMsgComposeParams->SetOriginalMsgURI(msgUri);
 
   // create the nsIMsgCompose object to send the object
   nsCOMPtr<nsIMsgCompose> pMsgCompose(
@@ -744,8 +754,9 @@ NS_IMETHODIMP nsMsgTemplateReplyHelper::OnStopRunningUrl(nsIURI* aUrl,
   rv = pMsgCompose->Initialize(pMsgComposeParams, parentWindow, nullptr);
   NS_ENSURE_SUCCESS(rv, rv);
 
+  RefPtr<mozilla::dom::Promise> promise;
   return pMsgCompose->SendMsg(nsIMsgSend::nsMsgDeliverNow, mIdentity, nullptr,
-                              nullptr, nullptr);
+                              nullptr, nullptr, getter_AddRefs(promise));
 }
 
 NS_IMETHODIMP
@@ -815,12 +826,13 @@ nsMsgTemplateReplyHelper::OnDataAvailable(nsIRequest* request,
 }
 
 NS_IMETHODIMP nsMsgComposeService::ReplyWithTemplate(
-    nsIMsgDBHdr* aMsgHdr, const char* templateUri, nsIMsgWindow* aMsgWindow,
-    nsIMsgIncomingServer* aServer) {
+    nsIMsgDBHdr* aMsgHdr, const nsACString& templateUri,
+    nsIMsgWindow* aMsgWindow, nsIMsgIncomingServer* aServer) {
   // To reply with template, we need the message body of the template.
   // I think we're going to need to stream the template message to ourselves,
   // and construct the body, and call setBody on the compFields.
   nsresult rv;
+  const nsPromiseFlatCString& templateUriFlat = PromiseFlatCString(templateUri);
   nsCOMPtr<nsIMsgAccountManager> accountManager =
       do_GetService(NS_MSGACCOUNTMANAGER_CONTRACTID, &rv);
   NS_ENSURE_SUCCESS(rv, rv);
@@ -870,16 +882,16 @@ NS_IMETHODIMP nsMsgComposeService::ReplyWithTemplate(
   nsCOMPtr<nsIMsgFolder> templateFolder;
   nsCOMPtr<nsIMsgDatabase> templateDB;
   nsCString templateMsgHdrUri;
-  const char* query = PL_strstr(templateUri, "?messageId=");
+  const char* query = PL_strstr(templateUriFlat.get(), "?messageId=");
   if (!query) return NS_ERROR_FAILURE;
 
-  nsAutoCString folderUri(Substring(templateUri, query));
+  nsAutoCString folderUri(Substring(templateUriFlat.get(), query));
   rv = GetExistingFolder(folderUri, getter_AddRefs(templateFolder));
   NS_ENSURE_SUCCESS(rv, rv);
   rv = templateFolder->GetMsgDatabase(getter_AddRefs(templateDB));
   NS_ENSURE_SUCCESS(rv, rv);
 
-  const char* subject = PL_strstr(templateUri, "&subject=");
+  const char* subject = PL_strstr(templateUriFlat.get(), "&subject=");
   if (subject) {
     const char* subjectEnd = subject + strlen(subject);
     nsAutoCString messageId(Substring(query + 11, subject));
@@ -910,7 +922,7 @@ NS_IMETHODIMP nsMsgComposeService::ReplyWithTemplate(
 
   nsCOMPtr<nsIURI> dummyNull;
   rv = msgService->StreamMessage(
-      templateMsgHdrUri.get(), listenerSupports, aMsgWindow, helper,
+      templateMsgHdrUri, listenerSupports, aMsgWindow, helper,
       false,  // convert data
       EmptyCString(), false, getter_AddRefs(dummyNull));
   NS_ENSURE_SUCCESS(rv, rv);
@@ -977,7 +989,7 @@ nsMsgComposeService::ForwardMessage(const nsAString& forwardTo,
   if (aForwardType == nsIMsgComposeService::kForwardInline)
     return RunMessageThroughMimeDraft(
         uriToOpen, nsMimeOutput::nsMimeMessageDraftOrTemplate, identity,
-        uriToOpen.get(), aMsgHdr, true, forwardTo, false, aMsgWindow);
+        uriToOpen, aMsgHdr, true, forwardTo, false, aMsgWindow);
 
   nsCOMPtr<mozIDOMWindowProxy> parentWindow;
   if (aMsgWindow) {
@@ -1000,7 +1012,7 @@ nsMsgComposeService::ForwardMessage(const nsAString& forwardTo,
   pMsgComposeParams->SetFormat(nsIMsgCompFormat::Default);
   pMsgComposeParams->SetIdentity(identity);
   pMsgComposeParams->SetComposeFields(compFields);
-  pMsgComposeParams->SetOriginalMsgURI(uriToOpen.get());
+  pMsgComposeParams->SetOriginalMsgURI(uriToOpen);
   // create the nsIMsgCompose object to send the object
   nsCOMPtr<nsIMsgCompose> pMsgCompose(
       do_CreateInstance(NS_MSGCOMPOSE_CONTRACTID, &rv));
@@ -1011,8 +1023,9 @@ nsMsgComposeService::ForwardMessage(const nsAString& forwardTo,
   rv = pMsgCompose->Initialize(pMsgComposeParams, parentWindow, nullptr);
   NS_ENSURE_SUCCESS(rv, rv);
 
+  RefPtr<Promise> promise;
   rv = pMsgCompose->SendMsg(nsIMsgSend::nsMsgDeliverNow, identity, nullptr,
-                            nullptr, nullptr);
+                            nullptr, nullptr, getter_AddRefs(promise));
   NS_ENSURE_SUCCESS(rv, rv);
 
   // nsMsgCompose::ProcessReplyFlags usually takes care of marking messages
@@ -1142,7 +1155,7 @@ nsMsgComposeService::RegisterComposeDocShell(nsIDocShell* aDocShell,
   NS_ENSURE_SUCCESS(rv, rv);
   nsWeakPtr weakMsgComposePtr = do_GetWeakReference(aComposeObject);
   NS_ENSURE_SUCCESS(rv, rv);
-  mOpenComposeWindows.Put(weakDocShell, weakMsgComposePtr);
+  mOpenComposeWindows.InsertOrUpdate(weakDocShell, weakMsgComposePtr);
 
   return rv;
 }
@@ -1192,7 +1205,7 @@ nsMsgComposeService::GetMsgComposeForDocShell(nsIDocShell* aDocShell,
  */
 nsresult nsMsgComposeService::LoadDraftOrTemplate(
     const nsACString& aMsgURI, nsMimeOutputType aOutType,
-    nsIMsgIdentity* aIdentity, const char* aOriginalMsgURI,
+    nsIMsgIdentity* aIdentity, const nsACString& aOriginalMsgURI,
     nsIMsgDBHdr* aOrigMsgHdr, bool aForwardInline, bool overrideComposeFormat,
     nsIMsgWindow* aMsgWindow) {
   return RunMessageThroughMimeDraft(
@@ -1225,7 +1238,7 @@ nsresult nsMsgComposeService::LoadDraftOrTemplate(
  */
 nsresult nsMsgComposeService::RunMessageThroughMimeDraft(
     const nsACString& aMsgURI, nsMimeOutputType aOutType,
-    nsIMsgIdentity* aIdentity, const char* aOriginalMsgURI,
+    nsIMsgIdentity* aIdentity, const nsACString& aOriginalMsgURI,
     nsIMsgDBHdr* aOrigMsgHdr, bool aForwardInline, const nsAString& aForwardTo,
     bool aOverrideComposeFormat, nsIMsgWindow* aMsgWindow) {
   nsCOMPtr<nsIMsgMessageService> messageService;
@@ -1251,22 +1264,21 @@ nsresult nsMsgComposeService::RunMessageThroughMimeDraft(
   mimeConverter->SetOrigMsgHdr(aOrigMsgHdr);
 
   nsCOMPtr<nsIURI> url;
-  bool fileUrl = StringBeginsWith(aMsgURI, NS_LITERAL_CSTRING("file:"));
+  bool fileUrl = StringBeginsWith(aMsgURI, "file:"_ns);
   nsCString mailboxUri(aMsgURI);
   if (fileUrl) {
     // We loaded a .eml file from a file: url. Construct equivalent mailbox url.
-    mailboxUri.Replace(0, 5, NS_LITERAL_CSTRING("mailbox:"));
+    mailboxUri.Replace(0, 5, "mailbox:"_ns);
     mailboxUri.AppendLiteral("&number=0");
     // Need this to prevent nsMsgCompose::TagEmbeddedObjects from setting
     // inline images as moz-do-not-send.
-    mimeConverter->SetOriginalMsgURI(mailboxUri.get());
+    mimeConverter->SetOriginalMsgURI(mailboxUri);
   }
   if (fileUrl || PromiseFlatCString(aMsgURI).Find(
                      "&type=application/x-message-display") >= 0)
     rv = NS_NewURI(getter_AddRefs(url), mailboxUri);
   else
-    rv = messageService->GetUrlForUri(PromiseFlatCString(aMsgURI).get(),
-                                      getter_AddRefs(url), aMsgWindow);
+    rv = messageService->GetUrlForUri(aMsgURI, aMsgWindow, getter_AddRefs(url));
   NS_ENSURE_SUCCESS(rv, rv);
 
   nsCOMPtr<nsIMsgMailNewsUrl> mailnewsurl = do_QueryInterface(url);
@@ -1281,30 +1293,24 @@ nsresult nsMsgComposeService::RunMessageThroughMimeDraft(
   rv = mailnewsurl->SetSpecInternal(mailboxUri);
   NS_ENSURE_SUCCESS(rv, rv);
 
-  // if we are forwarding a message and that message used a charset over ride
-  // then use that over ride charset instead of the charset specified in the
-  // message
-  nsCString mailCharset;
+  // if we are forwarding a message and that message used a charset override
+  // then forward that as auto-detect flag, too.
+  bool autodetectCharset = false;
   if (aMsgWindow) {
-    bool charsetOverride;
-    if (NS_SUCCEEDED(aMsgWindow->GetCharsetOverride(&charsetOverride)) &&
-        charsetOverride) {
-      if (NS_SUCCEEDED(aMsgWindow->GetMailCharacterSet(mailCharset))) {
-        nsCOMPtr<nsIMsgI18NUrl> i18nUrl(do_QueryInterface(url));
-        if (i18nUrl) (void)i18nUrl->SetCharsetOverRide(mailCharset.get());
-      }
+    if (NS_SUCCEEDED(aMsgWindow->GetCharsetOverride(&autodetectCharset)) &&
+        autodetectCharset) {
+      nsCOMPtr<nsIMsgI18NUrl> i18nUrl(do_QueryInterface(url));
+      if (i18nUrl) (void)i18nUrl->SetAutodetectCharset(true);
     }
   }
 
   nsCOMPtr<nsIPrincipal> nullPrincipal =
-      do_CreateInstance("@mozilla.org/nullprincipal;1", &rv);
-  NS_ASSERTION(NS_SUCCEEDED(rv), "CreateInstance of nullprincipal failed");
-  if (NS_FAILED(rv)) return rv;
+      NullPrincipal::CreateWithoutOriginAttributes();
 
   nsCOMPtr<nsIChannel> channel;
   rv = NS_NewInputStreamChannel(
       getter_AddRefs(channel), url, nullptr, nullPrincipal,
-      nsILoadInfo::SEC_ALLOW_CROSS_ORIGIN_DATA_IS_NULL,
+      nsILoadInfo::SEC_ALLOW_CROSS_ORIGIN_SEC_CONTEXT_IS_NULL,
       nsIContentPolicy::TYPE_OTHER);
   NS_ASSERTION(NS_SUCCEEDED(rv), "NS_NewChannel failed.");
   if (NS_FAILED(rv)) return rv;
@@ -1316,9 +1322,9 @@ nsresult nsMsgComposeService::RunMessageThroughMimeDraft(
   // Now, just plug the two together and get the hell out of the way!
   nsCOMPtr<nsIStreamListener> streamListener = do_QueryInterface(mimeConverter);
   nsCOMPtr<nsIURI> dummyNull;
-  return messageService->DisplayMessage(
-      PromiseFlatCString(aMsgURI).get(), streamListener, aMsgWindow, nullptr,
-      mailCharset.get(), getter_AddRefs(dummyNull));
+  return messageService->DisplayMessage(aMsgURI, streamListener, aMsgWindow,
+                                        nullptr, autodetectCharset,
+                                        getter_AddRefs(dummyNull));
 }
 
 NS_IMETHODIMP
@@ -1330,14 +1336,15 @@ nsMsgComposeService::Handle(nsICommandLine* aCmdLine) {
   nsAutoString uristr;
   bool composeShouldHandle = true;
 
-  rv = aCmdLine->FindFlag(NS_LITERAL_STRING("compose"), false, &found);
+  rv = aCmdLine->FindFlag(u"compose"_ns, false, &found);
   NS_ENSURE_SUCCESS(rv, rv);
 
 #ifndef MOZ_SUITE
   // MAC OS X passes in -url mailto:mscott@mozilla.org into the command line
   // instead of -compose.
   if (found == -1) {
-    rv = aCmdLine->FindFlag(NS_LITERAL_STRING("url"), false, &found);
+    rv = aCmdLine->FindFlag(u"url"_ns, false, &found);
+    NS_ENSURE_SUCCESS(rv, rv);
     // we don't want to consume the argument for -url unless we're sure it is a
     // mailto url and we'll figure that out shortly.
     composeShouldHandle = false;
@@ -1353,18 +1360,18 @@ nsMsgComposeService::Handle(nsICommandLine* aCmdLine) {
 
   if (count > found + 1) {
     aCmdLine->GetArgument(found + 1, uristr);
-    if (StringBeginsWith(uristr, NS_LITERAL_STRING("mailto:")) ||
-        StringBeginsWith(uristr, NS_LITERAL_STRING("preselectid=")) ||
-        StringBeginsWith(uristr, NS_LITERAL_STRING("to=")) ||
-        StringBeginsWith(uristr, NS_LITERAL_STRING("cc=")) ||
-        StringBeginsWith(uristr, NS_LITERAL_STRING("bcc=")) ||
-        StringBeginsWith(uristr, NS_LITERAL_STRING("newsgroups=")) ||
-        StringBeginsWith(uristr, NS_LITERAL_STRING("subject=")) ||
-        StringBeginsWith(uristr, NS_LITERAL_STRING("format=")) ||
-        StringBeginsWith(uristr, NS_LITERAL_STRING("body=")) ||
-        StringBeginsWith(uristr, NS_LITERAL_STRING("attachment=")) ||
-        StringBeginsWith(uristr, NS_LITERAL_STRING("message=")) ||
-        StringBeginsWith(uristr, NS_LITERAL_STRING("from="))) {
+    if (StringBeginsWith(uristr, u"mailto:"_ns) ||
+        StringBeginsWith(uristr, u"preselectid="_ns) ||
+        StringBeginsWith(uristr, u"to="_ns) ||
+        StringBeginsWith(uristr, u"cc="_ns) ||
+        StringBeginsWith(uristr, u"bcc="_ns) ||
+        StringBeginsWith(uristr, u"newsgroups="_ns) ||
+        StringBeginsWith(uristr, u"subject="_ns) ||
+        StringBeginsWith(uristr, u"format="_ns) ||
+        StringBeginsWith(uristr, u"body="_ns) ||
+        StringBeginsWith(uristr, u"attachment="_ns) ||
+        StringBeginsWith(uristr, u"message="_ns) ||
+        StringBeginsWith(uristr, u"from="_ns)) {
       composeShouldHandle = true;  // the -url argument looks like mailto
       end++;
       // mailto: URIs are frequently passed with spaces in them. They should be
@@ -1393,9 +1400,14 @@ nsMsgComposeService::Handle(nsICommandLine* aCmdLine) {
         do_CreateInstance(NS_SUPPORTS_STRING_CONTRACTID));
     if (arg) arg->SetData(uristr);
 
+    nsCOMPtr<nsIMutableArray> params(do_CreateInstance(NS_ARRAY_CONTRACTID));
+    params->AppendElement(arg);
+    params->AppendElement(aCmdLine);
+
     nsCOMPtr<mozIDOMWindowProxy> opened;
-    wwatch->OpenWindow(nullptr, DEFAULT_CHROME, "_blank",
-                       "chrome,dialog=no,all", arg, getter_AddRefs(opened));
+    wwatch->OpenWindow(nullptr, DEFAULT_CHROME, "_blank"_ns,
+                       "chrome,dialog=no,all"_ns, params,
+                       getter_AddRefs(opened));
 
     aCmdLine->SetPreventDefault(true);
   }
