@@ -6,29 +6,25 @@
  * Test that S/MIME messages are properly displayed and that the MimeMessage
  * representation is correct.
  */
-/* import-globals-from ../../../../test/resources/logHelper.js */
-/* import-globals-from ../../../../test/resources/asyncTestUtils.js */
-/* import-globals-from ../../../../test/resources/MessageGenerator.jsm */
-/* import-globals-from ../../../../test/resources/messageModifier.js */
-/* import-globals-from ../../../../test/resources/messageInjection.js */
-load("../../../../resources/logHelper.js");
-load("../../../../resources/asyncTestUtils.js");
-
-load("../../../../resources/MessageGenerator.jsm");
-load("../../../../resources/messageModifier.js");
-load("../../../../resources/messageInjection.js");
-
-var msgGen = (gMessageGenerator = new MessageGenerator());
 
 var { FileUtils } = ChromeUtils.import("resource://gre/modules/FileUtils.jsm");
+var { MessageGenerator, SyntheticMessageSet } = ChromeUtils.import(
+  "resource://testing-common/mailnews/MessageGenerator.jsm"
+);
+var { MessageInjection } = ChromeUtils.import(
+  "resource://testing-common/mailnews/MessageInjection.jsm"
+);
 var { MsgHdrToMimeMessage } = ChromeUtils.import(
   "resource:///modules/gloda/MimeMessage.jsm"
 );
 
+var msgGen;
+var messageInjection;
+
 function initNSS() {
   // Copy the NSS database files over.
   let profile = FileUtils.getDir("ProfD", []);
-  let files = ["cert9.db", "key4.db", "secmod.db"];
+  let files = ["cert9.db", "key4.db"];
   let directory = do_get_file("../../../../data/db-tinderbox-invalid");
   for (let f of files) {
     let keydb = directory.clone();
@@ -40,8 +36,13 @@ function initNSS() {
   Cc["@mozilla.org/psm;1"].getService(Ci.nsISupports);
 }
 
-var gInbox;
-function* test_smime_mimemsg() {
+add_task(async function setupTest() {
+  initNSS();
+  msgGen = new MessageGenerator();
+  messageInjection = new MessageInjection({ mode: "local" }, msgGen);
+});
+
+add_task(async function test_smime_mimemsg() {
   let msg = msgGen.makeEncryptedSMimeMessage({
     from: ["Tinderbox", "tinderbox@foo.invalid"],
     to: [["Tinderbox", "tinderbox@foo.invalid"]],
@@ -49,10 +50,17 @@ function* test_smime_mimemsg() {
     body: { body: encrypted_blurb },
   });
   let synSet = new SyntheticMessageSet([msg]);
-  yield add_sets_to_folder(gInbox, [synSet]);
+  await messageInjection.addSetsToFolders(
+    [messageInjection.getInboxFolder()],
+    [synSet]
+  );
 
   let msgHdr = synSet.getMsgHdr(0);
 
+  let promiseResolve;
+  let promise = new Promise(resolve => {
+    promiseResolve = resolve;
+  });
   // Make sure by default, MimeMessages do not include encrypted parts
   MsgHdrToMimeMessage(
     msgHdr,
@@ -66,13 +74,18 @@ function* test_smime_mimemsg() {
       Assert.equal(aMimeMsg.parts[0].parts.length, 0);
       // Make sure we can't see the attachment
       Assert.equal(aMimeMsg.allUserAttachments.length, 0);
-      async_driver();
+      promiseResolve();
     },
     true,
     {}
   );
 
-  yield false;
+  await promise;
+
+  // Reset promise.
+  promise = new Promise(resolve => {
+    promiseResolve = resolve;
+  });
 
   // Now what about we specifically ask to "see" the encrypted parts?
   MsgHdrToMimeMessage(
@@ -90,7 +103,7 @@ function* test_smime_mimemsg() {
       // Make sure we can see the attachment
       Assert.equal(aMimeMsg.allUserAttachments.length, 1);
       Assert.equal(aMimeMsg.allUserAttachments[0].contentType, "image/jpeg");
-      async_driver();
+      promiseResolve();
       // Extra little bit of testing
     },
     true,
@@ -98,16 +111,8 @@ function* test_smime_mimemsg() {
       examineEncryptedParts: true,
     }
   );
-  yield false;
-}
-
-var tests = [test_smime_mimemsg];
-
-function run_test() {
-  initNSS();
-  gInbox = configure_message_injection({ mode: "local" });
-  async_run_tests(tests);
-}
+  await promise;
+});
 
 var encrypted_blurb =
   "MIAGCSqGSIb3DQEHA6CAMIACAQAxgf8wgfwCAQAwZTBgMQswCQYDVQQGEwJTVzETMBEGA1UE\n" +
