@@ -28,10 +28,13 @@
 #include "nsIChannel.h"
 #include "nsDependentSubstring.h"
 #include "nsMemory.h"
+#include "nsUnicodeProperties.h"
 
 #include "mozilla/ArenaAllocatorExtensions.h"  // for ArenaStrdup
 
 using namespace mozilla;
+using mozilla::intl::Script;
+using mozilla::intl::UnicodeProperties;
 
 // needed to mark attachment flag on the db hdr
 #include "nsIMsgHdr.h"
@@ -61,7 +64,7 @@ static unsigned int kMaxLengthForToken =
     12;  // upper bound on the number of characters in a word to be declared as
          // a token
 
-#define FORGED_RECEIVED_HEADER_HINT NS_LITERAL_CSTRING("may be forged")
+#define FORGED_RECEIVED_HEADER_HINT "may be forged"_ns
 
 #ifndef M_LN2
 #  define M_LN2 0.69314718055994530942
@@ -500,8 +503,7 @@ void Tokenizer::tokenizeHeaders(nsIUTF8StringEnumerator* aHeaderNames,
                  // just fold the case and any leading/trailing white space
         // all headers beginning with x-mozilla are being changed by us, so
         // ignore
-        if (StringBeginsWith(headerName, NS_LITERAL_CSTRING("x-mozilla")))
-          break;
+        if (StringBeginsWith(headerName, "x-mozilla"_ns)) break;
         // fall through
         [[fallthrough]];
       case 'u':
@@ -564,6 +566,38 @@ void Tokenizer::tokenize_ascii_word(char* aWord) {
   }
 }
 
+// Copied from mozilla/intl/lwbrk/WordBreaker.cpp
+
+#define ASCII_IS_ALPHA(c) \
+  ((('a' <= (c)) && ((c) <= 'z')) || (('A' <= (c)) && ((c) <= 'Z')))
+#define ASCII_IS_DIGIT(c) (('0' <= (c)) && ((c) <= '9'))
+#define ASCII_IS_SPACE(c) \
+  ((' ' == (c)) || ('\t' == (c)) || ('\r' == (c)) || ('\n' == (c)))
+#define IS_ALPHABETICAL_SCRIPT(c) ((c) < 0x2E80)
+
+// we change the beginning of IS_HAN from 0x4e00 to 0x3400 to relfect
+// Unicode 3.0
+#define IS_HAN(c) \
+  ((0x3400 <= (c)) && ((c) <= 0x9fff)) || ((0xf900 <= (c)) && ((c) <= 0xfaff))
+#define IS_KATAKANA(c) ((0x30A0 <= (c)) && ((c) <= 0x30FF))
+#define IS_HIRAGANA(c) ((0x3040 <= (c)) && ((c) <= 0x309F))
+#define IS_HALFWIDTHKATAKANA(c) ((0xFF60 <= (c)) && ((c) <= 0xFF9F))
+
+// Return true if aChar belongs to a SEAsian script that is written without
+// word spaces, so we need to use the "complex breaker" to find possible word
+// boundaries. (https://en.wikipedia.org/wiki/Scriptio_continua)
+// (How well this works depends on the level of platform support for finding
+// possible line breaks - or possible word boundaries - in the particular
+// script. Thai, at least, works pretty well on the major desktop OSes. If
+// the script is not supported by the platform, we just won't find any useful
+// boundaries.)
+static bool IsScriptioContinua(char16_t aChar) {
+  Script sc = UnicodeProperties::GetScriptCode(aChar);
+  return sc == Script::THAI || sc == Script::MYANMAR || sc == Script::KHMER ||
+         sc == Script::JAVANESE || sc == Script::BALINESE ||
+         sc == Script::SUNDANESE || sc == Script::LAO;
+}
+
 // one subtract and one conditional jump should be faster than two conditional
 // jump on most recent system.
 #define IN_RANGE(x, low, high) ((uint16_t)((x) - (low)) <= (high) - (low))
@@ -617,7 +651,7 @@ static char_class getCharClass(char16_t c) {
 
 static bool isJapanese(const char* word) {
   nsString text = NS_ConvertUTF8toUTF16(word);
-  char16_t* p = (char16_t*)text.get();
+  const char16_t* p = (const char16_t*)text.get();
   char16_t c;
 
   // it is japanese chunk if it contains any hiragana or katakana.
@@ -670,14 +704,67 @@ nsresult Tokenizer::stripHTML(const nsAString& inString, nsAString& outString) {
   return utils->ConvertToPlainText(inString, flags, 80, outString);
 }
 
+// Copied from WorfdBreker.cpp due to changes in bug 1728708.
+enum WordBreakClass : uint8_t {
+  kWbClassSpace = 0,
+  kWbClassAlphaLetter,
+  kWbClassPunct,
+  kWbClassHanLetter,
+  kWbClassKatakanaLetter,
+  kWbClassHiraganaLetter,
+  kWbClassHWKatakanaLetter,
+  kWbClassScriptioContinua
+};
+
+WordBreakClass GetWordBreakClass(char16_t c) {
+  // begin of the hack
+
+  if (IS_ALPHABETICAL_SCRIPT(c)) {
+    if (IS_ASCII(c)) {
+      if (ASCII_IS_SPACE(c)) {
+        return WordBreakClass::kWbClassSpace;
+      }
+      if (ASCII_IS_ALPHA(c) || ASCII_IS_DIGIT(c) || (c == '_')) {
+        return WordBreakClass::kWbClassAlphaLetter;
+      }
+      return WordBreakClass::kWbClassPunct;
+    }
+    if (c == 0x00A0 /*NBSP*/) {
+      return WordBreakClass::kWbClassSpace;
+    }
+    if (mozilla::unicode::GetGenCategory(c) == nsUGenCategory::kPunctuation) {
+      return WordBreakClass::kWbClassPunct;
+    }
+    if (IsScriptioContinua(c)) {
+      return WordBreakClass::kWbClassScriptioContinua;
+    }
+    return WordBreakClass::kWbClassAlphaLetter;
+  }
+  if (IS_HAN(c)) {
+    return WordBreakClass::kWbClassHanLetter;
+  }
+  if (IS_KATAKANA(c)) {
+    return kWbClassKatakanaLetter;
+  }
+  if (IS_HIRAGANA(c)) {
+    return WordBreakClass::kWbClassHiraganaLetter;
+  }
+  if (IS_HALFWIDTHKATAKANA(c)) {
+    return WordBreakClass::kWbClassHWKatakanaLetter;
+  }
+  if (mozilla::unicode::GetGenCategory(c) == nsUGenCategory::kPunctuation) {
+    return WordBreakClass::kWbClassPunct;
+  }
+  if (IsScriptioContinua(c)) {
+    return WordBreakClass::kWbClassScriptioContinua;
+  }
+  return WordBreakClass::kWbClassAlphaLetter;
+}
+
 // Copied from nsSemanticUnitScanner.cpp which was removed in bug 1368418.
 nsresult Tokenizer::ScannerNext(const char16_t* text, int32_t length,
                                 int32_t pos, bool isLastBuffer, int32_t* begin,
                                 int32_t* end, bool* _retval) {
-  if (!mWordBreaker) {
-    mWordBreaker = mozilla::intl::WordBreaker::Create();
-  }
-
   // if we reach the end, just return
   if (pos >= length) {
     *begin = pos;
@@ -686,12 +773,11 @@ nsresult Tokenizer::ScannerNext(const char16_t* text, int32_t length,
     return NS_OK;
   }
 
-  mozilla::intl::WordBreakClass char_class =
-      mozilla::intl::WordBreaker::GetClass(text[pos]);
+  WordBreakClass char_class = GetWordBreakClass(text[pos]);
 
   // If we are in Chinese mode, return one Han letter at a time.
   // We should not do this if we are in Japanese or Korean mode.
-  if (mozilla::intl::kWbClassHanLetter == char_class) {
+  if (WordBreakClass::kWbClassHanLetter == char_class) {
     *begin = pos;
     *end = pos + 1;
     *_retval = true;
@@ -700,7 +786,8 @@ nsresult Tokenizer::ScannerNext(const char16_t* text, int32_t length,
 
   int32_t next;
   // Find the next "word".
-  next = mWordBreaker->NextWord(text, (uint32_t)length, (uint32_t)pos);
+  next =
+      mozilla::intl::WordBreaker::Next(text, (uint32_t)length, (uint32_t)pos);
 
   // If we don't have enough text to make decision, return.
   if (next == NS_WORDBREAKER_NEED_MORE_TEXT) {
@@ -711,8 +798,8 @@ nsresult Tokenizer::ScannerNext(const char16_t* text, int32_t length,
   }
 
   // If what we got is space or punct, look at the next break.
-  if (char_class == mozilla::intl::kWbClassSpace ||
-      char_class == mozilla::intl::kWbClassPunct) {
+  if (char_class == WordBreakClass::kWbClassSpace ||
+      char_class == WordBreakClass::kWbClassPunct) {
     // If the next "word" is not letters,
     // call itself recursively with the new pos.
     return ScannerNext(text, length, next, isLastBuffer, begin, end, _retval);
@@ -742,10 +829,8 @@ void Tokenizer::tokenize(const char* aText) {
   // to RSS
 
   if (mIframeToDiv) {
-    text.ReplaceSubstring(NS_LITERAL_STRING("<iframe"),
-                          NS_LITERAL_STRING("<div"));
-    text.ReplaceSubstring(NS_LITERAL_STRING("/iframe>"),
-                          NS_LITERAL_STRING("/div>"));
+    text.ReplaceSubstring(u"<iframe"_ns, u"<div"_ns);
+    text.ReplaceSubstring(u"/iframe>"_ns, u"/div>"_ns);
   }
 
   stripHTML(text, strippedUCS2);
@@ -870,7 +955,7 @@ class TokenAnalyzer {
     mTokenListener = aTokenListener;
   }
 
-  void setSource(const char* sourceURI) { mTokenSource = sourceURI; }
+  void setSource(const nsACString& sourceURI) { mTokenSource = sourceURI; }
 
   nsCOMPtr<nsIStreamListener> mTokenListener;
   nsCString mTokenSource;
@@ -926,8 +1011,8 @@ NS_IMETHODIMP TokenStreamListener::ProcessHeaders(
 }
 
 NS_IMETHODIMP TokenStreamListener::HandleAttachment(
-    const char* contentType, const char* url, const char16_t* displayName,
-    const char* uri, bool aIsExternalAttachment) {
+    const char* contentType, const nsACString& url, const char16_t* displayName,
+    const nsACString& uri, bool aIsExternalAttachment) {
   mTokenizer.tokenizeAttachment(contentType,
                                 NS_ConvertUTF16toUTF8(displayName).get());
   return NS_OK;
@@ -1134,14 +1219,6 @@ nsBayesianFilter::nsBayesianFilter() : mTrainingDataDirty(false) {
   MOZ_LOG(BayesianFilterLogModule, LogLevel::Warning,
           ("maximum junk tokens: %d", mMaximumTokenCount));
 
-  mTimer = do_CreateInstance(NS_TIMER_CONTRACTID, &rv);
-  NS_ASSERTION(
-      NS_SUCCEEDED(rv),
-      "unable to create a timer; training data will only be written on exit");
-
-  // the timer is not used on object construction, since for
-  // the time being there are no dirying messages
-
   // give a default capacity to the memory structure used to store
   // per-message/per-trait token data
   mAnalysisStore.SetCapacity(kAnalysisStoreCapacity);
@@ -1227,9 +1304,8 @@ class MessageClassifier : public TokenAnalyzer {
 
   virtual ~MessageClassifier() {}
   virtual void analyzeTokens(Tokenizer& tokenizer) {
-    mFilter->classifyMessage(tokenizer, mTokenSource.get(), mProTraits,
-                             mAntiTraits, mJunkListener, mTraitListener,
-                             mDetailListener);
+    mFilter->classifyMessage(tokenizer, mTokenSource, mProTraits, mAntiTraits,
+                             mJunkListener, mTraitListener, mDetailListener);
     tokenizer.clearTokens();
     classifyNextMessage();
   }
@@ -1239,17 +1315,17 @@ class MessageClassifier : public TokenAnalyzer {
       MOZ_LOG(BayesianFilterLogModule, LogLevel::Warning,
               ("classifyNextMessage(%s)",
                mMessageURIs[mCurMessageToClassify].get()));
-      mFilter->tokenizeMessage(mMessageURIs[mCurMessageToClassify].get(),
-                               mMsgWindow, this);
+      mFilter->tokenizeMessage(mMessageURIs[mCurMessageToClassify], mMsgWindow,
+                               this);
     } else {
       // call all listeners with null parameters to signify end of batch
       if (mJunkListener)
-        mJunkListener->OnMessageClassified(nullptr,
+        mJunkListener->OnMessageClassified(EmptyCString(),
                                            nsIJunkMailPlugin::UNCLASSIFIED, 0);
       if (mTraitListener) {
         nsTArray<uint32_t> nullTraits;
         nsTArray<uint32_t> nullPercents;
-        mTraitListener->OnMessageTraitsClassified(nullptr, nullTraits,
+        mTraitListener->OnMessageTraitsClassified(EmptyCString(), nullTraits,
                                                   nullPercents);
       }
       mTokenListener =
@@ -1271,22 +1347,19 @@ class MessageClassifier : public TokenAnalyzer {
   uint32_t mCurMessageToClassify;  // 0-based index
 };
 
-nsresult nsBayesianFilter::tokenizeMessage(const char* aMessageURI,
+nsresult nsBayesianFilter::tokenizeMessage(const nsACString& aMessageURI,
                                            nsIMsgWindow* aMsgWindow,
                                            TokenAnalyzer* aAnalyzer) {
-  NS_ENSURE_ARG_POINTER(aMessageURI);
-
   nsCOMPtr<nsIMsgMessageService> msgService;
-  nsresult rv = GetMessageServiceFromURI(nsDependentCString(aMessageURI),
-                                         getter_AddRefs(msgService));
+  nsresult rv =
+      GetMessageServiceFromURI(aMessageURI, getter_AddRefs(msgService));
   NS_ENSURE_SUCCESS(rv, rv);
 
   aAnalyzer->setSource(aMessageURI);
   nsCOMPtr<nsIURI> dummyNull;
-  return msgService->StreamMessage(aMessageURI, aAnalyzer->mTokenListener,
-                                   aMsgWindow, nullptr, true /* convert data */,
-                                   NS_LITERAL_CSTRING("filter"), false,
-                                   getter_AddRefs(dummyNull));
+  return msgService->StreamMessage(
+      aMessageURI, aAnalyzer->mTokenListener, aMsgWindow, nullptr,
+      true /* convert data */, "filter"_ns, false, getter_AddRefs(dummyNull));
 }
 
 // a TraitAnalysis is the per-token representation of the statistical
@@ -1335,11 +1408,15 @@ static inline double chi2P(double chi2, double nu, int32_t* error) {
 }
 
 void nsBayesianFilter::classifyMessage(
-    Tokenizer& tokenizer, const char* messageURI,
+    Tokenizer& tokenizer, const nsACString& messageURI,
     nsTArray<uint32_t>& aProTraits, nsTArray<uint32_t>& aAntiTraits,
     nsIJunkMailClassificationListener* listener,
     nsIMsgTraitClassificationListener* aTraitListener,
     nsIMsgTraitDetailListener* aDetailListener) {
+  if (aProTraits.Length() != aAntiTraits.Length()) {
+    NS_ERROR("Each Pro trait needs a matching Anti trait");
+    return;
+  }
   Token* tokens = tokenizer.copyTokens();
   uint32_t tokenCount;
   if (!tokens) {
@@ -1351,11 +1428,6 @@ void nsBayesianFilter::classifyMessage(
     tokenCount = tokenizer.countTokens();
   }
 
-  if (aProTraits.Length() != aAntiTraits.Length()) {
-    NS_ERROR("Each Pro trait needs a matching Anti trait");
-    return;
-  }
-
   /* this part is similar to the Graham algorithm with some adjustments. */
   uint32_t traitCount = aProTraits.Length();
 
@@ -1592,7 +1664,7 @@ void nsBayesianFilter::classifyMessage(
       bool isJunk = (prob >= mJunkProbabilityThreshold);
       MOZ_LOG(BayesianFilterLogModule, LogLevel::Info,
               ("%s is junk probability = (%f)  HAM SCORE:%f SPAM SCORE:%f",
-               messageURI, prob, H, S));
+               PromiseFlatCString(messageURI).get(), prob, H, S));
 
       // the algorithm in "A Plan For Spam" assumes that you have a large good
       // corpus and a large junk corpus.
@@ -1639,7 +1711,7 @@ void nsBayesianFilter::classifyMessage(
 }
 
 void nsBayesianFilter::classifyMessage(
-    Tokenizer& tokens, const char* messageURI,
+    Tokenizer& tokens, const nsACString& messageURI,
     nsIJunkMailClassificationListener* aJunkListener) {
   AutoTArray<uint32_t, 1> proTraits;
   AutoTArray<uint32_t, 1> antiTraits;
@@ -1675,9 +1747,9 @@ NS_IMETHODIMP nsBayesianFilter::GetShouldDownloadAllHeaders(
 /* void classifyMessage (in string aMsgURL, in nsIJunkMailClassificationListener
  * aListener); */
 NS_IMETHODIMP nsBayesianFilter::ClassifyMessage(
-    const char* aMessageURL, nsIMsgWindow* aMsgWindow,
+    const nsACString& aMessageURL, nsIMsgWindow* aMsgWindow,
     nsIJunkMailClassificationListener* aListener) {
-  AutoTArray<nsCString, 1> urls = {nsDependentCString(aMessageURL)};
+  AutoTArray<nsCString, 1> urls = {PromiseFlatCString(aMessageURL)};
   MessageClassifier* analyzer =
       new MessageClassifier(this, aListener, aMsgWindow, urls);
   NS_ENSURE_TRUE(analyzer, NS_ERROR_OUT_OF_MEMORY);
@@ -1699,7 +1771,7 @@ NS_IMETHODIMP nsBayesianFilter::ClassifyMessages(
   TokenStreamListener* tokenListener = new TokenStreamListener(analyzer);
   NS_ENSURE_TRUE(tokenListener, NS_ERROR_OUT_OF_MEMORY);
   analyzer->setTokenListener(tokenListener);
-  return tokenizeMessage(aMsgURLs[0].get(), aMsgWindow, analyzer);
+  return tokenizeMessage(aMsgURLs[0], aMsgWindow, analyzer);
 }
 
 nsresult nsBayesianFilter::setAnalysis(Token& token, uint32_t aTraitIndex,
@@ -1782,7 +1854,7 @@ NS_IMETHODIMP nsBayesianFilter::ClassifyTraitsInMessages(
   TokenStreamListener* tokenListener = new TokenStreamListener(analyzer);
 
   analyzer->setTokenListener(tokenListener);
-  return tokenizeMessage(aMsgURIs[0].get(), aMsgWindow, analyzer);
+  return tokenizeMessage(aMsgURIs[0], aMsgWindow, analyzer);
 }
 
 class MessageObserver : public TokenAnalyzer {
@@ -1800,7 +1872,7 @@ class MessageObserver : public TokenAnalyzer {
         mNewClassifications(aNewClassifications.Clone()) {}
 
   virtual void analyzeTokens(Tokenizer& tokenizer) {
-    mFilter->observeMessage(tokenizer, mTokenSource.get(), mOldClassifications,
+    mFilter->observeMessage(tokenizer, mTokenSource, mOldClassifications,
                             mNewClassifications, mJunkListener, mTraitListener);
     // release reference to listener, which will allow us to go away as well.
     mTokenListener = nullptr;
@@ -1816,7 +1888,7 @@ class MessageObserver : public TokenAnalyzer {
 };
 
 NS_IMETHODIMP nsBayesianFilter::SetMsgTraitClassification(
-    const char* aMsgURI, const nsTArray<uint32_t>& aOldTraits,
+    const nsACString& aMsgURI, const nsTArray<uint32_t>& aOldTraits,
     const nsTArray<uint32_t>& aNewTraits,
     nsIMsgTraitClassificationListener* aTraitListener, nsIMsgWindow* aMsgWindow,
     nsIJunkMailClassificationListener* aJunkListener) {
@@ -1833,7 +1905,7 @@ NS_IMETHODIMP nsBayesianFilter::SetMsgTraitClassification(
 
 // set new message classifications for a message
 void nsBayesianFilter::observeMessage(
-    Tokenizer& tokenizer, const char* messageURL,
+    Tokenizer& tokenizer, const nsACString& messageURL,
     nsTArray<uint32_t>& oldClassifications,
     nsTArray<uint32_t>& newClassifications,
     nsIJunkMailClassificationListener* aJunkListener,
@@ -1905,14 +1977,19 @@ void nsBayesianFilter::observeMessage(
     aTraitListener->OnMessageTraitsClassified(messageURL, traits, percents);
   }
 
-  if (mTrainingDataDirty && !trainingDataWasDirty && (mTimer != nullptr)) {
+  if (mTrainingDataDirty && !trainingDataWasDirty) {
     // if training data became dirty just now, schedule flush
     // mMinFlushInterval msec from now
     MOZ_LOG(BayesianFilterLogModule, LogLevel::Debug,
             ("starting training data flush timer %i msec", mMinFlushInterval));
-    mTimer->InitWithNamedFuncCallback(
-        nsBayesianFilter::TimerCallback, this, mMinFlushInterval,
-        nsITimer::TYPE_ONE_SHOT, "nsBayesianFilter::TimerCallback");
+
+    nsresult rv = NS_NewTimerWithFuncCallback(
+        getter_AddRefs(mTimer), nsBayesianFilter::TimerCallback, (void*)this,
+        mMinFlushInterval, nsITimer::TYPE_ONE_SHOT,
+        "nsBayesianFilter::TimerCallback", nullptr);
+    if (NS_FAILED(rv)) {
+      NS_WARNING("Could not start nsBayesianFilter timer");
+    }
   }
 }
 
@@ -1925,7 +2002,7 @@ NS_IMETHODIMP nsBayesianFilter::GetUserHasClassified(bool* aResult) {
 
 // Set message classification (only allows junk and good)
 NS_IMETHODIMP nsBayesianFilter::SetMessageClassification(
-    const char* aMsgURL, nsMsgJunkStatus aOldClassification,
+    const nsACString& aMsgURL, nsMsgJunkStatus aOldClassification,
     nsMsgJunkStatus aNewClassification, nsIMsgWindow* aMsgWindow,
     nsIJunkMailClassificationListener* aListener) {
   AutoTArray<uint32_t, 1> oldClassifications;
@@ -1957,11 +2034,11 @@ NS_IMETHODIMP nsBayesianFilter::ResetTrainingData() {
 }
 
 NS_IMETHODIMP nsBayesianFilter::DetailMessage(
-    const char* aMsgURI, uint32_t aProTrait, uint32_t aAntiTrait,
+    const nsACString& aMsgURI, uint32_t aProTrait, uint32_t aAntiTrait,
     nsIMsgTraitDetailListener* aDetailListener, nsIMsgWindow* aMsgWindow) {
   AutoTArray<uint32_t, 1> proTraits = {aProTrait};
   AutoTArray<uint32_t, 1> antiTraits = {aAntiTrait};
-  AutoTArray<nsCString, 1> uris = {nsDependentCString(aMsgURI)};
+  AutoTArray<nsCString, 1> uris = {PromiseFlatCString(aMsgURI)};
 
   MessageClassifier* analyzer =
       new MessageClassifier(this, nullptr, nullptr, aDetailListener, proTraits,
@@ -2168,7 +2245,7 @@ nsresult CorpusStore::getTrainingFile(nsIFile** aTrainingFile) {
   nsresult rv = NS_GetSpecialDirectory(NS_APP_USER_PROFILE_50_DIR,
                                        getter_AddRefs(profileDir));
   NS_ENSURE_SUCCESS(rv, rv);
-  rv = profileDir->Append(NS_LITERAL_STRING("training.dat"));
+  rv = profileDir->Append(u"training.dat"_ns);
   NS_ENSURE_SUCCESS(rv, rv);
 
   return profileDir->QueryInterface(NS_GET_IID(nsIFile), (void**)aTrainingFile);
@@ -2182,7 +2259,7 @@ nsresult CorpusStore::getTraitFile(nsIFile** aTraitFile) {
                                        getter_AddRefs(profileDir));
   NS_ENSURE_SUCCESS(rv, rv);
 
-  rv = profileDir->Append(NS_LITERAL_STRING("traits.dat"));
+  rv = profileDir->Append(u"traits.dat"_ns);
   NS_ENSURE_SUCCESS(rv, rv);
 
   return profileDir->QueryInterface(NS_GET_IID(nsIFile), (void**)aTraitFile);
