@@ -62,9 +62,6 @@ class nsImapIncomingServer : public nsMsgIncomingServer,
   NS_IMETHOD GetSearchScope(nsMsgSearchScopeValue* searchScope) override;
   NS_IMETHOD GetServerRequiresPasswordForBiff(
       bool* aServerRequiresPasswordForBiff) override;
-  NS_IMETHOD OnUserOrHostNameChanged(const nsACString& oldName,
-                                     const nsACString& newName,
-                                     bool hostnameChanged) override;
   NS_IMETHOD GetNumIdleConnections(int32_t* aNumIdleConnections);
   NS_IMETHOD ForgetSessionPassword() override;
   NS_IMETHOD GetMsgFolderFromURI(nsIMsgFolder* aFolderResource,
@@ -77,13 +74,12 @@ class nsImapIncomingServer : public nsMsgIncomingServer,
  protected:
   virtual ~nsImapIncomingServer();
   nsresult GetFolder(const nsACString& name, nsIMsgFolder** pFolder);
-  virtual nsresult CreateRootFolderFromUri(const nsCString& serverUri,
+  virtual nsresult CreateRootFolderFromUri(const nsACString& serverUri,
                                            nsIMsgFolder** rootFolder) override;
   nsresult ResetFoldersToUnverified(nsIMsgFolder* parentFolder);
   void GetUnverifiedSubFolders(nsIMsgFolder* parentFolder,
                                nsCOMArray<nsIMsgImapMailFolder>& aFoldersArray);
   void GetUnverifiedFolders(nsCOMArray<nsIMsgImapMailFolder>& aFolderArray);
-  nsresult DeleteNonVerifiedFolders(nsIMsgFolder* parentFolder);
   bool NoDescendentsAreVerified(nsIMsgFolder* parentFolder);
   bool AllDescendentsAreNoSelect(nsIMsgFolder* parentFolder);
 
@@ -134,6 +130,8 @@ class nsImapIncomingServer : public nsMsgIncomingServer,
   bool mDoingSubscribeDialog;
   bool mDoingLsub;
   bool m_shuttingDown;
+  bool mUtf8AcceptEnabled;
+  bool m_skipRetryQueued;
 
   mozilla::Mutex mLock;
   // subscribe dialog stuff
