@@ -25,8 +25,7 @@
 #include "nsIMsgFilterPlugin.h"
 #include "nsISimpleEnumerator.h"
 #include "nsIStringEnumerator.h"
-#include "nsDataHashtable.h"
-#include "nsIMutableArray.h"
+#include "nsTHashMap.h"
 #include "nsITimer.h"
 #include "nsCOMArray.h"
 #include "nsAutoSyncState.h"
@@ -52,11 +51,10 @@ class nsImapMailCopyState : public nsISupports {
 
   nsImapMailCopyState();
 
-  nsCOMPtr<nsISupports> m_srcSupport;  // source file spec or folder
-  nsCOMPtr<nsIArray> m_messages;       // array of source messages
+  nsCOMPtr<nsISupports> m_srcSupport;        // source file spec or folder
+  nsTArray<RefPtr<nsIMsgDBHdr>> m_messages;  // array of source messages
   RefPtr<nsImapMoveCopyMsgTxn>
-      m_undoMsgTxn;                 // undo object with this copy operation
-  nsCOMPtr<nsIMsgDBHdr> m_message;  // current message to be copied
+      m_undoMsgTxn;  // undo object with this copy operation
   nsCOMPtr<nsIMsgCopyServiceListener> m_listener;  // listener of this copy
                                                    // operation
   nsCOMPtr<nsIFile> m_tmpFile;         // temp file spec for copy operation
@@ -70,7 +68,6 @@ class nsImapMailCopyState : public nsISupports {
   bool m_isCrossServerOp;  // are we copying between imap servers?
   uint32_t m_curIndex;     // message index to the message array which we are
                            // copying
-  uint32_t m_totalCount;   // total count of messages we have to do
   uint32_t m_unreadCount;  // num unread messages we're moving
   bool m_streamCopy;
   char* m_dataBuffer;  // temporary buffer for this copy operation
@@ -79,7 +76,7 @@ class nsImapMailCopyState : public nsISupports {
   uint32_t m_leftOver;
   bool m_allowUndo;
   bool m_eatLF;
-  uint32_t m_newMsgFlags;      // only used if there's no m_message
+  uint32_t m_newMsgFlags;      // only used if m_messages is empty
   nsCString m_newMsgKeywords;  // ditto
   // If the server supports UIDPLUS, this is the UID for the append,
   // if we're doing an append.
@@ -177,7 +174,7 @@ class nsMsgIMAPFolderACL {
   void UpdateACLCache();
 
  protected:
-  nsDataHashtable<nsCStringHashKey, nsCString>
+  nsTHashMap<nsCStringHashKey, nsCString>
       m_rightsHash;  // Hash table, mapping username strings to rights strings.
   nsImapMailFolder* m_folder;
   int32_t m_aclCount;
@@ -224,9 +221,8 @@ class nsImapMailFolder : public nsMsgDBFolder,
   NS_DECL_ISUPPORTS_INHERITED
 
   // nsIMsgFolder methods:
-  NS_IMETHOD GetSubFolders(nsISimpleEnumerator** aResult) override;
+  NS_IMETHOD GetSubFolders(nsTArray<RefPtr<nsIMsgFolder>>& folders) override;
 
-  NS_IMETHOD GetMessages(nsISimpleEnumerator** result) override;
   NS_IMETHOD UpdateFolder(nsIMsgWindow* aWindow) override;
 
   NS_IMETHOD CreateSubfolder(const nsAString& folderName,
@@ -237,8 +233,8 @@ class nsImapMailFolder : public nsMsgDBFolder,
 
   NS_IMETHOD Compact(nsIUrlListener* aListener,
                      nsIMsgWindow* aMsgWindow) override;
-  NS_IMETHOD CompactAll(nsIUrlListener* aListener, nsIMsgWindow* aMsgWindow,
-                        bool aCompactOfflineAlso) override;
+  NS_IMETHOD CompactAll(nsIUrlListener* aListener,
+                        nsIMsgWindow* aMsgWindow) override;
   NS_IMETHOD EmptyTrash(nsIMsgWindow* msgWindow,
                         nsIUrlListener* aListener) override;
   NS_IMETHOD CopyDataToOutputStreamForAppend(
@@ -269,16 +265,18 @@ class nsImapMailFolder : public nsMsgDBFolder,
 
   NS_IMETHOD AddMessageDispositionState(
       nsIMsgDBHdr* aMessage, nsMsgDispositionState aDispositionFlag) override;
-  NS_IMETHOD MarkMessagesRead(nsIArray* messages, bool markRead) override;
+  NS_IMETHOD MarkMessagesRead(const nsTArray<RefPtr<nsIMsgDBHdr>>& messages,
+                              bool markRead) override;
   NS_IMETHOD MarkAllMessagesRead(nsIMsgWindow* aMsgWindow) override;
-  NS_IMETHOD MarkMessagesFlagged(nsIArray* messages, bool markFlagged) override;
+  NS_IMETHOD MarkMessagesFlagged(const nsTArray<RefPtr<nsIMsgDBHdr>>& messages,
+                                 bool markFlagged) override;
   NS_IMETHOD MarkThreadRead(nsIMsgThread* thread) override;
-  NS_IMETHOD SetLabelForMessages(nsIArray* aMessages,
+  NS_IMETHOD SetLabelForMessages(const nsTArray<RefPtr<nsIMsgDBHdr>>& aMessages,
                                  nsMsgLabelValue aLabel) override;
-  NS_IMETHOD SetJunkScoreForMessages(nsIArray *aMessages,
-                                     const nsACString& aJunkScore) override;
-  NS_IMETHOD DeleteSubFolders(nsIArray* folders,
-                              nsIMsgWindow* msgWindow) override;
+  NS_IMETHOD SetJunkScoreForMessages(
+      const nsTArray<RefPtr<nsIMsgDBHdr>>& aMessages,
+      const nsACString& aJunkScore) override;
+  NS_IMETHOD DeleteSelf(nsIMsgWindow* msgWindow) override;
   NS_IMETHOD ReadFromFolderCacheElem(
       nsIMsgFolderCacheElement* element) override;
   NS_IMETHOD WriteToFolderCacheElem(nsIMsgFolderCacheElement* element) override;
@@ -286,10 +284,11 @@ class nsImapMailFolder : public nsMsgDBFolder,
   NS_IMETHOD GetDBFolderInfoAndDB(nsIDBFolderInfo** folderInfo,
                                   nsIMsgDatabase** db) override;
   MOZ_CAN_RUN_SCRIPT_BOUNDARY NS_IMETHOD
-  DeleteMessages(nsIArray* messages, nsIMsgWindow* msgWindow,
-                 bool deleteStorage, bool isMove,
+  DeleteMessages(nsTArray<RefPtr<nsIMsgDBHdr>> const& msgHeaders,
+                 nsIMsgWindow* msgWindow, bool deleteStorage, bool isMove,
                  nsIMsgCopyServiceListener* listener, bool allowUndo) override;
-  NS_IMETHOD CopyMessages(nsIMsgFolder* srcFolder, nsIArray* messages,
+  NS_IMETHOD CopyMessages(nsIMsgFolder* srcFolder,
+                          nsTArray<RefPtr<nsIMsgDBHdr>> const& messages,
                           bool isMove, nsIMsgWindow* msgWindow,
                           nsIMsgCopyServiceListener* listener, bool isFolder,
                           bool allowUndo) override;
@@ -309,30 +308,31 @@ class nsImapMailFolder : public nsMsgDBFolder,
 
   NS_IMETHOD Shutdown(bool shutdownChildren) override;
 
-  NS_IMETHOD DownloadMessagesForOffline(nsIArray* messages,
-                                        nsIMsgWindow* msgWindow) override;
+  NS_IMETHOD DownloadMessagesForOffline(
+      nsTArray<RefPtr<nsIMsgDBHdr>> const& messages,
+      nsIMsgWindow* msgWindow) override;
 
   NS_IMETHOD DownloadAllForOffline(nsIUrlListener* listener,
                                    nsIMsgWindow* msgWindow) override;
   NS_IMETHOD GetCanFileMessages(bool* aCanFileMessages) override;
   NS_IMETHOD GetCanDeleteMessages(bool* aCanDeleteMessages) override;
   NS_IMETHOD FetchMsgPreviewText(nsTArray<nsMsgKey> const& aKeysToFetch,
-                                 bool aLocalOnly, nsIUrlListener* aUrlListener,
+                                 nsIUrlListener* aUrlListener,
                                  bool* aAsyncResults) override;
 
-  NS_IMETHOD AddKeywordsToMessages(nsIArray* aMessages,
-                                   const nsACString& aKeywords) override;
-  NS_IMETHOD RemoveKeywordsFromMessages(nsIArray* aMessages,
-                                        const nsACString& aKeywords) override;
+  NS_IMETHOD AddKeywordsToMessages(
+      const nsTArray<RefPtr<nsIMsgDBHdr>>& aMessages,
+      const nsACString& aKeywords) override;
+  NS_IMETHOD RemoveKeywordsFromMessages(
+      const nsTArray<RefPtr<nsIMsgDBHdr>>& aMessages,
+      const nsACString& aKeywords) override;
 
   NS_IMETHOD NotifyCompactCompleted() override;
 
   // overrides nsMsgDBFolder::HasMsgOffline()
   NS_IMETHOD HasMsgOffline(nsMsgKey msgKey, bool* _retval) override;
-  // overrides nsMsgDBFolder::GetOfflineFileStream()
-  NS_IMETHOD GetOfflineFileStream(nsMsgKey msgKey, int64_t* offset,
-                                  uint32_t* size,
-                                  nsIInputStream** aFileStream) override;
+  NS_IMETHOD GetLocalMsgStream(nsIMsgDBHdr* hdr,
+                               nsIInputStream** stream) override;
 
   NS_DECL_NSIMSGIMAPMAILFOLDER
   NS_DECL_NSIIMAPMAILFOLDERSINK
@@ -351,17 +351,6 @@ class nsImapMailFolder : public nsMsgDBFolder,
   NS_IMETHOD SetFilterList(nsIMsgFilterList* aMsgFilterList) override;
   NS_IMETHOD GetCustomIdentity(nsIMsgIdentity** aIdentity) override;
 
-  /**
-   * This method is used to locate a folder where a msg could be present, not
-   * just the folder where the message first arrives, this method searches for
-   * the existence of msg in all the folders/labels that we retrieve from
-   * X-GM-LABELS also. overrides nsMsgDBFolder::GetOfflineMsgFolder()
-   *  @param msgKey key  of the msg for which we are trying to get the folder;
-   *  @param aMsgFolder  required folder;
-   */
-  NS_IMETHOD GetOfflineMsgFolder(nsMsgKey msgKey,
-                                 nsIMsgFolder** aMsgFolder) override;
-
   NS_IMETHOD GetIncomingServerType(nsACString& serverType) override;
 
   nsresult AddSubfolderWithPath(nsAString& name, nsIFile* dbPath,
@@ -377,8 +366,9 @@ class nsImapMailFolder : public nsMsgDBFolder,
 
   static nsresult AllocateUidStringFromKeys(const nsTArray<nsMsgKey>& keys,
                                             nsCString& msgIds);
-  static nsresult BuildIdsAndKeyArray(nsIArray* messages, nsCString& msgIds,
-                                      nsTArray<nsMsgKey>& keyArray);
+  static nsresult BuildIdsAndKeyArray(
+      const nsTArray<RefPtr<nsIMsgDBHdr>>& messages, nsCString& msgIds,
+      nsTArray<nsMsgKey>& keyArray);
 
   // these might end up as an nsIImapMailFolder attribute.
   nsresult SetSupportedUserFlags(uint32_t userFlags);
@@ -391,7 +381,7 @@ class nsImapMailFolder : public nsMsgDBFolder,
   virtual ~nsImapMailFolder();
   // Helper methods
 
-  virtual nsresult CreateChildFromURI(const nsCString& uri,
+  virtual nsresult CreateChildFromURI(const nsACString& uri,
                                       nsIMsgFolder** folder) override;
   void FindKeysToAdd(const nsTArray<nsMsgKey>& existingKeys,
                      nsTArray<nsMsgKey>& keysToFetch, uint32_t& numNewUnread,
@@ -449,21 +439,23 @@ class nsImapMailFolder : public nsMsgDBFolder,
   nsresult GetDatabase() override;
 
   nsresult GetFolderOwnerUserName(nsACString& userName);
-  nsIMAPNamespace* GetNamespaceForFolder();
-  void SetNamespaceForFolder(nsIMAPNamespace* ns);
+  nsImapNamespace* GetNamespaceForFolder();
+  void SetNamespaceForFolder(nsImapNamespace* ns);
 
   nsMsgIMAPFolderACL* GetFolderACL();
   nsresult CreateACLRightsStringForFolder(nsAString& rightsString);
   nsresult GetBodysToDownload(nsTArray<nsMsgKey>* keysOfMessagesToDownload);
   // Uber message copy service
-  nsresult CopyMessagesWithStream(nsIMsgFolder* srcFolder, nsIArray* messages,
+  nsresult CopyMessagesWithStream(nsIMsgFolder* srcFolder,
+                                  nsTArray<RefPtr<nsIMsgDBHdr>> const& messages,
                                   bool isMove, bool isCrossServerOp,
                                   nsIMsgWindow* msgWindow,
                                   nsIMsgCopyServiceListener* listener,
                                   bool allowUndo);
   nsresult CopyStreamMessage(nsIMsgDBHdr* message, nsIMsgFolder* dstFolder,
                              nsIMsgWindow* msgWindow, bool isMove);
-  nsresult InitCopyState(nsISupports* srcSupport, nsIArray* messages,
+  nsresult InitCopyState(nsISupports* srcSupport,
+                         nsTArray<RefPtr<nsIMsgDBHdr>> const& messages,
                          bool isMove, bool selectedState, bool acrossServers,
                          uint32_t newMsgFlags, const nsACString& newMsgKeywords,
                          nsIMsgCopyServiceListener* listener,
@@ -479,8 +471,9 @@ class nsImapMailFolder : public nsMsgDBFolder,
                          nsIMsgOfflineImapOperation** originalOp,
                          nsIMsgDatabase** originalDB);
   MOZ_CAN_RUN_SCRIPT_BOUNDARY nsresult CopyMessagesOffline(
-      nsIMsgFolder* srcFolder, nsIArray* messages, bool isMove,
-      nsIMsgWindow* msgWindow, nsIMsgCopyServiceListener* listener);
+      nsIMsgFolder* srcFolder, nsTArray<RefPtr<nsIMsgDBHdr>> const& messages,
+      bool isMove, nsIMsgWindow* msgWindow,
+      nsIMsgCopyServiceListener* listener);
   void SetPendingAttributes(const nsTArray<RefPtr<nsIMsgDBHdr>>& messages,
                             bool aIsMove, bool aSetOffline);
 
@@ -494,11 +487,13 @@ class nsImapMailFolder : public nsMsgDBFolder,
   // Pseudo-Offline operation playback timer
   static void PlaybackTimerCallback(nsITimer* aTimer, void* aClosure);
 
-  nsresult CreatePlaybackTimer();
-
   // Allocate and initialize associated auto-sync state object.
   void InitAutoSyncState();
 
+  virtual nsresult GetOfflineFileStream(nsMsgKey msgKey, uint64_t* offset,
+                                        uint32_t* size,
+                                        nsIInputStream** aFileStream) override;
+
   bool m_initialized;
   bool m_haveDiscoveredAllFolders;
   nsCOMPtr<nsIMsgParseMailMsgState> m_msgParser;
@@ -509,7 +504,7 @@ class nsImapMailFolder : public nsMsgDBFolder,
   bool m_msgMovedByFilter;
   RefPtr<nsImapMoveCoalescer>
       m_moveCoalescer;  // strictly owned by the nsImapMailFolder
-  nsCOMPtr<nsIMutableArray> m_junkMessagesToMarkAsRead;
+  nsTArray<RefPtr<nsIMsgDBHdr>> m_junkMessagesToMarkAsRead;
   /// list of keys to be moved to the junk folder
   nsTArray<nsMsgKey> mSpamKeysToMove;
   /// the junk destination folder
@@ -540,7 +535,7 @@ class nsImapMailFolder : public nsMsgDBFolder,
   // "Other Users' Mailboxes"
 
   nsCString m_adminUrl;  // url to run to set admin privileges for this folder
-  nsIMAPNamespace* m_namespace;
+  nsImapNamespace* m_namespace;
   bool m_verifiedAsOnlineFolder;
   bool m_explicitlyVerify;  // whether or not we need to explicitly verify this
                             // through LIST
@@ -580,7 +575,7 @@ class nsImapMailFolder : public nsMsgDBFolder,
   nsTArray<RefPtr<nsImapMoveCopyMsgTxn>> m_pendingOfflineMoves;
   // hash table of mapping between messageids and message keys
   // for pseudo hdrs.
-  nsDataHashtable<nsCStringHashKey, nsMsgKey> m_pseudoHdrs;
+  nsTHashMap<nsCStringHashKey, nsMsgKey> m_pseudoHdrs;
 
   nsTArray<nsMsgKey> m_keysToFetch;
   uint32_t m_totalKeysToFetch;
@@ -596,5 +591,14 @@ class nsImapMailFolder : public nsMsgDBFolder,
   void DeleteStoreMessages(const nsTArray<nsMsgKey>& aMessages);
   static void DeleteStoreMessages(const nsTArray<nsMsgKey>& aMessages,
                                   nsIMsgFolder* aFolder);
+  /**
+   * This method is used to locate a folder where a msg could be present, not
+   * just the folder where the message first arrives, this method searches for
+   * the existence of msg in all the folders/labels that we retrieve from
+   * X-GM-LABELS also.
+   *  @param msgKey key  of the msg for which we are trying to get the folder;
+   *  @param aMsgFolder  required folder;
+   */
+  nsresult GetOfflineMsgFolder(nsMsgKey msgKey, nsIMsgFolder** aMsgFolder);
 };
 #endif
