@@ -7,7 +7,7 @@
 #include "nsMsgImapCID.h"
 
 #include "nsImapUrl.h"
-#include "nsIIMAPHostSessionList.h"
+#include "nsIImapHostSessionList.h"
 #include "nsThreadUtils.h"
 #include "nsString.h"
 #include "prmem.h"
@@ -21,12 +21,13 @@
 #include "nsIImapMailFolderSink.h"
 #include "nsIImapMessageSink.h"
 #include "nsIImapServerSink.h"
-#include "nsIMAPNamespace.h"
+#include "nsImapNamespace.h"
 #include "nsICacheEntry.h"
 #include "nsIMsgFolder.h"
 #include "nsMsgUtils.h"
 #include "nsIMsgHdr.h"
 #include "nsServiceManagerUtils.h"
+#include "mozilla/Logging.h"
 
 using namespace mozilla;
 extern LazyLogModule IMAPCache;  // defined in nsImapProtocol.cpp
@@ -39,6 +40,7 @@ nsImapUrl::nsImapUrl() : mLock("nsImapUrl.mLock") {
   m_destinationCanonicalFolderPathSubString = nullptr;
   m_listOfMessageIds = nullptr;
   m_tokenPlaceHolder = nullptr;
+  m_urlidSubString = nullptr;
   m_searchCriteriaString = nullptr;
   m_idsAreUids = false;
   m_mimePartSelectorDetected = false;
@@ -53,11 +55,14 @@ nsImapUrl::nsImapUrl() : mLock("nsImapUrl.mLock") {
   m_externalLinkUrl = true;  // we'll start this at true, and set it false in
                              // nsImapService::CreateStartOfImapUrl
   m_contentModified = IMAP_CONTENT_NOT_MODIFIED;
+  m_numBytesToFetch = 0;
   m_validUrl = true;  // assume the best.
+  m_runningUrl = false;
   m_flags = 0;
   m_extraStatus = ImapStatusNone;
   m_onlineSubDirSeparator = '/';
   m_imapAction = 0;
+  mAutodetectCharset = false;
 
   // ** jt - the following are not ref counted
   m_copyState = nullptr;
@@ -608,7 +613,7 @@ NS_IMETHODIMP nsImapUrl::AddOnlineDirectoryIfNecessary(
   nsAutoCString onlineDir;
   LossyCopyUTF16toASCII(onlineDirString, onlineDir);
 
-  nsIMAPNamespace* ns = nullptr;
+  nsImapNamespace* ns = nullptr;
   rv = hostSessionList->GetNamespaceForMailboxForHost(m_serverKey.get(),
                                                       onlineMailboxName, ns);
   if (!ns)
@@ -739,9 +744,10 @@ NS_IMETHODIMP nsImapUrl::AllocateServerPath(const char* canonicalPath,
   return NS_OK;
 }
 
-/* static */ nsresult nsImapUrl::UnescapeSlashes(char* sourcePath) {
-  char* src = sourcePath;
-  char* dst = sourcePath;
+static void unescapeSlashes(char* path, size_t* newLength) {
+  char* src = path;
+  char* start = src;
+  char* dst = path;
 
   while (*src) {
     if (*src == '^') {
@@ -755,7 +761,20 @@ NS_IMETHODIMP nsImapUrl::AllocateServerPath(const char* canonicalPath,
       *dst++ = *src++;
   }
 
-  *dst = 0;
+  *newLength = dst - start;
+}
+
+/* static */ nsresult nsImapUrl::UnescapeSlashes(char* path) {
+  size_t newLength;
+  unescapeSlashes(path, &newLength);
+  path[newLength] = 0;
+  return NS_OK;
+}
+
+/* static */ nsresult nsImapUrl::UnescapeSlashes(nsACString& path) {
+  size_t newLength;
+  unescapeSlashes(path.BeginWriting(), &newLength);
+  path.SetLength(newLength);
   return NS_OK;
 }
 
@@ -781,39 +800,35 @@ NS_IMETHODIMP nsImapUrl::AllocateServerPath(const char* canonicalPath,
 
 // Converts the real online name on the server to canonical format:
 // result is hierarchy is indicated by '/' and all real slashes ('/') are
-// escaped. The caller has already converted m-utf-7 to 8 bit ascii, which is a
-// problem. this method is only called from the imap thread
+// escaped. This method is only called from the IMAP thread.
 NS_IMETHODIMP nsImapUrl::AllocateCanonicalPath(const char* serverPath,
                                                char onlineDelimiter,
                                                char** allocatedPath) {
-  nsresult rv = NS_ERROR_NULL_POINTER;
-  char delimiterToUse = onlineDelimiter;
-  char* serverKey = nullptr;
-  nsString aString;
-  char* currentPath = (char*)serverPath;
-  nsAutoCString onlineDir;
-  nsCOMPtr<nsIMsgIncomingServer> server;
+  NS_ENSURE_ARG_POINTER(serverPath);
 
+  char delimiterToUse = onlineDelimiter;
+  *allocatedPath = nullptr;
+
+  char* currentPath = (char*)serverPath;
+
+  nsresult rv;
   nsCOMPtr<nsIImapHostSessionList> hostSessionList =
       do_GetService(kCImapHostSessionListCID, &rv);
-
-  *allocatedPath = nullptr;
+  NS_ENSURE_SUCCESS(rv, rv);
 
   if (onlineDelimiter == kOnlineHierarchySeparatorUnknown ||
       onlineDelimiter == 0)
     GetOnlineSubDirSeparator(&delimiterToUse);
 
-  NS_ASSERTION(serverPath, "Oops... null serverPath");
-
-  if (!serverPath || NS_FAILED(rv)) goto done;
-
-  hostSessionList->GetOnlineDirForHost(m_serverKey.get(), aString);
+  nsString dir;
+  hostSessionList->GetOnlineDirForHost(m_serverKey.get(), dir);
   // First we have to check to see if we should strip off an online server
   // subdirectory
   // If this host has an online server directory configured
-  LossyCopyUTF16toASCII(aString, onlineDir);
+  nsAutoCString onlineDir;
+  LossyCopyUTF16toASCII(dir, onlineDir);
 
-  if (currentPath && !onlineDir.IsEmpty()) {
+  if (!onlineDir.IsEmpty()) {
     // By definition, the online dir must be at the root.
     if (delimiterToUse && delimiterToUse != kOnlineHierarchySeparatorUnknown) {
       // try to change the canonical online dir name to real dir name first
@@ -836,13 +851,7 @@ NS_IMETHODIMP nsImapUrl::AllocateCanonicalPath(const char* serverPath,
     }
   }
 
-  if (!currentPath) goto done;
-
-  rv = ConvertToCanonicalFormat(currentPath, delimiterToUse, allocatedPath);
-
-done:
-  PR_Free(serverKey);
-  return rv;
+  return ConvertToCanonicalFormat(currentPath, delimiterToUse, allocatedPath);
 }
 
 // this method is only called from the imap thread
@@ -1011,7 +1020,7 @@ nsresult nsImapUrl::Clone(nsIURI** _retval) {
   // also clone the mURI member, because GetUri below won't work if
   // mURI isn't set due to escaping issues.
   nsCOMPtr<nsIMsgMessageUrl> clonedUrl = do_QueryInterface(*_retval);
-  if (clonedUrl) clonedUrl->SetUri(mURI.get());
+  if (clonedUrl) clonedUrl->SetUri(mURI);
   return rv;
 }
 
@@ -1033,17 +1042,16 @@ NS_IMETHODIMP nsImapUrl::GetNormalizedSpec(nsACString& aPrincipalSpec) {
   return NS_OK;
 }
 
-NS_IMETHODIMP nsImapUrl::SetUri(const char* aURI) {
+NS_IMETHODIMP nsImapUrl::SetUri(const nsACString& aURI) {
   mURI = aURI;
   return NS_OK;
 }
 
-NS_IMETHODIMP nsImapUrl::GetUri(char** aURI) {
+NS_IMETHODIMP nsImapUrl::GetUri(nsACString& aURI) {
   nsresult rv = NS_OK;
   if (!mURI.IsEmpty())
-    *aURI = ToNewCString(mURI);
+    aURI = mURI;
   else {
-    *aURI = nullptr;
     uint32_t key =
         m_listOfMessageIds ? strtoul(m_listOfMessageIds, nullptr, 10) : 0;
     nsCString canonicalPath;
@@ -1061,9 +1069,7 @@ NS_IMETHODIMP nsImapUrl::GetUri(char** aURI) {
 
     nsCString baseMessageURI;
     nsCreateImapBaseMessageURI(fullFolderPath, baseMessageURI);
-    nsAutoCString uriStr;
-    rv = nsBuildImapMessageURI(baseMessageURI.get(), key, uriStr);
-    *aURI = ToNewCString(uriStr);
+    rv = nsBuildImapMessageURI(baseMessageURI.get(), key, aURI);
   }
   return rv;
 }
@@ -1123,10 +1129,12 @@ NS_IMETHODIMP nsImapUrl::IsUrlType(uint32_t type, bool* isType) {
 }
 
 NS_IMETHODIMP
-nsImapUrl::GetOriginalSpec(char** aSpec) { return NS_ERROR_NOT_IMPLEMENTED; }
+nsImapUrl::GetOriginalSpec(nsACString& aSpec) {
+  return NS_ERROR_NOT_IMPLEMENTED;
+}
 
 NS_IMETHODIMP
-nsImapUrl::SetOriginalSpec(const char* aSpec) {
+nsImapUrl::SetOriginalSpec(const nsACString& aSpec) {
   return NS_ERROR_NOT_IMPLEMENTED;
 }
 
@@ -1285,11 +1293,11 @@ nsresult nsImapUrl::GetMsgFolder(nsIMsgFolder** msgFolder) {
   // the folder for it's charset....
 
   nsCString uri;
-  GetUri(getter_Copies(uri));
+  GetUri(uri);
   NS_ENSURE_TRUE(!uri.IsEmpty(), NS_ERROR_FAILURE);
 
   nsCOMPtr<nsIMsgDBHdr> msg;
-  GetMsgDBHdrFromURI(uri.get(), getter_AddRefs(msg));
+  GetMsgDBHdrFromURI(uri, getter_AddRefs(msg));
   NS_ENSURE_TRUE(msg, NS_ERROR_FAILURE);
   nsresult rv = msg->GetFolder(msgFolder);
   NS_ENSURE_SUCCESS(rv, rv);
@@ -1298,35 +1306,13 @@ nsresult nsImapUrl::GetMsgFolder(nsIMsgFolder** msgFolder) {
   return NS_OK;
 }
 
-NS_IMETHODIMP nsImapUrl::GetFolderCharset(char** aCharacterSet) {
-  nsCOMPtr<nsIMsgFolder> folder;
-  nsresult rv = GetMsgFolder(getter_AddRefs(folder));
-  NS_ENSURE_SUCCESS(rv, rv);
-  nsCString tmpStr;
-  folder->GetCharset(tmpStr);
-  *aCharacterSet = ToNewCString(tmpStr);
+NS_IMETHODIMP nsImapUrl::GetAutodetectCharset(bool* aAutodetectCharset) {
+  *aAutodetectCharset = mAutodetectCharset;
   return NS_OK;
 }
 
-NS_IMETHODIMP nsImapUrl::GetFolderCharsetOverride(bool* aCharacterSetOverride) {
-  nsCOMPtr<nsIMsgFolder> folder;
-  nsresult rv = GetMsgFolder(getter_AddRefs(folder));
-  NS_ENSURE_SUCCESS(rv, rv);
-  NS_ENSURE_TRUE(folder, NS_ERROR_FAILURE);
-  folder->GetCharsetOverride(aCharacterSetOverride);
-  return NS_OK;
-}
-
-NS_IMETHODIMP nsImapUrl::GetCharsetOverRide(char** aCharacterSet) {
-  if (!mCharsetOverride.IsEmpty())
-    *aCharacterSet = ToNewCString(mCharsetOverride);
-  else
-    *aCharacterSet = nullptr;
-  return NS_OK;
-}
-
-NS_IMETHODIMP nsImapUrl::SetCharsetOverRide(const char* aCharacterSet) {
-  mCharsetOverride = aCharacterSet;
+NS_IMETHODIMP nsImapUrl::SetAutodetectCharset(bool aAutodetectCharset) {
+  mAutodetectCharset = aAutodetectCharset;
   return NS_OK;
 }
 
@@ -1356,9 +1342,9 @@ NS_IMETHODIMP nsImapUrl::SetStoreOfflineOnFallback(
 
 NS_IMETHODIMP nsImapUrl::GetMessageHeader(nsIMsgDBHdr** aMsgHdr) {
   nsCString uri;
-  nsresult rv = GetUri(getter_Copies(uri));
+  nsresult rv = GetUri(uri);
   NS_ENSURE_SUCCESS(rv, rv);
-  return GetMsgDBHdrFromURI(uri.get(), aMsgHdr);
+  return GetMsgDBHdrFromURI(uri, aMsgHdr);
 }
 
 NS_IMETHODIMP nsImapUrl::SetMessageHeader(nsIMsgDBHdr* aMsgHdr) {
