@@ -18,15 +18,14 @@
 #include "nsMsgAttachmentData.h"
 #include "nsMsgBaseCID.h"
 #include "nsMsgCompCID.h"
-#include "nsIArray.h"
 #include "nsIMsgCompFields.h"
 #include "nsIMsgAccountManager.h"
 #include "nsIMsgSend.h"
-#include "nsIMutableArray.h"
 #include "nsImportEmbeddedImageData.h"
 #include "nsNetCID.h"
 #include "nsCRT.h"
 #include "nsOutlookCompose.h"
+#include "nsTArray.h"
 
 #include "ImportDebug.h"
 
@@ -156,7 +155,10 @@ class OutlookSendListener : public nsIMsgSendListener {
   }
 
   /* void OnGetDraftFolderURI (); */
-  NS_IMETHOD OnGetDraftFolderURI(const char* aFolderURI) { return NS_OK; }
+  NS_IMETHOD OnGetDraftFolderURI(const char* aMsgID,
+                                 const nsACString& aFolderURI) {
+    return NS_OK;
+  }
 
   static nsresult CreateSendListener(nsIMsgSendListener** ppListener);
   void Reset() {
@@ -212,7 +214,7 @@ nsresult nsOutlookCompose::CreateIdentity(void) {
   name.AssignLiteral("Import Identity");
   if (m_pIdentity) {
     m_pIdentity->SetFullName(name);
-    m_pIdentity->SetEmail(NS_LITERAL_CSTRING("import@service.invalid"));
+    m_pIdentity->SetEmail("import@service.invalid"_ns);
   }
   return rv;
 }
@@ -260,7 +262,6 @@ nsresult nsOutlookCompose::ComposeTheMessage(nsMsgDeliverMode mode,
   headers->UnfoldValue(CMapiMessageHeaders::hdrSubject, unival,
                        msg.GetBodyCharset());
   m_pMsgFields->SetSubject(unival);
-  m_pMsgFields->SetCharacterSet(msg.GetBodyCharset());
   headers->UnfoldValue(CMapiMessageHeaders::hdrCc, unival,
                        msg.GetBodyCharset());
   m_pMsgFields->SetCc(unival);
@@ -274,13 +275,13 @@ nsresult nsOutlookCompose::ComposeTheMessage(nsMsgDeliverMode mode,
   // These will replace the originals on import. All the other headers
   // will be copied to the destination unaltered in CopyComposedMessage().
 
-  nsCOMPtr<nsIArray> pAttach;
-  msg.GetAttachments(getter_AddRefs(pAttach));
+  nsTArray<RefPtr<nsIMsgAttachedFile>> attachments;
+  msg.GetAttachments(attachments);
 
   nsString bodyW;
   bodyW = msg.GetBody();
 
-  nsCOMPtr<nsIMutableArray> embeddedObjects;
+  nsTArray<RefPtr<nsIMsgEmbeddedImageData>> embeddedObjects;
 
   if (msg.BodyIsHtml()) {
     for (unsigned int i = 0; i < msg.EmbeddedAttachmentsCount(); i++) {
@@ -288,14 +289,10 @@ nsresult nsOutlookCompose::ComposeTheMessage(nsMsgDeliverMode mode,
       const char* cid;
       const char* name;
       if (msg.GetEmbeddedAttachmentInfo(i, &uri, &cid, &name)) {
-        if (!embeddedObjects) {
-          embeddedObjects = do_CreateInstance(NS_ARRAY_CONTRACTID, &rv);
-          NS_ENSURE_SUCCESS(rv, rv);
-        }
         nsCOMPtr<nsIMsgEmbeddedImageData> imageData =
             new nsImportEmbeddedImageData(uri, nsDependentCString(cid),
                                           nsDependentCString(name));
-        embeddedObjects->AppendElement(imageData);
+        embeddedObjects.AppendElement(imageData);
       }
     }
   }
@@ -317,19 +314,19 @@ nsresult nsOutlookCompose::ComposeTheMessage(nsMsgDeliverMode mode,
       msg.BodyIsHtml() ? "text/html" : "text/plain",
       bodyA,  // body pointer
       mode == nsIMsgSend::nsMsgSaveAsDraft,
-      pAttach,  // local attachments
+      attachments,  // local attachments
       embeddedObjects,
       m_pListener);  // listener
 
   OutlookSendListener* pListen =
       static_cast<OutlookSendListener*>(m_pListener.get());
   if (NS_FAILED(rv)) {
-    IMPORT_LOG1("*** Error, CreateAndSendMessage FAILED: 0x%lx\n", rv);
+    IMPORT_LOG1("*** Error, CreateAndSendMessage FAILED: 0x%x\n", rv);
   } else {
     // Wait for the listener to get done.
     nsCOMPtr<nsIThread> thread(do_GetCurrentThread());
     while (!pListen->m_done) {
-      NS_ProcessPendingEvents(thread);
+      NS_ProcessNextEvent(thread, true);
     }
   }
 
