@@ -49,7 +49,11 @@ GenericImportHelper.prototype = {
     }
 
     Assert.ok(this.mInterface.WantsProgress());
-    Assert.ok(this.mInterface.BeginImport(null, null));
+    const error = Cc["@mozilla.org/supports-string;1"].createInstance(
+      Ci.nsISupportsString
+    );
+    Assert.ok(this.mInterface.BeginImport(null, error));
+    Assert.equal(error.data, "");
     do_test_pending();
     this.checkProgress();
   },
@@ -143,8 +147,7 @@ function AbImportHelper(aFile, aModuleSearchString, aAbName, aJsonName) {
 
   this.mAbName = aAbName;
   /* Attribute notes:  The attributes listed in the declaration below are
-   * supported by all three text export/import types. PreferMailFormat is only
-   * supported by LDIF.
+   * supported by all three text export/import types.
    * The following are not supported: anniversaryYear, anniversaryMonth,
    * anniversaryDay, popularityIndex, isMailList, mailListURI, lastModifiedDate.
    */
@@ -186,14 +189,12 @@ function AbImportHelper(aFile, aModuleSearchString, aAbName, aJsonName) {
     "Custom4",
     "Notes",
     "_AimScreenName",
+    "_vCard",
   ];
 
   // get the extra attributes supported for the given type of import
   if (this.mFile.leafName.toLowerCase().endsWith(".ldif")) {
-    // LDIF: add PreferMailFormat
-    this.mSupportedAttributes = supportedAttributes.concat([
-      "PreferMailFormat",
-    ]);
+    this.mSupportedAttributes = supportedAttributes;
   } else if (this.mFile.leafName.toLowerCase().endsWith(".csv")) {
     this.mSupportedAttributes = supportedAttributes;
     this.setFieldMap(this.getDefaultFieldMap(true));
@@ -224,6 +225,10 @@ AbImportHelper.prototype = {
     var fieldMap = importService.CreateNewFieldMap();
 
     fieldMap.DefaultFieldMap(fieldMap.numMozFields);
+    this.mInterface
+      .GetData("addressInterface")
+      .QueryInterface(Ci.nsIImportAddressBooks)
+      .InitFieldMap(fieldMap);
     fieldMap.skipFirstRecord = aSkipFirstRecord;
 
     return fieldMap;
@@ -278,10 +283,7 @@ AbImportHelper.prototype = {
       // make sure an address book was created
       var newAb = this.getAbByName(this.mAbName);
       Assert.ok(newAb !== null);
-      Assert.ok(
-        newAb.QueryInterface(Ci.nsIAbDirectory) &&
-          newAb.childCards.QueryInterface(Ci.nsISimpleEnumerator)
-      );
+      Assert.ok(newAb.QueryInterface(Ci.nsIAbDirectory));
       // get the imported card(s) and check each one
       var count = 0;
       for (let importedCard of newAb.childCards) {
@@ -323,9 +325,23 @@ AbImportHelper.prototype = {
    * @param aCard     The imported card to compare with.
    */
   compareCards(aJsonCard, aCard) {
-    for (var i in aJsonCard) {
-      if (this.mSupportedAttributes.includes(i)) {
-        Assert.equal(aJsonCard[i], aCard.getProperty(i, "BAD"));
+    for (let [key, value] of Object.entries(aJsonCard)) {
+      if (!this.mSupportedAttributes.includes(key)) {
+        continue;
+      }
+      if (key == "_vCard") {
+        equal(
+          aCard
+            .getProperty(key, "")
+            .replace(
+              /UID:[a-f0-9-]{36}/i,
+              "UID:xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx"
+            ),
+          `BEGIN:VCARD\r\n${value.join("\r\n")}\r\nEND:VCARD\r\n`,
+          "_vCard should be correct"
+        );
+      } else {
+        equal(aCard.getProperty(key, ""), value, `${key} should be correct`);
       }
     }
   },
@@ -403,24 +419,17 @@ MailImportHelper.prototype = {
   interfaceType: Ci.nsIImportGeneric,
   _checkEqualFolder(expectedFolder, actualFolder) {
     Assert.equal(expectedFolder.leafName, actualFolder.name);
-    let expectedSubFolderCount = 0;
 
     let expectedSubFolders = [];
     for (let entry of expectedFolder.directoryEntries) {
       if (entry.isDirectory()) {
-        expectedSubFolderCount++;
         expectedSubFolders.push(entry);
       }
     }
-    Assert.equal(expectedSubFolderCount, actualFolder.numSubFolders);
-
-    let actualEnumerator = actualFolder.subFolders;
-    for (let i = 0; i < expectedSubFolderCount; i++) {
-      let expectedSubFolder = expectedSubFolders[i];
-      let actualSubFolder = actualEnumerator
-        .getNext()
-        .QueryInterface(Ci.nsIMsgFolder);
-      this._checkEqualFolder(expectedSubFolder, actualSubFolder);
+    let actualSubFolders = actualFolder.subFolders;
+    Assert.equal(expectedSubFolders.length, actualSubFolders.length);
+    for (let i = 0; i < expectedSubFolders.length; i++) {
+      this._checkEqualFolder(expectedSubFolders[i], actualSubFolders[i]);
     }
   },
 
@@ -581,7 +590,7 @@ SettingsImportHelper.prototype = {
  * @param aModuleSearchString
  *                   The string to search the module names for, such as
  *                   "Outlook Express", etc.
- * @param aExpected  The number of filters that should exists after import.
+ * @param aExpected  The number of filters that should exist after import.
  *
  * @constructor
  * @class
