@@ -39,11 +39,11 @@ static nsresult nsGetMailboxServer(const char* uriStr,
 
   // find all local mail "no servers" matching the given hostname
   nsCOMPtr<nsIMsgIncomingServer> none_server;
-  rv = NS_MutateURI(url).SetScheme(NS_LITERAL_CSTRING("none")).Finalize(url);
+  rv = NS_MutateURI(url).SetScheme("none"_ns).Finalize(url);
   NS_ENSURE_SUCCESS(rv, rv);
   // No unescaping of username or hostname done here.
   // The unescaping is done inside of FindServerByURI
-  rv = accountManager->FindServerByURI(url, false, getter_AddRefs(none_server));
+  rv = accountManager->FindServerByURI(url, getter_AddRefs(none_server));
   if (NS_SUCCEEDED(rv)) {
     none_server.forget(aResult);
     return rv;
@@ -51,41 +51,27 @@ static nsresult nsGetMailboxServer(const char* uriStr,
 
   // if that fails, look for the rss hosts matching the given hostname
   nsCOMPtr<nsIMsgIncomingServer> rss_server;
-  rv = NS_MutateURI(url).SetScheme(NS_LITERAL_CSTRING("rss")).Finalize(url);
+  rv = NS_MutateURI(url).SetScheme("rss"_ns).Finalize(url);
   NS_ENSURE_SUCCESS(rv, rv);
-  rv = accountManager->FindServerByURI(url, false, getter_AddRefs(rss_server));
+  rv = accountManager->FindServerByURI(url, getter_AddRefs(rss_server));
   if (NS_SUCCEEDED(rv)) {
     rss_server.forget(aResult);
     return rv;
   }
-#ifdef HAVE_MOVEMAIL
-  // find all movemail "servers" matching the given hostname
-  nsCOMPtr<nsIMsgIncomingServer> movemail_server;
-  rv =
-      NS_MutateURI(url).SetScheme(NS_LITERAL_CSTRING("movemail")).Finalize(url);
-  NS_ENSURE_SUCCESS(rv, rv);
-  rv = accountManager->FindServerByURI(url, false,
-                                       getter_AddRefs(movemail_server));
-  if (NS_SUCCEEDED(rv)) {
-    movemail_server.forget(aResult);
-    return rv;
-  }
-#endif /* HAVE_MOVEMAIL */
 
   // if that fails, look for the pop hosts matching the given hostname
   nsCOMPtr<nsIMsgIncomingServer> server;
   if (NS_FAILED(rv)) {
-    rv = NS_MutateURI(url).SetScheme(NS_LITERAL_CSTRING("pop3")).Finalize(url);
+    rv = NS_MutateURI(url).SetScheme("pop3"_ns).Finalize(url);
     NS_ENSURE_SUCCESS(rv, rv);
-    rv = accountManager->FindServerByURI(url, false, getter_AddRefs(server));
+    rv = accountManager->FindServerByURI(url, getter_AddRefs(server));
 
     // if we can't find a pop server, maybe it's a local message
     // in an imap hierarchy. look for an imap server.
     if (NS_FAILED(rv)) {
-      rv =
-          NS_MutateURI(url).SetScheme(NS_LITERAL_CSTRING("imap")).Finalize(url);
+      rv = NS_MutateURI(url).SetScheme("imap"_ns).Finalize(url);
       NS_ENSURE_SUCCESS(rv, rv);
-      rv = accountManager->FindServerByURI(url, false, getter_AddRefs(server));
+      rv = accountManager->FindServerByURI(url, getter_AddRefs(server));
     }
   }
   if (NS_SUCCEEDED(rv)) {
@@ -93,8 +79,7 @@ static nsresult nsGetMailboxServer(const char* uriStr,
     return rv;
   }
 
-  // if you fail after looking at all "pop3", "movemail" and "none" servers, you
-  // fail.
+  // If you fail after looking at all "pop3", "none" servers, you fail.
   return rv;
 }
 
@@ -151,14 +136,9 @@ nsresult nsLocalURI2Path(const char* rootURI, const char* uriStr,
     nsAutoCString newPath("");
 
     // Unescape folder name
-    if (curPos) {
-      nsCString unescapedStr;
-      MsgUnescapeString(nsDependentCString(curPos), 0, unescapedStr);
-      NS_MsgCreatePathStringFromFolderURI(unescapedStr.get(), newPath,
-                                          NS_LITERAL_CSTRING("none"));
-    } else
-      NS_MsgCreatePathStringFromFolderURI(curPos, newPath,
-                                          NS_LITERAL_CSTRING("none"));
+    nsCString unescapedStr;
+    MsgUnescapeString(nsDependentCString(curPos), 0, unescapedStr);
+    NS_MsgCreatePathStringFromFolderURI(unescapedStr.get(), newPath, "none"_ns);
 
     pathResult.Append('/');
     pathResult.Append(newPath);
@@ -174,11 +154,11 @@ nsresult nsLocalURI2Path(const char* rootURI, const char* uriStr,
  * puts folder URI in folderURI (mailbox://folder1/folder2)
  * message key number in key
  */
-nsresult nsParseLocalMessageURI(const char* uri, nsCString& folderURI,
+nsresult nsParseLocalMessageURI(const nsACString& uri, nsCString& folderURI,
                                 nsMsgKey* key) {
   if (!key) return NS_ERROR_NULL_POINTER;
 
-  nsAutoCString uriStr(uri);
+  const nsPromiseFlatCString& uriStr = PromiseFlatCString(uri);
   int32_t keySeparator = uriStr.FindChar('#');
   if (keySeparator != -1) {
     int32_t keyEndSeparator = MsgFindCharInSet(uriStr, "?&", keySeparator);
@@ -198,8 +178,8 @@ nsresult nsParseLocalMessageURI(const char* uri, nsCString& folderURI,
   return NS_ERROR_FAILURE;
 }
 
-nsresult nsBuildLocalMessageURI(const char* baseURI, nsMsgKey key,
-                                nsCString& uri) {
+nsresult nsBuildLocalMessageURI(const nsACString& baseURI, nsMsgKey key,
+                                nsACString& uri) {
   // need to convert mailbox://hostname/.. to mailbox-message://hostname/..
   uri.Append(baseURI);
   uri.Append('#');
