@@ -1,4 +1,4 @@
-/* -*- Mode: C++; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*- */
+/* -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
 /* This Source Code Form is subject to the terms of the Mozilla Public
  * License, v. 2.0. If a copy of the MPL was not distributed with this
  * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
@@ -30,6 +30,7 @@
 #include "nsComponentManagerUtils.h"
 #include "nsServiceManagerUtils.h"
 #include "mozilla/Services.h"
+#include "mozilla/LoadInfo.h"
 
 #define PREF_MAIL_ROOT_POP3_REL "mail.root.pop3-rel"
 // old - for backward compatibility only
@@ -44,32 +45,35 @@ nsPop3Service::~nsPop3Service() {}
 NS_IMPL_ISUPPORTS(nsPop3Service, nsIPop3Service, nsIProtocolHandler,
                   nsIMsgProtocolInfo)
 
-NS_IMETHODIMP nsPop3Service::CheckForNewMail(nsIMsgWindow *aMsgWindow,
-                                             nsIUrlListener *aUrlListener,
-                                             nsIMsgFolder *aInbox,
-                                             nsIPop3IncomingServer *aPopServer,
-                                             nsIURI **aURL) {
+NS_IMETHODIMP nsPop3Service::CheckForNewMail(nsIMsgWindow* aMsgWindow,
+                                             nsIUrlListener* aUrlListener,
+                                             nsIMsgFolder* aInbox,
+                                             nsIPop3IncomingServer* aPopServer,
+                                             nsIURI** aURL) {
   return GetMail(false /* don't download, just check */, aMsgWindow,
                  aUrlListener, aInbox, aPopServer, aURL);
 }
 
-nsresult nsPop3Service::GetNewMail(nsIMsgWindow *aMsgWindow,
-                                   nsIUrlListener *aUrlListener,
-                                   nsIMsgFolder *aInbox,
-                                   nsIPop3IncomingServer *aPopServer,
-                                   nsIURI **aURL) {
+nsresult nsPop3Service::GetNewMail(nsIMsgWindow* aMsgWindow,
+                                   nsIUrlListener* aUrlListener,
+                                   nsIMsgFolder* aInbox,
+                                   nsIPop3IncomingServer* aPopServer,
+                                   nsIURI** aURL) {
   return GetMail(true /* download */, aMsgWindow, aUrlListener, aInbox,
                  aPopServer, aURL);
 }
 
-nsresult nsPop3Service::GetMail(bool downloadNewMail, nsIMsgWindow *aMsgWindow,
-                                nsIUrlListener *aUrlListener,
-                                nsIMsgFolder *aInbox,
-                                nsIPop3IncomingServer *aPopServer,
-                                nsIURI **aURL) {
+nsresult nsPop3Service::GetMail(bool downloadNewMail, nsIMsgWindow* aMsgWindow,
+                                nsIUrlListener* aUrlListener,
+                                nsIMsgFolder* aInbox,
+                                nsIPop3IncomingServer* aPopServer,
+                                nsIURI** aURL) {
   NS_ENSURE_ARG_POINTER(aInbox);
-  int32_t popPort = -1;
+  if (aURL) {
+    *aURL = nullptr;
+  }
 
+  int32_t popPort = -1;
   nsCOMPtr<nsIMsgIncomingServer> server;
   nsCOMPtr<nsIURI> url;
 
@@ -106,7 +110,7 @@ nsresult nsPop3Service::GetMail(bool downloadNewMail, nsIMsgWindow *aMsgWindow,
     // now construct a pop3 url...
     // we need to escape the username because it may contain
     // characters like / % or @
-    char *urlSpec =
+    char* urlSpec =
         (downloadNewMail)
             ? PR_smprintf("pop3://%s@%s:%d", escapedUsername.get(),
                           popHost.get(), popPort)
@@ -128,10 +132,10 @@ nsresult nsPop3Service::GetMail(bool downloadNewMail, nsIMsgWindow *aMsgWindow,
   return rv;
 }
 
-NS_IMETHODIMP nsPop3Service::VerifyLogon(nsIMsgIncomingServer *aServer,
-                                         nsIUrlListener *aUrlListener,
-                                         nsIMsgWindow *aMsgWindow,
-                                         nsIURI **aURL) {
+NS_IMETHODIMP nsPop3Service::VerifyLogon(nsIMsgIncomingServer* aServer,
+                                         nsIUrlListener* aUrlListener,
+                                         nsIMsgWindow* aMsgWindow,
+                                         nsIURI** aURL) {
   NS_ENSURE_ARG_POINTER(aServer);
   nsCString popHost;
   nsCString popUser;
@@ -158,7 +162,7 @@ NS_IMETHODIMP nsPop3Service::VerifyLogon(nsIMsgIncomingServer *aServer,
   // now construct a pop3 url...
   // we need to escape the username because it may contain
   // characters like / % or @
-  char *urlSpec = PR_smprintf("pop3://%s@%s:%d/?verifyLogon",
+  char* urlSpec = PR_smprintf("pop3://%s@%s:%d/?verifyLogon",
                               escapedUsername.get(), popHost.get(), popPort);
   NS_ENSURE_TRUE(urlSpec, NS_ERROR_OUT_OF_MEMORY);
 
@@ -175,13 +179,13 @@ NS_IMETHODIMP nsPop3Service::VerifyLogon(nsIMsgIncomingServer *aServer,
   return rv;
 }
 
-nsresult nsPop3Service::BuildPop3Url(const char *urlSpec, nsIMsgFolder *inbox,
-                                     nsIPop3IncomingServer *server,
-                                     nsIUrlListener *aUrlListener,
-                                     nsIURI **aUrl, nsIMsgWindow *aMsgWindow) {
+nsresult nsPop3Service::BuildPop3Url(const char* urlSpec, nsIMsgFolder* inbox,
+                                     nsIPop3IncomingServer* server,
+                                     nsIUrlListener* aUrlListener,
+                                     nsIURI** aUrl, nsIMsgWindow* aMsgWindow) {
   nsresult rv;
 
-  nsPop3Sink *pop3Sink = new nsPop3Sink();
+  nsPop3Sink* pop3Sink = new nsPop3Sink();
 
   NS_ENSURE_TRUE(pop3Sink, NS_ERROR_OUT_OF_MEMORY);
 
@@ -210,8 +214,8 @@ nsresult nsPop3Service::BuildPop3Url(const char *urlSpec, nsIMsgFolder *inbox,
   return rv;
 }
 
-nsresult nsPop3Service::RunPopUrl(nsIMsgIncomingServer *aServer,
-                                  nsIURI *aUrlToRun) {
+nsresult nsPop3Service::RunPopUrl(nsIMsgIncomingServer* aServer,
+                                  nsIURI* aUrlToRun) {
   NS_ENSURE_ARG_POINTER(aServer);
   NS_ENSURE_ARG_POINTER(aUrlToRun);
 
@@ -220,7 +224,8 @@ nsresult nsPop3Service::RunPopUrl(nsIMsgIncomingServer *aServer,
   // load up required server information
   // we store the username unescaped in the server
   // so there is no need to unescape it
-  nsresult rv = aServer->GetRealUsername(userName);
+  nsresult rv = aServer->GetUsername(userName);
+  NS_ENSURE_SUCCESS(rv, rv);
 
   // find out if the server is busy or not...if the server is busy, we are
   // *NOT* going to run the url
@@ -229,13 +234,16 @@ nsresult nsPop3Service::RunPopUrl(nsIMsgIncomingServer *aServer,
   NS_ENSURE_SUCCESS(rv, rv);
   if (!serverBusy) {
     RefPtr<nsPop3Protocol> protocol = new nsPop3Protocol(aUrlToRun);
-    if (protocol) {
-      // the protocol stores the unescaped username, so there is no need to
-      // escape it.
-      protocol->SetUsername(userName.get());
-      rv = protocol->LoadUrl(aUrlToRun);
-      if (NS_FAILED(rv)) aServer->SetServerBusy(false);
-    }
+    // It implements nsIChannel, and all channels require loadInfo.
+    protocol->SetLoadInfo(new mozilla::net::LoadInfo(
+        nsContentUtils::GetSystemPrincipal(), nullptr, nullptr,
+        nsILoadInfo::SEC_ALLOW_CROSS_ORIGIN_SEC_CONTEXT_IS_NULL,
+        nsIContentPolicy::TYPE_OTHER));
+    // the protocol stores the unescaped username, so there is no need to
+    // escape it.
+    protocol->SetUsername(userName.get());
+    rv = protocol->LoadUrl(aUrlToRun);
+    if (NS_FAILED(rv)) aServer->SetServerBusy(false);
   } else {
     nsCOMPtr<nsIMsgMailNewsUrl> url = do_QueryInterface(aUrlToRun);
     if (url) AlertServerBusy(url);
@@ -244,47 +252,49 @@ nsresult nsPop3Service::RunPopUrl(nsIMsgIncomingServer *aServer,
   return rv;
 }
 
-NS_IMETHODIMP nsPop3Service::GetScheme(nsACString &aScheme) {
+NS_IMETHODIMP nsPop3Service::GetScheme(nsACString& aScheme) {
   aScheme.AssignLiteral("pop3");
   return NS_OK;
 }
 
-NS_IMETHODIMP nsPop3Service::GetDefaultPort(int32_t *aDefaultPort) {
+NS_IMETHODIMP nsPop3Service::GetDefaultPort(int32_t* aDefaultPort) {
   NS_ENSURE_ARG_POINTER(aDefaultPort);
   *aDefaultPort = nsIPop3URL::DEFAULT_POP3_PORT;
   return NS_OK;
 }
 
-NS_IMETHODIMP nsPop3Service::AllowPort(int32_t port, const char *scheme,
-                                       bool *_retval) {
+NS_IMETHODIMP nsPop3Service::AllowPort(int32_t port, const char* scheme,
+                                       bool* _retval) {
   *_retval = true;  // allow pop on any port
   return NS_OK;
 }
 
-NS_IMETHODIMP nsPop3Service::GetDefaultDoBiff(bool *aDoBiff) {
+NS_IMETHODIMP nsPop3Service::GetDefaultDoBiff(bool* aDoBiff) {
   NS_ENSURE_ARG_POINTER(aDoBiff);
   // by default, do biff for POP3 servers
   *aDoBiff = true;
   return NS_OK;
 }
 
-NS_IMETHODIMP nsPop3Service::GetProtocolFlags(uint32_t *result) {
+NS_IMETHODIMP nsPop3Service::GetProtocolFlags(uint32_t* result) {
   NS_ENSURE_ARG_POINTER(result);
   *result = URI_NORELATIVE | URI_DANGEROUS_TO_LOAD | ALLOWS_PROXY |
             URI_FORBIDS_COOKIE_ACCESS;
   return NS_OK;
 }
 
-NS_IMETHODIMP nsPop3Service::NewURI(const nsACString &aSpec,
-                                    const char *aOriginCharset,  // ignored
-                                    nsIURI *aBaseURI, nsIURI **_retval) {
+nsresult nsPop3Service::NewURI(const nsACString& aSpec,
+                               const char* aOriginCharset,  // ignored
+                               nsIURI* aBaseURI, nsIURI** _retval) {
   NS_ENSURE_ARG_POINTER(_retval);
 
   nsAutoCString folderUri(aSpec);
   int32_t offset = folderUri.FindChar('?');
   if (offset != kNotFound) folderUri.SetLength(offset);
 
-  const char *uidl = PL_strstr(nsCString(aSpec).get(), "uidl=");
+  // Hold onto the string until it goes out of scope.
+  const nsPromiseFlatCString& flat = PromiseFlatCString(aSpec);
+  const char* uidl = PL_strstr(flat.get(), "uidl=");
   NS_ENSURE_TRUE(uidl, NS_ERROR_FAILURE);
 
   nsresult rv;
@@ -372,7 +382,7 @@ NS_IMETHODIMP nsPop3Service::NewURI(const nsACString &aSpec,
   if (offset != kNotFound) messageUri.Replace(offset, 8, "#");
   offset = messageUri.FindChar('&');
   if (offset != kNotFound) messageUri.SetLength(offset);
-  popurl->SetMessageUri(messageUri.get());
+  popurl->SetMessageUri(messageUri);
   nsCOMPtr<nsIPop3Sink> pop3Sink;
   rv = popurl->GetPop3Sink(getter_AddRefs(pop3Sink));
   NS_ENSURE_SUCCESS(rv, rv);
@@ -383,7 +393,7 @@ NS_IMETHODIMP nsPop3Service::NewURI(const nsACString &aSpec,
   return NS_OK;
 }
 
-void nsPop3Service::AlertServerBusy(nsIMsgMailNewsUrl *url) {
+void nsPop3Service::AlertServerBusy(nsIMsgMailNewsUrl* url) {
   nsresult rv;
   nsCOMPtr<nsIStringBundleService> bundleService =
       mozilla::services::GetStringBundleService();
@@ -409,22 +419,23 @@ void nsPop3Service::AlertServerBusy(nsIMsgMailNewsUrl *url) {
   rv = server->GetPrettyName(accountName);
   NS_ENSURE_SUCCESS_VOID(rv);
 
-  const char16_t *params[] = {accountName.get()};
+  AutoTArray<nsString, 1> params = {accountName};
   nsString alertString;
   nsString dialogTitle;
-  bundle->FormatStringFromName("pop3ServerBusy", params, 1, alertString);
-  bundle->FormatStringFromName("pop3ErrorDialogTitle", params, 1, dialogTitle);
+  bundle->FormatStringFromName("pop3ServerBusy", params, alertString);
+  bundle->FormatStringFromName("pop3ErrorDialogTitle", params, dialogTitle);
   if (!alertString.IsEmpty())
     dialog->Alert(dialogTitle.get(), alertString.get());
 }
 
-NS_IMETHODIMP nsPop3Service::NewChannel(nsIURI *aURI, nsILoadInfo *aLoadInfo,
-                                        nsIChannel **_retval) {
+NS_IMETHODIMP nsPop3Service::NewChannel(nsIURI* aURI, nsILoadInfo* aLoadInfo,
+                                        nsIChannel** _retval) {
   NS_ENSURE_ARG_POINTER(aURI);
+  MOZ_ASSERT(aLoadInfo);
   nsresult rv;
 
   nsCOMPtr<nsIMsgMailNewsUrl> url = do_QueryInterface(aURI, &rv);
-  nsCString realUserName;
+  nsCString userName;
   if (NS_SUCCEEDED(rv) && url) {
     nsCOMPtr<nsIMsgIncomingServer> server;
     url->GetServer(getter_AddRefs(server));
@@ -438,7 +449,7 @@ NS_IMETHODIMP nsPop3Service::NewChannel(nsIURI *aURI, nsILoadInfo *aLoadInfo,
         AlertServerBusy(url);
         return NS_MSG_FOLDER_BUSY;
       }
-      server->GetRealUsername(realUserName);
+      server->GetUsername(userName);
     }
   }
 
@@ -451,21 +462,21 @@ NS_IMETHODIMP nsPop3Service::NewChannel(nsIURI *aURI, nsILoadInfo *aLoadInfo,
   rv = protocol->SetLoadInfo(aLoadInfo);
   NS_ENSURE_SUCCESS(rv, rv);
 
-  protocol->SetUsername(realUserName.get());
+  protocol->SetUsername(userName.get());
 
   protocol.forget(_retval);
   return NS_OK;
 }
 
 NS_IMETHODIMP
-nsPop3Service::SetDefaultLocalPath(nsIFile *aPath) {
+nsPop3Service::SetDefaultLocalPath(nsIFile* aPath) {
   NS_ENSURE_ARG(aPath);
   return NS_SetPersistentFile(PREF_MAIL_ROOT_POP3_REL, PREF_MAIL_ROOT_POP3,
                               aPath);
 }
 
 NS_IMETHODIMP
-nsPop3Service::GetDefaultLocalPath(nsIFile **aResult) {
+nsPop3Service::GetDefaultLocalPath(nsIFile** aResult) {
   NS_ENSURE_ARG_POINTER(aResult);
   *aResult = nullptr;
 
@@ -493,13 +504,13 @@ nsPop3Service::GetDefaultLocalPath(nsIFile **aResult) {
 }
 
 NS_IMETHODIMP
-nsPop3Service::GetServerIID(nsIID **aServerIID) {
+nsPop3Service::GetServerIID(nsIID** aServerIID) {
   *aServerIID = new nsIID(NS_GET_IID(nsIPop3IncomingServer));
   return NS_OK;
 }
 
 NS_IMETHODIMP
-nsPop3Service::GetRequiresUsername(bool *aRequiresUsername) {
+nsPop3Service::GetRequiresUsername(bool* aRequiresUsername) {
   NS_ENSURE_ARG_POINTER(aRequiresUsername);
   *aRequiresUsername = true;
   return NS_OK;
@@ -507,63 +518,63 @@ nsPop3Service::GetRequiresUsername(bool *aRequiresUsername) {
 
 NS_IMETHODIMP
 nsPop3Service::GetPreflightPrettyNameWithEmailAddress(
-    bool *aPreflightPrettyNameWithEmailAddress) {
+    bool* aPreflightPrettyNameWithEmailAddress) {
   NS_ENSURE_ARG_POINTER(aPreflightPrettyNameWithEmailAddress);
   *aPreflightPrettyNameWithEmailAddress = true;
   return NS_OK;
 }
 
 NS_IMETHODIMP
-nsPop3Service::GetCanLoginAtStartUp(bool *aCanLoginAtStartUp) {
+nsPop3Service::GetCanLoginAtStartUp(bool* aCanLoginAtStartUp) {
   NS_ENSURE_ARG_POINTER(aCanLoginAtStartUp);
   *aCanLoginAtStartUp = true;
   return NS_OK;
 }
 
 NS_IMETHODIMP
-nsPop3Service::GetCanDelete(bool *aCanDelete) {
+nsPop3Service::GetCanDelete(bool* aCanDelete) {
   NS_ENSURE_ARG_POINTER(aCanDelete);
   *aCanDelete = true;
   return NS_OK;
 }
 
 NS_IMETHODIMP
-nsPop3Service::GetCanDuplicate(bool *aCanDuplicate) {
+nsPop3Service::GetCanDuplicate(bool* aCanDuplicate) {
   NS_ENSURE_ARG_POINTER(aCanDuplicate);
   *aCanDuplicate = true;
   return NS_OK;
 }
 
 NS_IMETHODIMP
-nsPop3Service::GetCanGetMessages(bool *aCanGetMessages) {
+nsPop3Service::GetCanGetMessages(bool* aCanGetMessages) {
   NS_ENSURE_ARG_POINTER(aCanGetMessages);
   *aCanGetMessages = true;
   return NS_OK;
 }
 
 NS_IMETHODIMP
-nsPop3Service::GetCanGetIncomingMessages(bool *aCanGetIncomingMessages) {
+nsPop3Service::GetCanGetIncomingMessages(bool* aCanGetIncomingMessages) {
   NS_ENSURE_ARG_POINTER(aCanGetIncomingMessages);
   *aCanGetIncomingMessages = true;
   return NS_OK;
 }
 
 NS_IMETHODIMP
-nsPop3Service::GetShowComposeMsgLink(bool *showComposeMsgLink) {
+nsPop3Service::GetShowComposeMsgLink(bool* showComposeMsgLink) {
   NS_ENSURE_ARG_POINTER(showComposeMsgLink);
   *showComposeMsgLink = true;
   return NS_OK;
 }
 
 NS_IMETHODIMP
-nsPop3Service::GetFoldersCreatedAsync(bool *aAsyncCreation) {
+nsPop3Service::GetFoldersCreatedAsync(bool* aAsyncCreation) {
   NS_ENSURE_ARG_POINTER(aAsyncCreation);
   *aAsyncCreation = false;
   return NS_OK;
 }
 
 NS_IMETHODIMP
-nsPop3Service::GetDefaultServerPort(bool isSecure, int32_t *aPort) {
+nsPop3Service::GetDefaultServerPort(bool isSecure, int32_t* aPort) {
   NS_ENSURE_ARG_POINTER(aPort);
 
   if (!isSecure) return GetDefaultPort(aPort);
@@ -574,7 +585,7 @@ nsPop3Service::GetDefaultServerPort(bool isSecure, int32_t *aPort) {
 }
 
 NS_IMETHODIMP
-nsPop3Service::NotifyDownloadStarted(nsIMsgFolder *aFolder) {
+nsPop3Service::NotifyDownloadStarted(nsIMsgFolder* aFolder) {
   nsTObserverArray<nsCOMPtr<nsIPop3ServiceListener> >::ForwardIterator iter(
       mListeners);
   nsCOMPtr<nsIPop3ServiceListener> listener;
@@ -586,7 +597,7 @@ nsPop3Service::NotifyDownloadStarted(nsIMsgFolder *aFolder) {
 }
 
 NS_IMETHODIMP
-nsPop3Service::NotifyDownloadProgress(nsIMsgFolder *aFolder,
+nsPop3Service::NotifyDownloadProgress(nsIMsgFolder* aFolder,
                                       uint32_t aNumMessages,
                                       uint32_t aNumTotalMessages) {
   nsTObserverArray<nsCOMPtr<nsIPop3ServiceListener> >::ForwardIterator iter(
@@ -600,7 +611,7 @@ nsPop3Service::NotifyDownloadProgress(nsIMsgFolder *aFolder,
 }
 
 NS_IMETHODIMP
-nsPop3Service::NotifyDownloadCompleted(nsIMsgFolder *aFolder,
+nsPop3Service::NotifyDownloadCompleted(nsIMsgFolder* aFolder,
                                        uint32_t aNumMessages) {
   nsTObserverArray<nsCOMPtr<nsIPop3ServiceListener> >::ForwardIterator iter(
       mListeners);
@@ -612,13 +623,13 @@ nsPop3Service::NotifyDownloadCompleted(nsIMsgFolder *aFolder,
   return NS_OK;
 }
 
-NS_IMETHODIMP nsPop3Service::AddListener(nsIPop3ServiceListener *aListener) {
+NS_IMETHODIMP nsPop3Service::AddListener(nsIPop3ServiceListener* aListener) {
   NS_ENSURE_ARG_POINTER(aListener);
   mListeners.AppendElementUnlessExists(aListener);
   return NS_OK;
 }
 
-NS_IMETHODIMP nsPop3Service::RemoveListener(nsIPop3ServiceListener *aListener) {
+NS_IMETHODIMP nsPop3Service::RemoveListener(nsIPop3ServiceListener* aListener) {
   NS_ENSURE_ARG_POINTER(aListener);
   mListeners.RemoveElement(aListener);
   return NS_OK;
