@@ -13,7 +13,6 @@
 #include "plstr.h"
 #include "prenv.h"
 #include "nsIMsgFolder.h"
-#include "nsAutoPtr.h"
 #include "nsTArray.h"
 #include "nsString.h"
 
@@ -34,17 +33,16 @@ class nsPop3Sink : public nsIPop3Sink {
 
   NS_DECL_THREADSAFE_ISUPPORTS
   NS_DECL_NSIPOP3SINK
-  nsresult GetServerFolder(nsIMsgFolder **aFolder);
+  nsresult GetServerFolder(nsIMsgFolder** aFolder);
   nsresult FindPartialMessages();
-  void CheckPartialMessages(nsIPop3Protocol *protocol);
+  void CheckPartialMessages(nsIPop3Protocol* protocol);
 
-  static char *GetDummyEnvelope(void);
+  static char* GetDummyEnvelope(void);
 
  protected:
   virtual ~nsPop3Sink();
-  nsresult WriteLineToMailbox(const nsACString &buffer);
+  nsresult WriteLineToMailbox(const nsACString& buffer);
   nsresult ReleaseFolderLock();
-  nsresult HandleTempDownloadFailed(nsIMsgWindow *msgWindow);
 
   bool m_authed;
   nsCString m_accountUrl;
@@ -63,14 +61,12 @@ class nsPop3Sink : public nsIPop3Sink {
   nsCOMPtr<nsIMsgPluggableStore> m_msgStore;
   bool m_uidlDownload;
   bool m_buildMessageUri;
-  bool m_downloadingToTempFile;
-  nsCOMPtr<nsIFile> m_tmpDownloadFile;
   nsCOMPtr<nsIMsgWindow> m_window;
   nsCString m_messageUri;
   nsCString m_baseMessageUri;
   nsCString m_origMessageUri;
   nsCString m_accountKey;
-  nsTArray<partialRecord *> m_partialMsgsArray;
+  nsTArray<partialRecord*> m_partialMsgsArray;
 };
 
 #endif
