@@ -2,27 +2,36 @@
 /**
  * Protocol tests for POP3.
  */
-ChromeUtils.import("resource:///modules/mailServices.js");
 
-var type = null;
-var test = null;
 var server;
 var daemon;
 var incomingServer;
 var thisTest;
 
 var tests = [
-  { title: "Get New Mail, One Message",
-    messages: [ "message2.eml", "message2.eml", "message3.eml" ],
-    transaction: [ "AUTH", "CAPA", "AUTH PLAIN", "STAT", "LIST", "UIDL",
-                   "RETR 1", "DELE 1", "RETR 2", "DELE 2", "RETR 3", "DELE 3" ] }
+  {
+    title: "Get New Mail, One Message",
+    messages: ["message2.eml", "message2.eml", "message3.eml"],
+    transaction: [
+      "AUTH",
+      "CAPA",
+      "AUTH PLAIN",
+      "STAT",
+      "LIST",
+      "UIDL",
+      "RETR 1",
+      "DELE 1",
+      "RETR 2",
+      "DELE 2",
+      "RETR 3",
+      "DELE 3",
+    ],
+  },
 ];
 
-var urlListener =
-{
-  OnStartRunningUrl: function (url) {
-  },
-  OnStopRunningUrl: function (url, result) {
+var urlListener = {
+  OnStartRunningUrl(url) {},
+  OnStopRunningUrl(url, result) {
     try {
       var transaction = server.playTransaction();
 
@@ -32,21 +41,21 @@ var urlListener =
       Assert.equal(localAccountUtils.inboxFolder.getNumUnread(false), 1);
 
       Assert.equal(result, 0);
-    }
-    catch (e) {
+    } catch (e) {
       // If we have an error, clean up nicely before we throw it.
       server.stop();
 
       var thread = gThreadManager.currentThread;
-      while (thread.hasPendingEvents())
+      while (thread.hasPendingEvents()) {
         thread.processNextEvent(true);
+      }
 
       do_throw(e);
     }
 
     // Let OnStopRunningUrl return cleanly before doing anything else.
     do_timeout(0, checkBusy);
-  }
+  },
 };
 
 function checkBusy() {
@@ -57,17 +66,20 @@ function checkBusy() {
     server.stop();
 
     var thread = gThreadManager.currentThread;
-    while (thread.hasPendingEvents())
+    while (thread.hasPendingEvents()) {
       thread.processNextEvent(true);
+    }
 
     do_test_finished();
     return;
   }
 
   // If the server hasn't quite finished, just delay a little longer.
-  if (incomingServer.serverBusy ||
-      (incomingServer instanceof Ci.nsIPop3IncomingServer &&
-       incomingServer.runningProtocol)) {
+  if (
+    incomingServer.serverBusy ||
+    (incomingServer instanceof Ci.nsIPop3IncomingServer &&
+      incomingServer.runningProtocol)
+  ) {
     do_timeout(20, checkBusy);
     return;
   }
@@ -89,8 +101,12 @@ function testNext() {
     daemon.setMessages(thisTest.messages);
 
     // Now get the mail
-    MailServices.pop3.GetNewMail(null, urlListener, localAccountUtils.inboxFolder,
-                                 incomingServer);
+    MailServices.pop3.GetNewMail(
+      null,
+      urlListener,
+      localAccountUtils.inboxFolder,
+      incomingServer
+    );
 
     server.performTest();
   } catch (e) {
@@ -99,8 +115,9 @@ function testNext() {
     do_throw(e);
   } finally {
     var thread = gThreadManager.currentThread;
-    while (thread.hasPendingEvents())
+    while (thread.hasPendingEvents()) {
       thread.processNextEvent(true);
+    }
   }
 }
 
@@ -117,8 +134,6 @@ function run_test() {
   server = server[1];
   server.start();
 
-  type = "RFC 1939";
-
   // Set up the basic accounts and folders
   incomingServer = createPop3ServerAndLocalFolders(server.port);
 
