@@ -104,7 +104,7 @@ STDMETHODIMP CMapiImp::Login(unsigned long aUIArg, LPSTR aLogin,
   nsCString id_key;
 
   MOZ_LOG(MAPI, mozilla::LogLevel::Debug,
-          ("CMapiImp::Login using flags %d", aFlags));
+          ("CMapiImp::Login using flags %lu", aFlags));
   if (aFlags & MAPI_NEW_SESSION) bNewSession = true;
 
   // Check For Profile Name
@@ -239,7 +239,7 @@ STDMETHODIMP CMapiImp::SendDocuments(unsigned long aSession, LPSTR aDelimChar,
   nsresult rv = NS_OK;
 
   MOZ_LOG(MAPI, mozilla::LogLevel::Debug,
-          ("CMapiImp::SendDocument using flags %d", aFlags));
+          ("CMapiImp::SendDocument using flags %lu", aFlags));
   /** create nsIMsgCompFields obj and populate it **/
   nsCOMPtr<nsIMsgCompFields> pCompFields =
       do_CreateInstance(NS_MSGCOMPFIELDS_CONTRACTID, &rv);
@@ -254,7 +254,7 @@ STDMETHODIMP CMapiImp::SendDocuments(unsigned long aSession, LPSTR aDelimChar,
     rv = nsMapiHook::ShowComposerWindow(aSession, pCompFields);
   else
     MOZ_LOG(MAPI, mozilla::LogLevel::Debug,
-            ("CMapiImp::SendDocument error rv = %lx, paths = %s names = %s", rv,
+            ("CMapiImp::SendDocument error rv = %x, paths = %s names = %s", rv,
              aFilePaths, aFileNames));
 
   return nsMAPIConfiguration::GetMAPIErrorFromNSError(rv);
@@ -322,7 +322,7 @@ class MsgMapiListContext {
 
   nsCOMPtr<nsIMsgFolder> m_folder;
   nsCOMPtr<nsIMsgDatabase> m_db;
-  nsCOMPtr<nsISimpleEnumerator> m_msgEnumerator;
+  nsCOMPtr<nsIMsgEnumerator> m_msgEnumerator;
 };
 
 LONG CMapiImp::InitContext(unsigned long session,
@@ -494,12 +494,9 @@ nsMsgKey MsgMapiListContext::GetNext() {
   if (m_msgEnumerator && m_db) {
     do {
       keepTrying = FALSE;
-      nsCOMPtr<nsISupports> hdrISupports;
       nsCOMPtr<nsIMsgDBHdr> msgHdr;
-      if (NS_SUCCEEDED(
-              m_msgEnumerator->GetNext(getter_AddRefs(hdrISupports))) &&
-          hdrISupports) {
-        msgHdr = do_QueryInterface(hdrISupports);
+      if (NS_SUCCEEDED(m_msgEnumerator->GetNext(getter_AddRefs(msgHdr))) &&
+          msgHdr) {
         msgHdr->GetMessageKey(&key);
 
         // Check here for IMAP message...if not, just return...
@@ -590,7 +587,7 @@ lpnsMapiMessage MsgMapiListContext::GetMessage(nsMsgKey key,
       }
 
       MOZ_LOG(MAPI, mozilla::LogLevel::Debug,
-              ("MsgMapiListContext::GetMessage flags=%x subject %s date %s "
+              ("MsgMapiListContext::GetMessage flags=%lu subject %s date %s "
                "sender %s",
                flFlags, (char*)message->lpszSubject,
                (char*)message->lpszDateReceived, author.get()));
