@@ -97,7 +97,8 @@ bool nsMimeHtmlDisplayEmitter::BroadCastHeadersAndAttachments() {
 nsresult nsMimeHtmlDisplayEmitter::WriteHeaderFieldHTMLPrefix(
     const nsACString& name) {
   if (!BroadCastHeadersAndAttachments() ||
-      (mFormat == nsMimeOutput::nsMimeMessagePrintOutput))
+      (mFormat == nsMimeOutput::nsMimeMessagePrintOutput) ||
+      (mFormat == nsMimeOutput::nsMimeMessageBodyDisplay))
     return nsMimeBaseEmitter::WriteHeaderFieldHTMLPrefix(name);
   else
     return NS_OK;
@@ -106,7 +107,8 @@ nsresult nsMimeHtmlDisplayEmitter::WriteHeaderFieldHTMLPrefix(
 nsresult nsMimeHtmlDisplayEmitter::WriteHeaderFieldHTML(const char* field,
                                                         const char* value) {
   if (!BroadCastHeadersAndAttachments() ||
-      (mFormat == nsMimeOutput::nsMimeMessagePrintOutput))
+      (mFormat == nsMimeOutput::nsMimeMessagePrintOutput) ||
+      (mFormat == nsMimeOutput::nsMimeMessageBodyDisplay))
     return nsMimeBaseEmitter::WriteHeaderFieldHTML(field, value);
   else
     return NS_OK;
@@ -114,7 +116,8 @@ nsresult nsMimeHtmlDisplayEmitter::WriteHeaderFieldHTML(const char* field,
 
 nsresult nsMimeHtmlDisplayEmitter::WriteHeaderFieldHTMLPostfix() {
   if (!BroadCastHeadersAndAttachments() ||
-      (mFormat == nsMimeOutput::nsMimeMessagePrintOutput))
+      (mFormat == nsMimeOutput::nsMimeMessagePrintOutput) ||
+      (mFormat == nsMimeOutput::nsMimeMessageBodyDisplay))
     return nsMimeBaseEmitter::WriteHeaderFieldHTMLPostfix();
   else
     return NS_OK;
@@ -164,6 +167,9 @@ nsresult nsMimeHtmlDisplayEmitter::BroadcastHeaders(
   bool pushAllHeaders = false;
   bool checkExtraHeaders = false;
   bool checkAddonHeaders = false;
+  nsCString otherHeaders;
+  nsTArray<nsCString> otherHeadersArray;
+  bool checkOtherHeaders = false;
 
   nsresult rv;
   nsCOMPtr<nsIPrefBranch> pPrefBranch(
@@ -189,6 +195,17 @@ nsresult nsMimeHtmlDisplayEmitter::BroadcastHeaders(
         checkAddonHeaders = true;
       }
     }
+
+    pPrefBranch->GetCharPref("mail.compose.other.header", otherHeaders);
+    if (!otherHeaders.IsEmpty()) {
+      ToLowerCase(otherHeaders);
+      ParseString(otherHeaders, ',', otherHeadersArray);
+      for (uint32_t i = 0; i < otherHeadersArray.Length(); i++) {
+        otherHeadersArray[i].Trim(" ");
+      }
+
+      checkOtherHeaders = true;
+    }
   }
 
   for (size_t i = 0; i < mHeaderArray->Length(); i++) {
@@ -232,7 +249,7 @@ nsresult nsMimeHtmlDisplayEmitter::BroadcastHeaders(
                  !PL_strcasecmp("delivered-to", headerName)) {
         skip = false;
 
-      } else if (checkExtraHeaders || checkAddonHeaders) {
+      } else if (checkExtraHeaders || checkAddonHeaders || checkOtherHeaders) {
         // Make headerStr lowercase because
         // extraExpandedHeaders/extraAddonHeadersArray was made lowercase above.
         nsDependentCString headerStr(headerInfo->name);
@@ -242,6 +259,8 @@ nsresult nsMimeHtmlDisplayEmitter::BroadcastHeaders(
           skip = false;
         if (checkAddonHeaders && extraAddonHeadersArray.Contains(headerStr))
           skip = false;
+        if (checkOtherHeaders && otherHeadersArray.Contains(headerStr))
+          skip = false;
       }
 
       if (skip) continue;
@@ -266,19 +285,17 @@ nsresult nsMimeHtmlDisplayEmitter::BroadcastHeaders(
 
 NS_IMETHODIMP nsMimeHtmlDisplayEmitter::WriteHTMLHeaders(
     const nsACString& name) {
-  // if we aren't broadcasting headers OR printing...just do whatever
-  // our base class does...
-  if (mFormat == nsMimeOutput::nsMimeMessagePrintOutput) {
-    return nsMimeBaseEmitter::WriteHTMLHeaders(name);
-  } else if (!BroadCastHeadersAndAttachments() || !mDocHeader) {
-    // This needs to be here to correct the output format if we are
-    // not going to broadcast headers to the XUL document.
-    if (mFormat == nsMimeOutput::nsMimeMessageBodyDisplay)
-      mFormat = nsMimeOutput::nsMimeMessagePrintOutput;
+  if (!BroadCastHeadersAndAttachments() ||
+      (mFormat == nsMimeOutput::nsMimeMessagePrintOutput) ||
+      (mFormat == nsMimeOutput::nsMimeMessageBodyDisplay)) {
+    nsMimeBaseEmitter::WriteHTMLHeaders(name);
+  }
 
-    return nsMimeBaseEmitter::WriteHTMLHeaders(name);
-  } else
-    mFirstHeaders = false;
+  if (!BroadCastHeadersAndAttachments() || !mDocHeader) {
+    return NS_OK;
+  }
+
+  mFirstHeaders = false;
 
   bool bFromNewsgroups = false;
   for (size_t j = 0; j < mHeaderArray->Length(); j++) {
@@ -310,7 +327,8 @@ NS_IMETHODIMP nsMimeHtmlDisplayEmitter::WriteHTMLHeaders(
 
 nsresult nsMimeHtmlDisplayEmitter::EndHeader(const nsACString& name) {
   if (mDocHeader && (mFormat != nsMimeOutput::nsMimeMessageFilterSniffer)) {
-    UtilityWriteCRLF("<html>");
+    // Start with a UTF-8 BOM so this can't be mistaken for another charset.
+    UtilityWriteCRLF("\xEF\xBB\xBF<html>");
     UtilityWriteCRLF("<head>");
 
     const char* val = GetHeaderValue(HEADER_SUBJECT);  // do not free this value
@@ -352,9 +370,9 @@ nsresult nsMimeHtmlDisplayEmitter::StartAttachment(const nsACString& name,
       // else uses GetURI to get the RDF resource which describes the message.
       nsCOMPtr<nsINntpUrl> nntpUrl(do_QueryInterface(mURL, &rv));
       if (NS_SUCCEEDED(rv) && nntpUrl)
-        rv = msgurl->GetOriginalSpec(getter_Copies(uriString));
+        rv = msgurl->GetOriginalSpec(uriString);
       else
-        rv = msgurl->GetUri(getter_Copies(uriString));
+        rv = msgurl->GetUri(uriString);
     }
 
     // we need to convert the attachment name from UTF-8 to unicode before
@@ -364,29 +382,19 @@ nsresult nsMimeHtmlDisplayEmitter::StartAttachment(const nsACString& name,
     nsString unicodeHeaderValue;
     CopyUTF8toUTF16(name, unicodeHeaderValue);
 
-    headerSink->HandleAttachment(contentType, url /* was escapedUrl */,
-                                 unicodeHeaderValue.get(), uriString.get(),
-                                 aIsExternalAttachment);
-
-    mSkipAttachment = false;
-  } else if (mFormat == nsMimeOutput::nsMimeMessagePrintOutput) {
-    // then we need to deal with the attachments in the body by inserting
-    // them into a table..
-    rv = StartAttachmentInBody(name, contentType, url);
-  } else {
-    // If we don't need or cannot broadcast attachment info, just ignore it
-    mSkipAttachment = true;
-    rv = NS_OK;
+    headerSink->HandleAttachment(
+        contentType, nsDependentCString(url) /* was escapedUrl */,
+        unicodeHeaderValue.get(), uriString, aIsExternalAttachment);
   }
 
+  // List the attachments for printing.
+  rv = StartAttachmentInBody(name, contentType, url);
+
   return rv;
 }
 
 // Attachment handling routines
-// Ok, we are changing the way we handle these now...It used to be that we
-// output HTML to make a clickable link, etc... but now, this should just be
-// informational and only show up during printing
-// XXX should they also show up during quoting?
+
 nsresult nsMimeHtmlDisplayEmitter::StartAttachmentInBody(
     const nsACString& name, const char* contentType, const char* url) {
   mSkipAttachment = false;
@@ -399,14 +407,16 @@ nsresult nsMimeHtmlDisplayEmitter::StartAttachmentInBody(
       ((!p7mExternal && !strcmp(contentType, APPLICATION_XPKCS7_MIME)) ||
        (!p7mExternal && !strcmp(contentType, APPLICATION_PKCS7_MIME)) ||
        (!strcmp(contentType, APPLICATION_XPKCS7_SIGNATURE)) ||
-       (!strcmp(contentType, APPLICATION_PKCS7_SIGNATURE)) ||
-       (!strcmp(contentType, TEXT_VCARD)))) {
+       (!strcmp(contentType, APPLICATION_PKCS7_SIGNATURE)))) {
     mSkipAttachment = true;
     return NS_OK;
   }
 
+  // Add the list of attachments. This is only visible when printing.
+
   if (mFirst) {
-    UtilityWrite("<br><fieldset class=\"mimeAttachmentHeader\">");
+    UtilityWrite(
+        "<fieldset class=\"moz-mime-attachment-header moz-print-only\">");
     if (!name.IsEmpty()) {
       nsresult rv;
 
@@ -423,7 +433,8 @@ nsresult nsMimeHtmlDisplayEmitter::StartAttachmentInBody(
       nsString attachmentsHeader;
       bundle->GetStringFromName("attachmentsPrintHeader", attachmentsHeader);
 
-      UtilityWrite("<legend class=\"mimeAttachmentHeaderName\">");
+      UtilityWrite(
+          "<legend class=\"moz-mime-attachment-headerName moz-print-only\">");
       nsCString escapedName;
       nsAppendEscapedHTML(NS_ConvertUTF16toUTF8(attachmentsHeader),
                           escapedName);
@@ -431,14 +442,16 @@ nsresult nsMimeHtmlDisplayEmitter::StartAttachmentInBody(
       UtilityWrite("</legend>");
     }
     UtilityWrite("</fieldset>");
-    UtilityWrite("<div class=\"mimeAttachmentWrap\">");
-    UtilityWrite("<table class=\"mimeAttachmentTable\">");
+    UtilityWrite("<div class=\"moz-mime-attachment-wrap moz-print-only\">");
+    UtilityWrite("<table class=\"moz-mime-attachment-table\">");
   }
 
   UtilityWrite("<tr>");
 
-  UtilityWrite("<td class=\"mimeAttachmentFile\">");
-  UtilityWrite(name);
+  UtilityWrite("<td class=\"moz-mime-attachment-file\">");
+  nsCString escapedName;
+  nsAppendEscapedHTML(name, escapedName);
+  UtilityWrite(escapedName.get());
   UtilityWrite("</td>");
 
   mFirst = false;
@@ -459,30 +472,27 @@ nsresult nsMimeHtmlDisplayEmitter::AddAttachmentField(const char* field,
   nsresult rv = GetHeaderSink(getter_AddRefs(headerSink));
   if (NS_SUCCEEDED(rv) && headerSink) {
     headerSink->AddAttachmentField(field, value);
-  } else {
-    // Currently, we only care about the part size.
-    if (strcmp(field, HEADER_X_MOZILLA_PART_SIZE)) return NS_OK;
-
-    uint64_t size = atoi(value);
-    nsAutoString sizeString;
-    rv = FormatFileSize(size, false, sizeString);
-    UtilityWrite("<td class=\"mimeAttachmentSize\">");
-    UtilityWrite(NS_ConvertUTF16toUTF8(sizeString).get());
-    UtilityWrite("</td>");
   }
 
+  // Currently, we only care about the part size.
+  if (strcmp(field, HEADER_X_MOZILLA_PART_SIZE)) return NS_OK;
+
+  uint64_t size = atoi(value);
+  nsAutoString sizeString;
+  rv = FormatFileSize(size, false, sizeString);
+  UtilityWrite("<td class=\"moz-mime-attachment-size\">");
+  UtilityWrite(NS_ConvertUTF16toUTF8(sizeString).get());
+  UtilityWrite("</td>");
+
   return NS_OK;
 }
 
 nsresult nsMimeHtmlDisplayEmitter::EndAttachment() {
-  if (mSkipAttachment) return NS_OK;
+  if (!mSkipAttachment) {
+    UtilityWrite("</tr>");
+  }
 
   mSkipAttachment = false;  // reset it for next attachment round
-
-  if (BroadCastHeadersAndAttachments()) return NS_OK;
-
-  if (mFormat == nsMimeOutput::nsMimeMessagePrintOutput) UtilityWrite("</tr>");
-
   return NS_OK;
 }
 
@@ -492,10 +502,8 @@ nsresult nsMimeHtmlDisplayEmitter::EndAllAttachments() {
   rv = GetHeaderSink(getter_AddRefs(headerSink));
   if (headerSink) headerSink->OnEndAllAttachments();
 
-  if (mFormat == nsMimeOutput::nsMimeMessagePrintOutput) {
-    UtilityWrite("</table>");
-    UtilityWrite("</div>");
-  }
+  UtilityWrite("</table>");
+  UtilityWrite("</div>");
 
   return rv;
 }
