@@ -678,7 +678,7 @@ define(function(require) {
         ],
         // Collapse extraneous whitespace and make sure unexpected characters aren't there.
         [
-          'Friend "<friend@huhu.com>" \t \t  \u00A0\u00A0\u2003 \u00AD \x20\u200B\x20\u200B\x20 \t \u034F \u2028 \uDB40\uDD01 \t <ws@example.com>',
+          'Friend "<friend@huhu.com>" \t \t  \u00A0\u00A0\u2003 \u00AD \x20\u200B\x20\u200B\x20 \t \u034F \u2028 \uDB40\uDD01 \u2800 \t <ws@example.com>',
           [{ name: "Friend <friend@huhu.com>", email: "ws@example.com" }],
         ],
         // Collapse multiple "special" spaces like NBSP (\u00A0), EM space (\u2003), etc.
@@ -859,7 +859,7 @@ define(function(require) {
         // etc. also when encoded.
         // \u00AD is soft hyphen. \u200B is zero width space.
         // unescape(encodeURIComponent(source)) encodes the JavaScript UTF-16 representation
-        // of the string into UTF-8. Example: encodeURIComponent("ö") returns %C3%B6,
+        // of the string into UTF-8. Example: encodeURIComponent("ö") returns "%C3%B6",
         // unescape("%C3%B6") returns the bytes 0xC3B6 which is the UTF-8 encoding of "ö".
         // See bug 1551746 for other ways.
         [
