@@ -335,10 +335,10 @@ void getMsgHdrForCurrentURL(MimeDisplayOptions* opts, nsIMsgDBHdr** aMsgHdr) {
         msgURI->GetMessageHeader(aMsgHdr);
         if (*aMsgHdr) return;
         nsCString rdfURI;
-        msgURI->GetUri(getter_Copies(rdfURI));
+        msgURI->GetUri(rdfURI);
         if (!rdfURI.IsEmpty()) {
           nsCOMPtr<nsIMsgDBHdr> msgHdr;
-          GetMsgDBHdrFromURI(rdfURI.get(), getter_AddRefs(msgHdr));
+          GetMsgDBHdrFromURI(rdfURI, getter_AddRefs(msgHdr));
           NS_IF_ADDREF(*aMsgHdr = msgHdr);
         }
       }
@@ -441,7 +441,8 @@ MimeObjectClass* mime_find_class(const char* content_type, MimeHeaders* hdrs,
 #endif
 
     if (types_of_classes_to_disallow > 0 &&
-        !PL_strncasecmp(content_type, "text/x-vcard", 12))
+        (!PL_strncasecmp(content_type, "text/vcard", 10) ||
+         !PL_strncasecmp(content_type, "text/x-vcard", 12)))
     /* Use a little hack to prevent some dangerous plugins, which ship
        with Mozilla, to run.
        For the truly user-installed plugins, we rely on the judgement
@@ -657,7 +658,8 @@ MimeObjectClass* mime_find_class(const char* content_type, MimeHeaders* hdrs,
      */
     else if (!PL_strncasecmp(content_type, "image/", 6)) {
       if (imgLoader::SupportImageWithMimeType(
-              content_type, AcceptedMimeTypes::IMAGES_AND_DOCUMENTS))
+              nsDependentCString(content_type),
+              AcceptedMimeTypes::IMAGES_AND_DOCUMENTS))
         clazz = (MimeObjectClass*)&mimeInlineImageClass;
       else
         clazz = (MimeObjectClass*)&mimeExternalObjectClass;
@@ -1526,7 +1528,7 @@ int MimeOptions_write(MimeHeaders* hdrs, MimeDisplayOptions* opt,
     if (opt->state->separator_suppressed_p)
       opt->state->separator_suppressed_p = false;
     else {
-      const char* sep = "<BR><FIELDSET CLASS=\"mimeAttachmentHeader\">";
+      const char* sep = "<BR><FIELDSET CLASS=\"moz-mime-attachment-header\">";
       int lstatus = opt->output_fn(sep, strlen(sep), closure);
       opt->state->separator_suppressed_p = false;
       if (lstatus < 0) return lstatus;
@@ -1536,7 +1538,7 @@ int MimeOptions_write(MimeHeaders* hdrs, MimeDisplayOptions* opt,
       MimeHeaders_convert_header_value(opt, name, false);
 
       if (!name.IsEmpty()) {
-        sep = "<LEGEND CLASS=\"mimeAttachmentHeaderName\">";
+        sep = "<LEGEND CLASS=\"moz-mime-attachment-header-name\">";
         lstatus = opt->output_fn(sep, strlen(sep), closure);
         opt->state->separator_suppressed_p = false;
         if (lstatus < 0) return lstatus;
@@ -1576,7 +1578,8 @@ int MimeObject_write(MimeObject* obj, const char* output, int32_t length,
   if (!obj->output_p) return 0;
 
   // if we're stripping attachments, check if any parent is not being output
-  if (obj->options->format_out == nsMimeOutput::nsMimeMessageAttach) {
+  if (obj->options &&
+      obj->options->format_out == nsMimeOutput::nsMimeMessageAttach) {
     // if true, mime generates a separator in html - we don't want that.
     user_visible_p = false;
 
@@ -1584,7 +1587,7 @@ int MimeObject_write(MimeObject* obj, const char* output, int32_t length,
       if (!parent->output_p) return 0;
     }
   }
-  if (!obj->options->state->first_data_written_p) {
+  if (obj->options && !obj->options->state->first_data_written_p) {
     int status = MimeObject_output_init(obj, 0);
     if (status < 0) return status;
     NS_ASSERTION(obj->options->state->first_data_written_p,
@@ -1689,6 +1692,7 @@ char* mime_get_base_url(const char* url) {
   if (s && !strncmp(s, "?type=application/x-message-display",
                     sizeof("?type=application/x-message-display") - 1)) {
     const char* nextTerm = strchr(s, '&');
+    // strlen(s) cannot be zero, because it matches the above text
     s = nextTerm ? nextTerm : s + strlen(s) - 1;
   }
   // we need to keep the ?number part of the url, or we won't know
