@@ -75,6 +75,23 @@ mime_stream_data::mime_stream_data()
 //
 MimeObject* mime_get_main_object(MimeObject* obj);
 
+// Appends a "filename" parameter with the attachment name to the object url.
+void AppendFilenameParameterToAttachmentDataUrl(
+    const nsMsgAttachmentData* attachmentData, nsCString& url) {
+  url.AppendLiteral("&filename=");
+  nsAutoCString aResult;
+  if (NS_SUCCEEDED(MsgEscapeString(attachmentData->m_realName,
+                                   nsINetUtil::ESCAPE_XALPHAS, aResult))) {
+    url.Append(aResult);
+  } else {
+    url.Append(attachmentData->m_realName);
+  }
+  if (attachmentData->m_realType.EqualsLiteral("message/rfc822") &&
+      !StringEndsWith(url, ".eml"_ns, nsCaseInsensitiveCStringComparator)) {
+    url.AppendLiteral(".eml");
+  }
+}
+
 nsresult MimeGetSize(MimeObject* child, int32_t* size) {
   bool isLeaf = mime_subclass_p(child->clazz, (MimeObjectClass*)&mimeLeafClass);
   bool isContainer =
@@ -137,7 +154,7 @@ nsresult ProcessBodyAsAttachment(MimeObject* obj, nsMsgAttachmentData** data) {
   tmp->m_hasFilename = !tmp->m_realName.IsEmpty();
 
   if (tmp->m_realName.IsEmpty() &&
-      StringBeginsWith(tmp->m_realType, NS_LITERAL_CSTRING("text"),
+      StringBeginsWith(tmp->m_realType, "text"_ns,
                        nsCaseInsensitiveCStringComparator))
     ValidateRealName(tmp, child->headers);
 
@@ -170,7 +187,11 @@ nsresult ProcessBodyAsAttachment(MimeObject* obj, nsMsgAttachmentData** data) {
     } else {
       // This is just a normal MIME part as usual.
       tmpURL = mime_set_url_part(url, id, true);
-      rv = nsMimeNewURI(getter_AddRefs(tmp->m_url), tmpURL, nullptr);
+      nsCString urlString(tmpURL);
+      if (!tmp->m_realName.IsEmpty()) {
+        AppendFilenameParameterToAttachmentDataUrl(tmp, urlString);
+      }
+      rv = nsMimeNewURI(getter_AddRefs(tmp->m_url), urlString.get(), nullptr);
     }
 
     if (!tmp->m_url || NS_FAILED(rv)) {
@@ -217,7 +238,7 @@ void ValidateRealName(nsMsgAttachmentData* aAttach, MimeHeaders* aHdrs) {
 
   // Internal MIME structures need not be named!
   if (aAttach->m_realType.IsEmpty() ||
-      StringBeginsWith(aAttach->m_realType, NS_LITERAL_CSTRING("multipart"),
+      StringBeginsWith(aAttach->m_realType, "multipart"_ns,
                        nsCaseInsensitiveCStringComparator))
     return;
 
@@ -300,7 +321,7 @@ nsresult GenerateAttachmentData(MimeObject* object, const char* aMessageURL,
   tmp->m_isExternalAttachment = isExternalAttachment;
   tmp->m_isExternalLinkAttachment =
       (isExternalAttachment &&
-       StringBeginsWith(urlString, NS_LITERAL_CSTRING("http"),
+       StringBeginsWith(urlString, "http"_ns,
                         nsCaseInsensitiveCStringComparator));
   tmp->m_size = attSize;
   tmp->m_sizeExternalStr = "-1";
@@ -420,17 +441,7 @@ nsresult GenerateAttachmentData(MimeObject* object, const char* aMessageURL,
   }
 
   if (!tmp->m_realName.IsEmpty() && !tmp->m_isExternalAttachment) {
-    urlString.AppendLiteral("&filename=");
-    nsAutoCString aResult;
-    if (NS_SUCCEEDED(MsgEscapeString(tmp->m_realName,
-                                     nsINetUtil::ESCAPE_XALPHAS, aResult)))
-      urlString.Append(aResult);
-    else
-      urlString.Append(tmp->m_realName);
-    if (tmp->m_realType.EqualsLiteral("message/rfc822") &&
-        !StringEndsWith(urlString, NS_LITERAL_CSTRING(".eml"),
-                        nsCaseInsensitiveCStringComparator))
-      urlString.AppendLiteral(".eml");
+    AppendFilenameParameterToAttachmentDataUrl(tmp, urlString);
   } else if (tmp->m_isExternalAttachment) {
     // Allows the JS mime emitter to figure out the part information.
     urlString.AppendLiteral("?part=");
@@ -590,20 +601,18 @@ extern "C" nsresult MimeGetAttachmentList(MimeObject* tobj,
                                 *data);
     if (NS_FAILED(rv)) {
       delete[] * data;  // release data in case of error return.
+      *data = nullptr;
       return rv;
     }
   }
   rv = BuildAttachmentList((MimeObject*)cobj, *data, aMessageURL);
   if (NS_FAILED(rv)) {
     delete[] * data;  // release data in case of error return.
+    *data = nullptr;
   }
   return rv;
 }
 
-extern "C" void MimeFreeAttachmentList(nsMsgAttachmentData* data) {
-  delete[] data;
-}
-
 extern "C" void NotifyEmittersOfAttachmentList(MimeDisplayOptions* opt,
                                                nsMsgAttachmentData* data) {
   int32_t i = 0;
@@ -708,8 +717,7 @@ extern "C" nsresult SetMailCharacterSetToMsgWindow(MimeObject* obj,
             if (msgWindow)
               rv = msgWindow->SetMailCharacterSet(
                   !PL_strcasecmp(aCharacterSet, "us-ascii")
-                      ? static_cast<const nsCString&>(
-                            NS_LITERAL_CSTRING("ISO-8859-1"))
+                      ? static_cast<const nsCString&>("ISO-8859-1"_ns)
                       : static_cast<const nsCString&>(
                             nsDependentCString(aCharacterSet)));
           }
@@ -875,8 +883,8 @@ extern "C" void mime_display_stream_complete(nsMIMESession* stream) {
       nsresult rv = MimeGetAttachmentList(obj, msd->url_name, &attachments);
       if (NS_SUCCEEDED(rv)) {
         NotifyEmittersOfAttachmentList(msd->options, attachments);
-        MimeFreeAttachmentList(attachments);
       }
+      delete[] attachments;
     }
 
     // Release the conversion object - this has to be done after
@@ -1285,7 +1293,11 @@ extern "C" void* mime_bridge_create_display_stream(
         return NULL;
       }
       nsCOMPtr<nsIMsgMessageUrl> msgUrl = do_QueryInterface(uri);
-      if (msgUrl) msgUrl->GetOriginalSpec(&msd->orig_url_name);
+      if (msgUrl) {
+        nsAutoCString orgSpec;
+        msgUrl->GetOriginalSpec(orgSpec);
+        msd->orig_url_name = ToNewCString(orgSpec);
+      }
     }
   }
 
@@ -1505,8 +1517,10 @@ mime_stream_data* GetMSD(MimeDisplayOptions* opt) {
 }
 
 bool NoEmitterProcessing(nsMimeOutputType format_out) {
-  if ((format_out == nsMimeOutput::nsMimeMessageDraftOrTemplate) ||
-      (format_out == nsMimeOutput::nsMimeMessageEditorTemplate))
+  if (format_out == nsMimeOutput::nsMimeMessageDraftOrTemplate ||
+      format_out == nsMimeOutput::nsMimeMessageEditorTemplate ||
+      format_out == nsMimeOutput::nsMimeMessageQuoting ||
+      format_out == nsMimeOutput::nsMimeMessageBodyQuoting)
     return true;
   else
     return false;
