@@ -26,6 +26,7 @@
 #include "nsNetUtil.h"
 #include "mozITXTToHTMLConv.h"
 #include "nsIMsgMailNewsUrl.h"
+#include "nsINntpUrl.h"
 #include "nsIMsgWindow.h"
 #include "nsICategoryManager.h"
 #include "nsMsgUtils.h"
@@ -102,33 +103,35 @@ nsresult bridge_new_new_uri(void* bridgeStream, nsIURI* aURI,
         }
       }
 
-      if ((default_charset) && (override_charset) && (url_name)) {
-        //
-        // set the default charset to be the folder charset if we have one
-        // associated with this url...
+      if (default_charset && override_charset && url_name) {
+        // Check whether we need to auto-detect the charset.
         nsCOMPtr<nsIMsgI18NUrl> i18nUrl(do_QueryInterface(aURI));
         if (i18nUrl) {
-          nsCString charset;
-
-          // check to see if we have a charset override...and if we do, set that
-          // field appropriately too...
-          nsresult rv = i18nUrl->GetCharsetOverRide(getter_Copies(charset));
-          if (NS_SUCCEEDED(rv) && !charset.IsEmpty()) {
+          bool autodetectCharset = false;
+          nsresult rv = i18nUrl->GetAutodetectCharset(&autodetectCharset);
+          if (NS_SUCCEEDED(rv) && autodetectCharset) {
             *override_charset = true;
-            *default_charset = ToNewCString(charset);
+            *default_charset = nullptr;
           } else {
-            i18nUrl->GetFolderCharset(getter_Copies(charset));
-            if (!charset.IsEmpty()) *default_charset = ToNewCString(charset);
+            *override_charset = false;
+            // Special treatment for news: URLs. Get the server default charset.
+            nsCOMPtr<nsINntpUrl> nntpURL(do_QueryInterface(aURI));
+            if (nntpURL) {
+              nsCString charset;
+              rv = nntpURL->GetCharset(charset);
+              if (NS_SUCCEEDED(rv)) {
+                *default_charset = ToNewCString(charset);
+              } else {
+                *default_charset = strdup("UTF-8");
+              }
+            } else {
+              *default_charset = strdup("UTF-8");
+            }
           }
 
-          // if there is no manual override and a folder charset exists
-          // then check if we have a folder level override
+          // If there is no override and a default charset exists,
+          // update the message window.
           if (!(*override_charset) && *default_charset && **default_charset) {
-            bool folderCharsetOverride;
-            rv = i18nUrl->GetFolderCharsetOverride(&folderCharsetOverride);
-            if (NS_SUCCEEDED(rv) && folderCharsetOverride)
-              *override_charset = true;
-
             // notify the default to msgWindow (for the menu check mark)
             // do not set the default in case of nsMimeMessageDraftOrTemplate
             // or nsMimeMessageEditorTemplate because it is already set
@@ -147,21 +150,6 @@ nsresult bridge_new_new_uri(void* bridgeStream, nsIURI* aURI,
                 }
               }
             }
-
-            // if the pref says always override and no manual override then set
-            // the folder charset to override
-            if (!*override_charset) {
-              nsCOMPtr<nsIPrefBranch> pPrefBranch(
-                  do_GetService(NS_PREFSERVICE_CONTRACTID, &rv));
-              if (pPrefBranch) {
-                bool force_override;
-                rv = pPrefBranch->GetBoolPref("mailnews.force_charset_override",
-                                              &force_override);
-                if (NS_SUCCEEDED(rv) && force_override) {
-                  *override_charset = true;
-                }
-              }
-            }
           }
         }
         nsAutoCString urlString;
@@ -720,15 +708,14 @@ nsStreamConverter::SetIdentity(nsIMsgIdentity* aIdentity) {
 }
 
 NS_IMETHODIMP
-nsStreamConverter::SetOriginalMsgURI(const char* originalMsgURI) {
+nsStreamConverter::SetOriginalMsgURI(const nsACString& originalMsgURI) {
   mOriginalMsgURI = originalMsgURI;
   return NS_OK;
 }
 
 NS_IMETHODIMP
-nsStreamConverter::GetOriginalMsgURI(char** result) {
-  if (!result) return NS_ERROR_NULL_POINTER;
-  *result = ToNewCString(mOriginalMsgURI);
+nsStreamConverter::GetOriginalMsgURI(nsACString& result) {
+  result = mOriginalMsgURI;
   return NS_OK;
 }
 
