@@ -8,6 +8,9 @@
  * or bad as expected.
  */
 
+var { MessageInjection } = ChromeUtils.import(
+  "resource://testing-common/mailnews/MessageInjection.jsm"
+);
 var { PromiseTestUtils } = ChromeUtils.import(
   "resource://testing-common/mailnews/PromiseTestUtils.jsm"
 );
@@ -18,18 +21,6 @@ var { SmimeUtils } = ChromeUtils.import(
   "resource://testing-common/mailnews/smimeUtils.jsm"
 );
 
-/* import-globals-from ../../../test/resources/logHelper.js */
-/* import-globals-from ../../../test/resources/asyncTestUtils.js */
-load("../../../resources/logHelper.js");
-load("../../../resources/asyncTestUtils.js");
-
-/* import-globals-from ../../../test/resources/MessageGenerator.jsm */
-/* import-globals-from ../../../test/resources/messageModifier.js */
-/* import-globals-from ../../../test/resources/messageInjection.js */
-load("../../../resources/MessageGenerator.jsm");
-load("../../../resources/messageModifier.js");
-load("../../../resources/messageInjection.js");
-
 let gCertValidityResult = 0;
 
 /**
@@ -65,6 +56,24 @@ function testCertValidity(cert, date) {
   return prom;
 }
 
+add_task(function setupTest() {
+  let messageInjection = new MessageInjection({ mode: "local" });
+  gInbox = messageInjection.getInboxFolder();
+  SmimeUtils.ensureNSS();
+
+  SmimeUtils.loadPEMCertificate(
+    do_get_file(smimeDataDirectory + "TestCA.pem"),
+    Ci.nsIX509Cert.CA_CERT
+  );
+  SmimeUtils.loadCertificateAndKey(
+    do_get_file(smimeDataDirectory + "Alice.p12")
+  );
+  SmimeUtils.loadCertificateAndKey(do_get_file(smimeDataDirectory + "Bob.p12"));
+  SmimeUtils.loadCertificateAndKey(
+    do_get_file(smimeDataDirectory + "Dave.p12")
+  );
+});
+
 add_task(async function verifyTestCertsStillValid() {
   let composeSecure = Cc[
     "@mozilla.org/messengercompose/composesecure;1"
@@ -123,9 +132,6 @@ let smimeHeaderSink = {
     this.resultSigFirst = undefined;
     return this._deferred.promise;
   },
-  maxWantedNesting() {
-    return 2;
-  },
   signedStatus(aNestingLevel, aSignedStatus, aSignerCert) {
     console.log("signedStatus " + aSignedStatus + " level " + aNestingLevel);
     // dump("Signed message\n");
@@ -186,7 +192,7 @@ let smimeHeaderSink = {
       this._deferred.resolve(this._results);
     }
   },
-  QueryInterface: ChromeUtils.generateQI([Ci.nsIMsgSMIMEHeaderSink]),
+  QueryInterface: ChromeUtils.generateQI(["nsIMsgSMIMEHeaderSink"]),
 };
 
 /**
@@ -631,7 +637,7 @@ add_task(async function copy_messages() {
   for (let msg of gMessages) {
     let promiseCopyListener = new PromiseTestUtils.PromiseCopyListener();
 
-    MailServices.copy.CopyFileMessage(
+    MailServices.copy.copyFileMessage(
       do_get_file(smimeDataDirectory + msg.filename),
       gInbox,
       null,
@@ -721,22 +727,3 @@ add_task(async function check_smime_message() {
     hdrIndex++;
   }
 });
-
-function run_test() {
-  gInbox = configure_message_injection({ mode: "local" });
-  SmimeUtils.ensureNSS();
-
-  SmimeUtils.loadPEMCertificate(
-    do_get_file(smimeDataDirectory + "TestCA.pem"),
-    Ci.nsIX509Cert.CA_CERT
-  );
-  SmimeUtils.loadCertificateAndKey(
-    do_get_file(smimeDataDirectory + "Alice.p12")
-  );
-  SmimeUtils.loadCertificateAndKey(do_get_file(smimeDataDirectory + "Bob.p12"));
-  SmimeUtils.loadCertificateAndKey(
-    do_get_file(smimeDataDirectory + "Dave.p12")
-  );
-
-  run_next_test();
-}
