@@ -15,9 +15,7 @@
 #include "MailNewsTypes.h"
 #include "prprf.h"
 #include "prsystem.h"
-#include "nsIArray.h"
 #include "nsTArray.h"
-#include "nsArrayUtils.h"
 #include "nsINntpService.h"
 #include "nsIMsgFilterService.h"
 #include "nsCOMPtr.h"
@@ -26,7 +24,6 @@
 #include "nsMsgUtils.h"
 #include "nsNewsUtils.h"
 
-#include "nsCOMPtr.h"
 #include "nsIMsgIncomingServer.h"
 #include "nsINntpIncomingServer.h"
 #include "nsINewsDatabase.h"
@@ -42,18 +39,17 @@
 #include "nsNetCID.h"
 #include "nsINntpUrl.h"
 
-#include "nsArrayEnumerator.h"
 #include "nsNewsDownloader.h"
 #include "nsIStringBundle.h"
 #include "nsMsgI18N.h"
 #include "nsNativeCharsetUtils.h"
 
 #include "nsIMsgFolderNotificationService.h"
-#include "nsIMutableArray.h"
 #include "nsILoginInfo.h"
 #include "nsILoginManager.h"
 #include "nsEmbedCID.h"
 #include "mozilla/Services.h"
+#include "mozilla/SlicedInputStream.h"
 #include "nsIInputStream.h"
 #include "nsMemory.h"
 #include "nsIURIMutator.h"
@@ -152,6 +148,11 @@ nsMsgNewsFolder::AddNewsgroup(const nsACString& name, const nsACString& setStr,
   nsCOMPtr<nsIMsgNewsFolder> newsFolder(do_QueryInterface(folder, &rv));
   NS_ENSURE_SUCCESS(rv, rv);
 
+  // Ensure any containing .sdb dir exists.
+  nsCOMPtr<nsIFile> path;
+  rv = CreateDirectoryForFolder(getter_AddRefs(path));
+  NS_ENSURE_SUCCESS(rv, rv);
+
   // cache this for when we open the db
   rv = newsFolder->SetReadSetFromStr(setStr);
 
@@ -191,7 +192,7 @@ nsresult nsMsgNewsFolder::AddDirectorySeparator(nsIFile* path) {
 }
 
 NS_IMETHODIMP
-nsMsgNewsFolder::GetSubFolders(nsISimpleEnumerator** aResult) {
+nsMsgNewsFolder::GetSubFolders(nsTArray<RefPtr<nsIMsgFolder>>& folders) {
   if (!mInitialized) {
     // do this first, so we make sure to do it, even on failure.
     // see bug #70494
@@ -210,9 +211,7 @@ nsMsgNewsFolder::GetSubFolders(nsISimpleEnumerator** aResult) {
     (void)UpdateSummaryTotals(false);
   }
 
-  return aResult ? NS_NewArrayEnumerator(aResult, mSubFolders,
-                                         NS_GET_IID(nsIMsgFolder))
-                 : NS_ERROR_NULL_POINTER;
+  return nsMsgDBFolder::GetSubFolders(folders);
 }
 
 // Makes sure the database is open and exists.  If the database is valid then
@@ -346,16 +345,6 @@ nsMsgNewsFolder::GetCanCompact(bool* aResult) {
   return NS_OK;
 }
 
-NS_IMETHODIMP
-nsMsgNewsFolder::GetMessages(nsISimpleEnumerator** result) {
-  nsresult rv = GetDatabase();
-  *result = nullptr;
-
-  if (NS_SUCCEEDED(rv)) rv = mDatabase->EnumerateMessages(result);
-
-  return rv;
-}
-
 NS_IMETHODIMP nsMsgNewsFolder::GetFolderURL(nsACString& aUrl) {
   nsCString hostName;
   nsresult rv = GetHostname(hostName);
@@ -395,11 +384,11 @@ NS_IMETHODIMP nsMsgNewsFolder::SetNewsrcHasChanged(bool newsrcHasChanged) {
   return nntpServer->SetNewsrcHasChanged(newsrcHasChanged);
 }
 
-nsresult nsMsgNewsFolder::CreateChildFromURI(const nsCString& uri,
+nsresult nsMsgNewsFolder::CreateChildFromURI(const nsACString& uri,
                                              nsIMsgFolder** folder) {
   nsMsgNewsFolder* newFolder = new nsMsgNewsFolder;
   NS_ADDREF(*folder = newFolder);
-  newFolder->Init(uri.get());
+  newFolder->Init(uri);
   return NS_OK;
 }
 
@@ -409,10 +398,6 @@ NS_IMETHODIMP nsMsgNewsFolder::CreateSubfolder(const nsAString& newsgroupName,
   if (newsgroupName.IsEmpty()) return NS_MSG_ERROR_INVALID_FOLDER_NAME;
 
   nsCOMPtr<nsIMsgFolder> child;
-  // Create an empty database for this mail folder, set its name from the user
-  nsCOMPtr<nsIMsgDatabase> newsDBFactory;
-  nsCOMPtr<nsIMsgDatabase> newsDB;
-
   // Now let's create the actual new folder
   rv = AddNewsgroup(NS_ConvertUTF16toUTF8(newsgroupName), EmptyCString(),
                     getter_AddRefs(child));
@@ -426,12 +411,14 @@ NS_IMETHODIMP nsMsgNewsFolder::CreateSubfolder(const nsAString& newsgroupName,
     rv = GetNntpServer(getter_AddRefs(nntpServer));
     if (NS_FAILED(rv)) return rv;
 
-    nsAutoCString dataCharset;
-    rv = nntpServer->GetCharset(dataCharset);
-    if (NS_FAILED(rv)) return rv;
+    nsCOMPtr<nsIDBFolderInfo> folderInfo;
+    nsCOMPtr<nsIMsgDatabase> db;
+    // Used to init some folder status of child.
+    rv = child->GetDBFolderInfoAndDB(getter_AddRefs(folderInfo),
+                                     getter_AddRefs(db));
+    NS_ENSURE_SUCCESS(rv, rv);
 
-    child->SetCharset(dataCharset);
-    NotifyItemAdded(child);
+    NotifyFolderAdded(child);
     nsCOMPtr<nsIMsgFolderNotificationService> notifier(
         do_GetService(NS_MSGNOTIFICATIONSERVICE_CONTRACTID));
     if (notifier) notifier->NotifyFolderAdded(child);
@@ -693,25 +680,18 @@ NS_IMETHODIMP nsMsgNewsFolder::GetSizeOnDisk(int64_t* size) {
 }
 
 NS_IMETHODIMP
-nsMsgNewsFolder::DeleteMessages(nsIArray* messages, nsIMsgWindow* aMsgWindow,
-                                bool deleteStorage, bool isMove,
+nsMsgNewsFolder::DeleteMessages(nsTArray<RefPtr<nsIMsgDBHdr>> const& msgHdrs,
+                                nsIMsgWindow* aMsgWindow, bool deleteStorage,
+                                bool isMove,
                                 nsIMsgCopyServiceListener* listener,
                                 bool allowUndo) {
   nsresult rv = NS_OK;
-
-  NS_ENSURE_ARG_POINTER(messages);
   NS_ENSURE_ARG_POINTER(aMsgWindow);
 
-  // Stopgap. Build a parallel array of message headers while we complete
-  // removal of nsIArray usage (Bug 1583030).
-
-  nsTArray<RefPtr<nsIMsgDBHdr>> msgHeaders;
-  MsgHdrsToTArray(messages, msgHeaders);
-
   if (!isMove) {
     nsCOMPtr<nsIMsgFolderNotificationService> notifier(
         do_GetService(NS_MSGNOTIFICATIONSERVICE_CONTRACTID));
-    if (notifier) notifier->NotifyMsgsDeleted(msgHeaders);
+    if (notifier) notifier->NotifyMsgsDeleted(msgHdrs);
   }
 
   rv = GetDatabase();
@@ -719,7 +699,7 @@ nsMsgNewsFolder::DeleteMessages(nsIArray* messages, nsIMsgWindow* aMsgWindow,
 
   rv = EnableNotifications(allMessageCountNotifications, false);
   if (NS_SUCCEEDED(rv)) {
-    for (auto msgHdr : msgHeaders) {
+    for (auto msgHdr : msgHdrs) {
       rv = mDatabase->DeleteHeader(msgHdr, nullptr, true, true);
       if (NS_FAILED(rv)) {
         break;
@@ -784,9 +764,10 @@ NS_IMETHODIMP nsMsgNewsFolder::CancelMessage(nsIMsgDBHdr* msgHdr,
   rv = GetUriForMsg(msgHdr, messageURI);
   NS_ENSURE_SUCCESS(rv, rv);
 
-  return nntpService->CancelMessage(cancelURL.get(), messageURI.get(),
+  nsCOMPtr<nsIURI> resultUri;
+  return nntpService->CancelMessage(cancelURL, messageURI,
                                     nullptr /* consumer */, nullptr, aMsgWindow,
-                                    nullptr);
+                                    getter_AddRefs(resultUri));
 }
 
 NS_IMETHODIMP nsMsgNewsFolder::GetNewMessages(nsIMsgWindow* aMsgWindow,
@@ -805,11 +786,15 @@ nsresult nsMsgNewsFolder::GetNewsMessages(nsIMsgWindow* aMsgWindow,
 
   bool isNewsServer = false;
   rv = GetIsServer(&isNewsServer);
-  if (NS_FAILED(rv)) return rv;
+  NS_ENSURE_SUCCESS(rv, rv);
 
-  if (isNewsServer)
-    // get new messages only works on a newsgroup, not a news server
-    return NS_OK;
+  if (isNewsServer) {
+    nsCOMPtr<nsIMsgIncomingServer> server;
+    rv = GetServer(getter_AddRefs(server));
+    NS_ENSURE_SUCCESS(rv, rv);
+
+    return server->PerformExpand(aMsgWindow);
+  }
 
   nsCOMPtr<nsINntpService> nntpService =
       do_GetService(NS_NNTPSERVICE_CONTRACTID, &rv);
@@ -817,11 +802,11 @@ nsresult nsMsgNewsFolder::GetNewsMessages(nsIMsgWindow* aMsgWindow,
 
   nsCOMPtr<nsINntpIncomingServer> nntpServer;
   rv = GetNntpServer(getter_AddRefs(nntpServer));
-  if (NS_FAILED(rv)) return rv;
+  NS_ENSURE_SUCCESS(rv, rv);
 
   nsCOMPtr<nsIURI> resultUri;
-  rv = nntpService->GetNewNews(nntpServer, mURI.get(), aGetOld, this,
-                               aMsgWindow, getter_AddRefs(resultUri));
+  rv = nntpService->GetNewNews(nntpServer, mURI, aGetOld, this, aMsgWindow,
+                               getter_AddRefs(resultUri));
   if (aUrlListener && NS_SUCCEEDED(rv) && resultUri) {
     nsCOMPtr<nsIMsgMailNewsUrl> msgUrl(do_QueryInterface(resultUri));
     if (msgUrl) msgUrl->RegisterListener(aUrlListener);
@@ -987,10 +972,12 @@ nsresult nsMsgNewsFolder::CreateNewsgroupUrlForSignon(const char* ref,
 
   nsCOMPtr<nsIURL> url;
   if (singleSignon) {
-    nsCString serverURI;
-    rv = server->GetServerURI(serverURI);
+    // Do not include username in the url when interacting with LoginManager.
+    nsCString serverURI = "news://"_ns;
+    nsCString hostName;
+    rv = server->GetHostName(hostName);
     NS_ENSURE_SUCCESS(rv, rv);
-
+    serverURI.Append(hostName);
     rv = NS_MutateURI(NS_STANDARDURLMUTATOR_CONTRACTID)
              .SetSpec(serverURI)
              .Finalize(url);
@@ -1233,7 +1220,7 @@ NS_IMETHODIMP nsMsgNewsFolder::MoveFolder(nsIMsgFolder* aNewsgroupToMove,
   }
 
   // move NewsgroupToMove to new index and set new sort order
-  NotifyItemRemoved(aNewsgroupToMove);
+  NotifyFolderRemoved(aNewsgroupToMove);
 
   if (indexNewsgroupToMove != indexRefNewsgroup) {
     nsCOMPtr<nsIMsgFolder> newsgroup = mSubFolders[indexNewsgroupToMove];
@@ -1247,7 +1234,7 @@ NS_IMETHODIMP nsMsgNewsFolder::MoveFolder(nsIMsgFolder* aNewsgroupToMove,
   for (uint32_t i = indexMin; i <= indexMax; i++)
     mSubFolders[i]->SetSortOrder(kNewsSortOffset + i);
 
-  NotifyItemAdded(aNewsgroupToMove);
+  NotifyFolderAdded(aNewsgroupToMove);
 
   // write changes back to file
   nsCOMPtr<nsINntpIncomingServer> nntpServer;
@@ -1264,7 +1251,16 @@ NS_IMETHODIMP nsMsgNewsFolder::MoveFolder(nsIMsgFolder* aNewsgroupToMove,
 }
 
 nsresult nsMsgNewsFolder::CreateBaseMessageURI(const nsACString& aURI) {
-  return nsCreateNewsBaseMessageURI(nsCString(aURI).get(), mBaseMessageURI);
+  return nsCreateNewsBaseMessageURI(aURI, mBaseMessageURI);
+}
+
+NS_IMETHODIMP nsMsgNewsFolder::GetCharset(nsACString& charset) {
+  nsCOMPtr<nsIMsgIncomingServer> server;
+  nsresult rv = GetServer(getter_AddRefs(server));
+  NS_ENSURE_SUCCESS(rv, rv);
+  nsCOMPtr<nsINntpIncomingServer> nserver(do_QueryInterface(server));
+  NS_ENSURE_TRUE(nserver, NS_ERROR_NULL_POINTER);
+  return nserver->GetCharset(charset);
 }
 
 NS_IMETHODIMP
@@ -1292,7 +1288,7 @@ nsMsgNewsFolder::GetNewsrcLine(nsACString& newsrcLine) {
 
 NS_IMETHODIMP nsMsgNewsFolder::SetReadSetFromStr(const nsACString& newsrcLine) {
   delete mReadSet;
-  mReadSet = nsMsgKeySet::Create(nsCString(newsrcLine).get());
+  mReadSet = nsMsgKeySet::Create(PromiseFlatCString(newsrcLine).get());
   NS_ENSURE_TRUE(mReadSet, NS_ERROR_OUT_OF_MEMORY);
 
   // Now that mReadSet is recreated, make sure it's stored in the db as well.
@@ -1381,7 +1377,8 @@ NS_IMETHODIMP nsMsgNewsFolder::RemoveMessage(nsMsgKey key) {
   return mDatabase->DeleteMessage(key, nullptr, false);
 }
 
-NS_IMETHODIMP nsMsgNewsFolder::RemoveMessages(nsTArray<nsMsgKey>& aMsgKeys) {
+NS_IMETHODIMP nsMsgNewsFolder::RemoveMessages(
+    const nsTArray<nsMsgKey>& aMsgKeys) {
   nsresult rv = GetDatabase();
   NS_ENSURE_SUCCESS(rv,
                     rv);  // if GetDatabase succeeds, mDatabase will be non-null
@@ -1392,7 +1389,7 @@ NS_IMETHODIMP nsMsgNewsFolder::RemoveMessages(nsTArray<nsMsgKey>& aMsgKeys) {
 
   if (notifier) {
     nsTArray<RefPtr<nsIMsgDBHdr>> msgHdrs;
-    rv = MsgGetHeadersFromKeys2(mDatabase, aMsgKeys, msgHdrs);
+    rv = MsgGetHeadersFromKeys(mDatabase, aMsgKeys, msgHdrs);
     NS_ENSURE_SUCCESS(rv, rv);
     notifier->NotifyMsgsDeleted(msgHdrs);
   }
@@ -1429,20 +1426,18 @@ NS_IMETHODIMP nsMsgNewsFolder::DownloadAllForOffline(nsIUrlListener* listener,
 
   // build up message keys.
   if (mDatabase) {
-    nsCOMPtr<nsISimpleEnumerator> enumerator;
+    nsCOMPtr<nsIMsgEnumerator> enumerator;
     rv = mDatabase->EnumerateMessages(getter_AddRefs(enumerator));
     if (NS_SUCCEEDED(rv) && enumerator) {
       bool hasMore;
       while (NS_SUCCEEDED(rv = enumerator->HasMoreElements(&hasMore)) &&
              hasMore) {
-        nsCOMPtr<nsISupports> supports;
-        rv = enumerator->GetNext(getter_AddRefs(supports));
-        NS_ASSERTION(NS_SUCCEEDED(rv), "nsMsgDBEnumerator broken");
-        nsCOMPtr<nsIMsgDBHdr> pHeader = do_QueryInterface(supports);
-        if (pHeader && NS_SUCCEEDED(rv)) {
+        nsCOMPtr<nsIMsgDBHdr> header;
+        rv = enumerator->GetNext(getter_AddRefs(header));
+        if (header && NS_SUCCEEDED(rv)) {
           bool shouldStoreMsgOffline = false;
           nsMsgKey msgKey;
-          pHeader->GetMessageKey(&msgKey);
+          header->GetMessageKey(&msgKey);
           MsgFitsDownloadCriteria(msgKey, &shouldStoreMsgOffline);
           if (shouldStoreMsgOffline) srcKeyArray.AppendElement(msgKey);
         }
@@ -1458,20 +1453,15 @@ NS_IMETHODIMP nsMsgNewsFolder::DownloadAllForOffline(nsIUrlListener* listener,
 }
 
 NS_IMETHODIMP nsMsgNewsFolder::DownloadMessagesForOffline(
-    nsIArray* messages, nsIMsgWindow* window) {
-  nsTArray<nsMsgKey> srcKeyArray;
+    nsTArray<RefPtr<nsIMsgDBHdr>> const& messages, nsIMsgWindow* window) {
+  nsresult rv;
   SetSaveArticleOffline(
       true);  // ### TODO need to clear this when we've finished
-  uint32_t count = 0;
-  uint32_t i;
-  nsresult rv = messages->GetLength(&count);
-  NS_ENSURE_SUCCESS(rv, rv);
-
   // build up message keys.
-  for (i = 0; i < count; i++) {
+  nsTArray<nsMsgKey> srcKeyArray(messages.Length());
+  for (nsIMsgDBHdr* hdr : messages) {
     nsMsgKey key;
-    nsCOMPtr<nsIMsgDBHdr> msgDBHdr = do_QueryElementAt(messages, i, &rv);
-    if (msgDBHdr) rv = msgDBHdr->GetMessageKey(&key);
+    rv = hdr->GetMessageKey(&key);
     if (NS_SUCCEEDED(rv)) srcKeyArray.AppendElement(key);
   }
   RefPtr<DownloadNewsArticlesToOfflineStore> downloadState =
@@ -1483,6 +1473,25 @@ NS_IMETHODIMP nsMsgNewsFolder::DownloadMessagesForOffline(
   return rv;
 }
 
+NS_IMETHODIMP nsMsgNewsFolder::GetLocalMsgStream(nsIMsgDBHdr* hdr,
+                                                 nsIInputStream** stream) {
+  nsMsgKey key;
+  hdr->GetMessageKey(&key);
+
+  uint64_t offset = 0;
+  uint32_t size = 0;
+  nsCOMPtr<nsIInputStream> rawStream;
+  nsresult rv =
+      GetOfflineFileStream(key, &offset, &size, getter_AddRefs(rawStream));
+  NS_ENSURE_SUCCESS(rv, rv);
+
+  RefPtr<mozilla::SlicedInputStream> slicedStream =
+      new mozilla::SlicedInputStream(rawStream.forget(), offset,
+                                     uint64_t(size));
+  slicedStream.forget(stream);
+  return NS_OK;
+}
+
 // line does not have a line terminator (e.g., CR or CRLF)
 NS_IMETHODIMP nsMsgNewsFolder::NotifyDownloadedLine(const char* line,
                                                     nsMsgKey keyOfArticle) {
@@ -1508,6 +1517,8 @@ NS_IMETHODIMP nsMsgNewsFolder::NotifyDownloadedLine(const char* line,
     } else {
       uint32_t count = 0;
       rv = m_tempMessageStream->Write(line, strlen(line), &count);
+      NS_ENSURE_SUCCESS(rv, rv);
+      m_tempMessageStreamBytesWritten += count;
     }
   }
 
