@@ -38,7 +38,6 @@
 #include "nsIMsgMailNewsUrl.h"
 #include "nsIMsgMailSession.h"
 #include "nsISupportsPrimitives.h"
-#include "nsArrayUtils.h"
 #include "nsIStreamListener.h"
 #include "nsIInputStream.h"
 #include "nsIURIMutator.h"
@@ -57,17 +56,14 @@
   "mail.root.nntp"  // old - for backward compatibility only
 #define PREF_MAIL_ROOT_NNTP_REL "mail.root.nntp-rel"
 
-nsNntpService::nsNntpService() {
-  mPrintingOperation = false;
-  mOpenAttachmentOperation = false;
-}
+nsNntpService::nsNntpService() { mOpenAttachmentOperation = false; }
 
 nsNntpService::~nsNntpService() {
   // do nothing
 }
 
 NS_IMPL_ISUPPORTS(nsNntpService, nsINntpService, nsIMsgMessageService,
-                  nsIProtocolHandler, nsIMsgProtocolInfo, nsICommandLineHandler,
+                  nsIProtocolHandler, nsIMsgProtocolInfo,
                   nsIMsgMessageFetchPartService, nsIContentHandler)
 
 ////////////////////////////////////////////////////////////////////////////////////////
@@ -75,16 +71,15 @@ NS_IMPL_ISUPPORTS(nsNntpService, nsINntpService, nsIMsgMessageService,
 ////////////////////////////////////////////////////////////////////////////////////////
 
 NS_IMETHODIMP
-nsNntpService::SaveMessageToDisk(const char* aMessageURI, nsIFile* aFile,
+nsNntpService::SaveMessageToDisk(const nsACString& aMessageURI, nsIFile* aFile,
                                  bool aAddDummyEnvelope,
                                  nsIUrlListener* aUrlListener, nsIURI** aURL,
                                  bool canonicalLineEnding,
                                  nsIMsgWindow* aMsgWindow) {
   nsresult rv = NS_OK;
-  NS_ENSURE_ARG_POINTER(aMessageURI);
 
   // double check it is a news-message:/ uri
-  if (PL_strncmp(aMessageURI, kNewsMessageRootURI, kNewsMessageRootURILen)) {
+  if (!StringBeginsWith(aMessageURI, kNewsMessageRootURI ""_ns)) {
     rv = NS_ERROR_UNEXPECTED;
     NS_ENSURE_SUCCESS(rv, rv);
   }
@@ -99,8 +94,8 @@ nsNntpService::SaveMessageToDisk(const char* aMessageURI, nsIFile* aFile,
   NS_ENSURE_SUCCESS(rv, rv);
 
   nsCOMPtr<nsIURI> url;
-  rv = ConstructNntpUrl(messageIdURL.get(), aUrlListener, aMsgWindow,
-                        aMessageURI, nsINntpUrl::ActionSaveMessageToDisk,
+  rv = ConstructNntpUrl(messageIdURL, aUrlListener, aMsgWindow, aMessageURI,
+                        nsINntpUrl::ActionSaveMessageToDisk,
                         getter_AddRefs(url));
   NS_ENSURE_SUCCESS(rv, rv);
 
@@ -131,7 +126,7 @@ nsNntpService::SaveMessageToDisk(const char* aMessageURI, nsIFile* aFile,
 
     rv = DisplayMessage(aMessageURI, saveAsListener,
                         /* nsIMsgWindow *aMsgWindow */ nullptr, aUrlListener,
-                        nullptr /*aCharsetOverride */, aURL);
+                        false /*aAutodetectCharset */, aURL);
   }
   return rv;
 }
@@ -184,13 +179,12 @@ nsresult nsNntpService::CreateMessageIDURL(nsIMsgFolder* folder, nsMsgKey key,
 }
 
 NS_IMETHODIMP
-nsNntpService::DisplayMessage(const char* aMessageURI,
+nsNntpService::DisplayMessage(const nsACString& aMessageURI,
                               nsISupports* aDisplayConsumer,
                               nsIMsgWindow* aMsgWindow,
                               nsIUrlListener* aUrlListener,
-                              const char* aCharsetOverride, nsIURI** aURL) {
+                              bool aAutodetectCharset, nsIURI** aURL) {
   nsresult rv = NS_OK;
-  NS_ENSURE_ARG_POINTER(aMessageURI);
 
   nsCOMPtr<nsIMsgFolder> folder;
   nsMsgKey key = nsMsgKey_None;
@@ -209,22 +203,18 @@ nsNntpService::DisplayMessage(const char* aMessageURI,
   rv = CreateMessageIDURL(folder, key, getter_Copies(urlStr));
   NS_ENSURE_SUCCESS(rv, rv);
 
-  // rhp: If we are displaying this message for the purposes of printing, append
-  // the magic operand.
-  if (mPrintingOperation) urlStr.AppendLiteral("?header=print");
-
   nsNewsAction action = nsINntpUrl::ActionFetchArticle;
   if (mOpenAttachmentOperation) action = nsINntpUrl::ActionFetchPart;
 
   nsCOMPtr<nsIURI> url;
-  rv = ConstructNntpUrl(urlStr.get(), aUrlListener, aMsgWindow, aMessageURI,
-                        action, getter_AddRefs(url));
+  rv = ConstructNntpUrl(urlStr, aUrlListener, aMsgWindow, aMessageURI, action,
+                        getter_AddRefs(url));
   NS_ENSURE_SUCCESS(rv, rv);
 
   nsCOMPtr<nsIMsgI18NUrl> i18nurl = do_QueryInterface(url, &rv);
   NS_ENSURE_SUCCESS(rv, rv);
 
-  i18nurl->SetCharsetOverRide(aCharsetOverride);
+  i18nurl->SetAutodetectCharset(aAutodetectCharset);
 
   bool shouldStoreMsgOffline = false;
 
@@ -320,7 +310,7 @@ nsresult nsNntpService::GetMessageFromUrl(nsIURI* aUrl,
       }
       nsCOMPtr<nsILoadInfo> loadInfo = new mozilla::net::LoadInfo(
           nsContentUtils::GetSystemPrincipal(), nullptr, nullptr,
-          nsILoadInfo::SEC_ALLOW_CROSS_ORIGIN_DATA_IS_NULL,
+          nsILoadInfo::SEC_ALLOW_CROSS_ORIGIN_SEC_CONTEXT_IS_NULL,
           nsIContentPolicy::TYPE_OTHER);
       rv = NewChannel(aUrl, loadInfo, getter_AddRefs(aChannel));
       if (NS_FAILED(rv)) return rv;
@@ -360,9 +350,9 @@ nsNntpService::FetchMessage(nsIMsgFolder* folder, nsMsgKey key,
   NS_ENSURE_SUCCESS(rv, rv);
 
   nsCOMPtr<nsIURI> url;
-  rv = ConstructNntpUrl(messageIdURL.get(), aUrlListener, aMsgWindow,
-                        originalMessageUri.get(),
-                        nsINntpUrl::ActionFetchArticle, getter_AddRefs(url));
+  rv = ConstructNntpUrl(messageIdURL, aUrlListener, aMsgWindow,
+                        originalMessageUri, nsINntpUrl::ActionFetchArticle,
+                        getter_AddRefs(url));
   NS_ENSURE_SUCCESS(rv, rv);
 
   rv = RunNewsUrl(url, aMsgWindow, aConsumer);
@@ -373,7 +363,7 @@ nsNntpService::FetchMessage(nsIMsgFolder* folder, nsMsgKey key,
 }
 
 NS_IMETHODIMP nsNntpService::FetchMimePart(
-    nsIURI* aURI, const char* aMessageURI, nsISupports* aDisplayConsumer,
+    nsIURI* aURI, const nsACString& aMessageURI, nsISupports* aDisplayConsumer,
     nsIMsgWindow* aMsgWindow, nsIUrlListener* aUrlListener, nsIURI** aURL) {
   nsresult rv;
   nsCOMPtr<nsIMsgMailNewsUrl> msgUrl(do_QueryInterface(aURI, &rv));
@@ -392,22 +382,21 @@ NS_IMETHODIMP nsNntpService::FetchMimePart(
   //      nsAutoCString spec;
   //      rv = aURI->GetSpec(spec);
   //      NS_ENSURE_SUCCESS(rv, rv);
-  //      msgMessageUrl->SetOriginalSpec(spec.get());
+  //      msgMessageUrl->SetOriginalSpec(spec);
   //    }
   return RunNewsUrl(msgUrl, aMsgWindow, aDisplayConsumer);
 }
 
-NS_IMETHODIMP nsNntpService::OpenAttachment(
-    const char* aContentType, const char* aFileName, const char* aUrl,
-    const char* aMessageUri, nsISupports* aDisplayConsumer,
-    nsIMsgWindow* aMsgWindow, nsIUrlListener* aUrlListener) {
-  NS_ENSURE_ARG_POINTER(aUrl);
-  NS_ENSURE_ARG_POINTER(aFileName);
-
+NS_IMETHODIMP nsNntpService::OpenAttachment(const nsACString& aContentType,
+                                            const nsACString& aFileName,
+                                            const nsACString& aUrl,
+                                            const nsACString& aMessageUri,
+                                            nsISupports* aDisplayConsumer,
+                                            nsIMsgWindow* aMsgWindow,
+                                            nsIUrlListener* aUrlListener) {
   nsCOMPtr<nsIURI> url;
   nsresult rv = NS_OK;
-  nsAutoCString newsUrl;
-  newsUrl = aUrl;
+  nsAutoCString newsUrl(aUrl);
   newsUrl += "&type=";
   newsUrl += aContentType;
   newsUrl += "&filename=";
@@ -420,12 +409,12 @@ NS_IMETHODIMP nsNntpService::OpenAttachment(
     NS_ENSURE_SUCCESS(rv, rv);
 
     msgUrl->SetMsgWindow(aMsgWindow);
-    msgUrl->SetFileNameInternal(nsDependentCString(aFileName));
+    msgUrl->SetFileNameInternal(aFileName);
     // this code isn't ready yet, but it helps getting opening attachments
     // while offline working
     //   nsCOMPtr<nsIMsgMessageUrl> msgMessageUrl = do_QueryInterface(url);
     //    if (msgMessageUrl)
-    //      msgMessageUrl->SetOriginalSpec(newsUrl.get());
+    //      msgMessageUrl->SetOriginalSpec(newsUrl);
     // set up the url listener
     if (aUrlListener) msgUrl->RegisterListener(aUrlListener);
 
@@ -444,15 +433,13 @@ NS_IMETHODIMP nsNntpService::OpenAttachment(
   return NS_OK;
 }
 
-NS_IMETHODIMP nsNntpService::GetUrlForUri(const char* aMessageURI,
-                                          nsIURI** aURL,
-                                          nsIMsgWindow* aMsgWindow) {
+NS_IMETHODIMP nsNntpService::GetUrlForUri(const nsACString& aMessageURI,
+                                          nsIMsgWindow* aMsgWindow,
+                                          nsIURI** aURL) {
   nsresult rv = NS_OK;
 
-  NS_ENSURE_ARG_POINTER(aMessageURI);
-
   // double check that it is a news-message:/ uri
-  if (PL_strncmp(aMessageURI, kNewsMessageRootURI, kNewsMessageRootURILen)) {
+  if (!StringBeginsWith(aMessageURI, kNewsMessageRootURI ""_ns)) {
     rv = NS_ERROR_UNEXPECTED;
     NS_ENSURE_SUCCESS(rv, rv);
   }
@@ -467,7 +454,7 @@ NS_IMETHODIMP nsNntpService::GetUrlForUri(const char* aMessageURI,
   NS_ENSURE_SUCCESS(rv, rv);
 
   // this is only called by view message source
-  rv = ConstructNntpUrl(messageIdURL.get(), nullptr, aMsgWindow, aMessageURI,
+  rv = ConstructNntpUrl(messageIdURL, nullptr, aMsgWindow, aMessageURI,
                         nsINntpUrl::ActionFetchArticle, aURL);
   NS_ENSURE_SUCCESS(rv, rv);
   if (folder && *aURL) {
@@ -482,7 +469,7 @@ NS_IMETHODIMP nsNntpService::GetUrlForUri(const char* aMessageURI,
 }
 
 NS_IMETHODIMP
-nsNntpService::DecomposeNewsURI(const char* uri, nsIMsgFolder** folder,
+nsNntpService::DecomposeNewsURI(const nsACString& uri, nsIMsgFolder** folder,
                                 nsMsgKey* aMsgKey) {
   nsresult rv;
 
@@ -491,10 +478,9 @@ nsNntpService::DecomposeNewsURI(const char* uri, nsIMsgFolder** folder,
   return rv;
 }
 
-nsresult nsNntpService::DecomposeNewsMessageURI(const char* aMessageURI,
+nsresult nsNntpService::DecomposeNewsMessageURI(const nsACString& aMessageURI,
                                                 nsIMsgFolder** aFolder,
                                                 nsMsgKey* aMsgKey) {
-  NS_ENSURE_ARG_POINTER(aMessageURI);
   NS_ENSURE_ARG_POINTER(aFolder);
   NS_ENSURE_ARG_POINTER(aMsgKey);
 
@@ -506,7 +492,7 @@ nsresult nsNntpService::DecomposeNewsMessageURI(const char* aMessageURI,
   NS_ENSURE_SUCCESS(rv, rv);
   nsCOMPtr<nsINntpUrl> nntpUrl = do_QueryInterface(mailnewsurl, &rv);
   NS_ENSURE_SUCCESS(rv, rv);
-  rv = mailnewsurl->SetSpecInternal(nsDependentCString(aMessageURI));
+  rv = mailnewsurl->SetSpecInternal(aMessageURI);
   NS_ENSURE_SUCCESS(rv, rv);
 
   // Get the group name and key from the url
@@ -526,13 +512,12 @@ nsresult nsNntpService::DecomposeNewsMessageURI(const char* aMessageURI,
   return mailnewsurl->GetFolder(aFolder);
 }
 
-nsresult nsNntpService::GetFolderFromUri(const char* aUri,
+nsresult nsNntpService::GetFolderFromUri(const nsACString& aUri,
                                          nsIMsgFolder** aFolder) {
-  NS_ENSURE_ARG_POINTER(aUri);
   NS_ENSURE_ARG_POINTER(aFolder);
 
   nsCOMPtr<nsIURI> uri;
-  nsresult rv = NS_NewURI(getter_AddRefs(uri), nsDependentCString(aUri));
+  nsresult rv = NS_NewURI(getter_AddRefs(uri), aUri);
   NS_ENSURE_SUCCESS(rv, rv);
 
   nsAutoCString path;
@@ -544,7 +529,7 @@ nsresult nsNntpService::GetFolderFromUri(const char* aUri,
   NS_ENSURE_SUCCESS(rv, rv);
 
   nsCOMPtr<nsIMsgIncomingServer> server;
-  rv = accountManager->FindServerByURI(uri, false, getter_AddRefs(server));
+  rv = accountManager->FindServerByURI(uri, getter_AddRefs(server));
   NS_ENSURE_SUCCESS(rv, rv);
 
   nsCOMPtr<nsIMsgFolder> rootFolder;
@@ -576,11 +561,10 @@ nsresult nsNntpService::GetFolderFromUri(const char* aUri,
 }
 
 NS_IMETHODIMP
-nsNntpService::CopyMessage(const char* aSrcMessageURI,
+nsNntpService::CopyMessage(const nsACString& aSrcMessageURI,
                            nsIStreamListener* aMailboxCopyHandler,
                            bool moveMessage, nsIUrlListener* aUrlListener,
                            nsIMsgWindow* aMsgWindow, nsIURI** aURL) {
-  NS_ENSURE_ARG_POINTER(aSrcMessageURI);
   NS_ENSURE_ARG_POINTER(aMailboxCopyHandler);
 
   nsresult rv;
@@ -589,7 +573,7 @@ nsNntpService::CopyMessage(const char* aSrcMessageURI,
   NS_ENSURE_SUCCESS(rv, rv);
 
   rv = DisplayMessage(aSrcMessageURI, streamSupport, aMsgWindow, aUrlListener,
-                      nullptr, aURL);
+                      false, aURL);
   return rv;
 }
 
@@ -703,7 +687,7 @@ nsNntpService::GenerateNewsHeaderValsForPosting(
       nsAutoCString currentHost;
       nsAutoCString theRest;
       // does list[index] start with "news:/"?
-      if (StringBeginsWith(list[index], NS_LITERAL_CSTRING(kNewsRootURI))) {
+      if (StringBeginsWith(list[index], kNewsRootURI ""_ns)) {
         // we have news://group or news://host/group
         // set theRest to what's after news://
         theRest = Substring(list[index], kNewsRootURILen /* for news:/ */ +
@@ -778,8 +762,8 @@ nsresult nsNntpService::GetNntpServerByAccount(
 
   // if we don't have a news host, find the first news server and use it
   if (NS_FAILED(rv) || !*aNntpServer)
-    rv = accountManager->FindServer(EmptyCString(), EmptyCString(),
-                                    NS_LITERAL_CSTRING("nntp"), aNntpServer);
+    rv = accountManager->FindServer(EmptyCString(), EmptyCString(), "nntp"_ns,
+                                    0, aNntpServer);
 
   return rv;
 }
@@ -834,10 +818,10 @@ nsNntpService::PostMessage(nsIFile* aFileToPost, const char* newsgroupsNames,
   return rv;
 }
 
-nsresult nsNntpService::ConstructNntpUrl(const char* urlString,
+nsresult nsNntpService::ConstructNntpUrl(const nsACString& urlString,
                                          nsIUrlListener* aUrlListener,
                                          nsIMsgWindow* aMsgWindow,
-                                         const char* originalMessageUri,
+                                         const nsACString& originalMessageUri,
                                          int32_t action, nsIURI** aUrl) {
   nsresult rv = NS_OK;
 
@@ -847,12 +831,13 @@ nsresult nsNntpService::ConstructNntpUrl(const char* urlString,
   nsCOMPtr<nsIMsgMailNewsUrl> mailnewsurl = do_QueryInterface(nntpUrl);
   mailnewsurl->SetMsgWindow(aMsgWindow);
   nsCOMPtr<nsIMsgMessageUrl> msgUrl = do_QueryInterface(nntpUrl);
-  msgUrl->SetUri(originalMessageUri);
-  rv = mailnewsurl->SetSpecInternal(nsDependentCString(urlString));
+  rv = mailnewsurl->SetSpecInternal(urlString);
   NS_ENSURE_SUCCESS(rv, rv);
   nntpUrl->SetNewsAction(action);
 
-  if (originalMessageUri) {
+  if (!originalMessageUri.IsEmpty()) {
+    msgUrl->SetUri(originalMessageUri);
+    NS_ENSURE_SUCCESS(rv, rv);
     // we'll use this later in nsNNTPProtocol::ParseURL()
     rv = msgUrl->SetOriginalSpec(originalMessageUri);
     NS_ENSURE_SUCCESS(rv, rv);
@@ -882,8 +867,7 @@ nsresult nsNntpService::CreateNewsAccount(const char* aHostname, bool aUseSSL,
 
   // for news, username is always null
   rv = accountManager->CreateIncomingServer(
-      EmptyCString(), nsDependentCString(aHostname), NS_LITERAL_CSTRING("nntp"),
-      aServer);
+      EmptyCString(), nsDependentCString(aHostname), "nntp"_ns, aServer);
   if (NS_FAILED(rv)) return rv;
 
   if (aUseSSL) {
@@ -999,17 +983,17 @@ nsresult nsNntpService::GetServerForUri(nsIURI* aUri,
 
   // XXX todo, or do we want to check if it is a news-message:// uri or
   // a news:// uri (but action is not a fetch related action?)
-  if (!PL_strncmp(spec.get(), kNewsMessageRootURI, kNewsMessageRootURILen) ||
+  if (StringBeginsWith(spec, kNewsMessageRootURI)) ||
       (action == nsINntpUrl::ActionFetchPart || action == nsINntpUrl::ActionFetchArticle))
   {
 #else
   // if this is a news-message:/ uri, decompose it and set hasMsgOffline on the
   // uri
-  if (!PL_strncmp(spec.get(), kNewsMessageRootURI, kNewsMessageRootURILen)) {
+  if (StringBeginsWith(spec, kNewsMessageRootURI ""_ns)) {
 #endif
   nsCOMPtr<nsIMsgFolder> folder;
   nsMsgKey key = nsMsgKey_None;
-  rv = DecomposeNewsMessageURI(spec.get(), getter_AddRefs(folder), &key);
+  rv = DecomposeNewsMessageURI(spec, getter_AddRefs(folder), &key);
   if (NS_SUCCEEDED(rv) && folder) {
     bool hasMsgOffline = false;
     folder->HasMsgOffline(key, &hasMsgOffline);
@@ -1036,21 +1020,19 @@ nsresult nsNntpService::RunNewsUrl(nsIURI* aUri, nsIMsgWindow* aMsgWindow,
 }
 
 NS_IMETHODIMP nsNntpService::GetNewNews(nsINntpIncomingServer* nntpServer,
-                                        const char* uri, bool aGetOld,
+                                        const nsACString& uri, bool aGetOld,
                                         nsIUrlListener* aUrlListener,
                                         nsIMsgWindow* aMsgWindow,
                                         nsIURI** _retval) {
-  NS_ENSURE_ARG_POINTER(uri);
-
   nsresult rv = NS_OK;
 
   nsCOMPtr<nsIMsgIncomingServer> server;
   server = do_QueryInterface(nntpServer);
 
   /* double check that it is a "news:/" url */
-  if (strncmp(uri, kNewsRootURI, kNewsRootURILen) == 0) {
+  if (StringBeginsWith(uri, kNewsRootURI ""_ns)) {
     nsCOMPtr<nsIURI> url;
-    rv = ConstructNntpUrl(uri, aUrlListener, aMsgWindow, nullptr,
+    rv = ConstructNntpUrl(uri, aUrlListener, aMsgWindow, EmptyCString(),
                           nsINntpUrl::ActionGetNewNews, getter_AddRefs(url));
     if (NS_FAILED(rv)) return rv;
 
@@ -1075,13 +1057,12 @@ NS_IMETHODIMP nsNntpService::GetNewNews(nsINntpIncomingServer* nntpServer,
 }
 
 NS_IMETHODIMP
-nsNntpService::CancelMessage(const char* cancelURL, const char* messageURI,
+nsNntpService::CancelMessage(const nsACString& cancelURL,
+                             const nsACString& messageURI,
                              nsISupports* aConsumer,
                              nsIUrlListener* aUrlListener,
                              nsIMsgWindow* aMsgWindow, nsIURI** aURL) {
   nsresult rv;
-  NS_ENSURE_ARG_POINTER(cancelURL);
-  NS_ENSURE_ARG_POINTER(messageURI);
 
   nsCOMPtr<nsIURI> url;
   // the url should be "news://host/message-id?cancel"
@@ -1173,6 +1154,7 @@ nsresult nsNntpService::NewURI(const nsACString& aSpec,
 NS_IMETHODIMP nsNntpService::NewChannel(nsIURI* aURI, nsILoadInfo* aLoadInfo,
                                         nsIChannel** _retval) {
   NS_ENSURE_ARG_POINTER(aURI);
+  MOZ_ASSERT(aLoadInfo);
   nsresult rv = NS_OK;
   nsCOMPtr<nsINntpIncomingServer> server;
   rv = GetServerForUri(aURI, getter_AddRefs(server));
@@ -1185,6 +1167,25 @@ NS_IMETHODIMP nsNntpService::NewChannel(nsIURI* aURI, nsILoadInfo* aLoadInfo,
   rv = channel->SetLoadInfo(aLoadInfo);
   NS_ENSURE_SUCCESS(rv, rv);
 
+  nsAutoCString spec;
+  rv = aURI->GetSpec(spec);
+  NS_ENSURE_SUCCESS(rv, rv);
+
+  // Add the attachment disposition. This forces docShell to open the
+  // attachment instead of displaying it. Content types we have special
+  // handlers for are white-listed. This white list also exists in
+  // nsMailboxService::NewChannel and nsImapService::NewChannel, so if you're
+  // changing this, update those too.
+  if (spec.Find("part=") >= 0 && spec.Find("type=message/rfc822") < 0 &&
+      spec.Find("type=application/x-message-display") < 0 &&
+      spec.Find("type=application/pdf") < 0) {
+    rv = channel->SetContentDisposition(nsIChannel::DISPOSITION_ATTACHMENT);
+    NS_ENSURE_SUCCESS(rv, rv);
+  } else {
+    rv = channel->SetContentDisposition(nsIChannel::DISPOSITION_INLINE);
+    NS_ENSURE_SUCCESS(rv, rv);
+  }
+
   channel.forget(_retval);
   return NS_OK;
 }
@@ -1297,23 +1298,9 @@ nsNntpService::GetFoldersCreatedAsync(bool* aAsyncCreation) {
   return NS_OK;
 }
 
-//
-// rhp: Right now, this is the same as simple DisplayMessage, but it will change
-// to support print rendering.
-//
-NS_IMETHODIMP nsNntpService::DisplayMessageForPrinting(
-    const char* aMessageURI, nsISupports* aDisplayConsumer,
-    nsIMsgWindow* aMsgWindow, nsIUrlListener* aUrlListener, nsIURI** aURL) {
-  mPrintingOperation = true;
-  nsresult rv = DisplayMessage(aMessageURI, aDisplayConsumer, aMsgWindow,
-                               aUrlListener, nullptr, aURL);
-  mPrintingOperation = false;
-  return rv;
-}
-
 NS_IMETHODIMP
-nsNntpService::StreamMessage(const char* aMessageURI, nsISupports* aConsumer,
-                             nsIMsgWindow* aMsgWindow,
+nsNntpService::StreamMessage(const nsACString& aMessageURI,
+                             nsISupports* aConsumer, nsIMsgWindow* aMsgWindow,
                              nsIUrlListener* aUrlListener,
                              bool /* convertData */,
                              const nsACString& aAdditionalHeader,
@@ -1345,8 +1332,8 @@ nsNntpService::StreamMessage(const char* aMessageURI, nsISupports* aConsumer,
   if (mOpenAttachmentOperation) action = nsINntpUrl::ActionFetchPart;
 
   nsCOMPtr<nsIURI> url;
-  rv = ConstructNntpUrl(urlStr.get(), aUrlListener, aMsgWindow,
-                        aURIString.get(), action, getter_AddRefs(url));
+  rv = ConstructNntpUrl(urlStr, aUrlListener, aMsgWindow, aURIString, action,
+                        getter_AddRefs(url));
   NS_ENSURE_SUCCESS(rv, rv);
 
   if (aLocalOnly || WeAreOffline()) {
@@ -1385,11 +1372,10 @@ nsNntpService::StreamMessage(const char* aMessageURI, nsISupports* aConsumer,
   return rv;
 }
 
-NS_IMETHODIMP nsNntpService::StreamHeaders(const char* aMessageURI,
+NS_IMETHODIMP nsNntpService::StreamHeaders(const nsACString& aMessageURI,
                                            nsIStreamListener* aConsumer,
                                            nsIUrlListener* aUrlListener,
                                            bool aLocalOnly, nsIURI** aURL) {
-  NS_ENSURE_ARG_POINTER(aMessageURI);
   NS_ENSURE_ARG_POINTER(aConsumer);
   nsCOMPtr<nsIMsgFolder> folder;
   nsMsgKey key;
@@ -1404,11 +1390,12 @@ NS_IMETHODIMP nsNntpService::StreamHeaders(const char* aMessageURI,
   bool hasMsgOffline = false;
   folder->HasMsgOffline(key, &hasMsgOffline);
   if (hasMsgOffline) {
-    int64_t messageOffset;
-    uint32_t messageSize;
-    folder->GetOfflineFileStream(key, &messageOffset, &messageSize,
-                                 getter_AddRefs(inputStream));
-    if (inputStream) return MsgStreamMsgHeaders(inputStream, aConsumer);
+    nsCOMPtr<nsIMsgDBHdr> hdr;
+    rv = folder->GetMessageHeader(key, getter_AddRefs(hdr));
+    NS_ENSURE_SUCCESS(rv, rv);
+    rv = folder->GetLocalMsgStream(hdr, getter_AddRefs(inputStream));
+    NS_ENSURE_SUCCESS(rv, rv);
+    return MsgStreamMsgHeaders(inputStream, aConsumer);
   }
   nsAutoCString urlStr;
   rv = CreateMessageIDURL(folder, key, getter_Copies(urlStr));
@@ -1452,9 +1439,8 @@ NS_IMETHODIMP nsNntpService::IsMsgInMemCache(nsIURI* aUrl,
 NS_IMETHODIMP nsNntpService::Search(nsIMsgSearchSession* aSearchSession,
                                     nsIMsgWindow* aMsgWindow,
                                     nsIMsgFolder* aMsgFolder,
-                                    const char* aSearchUri) {
+                                    const nsACString& aSearchUri) {
   NS_ENSURE_ARG(aMsgFolder);
-  NS_ENSURE_ARG(aSearchUri);
 
   nsresult rv;
 
@@ -1466,7 +1452,7 @@ NS_IMETHODIMP nsNntpService::Search(nsIMsgSearchSession* aSearchSession,
 
   nsCOMPtr<nsIUrlListener> urlListener = do_QueryInterface(aSearchSession);
   nsCOMPtr<nsIURI> url;
-  rv = ConstructNntpUrl(searchUrl.get(), urlListener, aMsgWindow, nullptr,
+  rv = ConstructNntpUrl(searchUrl, urlListener, aMsgWindow, EmptyCString(),
                         nsINntpUrl::ActionSearch, getter_AddRefs(url));
   NS_ENSURE_SUCCESS(rv, rv);
 
@@ -1504,7 +1490,7 @@ nsNntpService::GetListOfGroupsOnServer(nsINntpIncomingServer* aNntpServer,
   NS_ENSURE_SUCCESS(rv, rv);
 
   nsCOMPtr<nsIURI> url;
-  rv = ConstructNntpUrl(serverUri.get(), listener, aMsgWindow, nullptr,
+  rv = ConstructNntpUrl(serverUri, listener, aMsgWindow, EmptyCString(),
                         newsAction, getter_AddRefs(url));
   NS_ENSURE_SUCCESS(rv, rv);
 
@@ -1512,36 +1498,6 @@ nsNntpService::GetListOfGroupsOnServer(nsINntpIncomingServer* aNntpServer,
   return RunNewsUrl(url, aMsgWindow, nullptr);
 }
 
-NS_IMETHODIMP
-nsNntpService::Handle(nsICommandLine* aCmdLine) {
-  NS_ENSURE_ARG_POINTER(aCmdLine);
-
-  nsresult rv;
-  bool found;
-
-  rv = aCmdLine->HandleFlag(NS_LITERAL_STRING("news"), false, &found);
-  if (NS_SUCCEEDED(rv) && found) {
-    nsCOMPtr<nsIWindowWatcher> wwatch(
-        do_GetService(NS_WINDOWWATCHER_CONTRACTID));
-    NS_ENSURE_TRUE(wwatch, NS_ERROR_FAILURE);
-
-    nsCOMPtr<mozIDOMWindowProxy> opened;
-    wwatch->OpenWindow(
-        nullptr, "chrome://messenger/content/messenger.xhtml", "_blank",
-        "chrome,extrachrome,menubar,resizable,scrollbars,status,toolbar",
-        nullptr, getter_AddRefs(opened));
-    aCmdLine->SetPreventDefault(true);
-  }
-
-  return NS_OK;
-}
-
-NS_IMETHODIMP
-nsNntpService::GetHelpInfo(nsACString& aResult) {
-  aResult.AssignLiteral("  -news              Open the news client.\n");
-  return NS_OK;
-}
-
 NS_IMETHODIMP
 nsNntpService::HandleContent(const char* aContentType,
                              nsIInterfaceRequestor* aWindowContext,
@@ -1597,8 +1553,9 @@ nsNntpService::HandleContent(const char* aContentType,
 
           nsCOMPtr<mozIDOMWindowProxy> newWindow;
           rv = wwatcher->OpenWindow(
-              nullptr, "chrome://messenger/content/messenger.xhtml", "_blank",
-              "chome,all,dialog=no", arg, getter_AddRefs(newWindow));
+              nullptr, "chrome://messenger/content/messenger.xhtml"_ns,
+              "_blank"_ns, "chome,all,dialog=no"_ns, arg,
+              getter_AddRefs(newWindow));
           NS_ENSURE_SUCCESS(rv, rv);
         }
       }
@@ -1615,8 +1572,8 @@ nsNntpService::HandleContent(const char* aContentType,
 }
 
 NS_IMETHODIMP
-nsNntpService::MessageURIToMsgHdr(const char* uri, nsIMsgDBHdr** _retval) {
-  NS_ENSURE_ARG_POINTER(uri);
+nsNntpService::MessageURIToMsgHdr(const nsACString& uri,
+                                  nsIMsgDBHdr** _retval) {
   NS_ENSURE_ARG_POINTER(_retval);
   nsresult rv = NS_OK;
 
