@@ -12,8 +12,6 @@
 #include "nsDirectoryServiceDefs.h"
 #include "nsIPrompt.h"
 #include "nsIDocShell.h"
-#include "nsIInterfaceRequestor.h"
-#include "nsIInterfaceRequestorUtils.h"
 #include "nsIStringBundle.h"
 #include "nsIMsgSearchNotify.h"
 #include "nsIUrlListener.h"
@@ -21,7 +19,6 @@
 #include "nsIMsgLocalMailFolder.h"
 #include "nsIMsgDatabase.h"
 #include "nsIMsgHdr.h"
-#include "nsIDBFolderInfo.h"
 #include "nsMsgBaseCID.h"
 #include "nsIMsgCopyService.h"
 #include "nsIInputStream.h"
@@ -31,13 +28,9 @@
 #include "nsMsgCompCID.h"
 #include "nsNetUtil.h"
 #include "nsMsgUtils.h"
-#include "nsIMutableArray.h"
 #include "nsIMsgMailSession.h"
 #include "nsIFile.h"
-#include "nsArrayUtils.h"
-#include "nsCOMArray.h"
 #include "nsIMsgFilterCustomAction.h"
-#include "nsArrayEnumerator.h"
 #include "nsMsgMessageFlags.h"
 #include "nsIMsgWindow.h"
 #include "nsIMsgSearchCustomTerm.h"
@@ -45,49 +38,52 @@
 #include "nsIMsgThread.h"
 #include "nsIMsgFilter.h"
 #include "nsIMsgOperationListener.h"
-#include "mozilla/Attributes.h"
 #include "mozilla/Logging.h"
 
 using namespace mozilla;
 
 LazyLogModule FILTERLOGMODULE("Filters");
 
-#define BREAK_IF_FAILURE(_rv, _text)                                    \
-  if (NS_FAILED(_rv)) {                                                 \
-    MOZ_LOG(FILTERLOGMODULE, LogLevel::Error,                           \
-            ("(Post) Filter error: %s", _text));                        \
-    m_filters->LogFilterMessage(NS_LITERAL_STRING(_text), m_curFilter); \
-    NS_WARNING(_text);                                                  \
-    mFinalResult = _rv;                                                 \
-    break;                                                              \
+#define BREAK_IF_FAILURE(_rv, _text)                                   \
+  if (NS_FAILED(_rv)) {                                                \
+    MOZ_LOG(FILTERLOGMODULE, LogLevel::Error,                          \
+            ("(Post) Filter error: %s", _text));                       \
+    m_filters->LogFilterMessage(NS_LITERAL_STRING_FROM_CSTRING(_text), \
+                                m_curFilter);                          \
+    NS_WARNING(_text);                                                 \
+    mFinalResult = _rv;                                                \
+    break;                                                             \
   }
 
 #define CONTINUE_IF_FAILURE(_rv, _text)                                     \
   if (NS_FAILED(_rv)) {                                                     \
     MOZ_LOG(FILTERLOGMODULE, LogLevel::Warning,                             \
             ("(Post) Filter problem: %s", _text));                          \
-    m_filters->LogFilterMessage(NS_LITERAL_STRING(_text), m_curFilter);     \
+    m_filters->LogFilterMessage(NS_LITERAL_STRING_FROM_CSTRING(_text),      \
+                                m_curFilter);                               \
     NS_WARNING(_text);                                                      \
     mFinalResult = _rv;                                                     \
     if (m_msgWindow && !ContinueExecutionPrompt()) return OnEndExecution(); \
     continue;                                                               \
   }
 
-#define BREAK_IF_FALSE(_assertTrue, _text)                              \
-  if (MOZ_UNLIKELY(!(_assertTrue))) {                                   \
-    MOZ_LOG(FILTERLOGMODULE, LogLevel::Error,                           \
-            ("(Post) Filter error: %s", _text));                        \
-    m_filters->LogFilterMessage(NS_LITERAL_STRING(_text), m_curFilter); \
-    NS_WARNING(_text);                                                  \
-    mFinalResult = NS_ERROR_FAILURE;                                    \
-    break;                                                              \
+#define BREAK_IF_FALSE(_assertTrue, _text)                             \
+  if (MOZ_UNLIKELY(!(_assertTrue))) {                                  \
+    MOZ_LOG(FILTERLOGMODULE, LogLevel::Error,                          \
+            ("(Post) Filter error: %s", _text));                       \
+    m_filters->LogFilterMessage(NS_LITERAL_STRING_FROM_CSTRING(_text), \
+                                m_curFilter);                          \
+    NS_WARNING(_text);                                                 \
+    mFinalResult = NS_ERROR_FAILURE;                                   \
+    break;                                                             \
   }
 
 #define CONTINUE_IF_FALSE(_assertTrue, _text)                               \
   if (MOZ_UNLIKELY(!(_assertTrue))) {                                       \
     MOZ_LOG(FILTERLOGMODULE, LogLevel::Warning,                             \
             ("(Post) Filter problem: %s", _text));                          \
-    m_filters->LogFilterMessage(NS_LITERAL_STRING(_text), m_curFilter);     \
+    m_filters->LogFilterMessage(NS_LITERAL_STRING_FROM_CSTRING(_text),      \
+                                m_curFilter);                               \
     NS_WARNING(_text);                                                      \
     mFinalResult = NS_ERROR_FAILURE;                                        \
     if (m_msgWindow && !ContinueExecutionPrompt()) return OnEndExecution(); \
@@ -98,7 +94,8 @@ LazyLogModule FILTERLOGMODULE("Filters");
   MOZ_LOG(FILTERLOGMODULE, LogLevel::Error,                               \
           ("(Post) Filter Error: %s", _text));                            \
   if (loggingEnabled)                                                     \
-    m_filters->LogFilterMessage(NS_LITERAL_STRING(_text), m_curFilter);   \
+    m_filters->LogFilterMessage(NS_LITERAL_STRING_FROM_CSTRING(_text),    \
+                                m_curFilter);                             \
   NS_WARNING(_text);                                                      \
   if (m_msgWindow && !ContinueExecutionPrompt()) return OnEndExecution(); \
   break;
@@ -131,9 +128,11 @@ NS_IMETHODIMP nsMsgFilterService::OpenFilterList(
   NS_ENSURE_ARG_POINTER(aFilterFile);
   NS_ENSURE_ARG_POINTER(resultFilterList);
 
+  nsresult rv;
   if (rootFolder) {
     nsCOMPtr<nsIMsgIncomingServer> server;
-    rootFolder->GetServer(getter_AddRefs(server));
+    rv = rootFolder->GetServer(getter_AddRefs(server));
+    NS_ENSURE_SUCCESS(rv, rv);
     nsString serverName;
     server->GetPrettyName(serverName);
     MOZ_LOG(FILTERLOGMODULE, LogLevel::Info,
@@ -148,7 +147,7 @@ NS_IMETHODIMP nsMsgFilterService::OpenFilterList(
            NS_ConvertUTF16toUTF8(fileName).get()));
 
   bool exists = false;
-  nsresult rv = aFilterFile->Exists(&exists);
+  rv = aFilterFile->Exists(&exists);
   if (NS_FAILED(rv) || !exists) {
     rv = aFilterFile->Create(nsIFile::NORMAL_FILE_TYPE, 0644);
     NS_ENSURE_SUCCESS(rv, rv);
@@ -253,13 +252,12 @@ nsresult nsMsgFilterService::BackUpFilterFile(nsIFile* aFilterFile,
   nsCOMPtr<nsIFile> backupFile;
   rv = localParentDir->Clone(getter_AddRefs(backupFile));
   NS_ENSURE_SUCCESS(rv, rv);
-  backupFile->AppendNative(NS_LITERAL_CSTRING("rulesbackup.dat"));
+  backupFile->AppendNative("rulesbackup.dat"_ns);
   bool exists;
   backupFile->Exists(&exists);
   if (exists) backupFile->Remove(false);
 
-  return aFilterFile->CopyToNative(localParentDir,
-                                   NS_LITERAL_CSTRING("rulesbackup.dat"));
+  return aFilterFile->CopyToNative(localParentDir, "rulesbackup.dat"_ns);
 }
 
 nsresult nsMsgFilterService::AlertBackingUpFilterFile(
@@ -344,7 +342,8 @@ class nsMsgFilterAfterTheFact : public nsIUrlListener,
                                 public nsIMsgCopyServiceListener {
  public:
   nsMsgFilterAfterTheFact(nsIMsgWindow* aMsgWindow,
-                          nsIMsgFilterList* aFilterList, nsIArray* aFolderList,
+                          nsIMsgFilterList* aFilterList,
+                          const nsTArray<RefPtr<nsIMsgFolder>>& aFolderList,
                           nsIMsgOperationListener* aCallback);
   NS_DECL_ISUPPORTS
   NS_DECL_NSIURLLISTENER
@@ -366,16 +365,15 @@ class nsMsgFilterAfterTheFact : public nsIUrlListener,
                                      bool* confirmed);
   nsCOMPtr<nsIMsgWindow> m_msgWindow;
   nsCOMPtr<nsIMsgFilterList> m_filters;
-  nsCOMPtr<nsIArray> m_folders;
+  nsTArray<RefPtr<nsIMsgFolder>> m_folders;
   nsCOMPtr<nsIMsgFolder> m_curFolder;
   nsCOMPtr<nsIMsgDatabase> m_curFolderDB;
   nsCOMPtr<nsIMsgFilter> m_curFilter;
   uint32_t m_curFilterIndex;
   uint32_t m_curFolderIndex;
   uint32_t m_numFilters;
-  uint32_t m_numFolders;
   nsTArray<nsMsgKey> m_searchHits;
-  nsCOMPtr<nsIMutableArray> m_searchHitHdrs;
+  nsTArray<RefPtr<nsIMsgDBHdr>> m_searchHitHdrs;
   nsTArray<nsMsgKey> m_stopFiltering;
   nsCOMPtr<nsIMsgSearchSession> m_searchSession;
   nsCOMPtr<nsIMsgOperationListener> m_callback;
@@ -389,20 +387,19 @@ NS_IMPL_ISUPPORTS(nsMsgFilterAfterTheFact, nsIUrlListener, nsIMsgSearchNotify,
 
 nsMsgFilterAfterTheFact::nsMsgFilterAfterTheFact(
     nsIMsgWindow* aMsgWindow, nsIMsgFilterList* aFilterList,
-    nsIArray* aFolderList, nsIMsgOperationListener* aCallback) {
+    const nsTArray<RefPtr<nsIMsgFolder>>& aFolderList,
+    nsIMsgOperationListener* aCallback) {
   MOZ_LOG(FILTERLOGMODULE, LogLevel::Debug, ("(Post) nsMsgFilterAfterTheFact"));
   m_curFilterIndex = m_curFolderIndex = m_nextAction = 0;
   m_msgWindow = aMsgWindow;
   m_filters = aFilterList;
-  m_folders = aFolderList;
+  m_folders = aFolderList.Clone();
   m_filters->GetFilterCount(&m_numFilters);
-  m_folders->GetLength(&m_numFolders);
 
   NS_ADDREF_THIS();  // we own ourselves, and will release ourselves when
                      // execution is done.
   mNeedsRelease = true;
 
-  m_searchHitHdrs = do_CreateInstance(NS_ARRAY_CONTRACTID);
   m_callback = aCallback;
   mFinalResult = NS_OK;
 }
@@ -455,8 +452,8 @@ nsresult nsMsgFilterAfterTheFact::RunNextFilter() {
             ("(Post) Filter name: %s", NS_ConvertUTF16toUTF8(filterName).get()));
     // clang-format on
 
-    nsCOMPtr<nsIMutableArray> searchTerms;
-    rv = m_curFilter->GetSearchTerms(getter_AddRefs(searchTerms));
+    nsTArray<RefPtr<nsIMsgSearchTerm>> searchTerms;
+    rv = m_curFilter->GetSearchTerms(searchTerms);
     CONTINUE_IF_FAILURE(rv, "Could not get searchTerms");
 
     if (m_searchSession) m_searchSession->UnregisterListener(this);
@@ -464,12 +461,7 @@ nsresult nsMsgFilterAfterTheFact::RunNextFilter() {
     BREAK_IF_FAILURE(rv, "Failed to get search session");
 
     nsMsgSearchScopeValue searchScope = nsMsgSearchScope::offlineMail;
-    uint32_t termCount = 0;
-    searchTerms->GetLength(&termCount);
-    for (uint32_t termIndex = 0; termIndex < termCount; termIndex++) {
-      nsCOMPtr<nsIMsgSearchTerm> term =
-          do_QueryElementAt(searchTerms, termIndex, &rv);
-      BREAK_IF_FAILURE(rv, "Could not get search term");
+    for (nsIMsgSearchTerm* term : searchTerms) {
       rv = m_searchSession->AppendTerm(term);
       BREAK_IF_FAILURE(rv, "Could not append search term");
     }
@@ -488,8 +480,7 @@ nsresult nsMsgFilterAfterTheFact::RunNextFilter() {
   if (NS_FAILED(rv)) {
     MOZ_LOG(FILTERLOGMODULE, LogLevel::Error,
             ("(Post) Filter evaluation failed"));
-    m_filters->LogFilterMessage(NS_LITERAL_STRING("Filter evaluation failed"),
-                                m_curFilter);
+    m_filters->LogFilterMessage(u"Filter evaluation failed"_ns, m_curFilter);
   }
 
   m_curFilter = nullptr;
@@ -505,7 +496,7 @@ nsresult nsMsgFilterAfterTheFact::AdvanceToNextFolder() {
   while (true) {
     m_stopFiltering.Clear();
     m_curFolder = nullptr;
-    if (m_curFolderIndex >= m_numFolders) {
+    if (m_curFolderIndex >= m_folders.Length()) {
       // final end of nsMsgFilterAfterTheFact object
       return OnEndExecution();
     }
@@ -516,8 +507,7 @@ nsresult nsMsgFilterAfterTheFact::AdvanceToNextFolder() {
     // reset the filter index to apply all filters to this new folder
     m_curFilterIndex = 0;
     m_nextAction = 0;
-    m_curFolder = do_QueryElementAt(m_folders, m_curFolderIndex++, &rv);
-    CONTINUE_IF_FAILURE(rv, "Could not get next folder");
+    m_curFolder = m_folders[m_curFolderIndex++];
 
     // Note: I got rv = NS_OK but null m_curFolder after deleting a folder
     // outside of TB, when I select a single message and "run filter on message"
@@ -580,7 +570,6 @@ NS_IMETHODIMP nsMsgFilterAfterTheFact::OnStopRunningUrl(nsIURI* aUrl,
 NS_IMETHODIMP nsMsgFilterAfterTheFact::OnSearchHit(nsIMsgDBHdr* header,
                                                    nsIMsgFolder* folder) {
   NS_ENSURE_ARG_POINTER(header);
-  NS_ENSURE_TRUE(m_searchHitHdrs, NS_ERROR_NOT_INITIALIZED);
 
   nsMsgKey msgKey;
   header->GetMessageKey(&msgKey);
@@ -604,7 +593,7 @@ NS_IMETHODIMP nsMsgFilterAfterTheFact::OnSearchHit(nsIMsgDBHdr* header,
   }
 
   m_searchHits.AppendElement(msgKey);
-  m_searchHitHdrs->AppendElement(header);
+  m_searchHitHdrs.AppendElement(header);
   return NS_OK;
 }
 
@@ -624,7 +613,7 @@ NS_IMETHODIMP nsMsgFilterAfterTheFact::OnSearchDone(nsresult status) {
 
 NS_IMETHODIMP nsMsgFilterAfterTheFact::OnNewSearch() {
   m_searchHits.Clear();
-  m_searchHitHdrs->Clear();
+  m_searchHitHdrs.Clear();
   return NS_OK;
 }
 
@@ -640,7 +629,6 @@ nsresult nsMsgFilterAfterTheFact::ApplyFilter() {
   do {
     // Error management block, break if unable to continue with filter.
 
-    BREAK_IF_FALSE(m_searchHitHdrs, "No search headers object");
     if (!m_curFilter)
       break;  // Maybe not an error, we just need to call RunNextFilter();
     if (!m_curFolder)
@@ -701,14 +689,8 @@ nsresult nsMsgFilterAfterTheFact::ApplyFilter() {
       }
 
       if (loggingEnabled) {
-        for (uint32_t msgIndex = 0; msgIndex < m_searchHits.Length();
-             msgIndex++) {
-          nsCOMPtr<nsIMsgDBHdr> msgHdr =
-              do_QueryElementAt(m_searchHitHdrs, msgIndex);
-          if (msgHdr)
-            (void)curFilter->LogRuleHit(filterAction, msgHdr);
-          else
-            NS_WARNING("could not QI element to nsIMsgDBHdr");
+        for (auto msgHdr : m_searchHitHdrs) {
+          (void)curFilter->LogRuleHit(filterAction, msgHdr);
         }
       }
 
@@ -751,7 +733,8 @@ nsresult nsMsgFilterAfterTheFact::ApplyFilter() {
 
           if (uri.Equals(actionTargetFolderUri)) {
             MOZ_LOG(FILTERLOGMODULE, LogLevel::Info,
-                    ("(Post) Target folder is the same as source folder, skipping"));
+                    ("(Post) Target folder is the same as source folder, "
+                     "skipping"));
             break;
           }
 
@@ -818,12 +801,7 @@ nsresult nsMsgFilterAfterTheFact::ApplyFilter() {
           break;
         case nsMsgFilterAction::KillThread:
         case nsMsgFilterAction::WatchThread: {
-          for (uint32_t msgIndex = 0; msgIndex < m_searchHits.Length();
-               msgIndex++) {
-            nsCOMPtr<nsIMsgDBHdr> msgHdr =
-                do_QueryElementAt(m_searchHitHdrs, msgIndex);
-            BREAK_ACTION_IF_FALSE(msgHdr, "Could not get msg header");
-
+          for (auto msgHdr : m_searchHitHdrs) {
             nsCOMPtr<nsIMsgThread> msgThread;
             nsMsgKey threadKey;
             m_curFolderDB->GetThreadContainingMsgHdr(msgHdr,
@@ -842,12 +820,7 @@ nsresult nsMsgFilterAfterTheFact::ApplyFilter() {
           }
         } break;
         case nsMsgFilterAction::KillSubthread: {
-          for (uint32_t msgIndex = 0; msgIndex < m_searchHits.Length();
-               msgIndex++) {
-            nsCOMPtr<nsIMsgDBHdr> msgHdr =
-                do_QueryElementAt(m_searchHitHdrs, msgIndex, &rv);
-            BREAK_ACTION_IF_FAILURE(rv, "Could not get msg header");
-            BREAK_ACTION_IF_FALSE(msgHdr, "Could not get msg header");
+          for (auto msgHdr : m_searchHitHdrs) {
             rv = m_curFolderDB->MarkHeaderKilled(msgHdr, true, nullptr);
             BREAK_ACTION_IF_FAILURE(rv, "Setting message flags failed");
           }
@@ -855,12 +828,7 @@ nsresult nsMsgFilterAfterTheFact::ApplyFilter() {
         case nsMsgFilterAction::ChangePriority: {
           nsMsgPriorityValue filterPriority;
           filterAction->GetPriority(&filterPriority);
-          for (uint32_t msgIndex = 0; msgIndex < m_searchHits.Length();
-               msgIndex++) {
-            nsCOMPtr<nsIMsgDBHdr> msgHdr =
-                do_QueryElementAt(m_searchHitHdrs, msgIndex, &rv);
-            BREAK_ACTION_IF_FAILURE(rv, "Could not get msg header");
-            BREAK_ACTION_IF_FALSE(msgHdr, "Could not get msg header");
+          for (auto msgHdr : m_searchHitHdrs) {
             rv = msgHdr->SetPriority(filterPriority);
             BREAK_ACTION_IF_FAILURE(rv, "Setting message flags failed");
           }
@@ -897,16 +865,11 @@ nsresult nsMsgFilterAfterTheFact::ApplyFilter() {
               do_GetService(NS_MSGCOMPOSESERVICE_CONTRACTID, &rv);
           BREAK_ACTION_IF_FAILURE(rv, "Could not get compose service");
 
-          for (uint32_t msgIndex = 0; msgIndex < m_searchHits.Length();
-               msgIndex++) {
-            nsCOMPtr<nsIMsgDBHdr> msgHdr(
-                do_QueryElementAt(m_searchHitHdrs, msgIndex));
-            if (msgHdr)
-              rv = compService->ForwardMessage(
-                  NS_ConvertASCIItoUTF16(forwardTo), msgHdr, m_msgWindow,
-                  server, nsIMsgComposeService::kForwardAsDefault);
+          for (auto msgHdr : m_searchHitHdrs) {
+            rv = compService->ForwardMessage(
+                NS_ConvertASCIItoUTF16(forwardTo), msgHdr, m_msgWindow, server,
+                nsIMsgComposeService::kForwardAsDefault);
             BREAK_ACTION_IF_FAILURE(rv, "Forward action failed");
-            BREAK_ACTION_IF_FALSE(msgHdr, "Forward action failed");
           }
         } break;
         case nsMsgFilterAction::Reply: {
@@ -922,23 +885,18 @@ nsresult nsMsgFilterAfterTheFact::ApplyFilter() {
           nsCOMPtr<nsIMsgComposeService> compService =
               do_GetService(NS_MSGCOMPOSESERVICE_CONTRACTID, &rv);
           BREAK_ACTION_IF_FAILURE(rv, "Could not get compose service");
-          for (uint32_t msgIndex = 0; msgIndex < m_searchHits.Length();
-               msgIndex++) {
-            nsCOMPtr<nsIMsgDBHdr> msgHdr =
-                do_QueryElementAt(m_searchHitHdrs, msgIndex, &rv);
-            BREAK_ACTION_IF_FAILURE(rv, "Could not get msgHdr");
-            BREAK_ACTION_IF_FALSE(msgHdr, "Could not get msgHdr");
-            rv = compService->ReplyWithTemplate(msgHdr, replyTemplateUri.get(),
+          for (auto msgHdr : m_searchHitHdrs) {
+            rv = compService->ReplyWithTemplate(msgHdr, replyTemplateUri,
                                                 m_msgWindow, server);
             if (NS_FAILED(rv)) {
               if (rv == NS_ERROR_ABORT) {
                 (void)curFilter->LogRuleHitFail(
                     filterAction, msgHdr, rv,
-                    NS_LITERAL_CSTRING("filterFailureSendingReplyAborted"));
+                    "filterFailureSendingReplyAborted"_ns);
               } else {
                 (void)curFilter->LogRuleHitFail(
                     filterAction, msgHdr, rv,
-                    NS_LITERAL_CSTRING("filterFailureSendingReplyError"));
+                    "filterFailureSendingReplyError"_ns);
               }
             }
             BREAK_ACTION_IF_FAILURE(rv, "ReplyWithTemplate failed");
@@ -951,37 +909,26 @@ nsresult nsMsgFilterAfterTheFact::ApplyFilter() {
           BREAK_ACTION_IF_FALSE(localFolder,
                                 "Current folder not a local folder");
           // This action ignores the deleteMailLeftOnServer preference
-          {
-            nsTArray<RefPtr<nsIMsgDBHdr>> hdrs;
-            MsgHdrsToTArray(m_searchHitHdrs, hdrs);
-            rv = localFolder->MarkMsgsOnPop3Server(hdrs, POP3_FORCE_DEL);
-            BREAK_ACTION_IF_FAILURE(rv, "MarkMsgsOnPop3Server failed");
-          }
+          rv = localFolder->MarkMsgsOnPop3Server(m_searchHitHdrs,
+                                                 POP3_FORCE_DEL);
+          BREAK_ACTION_IF_FAILURE(rv, "MarkMsgsOnPop3Server failed");
 
-          nsCOMPtr<nsIMutableArray> partialMsgs;
           // Delete the partial headers. They're useless now
           //   that the server copy is being deleted.
-          for (uint32_t msgIndex = 0; msgIndex < m_searchHits.Length();
-               msgIndex++) {
-            nsCOMPtr<nsIMsgDBHdr> msgHdr =
-                do_QueryElementAt(m_searchHitHdrs, msgIndex, &rv);
-            BREAK_ACTION_IF_FAILURE(rv, "Could not get msgHdr");
-            BREAK_ACTION_IF_FALSE(msgHdr, "Could not get msgHdr");
+          nsTArray<RefPtr<nsIMsgDBHdr>> partialMsgs;
+          for (uint32_t i = 0; i < m_searchHits.Length(); ++i) {
+            nsIMsgDBHdr* msgHdr = m_searchHitHdrs[i];
+            nsMsgKey msgKey = m_searchHits[i];
             uint32_t flags;
             msgHdr->GetFlags(&flags);
             if (flags & nsMsgMessageFlags::Partial) {
-              if (!partialMsgs) {
-                partialMsgs = do_CreateInstance(NS_ARRAY_CONTRACTID);
-                BREAK_ACTION_IF_FALSE(partialMsgs,
-                                      "Could not create partialMsgs array");
-              }
-              partialMsgs->AppendElement(msgHdr);
-              m_stopFiltering.AppendElement(m_searchHits[msgIndex]);
-              curFolder->OrProcessingFlags(m_searchHits[msgIndex],
+              partialMsgs.AppendElement(msgHdr);
+              m_stopFiltering.AppendElement(msgKey);
+              curFolder->OrProcessingFlags(msgKey,
                                            nsMsgProcessingFlags::FilterToMove);
             }
           }
-          if (partialMsgs) {
+          if (!partialMsgs.IsEmpty()) {
             rv = curFolder->DeleteMessages(partialMsgs, m_msgWindow, true,
                                            false, nullptr, false);
             BREAK_ACTION_IF_FAILURE(rv, "Delete messages failed");
@@ -992,23 +939,14 @@ nsresult nsMsgFilterAfterTheFact::ApplyFilter() {
               do_QueryInterface(curFolder, &rv);
           BREAK_ACTION_IF_FAILURE(rv, "current folder not local");
           BREAK_ACTION_IF_FALSE(localFolder, "current folder not local");
-          nsCOMPtr<nsIMutableArray> messages(
-              do_CreateInstance(NS_ARRAY_CONTRACTID, &rv));
-          BREAK_ACTION_IF_FAILURE(rv, "Could not create messages array");
-          for (uint32_t msgIndex = 0; msgIndex < m_searchHits.Length();
-               msgIndex++) {
-            nsCOMPtr<nsIMsgDBHdr> msgHdr =
-                do_QueryElementAt(m_searchHitHdrs, msgIndex, &rv);
-            BREAK_ACTION_IF_FAILURE(rv, "Could not get msgHdr");
-            BREAK_ACTION_IF_FALSE(msgHdr, "Could not get msgHdr");
+          nsTArray<RefPtr<nsIMsgDBHdr>> messages;
+          for (nsIMsgDBHdr* msgHdr : m_searchHitHdrs) {
             uint32_t flags = 0;
             msgHdr->GetFlags(&flags);
             if (flags & nsMsgMessageFlags::Partial)
-              messages->AppendElement(msgHdr);
+              messages.AppendElement(msgHdr);
           }
-          uint32_t msgsToFetch;
-          messages->GetLength(&msgsToFetch);
-          if (msgsToFetch > 0) {
+          if (messages.Length() > 0) {
             rv = curFolder->DownloadMessagesForOffline(messages, m_msgWindow);
             BREAK_ACTION_IF_FAILURE(rv, "DownloadMessagesForOffline failed");
           }
@@ -1032,8 +970,8 @@ nsresult nsMsgFilterAfterTheFact::ApplyFilter() {
           BREAK_ACTION_IF_FAILURE(rv, "Could not get custom action value");
           bool isAsync = false;
           customAction->GetIsAsync(&isAsync);
-          rv = customAction->Apply(m_searchHitHdrs, value, this, filterType,
-                                   m_msgWindow);
+          rv = customAction->ApplyAction(m_searchHitHdrs, value, this,
+                                         filterType, m_msgWindow);
           BREAK_ACTION_IF_FAILURE(rv, "custom action failed to apply");
           if (isAsync) {
             MOZ_LOG(FILTERLOGMODULE, LogLevel::Debug,
@@ -1052,11 +990,10 @@ nsresult nsMsgFilterAfterTheFact::ApplyFilter() {
         MOZ_LOG(FILTERLOGMODULE, LogLevel::Error,
                 ("(Post) Action execution failed with error: %" PRIx32,
                  static_cast<uint32_t>(mFinalResult)));
-        if (loggingEnabled) {
-          nsCOMPtr<nsIMsgDBHdr> msgHdr = do_QueryElementAt(m_searchHitHdrs, 0);
-          (void)curFilter->LogRuleHitFail(
-              filterAction, msgHdr, mFinalResult,
-              NS_LITERAL_CSTRING("filterActionFailed"));
+        if (loggingEnabled && m_searchHitHdrs.Length() > 0) {
+          (void)curFilter->LogRuleHitFail(filterAction, m_searchHitHdrs[0],
+                                          mFinalResult,
+                                          "filterActionFailed"_ns);
         }
       } else {
         MOZ_LOG(FILTERLOGMODULE, LogLevel::Info,
@@ -1081,26 +1018,23 @@ NS_IMETHODIMP nsMsgFilterService::GetTempFilterList(
 }
 
 NS_IMETHODIMP
-nsMsgFilterService::ApplyFiltersToFolders(nsIMsgFilterList* aFilterList,
-                                          nsIArray* aFolders,
-                                          nsIMsgWindow* aMsgWindow,
-                                          nsIMsgOperationListener* aCallback) {
+nsMsgFilterService::ApplyFiltersToFolders(
+    nsIMsgFilterList* aFilterList,
+    const nsTArray<RefPtr<nsIMsgFolder>>& aFolders, nsIMsgWindow* aMsgWindow,
+    nsIMsgOperationListener* aCallback) {
   MOZ_LOG(FILTERLOGMODULE, LogLevel::Debug,
           ("(Post) nsMsgFilterService::ApplyFiltersToFolders"));
   NS_ENSURE_ARG_POINTER(aFilterList);
-  NS_ENSURE_ARG_POINTER(aFolders);
 
   uint32_t filterCount;
   aFilterList->GetFilterCount(&filterCount);
   nsCString listId;
   aFilterList->GetListId(listId);
-  uint32_t folderCount;
-  aFolders->GetLength(&folderCount);
   MOZ_LOG(FILTERLOGMODULE, LogLevel::Info,
           ("(Post) Manual filter run initiated"));
   MOZ_LOG(FILTERLOGMODULE, LogLevel::Info,
           ("(Post) Running %" PRIu32 " filters from %s on %" PRIu32 " folders",
-           filterCount, listId.get(), folderCount));
+           filterCount, listId.get(), (int)aFolders.Length()));
 
   RefPtr<nsMsgFilterAfterTheFact> filterExecutor =
       new nsMsgFilterAfterTheFact(aMsgWindow, aFilterList, aFolders, aCallback);
@@ -1112,16 +1046,14 @@ nsMsgFilterService::ApplyFiltersToFolders(nsIMsgFilterList* aFilterList,
 
 NS_IMETHODIMP nsMsgFilterService::AddCustomAction(
     nsIMsgFilterCustomAction* aAction) {
-  mCustomActions.AppendObject(aAction);
+  mCustomActions.AppendElement(aAction);
   return NS_OK;
 }
 
 NS_IMETHODIMP nsMsgFilterService::GetCustomActions(
-    nsISimpleEnumerator** aResult) {
-  NS_ENSURE_ARG_POINTER(aResult);
-
-  return NS_NewArrayEnumerator(aResult, mCustomActions,
-                               NS_GET_IID(nsIMsgFilterCustomAction));
+    nsTArray<RefPtr<nsIMsgFilterCustomAction>>& actions) {
+  actions = mCustomActions.Clone();
+  return NS_OK;
 }
 
 NS_IMETHODIMP
@@ -1129,11 +1061,11 @@ nsMsgFilterService::GetCustomAction(const nsACString& aId,
                                     nsIMsgFilterCustomAction** aResult) {
   NS_ENSURE_ARG_POINTER(aResult);
 
-  for (int32_t i = 0; i < mCustomActions.Count(); i++) {
+  for (nsIMsgFilterCustomAction* action : mCustomActions) {
     nsAutoCString id;
-    nsresult rv = mCustomActions[i]->GetId(id);
+    nsresult rv = action->GetId(id);
     if (NS_SUCCEEDED(rv) && aId.Equals(id)) {
-      NS_ADDREF(*aResult = mCustomActions[i]);
+      NS_ADDREF(*aResult = action);
       return NS_OK;
     }
   }
@@ -1142,16 +1074,14 @@ nsMsgFilterService::GetCustomAction(const nsACString& aId,
 }
 
 NS_IMETHODIMP nsMsgFilterService::AddCustomTerm(nsIMsgSearchCustomTerm* aTerm) {
-  mCustomTerms.AppendObject(aTerm);
+  mCustomTerms.AppendElement(aTerm);
   return NS_OK;
 }
 
 NS_IMETHODIMP nsMsgFilterService::GetCustomTerms(
-    nsISimpleEnumerator** aResult) {
-  NS_ENSURE_ARG_POINTER(aResult);
-
-  return NS_NewArrayEnumerator(aResult, mCustomTerms,
-                               NS_GET_IID(nsIMsgSearchCustomTerm));
+    nsTArray<RefPtr<nsIMsgSearchCustomTerm>>& terms) {
+  terms = mCustomTerms.Clone();
+  return NS_OK;
 }
 
 NS_IMETHODIMP
@@ -1159,11 +1089,11 @@ nsMsgFilterService::GetCustomTerm(const nsACString& aId,
                                   nsIMsgSearchCustomTerm** aResult) {
   NS_ENSURE_ARG_POINTER(aResult);
 
-  for (int32_t i = 0; i < mCustomTerms.Count(); i++) {
+  for (nsIMsgSearchCustomTerm* term : mCustomTerms) {
     nsAutoCString id;
-    nsresult rv = mCustomTerms[i]->GetId(id);
+    nsresult rv = term->GetId(id);
     if (NS_SUCCEEDED(rv) && aId.Equals(id)) {
-      NS_ADDREF(*aResult = mCustomTerms[i]);
+      NS_ADDREF(*aResult = term);
       return NS_OK;
     }
   }
@@ -1226,40 +1156,28 @@ class nsMsgApplyFiltersToMessages : public nsMsgFilterAfterTheFact {
  public:
   nsMsgApplyFiltersToMessages(nsIMsgWindow* aMsgWindow,
                               nsIMsgFilterList* aFilterList,
-                              nsIArray* aFolderList, nsIArray* aMsgHdrList,
+                              const nsTArray<RefPtr<nsIMsgFolder>>& aFolderList,
+                              const nsTArray<RefPtr<nsIMsgDBHdr>>& aMsgHdrList,
                               nsMsgFilterTypeType aFilterType,
                               nsIMsgOperationListener* aCallback);
 
  protected:
   virtual nsresult RunNextFilter();
 
-  nsCOMArray<nsIMsgDBHdr> m_msgHdrList;
+  nsTArray<RefPtr<nsIMsgDBHdr>> m_msgHdrList;
   nsMsgFilterTypeType m_filterType;
 };
 
 nsMsgApplyFiltersToMessages::nsMsgApplyFiltersToMessages(
     nsIMsgWindow* aMsgWindow, nsIMsgFilterList* aFilterList,
-    nsIArray* aFolderList, nsIArray* aMsgHdrList,
+    const nsTArray<RefPtr<nsIMsgFolder>>& aFolderList,
+    const nsTArray<RefPtr<nsIMsgDBHdr>>& aMsgHdrList,
     nsMsgFilterTypeType aFilterType, nsIMsgOperationListener* aCallback)
     : nsMsgFilterAfterTheFact(aMsgWindow, aFilterList, aFolderList, aCallback),
+      m_msgHdrList(aMsgHdrList.Clone()),
       m_filterType(aFilterType) {
   MOZ_LOG(FILTERLOGMODULE, LogLevel::Debug,
           ("(Post) nsMsgApplyFiltersToMessages"));
-  nsCOMPtr<nsISimpleEnumerator> msgEnumerator;
-  if (NS_SUCCEEDED(aMsgHdrList->Enumerate(getter_AddRefs(msgEnumerator)))) {
-    uint32_t length;
-    if (NS_SUCCEEDED(aMsgHdrList->GetLength(&length)))
-      m_msgHdrList.SetCapacity(length);
-
-    bool hasMore;
-    while (NS_SUCCEEDED(msgEnumerator->HasMoreElements(&hasMore)) && hasMore) {
-      nsCOMPtr<nsISupports> supports;
-      nsCOMPtr<nsIMsgDBHdr> msgHdr;
-      if (NS_SUCCEEDED(msgEnumerator->GetNext(getter_AddRefs(supports))) &&
-          (msgHdr = do_QueryInterface(supports)))
-        m_msgHdrList.AppendObject(msgHdr);
-    }
-  }
 }
 
 nsresult nsMsgApplyFiltersToMessages::RunNextFilter() {
@@ -1301,10 +1219,7 @@ nsresult nsMsgApplyFiltersToMessages::RunNextFilter() {
     m_curFilter->SetScope(scope);
     OnNewSearch();
 
-    for (int32_t i = 0; i < m_msgHdrList.Count(); i++) {
-      nsCOMPtr<nsIMsgDBHdr> msgHdr = m_msgHdrList[i];
-      CONTINUE_IF_FALSE(msgHdr, "null msgHdr");
-
+    for (auto msgHdr : m_msgHdrList) {
       bool matched;
       rv = m_curFilter->MatchHdr(msgHdr, m_curFolder, m_curFolderDB,
                                  EmptyCString(), &matched);
@@ -1331,8 +1246,7 @@ nsresult nsMsgApplyFiltersToMessages::RunNextFilter() {
             ("(Post) Filter run failed (%" PRIx32 ")",
              static_cast<uint32_t>(rv)));
     // clang-format on
-    m_filters->LogFilterMessage(NS_LITERAL_STRING("Filter run failed"),
-                                m_curFilter);
+    m_filters->LogFilterMessage(u"Filter run failed"_ns, m_curFilter);
     NS_WARNING_ASSERTION(false, "Failed to run filters");
   } else {
     MOZ_LOG(FILTERLOGMODULE, LogLevel::Info,
@@ -1348,9 +1262,9 @@ nsresult nsMsgApplyFiltersToMessages::RunNextFilter() {
 }
 
 NS_IMETHODIMP nsMsgFilterService::ApplyFilters(
-    nsMsgFilterTypeType aFilterType, nsIArray* aMsgHdrList,
-    nsIMsgFolder* aFolder, nsIMsgWindow* aMsgWindow,
-    nsIMsgOperationListener* aCallback) {
+    nsMsgFilterTypeType aFilterType,
+    const nsTArray<RefPtr<nsIMsgDBHdr>>& aMsgHdrList, nsIMsgFolder* aFolder,
+    nsIMsgWindow* aMsgWindow, nsIMsgOperationListener* aCallback) {
   MOZ_LOG(FILTERLOGMODULE, LogLevel::Debug,
           ("(Post) nsMsgApplyFiltersToMessages::ApplyFilters"));
   NS_ENSURE_ARG_POINTER(aFolder);
@@ -1359,16 +1273,8 @@ NS_IMETHODIMP nsMsgFilterService::ApplyFilters(
   nsresult rv = aFolder->GetFilterList(aMsgWindow, getter_AddRefs(filterList));
   NS_ENSURE_SUCCESS(rv, rv);
 
-  nsCOMPtr<nsIMutableArray> folderList(
-      do_CreateInstance(NS_ARRAY_CONTRACTID, &rv));
-  NS_ENSURE_SUCCESS(rv, rv);
-
-  folderList->AppendElement(aFolder);
-
   uint32_t filterCount;
   filterList->GetFilterCount(&filterCount);
-  uint32_t msgCount;
-  aMsgHdrList->GetLength(&msgCount);
   nsCString listId;
   filterList->GetListId(listId);
   nsString folderName;
@@ -1381,13 +1287,13 @@ NS_IMETHODIMP nsMsgFilterService::ApplyFilters(
   MOZ_LOG(FILTERLOGMODULE, LogLevel::Info,
           ("(Post) Running %" PRIu32 " filters from %s on %" PRIu32
            " message(s) in folder '%s'",
-           filterCount, listId.get(), msgCount,
+           filterCount, listId.get(), (uint32_t)aMsgHdrList.Length(),
            NS_ConvertUTF16toUTF8(folderName).get()));
 
   // Create our nsMsgApplyFiltersToMessages object which will be called when
   // ApplyFiltersToHdr finds one or more filters that hit.
   RefPtr<nsMsgApplyFiltersToMessages> filterExecutor =
-      new nsMsgApplyFiltersToMessages(aMsgWindow, filterList, folderList,
+      new nsMsgApplyFiltersToMessages(aMsgWindow, filterList, {aFolder},
                                       aMsgHdrList, aFilterType, aCallback);
 
   if (filterExecutor) return filterExecutor->AdvanceToNextFolder();
