@@ -24,9 +24,8 @@
 #include "nsServiceManagerUtils.h"
 #include "nsMemory.h"
 #include "nsMsgMessageFlags.h"
-#include "nsIArray.h"
-#include "nsArrayUtils.h"
 #include "mozilla/Attributes.h"
+#include "nsIMsgNewsFolder.h"
 
 // This stuff lives in the base class because the IMAP search syntax
 // is used by the Dredd SEARCH command as well as IMAP itself
@@ -80,11 +79,10 @@ NS_IMETHODIMP nsMsgSearchAdapter::OpenResultElement(nsMsgResultElement*) {
 
 NS_IMPL_ISUPPORTS(nsMsgSearchAdapter, nsIMsgSearchAdapter)
 
-nsMsgSearchAdapter::nsMsgSearchAdapter(nsIMsgSearchScopeTerm* scope,
-                                       nsIArray* searchTerms)
-    : m_searchTerms(searchTerms) {
-  m_scope = scope;
-}
+nsMsgSearchAdapter::nsMsgSearchAdapter(
+    nsIMsgSearchScopeTerm* scope,
+    nsTArray<RefPtr<nsIMsgSearchTerm>> const& searchTerms)
+    : m_scope(scope), m_searchTerms(searchTerms.Clone()) {}
 
 nsMsgSearchAdapter::~nsMsgSearchAdapter() {}
 
@@ -196,46 +194,26 @@ nsresult nsMsgSearchAdapter::GetSearchCharsets(nsAString& srcCharset,
   nsresult rv;
   bool forceAsciiSearch = false;
 
-  if (m_defaultCharset.IsEmpty()) {
-    nsCOMPtr<nsIPrefBranch> prefs(
-        do_GetService(NS_PREFSERVICE_CONTRACTID, &rv));
-    if (NS_SUCCEEDED(rv)) {
-      nsCOMPtr<nsIPrefLocalizedString> localizedstr;
-      rv = prefs->GetComplexValue("mailnews.view_default_charset",
-                                  NS_GET_IID(nsIPrefLocalizedString),
-                                  getter_AddRefs(localizedstr));
-      if (NS_SUCCEEDED(rv)) localizedstr->GetData(m_defaultCharset);
-
-      prefs->GetBoolPref("mailnews.force_ascii_search", &forceAsciiSearch);
-    }
+  nsCOMPtr<nsIPrefBranch> prefs(do_GetService(NS_PREFSERVICE_CONTRACTID, &rv));
+  if (NS_SUCCEEDED(rv)) {
+    prefs->GetBoolPref("mailnews.force_ascii_search", &forceAsciiSearch);
   }
-  srcCharset =
-      m_defaultCharset.IsEmpty()
-          ? static_cast<const nsAString&>(NS_LITERAL_STRING("ISO-8859-1"))
-          : m_defaultCharset;
+
+  srcCharset = m_defaultCharset;
+  dstCharset.Assign(srcCharset);
 
   if (m_scope) {
-    // ### DMB is there a way to get the charset for the "window"?
-
     nsCOMPtr<nsIMsgFolder> folder;
     rv = m_scope->GetFolder(getter_AddRefs(folder));
-
-    // Ask the newsgroup/folder for its csid.
     if (NS_SUCCEEDED(rv) && folder) {
-      nsCString folderCharset;
-      folder->GetCharset(folderCharset);
-      dstCharset.Append(NS_ConvertASCIItoUTF16(folderCharset));
+      nsCOMPtr<nsIMsgNewsFolder> newsfolder(do_QueryInterface(folder));
+      if (newsfolder) {
+        nsCString folderCharset;
+        rv = newsfolder->GetCharset(folderCharset);
+        if (NS_SUCCEEDED(rv))
+          dstCharset.Assign(NS_ConvertASCIItoUTF16(folderCharset));
+      }
     }
-  } else
-    dstCharset.Assign(srcCharset);
-
-  // If
-  // the destination is still CS_DEFAULT, make the destination match
-  // the source. (CS_DEFAULT is an indication that the charset
-  // was undefined or unavailable.)
-  // ### well, it's not really anymore. Is there an equivalent?
-  if (dstCharset.Equals(m_defaultCharset)) {
-    dstCharset.Assign(srcCharset);
   }
 
   if (forceAsciiSearch) {
@@ -605,11 +583,9 @@ nsresult nsMsgSearchAdapter::EncodeImapValue(char* encoding, const char* value,
   return NS_OK;
 }
 
-nsresult nsMsgSearchAdapter::EncodeImap(char** ppOutEncoding,
-                                        nsIArray* searchTerms,
-                                        const char16_t* srcCharset,
-                                        const char16_t* destCharset,
-                                        bool reallyDredd) {
+nsresult nsMsgSearchAdapter::EncodeImap(
+    char** ppOutEncoding, nsTArray<RefPtr<nsIMsgSearchTerm>> const& searchTerms,
+    const char16_t* srcCharset, const char16_t* destCharset, bool reallyDredd) {
   // i've left the old code (before using CBoolExpression for debugging purposes
   // to make sure that the new code generates the same encoding string as the
   // old code.....
@@ -617,26 +593,23 @@ nsresult nsMsgSearchAdapter::EncodeImap(char** ppOutEncoding,
   nsresult err = NS_OK;
   *ppOutEncoding = nullptr;
 
-  uint32_t termCount;
-  searchTerms->GetLength(&termCount);
-  uint32_t i = 0;
-
   // create our expression
   nsMsgSearchBoolExpression* expression = new nsMsgSearchBoolExpression();
   if (!expression) return NS_ERROR_OUT_OF_MEMORY;
 
-  for (i = 0; i < termCount && NS_SUCCEEDED(err); i++) {
-    char* termEncoding;
+  for (nsIMsgSearchTerm* pTerm : searchTerms) {
     bool matchAll;
-    nsCOMPtr<nsIMsgSearchTerm> pTerm = do_QueryElementAt(searchTerms, i);
     pTerm->GetMatchAll(&matchAll);
     if (matchAll) continue;
+    char* termEncoding;
     err = EncodeImapTerm(pTerm, reallyDredd, srcCharset, destCharset,
                          &termEncoding);
     if (NS_SUCCEEDED(err) && nullptr != termEncoding) {
       expression = nsMsgSearchBoolExpression::AddSearchTerm(expression, pTerm,
                                                             termEncoding);
       delete[] termEncoding;
+    } else {
+      break;
     }
   }
 
@@ -736,37 +709,6 @@ nsresult nsMsgSearchValidityTable::GetNumAvailAttribs(int32_t* aResult) {
   return NS_OK;
 }
 
-nsresult nsMsgSearchValidityTable::ValidateTerms(nsIArray* searchTerms) {
-  nsresult rv = NS_OK;
-  uint32_t count;
-
-  NS_ENSURE_ARG_POINTER(searchTerms);
-
-  searchTerms->GetLength(&count);
-  for (uint32_t i = 0; i < count; i++) {
-    nsCOMPtr<nsIMsgSearchTerm> pTerm = do_QueryElementAt(searchTerms, i);
-
-    nsIMsgSearchTerm* iTerm = pTerm;
-    nsMsgSearchTerm* term = static_cast<nsMsgSearchTerm*>(iTerm);
-    //    XP_ASSERT(term->IsValid());
-    bool enabled;
-    bool available;
-    rv = GetEnabled(term->m_attribute, term->m_operator, &enabled);
-    NS_ENSURE_SUCCESS(rv, rv);
-    rv = GetAvailable(term->m_attribute, term->m_operator, &available);
-    NS_ENSURE_SUCCESS(rv, rv);
-    if (!enabled || !available) {
-      bool validNotShown;
-      rv = GetValidButNotShown(term->m_attribute, term->m_operator,
-                               &validNotShown);
-      NS_ENSURE_SUCCESS(rv, rv);
-      if (!validNotShown) return NS_MSG_ERROR_INVALID_SEARCH_SCOPE;
-    }
-  }
-
-  return rv;
-}
-
 nsresult nsMsgSearchValidityTable::GetAvailableAttributes(
     nsTArray<nsMsgSearchAttribValue>& aResult) {
   aResult.Clear();
