@@ -43,6 +43,7 @@
 #include "nsIAbManager.h"
 #include "mozilla/ArrayUtils.h"
 #include "mozilla/mailnews/MimeHeaderParser.h"
+#include "mozilla/Utf8.h"
 
 using namespace mozilla::mailnews;
 
@@ -356,6 +357,8 @@ nsMsgSearchTerm::nsMsgSearchTerm(nsMsgSearchAttribValue attrib,
 
   nsMsgResultElement::AssignValues(val, &m_value);
   m_matchAll = false;
+  mBeginsGrouping = false;
+  mEndsGrouping = false;
 }
 
 nsMsgSearchTerm::~nsMsgSearchTerm() {}
@@ -574,9 +577,9 @@ nsresult nsMsgSearchTerm::ParseOperator(char* inStream,
 
   if (commaSep) *commaSep = '\0';
 
-  nsresult err = NS_MsgGetOperatorFromString(inStream, &operatorVal);
-  *value = (nsMsgSearchOpValue)operatorVal;
-  return err;
+  nsresult rv = NS_MsgGetOperatorFromString(inStream, &operatorVal);
+  if (NS_SUCCEEDED(rv)) *value = (nsMsgSearchOpValue)operatorVal;
+  return rv;
 }
 
 // find the attribute code for this comma-delimited attribute.
@@ -850,7 +853,7 @@ nsresult nsMsgSearchTerm::MatchBody(nsIMsgSearchScopeTerm* scope,
       bool softLineBreak = false;
       // Do in-place decoding of quoted printable
       if (bodyHan->IsQP()) {
-        softLineBreak = StringEndsWith(buf, NS_LITERAL_CSTRING("="));
+        softLineBreak = StringEndsWith(buf, "="_ns);
         MsgStripQuotedPrintable(buf);
         // If soft line break, chop off the last char as well.
         size_t bufLength = buf.Length();
@@ -1690,7 +1693,8 @@ nsresult nsMsgSearchScopeTerm::TimeSlice(bool* aDone) {
   return m_adapter->Search(aDone);
 }
 
-nsresult nsMsgSearchScopeTerm::InitializeAdapter(nsIArray* termList) {
+nsresult nsMsgSearchScopeTerm::InitializeAdapter(
+    nsTArray<RefPtr<nsIMsgSearchTerm>> const& termList) {
   if (m_adapter) return NS_OK;
 
   nsresult rv = NS_OK;
