@@ -338,16 +338,12 @@ function test_permissions_panel(aWin) {
                      "offline-app", Services.perms.ALLOW_ACTION);
   Services.perms.add(Services.io.newURI("http://popup.getpersonas.com/"),
                      "popup", Services.perms.ALLOW_ACTION);
-  Services.perms.add(Services.io.newURI("http://stsuse.getpersonas.com/"),
-                     "sts/use", Services.perms.ALLOW_ACTION);
-  Services.perms.add(Services.io.newURI("http://stssubd.getpersonas.com/"),
-                     "sts/subd", Services.perms.ALLOW_ACTION);
   Services.perms.add(Services.io.newURI("http://test.getpersonas.com/"),
                      "test", Services.perms.DENY_ACTION);
   Services.perms.add(Services.io.newURI("http://xul.getpersonas.com/"),
                      "allowXULXBL", Services.perms.ALLOW_ACTION);
   Services.logins.setLoginSavingEnabled("password.getpersonas.com", false);
-  is(aWin.gPerms.list.children.length, 14,
+  is(aWin.gPerms.list.children.length, 12,
      "The correct number of permissions is displayed in the list");
   for (let i = 1; i < aWin.gPerms.list.children.length; i++) {
     let perm = aWin.gPerms.list.children[i];
@@ -433,25 +429,6 @@ function test_permissions_panel(aWin) {
         is(perm.capability, 1,
            "Set back to correct default");
         break;
-      case "sts/use":
-        is(perm.getAttribute("label"), "Use Strict Transport Security",
-           "Correct label for type: " + perm.type);
-        is(perm.capability, 1,
-           "Correct capability for: " + perm.host);
-        perm.useDefault(true);
-        is(perm.capability, 0,
-           "Set back to correct default");
-        break;
-      case "sts/subd":
-        is(perm.getAttribute("label"), "Apply Strict Transport Security to subdomains",
-           "Correct label for type: " + perm.type);
-        is(perm.capability, 1,
-           "Correct capability for: " + perm.host);
-        perm.useDefault(true);
-        is(perm.capability, 0,
-           "Set back to correct default");
-        break;
-      default:
         is(perm.getAttribute("label"), perm.type,
            "Correct default label for type: " + perm.type);
         is(perm.capability, 2,
