@@ -4,10 +4,14 @@
 
 const XULNS = "http://www.mozilla.org/keymaster/gatekeeper/there.is.only.xul";
 
-var {Services} = ChromeUtils.import("resource://gre/modules/Services.jsm");
-var {XPCOMUtils} = ChromeUtils.import("resource://gre/modules/XPCOMUtils.jsm");
-ChromeUtils.import("resource:///modules/mailnewsMigrator.js");
-ChromeUtils.import("resource:///modules/extensionSupport.jsm");
+var { Services } = ChromeUtils.import("resource://gre/modules/Services.jsm");
+var { XPCOMUtils } = ChromeUtils.import("resource://gre/modules/XPCOMUtils.jsm");
+var { migrateMailnews } =
+  ChromeUtils.import("resource:///modules/mailnewsMigrator.js");
+var { ExtensionSupport } = 
+  ChromeUtils.import("resource:///modules/ExtensionSupport.jsm");
+var { LightweightThemeConsumer } =
+  ChromeUtils.import("resource://gre/modules/LightweightThemeConsumer.jsm");
 
 XPCOMUtils.defineLazyModuleGetters(this, {
   AddonManager: "resource://gre/modules/AddonManager.jsm",
@@ -19,6 +23,7 @@ XPCOMUtils.defineLazyModuleGetters(this, {
   PlacesBackups: "resource://gre/modules/PlacesBackups.jsm",
   AsyncShutdown: "resource://gre/modules/AsyncShutdown.jsm",
   AutoCompletePopup: "resource://gre/modules/AutoCompletePopup.jsm",
+  DateTimePickerHelper: "resource://gre/modules/DateTimePickerHelper.jsm",
   BookmarkHTMLUtils: "resource://gre/modules/BookmarkHTMLUtils.jsm",
   BookmarkJSONUtils: "resource://gre/modules/BookmarkJSONUtils.jsm",
   RecentWindow: "resource:///modules/RecentWindow.jsm",
@@ -37,9 +42,9 @@ XPCOMUtils.defineLazyGetter(this, "DebuggerServer", () => {
   return tmp.require("devtools/server/main").DebuggerServer;
 });
 
-const global = this;
+var global = this;
 
-const listeners = {
+var listeners = {
   mm: {
     // PLEASE KEEP THIS LIST IN SYNC WITH THE MOBILE LISTENERS IN nsBrowserGlue.js
     "RemoteLogins:findLogins": ["LoginManagerParent"],
@@ -192,6 +197,15 @@ SuiteGlue.prototype = {
       case "profile-after-change":
         this._onProfileAfterChange();
         break;
+      case "chrome-document-global-created":
+        // Set up lwt, but only if the "lightweightthemes" attr is set on the root
+        // (i.e. in messenger.xul).
+        subject.addEventListener("DOMContentLoaded", () => {
+          if (subject.document.documentElement.hasAttribute("lightweightthemes")) {
+            new LightweightThemeConsumer(subject.document);
+          }
+        }, {once: true});
+        break;
       case "final-ui-startup":
         this._onProfileStartup();
         this._promptForMasterPassword();
@@ -199,11 +213,17 @@ SuiteGlue.prototype = {
         Services.search.init();
         listeners.init();
 
-        Cc["@mozilla.org/globalmessagemanager;1"]
-          .getService(Ci.nsIMessageListenerManager)
-          .loadFrameScript("chrome://navigator/content/content.js", true);
+        Services.mm.loadFrameScript("chrome://navigator/content/content.js",
+                                    true);
         ChromeUtils.import("resource://gre/modules/NotificationDB.jsm");
         break;
+      case "browser-delayed-startup-finished":
+         // Intended fallthrough.
+      case "mail-startup-done":
+        Services.obs.removeObserver(this, "browser-delayed-startup-finished");
+        Services.obs.removeObserver(this, "mail-startup-done");
+        this._onFirstWindowLoaded(subject);
+        break;
       case "sessionstore-windows-restored":
         this._onBrowserStartup(subject);
         break;
@@ -248,11 +268,6 @@ SuiteGlue.prototype = {
 
         Services.obs.removeObserver(this, "places-init-complete");
         break;
-      case "places-shutdown":
-        Services.obs.removeObserver(this, "places-shutdown");
-        // places-shutdown is fired when the profile is about to disappear.
-        this._onPlacesShutdown();
-        break;
       case "idle":
         this._backupBookmarks();
         break;
@@ -331,6 +346,8 @@ SuiteGlue.prototype = {
     Services.obs.addObserver(this, "profile-before-change", true);
     Services.obs.addObserver(this, "profile-after-change", true);
     Services.obs.addObserver(this, "final-ui-startup", true);
+    Services.obs.addObserver(this, "browser-delayed-startup-finished", true);
+    Services.obs.addObserver(this, "mail-startup-done", true);
     Services.obs.addObserver(this, "sessionstore-windows-restored", true);
     Services.obs.addObserver(this, "browser:purge-session-history", true);
     Services.obs.addObserver(this, "quit-application-requested", true);
@@ -342,9 +359,9 @@ SuiteGlue.prototype = {
     Services.obs.addObserver(this, "weave:engine:clients:display-uri", true);
     Services.obs.addObserver(this, "session-save", true);
     Services.obs.addObserver(this, "places-init-complete", true);
-    Services.obs.addObserver(this, "places-shutdown", true);
     Services.obs.addObserver(this, "browser-search-engine-modified", true);
     Services.obs.addObserver(this, "notifications-open-settings", true);
+    Services.obs.addObserver(this, "chrome-document-global-created", true);
     Services.prefs.addObserver("devtools.debugger.", this, true);
     Services.obs.addObserver(this, "handle-xul-text-link", true);
     Cc['@mozilla.org/docloaderservice;1']
@@ -354,6 +371,10 @@ SuiteGlue.prototype = {
 
   // cleanup (called on application shutdown)
   _dispose: function BG__dispose() {
+    try {
+      Services.obs.removeObserver(this, "chrome-document-global-created");
+    }
+    catch (ex) {}
     if (this._isIdleObserver) {
       this._idleService.removeIdleObserver(this, BOOKMARKS_BACKUP_IDLE_TIME_SEC);
       delete this._isIdleObserver;
@@ -375,34 +396,28 @@ SuiteGlue.prototype = {
   _onProfileStartup: function()
   {
     this._migrateUI();
+    this._migrateUI2();
     migrateMailnews(); // mailnewsMigrator.js
 
-    Sanitizer.checkSettings();
-    Sanitizer.doPendingSanitize();
-
-    if (Services.prefs.prefHasUserValue("privacy.sanitize.didShutdownSanitize")) {
-      Services.prefs.clearUserPref("privacy.sanitize.didShutdownSanitize");
-      // We need to persist this preference change, since we want to
-      // check it at next app start even if the browser exits abruptly
-      Services.prefs.savePrefFile(null);
-    }
+    Sanitizer.onStartup();
 
     var timer = Cc["@mozilla.org/timer;1"]
                   .createInstance(Ci.nsITimer);
     timer.init(this, 3000, timer.TYPE_ONE_SHOT);
   },
 
-  _migrateUI: function()
-  {
-    const UI_VERSION = 6;
+  /**
+   * Determine if the UI has been upgraded for this release. If not
+   * reset or migrate some user configurations depending on the migration
+   * level.
+   */
+  _migrateUI() {
+    const UI_VERSION = 9;
 
     // If the pref is not set this is a new or pre SeaMonkey 2.49 profile.
     // We can't tell so we just run migration with version 0.
-    let currentUIVersion = 0;
-
-    if (Services.prefs.prefHasUserValue("suite.migration.version")) {
-      currentUIVersion = Services.prefs.getIntPref("suite.migration.version");
-    }
+    let currentUIVersion =
+      Services.prefs.getIntPref("suite.migration.version", 0);
 
     if (currentUIVersion >= UI_VERSION)
       return;
@@ -469,7 +484,7 @@ SuiteGlue.prototype = {
       } catch (ex) {}
     }
 
-    // Pretend currentUIVersion 3 never happend (used in 2.57 for a time and became 6).
+    // Pretend currentUIVersion 3 never happened (used in 2.57 for a time).
 
     // Remove obsolete download preferences set by user.
     if (currentUIVersion < 4) {
@@ -498,7 +513,7 @@ SuiteGlue.prototype = {
       } catch (ex) {}
     }
 
-    if (currentUIVersion < 6) {
+    if (currentUIVersion < 5) {
       // Delete obsolete ssl and strict transport security permissions.
       let perms = Services.perms.enumerator;
       while (perms.hasMoreElements()) {
@@ -510,7 +525,99 @@ SuiteGlue.prototype = {
           Services.perms.removePermission(perm);
         }
       }
+    }
 
+    // Pretend currentUIVersion 6 and 7 never happened (used in 2.57 for a
+    // time).
+
+    // Migrate sanitizer options.
+    if (currentUIVersion < 8) {
+      const prefs = [ "history", "urlbar", "formdata", "passwords",
+                      "downloads", "cookies", "cache", "sessions",
+                      "offlineApps" ];
+
+      for (let pref of prefs) {
+        try {
+          let prefOld = "privacy.item." + pref;
+
+          // Migrate user value otherwise use default.
+          // Only the names have changed but not the default values.
+          if (Services.prefs.prefHasUserValue(prefOld)) {
+            let prefCpd = "privacy.cpd." + pref;
+            let prefShutdown = "privacy.clearOnShutdown." + pref;
+
+            // If it has a value this should never fail.
+            let oldValue = Services.prefs.getBoolPref(prefOld);
+            Services.prefs.setBoolPref(prefCpd, oldValue);
+            Services.prefs.setBoolPref(prefShutdown, oldValue);
+            Services.prefs.clearUserPref(prefOld);
+          }
+        } catch (ex) {
+          // Better safe than sorry.
+          Cu.reportError(ex);
+        }
+      }
+
+      // We might bring this back later but currently set to default.
+      Services.prefs.clearUserPref("privacy.sanitize.promptOnSanitize");
+
+      // As a precaution set to default if the user has enabled
+      // clearing data on shutdown because there will no longer be
+      // a possible prompt.
+      Services.prefs.clearUserPref("privacy.sanitize.sanitizeOnShutdown");
+    }
+
+    // Migrate mail tab options.
+    if (currentUIVersion < 9) {
+      const tabPrefs = [ "autoHide", "opentabfor.doubleclick",
+                         "opentabfor.middleclick" ];
+      for (let pref of tabPrefs) {
+        try {
+          let prefBT = "browser.tabs." + pref;
+
+          // Copy user value otherwise use default.
+          if (Services.prefs.prefHasUserValue(prefBT)) {
+            let prefMT = "mail.tabs." + pref;
+
+            // If it has a value this should never fail.
+            let valueBT = Services.prefs.getBoolPref(prefBT);
+            Services.prefs.setBoolPref(prefMT, valueBT);
+          }
+        } catch (ex) {
+          // Better safe than sorry.
+          Cu.reportError(ex);
+        }
+      }
+
+      // We might bring this back later but currently set to default.
+      Services.prefs.clearUserPref("browser.tabs.opentabfor.doubleclick");
+    }
+
+    // Update the migration version.
+    Services.prefs.setIntPref("suite.migration.version", UI_VERSION);
+  },
+
+  /**
+   * Determine if the UI has been upgraded for this 2.57 or later release.
+   * If not reset or migrate some user configurations depending on the
+   * migration level.
+   * Only migration steps for 2.57 and higher are included in this function.
+   * When the 2.53 branch is retired this function can be merged with
+   * _migrateUI again.
+   */
+  _migrateUI2() {
+    const UI_VERSION2 = 1;
+
+    // If the pref is not set this is a new or pre SeaMonkey 2.57 profile.
+    // We can't tell so we just run migration with version 0.
+    let currentUIVersion2 =
+      Services.prefs.getIntPref("suite.migration.version2", 0);
+
+    if (currentUIVersion2 >= UI_VERSION2)
+      return;
+
+    // Run any migrations due prior to 2.57.
+    if (currentUIVersion2 < 1) {
       // The XUL directory viewer is no longer provided.
       try {
         if (Services.prefs.getIntPref("network.dir.format") == 3) {
@@ -520,7 +627,7 @@ SuiteGlue.prototype = {
     }
 
     // Update the migration version.
-    Services.prefs.setIntPref("suite.migration.version", UI_VERSION);
+    Services.prefs.setIntPref("suite.migration.version2", UI_VERSION2);
   },
 
   // Copies additional profile files from the default profile tho the current profile.
@@ -583,13 +690,7 @@ SuiteGlue.prototype = {
   },
 
   // Browser startup complete. All initial windows have opened.
-  _onBrowserStartup: function(aWindow)
-  {
-    AutoCompletePopup.init();
-
-    if (Services.prefs.getBoolPref("plugins.update.notifyUser"))
-      this._showPluginUpdatePage(aWindow);
-
+  _onBrowserStartup: function(aWindow) {
     // For any add-ons that were installed disabled and can be enabled offer
     // them to the user.
     var browser = aWindow.getBrowser();
@@ -612,14 +713,6 @@ SuiteGlue.prototype = {
     if (this._shouldShowRights())
       this._showRightsNotification(notifyBox);
 
-    if ("@mozilla.org/windows-taskbar;1" in Cc &&
-        Cc["@mozilla.org/windows-taskbar;1"]
-          .getService(Ci.nsIWinTaskbar).available) {
-      let temp = {};
-      ChromeUtils.import("resource:///modules/WindowsJumpLists.jsm", temp);
-      temp.WinTaskbarJumpList.startup();
-    }
-
     // Load the "more info" page for a locked places.sqlite
     // This property is set earlier by places-database-locked topic.
     if (this._isPlacesDatabaseLocked) {
@@ -631,6 +724,20 @@ SuiteGlue.prototype = {
       notifyBox.showUpdateWarning();
 
     this._checkForDefaultClient(aWindow);
+  },
+
+  // First mail or browser window loaded.
+  _onFirstWindowLoaded: function(aWindow) {
+    AutoCompletePopup.init();
+    DateTimePickerHelper.init();
+
+    if ("@mozilla.org/windows-taskbar;1" in Cc &&
+        Cc["@mozilla.org/windows-taskbar;1"]
+          .getService(Ci.nsIWinTaskbar).available) {
+      let temp = {};
+      ChromeUtils.import("resource:///modules/WindowsJumpLists.jsm", temp);
+      temp.WinTaskbarJumpList.startup();
+    }
 
     // Initialize the download manager after the app starts so that
     // auto-resume downloads begin (such as after crashing or quitting with
@@ -658,8 +765,8 @@ SuiteGlue.prototype = {
     if (this._saveSession) {
       this._setPrefToSaveSession();
     }
-    Sanitizer.checkSettings();
     AutoCompletePopup.uninit();
+    DateTimePickerHelper.uninit();
   },
 
   _promptForMasterPassword: function()
@@ -720,6 +827,7 @@ SuiteGlue.prototype = {
     var pagecount = 0;
     var browserEnum = Services.wm.getEnumerator("navigator:browser");
     while (browserEnum.hasMoreElements()) {
+      // XXXbz should we skip closed windows here?
       windowcount++;
 
       var browser = browserEnum.getNext();
@@ -817,14 +925,6 @@ SuiteGlue.prototype = {
     }
   },
 
-  _showPluginUpdatePage: function(aWindow) {
-    Services.prefs.setBoolPref("plugins.update.notifyUser", false);
-
-    var url = Services.urlFormatter.formatURLPref("plugins.update.url");
-
-    aWindow.getBrowser().addTab(url, { focusNewTab: true });
-  },
-
   /*
    * _shouldShowRights - Determines if the user should be shown the
    * about:rights notification. The notification should *not* be shown if
@@ -1099,16 +1199,6 @@ SuiteGlue.prototype = {
     });
   },
 
-  /**
-   * Places shut-down tasks
-   * - finalize components depending on Places.
-   */
-  _onPlacesShutdown: function() {
-    if (!Sanitizer.doPendingSanitize()) {
-      Services.prefs.setBoolPref("privacy.sanitize.didShutdownSanitize", true);
-    }
-  },
-
   /**
    * If a backup for today doesn't exist, this creates one.
    */
@@ -1146,13 +1236,9 @@ SuiteGlue.prototype = {
       }
     } catch (ex) {}
 
-    // try to get dictionary preference and initialize with blank if not set
+    // Try to get dictionary preference and adjust if not valid.
     var prefName = "spellchecker.dictionary";
-    var prefValue = "";
-
-    try {
-      prefValue = Services.prefs.getCharPref(prefName);
-    } catch (ex) {}
+    var prefValue = Services.prefs.getCharPref(prefName);
 
     // replace underscore with dash if found in language
     if (/_/.test(prefValue)) {
@@ -1162,12 +1248,10 @@ SuiteGlue.prototype = {
 
     var spellChecker = Cc["@mozilla.org/spellchecker/engine;1"]
                          .getService(Ci.mozISpellCheckingEngine);
-    var o1 = {};
-    spellChecker.getDictionaryList(o1, {});
-    var dictList = o1.value;
+    var dictList = spellChecker.getDictionaryList();
     // If the preference contains an invalid dictionary, set it to a valid
     // dictionary, any dictionary will do.
-    if (dictList.length && dictList.indexOf(prefValue) < 0)
+    if (dictList.length && !dictList.includes(prefValue))
       Services.prefs.setCharPref(prefName, dictList[0]);
   },
 
@@ -1264,7 +1348,8 @@ SuiteGlue.prototype = {
                         .createInstance(Ci.nsISupportsString);
       argString.data = "";
       gDownloadManager = Services.ww.openWindow(null, DOWNLOAD_MANAGER_URL,
-                                                null, "all,dialog=no",
+                                                null,
+                                                "all,dialog=no,non-private",
                                                 argString);
       gDownloadManager.addEventListener("load", function() {
         gDownloadManager.addEventListener("unload", function() {
@@ -1287,11 +1372,8 @@ SuiteGlue.prototype = {
     }
   },
 
-  sanitize: function(aParentWindow)
-  {
-    // call the Sanitizer object's sanitize, which might return errors
-    // but do not forward them anywhere, as we are defined as void here
-    Sanitizer.sanitize(aParentWindow);
+  sanitize(aParentWindow) {
+    Sanitizer.showUI(aParentWindow);
   },
 
   async ensurePlacesDefaultQueriesInitialized() {
@@ -1476,7 +1558,7 @@ SuiteGlue.prototype = {
  * PermissionUI.jsm for an example of how to provide a new prompt
  * from an add-on.
  */
-const ContentPermissionIntegration = {
+var ContentPermissionIntegration = {
   /**
    * Creates a PermissionPrompt for a given permission type and
    * nsIContentPermissionRequest.
