@@ -4,6 +4,9 @@
  * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
 const {MailServices} = ChromeUtils.import("resource:///modules/MailServices.jsm");
+const {PluralForm} = ChromeUtils.import("resource://gre/modules/PluralForm.jsm");
+const {Services} = ChromeUtils.import("resource://gre/modules/Services.jsm");
+
 
 //NOTE: gMessengerBundle must be defined and set or this Overlay won't work
 
@@ -68,8 +71,12 @@ function RestoreSelectionWithoutContentLoad(tree)
  * on a non-selected row doesn't move the selection.
  * @param aTarget the target of the popup event
  */
-function MailContextOnPopupHiding(aTarget)
-{
+function MailContextOnPopupHiding(aTarget, aEvent) {
+  // Don't do anything if it's a submenu's onpopuphiding that's just bubbling
+  // up to the top.
+  if (aEvent.target != aTarget)
+    return;
+
   gContextMenu.hiding();
   gContextMenu = null;
   if (InThreadPane(aTarget))
@@ -99,8 +106,11 @@ function InThreadPane(aTarget)
  * @param aTarget the target of the popup event
  * @return true always
  */
-function FillMailContextMenu(aTarget, aEvent)
-{
+function FillMailContextMenu(aTarget, aEvent) {
+  // If the popupshowing was for a submenu, we don't need to do anything.
+  if (aEvent.target != aTarget)
+    return true;
+
   var inThreadPane = InThreadPane(aTarget);
   gContextMenu = new nsContextMenu(aTarget);
 
@@ -137,6 +147,23 @@ function FillMailContextMenu(aTarget, aEvent)
                inThreadPane || (numSelected > 1));
 
   ShowMenuItem("mailContext-editAsNew", showMailItems && oneOrMore);
+  // Show "Edit Draft Message" menus only in a drafts folder;
+  // otherwise hide them.
+  let showEditDraft = showCommandInSpecialFolder("cmd_editDraftMsg",
+                                                 Ci.nsMsgFolderFlags.Drafts);
+  ShowMenuItem("mailContext-editDraftMsg",
+               showMailItems && oneOrMore && showEditDraft);
+  // Show "New Message from Template" and "Edit Template" menus only in a
+  // templates folder; otherwise hide them.
+  let showTemplates = showCommandInSpecialFolder("cmd_newMsgFromTemplate",
+                                                 Ci.nsMsgFolderFlags.Templates);
+  ShowMenuItem("mailContext-newMsgFromTemplate",
+               showMailItems && oneOrMore && showTemplates);
+  showTemplates = showCommandInSpecialFolder("cmd_editTemplateMsg",
+                                             Ci.nsMsgFolderFlags.Templates);
+  ShowMenuItem("mailContext-editTemplateMsg",
+               showMailItems && oneOrMore && showTemplates);
+
   ShowMenuItem("mailContext-replySender", showMailItems && single);
   ShowMenuItem("mailContext-replyList",
                showMailItems && single && !isNewsgroup && IsListPost());
@@ -185,192 +212,284 @@ function FillMailContextMenu(aTarget, aEvent)
   ShowMenuItem("context-createfilterfrom", gContextMenu.onMailtoLink);
 
   // Figure out separators.
-  ShowSeparator("mailContext-sep-open");
-  ShowSeparator("mailContext-sep-edit");
-  ShowSeparator("mailContext-sep-link");
-  ShowSeparator("mailContext-sep-image");
-  ShowSeparator("mailContext-sep-copy");
-  ShowSeparator("mailContext-sep-print");
-  ShowSeparator("mailContext-sep-tags");
-  ShowSeparator("mailContext-sep-mark");
-  ShowSeparator("mailContext-sep-move");
-
-  // If we are on a link, go ahead and hide this separator.
-  if (gContextMenu.onLink)
-    ShowMenuItem("mailContext-sep-copy", false);
+  initSeparators();
 
   return true;
 }
 
+/**
+ * Hide separators with no active menu items.
+ *
+ */
+function initSeparators() {
+  const mailContextSeparators = [
+    "mailContext-sep-link", "mailContext-sep-open",
+    "mailContext-sep-tags", "mailContext-sep-mark",
+    "mailContext-sep-move", "mailContext-sep-print",
+    "mailContext-sep-edit", "mailContext-sep-image",
+    "mailContext-sep-blockimage", "mailContext-sep-copy",
+  ];
+
+  mailContextSeparators.forEach(hideIfAppropriate);
+}
+
+/**
+ * Hide a separator based on whether there are any non-hidden items between
+ * it and the previous separator.
+ *
+ * @param aID  The id of the separator element.
+ */
+function hideIfAppropriate(aID) {
+  let separator = document.getElementById(aID);
+
+  function hasAVisibleNextSibling(aNode) {
+    let sibling = aNode.nextSibling;
+    while (sibling) {
+      if (sibling.getAttribute("hidden") != "true" &&
+          sibling.localName != "menuseparator")
+        return true;
+      sibling = sibling.nextSibling;
+    }
+    return false;
+  }
+
+  let sibling = separator.previousSibling;
+  while (sibling) {
+    if (sibling.getAttribute("hidden") != "true") {
+      ShowMenuItem(aID, sibling.localName != "menuseparator" &&
+                        hasAVisibleNextSibling(separator));
+      return;
+    }
+    sibling = sibling.previousSibling;
+  }
+  ShowMenuItem(aID, false);
+}
+
 function FolderPaneOnPopupHiding()
 {
-  RestoreSelectionWithoutContentLoad(GetFolderTree());
+  RestoreSelectionWithoutContentLoad(document.getElementById("folderTree"));
 }
 
 function FillFolderPaneContextMenu()
 {
-  let folders = GetSelectedMsgFolders();
+  // Do not show menu if rows are selected.
+  let folders = gFolderTreeView.getSelectedFolders();
   let numSelected = folders.length;
-  if (numSelected != 1)
+  if (!numSelected)
     return false;
 
+  function checkIsVirtualFolder(folder) {
+    return folder.getFlag(Ci.nsMsgFolderFlags.Virtual);
+  }
+  let haveAnyVirtualFolders = folders.some(checkIsVirtualFolder);
+
+  function checkIsServer(folder) {
+    return folder.isServer;
+  }
+  let selectedServers = folders.filter(checkIsServer);
+
   let folder = folders[0];
   let isServer = folder.isServer;
   let serverType = folder.server.type;
-  let specialFolder = getSpecialFolderString(folder);
-  var canSubscribeToFolder = (serverType == "nntp") ||
-                             (serverType == "imap") ||
-                             (serverType == "rss");
-  var isNewsgroup = !isServer && serverType == 'nntp';
-  var isMailFolder = !isServer && serverType != 'nntp';
-  var isVirtualFolder = (specialFolder == "Virtual");
-  const kTrashFlag = Ci.nsMsgFolderFlags.Trash;
-  let isChildOfTrash = folder.isSpecialFolder(kTrashFlag, true);
-  var canGetMessages =
-    (isServer && serverType != "none") ||
-    isNewsgroup ||
-    ((serverType == "rss") && !isChildOfTrash && !isVirtualFolder);
+  let specialFolder = haveAnyVirtualFolders ? "Virtual" :
+                                              FolderUtils.getSpecialFolderString(folder);
 
-  if (!isServer)
-  {
+  function checkCanSubscribeToFolder(folder) {
+    if (checkIsVirtualFolder(folder))
+      return false;
+
+    // All feed account folders, besides Trash, are subscribable.
+    if (folder.server.type == "rss" &&
+        !folder.getFlag(Ci.nsMsgFolderFlags.Trash))
+      return true;
+
+    // We only want the subscribe item on the account nodes.
+    if (!folder.isServer)
+      return false;
+
+    return folder.server.type == "nntp" ||
+           folder.server.type == "imap";
+  }
+  let haveOnlySubscribableFolders = folders.every(checkCanSubscribeToFolder);
+
+  function checkIsNewsgroup(folder) {
+    return !folder.isServer && folder.server.type == "nntp" &&
+           !folder.getFlag(Ci.nsMsgFolderFlags.Virtual);
+  }
+  let haveOnlyNewsgroups = folders.every(checkIsNewsgroup);
+
+  function checkIsMailFolder(folder) {
+    return !folder.isServer && folder.server.type != "nntp";
+  }
+  let haveOnlyMailFolders = folders.every(checkIsMailFolder);
+
+  function checkCanGetMessages(folder) {
+    return (folder.isServer && (folder.server.type != "none")) ||
+            checkIsNewsgroup(folder) ||
+            ((folder.server.type == "rss") &&
+             !folder.isSpecialFolder(Ci.nsMsgFolderFlags.Trash, true) &&
+             !checkIsVirtualFolder(folder));
+  }
+  let selectedFoldersThatCanGetMessages = folders.filter(checkCanGetMessages);
+
+  // --- Set up folder properties / account settings menu item.
+  if (numSelected != 1) {
+    ShowMenuItem("folderPaneContext-settings", false);
+    ShowMenuItem("folderPaneContext-properties", false);
+  }
+  else if (selectedServers.length != 1) {
     ShowMenuItem("folderPaneContext-settings", false);
     ShowMenuItem("folderPaneContext-properties", true);
-    EnableMenuItem("folderPaneContext-properties", true);
   }
-  else
-  {
+  else {
     ShowMenuItem("folderPaneContext-properties", false);
     ShowMenuItem("folderPaneContext-settings", true);
-    EnableMenuItem("folderPaneContext-settings", true);
   }
 
-  if ((numSelected <= 1) && canGetMessages)
-    if (isServer)
+  // --- Set up the get messages menu item.
+  // Show if only servers, or it's only newsgroups/feeds. We could mix,
+  // but it gets messy for situations where both server and a folder
+  // on the server are selected.
+  let showGet = selectedFoldersThatCanGetMessages.length == numSelected;
+  ShowMenuItem("folderPaneContext-getMessages", showGet);
+  if (showGet) {
+    if (selectedServers.length > 0 &&
+        selectedServers.length == selectedFoldersThatCanGetMessages.length) {
       SetMenuItemLabel("folderPaneContext-getMessages",
                        gMessengerBundle.getString("getMessagesFor"));
-    else
+    }
+    else {
       SetMenuItemLabel("folderPaneContext-getMessages",
                        gMessengerBundle.getString("getMessages"));
+    }
+  }
 
-  ShowMenuItem("folderPaneContext-getMessages", (numSelected <= 1) && canGetMessages);
-  EnableMenuItem("folderPaneContext-getMessages", true);
+  // --- Setup the Mark All Folders Read menu item.
+  // Show only in case the server item is selected.
+  ShowMenuItem("folderPaneContext-markAllFoldersRead",
+               selectedServers.length > 0);
 
-  ShowMenuItem("folderPaneContext-sep4", (numSelected <= 1) && isServer);
-  ShowMenuItem("folderPaneContext-markAllFoldersRead", (numSelected <= 1) && isServer);
+  // --- Set up new sub/folder menu item.
+  let isInbox = specialFolder == "Inbox";
+  let showNew =
+    (numSelected == 1) &&
+    ((serverType != "nntp" && folder.canCreateSubfolders) || isInbox);
+  ShowMenuItem("folderPaneContext-new", showNew);
+  if (showNew) {
+    EnableMenuItem("folderPaneContext-new",
+                   serverType != "imap" || !Services.io.offline);
+    let label = (isServer || isInbox) ? "newFolder" : "newSubfolder";
+    SetMenuItemLabel("folderPaneContext-new",
+                     gMessengerBundle.getString(label));
+  }
 
-  ShowMenuItem("folderPaneContext-openNewWindow", (numSelected <= 1));
-  EnableMenuItem("folderPaneContext-openNewWindow", true);
+  // --- Set up rename menu item.
+  let canRename = (numSelected == 1) && !isServer && folder.canRename &&
+                  (specialFolder == "none" || specialFolder == "Virtual" ||
+                   (specialFolder == "Junk" &&
+                    CanRenameDeleteJunkMail(folder.URI)));
+  ShowMenuItem("folderPaneContext-rename", canRename);
+  if (canRename) {
+    EnableMenuItem("folderPaneContext-rename",
+                   !isServer && folder.isCommandEnabled("cmd_renameFolder"));
+    SetMenuItemLabel("folderPaneContext-rename",
+                     gMessengerBundle.getString("renameFolder"));
+  }
 
-  ShowMenuItem("folderPaneContext-openNewTab", (numSelected <= 1));
-  EnableMenuItem("folderPaneContext-openNewTab", true);
+  // --- Set up the delete folder menu item.
+  function checkCanDeleteFolder(folder) {
+    if (folder.isSpecialFolder(Ci.nsMsgFolderFlags.Junk, false))
+      return CanRenameDeleteJunkMail(folder.URI);
+    return folder.deletable;
+  }
+  let haveOnlyDeletableFolders = folders.every(checkCanDeleteFolder);
+  ShowMenuItem("folderPaneContext-remove",
+               haveOnlyDeletableFolders && numSelected == 1);
+  if (haveOnlyDeletableFolders && numSelected == 1)
+    SetMenuItemLabel("folderPaneContext-remove",
+                     gMessengerBundle.getString("removeFolder"));
 
-  SetupRenameMenuItem(folder, numSelected, isServer, serverType, specialFolder);
-  SetupRemoveMenuItem(folder, numSelected, isServer, serverType, specialFolder);
-  SetupCompactMenuItem(folder, numSelected);
-  SetupFavoritesMenuItem(folder, numSelected, isServer, "folderPaneContext-favoriteFolder");
+  function checkIsDeleteEnabled(folder) {
+    return folder.isCommandEnabled("cmd_delete");
+  }
+  let haveOnlyDeleteEnabledFolders = folders.every(checkIsDeleteEnabled);
+  EnableMenuItem("folderPaneContext-remove", haveOnlyDeleteEnabledFolders);
 
-  ShowMenuItem("folderPaneContext-emptyTrash", (numSelected <= 1) && (specialFolder == 'Trash'));
-  EnableMenuItem("folderPaneContext-emptyTrash", true);
-  ShowMenuItem("folderPaneContext-emptyJunk", (numSelected <= 1) && (specialFolder == 'Junk'));
-  EnableMenuItem("folderPaneContext-emptyJunk", true);
+  // --- Set up the compact folder menu item.
+  function checkCanCompactFolder(folder) {
+    return folder.canCompact &&
+           !folder.getFlag(Ci.nsMsgFolderFlags.Virtual) &&
+           folder.isCommandEnabled("cmd_compactFolder");
+  }
+  let haveOnlyCompactableFolders = folders.every(checkCanCompactFolder);
+  ShowMenuItem("folderPaneContext-compact", haveOnlyCompactableFolders);
+  if (haveOnlyCompactableFolders)
+    SetMenuItemLabel("folderPaneContext-compact",
+                     PluralForm.get(numSelected, gMessengerBundle.getString("compactFolders")));
 
-  var showSendUnsentMessages = (numSelected <= 1) && (specialFolder == 'Outbox');
-  ShowMenuItem("folderPaneContext-sendUnsentMessages", showSendUnsentMessages);
-  if (showSendUnsentMessages)
-    EnableMenuItem("folderPaneContext-sendUnsentMessages", IsSendUnsentMsgsEnabled(folder));
+  function checkIsCompactEnabled(folder) {
+    return folder.isCommandEnabled("cmd_compactFolder");
+  }
+  let haveOnlyCompactEnabledFolders = folders.every(checkIsCompactEnabled);
+  EnableMenuItem("folderPaneContext-compact", haveOnlyCompactEnabledFolders);
 
-  ShowMenuItem("folderPaneContext-sep-edit", (numSelected <= 1));
-
-  SetupNewMenuItem(folder, numSelected, isServer, serverType, specialFolder);
-
-  ShowMenuItem("folderPaneContext-subscribe", (numSelected <= 1) && canSubscribeToFolder && !isVirtualFolder);
-  EnableMenuItem("folderPaneContext-subscribe", true);
-
-  ShowMenuItem("folderPaneContext-sep1", (numSelected <= 1) && !isServer);
-// News folder context menu =============================================
-
-  ShowMenuItem("folderPaneContext-newsUnsubscribe", (numSelected <= 1) && canSubscribeToFolder && isNewsgroup);
-  EnableMenuItem("folderPaneContext-newsUnsubscribe", true);
-  ShowMenuItem("folderPaneContext-markNewsgroupAllRead", (numSelected <= 1) && isNewsgroup);
-  EnableMenuItem("folderPaneContext-markNewsgroupAllRead", true);
-
-// End of News folder context menu =======================================
-
-  ShowMenuItem("folderPaneContext-markMailFolderAllRead", (numSelected <= 1) && isMailFolder && !isVirtualFolder);
-  EnableMenuItem("folderPaneContext-markMailFolderAllRead", true);
-
-  ShowMenuItem("folderPaneContext-searchMessages", (numSelected <= 1) && !isVirtualFolder);
-  goUpdateCommand('cmd_search');
-
-  return(true);
-}
-
-function SetupRenameMenuItem(msgFolder, numSelected, isServer, serverType, specialFolder)
-{
-  var canRename = (specialFolder == "Junk") ?
-                  CanRenameDeleteJunkMail(msgFolder.URI) : msgFolder.canRename;
-  ShowMenuItem("folderPaneContext-rename", (numSelected <= 1) && canRename);
-  EnableMenuItem("folderPaneContext-rename", !isServer && msgFolder.isCommandEnabled("cmd_renameFolder"));
-
-  if (canRename)
-    SetMenuItemLabel("folderPaneContext-rename", gMessengerBundle.getString("renameFolder"));
-}
-
-function SetupRemoveMenuItem(msgFolder, numSelected, isServer, serverType, specialFolder)
-{
-  var isMail = serverType != 'nntp';
-  var canDelete = (specialFolder == "Junk") ?
-                  CanRenameDeleteJunkMail(msgFolder.URI) : msgFolder.deletable;
-
-  var showRemove = (numSelected <=1) && isMail && canDelete;
-
-  ShowMenuItem("folderPaneContext-remove", showRemove);
-  if (showRemove)
-    EnableMenuItem("folderPaneContext-remove", msgFolder.isCommandEnabled("cmd_delete"));
-  if (canDelete)
-    SetMenuItemLabel("folderPaneContext-remove", gMessengerBundle.getString("removeFolder"));
-}
-
-function SetupCompactMenuItem(folder, numSelected)
-{
-  let canCompact = folder.canCompact;
-  ShowMenuItem("folderPaneContext-compact", numSelected <= 1 && canCompact);
-  EnableMenuItem("folderPaneContext-compact", folder.isCommandEnabled("cmd_compactFolder"));
-
-  if (canCompact)
-    SetMenuItemLabel("folderPaneContext-compact", gMessengerBundle.getString("compactFolder"));
-}
-
-function SetupFavoritesMenuItem(folder, numSelected, isServer, menuItemId)
-{
-  var showItem = !isServer && (numSelected <=1);
-  ShowMenuItem(menuItemId, showItem);
-
-  // adjust the checked state on the menu
-  if (showItem)
-  {
-    document.getElementById(menuItemId)
+  // --- Set up favorite folder menu item.
+  let showFavorite = (numSelected == 1) && !isServer;
+  ShowMenuItem("folderPaneContext-favoriteFolder", showFavorite);
+  if (showFavorite) {
+    // Adjust the checked state on the menu item.
+    document.getElementById("folderPaneContext-favoriteFolder")
             .setAttribute("checked",
                           folder.getFlag(Ci.nsMsgFolderFlags.Favorite));
   }
-}
 
-function SetupNewMenuItem(folder, numSelected, isServer, serverType, specialFolder)
-{
-  var isInbox = specialFolder == "Inbox";
+  // --- Set up the empty trash menu item.
+  ShowMenuItem("folderPaneContext-emptyTrash",
+               numSelected == 1 && specialFolder == "Trash");
 
-  let showNew = (numSelected <= 1 && serverType != "nntp" &&
-                 folder.canCreateSubfolders) || isInbox;
-  ShowMenuItem("folderPaneContext-new", showNew);
-  EnableMenuItem("folderPaneContext-new",
-                 folder.server.type != "imap" || !Services.io.offline);
-  if (showNew)
-  {
-    if (isServer || isInbox)
-      SetMenuItemLabel("folderPaneContext-new", gMessengerBundle.getString("newFolder"));
-    else
-      SetMenuItemLabel("folderPaneContext-new", gMessengerBundle.getString("newSubfolder"));
-  }
+  // --- Set up the empty junk menu item.
+  ShowMenuItem("folderPaneContext-emptyJunk",
+               numSelected == 1 && specialFolder == "Junk");
+
+  // --- Set up the send unsent messages menu item.
+  let showSendUnsentMessages = numSelected == 1 && specialFolder == "Outbox";
+  ShowMenuItem("folderPaneContext-sendUnsentMessages", showSendUnsentMessages);
+  if (showSendUnsentMessages)
+    EnableMenuItem("folderPaneContext-sendUnsentMessages",
+                   IsSendUnsentMsgsEnabled(folder));
+
+  // --- Set up the subscribe menu item.
+  ShowMenuItem("folderPaneContext-subscribe",
+               numSelected == 1 && haveOnlySubscribableFolders);
+
+  // --- Set up the unsubscribe menu item.
+  ShowMenuItem("folderPaneContext-newsUnsubscribe", haveOnlyNewsgroups);
+
+  // --- Set up the mark newsgroup/s read menu item.
+  ShowMenuItem("folderPaneContext-markNewsgroupAllRead", haveOnlyNewsgroups);
+  SetMenuItemLabel("folderPaneContext-markNewsgroupAllRead",
+                   PluralForm.get(numSelected, gMessengerBundle.getString("markNewsgroupRead")));
+
+  // --- Set up the mark folder/s read menu item.
+  ShowMenuItem("folderPaneContext-markMailFolderAllRead",
+               haveOnlyMailFolders && !haveAnyVirtualFolders);
+  SetMenuItemLabel("folderPaneContext-markMailFolderAllRead",
+                   PluralForm.get(numSelected, gMessengerBundle.getString("markFolderRead")));
+
+  // Set up the search menu item.
+  ShowMenuItem("folderPaneContext-searchMessages",
+               numSelected == 1 && !haveAnyVirtualFolders);
+  goUpdateCommand('cmd_search');
+
+  ShowMenuItem("folderPaneContext-openNewWindow", numSelected == 1);
+  ShowMenuItem("folderPaneContext-openNewTab", numSelected == 1);
+
+  // Hide / Show our menu separators based on the menu items we are showing.
+  hideIfAppropriate("folderPaneContext-sep1");
+  hideIfAppropriate("folderPaneContext-sep-edit");
+  hideIfAppropriate("folderPaneContext-sep4");
+
+  return true;
 }
 
 function ShowMenuItem(id, showItem)
@@ -407,34 +526,6 @@ function SetMenuItemAccessKey(id, accessKey)
     item.setAttribute('accesskey', accessKey);
 }
 
-function SiblingHidden(aSibling, aNext)
-{
-  var siblingID;
-  while (aSibling)
-  {
-    siblingID = aSibling.id;
-    // For some reason, context-blockimage and context-unblockimage are not
-    // hidden on the very first time the context menu is invoked. They are only
-    // hidden on subsequent triggers of the context menu. Since we're not
-    // using these two menuitems in mailnews, we can ignore them if encountered.
-    if (!aSibling.hidden && (siblingID != "context-blockimage") &&
-        (siblingID != "context-unblockimage"))
-      return aSibling.localName == "menuseparator";
-    aSibling = aNext ? aSibling.nextSibling : aSibling.previousSibling;
-  }
-  return true;
-}
-
-function ShowSeparator(aSeparatorID)
-{
-  var separator = document.getElementById(aSeparatorID);
-  // Check to see if there are visible siblings before the next and
-  // previous separators.
-  var hidden = SiblingHidden(separator.nextSibling, true) ||
-               SiblingHidden(separator.previousSibling, false);
-  ShowMenuItem(aSeparatorID, !hidden);
-}
-
 // message pane context menu helper methods
 function AddContact(aEmailAddressNode)
 {
@@ -571,12 +662,9 @@ function OpenMessageForMessageId(messageId)
   // if message id not found in current folder search in all folders
   if (!messageHeader)
   {
-    var accountManager = Cc["@mozilla.org/messenger/account-manager;1"]
-                           .getService(Ci.nsIMsgAccountManager);
-
     messageHeader = SearchForMessageIdInSubFolder(startServer.rootFolder, messageId);
 
-    for (let currentServer of accountManager.allServers)
+    for (let currentServer of MailServices.accounts.allServers)
     {
       if (currentServer && startServer != currentServer &&
           currentServer.canSearchMessages && !currentServer.isDeferredTo)
@@ -617,7 +705,7 @@ function OpenMessageByHeader(messageHeader, openInNewWindow)
   else
   {
     if (msgWindow.openFolder != folderURI)
-      SelectFolder(folderURI);
+      gFolderTreeView.selectFolder(folder)
 
     var tree = null;
     var wintype = document.documentElement.getAttribute('windowtype');
@@ -655,19 +743,14 @@ function OpenMessageByHeader(messageHeader, openInNewWindow)
 function SearchForMessageIdInSubFolder(folder, messageId)
 {
   var messageHeader;
-  var subFolders = folder.subFolders;
 
   // search in folder
   if (!folder.isServer)
     messageHeader = CheckForMessageIdInFolder(folder, messageId);
 
   // search subfolders recursively
-  while (subFolders.hasMoreElements() && !messageHeader)
-  {
+  for (let currentFolder of folder.subFolders) {
     // search in current folder
-    var currentFolder =
-      subFolders.getNext().QueryInterface(Ci.nsIMsgFolder);
-
     messageHeader = CheckForMessageIdInFolder(currentFolder, messageId);
 
     // search in its subfolder
@@ -694,9 +777,8 @@ function CheckForMessageIdInFolder(folder, messageId)
     dump("Failed to find message-id in folder!");
   }
 
-  const nsMsgFolderFlags = Ci.nsMsgFolderFlags;
   if (!MailServices.mailSession.IsFolderOpenInWindow(folder) &&
-      !(folder.flags & (nsMsgFolderFlags.Trash | nsMsgFolderFlags.Inbox)))
+      !folder.getFlag(Ci.nsMsgFolderFlags.Trash | Ci.nsMsgFolderFlags.Inbox))
   {
     folder.msgDatabase = null;
   }
