@@ -3,6 +3,9 @@
  * License, v. 2.0. If a copy of the MPL was not distributed with this
  * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
+var { AppConstants } =
+  ChromeUtils.import("resource://gre/modules/AppConstants.jsm");
+
 var gLastMessageUriToLoad = null;
 var gThreadPaneCommandUpdater = null;
 
@@ -179,12 +182,11 @@ function HandleColumnClick(columnID)
 }
 
 function ThreadPaneDoubleClick(event) {
-  const nsMsgFolderFlags = Ci.nsMsgFolderFlags;
-  if (IsSpecialFolderSelected(nsMsgFolderFlags.Drafts, true))
+  if (IsSpecialFolderSelected(Ci.nsMsgFolderFlags.Drafts, true))
   {
     MsgComposeDraftMessage();
   }
-  else if(IsSpecialFolderSelected(nsMsgFolderFlags.Templates, true))
+  else if (IsSpecialFolderSelected(Ci.nsMsgFolderFlags.Templates, true))
   {
     ComposeMsgByType(Ci.nsIMsgCompType.Template, null,
                      Ci.nsIMsgCompFormat.Default);
@@ -204,8 +206,14 @@ function ThreadPaneDoubleClick(event) {
 
 function ThreadPaneKeyPress(event)
 {
-  if (event.keyCode == KeyEvent.DOM_VK_RETURN)
-    ThreadPaneDoubleClick(event);
+  if (event.keyCode == KeyEvent.DOM_VK_RETURN) {
+    if ((AppConstants.platform == "macosx" ? event.metaKey : event.ctrlKey) &&
+        AllowOpenTabOnMiddleClick()) {
+      OpenMessageInNewTab(event);
+    } else {
+      ThreadPaneDoubleClick(event);
+    }
+  }
 }
 
 function MsgSortByThread()
@@ -465,4 +473,126 @@ function ThreadPaneSelectionChanged()
     GetThreadTree().view.selectionChanged();
 }
 
+var ThreadPaneDND = {
+  onDragStart(aEvent) {
+    if (aEvent.originalTarget.localName != "treechildren")
+      return;
+
+    let messageUris = gFolderDisplay.selectedMessageUris;
+    if (!messageUris)
+       return;
+
+    // A message can be dragged from one window and dropped on another window.
+    // Therefore we setNextMessageAfterDelete() here since there is no major
+    // disadvantage, even if it is a copy operation.
+    SetNextMessageAfterDelete();
+    let messengerBundle = document.getElementById("bundle_messenger");
+    let noSubject = messengerBundle.getString("defaultSaveMessageAsFileName");
+    if (noSubject.endsWith(".eml")) {
+      noSubject = noSubject.slice(0, -4);
+    }
+    let fileNames = [];
+    let dataTransfer = aEvent.dataTransfer;
+
+    for (let [index, msgUri] of messageUris.entries()) {
+      let msgService = messenger.messageServiceFromURI(msgUri);
+      let msgHdr = msgService.messageURIToMsgHdr(msgUri);
+      let subject = msgHdr.mime2DecodedSubject || noSubject;
+      if (msgHdr.flags & Ci.nsMsgMessageFlags.HasRe) {
+        subject = "Re: " + subject;
+      }
+      let uniqueFileName = suggestUniqueFileName(subject.substr(0, 120), ".eml",
+                                                 fileNames);
+      fileNames[index] = uniqueFileName;
+      let msgUrl = {};
+      msgService.GetUrlForUri(msgUri, msgUrl, null);
+      dataTransfer.mozSetDataAt("text/x-moz-message", msgUri, index);
+      dataTransfer.mozSetDataAt("text/x-moz-url", msgUrl.value.spec, index);
+      dataTransfer.mozSetDataAt("application/x-moz-file-promise-url",
+                                 msgUrl.value.spec + "?fileName=" +
+                                 encodeURIComponent(uniqueFileName),
+                                 index);
+      dataTransfer.mozSetDataAt("application/x-moz-file-promise",
+                                new messageFlavorDataProvider(), index);
+    }
+    dataTransfer.effectAllowed = "copyMove";
+    dataTransfer.addElement(aEvent.originalTarget);
+  },
+
+  onDragOver(aEvent) {
+    if (!gMsgFolderSelected.canFileMessages ||
+        gMsgFolderSelected.server.type == "rss")
+      return;
+    let dt = aEvent.dataTransfer;
+    dt.effectAllowed = "copy";
+    for (let i = 0; i < dt.mozItemCount; i++) {
+      if (Array.from(dt.mozTypesAt(i)).includes("application/x-moz-file")) {
+        let extFile = dt.mozGetDataAt("application/x-moz-file", i);
+        if (!extFile) {
+          return;
+        }
+
+        extFile = extFile.QueryInterface(Ci.nsIFile);
+        if (extFile.isFile() && /\.eml$/i.test(extFile.leafName)) {
+          aEvent.preventDefault();
+          return;
+        }
+      }
+    }
+  },
+
+  onDrop(aEvent) {
+    let dt = aEvent.dataTransfer;
+    for (let i = 0; i < dt.mozItemCount; i++) {
+      let extFile = dt.mozGetDataAt("application/x-moz-file", i);
+      if (!extFile) {
+        continue;
+      }
+
+      extFile = extFile.QueryInterface(Ci.nsIFile);
+      if (extFile.isFile() && /\.eml$/i.test(extFile.leafName))
+        MailServices.copy.CopyFileMessage(extFile, gMsgFolderSelected, null,
+                                          false, 1, "", null, msgWindow);
+    }
+  },
+}
+
+function messageFlavorDataProvider() {}
+
+messageFlavorDataProvider.prototype = {
+  QueryInterface: ChromeUtils.generateQI(["nsIFlavorDataProvider"]),
+
+  getFlavorData(aTransferable, aFlavor, aData, aDataLen) {
+    if (aFlavor !== "application/x-moz-file-promise") {
+      return;
+    }
+    let fileUriPrimitive = {};
+    let dataSize = {};
+    aTransferable.getTransferData("application/x-moz-file-promise-url",
+                                  fileUriPrimitive, dataSize);
+
+    let fileUriStr = fileUriPrimitive.value
+                                     .QueryInterface(Ci.nsISupportsString);
+    let fileUri = Services.io.newURI(fileUriStr.data);
+    let fileUrl = fileUri.QueryInterface(Ci.nsIURL);
+    let fileName = fileUrl.fileName;
+
+    let destDirPrimitive = {};
+    aTransferable.getTransferData("application/x-moz-file-promise-dir",
+                                  destDirPrimitive, dataSize);
+    let destDirectory = destDirPrimitive.value.QueryInterface(Ci.nsIFile);
+    let file = destDirectory.clone();
+    file.append(fileName);
+
+    let messageUriPrimitive = {};
+    aTransferable.getTransferData("text/x-moz-message", messageUriPrimitive,
+                                  dataSize);
+    let messageUri = messageUriPrimitive.value
+                                        .QueryInterface(Ci.nsISupportsString);
+
+    messenger.saveAs(messageUri.data, true, null, decodeURIComponent(file.path),
+                     true);
+  },
+};
+
 addEventListener("load",ThreadPaneOnLoad,true);
