@@ -8,8 +8,6 @@
 #include "mozilla/IntegerPrintfMacros.h"  // this must pick up <stdint.h>
 #include "mozilla/Sprintf.h"
 
-#include <stddef.h>
-#include <stdio.h>
 #include <string.h>
 
 /* Output array and poisoning method shared by all tests. */
